@@ -1,4 +1,4 @@
-/*  $Id: cmdline_flags.hpp 546749 2017-09-21 11:22:02Z ivanov $
+/*  $Id: cmdline_flags.hpp 562577 2018-04-24 15:51:23Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -116,7 +116,7 @@ NCBI_BLASTINPUT_EXPORT extern const string kArgEvalue;
 NCBI_BLASTINPUT_EXPORT extern const string kArgMinRawGappedScore;
 
 /// Argument to specify the maximum number of target sequences to keep (a.k.a.:
-/// hitlist size) 
+/// hitlist size)
 /// If not set in the command line, this value is the maximum of the number of
 /// alignments/descriptions to show in the traditional BLAST report
 NCBI_BLASTINPUT_EXPORT extern const string kArgMaxTargetSequences;
@@ -192,6 +192,9 @@ NCBI_BLASTINPUT_EXPORT extern const string kArgCullingLimit;
 /// Default argument to specify the culling limit
 NCBI_BLASTINPUT_EXPORT extern const int kDfltArgCullingLimit;
 
+/// Argument to specify the culling limit
+NCBI_BLASTINPUT_EXPORT extern const string kArgSubjectBestHit;
+
 /// Argument to specify the overhang parameter to the best hit algorithm
 NCBI_BLASTINPUT_EXPORT extern const string kArgBestHitOverhang;
 /// Default argument for the overhang parameter to the best hit algorithm
@@ -326,6 +329,18 @@ NCBI_BLASTINPUT_EXPORT extern const string kArgNoGreedyExtension;
 NCBI_BLASTINPUT_EXPORT extern const string kArgDMBTemplateType;
 /// Argument to specify the discontinuous megablast template length
 NCBI_BLASTINPUT_EXPORT extern const string kArgDMBTemplateLength;
+/// Argument to specify if non-concordant pairs should be displayed
+NCBI_BLASTINPUT_EXPORT extern const string kArgNoDiscordant;
+/// Argument to specify forward/reverse strand specificity
+NCBI_BLASTINPUT_EXPORT extern const string kArgFwdRev;
+/// Argument to specify reverse/forward strand specificity
+NCBI_BLASTINPUT_EXPORT extern const string kArgRevFwd;
+/// Argument to specify forward-only strand specificity
+NCBI_BLASTINPUT_EXPORT extern const string kArgFwdOnly;
+/// Argument to specify reverse-only strand specificity
+NCBI_BLASTINPUT_EXPORT extern const string kArgRevOnly;
+/// Argument to specify only strand specific results
+NCBI_BLASTINPUT_EXPORT extern const string kArgOnlyStrandSpecific;
 
 /// Argument to specify the maximum number of HPSs to save per subject for each query
 NCBI_BLASTINPUT_EXPORT extern const string kArgMaxHSPsPerSubject;
@@ -337,7 +352,7 @@ NCBI_BLASTINPUT_EXPORT extern const string kArgPercentIdentity;
 /// Argument to specify the search strategy file to read and use for a BLAST
 /// search
 NCBI_BLASTINPUT_EXPORT extern const string kArgInputSearchStrategy;
-/// Argument to specify the file name to save the search strategy used for a 
+/// Argument to specify the file name to save the search strategy used for a
 /// BLAST search
 NCBI_BLASTINPUT_EXPORT extern const string kArgOutputSearchStrategy;
 /// Flag to force using or not using megablast database index.
@@ -390,6 +405,16 @@ NCBI_BLASTINPUT_EXPORT extern const string kArgQueryCovHspPerc;
 /// Argument to specify line length for displaying alignments
 NCBI_BLASTINPUT_EXPORT extern const string kArgLineLength;
 
+/// Argument to specify taxonomy ids for filtering
+NCBI_BLASTINPUT_EXPORT extern const string kArgTaxIdList;
+/// Argument to specify negative taxonomy ids filtering
+NCBI_BLASTINPUT_EXPORT extern const string kArgNegativeTaxIdList;
+
+/// Argument to specify file with taxonomy ids for filtering
+NCBI_BLASTINPUT_EXPORT extern const string kArgTaxIdListFile;
+/// Argument to specify file with taxonomy ids for Negative filtering
+NCBI_BLASTINPUT_EXPORT extern const string kArgNegativeTaxIdListFile;
+
 
 // Mapper arguments
 /// Argument to specify whether mapped reads are paired
@@ -424,6 +449,9 @@ NCBI_BLASTINPUT_EXPORT extern const string kArgNoUnaligned;
 NCBI_BLASTINPUT_EXPORT extern const string kArgEnableSraCache;
 /// Argument to specify a cutoff edit distance fot an alignment
 NCBI_BLASTINPUT_EXPORT extern const string kArgMaxEditDist;
+/// Argument to specify a maximum number of times a word can be repeated in a
+/// database
+NCBI_BLASTINPUT_EXPORT extern const string kArgMaxDbWordCount;
 
 END_SCOPE(blast)
 END_NCBI_SCOPE
