@@ -1,4 +1,4 @@
-/* $Id: blast_options.h 546023 2017-09-13 11:14:25Z ivanov $
+/* $Id: blast_options.h 562577 2018-04-24 15:51:23Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -165,6 +165,10 @@ extern "C" {
                                    groups of three nucleotide bases to protein
                                    letters */
 
+/** Default max frequency for a database word. Words with higher frequency
+    will be masked in the lookup table. */
+#define MAX_DB_WORD_COUNT_MAPPER 60
+
 /** Value used to indicate that no IMPALA-style scaling should be performed
  * when scaling a PSSM */
 extern const double kPSSM_NoImpalaScaling;
@@ -204,6 +208,9 @@ typedef struct LookupTableOptions {
                            in the database between 1 and 9 times
                            (currently implemented only for MB lookuptable
                            and lookup table word size 16) */
+   Uint1 max_db_word_count; /**< words with larger frequency in the database
+                               will be masked in the lookup table, if the
+                               db_filter optoion is on */
 } LookupTableOptions;
 
 /** Options for dust algorithm, applies only to nucl.-nucl. comparisons.
@@ -345,6 +352,10 @@ typedef struct BlastHSPCullingOptions {
     int max_hits; /**< Maximum number of hits per area of query. */
 } BlastHSPCullingOptions;
 
+typedef struct BlastHSPSubjectBestHitOptions {
+    unsigned int max_range_diff;
+} BlastHSPSubjectBestHitOptions;
+
 /** Structure containing the HSP filtering/writing options */
 typedef struct BlastHSPFilteringOptions {
     /** Best Hit algorithm */
@@ -354,6 +365,9 @@ typedef struct BlastHSPFilteringOptions {
     /** culling algorithm */
     BlastHSPCullingOptions* culling_opts;
     EBlastStage culling_stage; /*<< when to apply the culling algorithm */
+
+    /** Subject Culling */
+    BlastHSPSubjectBestHitOptions * subject_besthit_opts;
 } BlastHSPFilteringOptions;
 
 /** Options used when evaluating and saving hits
@@ -1155,6 +1169,36 @@ NCBI_XBLAST_EXPORT
 Int2 BLAST_GetSuggestedWindowSize(EBlastProgramType program_number, 
                                  const char* matrixName, 
                                  Int4* window_size);
+
+/** Allocate a new object for subject besthit options.
+ * @params isProtein true if protein alignment [in]
+ */
+NCBI_XBLAST_EXPORT
+BlastHSPSubjectBestHitOptions* BlastHSPSubjectBestHitOptionsNew(Boolean isProtein);
+
+/** Validate subject besthit options.
+ * @param opts BlastHSPFilteringOptions structure
+ * @return 0 on success, else non-zero
+ */
+NCBI_XBLAST_EXPORT
+Int2
+BlastHSPSubjectBestHitOptionsValidate(const BlastHSPFilteringOptions* opts);
+
+/** Deallocates subject besthit structure.
+ * @param subject_besthit object to be deallocated. [in]
+ */
+NCBI_XBLAST_EXPORT
+BlastHSPSubjectBestHitOptions*
+BlastHSPSubjectBestHitOptionsFree(BlastHSPSubjectBestHitOptions* subject_besthit_opts);
+
+NCBI_XBLAST_EXPORT
+Int2
+BlastHSPFilteringOptions_AddSubjectBestHit(BlastHSPFilteringOptions* filt_opts,
+                                           BlastHSPSubjectBestHitOptions** subject_besthit);
+
+#define DEFAULT_SUBJECT_BESTHIT_PROT_MAX_RANGE_DIFF 3
+#define DEFAULT_SUBJECT_BESTHIT_NUCL_MAX_RANGE_DIFF 3
+
 #ifdef __cplusplus
 }
 #endif
