@@ -1,4 +1,4 @@
-/* $Id: blast_format.hpp 537527 2017-06-01 14:23:49Z madden $
+/* $Id: blast_format.hpp 556289 2018-01-29 17:33:51Z jianye $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -230,6 +230,7 @@ public:
                            CConstRef<blast::CBlastQueryVector> queries, 
                            SClone& clone_info,
                            bool fill_clone_info,
+                           bool print_airr_format_header,
                            int index = -1);
 
     /// Print all alignment information for aa PHI-BLAST run.
@@ -454,6 +455,14 @@ private:
                                SClone& clone_info,
                                bool fill_clone_info);
 
+    /// Prints AirrRearrangement format
+   /// @param results Results for one query or Phi-blast iteration [in]
+   void x_PrintAirrRearrangement(const blast::CIgBlastResults& results,
+                                 SClone& clone_info,
+                                 bool fill_clone_info,
+                                 bool print_airr_format_header);
+    
+
    /// Replace the query with its reversed-compliement
    /// @param results Ig Blast results [in, out]
    void x_ReverseQuery(blast::CIgBlastResults& results);
