@@ -1,4 +1,4 @@
-/*  $Id: seq_masker_ostat_opt.hpp 434184 2014-05-05 14:32:02Z morgulis $
+/*  $Id: seq_masker_ostat_opt.hpp 462550 2015-03-19 14:07:19Z morgulis $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -141,26 +141,6 @@ class NCBI_XALGOWINMASK_EXPORT CSeqMaskerOstatOpt : public CSeqMaskerOstat
          **/
         virtual void doSetUnitCount( Uint4 unit, Uint4 count );
 
-        /**
-         **\brief Noop.
-         **/
-        virtual void doSetComment( const string & msg ) {}
-
-        /**
-         **\brief Set a parameter value.
-         **
-         ** Only recognized parameters will be accepted.
-         **
-         **\param name the parameter name
-         **\param value the parameter value
-         **/
-        virtual void doSetParam( const string & name, Uint4 value );
-
-        /**
-         **\brief Noop.
-         **/
-        virtual void doSetBlank() {}
-
         /**
          **\brief Generate a hash function and dump the optimized unit counts
          **       data to the output stream.
@@ -195,8 +175,6 @@ class NCBI_XALGOWINMASK_EXPORT CSeqMaskerOstatOpt : public CSeqMaskerOstat
 
         vector< Uint4 > units;      /**<\internal Array of units with counts >= T_low. */
         vector< Uint2 > counts;     /**<\internal Array of corresponding counts. */
-
-        vector< Uint4 > pvalues;    /**<\internal Array of threshold parameters. */
 };
 
 END_NCBI_SCOPE
