@@ -1,7 +1,7 @@
 #ifndef CGI___CGIAPP__HPP
 #define CGI___CGIAPP__HPP
 
-/*  $Id: cgiapp.hpp 523183 2016-12-29 14:53:26Z grichenk $
+/*  $Id: cgiapp.hpp 559509 2018-03-12 15:54:55Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -152,6 +152,7 @@ private:
     CCgiRequest* GetSavedRequest(const string& rid);
     bool x_ProcessHelpRequest(void);
     bool x_ProcessVersionRequest(void);
+    bool x_ProcessAdminRequest(void);
 
 protected:
     /// Check the command line arguments before parsing them.
@@ -219,6 +220,7 @@ protected:
     ///  It is a no-op for the regular CGI.
     void FASTCGI_ScheduleExit(void) { m_ShouldExit = true; }
 
+    friend void s_ScheduleFastCGIExit(void);
 
     /// Factory method for the Context object construction
     virtual CCgiContext*   CreateContext(CNcbiArguments*   args = 0,
@@ -314,6 +316,19 @@ protected:
     /// (default), XML or JSON depending on the 'Accept:' HTTP header, if any.
     virtual void ProcessVersionRequest(EVersionType ver_type);
 
+    /// Admin commands passed through ncbi_admin_cmd argument.
+    enum EAdminCommand {
+        eAdmin_Health,       ///< Report health for this CGI only
+        eAdmin_HealthDeep,   ///< Report health for this CGI and any services used by it.
+        eAdmin_Unknown       ///< Unrecognized command. Overriden ProcessAdminRequest()
+                             ///< can use GetEntry() to fetch command name if necessary.
+    };
+
+    /// Process admin command passed through ncbi_admin_cmd argument.
+    /// Return true on success, false if the command was not processed
+    /// (in this case the default processing will be used).
+    virtual bool ProcessAdminRequest(EAdminCommand cmd);
+
     /// "Accept:" header entry.
     struct SAcceptEntry {
         SAcceptEntry(void) : m_Quality(1) {}
@@ -398,6 +413,7 @@ private:
 
     /// @sa FASTCGI_ScheduleExit()
     bool m_ShouldExit;
+    bool m_CaughtSigterm;
 
     /// Remember if request-start was printed, don't print request-stop
     /// without request-start.
