@@ -1,4 +1,4 @@
-/*  $Id: ncbi_source_ver.h 548044 2017-10-08 12:37:17Z ucko $
+/*  $Id: ncbi_source_ver.h 569379 2018-08-21 04:03:03Z syncbot $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -25,10 +25,19 @@
  */
 
 #include <ncbiconf.h>
+#include <common/ncbi_build_info.h>
 
-#define NCBI_PRODUCTION_VER 20170709
-#define NCBI_DEVELOPMENT_VER 20170605
+/* #undef NCBI_PRODUCTION_VER */
+#define NCBI_DEVELOPMENT_VER 20180821
 
-#ifdef HAVE_COMMON_NCBI_BUILD_VER_H
-#  include <common/ncbi_build_ver.h>
+#if defined(NCBI_PRODUCTION_VER)
+#  define NCBI_SRCTREE_VER_PROXY  NCBI_PRODUCTION_VER
+#  define NCBI_SRCTREE_NAME_PROXY SBuildInfo::eProductionVersion
+#elif defined(NCBI_DEVELOPMENT_VER)
+#  define NCBI_SRCTREE_VER_PROXY  NCBI_DEVELOPMENT_VER
+#  define NCBI_SRCTREE_NAME_PROXY SBuildInfo::eDevelopmentVersion
+#else
+#  define NCBI_SRCTREE_VER_PROXY  0
+#  define NCBI_SRCTREE_NAME_PROXY SBuildInfo::eDevelopmentVersion
 #endif
+
