@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_HTTP_SESSION__HPP
 #define CONNECT___NCBI_HTTP_SESSION__HPP
 
-/* $Id: ncbi_http_session.hpp 523493 2017-01-04 14:12:16Z grichenk $
+/* $Id: ncbi_http_session.hpp 567636 2018-07-21 15:53:54Z mcelhany $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -142,7 +142,8 @@ public:
         eRange,
         eReferer,
         eSetCookie,
-        eUserAgent
+        eUserAgent,
+        eHost
     };
 
     /// Helper class allowing to use both strings and enums as header names.
@@ -544,7 +545,7 @@ private:
 
     // Find cookies matching the url, add or replace 'Cookie' header with the
     // new values.
-    void x_AddCookieHeader(const CUrl& url);
+    void x_AddCookieHeader(const CUrl& url, bool initial);
 
     // CConn_HttpStream callback for parsing headers.
     // 'user_data' must point to a CHttpRequest object.
