@@ -1,4 +1,4 @@
-/* $Id: grid_app_version_info.hpp 523682 2017-01-05 14:56:34Z sadyrovr $
+/* $Id: grid_app_version_info.hpp 549816 2017-10-27 14:56:47Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -65,13 +65,13 @@
 
 namespace ncbi {
 namespace grid {
-namespace cgi {
 
 template <class TBase>
 class CVersionReporting : public TBase
 {
 public:
-    CVersionReporting()
+    template<typename... Args>
+    CVersionReporting(Args&&... args) : TBase(std::forward<Args>(args)...)
     {
         auto& version = this->GetFullVersion();
         auto package_version =  version.GetPackageVersion();
@@ -79,7 +79,6 @@ public:
     }
 };
 
-}
 }
 }
 
