@@ -1,7 +1,7 @@
 #ifndef NCBI_SAFE_STATIC__HPP
 #define NCBI_SAFE_STATIC__HPP
 
-/*  $Id: ncbi_safe_static.hpp 488881 2016-01-06 16:49:31Z grichenk $
+/*  $Id: ncbi_safe_static.hpp 530342 2017-03-14 15:08:00Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -108,9 +108,11 @@ public:
     ~CSafeStaticPtr_Base(void);
 
 protected:
+    typedef CGuard<CSafeStaticPtr_Base> TInstanceMutexGuard;
+
     /// Cleanup function type used by derived classes
     typedef void (*FSelfCleanup)(CSafeStaticPtr_Base* safe_static,
-                                 CMutexGuard& guard);
+                                 TInstanceMutexGuard& guard);
 
     /// Constructor.
     ///
@@ -136,7 +138,42 @@ protected:
     /// Pointer to the data
     const void* m_Ptr;
 
-    DECLARE_CLASS_STATIC_MUTEX(sm_Mutex);
+    DECLARE_CLASS_STATIC_MUTEX(sm_ClassMutex);
+
+private:
+    friend struct SSimpleLock<CSafeStaticPtr_Base>;
+    friend struct SSimpleUnlock<CSafeStaticPtr_Base>;
+
+    void Lock(void)
+    {
+        CMutexGuard guard(sm_ClassMutex);
+        if (!m_InstanceMutex || !m_MutexRefCount) {
+            m_InstanceMutex = new CMutex;
+            m_MutexRefCount = 1;
+        }
+        ++m_MutexRefCount;
+        guard.Release();
+        m_InstanceMutex->Lock();
+    }
+
+    void Unlock(void)
+    {
+        m_InstanceMutex->Unlock();
+        x_ReleaseInstanceMutex();
+    }
+
+    void x_ReleaseInstanceMutex(void)
+    {
+        CMutexGuard guard(sm_ClassMutex);
+        if (--m_MutexRefCount > 0) return;
+        // Workaround for over-optimization - member assignments done immediately
+        // before exiting destructor can be dropped, so mutex and counter should
+        // be set to 0 before deleting the mutex.
+        CMutex* tmp = m_InstanceMutex;
+        m_InstanceMutex = 0;
+        m_MutexRefCount = 0;
+        delete tmp;
+    }
 
 protected:
     friend class CSafeStatic_Less;
@@ -145,6 +182,8 @@ protected:
     FUserCleanup m_UserCleanup;   // User-provided  cleanup function
     int          m_LifeSpan;      // Life span of the object
     int          m_CreationOrder; // Creation order of the object
+    int          m_MutexRefCount; // Mutex reference counter.
+    CMutex*      m_InstanceMutex; // Mutex used to create/destroy value.
 
     static int x_GetCreationOrder(void);
 
@@ -157,10 +196,15 @@ protected:
 
     // To be called by CSafeStaticGuard on the program termination
     friend class CSafeStaticGuard;
-    void x_Cleanup(CMutexGuard& guard)
+    void x_Cleanup(void)
     {
-        if ( m_SelfCleanup )
+        // Note: x_Cleanup should always be called with sm_ClassMutex locked.
+        if ( m_SelfCleanup ) {
+            TInstanceMutexGuard guard(*this);
             m_SelfCleanup(this, guard);
+        }
+        // Delete instance mutex if it's not used by other threads.
+        x_ReleaseInstanceMutex();
     }
 };
 
@@ -213,6 +257,9 @@ public:
         sm_Stack->insert(ptr);
     }
 
+    /// Disable checking on child thread(s) running during destruction
+    static void DisableChildThreadsCheck();
+
 private:
     // Initialize the guard, return pointer to it.
     static CSafeStaticGuard* x_Get(void);
@@ -224,6 +271,9 @@ private:
     // Reference counter. The stack is destroyed when
     // the last reference is removed.
     static int sm_RefCount;
+
+    // Whether to check on child thread(s) running during destruction
+    static bool sm_ChildThreadsCheck;
 };
 
 
@@ -410,7 +460,7 @@ private:
     CSafeStatic& operator=(const CSafeStatic&);
 
     void x_Init(void) {
-        CMutexGuard guard(sm_Mutex);
+        TInstanceMutexGuard guard(*this);
         if ( m_Ptr == 0 ) {
             // Create the object and register for cleanup
             T* ptr = 0;
@@ -434,7 +484,7 @@ private:
 
     // "virtual" cleanup function
     static void sx_SelfCleanup(CSafeStaticPtr_Base* safe_static,
-                               CMutexGuard& guard)
+                               TInstanceMutexGuard& guard)
     {
         TThisType* this_ptr = static_cast<TThisType*>(safe_static);
         if ( T* ptr = static_cast<T*>(const_cast<void*>(this_ptr->m_Ptr)) ) {
@@ -548,7 +598,7 @@ private:
 
     // "virtual" cleanup function
     static void sx_SelfCleanup(CSafeStaticPtr_Base* safe_static,
-                               CMutexGuard& guard)
+                               TInstanceMutexGuard& guard)
     {
         CSafeStaticPtr<T>* this_ptr = static_cast<CSafeStaticPtr<T>*>(safe_static);
         if ( T* ptr = static_cast<T*>(const_cast<void*>(this_ptr->m_Ptr)) ) {
@@ -631,7 +681,7 @@ private:
 
     // "virtual" cleanup function
     static void sx_SelfCleanup(CSafeStaticPtr_Base* safe_static,
-                               CMutexGuard& guard)
+                               TInstanceMutexGuard& guard)
     {
         CSafeStaticRef<T>* this_ptr = static_cast<CSafeStaticRef<T>*>(safe_static);
         if ( T* ptr = static_cast<T*>(const_cast<void*>(this_ptr->m_Ptr)) ) {
@@ -664,7 +714,7 @@ template <class T>
 inline
 void CSafeStaticPtr<T>::Set(T* object)
 {
-    CMutexGuard guard(sm_Mutex);
+    TInstanceMutexGuard guard(*this);
     if ( m_Ptr == 0 ) {
         // Set the new object and register for cleanup
         if ( object ) {
@@ -679,7 +729,7 @@ template <class T>
 inline
 void CSafeStaticPtr<T>::x_Init(void)
 {
-    CMutexGuard guard(sm_Mutex);
+    TInstanceMutexGuard guard(*this);
     if ( m_Ptr == 0 ) {
         // Create the object and register for cleanup
         m_Ptr = new T;
@@ -693,7 +743,7 @@ template <class FUserCreate>
 inline
 void CSafeStaticPtr<T>::x_Init(FUserCreate user_create)
 {
-    CMutexGuard guard(sm_Mutex);
+    TInstanceMutexGuard guard(*this);
     if ( m_Ptr == 0 ) {
         // Create the object and register for cleanup
         m_Ptr = user_create();
@@ -708,7 +758,7 @@ template <class T>
 inline
 void CSafeStaticRef<T>::Set(T* object)
 {
-    CMutexGuard guard(sm_Mutex);
+    TInstanceMutexGuard guard(*this);
     if ( m_Ptr == 0 ) {
         // Set the new object and register for cleanup
         if ( object ) {
@@ -724,7 +774,7 @@ template <class T>
 inline
 void CSafeStaticRef<T>::x_Init(void)
 {
-    CMutexGuard guard(sm_Mutex);
+    TInstanceMutexGuard guard(*this);
     if ( m_Ptr == 0 ) {
         // Create the object and register for cleanup
         T* ptr = new T;
@@ -740,7 +790,7 @@ template <class FUserCreate>
 inline
 void CSafeStaticRef<T>::x_Init(FUserCreate user_create)
 {
-    CMutexGuard guard(sm_Mutex);
+    TInstanceMutexGuard guard(*this);
     if ( m_Ptr == 0 ) {
         // Create the object and register for cleanup
         T* ptr = user_create();
