@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBIMISC__HPP
 #define CORELIB___NCBIMISC__HPP
 
-/*  $Id: ncbimisc.hpp 547815 2017-10-04 15:03:57Z fukanchi $
+/*  $Id: ncbimisc.hpp 563123 2018-05-02 17:08:47Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -155,6 +155,14 @@ enum EInterruptOnSignal {
 };
 
 
+/// Can the action be retried?
+enum ERetriable {
+    eRetriable_No,      ///< It makes no sense to retry the action
+    eRetriable_Unknown, ///< It is unknown if the action can succeed if retried
+    eRetriable_Yes      ///< It makes sense to try again
+};
+
+
 /////////////////////////////////////////////////////////////////////////////
 /// Support for safe bool operators
 /////////////////////////////////////////////////////////////////////////////
@@ -270,7 +278,7 @@ public:
 
     void Swap(pair_base_member<first_type, second_type>& p)
         {
-            if (static_cast<void*>(&first()) != static_cast<void*>(&second())) {
+            if (static_cast<void*>(&first()) != static_cast<void*>(&second())){
                 // work around an IBM compiler bug which causes it to perform
                 // a spurious 1-byte swap, yielding mixed-up values.
                 swap(first(), p.first());
@@ -709,6 +717,17 @@ private:
     TValue m_Value;
 };
 
+template <class TValue, class TNullToValue = SThrowOnNull<TValue> >
+bool operator==(const CNullable<TValue, TNullToValue>& l, const CNullable<TValue, TNullToValue>& r)
+{
+    return (l.IsNull() && r.IsNull()) ||
+           (!l.IsNull() && !r.IsNull() && l.GetValue() == r.GetValue());
+}
+template <class TValue, class TNullToValue = SThrowOnNull<TValue> >
+bool operator!=(const CNullable<TValue, TNullToValue>& l, const CNullable<TValue, TNullToValue>& r)
+{
+    return !operator==(l, r);
+}
 
 // "min" and "max" templates
 //
@@ -1327,6 +1346,27 @@ private:
     TActions m_Actions;
 };
 
+/// Template helpers for unique_ptr to work with resource allocating/deallocating C functions.
+/// @{
+/// Eliminates the necessity for specifying type of deallocating C function.
+template <class T>
+using c_unique_ptr = unique_ptr<T, void(*)(T*)>;
+
+/// Eliminates the necessity for specifying types of both allocated resource and deallocating C function.
+template <class T, typename TDeleter>
+unique_ptr<T, TDeleter> make_c_unique(T* p, TDeleter d)
+{
+    return {p, d};
+}
+
+/// Overload for the above for all types of allocated memory that are to be deallocated by free()
+template <class T>
+unique_ptr<T, void(*)(T*)> make_c_unique(T* p)
+{
+    return make_c_unique(p, &free);
+}
+/// @}
+
 
 END_NCBI_NAMESPACE;
 
