@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBITHR__HPP
 #define CORELIB___NCBITHR__HPP
 
-/*  $Id: ncbithr.hpp 545524 2017-09-07 11:46:07Z ivanov $
+/*  $Id: ncbithr.hpp 545414 2017-09-06 15:40:16Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -521,8 +521,12 @@ public:
                                 ///< - may not be supported and will be
                                 ///< ignored on some platforms
         fRunNice     = 0x40,    ///< Run thread with low priority (MS-Win only)
-        fRunAllowST  = 0x100    ///< Allow threads to run in single thread
+        fRunAllowST  = 0x100,   ///< Allow threads to run in single thread
                                 ///< builds
+
+        fRunCloneRequestContext = 0x200  ///< Clone parent's request context and pass it to the
+                                         ///< new thread. The flag can be used when processing
+                                         ///< the same request in multiple child threads.
     };
 
     /// Bitwise OR'd flags for thread creation passed to Run().
@@ -593,6 +597,15 @@ public:
     /// native thread will return zero.
     static void InitializeMainThreadId(void);
 
+    /// Check if the application is exiting (entered the destructor).
+    /// Recommended to be used as while() condition by infinite threads
+    /// to stop them properly on exit.
+    /// @sa SetWaitForAllThreadsTimeout
+    static bool IsAppExiting(void) { return sm_IsExiting; }
+
+    /// Set timeout for stopping all threads on application exit.
+    static void SetWaitForAllThreadsTimeout(const CTimeout& timeout);
+
 protected:
     /// Derived (user-created) class must provide a real thread function.
     virtual void* Main(void) = 0;
@@ -617,6 +630,10 @@ private:
     bool          m_IsTerminated;  ///< if Exit() was called for the thread
     CRef<CThread> m_SelfRef;       ///< "this" -- to avoid premature destruction
     void*         m_ExitData;      ///< as returned by Main() or passed to Exit()
+    CRef<CRequestContext> m_ParentRequestContext;
+
+    static bool     sm_IsExiting;
+    static CTimeout sm_WaitForThreadsTimeout;
 
 #if defined NCBI_THREAD_PID_WORKAROUND
     friend class CProcess;
@@ -635,6 +652,12 @@ private:
     static TWrapperRes Wrapper(TWrapperArg arg);
     friend TWrapperRes ThreadWrapperCaller(TWrapperArg arg);
 
+    // Wait for all threads to terminate. Note that the method does not request threads
+    // to stop, it just waits for the number of running threads to become zero.
+    // Return true if all threads have been stopped, false on timeout.
+    static bool WaitForAllThreads(void);
+    friend class CNcbiApplication;
+
     /// Prohibit copying and assigning
     CThread(const CThread&);
     CThread& operator= (const CThread&);
