@@ -1,7 +1,7 @@
 #ifndef CORELIB___PLUGIN_MANAGER__HPP
 #define CORELIB___PLUGIN_MANAGER__HPP
 
-/* $Id: plugin_manager.hpp 485209 2015-11-19 15:11:24Z ivanov $
+/* $Id: plugin_manager.hpp 572204 2018-10-10 14:43:17Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -202,7 +202,6 @@ public:
         static const CVersionInfo vi(TDefaultDriverVersion::eMajor,
                                      TDefaultDriverVersion::eMinor,
                                      TDefaultDriverVersion::ePatchLevel);
-
         return vi;
     }
 
@@ -305,7 +304,6 @@ public:
         static const CVersionInfo vi(TDefaultDriverVersion::eMajor,
                                      TDefaultDriverVersion::eMinor,
                                      TDefaultDriverVersion::ePatchLevel);
-
         return vi;
     }
 
@@ -313,14 +311,12 @@ public:
     /// @return
     ///  Never returns NULL -- always throw exception on error.
     /// @sa GetFactory()
-    TClass* CreateInstance
-    (const string&       driver  = kEmptyStr,
-     const CVersionInfo& version = GetDefaultDrvVers(),
-     const TPluginManagerParamTree* params = 0)
+    TClass* CreateInstance(const string&       driver  = kEmptyStr,
+                           const CVersionInfo& version = GetDefaultDrvVers(),
+                           const TPluginManagerParamTree* params = 0)
     {
         string driver_name = driver;
-        TSubstituteMap::const_iterator subst_it =
-            m_SubstituteMap.find(driver_name);
+        TSubstituteMap::const_iterator subst_it = m_SubstituteMap.find(driver_name);
 
         if (subst_it != m_SubstituteMap.end()) {
             driver_name = subst_it->second;
@@ -332,7 +328,7 @@ public:
             string msg = "Cannot create a driver instance (driver: ";
             msg += driver;
             msg += ").";
-            NCBI_THROW(CPluginManagerException, eNullInstance,msg);
+            NCBI_THROW(CPluginManagerException, eNullInstance, msg);
         }
         return drv;
     }
@@ -363,9 +359,8 @@ public:
     ///  with the requested version.
     /// @return
     ///  Never return NULL -- always throw exception on error.
-    TClassFactory* GetFactory
-    (const string&       driver  = kEmptyStr,
-     const CVersionInfo& version = GetDefaultDrvVers());
+    TClassFactory* GetFactory(const string&       driver  = kEmptyStr,
+                              const CVersionInfo& version = GetDefaultDrvVers());
 
     #ifndef NCBI_SWIG
     /// Information about a driver, with maybe a pointer to an instantiated
@@ -534,11 +529,8 @@ protected:
 
     typedef set<FNCBI_EntryPoint>                TEntryPoints;
     typedef set<TClassFactory*>                  TFactories;
-
     typedef vector<CDllResolver::SResolvedEntry> TResolvedEntries;
-
     typedef vector<CPluginManager_DllResolver*>  TDllResolvers;
-
     typedef set<string>                          TStringSet;
     typedef map<string, string>                  TSubstituteMap;
 
@@ -593,11 +585,10 @@ public:
     ///   Interface version
     /// @param unload_dll
     ///   Whether to unload drivers DLL in the destructor
-    CPluginManager_DllResolver
-    (const string&       interface_name,
-     const string&       driver_name = kEmptyStr,
-     const CVersionInfo& version     = CVersionInfo::kAny,
-     CDll::EAutoUnload unload_dll = CDll::eNoAutoUnload);
+    CPluginManager_DllResolver(const string&       interface_name,
+                               const string&       driver_name = kEmptyStr,
+                               const CVersionInfo& version     = CVersionInfo::kAny,
+                               CDll::EAutoUnload   unload_dll  = CDll::eNoAutoUnload);
 
     //
     virtual ~CPluginManager_DllResolver(void);
@@ -614,13 +605,9 @@ public:
     ///   Reference on DLL resolver holding all entry points
     // Former "Resolve"
     CDllResolver& ResolveFile(const TSearchPaths&   paths,
-                              const string&         driver_name
-                                = kEmptyStr,
-                              const CVersionInfo&   version
-                                = CVersionInfo::kAny,
-                              CDllResolver::TExtraDllPath std_path
-                                = CDllResolver::fDefaultDllPath
-                              );
+                              const string&         driver_name = kEmptyStr,
+                              const CVersionInfo&   version     = CVersionInfo::kAny,
+                              CDllResolver::TExtraDllPath std_path = CDllResolver::fDefaultDllPath);
 
     /// Search for plugin DLLs, resolve entry points.
     ///
@@ -645,9 +632,8 @@ public:
     ///    Requested version of the driver
     virtual
     string GetDllName(const string&       interface_name,
-                      const string&       driver_name  = kEmptyStr,
-                      const CVersionInfo& version      = CVersionInfo::kAny)
-        const;
+                      const string&       driver_name = kEmptyStr,
+                      const CVersionInfo& version     = CVersionInfo::kAny) const;
 
     enum EVersionLocation { eBeforeSuffix, eAfterSuffix };
 
@@ -662,8 +648,7 @@ public:
     string GetDllNameMask(const string&       interface_name,
                           const string&       driver_name = kEmptyStr,
                           const CVersionInfo& version     = CVersionInfo::kAny,
-                          EVersionLocation    ver_lct     = eBeforeSuffix)
-        const;
+                          EVersionLocation    ver_lct     = eBeforeSuffix)  const;
 
     /// Return DLL entry point name:
     ///
@@ -729,15 +714,13 @@ protected:
 inline
 bool operator==(const SDriverInfo& i1, const SDriverInfo& i2)
 {
-    return i1.name == i2.name && i1.version == i2.version;
+    return i1.name == i2.name  &&  i1.version == i2.version;
 }
 
 inline
 bool operator<(const SDriverInfo& i1, const SDriverInfo& i2)
 {
-    return (i1.name < i2.name ||
-            (i1.name == i2.name &&
-             (i1.version < i2.version)));
+    return (i1.name < i2.name  || (i1.name == i2.name && (i1.version < i2.version)));
 }
 
 
@@ -751,6 +734,7 @@ TClass* CPluginManager<TClass>::CreateInstanceFromList(
 
     _TRACE("Creating an instance of a driver having version " <<
            version << " from a list " << driver_list);
+
     list<string> drivers;
     NStr::Split(driver_list, ":", drivers, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
     ITERATE ( list<string>, it, drivers ) {
@@ -807,6 +791,7 @@ CPluginManager<TClass>::GetFactory(const string&       driver,
     // Search among already registered factories
     _TRACE("Looking for an already loaded factory for driver " <<
            driver << " having version " << version);
+
     cf = FindClassFactory(driver, version);
     if (cf) {
         return cf;
@@ -1012,15 +997,13 @@ public:
     typedef typename CPluginManager<TClass>::SDriverInfo TValue;
 
     CInvalidDrvVer(const string& driver_name, const CVersionInfo& vi)
-    : m_DriverName(driver_name)
-    , m_VersionInfo(vi)
-    {
-    }
+        : m_DriverName(driver_name), m_VersionInfo(vi)
+    {}
 
-    bool operator() ( const TValue& val )
+    bool operator() (const TValue& val)
     {
         return m_DriverName != val.name ||
-            val.version.Match(m_VersionInfo) == CVersionInfo::eNonCompatible;
+               val.version.Match(m_VersionInfo) == CVersionInfo::eNonCompatible;
     }
 
 private:
@@ -1028,12 +1011,12 @@ private:
     const CVersionInfo m_VersionInfo;
 };
 
+
 template <class TClass>
 template <typename TEntryPoint>
-bool CPluginManager<TClass>::RegisterWithEntryPoint
-(TEntryPoint         plugin_entry_point,
- const string&       driver_name,
- const CVersionInfo& driver_version)
+bool CPluginManager<TClass>::RegisterWithEntryPoint(TEntryPoint         plugin_entry_point,
+                                                    const string&       driver_name,
+                                                    const CVersionInfo& driver_version)
 {
     CMutexGuard guard(m_Mutex);
     if ( !m_EntryPoints.insert(
@@ -1083,14 +1066,15 @@ bool CPluginManager<TClass>::RegisterWithEntryPoint
     return false;
 }
 
+
 template <class TClass>
-void CPluginManager<TClass>::AddResolver
-(CPluginManager_DllResolver* resolver)
+void CPluginManager<TClass>::AddResolver(CPluginManager_DllResolver* resolver)
 {
     _ASSERT(resolver);
     m_Resolvers.push_back(resolver);
 }
 
+
 template <class TClass>
 CPluginManager_DllResolver*
 CPluginManager<TClass>::DetachResolver(CPluginManager_DllResolver* resolver)
@@ -1104,12 +1088,14 @@ CPluginManager<TClass>::DetachResolver(CPluginManager_DllResolver* resolver)
     return 0;
 }
 
+
 template <class TClass>
 void CPluginManager<TClass>::AddDllSearchPath(const string& path)
 {
     m_DllSearchPaths.push_back(path);
 }
 
+
 template <class TClass>
 void CPluginManager<TClass>::ResetDllSearchPath(TSearchPaths* previous_paths)
 {
@@ -1121,10 +1107,10 @@ void CPluginManager<TClass>::ResetDllSearchPath(TSearchPaths* previous_paths)
     }
 }
 
+
 template <class TClass>
 CDllResolver::TExtraDllPath
-CPluginManager<TClass>::SetDllStdSearchPath
-(CDllResolver::TExtraDllPath standard_paths)
+CPluginManager<TClass>::SetDllStdSearchPath(CDllResolver::TExtraDllPath standard_paths)
 {
     CDllResolver::TExtraDllPath tmp_path = m_StdDllPath;
 
@@ -1132,6 +1118,7 @@ CPluginManager<TClass>::SetDllStdSearchPath
     return tmp_path;
 }
 
+
 template <class TClass>
 CDllResolver::TExtraDllPath
 CPluginManager<TClass>::GetDllStdSearchPath(void) const
@@ -1139,9 +1126,9 @@ CPluginManager<TClass>::GetDllStdSearchPath(void) const
     return m_StdDllPath;
 }
 
+
 template <class TClass>
-void CPluginManager<TClass>::FreezeResolution(const string& driver,
-                                              bool          value)
+void CPluginManager<TClass>::FreezeResolution(const string& driver, bool value)
 {
     if (value) {
         m_FreezeResolutionDrivers.insert(driver);
@@ -1150,6 +1137,7 @@ void CPluginManager<TClass>::FreezeResolution(const string& driver,
     }
 }
 
+
 template <class TClass>
 void CPluginManager<TClass>::ResolveFile(const string&       driver,
                                          const CVersionInfo& version)
@@ -1160,26 +1148,19 @@ void CPluginManager<TClass>::ResolveFile(const string&       driver,
     ITERATE(vector<CPluginManager_DllResolver*>, it, m_Resolvers) {
         // Try to get an exact match.
         CDllResolver* dll_resolver =
-            &(*it)->ResolveFile(m_DllSearchPaths,
-                                driver,
-                                version,
-                                m_StdDllPath);
+            &(*it)->ResolveFile(m_DllSearchPaths, driver, version, m_StdDllPath);
 
         if ( !version.IsAny() && !version.IsLatest() &&
             dll_resolver->GetResolvedEntries().empty() ) {
 
             // Try to get at least something (ignore version).
             dll_resolver =
-                &(*it)->ResolveFile(m_DllSearchPaths,
-                                    driver,
-                                    CVersionInfo::kAny,
-                                    m_StdDllPath);
+                &(*it)->ResolveFile(m_DllSearchPaths, driver, CVersionInfo::kAny, m_StdDllPath);
 
             if ( dll_resolver->GetResolvedEntries().empty() ) {
                 dll_resolver = 0;
             }
         }
-
         if ( dll_resolver ) {
             resolvers.push_back(dll_resolver);
         }
@@ -1201,8 +1182,7 @@ void CPluginManager<TClass>::ResolveFile(const string&       driver,
             const CDllResolver::SNamedEntryPoint& epoint = entry.entry_points.front();
 
             if (epoint.entry_point.func) {
-                FNCBI_EntryPoint ep =
-                   (FNCBI_EntryPoint)(void*)epoint.entry_point.func;
+                FNCBI_EntryPoint ep =  (FNCBI_EntryPoint)(void*)epoint.entry_point.func;
 
                 if ( RegisterWithEntryPoint(ep, driver, version) ) {
                     m_RegisteredEntries.push_back(entry);
@@ -1215,6 +1195,9 @@ void CPluginManager<TClass>::ResolveFile(const string&       driver,
                 }
             }
         }
+        // NOTE !!!:
+        // Next call is not necessary and lead to memory leak, but removing it 
+        // will break current CPluginManager usage and lead to coredamp in DLL mode on exit.
         entry_points.clear();
     }
 }
@@ -1238,15 +1221,10 @@ CPluginManager<TClass>::CPluginManager(void)
 
     if (registry) {
         list<string> entries;
-
         registry->EnumerateEntries(section_name, &entries);
         ITERATE(list<string>, cit, entries) {
             string driver_name = *cit;
-
-            string substitute_name = registry->GetString(section_name,
-                                                         driver_name,
-                                                         driver_name);
-
+            string substitute_name = registry->GetString(section_name, driver_name, driver_name);
             m_SubstituteMap[driver_name] = substitute_name;
         }
     }
@@ -1265,18 +1243,12 @@ CPluginManager<TClass>::~CPluginManager(void)
     ITERATE ( typename TFactories, it, m_Factories ) {
         delete *it;
     }
-
     ITERATE ( typename TDllResolvers, it, m_Resolvers ) {
         delete *it;
     }
-
-    NON_CONST_ITERATE(TResolvedEntries, it, m_RegisteredEntries) {
-        delete it->dll;
-    }
 }
 
 
-
 template <class TClass>
 string
 IClassFactory<TClass>::GetParam(
@@ -1294,6 +1266,7 @@ IClassFactory<TClass>::GetParam(
                           default_value);
 }
 
+
 template <class TClass>
 const string&
 IClassFactory<TClass>::GetParam(
