@@ -1,7 +1,7 @@
 #ifndef CORELIB___VERSION__HPP
 #define CORELIB___VERSION__HPP
 
-/*  $Id: version.hpp 523179 2016-12-29 14:03:50Z grichenk $
+/*  $Id: version.hpp 564103 2018-05-21 15:57:43Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,6 +50,82 @@ BEGIN_NCBI_SCOPE
 // CVersionInfo
 
 
+/// This class allows to add build info (date and tag) to application version.
+///
+/// This can be done by providing explicitly created SBuildInfo instance
+/// (e.g. SBuildInfo(__DATE__ " " __TIME__, "RC1"))
+///
+/// If clients do not explicitly set their own build info,
+/// C++ Toolkit build info will be used in the reporting instead.
+
+struct NCBI_XNCBI_EXPORT SBuildInfo
+{
+    enum EExtra
+    {
+        eBuildDate,
+        eBuildTag,
+        eTeamCityProjectName,
+        eTeamCityBuildConf,
+        eTeamCityBuildNumber,
+        eSubversionRevision,
+        eStableComponentsVersion,
+        eDevelopmentVersion,
+        eProductionVersion
+    };
+    string date;
+    string tag;
+    vector< pair<EExtra,string> > m_extra;
+
+    SBuildInfo(void);
+    SBuildInfo(const string& d, const string& t = kEmptyStr) : date(d), tag(t) {
+    }
+
+    SBuildInfo& Extra( EExtra key, const string& value);
+    SBuildInfo& Extra( EExtra key, int value);
+    string GetExtraValue( EExtra key, const string& default_value = kEmptyStr) const;
+
+    static string ExtraName(EExtra key);
+    static string ExtraNameXml(EExtra key);
+    static string ExtraNameJson(EExtra key);
+    static string ExtraNameAppLog(EExtra key);
+
+    string Print(size_t offset = 0) const;
+    string PrintXml(void) const;
+    string PrintJson(void) const;
+};
+
+#ifdef NCBI_BUILD_TAG
+#  define NCBI_BUILD_TAG_PROXY  NCBI_AS_STRING(NCBI_BUILD_TAG)
+#else
+#  define NCBI_BUILD_TAG_PROXY  ""
+#endif
+
+#if defined(NCBI_SRCTREE_NAME_PROXY)
+#define NCBI_SBUILDINFO_DEFAULT_IMPL() \
+    SBuildInfo( __DATE__ " " __TIME__, NCBI_BUILD_TAG_PROXY) \
+        .Extra(SBuildInfo::eTeamCityProjectName,     NCBI_TEAMCITY_PROJECT_NAME_PROXY) \
+        .Extra(SBuildInfo::eTeamCityBuildConf,       NCBI_TEAMCITY_BUILDCONF_NAME_PROXY) \
+        .Extra(SBuildInfo::eTeamCityBuildNumber,     NCBI_TEAMCITY_BUILD_NUMBER_PROXY) \
+        .Extra(SBuildInfo::eSubversionRevision,      NCBI_SUBVERSION_REVISION_PROXY) \
+        .Extra(SBuildInfo::eStableComponentsVersion, NCBI_SC_VERSION_PROXY) \
+        .Extra(NCBI_SRCTREE_NAME_PROXY,              NCBI_SRCTREE_VER_PROXY)
+#else //NCBI_SRCTREE_NAME_PROXY
+#define NCBI_SBUILDINFO_DEFAULT_IMPL() \
+    SBuildInfo( __DATE__ " " __TIME__, NCBI_BUILD_TAG_PROXY) \
+        .Extra(SBuildInfo::eTeamCityProjectName,     NCBI_TEAMCITY_PROJECT_NAME_PROXY) \
+        .Extra(SBuildInfo::eTeamCityBuildConf,       NCBI_TEAMCITY_BUILDCONF_NAME_PROXY) \
+        .Extra(SBuildInfo::eTeamCityBuildNumber,     NCBI_TEAMCITY_BUILD_NUMBER_PROXY) \
+        .Extra(SBuildInfo::eSubversionRevision,      NCBI_SUBVERSION_REVISION_PROXY) \
+        .Extra(SBuildInfo::eStableComponentsVersion, NCBI_SC_VERSION_PROXY)
+#endif //NCBI_SRCTREE_NAME_PROXY
+
+#if defined(NCBI_USE_PCH) && !defined(NCBI_TEAMCITY_BUILD_NUMBER)
+#define NCBI_SBUILDINFO_DEFAULT() SBuildInfo()
+#else
+#define NCBI_SBUILDINFO_DEFAULT() NCBI_SBUILDINFO_DEFAULT_IMPL()
+#endif
+#define NCBI_APP_SBUILDINFO_DEFAULT() NCBI_SBUILDINFO_DEFAULT_IMPL()
+
 /////////////////////////////////////////////////////////////////////////////
 ///
 /// CVersionInfo --
@@ -59,9 +135,6 @@ BEGIN_NCBI_SCOPE
 class NCBI_XNCBI_EXPORT CVersionInfo
 {
 public:
-    /// Default constructor
-    CVersionInfo(void) ;
-
     /// Constructor
     CVersionInfo(int  ver_major,
                  int  ver_minor,
@@ -78,11 +151,7 @@ public:
         kAny = 0,
         kLatest
     };
-    CVersionInfo(EVersionFlags flags);
-
-    /// Constructor.
-    CVersionInfo(const CVersionInfo& version);
-    CVersionInfo& operator=(const CVersionInfo& version);
+    CVersionInfo(EVersionFlags flags = kLatest);
 
     /// Destructor.
     virtual ~CVersionInfo() {}
@@ -177,7 +246,8 @@ public:
                            int  ver_major,
                            int  ver_minor,
                            int  patch_level = 0,
-                           const string& ver_name = kEmptyStr);
+                           const string& ver_name = kEmptyStr,
+                           const SBuildInfo& build_info = NCBI_SBUILDINFO_DEFAULT());
 
     /// Constructor
     ///
@@ -189,16 +259,8 @@ public:
     ///    version name
     CComponentVersionInfo( const string& component_name,
                            const string& version,
-                           const string& ver_name = kEmptyStr);
-
-    /// Copy constructor.
-    CComponentVersionInfo(const CComponentVersionInfo& version);
-
-    /// Assignment.
-    CComponentVersionInfo& operator=(const CComponentVersionInfo& version);
-
-    /// Destructor.
-    virtual ~CComponentVersionInfo() {}
+                           const string& ver_name = kEmptyStr,
+                           const SBuildInfo& build_info = NCBI_SBUILDINFO_DEFAULT());
 
     /// Get component name
     const string& GetComponentName(void) const
@@ -216,31 +278,8 @@ public:
     virtual string PrintJson(void) const;
 
 private:
-    // default ctor
-    CComponentVersionInfo(void);
     string m_ComponentName;
-};
-
-
-/// This class allows to add build info (date and tag) to application version.
-///
-/// This can be done by providing explicitly created SBuildInfo instance
-/// (e.g. SBuildInfo(__DATE__ " " __TIME__, "RC1"))
-/// instead of implicit SBuildInfo(), where SBuildInfo instance is accepted.
-///
-/// If clients do not explicitly set their own build info,
-/// C++ Toolkit build info will be used in the reporting instead.
-
-struct NCBI_XNCBI_EXPORT SBuildInfo
-{
-    string date;
-    string tag;
-
-    SBuildInfo();
-    SBuildInfo(const string& d, const string& t = kEmptyStr) : date(d), tag(t) {}
-
-    string PrintXml(void) const;
-    string PrintJson(void) const;
+    SBuildInfo m_BuildInfo;
 };
 
 
@@ -248,18 +287,17 @@ class NCBI_XNCBI_EXPORT CVersion : public CObject
 {
 public:
 
-    CVersion(const SBuildInfo& build_info = SBuildInfo());
+    CVersion(const SBuildInfo& build_info = NCBI_SBUILDINFO_DEFAULT());
     
     CVersion(const CVersionInfo& version,
-            const SBuildInfo& build_info = SBuildInfo());
+            const SBuildInfo& build_info = NCBI_SBUILDINFO_DEFAULT());
 
-    /// Copy constructor.
     CVersion(const CVersion& version);
-    /// Destructor.
-    virtual ~CVersion(void)
-    {
-    }
-    
+    CVersion(CVersion&& version) = default;
+
+    CVersion& operator=(const CVersion& version);
+    CVersion& operator=(CVersion&& version) = default;
+
     /// Set version information
     void SetVersionInfo( int  ver_major,
                          int  ver_minor,
@@ -282,7 +320,8 @@ public:
                               int           ver_major,
                               int           ver_minor,
                               int           patch_level = 0,
-                              const string& ver_name = kEmptyStr);
+                              const string& ver_name = kEmptyStr,
+                              const SBuildInfo& build_info = NCBI_SBUILDINFO_DEFAULT());
     /// Add component version information
     /// @note Takes the ownership over the passed VersionInfo object 
     void AddComponentVersion( CComponentVersionInfo* component);
@@ -301,7 +340,7 @@ public:
         fPackageFull    = 0x08,  ///< Print package info, if available
         fBuildInfo      = 0x10,  ///< Print build info (date and tag)
         fBuildSignature = 0x20,  ///< Print build signature, if available
-        fTCBuildNumber  = 0x40,  ///< Print TeamCity build number, if available
+        fTCBuildNumber  = 0x0,   ///< obsolete, has no effect
         fPrintAll       = 0xFF   ///< Print all version data
     };
     typedef int TPrintFlags;  ///< Binary OR of EPrintFlags
@@ -314,9 +353,10 @@ public:
     string PrintJson(const string& appname, TPrintFlags flags = fPrintAll) const;
 
 private:
+    static void x_Copy(CVersion& to, const CVersion& from);
 
-    AutoPtr< CVersionInfo > m_VersionInfo;
-    vector< AutoPtr< CComponentVersionInfo> > m_Components;
+    unique_ptr<CVersionInfo> m_VersionInfo;
+    vector<unique_ptr<CComponentVersionInfo>> m_Components;
     SBuildInfo m_BuildInfo;
 };
 
