@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___DBAPI_DRIVER_CONVERT__HPP
 #define DBAPI_DRIVER___DBAPI_DRIVER_CONVERT__HPP
 
-/* $Id: dbapi_driver_convert.hpp 437342 2014-06-05 13:56:19Z ucko $
+/* $Id: dbapi_driver_convert.hpp 498292 2016-04-14 19:07:55Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NTOICE
@@ -436,6 +436,12 @@ private:
             case eDB_Image:
                 value = ConvertFromLOB<TO, CDB_Image>();
                 break;
+            case eDB_VarCharMax:
+                value = ConvertFromLOB<TO, CDB_VarCharMax>();
+                break;
+            case eDB_VarBinaryMax:
+                value = ConvertFromLOB<TO, CDB_VarBinaryMax>();
+                break;
             case eDB_Bit:
                 value = ConvertFrom<TO, CDB_Bit>();
                 break;
