@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___EXCEPTION__HPP
 #define DBAPI_DRIVER___EXCEPTION__HPP
 
-/* $Id: exception.hpp 501456 2016-05-16 15:12:46Z ivanov $
+/* $Id: exception.hpp 563975 2018-05-17 18:33:33Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -220,10 +220,13 @@ public:
     void SetParams(const CDBParams* params);
     void SetParams(const CDBParams& params) { SetParams(&params); }
     CConstRef<SParams> GetParams(void) const { return m_Params; }
+
+    void SetRowsInBatch(unsigned int n) { m_RowsInBatch = n; }
+    unsigned int GetRowsInBatch(void) const { return m_RowsInBatch; }
     
 
     /// WARNING !!! Sybase severity value can be provided by Sybase/FreeTDS
-    /// ctlib/dblib drivers only.
+    /// ctlib drivers only.
     void SetSybaseSeverity(int severity) { m_SybaseSeverity = severity;}
     int GetSybaseSeverity(void) const { return m_SybaseSeverity;       }
 
@@ -248,6 +251,7 @@ public:
         , m_DBErrCode(db_err_code)
         , m_Context(message.context)
         , m_SybaseSeverity(0)
+        , m_RowsInBatch(0)
         NCBI_EXCEPTION_DEFAULT_IMPLEMENTATION(CDB_Exception, CException);
 
 protected:
@@ -266,6 +270,7 @@ private:
     CConstRef<SContext> m_Context;
     int                 m_SybaseSeverity;
     CRef<SParams>       m_Params;
+    unsigned int        m_RowsInBatch;
 };
 
 inline
@@ -693,8 +698,16 @@ typedef CDB_UserHandler_Diag CDB_UserHandler_Default;
 /// Generic macro to throw a database exception, given the exception class,
 /// database error code and message string.
 #define NCBI_DATABASE_THROW( exception_class, message, err_code, severity ) \
-    throw exception_class( DIAG_COMPILE_INFO, \
-        0, (message), severity, err_code )
+    do { \
+        exception_class ex( DIAG_COMPILE_INFO, \
+                            0, (message), severity, err_code ); \
+        if (severity == eDiag_Error || severity == eDiag_Fatal) \
+            ex.SetRetriable(eRetriable_No); \
+        else \
+            ex.SetRetriable(eRetriable_Unknown); \
+        throw ex; \
+    } while(0)
+
 #define NCBI_DATABASE_RETHROW( prev_exception, exception_class, message, \
     err_code, severity ) \
     throw exception_class( DIAG_COMPILE_INFO, \
@@ -703,8 +716,13 @@ typedef CDB_UserHandler_Diag CDB_UserHandler_Default;
 // Driver code typically redefines NCBI_DATABASE_(RE)THROW in terms of these.
 #define NCBI_DATABASE_THROW_ANNOTATED(ex_class, message, err_code, severity, \
                                       dbg_info, conn, params) \
-    throw ex_class(DIAG_COMPILE_INFO, NULL, (message), severity, err_code, \
-                   dbg_info, conn, params)
+    do { \
+        ex_class ex(DIAG_COMPILE_INFO, NULL, (message), severity, err_code, \
+                    dbg_info, conn, params); \
+        ex.SetRetriable(eRetriable_No); \
+        throw ex; \
+    } while (0)
+
 #define NCBI_DATABASE_RETHROW_ANNOTATED(prev_ex, ex_class, message, err_code, \
                                         severity, dbg_info, conn, params) \
     throw ex_class(DIAG_COMPILE_INFO, &(prev_ex), (message), severity, \
