@@ -1,7 +1,7 @@
 #ifndef DBAPI_DRIVER___INTERFACES__HPP
 #define DBAPI_DRIVER___INTERFACES__HPP
 
-/* $Id: interfaces.hpp 536536 2017-05-19 16:16:00Z ucko $
+/* $Id: interfaces.hpp 565697 2018-06-15 14:21:31Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -45,6 +45,13 @@
 #include <map>
 
 
+// Visual Studio 2013 and 2015 overreact on deprecated methods
+#if defined(_MSC_VER) && _MSC_VER < 1910
+    #pragma warning(push)
+    #pragma warning(disable : 4996)
+#endif
+
+
 /** @addtogroup DbInterfaces
  *
  * @{
@@ -546,15 +553,25 @@ protected:
     /// NOTE: the cursor must be declared for update in CDB_Connection::Cursor()
     virtual bool Update(const string& table_name, const string& upd_query) = 0;
 
+    /// @deprecated
+    ///   Please use I_Connection::SendData instead.
+    /// @sa
+    ///   I_Connection::SendData
+    NCBI_DEPRECATED
     virtual bool UpdateBlob(unsigned int item_num, CDB_Stream& data,
                             bool log_it = true) = 0;
-    bool UpdateTextImage(unsigned int item_num, CDB_Stream& data,
-                         bool log_it = true)
-        { return UpdateBlob(item_num, data, log_it); }
 
-    /// @brief 
+    /// @deprecated
+    ///   Please use I_Connection::SendData instead.
+    /// @sa
+    ///   I_Connection::SendData
+    NCBI_DEPRECATED
+    bool UpdateTextImage(unsigned int item_num, CDB_Stream& data,
+                         bool log_it = true);
+
+    /// @brief
     ///   Create send-data command.
-    /// 
+    ///
     /// @param item_num 
     ///   Column number to rewrite.
     /// @param size 
@@ -564,9 +581,14 @@ protected:
     /// @param discard_results
     ///   Discard all resultsets that might be returned from server
     ///   if this value is set to true.
-    /// 
+    ///
     /// @return 
     ///   Newly created send-data object.
+    /// @deprecated
+    ///   Please use I_Connection::SendDataCmd instead.
+    /// @sa
+    ///   I_Connection::SendDataCmd
+    NCBI_DEPRECATED
     virtual CDB_SendDataCmd* SendDataCmd(unsigned int item_num, size_t size,
                                          bool log_it = true,
                                          bool discard_results = true) = 0;
@@ -730,19 +752,28 @@ public:
     virtual size_t ReadItem(void* buffer, size_t buffer_size,
                             bool* is_null = 0) = 0;
 
-    /// @brief 
+    /// @brief
     ///   Get a descriptor for a BLOB column (for SendData).
-    /// 
-    /// @return 
+    ///
+    /// @return
     ///   Return NULL if this result doesn't (or can't) have a BLOB descriptor.
     ///
     /// @note
     ///   You need to call ReadItem (maybe even with buffer_size == 0)
     ///   before calling this method!
+    /// @deprecated
+    ///   Please use CDB_BlobDescriptor instead.
+    /// @sa
+    ///   CDB_BlobDescriptor
+    NCBI_DEPRECATED
     virtual I_BlobDescriptor* GetBlobDescriptor(void) = 0;
 
-    I_BlobDescriptor* GetImageOrTextDescriptor(void)
-        { return GetBlobDescriptor(); }
+    /// @deprecated
+    ///   Please use CDB_BlobDescriptor instead.
+    /// @sa
+    ///   CDB_BlobDescriptor
+    NCBI_DEPRECATED
+    I_BlobDescriptor* GetImageOrTextDescriptor(void);
 
     /// @brief 
     /// Skip result item
@@ -790,6 +821,8 @@ public:
     
     /// Set login timeout.
     ///
+    /// This timeout limits how long each login attempt can take.  (If you've
+    /// enabled retrying, the total time can be longer.)
     /// @param nof_secs
     ///   Timeout in seconds. If "nof_secs" is zero or is "too big" 
     ///    (depends on the underlying DB API), then set the timeout to infinite.
@@ -797,11 +830,13 @@ public:
     ///   FALSE on error.
     ///
     /// @sa
-    ///   GetLoginTimeout()
+    ///   GetLoginTimeout, SetTimeout, CDB_Connection::SetTimeout, CDB_Connection::SetCancelTimeout
     virtual bool SetLoginTimeout(unsigned int nof_secs = 0) = 0;
 
-    /// Set connection timeouts.
+    /// Set connection timeout.
     ///
+    /// This timeout limits how long each query or the like can take by default
+    /// in subsequently created connections.
     /// @param nof_secs
     ///   Timeout in seconds. If "nof_secs" is zero or is "too big" 
     ///   (depends on the underlying DB API), then set the timeout to infinite.
@@ -809,24 +844,28 @@ public:
     ///   FALSE on error.
     ///
     /// @sa
-    ///   GetTimeout()
+    ///   GetTimeout, SetLoginTimeout, CDB_Connection::SetTimeout, CDB_Connection::SetCancelTimeout
     virtual bool SetTimeout(unsigned int nof_secs = 0) = 0;
 
     /// Get login timeout
     ///
+    /// This timeout limits how long each login attempt can take.  (If you've
+    /// enabled retrying, the total time can be longer.)
     /// @return
     ///   Login timeout.
     /// @sa
-    ///   SetLoginTimeout()
+    ///   SetLoginTimeout, GetTimeout, CDB_Connection::GetTimeout, CDB_Connection::GetCancelTimeout
     virtual unsigned int GetLoginTimeout(void) const = 0;
 
     /// Get connection timeout
     ///
+    /// This timeout limits how long each query or the like can take by default
+    /// in subsequently created connections.
     /// @return
     ///   Connection timeout.
     ///
     /// @sa
-    ///   SetTimeout()
+    ///   SetTimeout, GetLoginTimeout, CDB_Connection::GetTimeout, CDB_Connection::GetCancelTimeout
     virtual unsigned int GetTimeout(void) const = 0;
 
     /// Set maximal size for BLOB data. 
@@ -1344,12 +1383,15 @@ protected:
     /// @brief 
     ///   Set connection timeout.    
     /// 
+    /// This timeout limits how long each query or the like can take.
     /// @param nof_secs 
     ///   Number of seconds.  If "nof_secs" is zero or is "too big" 
     ///   (depends on the underlying DB API), then set the timeout to infinite.
     virtual void SetTimeout(size_t nof_secs) = 0;
 
     /// Get connection timeout.    
+    ///
+    /// This timeout limits how long each query or the like can take.
     virtual size_t GetTimeout(void) const = 0;
 
     /// Get interface for extra features that could be implemented in the driver.
@@ -1392,4 +1434,9 @@ END_NCBI_SCOPE
 /* @} */
 
 
+#if defined(_MSC_VER) && _MSC_VER < 1910
+    #pragma warning(pop)
+#endif
+
+
 #endif  /* DBAPI_DRIVER___INTERFACES__HPP */
