@@ -1,7 +1,7 @@
 #ifndef DBAPI___VARIANT__HPP
 #define DBAPI___VARIANT__HPP
 
-/* $Id: variant.hpp 535289 2017-05-08 13:38:35Z ivanov $
+/* $Id: variant.hpp 563801 2018-05-15 15:55:00Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -80,7 +80,11 @@ public:
 
 enum EDateTimeFormat {
     eShort,
-    eLong
+    eLong,
+    eLonger,
+    eDateOnly,
+    eTimeOnly,
+    eDateTimeOffset
 };
 
 
@@ -130,6 +134,7 @@ public:
     static CVariant Binary       (size_t size, const void *p, size_t len);
     static CVariant SmallDateTime(CTime *p);
     static CVariant DateTime     (CTime *p);
+    static CVariant BigDateTime  (CTime *p, EDateTimeFormat fmt = eLonger);
     static CVariant Numeric      (unsigned int precision,
                                   unsigned int scale,
                                   const char* p);
@@ -281,7 +286,7 @@ inline I_BlobDescriptor&
 CVariant::GetBlobDescriptor(void) const
 {
     if (m_descr.get() == NULL) {
-        NCBI_THROW(CVariantException, eVariant,
+        NCBI_THROW(CVariantException, eVariant | Retriable(eRetriable_No),
                    "No blob descriptor available.");
     }
     return *m_descr;
@@ -297,7 +302,7 @@ inline
 void CVariant::x_Verify_AssignType(EDB_Type db_type, const char* cxx_type) const
 {
     if (db_type == eDB_UnsupportedType  ||  db_type != GetType()) {
-        NCBI_THROW(CVariantException, eVariant,
+        NCBI_THROW(CVariantException, eVariant  | Retriable(eRetriable_No),
              "Cannot assign type '" + string(cxx_type) + "' to type '"
              + string(CDB_Object::GetTypeName(GetType())) + "'");
     }
@@ -306,7 +311,7 @@ void CVariant::x_Verify_AssignType(EDB_Type db_type, const char* cxx_type) const
 inline
 void CVariant::x_Inapplicable_Method(const char* method) const
 {
-    NCBI_THROW(CVariantException, eVariant,
+    NCBI_THROW(CVariantException, eVariant | Retriable(eRetriable_No),
         "CVariant::" + string(method) + " is not applicable to type '"
         + string(CDB_Object::GetTypeName(GetType())) + "'");
 }
