@@ -1,7 +1,7 @@
 #ifndef ALIGN_CI__HPP
 #define ALIGN_CI__HPP
 
-/*  $Id: align_ci.hpp 439368 2014-06-27 17:33:59Z vasilche $
+/*  $Id: align_ci.hpp 536422 2017-05-18 14:44:05Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -133,6 +133,18 @@ public:
     CAlign_CI(const CSeq_annot_Handle& annot,
               const SAnnotSelector& sel);
 
+    /// Iterate all alignments from the seq-annot that annotate the location
+    CAlign_CI(const CSeq_loc& loc,
+              const CSeq_annot_Handle& annot);
+
+    /// Iterate all alignments from the seq-annot that annotate the location
+    ///
+    /// @sa
+    ///   SAnnotSelector
+    CAlign_CI(const CSeq_loc& loc,
+              const CSeq_annot_Handle& annot,
+              const SAnnotSelector& sel);
+
     /// Create an iterator that enumerates CSeq_align objects
     /// from the seq-entry regardless of their location
     explicit
