@@ -1,7 +1,7 @@
 #ifndef OBJMGR_IMPL_SCOPE_IMPL__HPP
 #define OBJMGR_IMPL_SCOPE_IMPL__HPP
 
-/*  $Id: scope_impl.hpp 534487 2017-04-27 17:21:02Z vasilche $
+/*  $Id: scope_impl.hpp 554030 2017-12-27 16:11:11Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -295,7 +295,7 @@ public:
     void ResetScope(void); // reset scope in initial state (no data)
     void ResetHistory(int action); // CScope::EActionIfLocked
     void ResetDataAndHistory(void);
-    void RemoveFromHistory(CTSE_Handle tse);
+    void RemoveFromHistory(const CTSE_Handle& tse, int action);
 
     // Revoke data sources from the scope. Throw exception if the
     // operation fails (e.g. data source is in use or not found).
@@ -303,7 +303,7 @@ public:
                           int action); // CScope::EActionIfLocked
     // Remove TSE previously added using AddTopLevelSeqEntry() or
     // AddBioseq().
-    void RemoveTopLevelSeqEntry(CTSE_Handle entry);
+    void RemoveTopLevelSeqEntry(const CTSE_Handle& entry);
 
     CSeq_entry_Handle GetSeq_entryHandle(CDataLoader* loader,
                                          const CBlobIdKey& blob_id,
@@ -326,6 +326,7 @@ public:
 
     typedef int TGetFlags;
 
+    bool Exists(const CSeq_id_Handle& id);
     // Get "native" bioseq ids without filtering and matching.
     TIds GetIds(const CSeq_id_Handle& idh, TGetFlags flags = 0);
     CSeq_id_Handle GetAccVer(const CSeq_id_Handle& idh, TGetFlags flags);
@@ -534,6 +535,12 @@ public:
                 CRef<CDataSource_ScopeInfo> ds2,
                 const CTSE_ScopeInfo* replaced_tse = 0);
 
+    bool GetKeepExternalAnnotsForEdit() const
+    {
+        return m_KeepExternalAnnotsForEdit;
+    }
+    void SetKeepExternalAnnotsForEdit(bool keep = true);
+
 private:
     // Get bioseq handles for sequences from the given TSE using the filter
     typedef vector<CBioseq_Handle> TBioseq_HandleSet;
@@ -623,6 +630,7 @@ private:
 
     int m_BioseqChangeCounter;
     int m_AnnotChangeCounter;
+    bool m_KeepExternalAnnotsForEdit;
 
     friend class CScope;
     friend class CHeapScope;
