@@ -1,7 +1,7 @@
 #ifndef OBJMGR_SCOPE__HPP
 #define OBJMGR_SCOPE__HPP
 
-/*  $Id: scope.hpp 493168 2016-02-24 19:28:38Z vasilche $
+/*  $Id: scope.hpp 554030 2017-12-27 16:11:11Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -351,12 +351,14 @@ public:
     /// live handles referencing the TSE, nothing is removed.
     /// @param tse
     ///  TSE to be removed from the cache.
-    void RemoveFromHistory(const CTSE_Handle& tse);
+    void RemoveFromHistory(const CTSE_Handle& tse,
+                           EActionIfLocked action = eKeepIfLocked);
     /// Remove the bioseq's TSE from the scope's history. If there are other
     /// live handles referencing the TSE, nothing is removed.
     /// @param bioseq
     ///  Bioseq, which TSE is to be removed from the cache.
-    void RemoveFromHistory(const CBioseq_Handle& bioseq);
+    void RemoveFromHistory(const CBioseq_Handle& bioseq,
+                           EActionIfLocked action = eKeepIfLocked);
 
     /// Revoke data loader from the scope. Throw exception if the
     /// operation fails (e.g. data source is in use or not found).
@@ -424,6 +426,10 @@ public:
         eForceLabelLoad   = eForceLoad
     };
 
+    /// Check existence of sequence with this id
+    bool Exists(const CSeq_id&        id);
+    bool Exists(const CSeq_id_Handle& id);
+
     /// Get "native" bioseq ids without filtering and matching.
     /// Returns empty ids if the sequence is not found
     /// @sa EGetflags
@@ -613,6 +619,46 @@ public:
 
     void UpdateAnnotIndex(void);
 
+    /// Return current application-wide KeepExternalAnnotsForEdit flag.
+    /// 
+    /// By default edited sequences lose their connection to external annotations.
+    /// This behavior can be changed for all newly created scopes by calling
+    /// SetDefaultKeepExternalAnnotsForEdit(true), and current application-wide
+    /// default can be retrieved by GetDefaultKeepExternalAnnotsForEdit().
+    /// @sa
+    ///   SetDefaultKeepExternalAnnotsForEdit(), SetKeepExternalAnnotsForEdit()
+    static bool GetDefaultKeepExternalAnnotsForEdit();
+    
+    /// Set new application-wide KeepExternalAnnotsForEdit flag.
+    /// 
+    /// By default edited sequences lose their connection to external annotations.
+    /// This behavior can be changed for all newly created scopes by calling
+    /// SetDefaultKeepExternalAnnotsForEdit(true), and current application-wide
+    /// default can be retrieved by GetDefaultKeepExternalAnnotsForEdit().
+    /// @sa
+    ///   GetDefaultKeepExternalAnnotsForEdit(), SetKeepExternalAnnotsForEdit()
+    static void SetDefaultKeepExternalAnnotsForEdit(bool keep = true);
+
+    /// Return this scope's KeepExternalAnnotsForEdit flag.
+    /// 
+    /// By default edited sequences lose their connection to external annotations.
+    /// This behavior can be changed for any scope by calling
+    /// SetKeepExternalAnnotsForEdit(true), and current scope's behavior
+    /// can be retrieved by GetKeepExternalAnnotsForEdit().
+    /// @sa
+    ///   SetDefaultKeepExternalAnnotsForEdit(), SetKeepExternalAnnotsForEdit()
+    bool GetKeepExternalAnnotsForEdit() const;
+
+    /// Change this scope's KeepExternalAnnotsForEdit flag.
+    /// 
+    /// By default edited sequences lose their connection to external annotations.
+    /// This behavior can be changed for any scope by calling
+    /// SetKeepExternalAnnotsForEdit(true), and current scope's behavior
+    /// can be retrieved by GetKeepExternalAnnotsForEdit().
+    /// @sa
+    ///   GetDefaultKeepExternalAnnotsForEdit(), GetKeepExternalAnnotsForEdit()
+    void SetKeepExternalAnnotsForEdit(bool keep = true);
+
 protected:
     CScope_Impl& GetImpl(void);
 
