@@ -1,7 +1,7 @@
 #ifndef FEAT__HPP
 #define FEAT__HPP
 
-/*  $Id: feature.hpp 497406 2016-04-06 15:30:54Z ludwigf $
+/*  $Id: feature.hpp 563762 2018-05-15 11:20:48Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -180,9 +180,17 @@ public:
     /// Construct a tree with features from a Seq-annot
     explicit
     CFeatTree(const CSeq_annot_Handle& sah);
+    /// Construct a tree with features from a Seq-annot, restricted by an annot
+    /// selector.
+    explicit
+    CFeatTree(const CSeq_annot_Handle& sah, const SAnnotSelector& sel);
     /// Construct a tree with features from a Seq-entry
     explicit
     CFeatTree(const CSeq_entry_Handle& seh);
+    /// Construct a tree with features from a Seq-entry, restricted by an annot
+    /// selector.
+    explicit
+    CFeatTree(const CSeq_entry_Handle& seh, const SAnnotSelector& sel);
     /// Destructor.
     ~CFeatTree(void);
 
@@ -332,12 +340,21 @@ public:
         bool IsSetParent(void) const {
             return m_IsSetParent;
         }
+        bool IsSetGene(void) const {
+            return m_IsSetGene;
+        }
         CSeqFeatData::ESubtype GetSubtype(void) const {
             return m_Feat.GetFeatSubtype();
         }
         bool IsGene(void) const {
             return GetSubtype() == CSeqFeatData::eSubtype_gene;
         }
+        bool GivesGeneToChildren(void) const {
+            return IsGene() || IsSetGene();
+        }
+        CFeatInfo* GetChildrenGene(void) {
+            return IsGene()? this: m_Gene;
+        }
 
         typedef vector<CFeatInfo*> TChildren;
         
@@ -345,7 +362,7 @@ public:
         CMappedFeat m_Feat;
         CRange<TSeqPos> m_MasterRange;
         bool m_CanMatchByQual;
-        bool m_IsSetParent, m_IsSetChildren, m_MultiId;
+        bool m_IsSetParent, m_IsSetGene, m_IsSetChildren, m_MultiId;
         enum EIsLinkedToRoot NCBI_PACKED_ENUM_TYPE(Int1) {
             eIsLinkedToRoot_unknown,
             eIsLinkedToRoot_linked,
@@ -384,7 +401,8 @@ protected:
     void x_SetNoParent(CFeatInfo& info);
     CFeatInfo* x_GetParent(CFeatInfo& info);
     const TChildren& x_GetChildren(CFeatInfo& info);
-    void x_SetGeneRecursive(CFeatInfo& info, CFeatInfo& gene);
+    void x_SetGeneRecursive(CFeatInfo& info, CFeatInfo* gene);
+    void x_SetGene(CFeatInfo& info, CFeatInfo* gene);
 
     typedef map<CSeq_feat_Handle, CFeatInfo> TInfoMap;
     typedef vector<CFeatInfo*> TInfoArray;
@@ -556,6 +574,16 @@ NCBI_XOBJUTIL_EXPORT bool AdjustProteinMolInfoToMatchCDS(CMolInfo& molinfo, cons
 /// @return           Boolean to indicate whether the Seq-entry-handle was changed
 NCBI_XOBJUTIL_EXPORT bool AdjustForCDSPartials(const CSeq_feat& cds, CSeq_entry_Handle seh);
 
+/// AdjustForCDSPartials
+/// A function to make all of the necessary related changes to
+/// a Seq-entry after the partialness of a coding region has been
+/// changed.
+/// @param cds        The feature for which adjustments are to be made
+/// @param scope      The scope in which to find the protein sequence
+///
+/// @return           Boolean to indicate whether the Seq-entry-handle was changed
+NCBI_XOBJUTIL_EXPORT bool AdjustForCDSPartials(const CSeq_feat& cds, CScope& scope);
+
 /// RetranslateCDS
 /// A function to replace the protein Bioseq pointed to by cds.product
 /// with the current translation of the coding region cds.
