@@ -66,7 +66,7 @@ class CFeatureIndex;
 // fetch policy and feature collection flags, as well as an optional feature exploration
 // depth parameter (for the default adaptive fetch policy):
 //
-//   CSeqEntryIndex idx(*m_entry, CSeqEntryIndex::fAdaptive);
+//   CSeqEntryIndex idx(*m_entry, CSeqEntryIndex::eAdaptive);
 //
 // A Seq-entry wrapper is created if the top-level object is a Bioseq or Bioseq-set.
 // Bioseqs within the Seq-entry are then indexed and added to a vector of CBioseqIndex.
@@ -89,20 +89,23 @@ public:
 
     enum EPolicy {
         // far feature fetch policy
-        fAdaptive = 0,
-        fInternal = 1,
-        fExternal = 2,
-        fExhaustive = 3
+        eAdaptive = 0,
+        eInternal = 1,
+        eExternal = 2,
+        eExhaustive = 3
     };
 
     enum EFlags {
-        fDefault =          0,
-        fHideImpFeats =     1,
-        fHideSNPFeats =     2,
-        fHideSTSFeats =     4,
-        fHideExonFeats =    8,
-        fHideIntronFeats = 16,
-        fHideMiscFeats =   32
+        fDefault =           0,
+        fHideImpFeats =      1,
+        fHideSNPFeats =      2,
+        fHideCDDFeats =      4,
+        fHideSTSFeats =      8,
+        fHideExonFeats =    16,
+        fHideIntronFeats =  32,
+        fHideMiscFeats =    64,
+        fShowSNPFeats =    128,
+        fShowCDDFeats =    256
     };
     typedef int TFlags; // Binary "OR" of EFlags
 
@@ -110,20 +113,21 @@ public:
     // Constructors take the top-level sequence object
 
     // The primary constructor uses an existing CScope created by the application
-    CSeqEntryIndex (CSeq_entry_Handle& topseh, EPolicy policy = fAdaptive, TFlags flags = fDefault, int depth = -1);
+    CSeqEntryIndex (CSeq_entry_Handle& topseh, EPolicy policy = eAdaptive, TFlags flags = fDefault, int depth = -1);
+    CSeqEntryIndex (CBioseq_Handle& bsh, EPolicy policy = eAdaptive, TFlags flags = fDefault, int depth = -1);
 
     // Alternative constructors take an object and create a new local default CScope
-    CSeqEntryIndex (CSeq_entry& topsep, EPolicy policy = fAdaptive, TFlags flags = fDefault, int depth = -1);
-    CSeqEntryIndex (CBioseq_set& seqset, EPolicy policy = fAdaptive, TFlags flags = fDefault, int depth = -1);
-    CSeqEntryIndex (CBioseq& bioseq, EPolicy policy = fAdaptive, TFlags flags = fDefault, int depth = -1);
-    CSeqEntryIndex (CSeq_submit& submit, EPolicy policy = fAdaptive, TFlags flags = fDefault, int depth = -1);
+    CSeqEntryIndex (CSeq_entry& topsep, EPolicy policy = eAdaptive, TFlags flags = fDefault, int depth = -1);
+    CSeqEntryIndex (CBioseq_set& seqset, EPolicy policy = eAdaptive, TFlags flags = fDefault, int depth = -1);
+    CSeqEntryIndex (CBioseq& bioseq, EPolicy policy = eAdaptive, TFlags flags = fDefault, int depth = -1);
+    CSeqEntryIndex (CSeq_submit& submit, EPolicy policy = eAdaptive, TFlags flags = fDefault, int depth = -1);
 
     // Specialized constructors are for streaming through release files, one component at a time
 
     // Submit-block obtained from top of Seq-submit release file
-    CSeqEntryIndex (CSeq_entry& topsep, CSubmit_block &sblock, EPolicy policy = fAdaptive, TFlags flags = fDefault, int depth = -1);
+    CSeqEntryIndex (CSeq_entry& topsep, CSubmit_block &sblock, EPolicy policy = eAdaptive, TFlags flags = fDefault, int depth = -1);
     // Seq-descr chain obtained from top of Bioseq-set release file
-    CSeqEntryIndex (CSeq_entry& topsep, CSeq_descr &descr, EPolicy policy = fAdaptive, TFlags flags = fDefault, int depth = -1);
+    CSeqEntryIndex (CSeq_entry& topsep, CSeq_descr &descr, EPolicy policy = eAdaptive, TFlags flags = fDefault, int depth = -1);
 
 private:
     // Prohibit copy constructor & assignment operator
@@ -165,6 +169,11 @@ public:
     // Check all Bioseqs for failure to fetch remote sequence components or feature annotation
     bool IsFetchFailure(void);
 
+    // Check for failure to create scope
+    bool IsIndexFailure (void);
+
+    CRef<CSeqMasterIndex> GetMasterIndex(void) const { return m_Idx; }
+
 private:
     // Implementation details are in a separate CSeqMasterIndex object wrapped in a CRef
     CRef<CSeqMasterIndex> m_Idx;
@@ -184,6 +193,7 @@ public:
 public:
     // Initializers take the top-level sequence object
     void x_Initialize (CSeq_entry_Handle& topseh, CSeqEntryIndex::EPolicy policy, CSeqEntryIndex::TFlags flags, int depth);
+    void x_Initialize (CBioseq_Handle& bsh, CSeqEntryIndex::EPolicy policy, CSeqEntryIndex::TFlags flags, int depth);
 
     void x_Initialize (CSeq_entry& topsep, CSeqEntryIndex::EPolicy policy, CSeqEntryIndex::TFlags flags, int depth);
     void x_Initialize (CBioseq_set& seqset, CSeqEntryIndex::EPolicy policy, CSeqEntryIndex::TFlags flags, int depth);
@@ -230,14 +240,24 @@ public:
     CConstRef<CSeq_entry> GetTopSEP (void) const { return m_Tsep; }
     CConstRef<CSubmit_block> GetSbtBlk (void) const { return m_SbtBlk; }
     CConstRef<CSeq_descr> GetTopDescr (void) const { return m_TopDescr; }
+    CRef<feature::CFeatTree> GetFeatTree (void) { return m_FeatTree; }
 
     const vector<CRef<CBioseqIndex>>& GetBioseqIndices(void);
 
     const vector<CRef<CSeqsetIndex>>& GetSeqsetIndices(void);
 
+    void SetHasOperon (bool hasOp) { m_HasOperon = hasOp; }
+    bool HasOperon (void) const { return m_HasOperon; }
+
+    bool IsSmallGenomeSet (void) const { return m_IsSmallGenomeSet; }
+
     // Check all Bioseqs for failure to fetch remote sequence components or remote feature annotation
     bool IsFetchFailure(void);
 
+    // Check for failure to create scope
+    bool IsIndexFailure (void) const { return m_IndexFailure; }
+    void SetIndexFailure (bool fails) { m_IndexFailure = fails; }
+
 private:
     // Common initialization function called by each Initialize variant
     void x_Init (void);
@@ -261,6 +281,7 @@ private:
     CConstRef<CSeq_entry> m_Tsep;
     CConstRef<CSubmit_block> m_SbtBlk;
     CConstRef<CSeq_descr> m_TopDescr;
+    CRef<feature::CFeatTree> m_FeatTree;
 
     CSeqEntryIndex::EPolicy m_Policy;
     CSeqEntryIndex::TFlags m_Flags;
@@ -278,7 +299,12 @@ private:
 
     vector<CRef<CSeqsetIndex>> m_SsxList;
 
+    bool m_HasOperon;
+    bool m_IsSmallGenomeSet;
+
     mutable CAtomicCounter m_Counter;
+
+    bool m_IndexFailure;
 };
 
 
@@ -382,36 +408,11 @@ public:
     CBioseq_Handle GetOrigBioseqHandle (void) const { return m_OrigBsh; }
     CRef<CSeqsetIndex> GetParent (void) const { return m_Prnt; }
     CRef<CScope> GetScope (void) const { return m_Scope; }
-    feature::CFeatTree& GetFeatTree (void) { return m_FeatTree; }
     CRef<CSeqVector> GetSeqVector (void) const { return m_SeqVec; }
 
     // Get master index
     CWeakRef<CSeqMasterIndex> GetSeqMasterIndex (void) const { return m_Idx; }
 
-    const string& GetAccession (void) const { return m_Accession; }
-
-    // Seq-inst fields
-    bool IsNA (void) const {  return m_IsNA; }
-    bool IsAA (void) const { return m_IsAA; }
-    CSeq_inst::TTopology GetTopology (void) const { return m_Topology; }
-    CSeq_inst::TLength GetLength (void) const { return m_Length; }
-
-    bool IsDelta (void) const { return m_IsDelta; }
-    bool IsVirtual (void) const { return m_IsVirtual; }
-    bool IsMap (void) const { return m_IsMap; }
-
-    // Most important descriptor fields
-
-    const string& GetTitle (void);
-
-    CConstRef<CMolInfo> GetMolInfo (void);
-    CMolInfo::TBiomol GetBiomol (void);
-    CMolInfo::TTech GetTech (void);
-    CMolInfo::TCompleteness GetCompleteness (void);
-
-    CConstRef<CBioSource> GetBioSource (void);
-    const string& GetTaxname (void);
-
     // Get sequence letters from Bioseq
     string GetSequence (void);
     void GetSequence (string& buffer);
@@ -437,14 +438,130 @@ public:
     // Get best (longest) protein feature on this protein Bioseq
     CRef<CFeatureIndex> GetBestProteinFeature(void);
 
-    // Check if Bioseq has BioSource features
-    bool HasSourceFeats (void);
-
     // Flag to indicate failure to fetch remote sequence components or feature annotation
     bool IsFetchFailure (void) const { return m_FetchFailure; }
 
     void SetFetchFailure (bool fails) { m_FetchFailure = fails; }
 
+public:
+    // Seq-inst fields
+    bool IsNA (void) const {  return m_IsNA; }
+    bool IsAA (void) const { return m_IsAA; }
+    CSeq_inst::TTopology GetTopology (void) const { return m_Topology; }
+    CSeq_inst::TLength GetLength (void) const { return m_Length; }
+
+    bool IsDelta (void) const { return m_IsDelta; }
+    bool IsVirtual (void) const { return m_IsVirtual; }
+    bool IsMap (void) const { return m_IsMap; }
+
+    // Seq-id fields
+    const string& GetAccession (void) const { return m_Accession; }
+
+    bool IsNC (void) const { return m_IsNC; }
+    bool IsNM (void) const { return m_IsNM; }
+    bool IsNR (void) const { return m_IsNR; }
+    bool IsPatent (void) const { return m_IsPatent; }
+    bool IsPDB (void) const { return m_IsPDB; }
+    bool IsWP (void) const { return m_IsWP; }
+    bool IsThirdParty (void) const { return m_ThirdParty; }
+    bool IsWGSMaster (void) const { return m_WGSMaster; }
+    bool IsTSAMaster (void) const { return m_TSAMaster; }
+    bool IsTLSMaster (void) const { return m_TLSMaster; }
+
+    string GetGeneralStr (void) const { return m_GeneralStr; }
+    int GetGeneralId (void) const { return m_GeneralId; }
+
+    string GetPatentCountry (void) const { return m_PatentCountry; }
+    string GetPatentNumber (void) const { return m_PatentNumber; }
+    int GetPatentSequence (void) const { return m_PatentSequence; }
+
+    int GetPDBChain (void) const { return m_PDBChain; }
+
+    // Most important descriptor fields
+
+    const string& GetTitle (void);
+
+    CConstRef<CMolInfo> GetMolInfo (void);
+    CMolInfo::TBiomol GetBiomol (void);
+    CMolInfo::TTech GetTech (void);
+    CMolInfo::TCompleteness GetCompleteness (void);
+
+    CConstRef<CBioSource> GetBioSource (void);
+    const string& GetTaxname (void);
+
+    const string& GetDescTaxname (void);
+
+    bool IsHTGTech (void);
+    bool IsHTGSUnfinished (void);
+    bool IsTLS (void);
+    bool IsTSA (void);
+    bool IsWGS (void);
+    bool IsEST_STS_GSS (void);
+
+    bool IsUseBiosrc (void);
+
+    const string& GetCommon (void);
+    const string& GetLineage (void);
+    int GetTaxid (void);
+    bool IsUsingAnamorph (void);
+
+    CTempString GetGenus (void);
+    CTempString GetSpecies (void);
+    bool IsMultispecies (void);
+    CBioSource::TGenome GetGenome (void);
+    bool IsPlasmid (void);
+    bool IsChromosome (void);
+
+    const string& GetOrganelle (void);
+
+    string GetFirstSuperKingdom (void);
+    string GetSecondSuperKingdom (void);
+    bool IsCrossKingdom (void);
+
+    CTempString GetChromosome (void);
+    CTempString GetLinkageGroup (void);
+    CTempString GetClone (void);
+    bool HasClone (void);
+    CTempString GetMap (void);
+    CTempString GetPlasmid (void);
+    CTempString GetSegment (void);
+
+    CTempString GetBreed (void);
+    CTempString GetCultivar (void);
+    CTempString GetIsolate (void);
+    CTempString GetStrain (void);
+    CTempString GetSubstrain (void);
+
+    bool IsHTGSCancelled (void);
+    bool IsHTGSDraft (void);
+    bool IsHTGSPooled (void);
+    bool IsTPAExp (void);
+    bool IsTPAInf (void);
+    bool IsTPAReasm (void);
+    bool IsUnordered (void);
+
+    CTempString GetPDBCompound (void);
+
+    bool IsForceOnlyNearFeats (void);
+
+    bool IsUnverified (void);
+    bool IsUnverifiedFeature (void);
+    bool IsUnverifiedOrganism (void);
+    bool IsUnverifiedMisassembled (void);
+    bool IsUnverifiedContaminant (void);
+
+    CTempString GetTargetedLocus (void);
+
+    const string& GetComment (void);
+    bool IsPseudogene (void);
+
+    bool HasOperon (void);
+    bool HasGene (void);
+    bool HasMultiIntervalGenes (void);
+    bool HasSource (void);
+
+    string GetrEnzyme (void);
+
 private:
     // Common gap collection, delayed until actually needed
     void x_InitGaps (void);
@@ -455,6 +572,9 @@ private:
     // Common feature collection, delayed until actually needed
     void x_InitFeats (void);
 
+    // Set BioSource flags
+    void x_InitSource (void);
+
 private:
     CBioseq_Handle m_Bsh;
     const CBioseq& m_Bsp;
@@ -473,17 +593,19 @@ private:
 
     bool m_FeatsInitialized;
     vector<CRef<CFeatureIndex>> m_SfxList;
-    feature::CFeatTree m_FeatTree;
 
+    bool m_SourcesInitialized;
+
+    bool m_FeatForProdInitialized;
     CRef<CFeatureIndex> m_FeatureForProduct;
+
+    bool m_BestProtFeatInitialized;
     CRef<CFeatureIndex> m_BestProteinFeature;
 
     // CFeatureIndex from CMappedFeat for use with GetBestGene
     typedef map<CMappedFeat, CRef<CFeatureIndex> > TFeatIndexMap;
     TFeatIndexMap m_FeatIndexMap;
 
-    bool m_HasSourceFeats;
-
     CRef<CSeqVector> m_SeqVec;
 
     CSeqEntryIndex::EPolicy m_Policy;
@@ -493,9 +615,6 @@ private:
     bool m_FetchFailure;
 
 private:
-    // Seq-id field
-    string m_Accession;
-
     // Seq-inst fields
     bool m_IsNA;
     bool m_IsAA;
@@ -506,6 +625,29 @@ private:
     bool m_IsVirtual;
     bool m_IsMap;
 
+    // Seq-id fields
+    string m_Accession;
+
+    bool m_IsNC;
+    bool m_IsNM;
+    bool m_IsNR;
+    bool m_IsPatent;
+    bool m_IsPDB;
+    bool m_IsWP;
+    bool m_ThirdParty;
+    bool m_WGSMaster;
+    bool m_TSAMaster;
+    bool m_TLSMaster;
+
+    string m_GeneralStr;
+    int m_GeneralId;
+
+    string m_PatentCountry;
+    string m_PatentNumber;
+    int m_PatentSequence;
+
+    int m_PDBChain;
+
     // Instantiated title
     string m_Title;
 
@@ -515,13 +657,92 @@ private:
     CMolInfo::TTech m_Tech;
     CMolInfo::TCompleteness m_Completeness;
 
+    bool m_HTGTech;
+    bool m_HTGSUnfinished;
+    bool m_IsTLS;
+    bool m_IsTSA;
+    bool m_IsWGS;
+    bool m_IsEST_STS_GSS;
+
+    bool m_UseBiosrc;
+
     // BioSource fields
+    CConstRef<CBioSource> m_DescBioSource;
+    string m_DescTaxname;
+
     CConstRef<CBioSource> m_BioSource;
     string m_Taxname;
 
+    string m_Common;
+    string m_Lineage;
+    int m_Taxid;
+    bool m_UsingAnamorph;
+
+    CTempString m_Genus;
+    CTempString m_Species;
+    bool m_Multispecies;
+    CBioSource::TGenome m_Genome;
+    bool m_IsPlasmid;
+    bool m_IsChromosome;
+
+    string m_Organelle;
+
+    string m_FirstSuperKingdom;
+    string m_SecondSuperKingdom;
+    bool m_IsCrossKingdom;
+
+    // Subsource fields
+    CTempString m_Chromosome;
+    CTempString m_LinkageGroup;
+    CTempString m_Clone;
+    bool m_has_clone;
+    CTempString m_Map;
+    CTempString m_Plasmid;
+    CTempString m_Segment;
+
+    // Orgmod fields
+    CTempString m_Breed;
+    CTempString m_Cultivar;
+    CTempString m_Isolate;
+    CTempString m_Strain;
+    CTempString m_Substrain;
+
+    // Keyword fields (genbank or embl blocks)
+    bool m_HTGSCancelled;
+    bool m_HTGSDraft;
+    bool m_HTGSPooled;
+    bool m_TPAExp;
+    bool m_TPAInf;
+    bool m_TPAReasm;
+    bool m_Unordered;
+
+    // PDB block fields
+    CTempString m_PDBCompound;
+
     // User object fields
     bool m_ForceOnlyNearFeats;
 
+    bool m_IsUnverified;
+    bool m_IsUnverifiedFeature;
+    bool m_IsUnverifiedOrganism;
+    bool m_IsUnverifiedMisassembled;
+    bool m_IsUnverifiedContaminant;
+    CTempString m_UnverifiedPrefix;
+
+    CTempString m_TargetedLocus;
+
+    // Comment fields
+    string m_Comment;
+    bool m_IsPseudogene;
+
+    // Feature fields
+    bool m_HasGene;
+    bool m_HasMultiIntervalGenes;
+    bool m_HasSource;
+
+    // Map fields
+    string m_rEnzyme;
+
     // true if this index is for a temporary subrange delta Bioseq
     bool m_Surrogate;
 };
@@ -656,6 +877,9 @@ public:
     // Map from feature to CFeatureIndex for best gene using CFeatTree in parent CBioseqIndex
     CRef<CFeatureIndex> GetBestGene (void);
 
+    // Find CFeatureIndex object for overlapping source feature using internal CFeatTree
+    CRef<CFeatureIndex> GetOverlappingSource (void);
+
 private:
     void SetFetchFailure (bool fails);
 
