@@ -1,7 +1,7 @@
 #ifndef SEQUENCE__HPP
 #define SEQUENCE__HPP
 
-/*  $Id: sequence.hpp 540892 2017-07-12 11:35:04Z ivanov $
+/*  $Id: sequence.hpp 567127 2018-07-13 17:55:39Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -44,8 +44,8 @@
 #include <objects/seqloc/Na_strand.hpp>
 #include <objects/seqloc/Seq_interval.hpp>
 #include <objects/seqloc/Seq_loc.hpp>
-#include <objects/seq/seq_id_mapper.hpp>
 #include <util/strsearch.hpp>
+#include <objects/seq/seq_id_mapper.hpp>
 #include <util/range_coll.hpp>
 #include <objmgr/feat_ci.hpp>
 #include <objmgr/bioseq_ci.hpp>
@@ -742,7 +742,7 @@ END_SCOPE(sequence)
 
 class NCBI_XOBJUTIL_EXPORT CFastaOstream {
 public:
-    enum EFlags {
+    enum EFlags : long {
         fAssembleParts      = 1 <<  0, ///< assemble FAR delta sequences; on by dflt
         fInstantiateGaps    = 1 <<  1, ///< honor specifed gap mode; on by default
         fSuppressRange      = 1 <<  2, ///< never include location details in defline
@@ -758,11 +758,14 @@ public:
         fShowGapsOfSizeZero = 1 << 12, ///< Use this to show gaps of size zero as a lone hyphen at the end of a line.
         fEnableGI           = 1 << 13, ///< Use this flag to enable GI output in the defline
         fHideGenBankPrefix  = 1 << 14, ///< Hide gb| prefix for genbank only seq_id's 
+        fHTMLEncode         = 1 << 15, ///< Encode the title string for HTML display
+        fIgnoreOriginalID   = 1 << 16, ///< Disregard original ID when constructing defline
         // historically misnamed as eFlagName
         eAssembleParts   = fAssembleParts,
-        eInstantiateGaps = fInstantiateGaps
+        eInstantiateGaps = fInstantiateGaps,
+        fUseAutoDef         = 1 << 17  ///< Disregard original ID when constructing defline
     };
-    typedef int TFlags; ///< binary OR of EFlags
+    typedef long TFlags; ///< binary OR of EFlags
 
     /// How to represent gaps with fInstantiateGaps enabled, as it is
     /// by default.  (Disabling fInstantiateGaps is equivalent to
@@ -944,9 +947,10 @@ public:
     /// @sa TTranslationFlags
     enum ETranslationFlags {
         fDefault = 0,
-        fNoStop = (1<<0), ///< = 0x1 Do not include stop in translation
+        fNoStop = (1<<0),          ///< = 0x1 Do not include stop in translation
         fRemoveTrailingX = (1<<1), ///< = 0x2 Remove trailing Xs from protein
-        fIs5PrimePartial = (1<<2) ///< = 0x4 Translate first codon even if not start codon (because sequence is 5' partial)
+        fIs5PrimePartial = (1<<2), ///< = 0x4 Translate first codon even if not start codon (because sequence is 5' partial)
+        fIs3PrimePartial = (1<<3)  ///< = 0x8 May not end in stop codon (because sequence is 3' partial)
     };
 
     typedef int TTranslationFlags;
@@ -977,7 +981,8 @@ public:
                           bool include_stop = true,
                           bool remove_trailing_X = false,
                           bool* alt_start = NULL,
-                          bool is_5prime_complete = true);
+                          bool is_5prime_complete = true,
+                          bool is_3prime_complete = true);
 
     /// Translate a string using a specified genetic code
     /// @param seq
@@ -1019,7 +1024,8 @@ public:
                           bool include_stop = true,
                           bool remove_trailing_X = false,
                           bool* alt_start = NULL,
-                          bool is_5prime_complete = true);
+                          bool is_5prime_complete = true,
+                          bool is_3prime_complete = true);
 
     /// Translate a seq-vector using a specified genetic code
     /// if the code is NULL, then the default genetic code is used
