@@ -1,4 +1,4 @@
-/*  $Id: align_format_util.hpp 548513 2017-10-16 11:11:13Z ivanov $
+/*  $Id: align_format_util.hpp 568473 2018-08-06 16:28:49Z zaretska $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -44,6 +44,7 @@
 #include <algo/blast/core/blast_stat.h>
 #include <objtools/align_format/ilinkoutdb.hpp>
 
+
 #ifdef _MSC_VER
 #define strcasecmp _stricmp
 #define strdup _strdup
@@ -66,10 +67,10 @@ static const char kClassInfo[] = "class=\"info\"";
 static const char kDefaultProtocol[] = "https:";
 ///entrez
 // .ncbirc alias: ENTREZ
-static const char kEntrezUrl[] = "<a title=\"Show report for <@acc@>\" <@cssInf@>href=\"<@protocol@>//www.ncbi.nlm.nih.gov/<@db@>/<@gi@>?report=genbank&log$=<@log@>&blast_rank=<@blast_rank@>&RID=<@rid@>\" <@target@>>";
+static const char kEntrezUrl[] = "<a title=\"Show report for <@acc@>\" <@cssInf@>href=\"<@protocol@>//www.ncbi.nlm.nih.gov/<@db@>/<@acc@>?report=genbank&log$=<@log@>&blast_rank=<@blast_rank@>&RID=<@rid@>\" <@target@>>";
 
 //.ncbirc alias: ENTREZ_TM
-static const char kEntrezTMUrl[] = "<@protocol@>//www.ncbi.nlm.nih.gov/<@db@>/<@gi@>?report=genbank&log$=<@log@>&blast_rank=<@blast_rank@>&RID=<@rid@>";
+static const char kEntrezTMUrl[] = "<@protocol@>//www.ncbi.nlm.nih.gov/<@db@>/<@acc@>?report=genbank&log$=<@log@>&blast_rank=<@blast_rank@>&RID=<@rid@>";
 
 //.ncbirc alias: WGS
 static const char kWGSUrl[] = "<@protocol@>//www.ncbi.nlm.nih.gov/nuccore/<@wgsacc@>";
@@ -91,7 +92,8 @@ using the Entrez Genomes MapViewer</td></tr></table><p>";
 
 ///unigene
 // .ncbirc alias: UNIGEN
-static const char kUnigeneUrl[] = "<a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=<@db@>&cmd=Display&dopt=<@dopt@>_unigene&from_uid=<@gi@>&RID=<@rid@>&log$=unigene<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>";
+static const char kUnigeneUrl[] = "<a class=\"gene\" term=\"<@uid@>\" href=\"<@protocol@>//www.ncbi.nlm.nih.gov/unigene/?<@termParam@>RID=<@rid@>&log$=unigene<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>\
+<input type=\"hidden\" value=\"<@label@>\" />";
 
 //substitues <@lnk_displ@>
 static const char kUnigeneImg[] = "<img border=0 height=16 width=16 src=\"images/U.gif\" alt=\"UniGene info linked to <@label@>\">";
@@ -115,7 +117,7 @@ gov/Structure/cblast/cblast.cgi?blast_RID=%s&blast_rep_gi=%d&hit=%d&%s\
 
 ///Geo
 // .ncbirc alias: GEO
-static const char kGeoUrl[] =  "<a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/geoprofiles/?LinkName=nucleotide_geoprofiles&from_uid=<@gi@>&RID=<@rid@>&log$=geo<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>";
+static const char kGeoUrl[] =  "<a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/geoprofiles/?term=genbank[Platform+Reporter+Type]+AND+<@label@>[Reporter+Identifier]&RID=<@rid@>&log$=geo<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>";
 
 
 //substitues <@lnk_displ@>
@@ -125,8 +127,10 @@ static const string kGeoDispl =  "<div><@lnk@>-<span class=\"rlLink\">microarray
 
 ///Gene
 // .ncbirc alias: GENE
-static const char kGeneUrl[] = "<a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/gene?term=<@gi@>[<@uid@>]&RID=<@rid@>&log$=gene<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>";
-
+//static const char kGeneUrl[] = "<a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/gene?term=<@label@>[<@uid@>]&RID=<@rid@>&log$=gene<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a><input type=\"hidden\" value=";
+static const char kGeneUrl[] = "<a class=\"gene\" term=\"<@uid@>\" href=\"<@protocol@>//www.ncbi.nlm.nih.gov/gene?<@termParam@>RID=<@rid@>&log$=gene<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>\
+<input type=\"hidden\" value=\"<@label@>\" />";
+static const char kGeneTerm[] = "term=<@label@><@uid@>&";
 //substitues <@lnk_displ@>
 static const char kGeneImg[] = "<img border=0 height=16 width=16 src=\"images/G.gif\" alt=\"Gene info linked to <@label@>\">";
 //For text link <@lnk@> is substituted by formatted url
@@ -162,13 +166,22 @@ static const string kMapviewBlastHitParams = "<a href=\"<@user_url@>&db=<@db@>&n
 
 ///Repr microbial Genome linkout
 // .ncbirc alias: REPR_MICROBIAL_GENOMES
-static const char kReprMicrobialGenomesUrl[] = "<a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/genome?term=<@gi@>[gi]&RID=<@rid@>&log$=map<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>";
+static const char kReprMicrobialGenomesUrl[] = "<a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/genome?term=<@label@>[<@uid@>]&RID=<@rid@>&log$=map<@log@>&blast_rank=<@blast_rank@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>";
 //substitues <@lnk_displ@>
 static const char kReprMicrobialGenomesImg[] = "<img border=0 height=16 width=16 src=\"images/L.gif\" alt=\"View genome information for <@label@>\">";
 //For text link <@lnk@> is substituted by formatted url
 static const string kReprMicrobialGenomesDispl =  "<div><@lnk@>-<span class=\"rlLink\">Genomic Sequence</span></div>";
 
-static const char kIdenticalProteinsUrl[] = "<a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/protein/<@gi@>?report=ipg\" title=\"View proteins identical to <@label@>\" <@lnkTarget@>><@lnk_displ@></a>";
+// .ncbirc alias: GENOME_DATA_VIEWER /genome/gdv/browser/?context=blast&id=NC_000019.10&alignid=<@label@>&rid=N9WGPH30015
+static const char kGenomeDataViewerNucUrl[] = "<span class=\"adNew\">New</span><a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/genome/gdv/browser/?context=blast&id=<@label@>&alignid=<@queryID@>&from=<@from@>&to=<@to@>&rid=<@rid@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>";
+static const char kGenomeDataViewerProtUrl[] = "<span class=\"adNew\">New</span><a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/genome/gdv/browser/?context=Protein&acc=<@label@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>";
+static const char kGenomeDataViewerNuclTranscriptUrl[] = "<span class=\"adNew\">New</span><a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/genome/gdv/browser/?context=nucleotide&acc=<@label@>\"<@lnkTitle@><@lnkTarget@>><@lnk_displ@></a>";
+//substitues <@lnk_displ@>
+static const char kGenomeDataViewerImg[] = "<div class=\"gdv\">V</div>";
+//For text link <@lnk@> is substituted by formatted url
+static const string kGenomeDataViewerDispl =  "<div><@lnk@>-<span class=\"rlLink\">aligned genomic context</span></div>";
+
+static const char kIdenticalProteinsUrl[] = "<a href=\"<@protocol@>//www.ncbi.nlm.nih.gov/ipg/<@label@>\" title=\"View proteins identical to <@label@>\" <@lnkTarget@>><@lnk_displ@></a>";
 static const string kIdenticalProteinsDispl =  "<div><@lnk@>-<span class=\"rlLink\">Identical proteins to <@label@></span></div>";
 
 
@@ -178,7 +191,7 @@ static const char kDownloadLink[] = "<a href=\"<@download_url@>&segs=<@segs@>\">
 //substitues <@lnk_displ@>
 static const char kDownloadImg[] = "<img border=0 height=16 width=16 src=\"images/D.gif\" alt=\"Download subject sequence <@label@> spanning the HSP\">";
 
-static const char kSeqViewerUrl[] = "<@protocol@>//www.ncbi.nlm.nih.gov/<@dbtype@>/<@gi@>?report=graph&rid=<@rid@>[<@gi@>]&<@seqViewerParams@>&v=<@from@>:<@to@>&appname=ncbiblast&link_loc=<@link_loc@>";
+static const char kSeqViewerUrl[] = "<@protocol@>//www.ncbi.nlm.nih.gov/<@dbtype@>/<@seqid@>?report=graph&rid=<@rid@>[<@seqid@>]&<@seqViewerParams@>&v=<@from@>:<@to@>&appname=ncbiblast&link_loc=<@link_loc@>";
 static const string kSeqViewerParams = "tracks=[key:sequence_track,name:Sequence,display_name:Sequence,id:STD1,category:Sequence,annots:Sequence,ShowLabel:true][key:gene_model_track,CDSProductFeats:false][key:alignment_track,name:other alignments,annots:NG Alignments|Refseq Alignments|Gnomon Alignments|Unnamed,shown:false]";
 
 static const char kSeqViewerUrlNonGi[] = "<@protocol@>//www.ncbi.nlm.nih.gov/projects/sviewer/?RID=<@rid@>&id=<@firstSeqID@>&<@seqViewerParams@>&v=<@from@>:<@to@>&appname=ncbiblast&link_loc=<@link_loc@>";
@@ -203,9 +216,8 @@ static const char kEntrezSubseqTMUrl[] = "<@protocol@>//www.ncbi.nlm.nih.gov/<@d
 
 ///Default linkout order 
 //.ncbirc alias: LINKOUT_ORDER
-static const char kLinkoutOrderStr[] = "G,U,E,S,B,R,M"; 
+static const char kLinkoutOrderStr[] = "G,U,E,S,B,R,M,V,T"; 
 
-///Bl2seq ************fix this and test titles Irena
 // .ncbirc alias: BL2SEQ
 //static const char kBl2seqUrl[] = "<a href=\"blast.ncbi.nlm.nih.gov/Blast.cgi?QUERY=<@query@>&SUBJECTS=<@subject@>&EXPECT=10&SHOW_OVERVIEW=on&OLD_BLAST=false&NEW_VIEW=on\">Get TBLASTX alignments</a>";
 static const char kBl2seqUrl[] = "<a href=\"blast.ncbi.nlm.nih.gov/Blast.cgi?QUERY=<@query@>&SUBJECTS=<@subject@>&PROGRAM=tblastx&EXPECT=10&CMD=request&SHOW_OVERVIEW=on&OLD_BLAST=false&NEW_VIEW=on\">Get TBLASTX alignments</a>";
@@ -284,6 +296,9 @@ static const TTagUrl s_TagUrls [] = {
   { "GENE",  kGeneUrl },
   { "GENE_INFO",  kGeneInfoUrl },
   { "GENOME_BTN",  kGenomeButton },
+  { "GENOME_DATA_VIEWER_NUC",  kGenomeDataViewerNucUrl },
+  { "GENOME_DATA_VIEWER_PROT", kGenomeDataViewerProtUrl },
+  { "GENOME_DATA_VIEWER_TRANSCR", kGenomeDataViewerNuclTranscriptUrl },  
   { "GEO",    kGeoUrl },
   { "GETSEQ_SEL_FRM",  k_GetSeqSelectForm },
   { "GETSEQ_SUB_FRM_0",  k_GetSeqSubmitForm_0 },
@@ -388,7 +403,7 @@ public:
         bool advancedView;      ///< bool indicating that advanced view design option should be used when contsructing links
         string seqUrl;          ///< sequence URL created
         string defline;         ///< sequence defline
-        
+        bool   hasTextSeqID;
         
         /// Constructor        
         SSeqURLInfo(string usurl,string bt, bool isnuc,string db, string rid_in,int qn, 
@@ -400,6 +415,73 @@ public:
 
     };
     
+    struct SLinkoutInfo {        
+        string rid;
+        string cdd_rid;
+        string entrez_term;
+        bool is_na;        
+        string database;
+        int query_number;        
+        string user_url;
+        string preComputedResID;
+
+        bool structure_linkout_as_group;
+        bool for_alignment;
+
+        int taxid;        
+        int cur_align;        
+        string taxName;
+        string gnl;
+        CRange<TSeqPos> subjRange;
+                
+        string linkoutOrder;
+        ILinkoutDB* linkoutdb;
+        string mv_build_name;
+
+        string giList;
+        string labelList;
+        TGi first_gi;
+                
+        string  queryID;        
+        
+        
+        void Init(string rid_in, string cdd_rid_in, string entrez_term_in, bool is_na_in, 
+                  string database_in, int query_number_in, string user_url_in, string preComputedResID_in, 
+                  string linkoutOrder_in, 
+                  bool structure_linkout_as_group_in = false, bool for_alignment_in = true) {
+            rid = rid_in;  
+            cdd_rid = cdd_rid_in;
+            entrez_term = entrez_term_in; 
+            is_na = is_na_in;
+            database = database_in;            
+            query_number = query_number_in;
+            user_url = user_url_in;  
+            preComputedResID = preComputedResID_in; 
+
+            linkoutOrder = linkoutOrder_in;            
+            //linkoutdb = linkoutdb_in; 
+            //mv_build_name = mv_build_name_in;
+
+
+            structure_linkout_as_group = structure_linkout_as_group_in;
+            for_alignment = for_alignment_in;
+        }
+
+        void Init(string rid_in, string cdd_rid_in, string entrez_term_in, bool is_na_in, 
+                  string database_in, int query_number_in, string user_url_in, string preComputedResID_in, 
+                  string linkoutOrder_in, ILinkoutDB* linkoutdb_in, string mv_build_name_in,                  
+                  bool structure_linkout_as_group_in = false, bool for_alignment_in = true) {
+
+            Init(rid_in,cdd_rid_in,entrez_term_in,is_na_in, 
+                  database_in,query_number_in, user_url_in,preComputedResID_in, 
+                  linkoutOrder_in, 
+                  structure_linkout_as_group_in, for_alignment_in);
+
+            linkoutdb = linkoutdb_in; 
+            mv_build_name = mv_build_name_in;
+        }
+    };
+
     ///Structure that holds information for all hits of one subject in Seq Align Set    
     struct SSeqAlignSetCalcParams {        
         //values used in descriptions display
@@ -701,6 +783,13 @@ public:
     ///@ret: bool=true if the match is found
     static bool MatchSeqInSeqList(TGi cur_gi, CRef<objects::CSeq_id> &seqID, list<string> &use_this_seq,bool *isGiList = NULL);
 
+    ///Matches string of seqIDs (gis or text seqID)    
+    ///@param alnSeqID: CSeq_id to extract label info to match
+    ///@param use_this_seq: list<string> containg gi:sssssss or seqid:sssssssss
+    ///@param seqList: string contaning comma separated seqIds
+    ///@ret: bool=true if the match is found
+    static bool MatchSeqInSeqList(CConstRef<objects::CSeq_id> &alnSeqID, list<string> &use_this_seq,vector <string> &seqList);
+
     ///Check if use_this_seq conatins gi list
     ///@param use_this_seq: list<string> containg gi:sssssss or seqid:sssssssss
     ///@ret: bool= true if use_this_seq conatins gi list
@@ -1032,6 +1121,16 @@ public:
 	                                                                 double evalueHigh,
 	                                                                 double percentIdentLow,
 	                                                                 double percentIdentHigh);
+
+    ///function for Filtering seqalign by specific subjects
+    ///@param source_aln
+    /// CSeq_align_set original seqalign
+    ///@param seqList 
+    /// vector of strings with seqIDs    
+    ///@return
+    /// CRef<CSeq_align_set> - filtered seq align
+    static CRef<objects::CSeq_align_set> FilterSeqalignBySeqList(objects::CSeq_align_set& source_aln,
+                                                               vector <string> &seqList);
     ///function for Limitting seqalign by hsps number
     ///(by default results are not cut off within the query)
     ///@param source_aln
@@ -1181,6 +1280,12 @@ public:
                                                  ILinkoutDB* linkoutdb,
                                                  const string& mv_build_name,
                                                  bool getIdentProteins);
+    static list<string> GetFullLinkoutUrl(const list< CRef< objects::CBlast_def_line > > &bdl, SLinkoutInfo &linkoutInfo);
+    static list<string> GetFullLinkoutUrl(objects::CBioseq::TId& cur_id,SLinkoutInfo &linkoutInfo,bool getIdentProteins);
+    static int GetSeqLinkoutInfo(objects::CBioseq::TId& cur_id,                                    
+                                    ILinkoutDB** linkoutdb, 
+                                    const string& mv_build_name,                                    
+                                    TGi gi = INVALID_GI);
     static int GetMasterCoverage(const objects::CSeq_align_set& alnset);
 	static CRange<TSeqPos> GetSeqAlignCoverageParams(const objects::CSeq_align_set& alnset,int *masterCoverage,bool *flip);
 												
@@ -1366,7 +1471,10 @@ public:
     static TGi GetDisplayIds(const list< CRef< objects::CBlast_def_line > > &bdl,
                                               const objects::CSeq_id& aln_id,
                                               list<TGi>& use_this_gi);
+    static bool GetTextSeqID(const list<CRef<objects::CSeq_id> > & ids,string *textSeqID = NULL);
+    static bool GetTextSeqID(CConstRef<objects::CSeq_id> seqID,string *textSeqID = NULL);
     
+
     ///Scan the the list of blast deflines and find seqID to be use in display    
     ///@param handle: CBioseq_Handle [in]
     ///@param aln_id: CSeq_id object for alignment seq [in]
