@@ -1,4 +1,4 @@
-/*  $Id: showalign.hpp 520798 2016-12-01 17:44:42Z zaretska $
+/*  $Id: showalign.hpp 553793 2017-12-21 16:51:53Z zaretska $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -536,12 +536,14 @@ protected:
     struct SAlnDispParams: public CObject {
         CAlignFormatUtil::SSeqURLInfo * seqUrlInfo;
         TGi gi;                         ///< gi used in defline
-        CRef<objects::CSeq_id> seqID;    ///< seqID used in defline
+        CRef<objects::CSeq_id> seqID;   ///< seqID used in defline
+        list<CRef<objects::CSeq_id> > ids;  ///
         string label;                   ///< sequence label
         string    id_url;                 ///< entrz, mapview etc id url
         string linkoutStr;              ///< string containing all linkout urls
         string dumpGnlUrl;              ///< download sequnce url
         string title;                   ///< sequnce title
+        bool hasTextSeqID;
     };
 
     ///Info used to contstruct seq url obtained from processing the whole seq align
@@ -631,6 +633,7 @@ protected:
     list <string> m_HSPLinksList;
     string m_FASTAlinkUrl;
     string m_AlignedRegionsUrl;
+    CAlignFormatUtil::SLinkoutInfo m_LinkoutInfo;
     
 
     CRef < objects::CObjectManager > m_FeatObj;  // used for fetching feature
@@ -716,11 +719,12 @@ protected:
     ///    
 
 
-    CAlignFormatUtil::SSeqURLInfo *x_InitSeqUrl(TGi giToUse,string accession,int linkout,
+    CAlignFormatUtil::SSeqURLInfo *x_InitSeqUrl(TGi giToUse,string accession,
         int taxid,const list<CRef<objects::CSeq_id> >& ids);
 
     string x_HTMLSeqIDLink(SAlnRowInfo *alnRoInfo, int row,TGi giToUse);
-    void x_InitAlignLinks(CAlignFormatUtil::SSeqURLInfo *seqUrlInfo,const list< CRef<objects::CBlast_def_line > > &bdl_list,CRef<objects::CSeq_id>  &seqID,int lnkDispPrarms);
+    void x_InitAlignLinks(SAlnDispParams *alnDispParams,const list< CRef<objects::CBlast_def_line > > &bdl_list,int lnkDispPrarms);
+
 
     ///get dumpgnl url to sequence record
     ///@param ids: id list
@@ -859,6 +863,9 @@ protected:
     ///
     void x_FillSeqid(string& id, int row) const;
 
+    
+    int x_GetLinkout(const objects::CSeq_id &  id);
+
     ///print out features and fill master_feat_str if applicable
     ///@param feature: the feature info
     ///@param row: row num
@@ -951,7 +958,8 @@ protected:
     SAlnDispParams *x_FillAlnDispParams(const CRef< objects::CBlast_def_line > &iter,
                                         const objects::CBioseq_Handle& bsp_handle,								       
                                         list<string> &use_this_seqid,
-								        TGi firstGi);
+								        TGi firstGi,
+                                        int numLinkoutReads);
 								   
     
 	SAlnDispParams *x_FillAlnDispParams(const objects::CBioseq_Handle& bsp_handle);	
@@ -972,7 +980,7 @@ protected:
     void x_DisplaySequenceIDForPairwise(SAlnRowInfo *alnRoInfo, int row, bool has_mismatch, CNcbiOstrstream &out);
     void x_DisplayInsertsForQueryAnchored(SAlnRowInfo *alnRoInfo, int row, CNcbiOstrstream &out);
     void x_DisplaySequenceLine(SAlnRowInfo *alnRoInfo, int row, int prev_stop,CNcbiOstrstream &out);
-    void x_DisplayMiddLine(SAlnRowInfo *alnRoInfo, int row, CNcbiOstrstream &out);
+    void x_DisplayMiddLine(SAlnRowInfo *alnRoInfo, int row, CNcbiOstrstream &out);    
 };
 
 
