@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_READER___SEQDB__HPP
 #define OBJTOOLS_BLAST_SEQDB_READER___SEQDB__HPP
 
-/*  $Id: seqdb.hpp 536658 2017-05-22 15:48:20Z zaretska $
+/*  $Id: seqdb.hpp 553487 2017-12-18 14:23:38Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -564,6 +564,16 @@ public:
                    vector<int> & taxids,
                    bool          persist = false) const;
 
+    /// Get all tax ids for an oid
+    ///
+    /// This includes leaf and non-leaf tax ids associated with the oid
+    /// @param oid
+    ///   The ordinal id of the sequence.
+    /// @param taxids
+    ///   A returned a set of taxids.
+    void GetAllTaxIDs(int           oid,
+                      set<int> & taxids) const;
+
     /// Get a CBioseq for a sequence.
     ///
     /// This builds and returns the header and sequence data
@@ -999,6 +1009,8 @@ public:
     /// Translate an Accession to a list of OIDs.
     void AccessionToOids(const string & acc, vector<int> & oids) const;
 
+    void AccessionsToOids(const vector<string>& accs, vector<blastdb::TOid>& oids) const;
+
     /// Translate a Seq-id to a list of OIDs.
     void SeqidToOids(const CSeq_id & seqid, vector<int> & oids) const;
 
@@ -1429,6 +1441,18 @@ public:
     /// @sa CDebugDumpable
     void DebugDump(CDebugDumpContext ddc, unsigned int depth) const;
 
+    /// Return blast db version
+    EBlastDbVersion GetBlastDbVersion() const;
+
+    /// Get Oid list for input tax ids
+    /// @param tax_ids	taxonomy ids, return only tax ids found in db
+    // @param rv		oids corrpond to tax ids
+    void TaxIdsToOids(set<Int4>& tax_ids, vector<blastdb::TOid>& rv) const;
+
+    /// Get all unique tax ids from db
+    /// @param tax_ids	return taxonomy ids in db
+    void GetDBTaxIds(set<Int4> & tax_ids) const;
+
 protected:
     /// Implementation details are hidden.  (See seqdbimpl.hpp).
     class CSeqDBImpl * m_Impl;
