@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_BLAST_SEQDB_WRITER___WRITEDB__HPP
 #define OBJTOOLS_BLAST_SEQDB_WRITER___WRITEDB__HPP
 
-/*  $Id: writedb.hpp 513693 2016-09-14 15:53:17Z boratyng $
+/*  $Id: writedb.hpp 553487 2017-12-18 14:23:38Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -40,6 +40,7 @@
 
 #include <objtools/blast/seqdb_writer/writedb_error.hpp>
 #include <objtools/blast/seqdb_reader/seqdbblob.hpp>
+#include <objtools/blast/seqdb_reader/seqdbcommon.hpp>
 #include <objects/blastdb/Blast_def_line.hpp>
 #include <objects/blastdb/Blast_def_line_set.hpp>
 #include <objects/blastdb/defline_extra.hpp>
@@ -155,7 +156,8 @@ public:
              int            itype = eDefault,
              bool           parse_ids = true,
              bool           long_ids = false,
-             bool           use_gi_mask = false);
+             bool           use_gi_mask = false,
+             EBlastDbVersion    dbver = eBDB_Version4);
 
     /// Destructor.
     ///
