@@ -1,7 +1,7 @@
 #ifndef CLEANUP___CLEANUP__HPP
 #define CLEANUP___CLEANUP__HPP
 
-/*  $Id: cleanup.hpp 541349 2017-07-17 16:05:30Z ivanov $
+/*  $Id: cleanup.hpp 567012 2018-07-11 17:07:11Z bollin $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,7 +50,9 @@ class CSeq_annot;
 class CSeq_feat;
 class CSeq_submit;
 class COrgName;
-
+class CSubmit_block;
+class CAuthor;
+class CName_std;
 
 class CSeq_entry_Handle;
 class CBioseq_Handle;
@@ -99,6 +101,8 @@ public:
     CConstRef<CCleanupChange> BasicCleanup(CSeq_feat& sf,   Uint4 options = 0);
     /// Cleanup a BioSource. 
     CConstRef<CCleanupChange> BasicCleanup(CBioSource& src,   Uint4 options = 0);
+	// Cleanup a Submit-block
+	CConstRef<CCleanupChange> BasicCleanup(CSubmit_block& block, Uint4 options = 0);
 
     // Handle versions.
     CConstRef<CCleanupChange> BasicCleanup(CSeq_entry_Handle& seh, Uint4 options = 0);
@@ -300,6 +304,7 @@ public:
 
     static void SetProteinName(CProt_ref& prot, const string& protein_name, bool append);
     static void SetProteinName(CSeq_feat& cds, const string& protein_name, bool append, CScope& scope);
+    static void SetMrnaName(CSeq_feat& mrna, const string& protein_name);
     static const string& GetProteinName(const CProt_ref& prot);
     static const string& GetProteinName(const CSeq_feat& cds, CScope& scope);
 
@@ -362,7 +367,7 @@ public:
 /// Performs WGS specific cleanup
 /// @param entry Seq-entry to edit
 /// @return Boolean return value indicates whether object was updated
-    static bool WGSCleanup(CSeq_entry_Handle entry);
+    static bool WGSCleanup(CSeq_entry_Handle entry, bool instantiate_missing_proteins = true, Uint4 options = 0);
 
 /// For table2asn -c s
 /// Adds an exception of "low-quality sequence region" to coding regions
@@ -512,6 +517,69 @@ public:
 /// @return bool indicates changes were made
     static bool MakeIRDFeatsFromSourceXrefs(CSeq_entry_Handle entry);
 
+/// From GB-7563
+/// An action has been requested that will do the following:
+///    1. This action should be limited to protein sequences where the product
+///       is an exact match to a specified text (the usual string constraint 
+///       is not needed).
+///    2. Protein sequences for which the coding region is 5' partial should 
+///       not be affected.
+///    3. When the protein name matches, the following actions should be taken
+///       if and only if the first amino acid of the protein sequence is not
+///       M (methionine):
+///     a. The first amino acid of the protein sequence should be changed to 
+///       methionine.
+///     b. The coding region should have the text "RNA editing" added to 
+///       Seq-feat.except_text (separated from any existing text by a semicolon).
+///       If Seq-feat.except is not already true, it should be set to true.
+///     c. A code-break should be added to Cdregion.code-break where the 
+///       Code-break.loc is the location of the first codon of the coding region
+///       and Code-break.aa is ncbieaa 'M' (Indexers will refer to "code-breaks"
+///       as "translation exceptions" because these appear in the flatfile as a
+///       /transl_except qualifier.
+///
+/// It will be the responsibility of the caller to only invoke this function
+/// for coding regions where the product name is a match, and the protein sequence
+/// does not already start with an M.
+
+    static bool FixRNAEditingCodingRegion(CSeq_feat& cds);
+
+    /// utility function for setting code break location given offset
+    /// pos is the position of the amino acid where the translation exception 
+    /// occurs (starts with 1)
+    static void SetCodeBreakLocation(CCode_break& cb, size_t pos, const CSeq_feat& cds);
+
+    static bool IsMethionine(const CCode_break& cb);
+
+    /// utility function for finding the code break for a given amino acid position
+    /// pos is the position of the amino acid where the translation exception 
+    /// occurs (starts with 1)
+    static CConstRef<CCode_break> GetCodeBreakForLocation(size_t pos, const CSeq_feat& cds);
+    
+    // From the request in GB-7166, we want to be able to move /gene 
+    // qualifiers that have been added to the coding region but not the
+    // parent gene to the parent gene.
+    // If the coding region also has /locus_tag qualifier which is different 
+    // from the one on the parent gene features, do not move the qualifier.
+    // If there are two coding regions that are mapped to the same gene,
+    // do not move the qualifier.
+    static bool NormalizeGeneQuals(CSeq_feat& cds, CSeq_feat& gene);
+    static bool NormalizeGeneQuals(CBioseq_Handle bsh);
+    static bool NormalizeGeneQuals(CSeq_entry_Handle seh);
+    typedef pair<CSeq_feat_Handle, CSeq_feat_Handle> TFeatGenePair; // by convention, cds first, gene second
+    static vector<TFeatGenePair> GetNormalizableGeneQualPairs(CBioseq_Handle bsh);
+
+    // This function is used to do generic string cleanup on User-object string fields
+    // and apply specific cleanups to known types of User-object
+    static bool CleanupUserObject(CUser_object& object);
+
+    static bool CleanupAuthor(CAuthor& author, bool fix_initials = true);
+
+    // for cleaning up collection-date subsource qualifiers
+    static bool CleanupCollectionDates(CSeq_entry_Handle seh, bool month_first);
+
+    static void AutodefId(CSeq_entry_Handle seh);
+
 private:
     // Prohibit copy constructor & assignment operator
     CCleanup(const CCleanup&);
@@ -519,12 +587,29 @@ private:
 
     CRef<CScope>            m_Scope;
 
+    static bool x_CleanupUserField(CUser_field& field);
+
     static bool x_MergeDupOrgNames(COrgName& on1, const COrgName& add);
     static bool x_MergeDupOrgRefs(COrg_ref& org1, const COrg_ref& add);
 
     static bool x_HasShortIntron(const CSeq_loc& loc, size_t min_len = 11);
     static bool x_AddLowQualityException(CSeq_feat& feat);
     static bool x_AddLowQualityException(CSeq_entry_Handle entry, CSeqFeatData::ESubtype subtype);
+
+    static bool s_IsProductOnFeat(const CSeq_feat& cds);
+    static void s_SetProductOnFeat(CSeq_feat& feat, const string& protein_name, bool append);
+
+    static bool s_CleanupGeneOntology(CUser_object& obj);
+    static bool s_CleanupStructuredComment(CUser_object& obj);
+    static bool s_RemoveEmptyFields(CUser_object& obj);
+    static bool s_CleanupGenomeAssembly(CUser_object& obj);
+    static bool s_CleanupDBLink(CUser_object& obj);
+    static bool s_AddNumToUserField(CUser_field &field);
+
+    static bool s_CleanupNameStdBC(CName_std& name, bool fix_initials);
+    static void s_ExtractSuffixFromInitials(CName_std& name);
+    static void s_FixEtAl(CName_std& name);
+
 };
 
 
