@@ -1,7 +1,7 @@
 #ifndef GBLOADER__HPP_INCLUDED
 #define GBLOADER__HPP_INCLUDED
 
-/*  $Id: gbloader.hpp 534579 2017-04-28 20:53:10Z vasilche $
+/*  $Id: gbloader.hpp 572532 2018-10-16 12:00:58Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -139,6 +139,15 @@ public:
     CGBLoaderParams(const CGBLoaderParams&);
     CGBLoaderParams& operator=(const CGBLoaderParams&);
 
+    void SetLoaderName(const string& loader_name)
+        {
+            m_LoaderName = loader_name;
+        }
+    const string& GetLoaderName(void) const
+        {
+            return m_LoaderName;
+        }
+
     void SetReaderName(const string& reader_name)
         {
             m_ReaderName = reader_name;
@@ -165,21 +174,31 @@ public:
         {
             return m_Preopen;
         }
-    void SetHUPIncluded(bool include_hup = true)
+    void SetHUPIncluded(bool include_hup = true, 
+                        const string& web_cookie = NcbiEmptyString)
         {
             m_HasHUPIncluded = include_hup;
+            if (include_hup && !web_cookie.empty())
+                m_WebCookie = web_cookie;
         }
     bool HasHUPIncluded(void) const
         {
             return m_HasHUPIncluded;
         }
 
+    const string& GetWebCookie(void) const
+        {
+            return m_WebCookie;
+        }
+
 private:
     string m_ReaderName;
     CRef<CReader> m_ReaderPtr;
     const TParamTree* m_ParamTree;
     EPreopenConnection m_Preopen;
     bool m_HasHUPIncluded;
+    string m_WebCookie;
+    string m_LoaderName;
 };
 
 class NCBI_XLOADER_GENBANK_EXPORT CGBDataLoader : public CDataLoader
@@ -192,57 +211,60 @@ public:
 
     virtual ~CGBDataLoader(void);
 
-    virtual void DropTSE(CRef<CTSE_Info> tse_info);
+    virtual void DropTSE(CRef<CTSE_Info> tse_info) override;
 
-    virtual void GetIds(const CSeq_id_Handle& idh, TIds& ids);
-    virtual SAccVerFound GetAccVerFound(const CSeq_id_Handle& idh);
-    virtual SGiFound GetGiFound(const CSeq_id_Handle& idh);
-    virtual string GetLabel(const CSeq_id_Handle& idh);
-    virtual int GetTaxId(const CSeq_id_Handle& idh);
-    virtual int GetSequenceState(const CSeq_id_Handle& idh);
-    virtual SHashFound GetSequenceHashFound(const CSeq_id_Handle& idh);
-    virtual TSeqPos GetSequenceLength(const CSeq_id_Handle& sih);
-    virtual STypeFound GetSequenceTypeFound(const CSeq_id_Handle& sih);
+    virtual void GetIds(const CSeq_id_Handle& idh, TIds& ids) override;
+    virtual SAccVerFound GetAccVerFound(const CSeq_id_Handle& idh) override;
+    virtual SGiFound GetGiFound(const CSeq_id_Handle& idh) override;
+    virtual string GetLabel(const CSeq_id_Handle& idh) override;
+    virtual int GetTaxId(const CSeq_id_Handle& idh) override;
+    virtual int GetSequenceState(const CSeq_id_Handle& idh) override;
+    virtual SHashFound GetSequenceHashFound(const CSeq_id_Handle& idh) override;
+    virtual TSeqPos GetSequenceLength(const CSeq_id_Handle& sih) override;
+    virtual STypeFound GetSequenceTypeFound(const CSeq_id_Handle& sih) override;
 
-    virtual void GetAccVers(const TIds& ids, TLoaded& loader, TIds& ret);
-    virtual void GetGis(const TIds& ids, TLoaded& loader, TGis& ret);
-    virtual void GetLabels(const TIds& ids, TLoaded& loader, TLabels& ret);
-    virtual void GetTaxIds(const TIds& ids, TLoaded& loader, TTaxIds& ret);
+    virtual void GetAccVers(const TIds& ids, TLoaded& loader, TIds& ret) override;
+    virtual void GetGis(const TIds& ids, TLoaded& loader, TGis& ret) override;
+    virtual void GetLabels(const TIds& ids, TLoaded& loader, TLabels& ret) override;
+    virtual void GetTaxIds(const TIds& ids, TLoaded& loader, TTaxIds& ret) override;
     virtual void GetSequenceStates(const TIds& ids, TLoaded& loader,
-                                   TSequenceStates& ret);
+                                   TSequenceStates& ret) override;
     virtual void GetSequenceHashes(const TIds& ids, TLoaded& loader,
-                                   TSequenceHashes& ret, THashKnown& known);
+                                   TSequenceHashes& ret, THashKnown& known) override;
     virtual void GetSequenceLengths(const TIds& ids, TLoaded& loader,
-                                    TSequenceLengths& ret);
+                                    TSequenceLengths& ret) override;
     virtual void GetSequenceTypes(const TIds& ids, TLoaded& loader,
-                                  TSequenceTypes& ret);
+                                  TSequenceTypes& ret) override;
 
     virtual TTSE_LockSet GetRecords(const CSeq_id_Handle& idh,
-                                    EChoice choice);
+                                    EChoice choice) override;
     virtual TTSE_LockSet GetDetailedRecords(const CSeq_id_Handle& idh,
-                                            const SRequestDetails& details);
-    virtual TTSE_LockSet GetExternalRecords(const CBioseq_Info& bioseq);
-    virtual TTSE_LockSet GetExternalAnnotRecords(const CSeq_id_Handle& idh,
-                                                 const SAnnotSelector* sel);
-    virtual TTSE_LockSet GetExternalAnnotRecords(const CBioseq_Info& bioseq,
-                                                 const SAnnotSelector* sel);
-    virtual TTSE_LockSet GetOrphanAnnotRecords(const CSeq_id_Handle& idh,
-                                               const SAnnotSelector* sel);
+                                            const SRequestDetails& details) override;
+    virtual TTSE_LockSet GetExternalRecords(const CBioseq_Info& bioseq) override;
+    virtual TTSE_LockSet GetExternalAnnotRecordsNA(const CSeq_id_Handle& idh,
+                                                   const SAnnotSelector* sel,
+                                                   TProcessedNAs* processed_nas) override;
+    virtual TTSE_LockSet GetExternalAnnotRecordsNA(const CBioseq_Info& bioseq,
+                                                   const SAnnotSelector* sel,
+                                                   TProcessedNAs* processed_nas) override;
+    virtual TTSE_LockSet GetOrphanAnnotRecordsNA(const CSeq_id_Handle& idh,
+                                                 const SAnnotSelector* sel,
+                                                 TProcessedNAs* processed_nas) override;
 
-    virtual void GetChunk(TChunk chunk);
-    virtual void GetChunks(const TChunkSet& chunks);
+    virtual void GetChunk(TChunk chunk) override;
+    virtual void GetChunks(const TChunkSet& chunks) override;
 
-    virtual void GetBlobs(TTSE_LockSets& tse_sets);
+    virtual void GetBlobs(TTSE_LockSets& tse_sets) override;
 
-    virtual TBlobId GetBlobId(const CSeq_id_Handle& idh);
-    virtual TBlobId GetBlobIdFromString(const string& str) const;
+    virtual TBlobId GetBlobId(const CSeq_id_Handle& idh) override;
+    virtual TBlobId GetBlobIdFromString(const string& str) const override;
     TBlobId GetBlobIdFromSatSatKey(int sat,
                                    int sat_key,
                                    int sub_sat = 0) const;
 
-    virtual TBlobVersion GetBlobVersion(const TBlobId& id);
-    bool CanGetBlobById(void) const;
-    TTSE_Lock GetBlobById(const TBlobId& id);
+    virtual TBlobVersion GetBlobVersion(const TBlobId& id) override;
+    virtual bool CanGetBlobById(void) const override;
+    virtual TTSE_Lock GetBlobById(const TBlobId& id) override;
 
     // Create GB loader and register in the object manager if
     // no loader with the same name is registered yet.
@@ -272,7 +294,7 @@ public:
     static string GetLoaderNameFromArgs(const string& reader_name);
 
     // GBLoader with HUP data included.
-    // The reader will be chosed from default configuration,
+    // The reader will be chosen from default configuration,
     // either pubseqos or pubseqos2.
     // The default loader priority will be slightly lower than for main data.
     static TRegisterLoaderInfo RegisterInObjectManager(
@@ -280,6 +302,12 @@ public:
         EIncludeHUP     include_hup,
         CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
         CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
+    static TRegisterLoaderInfo RegisterInObjectManager(
+        CObjectManager& om,
+        EIncludeHUP     include_hup,
+        const string& web_cookie,
+        CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
+        CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
     static string GetLoaderNameFromArgs(EIncludeHUP     include_hup);
 
     // GBLoader with HUP data included.
@@ -291,6 +319,13 @@ public:
         EIncludeHUP     include_hup,
         CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
         CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
+    static TRegisterLoaderInfo RegisterInObjectManager(
+        CObjectManager& om,
+        const string&   reader_name, // pubseqos or pubseqos2
+        EIncludeHUP     include_hup,
+        const string& web_cookie,
+        CObjectManager::EIsDefault is_default = CObjectManager::eNonDefault,
+        CObjectManager::TPriority  priority = CObjectManager::kPriority_NotSet);
     static string GetLoaderNameFromArgs(const string&   reader_name, // pubseqos or pubseqos2
                                         EIncludeHUP     include_hup);
 
@@ -320,9 +355,9 @@ public:
     //string BlobIdToString(const TBlobId& id) const;
 
     virtual TTSE_Lock ResolveConflict(const CSeq_id_Handle& handle,
-                                      const TTSE_LockSet& tse_set);
+                                      const TTSE_LockSet& tse_set) override;
 
-    virtual void GC(void);
+    virtual void GC(void) override;
 
     virtual TNamedAnnotNames GetNamedAnnotAccessions(const CSeq_id_Handle& idh);
     virtual TNamedAnnotNames GetNamedAnnotAccessions(const CSeq_id_Handle& idh,
@@ -419,8 +454,8 @@ public:
             return m_HasHUPIncluded;
         }
 
-    virtual CObjectManager::TPriority GetDefaultPriority(void) const;
-    
+    virtual CObjectManager::TPriority GetDefaultPriority(void) const override;
+
 protected:
     friend class CGBReaderRequestResult;
 
@@ -429,7 +464,8 @@ protected:
 
     TTSE_LockSet x_GetRecords(const CSeq_id_Handle& idh,
                               TBlobContentsMask sr_mask,
-                              const SAnnotSelector* sel);
+                              const SAnnotSelector* sel,
+                              TProcessedNAs* processed_nas = 0);
 
 private:
     typedef CParamLoaderMaker<CGBDataLoader, const CGBLoaderParams&> TGBMaker;
@@ -457,6 +493,7 @@ private:
     bool                    m_AlwaysLoadNamedAcc;
     bool                    m_AddWGSMasterDescr;
     bool                    m_HasHUPIncluded;
+    string                  m_WebCookie;
 
     //
     // private code
