@@ -1,4 +1,4 @@
-/*  $Id: feattable_edit.hpp 546402 2017-09-18 15:03:27Z ivanov $
+/*  $Id: feattable_edit.hpp 572213 2018-10-10 14:46:17Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -38,11 +38,10 @@
 #include <objects/seq/Seq_annot.hpp>
 #include <objmgr/scope.hpp>
 #include <objmgr/util/feature.hpp>
-#include <objtools/readers/message_listener.hpp>
 
 BEGIN_NCBI_SCOPE
-
 BEGIN_SCOPE(objects)
+class IObjtoolsListener;
 BEGIN_SCOPE(edit)
 
 //  ----------------------------------------------------------------------------
@@ -57,7 +56,7 @@ public:
 		const string& = "",
         unsigned int = 1, //starting locus tag
         unsigned int = 1, //starting feature id
-        ILineErrorListener* =0);
+        IObjtoolsListener* =nullptr);
     ~CFeatTableEdit();
 
 	void GenerateLocusTags();
@@ -66,6 +65,7 @@ public:
     void InferPartials();
     void EliminateBadQualifiers();
     void GenerateProteinAndTranscriptIds();
+    void InstantiateProducts();
     void GenerateLocusIds();
     void SubmitFixProducts();
     void GenerateMissingMrnaForCds();
@@ -94,15 +94,19 @@ protected:
 	string xNextTranscriptId(
 		const CMappedFeat&);
 
-    void xPutError(
-        ILineError::EProblem,
-        const std::string&);
+    void xPutError(const string& message);
+
     void xPutErrorMissingLocustag(
         CMappedFeat);
     void xPutErrorMissingTranscriptId(
         CMappedFeat);
     void xPutErrorMissingProteinId(
         CMappedFeat);
+    void xPutErrorDifferingTranscriptIds(
+        const CMappedFeat& mrna);
+    void xPutErrorDifferingProteinIds(
+        const CMappedFeat& mrna);
+
 
     void xFeatureAddQualifier(
         CMappedFeat,
@@ -129,7 +133,9 @@ protected:
         CMappedFeat);
     void xFeatureAddTranscriptIdDefault(
         CMappedFeat);
-
+    void xFeatureSetProduct(
+        CMappedFeat,
+        const string&);
     std::string xGenerateTranscriptOrProteinId(
         CMappedFeat,
         const std::string&);
@@ -150,17 +156,29 @@ protected:
     std::string xGetCurrentLocusTagPrefix(
         CMappedFeat);
 
+    void xAddTranscriptAndProteinIdsToCdsAndParentMrna(CMappedFeat& cds);
+    void xAddTranscriptAndProteinIdsToUnmatchedMrna(CMappedFeat& mrna);
+    void xAddTranscriptAndProteinIdsToMrna(const string& cds_transcript_id,
+                                           const string& cds_protein_id,
+                                           CMappedFeat& mrna);
+    void xConvertToGeneralIds(const CMappedFeat& mf,
+                              string& transcript_id,
+                              string& protein_id);
+
     CSeq_annot& mAnnot;
     CRef<CScope> mpScope;
     CSeq_annot_Handle mHandle;
     feature::CFeatTree mTree;
     CSeq_annot_EditHandle mEditHandle;
-    ILineErrorListener* mpMessageListener;
+    IObjtoolsListener* mpMessageListener;
     unsigned int mNextFeatId;
 	unsigned int mLocusTagNumber;
 	string mLocusTagPrefix;
 
 	map<string, int> mMapProtIdCounts;
+
+    using TFeatQualMap = map<CMappedFeat,string>;
+    set<CMappedFeat> mProcessedMrnas;
 };
 
 END_SCOPE(edit)
