@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___CONTEXT__HPP
 #define OBJTOOLS_FORMAT___CONTEXT__HPP
 
-/*  $Id: context.hpp 542550 2017-08-01 12:44:40Z dicuccio $
+/*  $Id: context.hpp 572065 2018-10-09 13:29:32Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -46,6 +46,7 @@
 #include <objmgr/annot_selector.hpp>
 #include <objmgr/seq_loc_mapper.hpp>
 #include <objmgr/util/feature.hpp>
+#include <objmgr/util/indexer.hpp>
 
 #include <util/range.hpp>
 
@@ -100,6 +101,8 @@ public:
     CBioseq_Handle& GetNextHandle(void) { return m_NextHandle; }
     CScope& GetScope(void) const { return m_Handle.GetScope(); }
     feature::CFeatTree& GetFeatTree(void) { return m_FeatTree; }
+    bool UsingSeqEntryIndex(void) const;
+    const CRef<CSeqEntryIndex> GetSeqEntryIndex(void) const;
 
     // -- id information
     CSeq_id* GetPrimaryId(void) { return m_PrimaryId; }
@@ -210,13 +213,17 @@ public:
     bool IsGenomeAssembly(void) const;
     bool IsCrossKingdom(void) const;
 
+    bool UsePDBCompoundForComment(void) const;
+    void SetPDBCompoundForComment(bool value);
+
     NCBI_DEPRECATED bool IsUnverified(void) const;
     enum FUnverified {
         fUnverified_None = 0, // for convenience
 
         fUnverified_Organism              = 1 << 0,
         fUnverified_SequenceOrAnnotation  = 1 << 1,
-        fUnverified_Misassembled          = 1 << 2
+        fUnverified_Misassembled          = 1 << 2,
+        fUnverified_Contaminant           = 1 << 3
         // if you add more here, make sure 
         // to update CFlatGatherer::x_UnverifiedComment
     };
@@ -247,7 +254,7 @@ public:
     void ThrowIfCanceled(void) const;
 
     /// Empty string if unavailable
-    const string & GetTaxname(void) const { return m_Taxname; }
+    const string & GetTaxname(void) const;
 
     /// Empty if no Filetrack URL.
     const string & GetFiletrackURL(void) const { return m_FiletrackURL; }
@@ -275,10 +282,10 @@ private:
     bool x_IsInNucProt(void) const;
     void x_SetLocation(const CSeq_loc* user_loc = 0);
     void x_SetMapper(const CSeq_loc& loc);
-    void x_SetHasMultiIntervalGenes(void);
+    void x_SetHasMultiIntervalGenes(void) const;
     void x_SetDataFromUserObjects(void);
     void x_CheckForShowComments() const;
-    void x_SetTaxname(void);
+    void x_SetTaxname(void) const;
     void x_SetFiletrackURL(const CUser_object& uo);
     void x_SetAuthorizedAccess(const CUser_object& uo);
     void x_SetOpticalMapPoints(void);
@@ -302,7 +309,7 @@ private:
     string                m_TLSMasterAccn;
     string                m_TLSMasterName;
     string                m_FinishingStatus;
-    string                m_Taxname;
+    mutable string        m_Taxname;
     string                m_FiletrackURL;
     vector< string >      m_BasemodURLs;
     string                m_AuthorizedAccess;
@@ -353,10 +360,11 @@ private:
     TGi  m_Gi;
     bool m_ShowGBBSource;
     int  m_PatSeqid;
-    bool m_HasOperon;
-    bool m_HasMultiIntervalGenes;
+    mutable bool m_HasOperon;
+    mutable bool m_HasMultiIntervalGenes;
     bool m_IsGenomeAssembly;
-    bool m_IsCrossKingdom;
+    mutable bool m_IsCrossKingdom;
+    mutable bool m_UsePDBCompoundForComment;
     TUnverified m_fUnverified;
     mutable bool m_ShowAnnotCommentAsCOMMENT;
     mutable bool m_ShowAnnotCommentAsCOMMENT_checked;
@@ -421,7 +429,7 @@ public:
     typedef vector< CRef<CBioseqContext> >  TSections;
 
     // constructor
-    CFlatFileContext(const CFlatFileConfig& cfg) : m_Cfg(cfg) { }
+    CFlatFileContext(const CFlatFileConfig& cfg) : m_Cfg(cfg), m_SmallGenomeSet(false) { }
     // destructor
     ~CFlatFileContext(void) {}
 
@@ -445,6 +453,11 @@ public:
     feature::CFeatTree* GetFeatTree(void) { return m_FeatTree; }
     void SetFeatTree(feature::CFeatTree* tree) { m_FeatTree.Reset(tree); }
     
+    bool UsingSeqEntryIndex(void) const { return (m_Idx != 0); }
+    const CRef<CSeqEntryIndex> GetSeqEntryIndex(void) const { return m_Idx; }
+    void SetSeqEntryIndex(CRef<CSeqEntryIndex> idx) { m_Idx = idx; }
+    void ResetSeqEntryIndex(void) { m_Idx.Reset(NULL); }
+
     bool GetSGS(void) const { return m_SmallGenomeSet; }
     void SetSGS(const bool sgs) { m_SmallGenomeSet = sgs; }
 
@@ -461,6 +474,7 @@ private:
     auto_ptr<SAnnotSelector>    m_Selector;
     CConstRef<CSeq_loc>         m_Loc;
     CRef<feature::CFeatTree>    m_FeatTree;
+    CRef<CSeqEntryIndex>        m_Idx;
     bool                        m_SmallGenomeSet;
 };
 
@@ -625,12 +639,6 @@ bool CBioseqContext::IsGenomeAssembly(void) const
     return m_IsGenomeAssembly;
 }
 
-inline
-bool CBioseqContext::IsCrossKingdom(void) const
-{
-    return m_IsCrossKingdom;
-}
-
 inline
 bool CBioseqContext::IsUnverified(void) const
 {
@@ -643,6 +651,19 @@ CBioseqContext::TUnverified CBioseqContext::GetUnverifiedType(void) const
     return m_fUnverified;
 }
 
+inline
+bool CBioseqContext::UsingSeqEntryIndex(void) const
+{
+    return m_FFCtx.UsingSeqEntryIndex();
+}
+
+inline
+const CRef<CSeqEntryIndex> CBioseqContext::GetSeqEntryIndex(void) const
+{
+    return m_FFCtx.GetSeqEntryIndex();
+}
+
+
 inline
 const CFlatFileConfig& CBioseqContext::Config(void) const
 {
@@ -709,18 +730,6 @@ bool CBioseqContext::IsGenbankFormat(void) const
 }
 
 
-inline
-bool CBioseqContext::HasOperon(void) const
-{
-    return m_HasOperon;
-}
-
-inline 
-bool CBioseqContext::HasMultiIntervalGenes(void) const
-{
-    return m_HasMultiIntervalGenes;
-}
-
 inline
 void CBioseqContext::ThrowIfCanceled(void) const
 {
