@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT___FLAT_FILE_CONFIG__HPP
 #define OBJTOOLS_FORMAT___FLAT_FILE_CONFIG__HPP
 
-/*  $Id: flat_file_config.hpp 544490 2017-08-23 17:44:11Z ivanov $
+/*  $Id: flat_file_config.hpp 573607 2018-10-30 11:57:34Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -85,6 +85,7 @@ public:
     virtual ~IHTMLFormatter() {};
     
     virtual void FormatProteinId(string& str, const CSeq_id& seq_id, const string& prot_id) const = 0;
+    virtual void FormatTranscriptId(string& str, const CSeq_id& seq_id, const string& nuc_id) const = 0;
     virtual void FormatNucSearch(CNcbiOstream& os, const string& id) const = 0;
     virtual void FormatNucId(string& str, const CSeq_id& seq_id, TIntId gi, const string& acc_id) const = 0;
     virtual void FormatTaxid(string& str, const int taxid, const string& taxname) const = 0;
@@ -92,6 +93,8 @@ public:
     virtual void FormatModelEvidence(string& str, const SModelEvidance& me) const = 0;
     virtual void FormatTranscript(string& str, const string& name) const = 0;
     virtual void FormatGeneralId(CNcbiOstream& os, const string& id) const = 0;
+    virtual void FormatGapLink(CNcbiOstream& os, TSeqPos gap_size, const string& id, bool is_prot) const = 0;
+    virtual void FormatUniProtId(string& str, const string& prot_id) const = 0;
 };
 
 class NCBI_FORMAT_EXPORT CHTMLEmptyFormatter : public IHTMLFormatter
@@ -100,6 +103,7 @@ public:
     virtual ~CHTMLEmptyFormatter() {};
 
     void FormatProteinId(string& str, const CSeq_id& seq_id, const string& prot_id) const;
+    void FormatTranscriptId(string& str, const CSeq_id& seq_id, const string& nuc_id) const;
     void FormatNucSearch(CNcbiOstream& os, const string& id) const;
     void FormatNucId(string& str, const CSeq_id& seq_id, TIntId gi, const string& acc_id) const;
     void FormatTaxid(string& str, const int taxid, const string& taxname) const;
@@ -107,6 +111,8 @@ public:
     void FormatModelEvidence(string& str, const SModelEvidance& me) const;
     void FormatTranscript(string& str, const string& name) const;
     void FormatGeneralId(CNcbiOstream& os, const string& id) const;
+    void FormatGapLink(CNcbiOstream& os, TSeqPos gap_size, const string& id, bool is_prot) const;
+    void FormatUniProtId(string& str, const string& prot_id) const;
 };
 
 class NCBI_FORMAT_EXPORT CFlatFileConfig
@@ -119,6 +125,7 @@ public:
         eFormat_EMBL,
         eFormat_DDBJ,
         eFormat_GBSeq,
+        eFormat_INSDSeq,
         eFormat_FTable,
         eFormat_FeaturesOnly,
         eFormat_SAM,
@@ -182,7 +189,14 @@ public:
         // additional customization flags
         fHideProteinID         = 1,
         fHideGI                = 1 << 1,
-        fLongLocusNames        = 1 << 2
+        fLongLocusNames        = 1 << 2,
+        fExpandGaps            = 1 << 3,
+        fShowSNPFeatures       = 1 << 6,
+        fShowCDDFeatures       = 1 << 7,
+        fDisableAnnotRefs      = 1 << 10,
+        fUseSeqEntryIndexer    = 1 << 11,
+        fUseAutoDef            = 1 << 12,
+        fIgnoreExistingTitle   = 1 << 13
     };
 
     enum EView {
@@ -193,23 +207,6 @@ public:
         fViewFirst        = 0x4
     };
 
-    enum EGffOptions {
-        // specifies special features of GFF(3) generation
-        fGffGenerateIdTags     = 1 << 0,
-        fGffGTFCompat          = 1 << 1, ///< Represent CDSs (and exons) per GTF.
-        fGffGTFOnly            = 1 << 2, ///< Omit all other features.
-        fGffShowSeq            = 1 << 3, ///< Show the actual sequence in a "##" comment.
-        // There are at least 3 flavours of GFF3, discounting multiple
-        // versions of each:
-        // - "Official specifications" by the Sequence Ontology group (SO),
-        //   see http://www.sequenceontology.org/gff3.shtml
-        // - GFF3 as modified by the Interoperability Working Group (IOWG),
-        //   see http://www.pathogenportal.org/gff3-usage-conventions.html
-        // - GFF3 as modified for exchange with Flybase. This alters
-        //   such critical information as how phase is represented.
-        fGffForFlybase         = 1 << 4, ///< Flybase flavour of GFF3.
-    };
-
     // These flags are used to select the GenBank sections to print or skip.
     enum FGenbankBlocks {
         // default is all sections
@@ -254,7 +251,6 @@ public:
     typedef EStyle          TStyle;
     typedef unsigned int    TFlags; // binary OR of "EFlags"
     typedef unsigned int    TView;
-    typedef unsigned int    TGffOptions;
     typedef unsigned int    TGenbankBlocks;
     typedef unsigned int    TCustom; // binary OR of "ECustom"
     
@@ -371,20 +367,6 @@ public:
             FGenbankBlocks /*which_block*/ ) { return eAction_Default; }
     };
 
-    // constructor
-    NCBI_DEPRECATED
-    CFlatFileConfig(TFormat format,
-                    TMode   mode,
-                    TStyle  style,
-                    TFlags  flags,
-                    TView   view,
-                    TGffOptions gff_options_OBSOLETE,
-                    TGenbankBlocks genbank_blocks = fGenbankBlocks_All,
-                    CGenbankBlockCallback* pGenbankBlockCallback = NULL,
-                    const ICanceled * pCanceledCallback = NULL,
-                    bool basicCleanup = false,
-                    TCustom custom = 0 );
-
     // constructor
     CFlatFileConfig(TFormat format = eFormat_GenBank,
                     TMode   mode = eMode_GBench,
@@ -413,9 +395,10 @@ public:
     bool IsFormatEMBL   (void) const { return m_Format == eFormat_EMBL;    }
     bool IsFormatDDBJ   (void) const { return m_Format == eFormat_DDBJ;    }
     bool IsFormatGBSeq  (void) const { return m_Format == eFormat_GBSeq;   }
+    bool IsFormatINSDSeq(void) const { return m_Format == eFormat_INSDSeq; }
     bool IsFormatFTable (void) const { return m_Format == eFormat_FTable;  }
-	bool IsFormatAGP    (void) const { return m_Format == eFormat_AGP;     }
-	bool IsFormatLite   (void) const { return m_Format == eFormat_Lite;    }
+    bool IsFormatAGP    (void) const { return m_Format == eFormat_AGP;     }
+    bool IsFormatLite   (void) const { return m_Format == eFormat_Lite;    }
 
     // setters
     void SetFormat(const TFormat& format) { m_Format = format; }
@@ -423,6 +406,7 @@ public:
     void SetFormatEMBL   (void) { m_Format = eFormat_EMBL;     }
     void SetFormatDDBJ   (void) { m_Format = eFormat_DDBJ;     }
     void SetFormatGBSeq  (void) { m_Format = eFormat_GBSeq;    }
+    void SetFormatINSDSeq(void) { m_Format = eFormat_INSDSeq;  }
     void SetFormatFTable (void) { m_Format = eFormat_FTable;   }
     void SetFormatAGP    (void) { m_Format = eFormat_AGP;      }
     void SetFormatLite   (void) { m_Format = eFormat_Lite;     }
@@ -591,103 +575,30 @@ public:
     bool HideProteinID         (void) const;
     bool HideGI                (void) const;
     bool LongLocusNames        (void) const;
+    bool ExpandGaps            (void) const;
+    bool ShowSNPFeatures       (void) const;
+    bool ShowCDDFeatures       (void) const;
+    bool DisableAnnotRefs      (void) const;
+    bool UseSeqEntryIndexer    (void) const;
+    bool UseAutoDef            (void) const;
+    bool IgnoreExistingTitle   (void) const;
 
     // setters
     void SetCustom(const TCustom& custom) { m_Custom = custom; }
     CFlatFileConfig& SetHideProteinID        (bool val = true);
     CFlatFileConfig& SetHideGI               (bool val = true);
     CFlatFileConfig& SetLongLocusNames       (bool val = true);
+    CFlatFileConfig& SetExpandGaps           (bool val = true);
+    CFlatFileConfig& SetShowSNPFeatures      (bool val = true);
+    CFlatFileConfig& SetShowCDDFeatures      (bool val = true);
+    CFlatFileConfig& SetDisableAnnotRefs     (bool val = true);
+    CFlatFileConfig& SetUseSeqEntryIndexer   (bool val = true);
+    CFlatFileConfig& SetUseAutoDef           (bool val = true);
+    CFlatFileConfig& SetIgnoreExistingTitle  (bool val = true);
 
     // adjust mode dependant flags for RefSeq
     void SetRefSeqConventions(void);
 
-    // -- GffOptions
-    // getters
-    NCBI_DEPRECATED
-    bool GffGenerateIdTags   (void) const 
-    { 
-        return (0 != (m_GffOptions & fGffGenerateIdTags));
-    };
-
-    NCBI_DEPRECATED
-    bool GffGTFCompat        (void) const 
-    { 
-        return (0 != (m_GffOptions & fGffGTFCompat));
-    };
-
-    NCBI_DEPRECATED
-    bool GffGTFOnly          (void) const 
-    { 
-        return (0 != (m_GffOptions & fGffGTFOnly));
-    };
-
-    NCBI_DEPRECATED
-    bool GffShowSeq          (void) const 
-    { 
-        return (0 != (m_GffOptions & fGffShowSeq));
-    };
-
-    NCBI_DEPRECATED
-    bool GffForFlybase       (void) const 
-    { 
-        return (0 != (m_GffOptions & fGffForFlybase));
-    };
-
-    // setters
-    NCBI_DEPRECATED
-    void SetGffGenerateIdTags (bool val = true)
-    {
-        if (val) {
-            m_GffOptions |= fGffGenerateIdTags;
-        }
-        else {
-            m_GffOptions &= ~fGffGenerateIdTags;
-        }
-    };
-
-    NCBI_DEPRECATED
-    void SetGffGTFCompat      (bool val = true)
-    {
-        if (val) {
-            m_GffOptions |= fGffGTFCompat;
-        }
-        else {
-            m_GffOptions &= ~fGffGTFCompat;
-        }
-    };
-
-    NCBI_DEPRECATED
-    void SetGffGTFOnly        (bool val = true)
-    {
-        if (val) {
-            m_GffOptions |= fGffGTFOnly;
-        }
-        else {
-            m_GffOptions &= ~fGffGTFOnly;
-        }
-    };
-
-    NCBI_DEPRECATED
-    void SetGffShowSeq        (bool val = true)
-    {
-        if (val) {
-            m_GffOptions |= fGffShowSeq;
-        }
-        else {
-            m_GffOptions &= ~fGffShowSeq;
-        }
-    };
-
-    NCBI_DEPRECATED
-    void SetGffForFlybase     (bool val = true)
-    {
-        if (val) {
-            m_GffOptions |= fGffForFlybase;
-        }
-        else {
-            m_GffOptions &= ~fGffForFlybase;
-        }
-    };
 
     void SetGenbankBlocks(const TGenbankBlocks& genbank_blocks) 
     {
@@ -768,7 +679,6 @@ private:
     TFlags      m_Flags;  // custom flags
     TView       m_View;
     bool        m_RefSeqConventions;
-    TGffOptions m_GffOptions;
     TGenbankBlocks m_fGenbankBlocks;
     CRef<CGenbankBlockCallback> m_GenbankBlockCallback;
     const ICanceled * m_pCanceledCallback; // instance does NOT own it
@@ -864,6 +774,13 @@ CUSTOM_ARG_SET(x)
 CUSTOM_ARG_IMP(HideProteinID)
 CUSTOM_ARG_IMP(HideGI)
 CUSTOM_ARG_IMP(LongLocusNames)
+CUSTOM_ARG_IMP(ExpandGaps)
+CUSTOM_ARG_IMP(ShowSNPFeatures)
+CUSTOM_ARG_IMP(ShowCDDFeatures)
+CUSTOM_ARG_IMP(DisableAnnotRefs)
+CUSTOM_ARG_IMP(UseSeqEntryIndexer)
+CUSTOM_ARG_IMP(UseAutoDef)
+CUSTOM_ARG_IMP(IgnoreExistingTitle)
 
 #undef FLAG_ARG_IMP
 #undef FLAG_ARG_GET
