@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT_ITEMS___ACCESSION_ITEM__HPP
 #define OBJTOOLS_FORMAT_ITEMS___ACCESSION_ITEM__HPP
 
-/*  $Id: accession_item.hpp 360035 2012-04-19 13:43:48Z kornbluh $
+/*  $Id: accession_item.hpp 554019 2017-12-27 15:18:41Z dondosha $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -69,6 +69,8 @@ public:
     const TExtra_accessions& GetExtraAccessions(void) const;
     bool  IsSetRegion(void) const;
     const CSeq_loc& GetRegion(void) const;
+    bool  IsNuc(void) const;
+    bool  IsProt(void) const;
 private:
     void x_GatherInfo(CBioseqContext& ctx);
 
@@ -79,6 +81,7 @@ private:
     TExtra_accessions   m_ExtraAccessions;
     CConstRef<CSeq_loc> m_Region;
     bool                m_IsSetRegion;
+    bool                m_IsNuc;
 };
 
 
@@ -126,6 +129,17 @@ const CSeq_loc& CAccessionItem::GetRegion(void) const
     return *m_Region;
 }
 
+inline
+bool CAccessionItem::IsNuc(void) const
+{
+    return m_IsNuc;
+}
+
+inline
+bool CAccessionItem::IsProt(void) const
+{
+    return !m_IsNuc;
+}
 
 END_SCOPE(objects)
 END_NCBI_SCOPE
