@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT_ITEMS___FLAT_FEATURE__HPP
 #define OBJTOOLS_FORMAT_ITEMS___FLAT_FEATURE__HPP
 
-/*  $Id: feature_item.hpp 541705 2017-07-20 17:36:58Z ivanov $
+/*  $Id: feature_item.hpp 561799 2018-04-11 22:46:12Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -179,6 +179,8 @@ protected:
 
     void x_GetAssociatedProtInfo( CBioseqContext&, CBioseq_Handle&,
         const CProt_ref*&, CMappedFeat& protFeat, CConstRef<CSeq_id>& );
+    void x_GetAssociatedProtInfoIdx( CBioseqContext&, CBioseq_Handle&,
+        const CProt_ref*&, CMappedFeat& protFeat, CConstRef<CSeq_id>& );
     void x_AddQualPartial( CBioseqContext& );
     void x_AddQualDbXref(
         CBioseqContext& );
@@ -194,7 +196,7 @@ protected:
         CSeqFeatData::ESubtype, bool );
     void x_AddQualExceptions( CBioseqContext& );
     void x_AddQualNote( CConstRef<CSeq_feat> );
-    void x_AddQualOldLocusTag( CConstRef<CSeq_feat> );
+    void x_AddQualOldLocusTag( const CBioseqContext& ctx, CConstRef<CSeq_feat> );
     void x_AddQualDb( const CGene_ref* );
     void x_AddQualSeqfeatNote( CBioseqContext & );
     void x_AddQualTranslation( CBioseq_Handle&, CBioseqContext&, bool );
@@ -217,6 +219,8 @@ protected:
     // qualifier collection
     void x_AddQualsCdregion(const CMappedFeat& cds, CBioseqContext& ctx,
         bool pseudo);
+    void x_AddQualsCdregionIdx(const CMappedFeat& cds, CBioseqContext& ctx,
+        bool pseudo);
     virtual void x_AddQualsRna(const CMappedFeat& feat, CBioseqContext& ctx,
          bool pseudo);
     void x_AddQualsExt( const CSeq_feat::TExt& );
@@ -230,9 +234,12 @@ protected:
 
     void x_AddQuals( CBioseqContext& ctx, CConstRef<CFeatureItem> parentFeatureItem );
     void x_AddQuals( CBioseqContext& ctx ) { x_AddQuals( ctx, CConstRef<CFeatureItem>() ); }
+    void x_AddQualsIdx( CBioseqContext& ctx, CConstRef<CFeatureItem> parentFeatureItem );
+    void x_AddQualsIdx( CBioseqContext& ctx ) { x_AddQualsIdx( ctx, CConstRef<CFeatureItem>() ); }
+
     void x_AddQuals(const CProt_ref& prot);
     void x_AddProductIdQuals(CBioseq_Handle& prod, EFeatureQualifier slot);
-    void x_AddQualsGene(const CGene_ref*, CConstRef<CSeq_feat>&,
+    void x_AddQualsGene(const CBioseqContext& ctx, const CGene_ref*, CConstRef<CSeq_feat>&,
         bool from_overlap);
     void x_AddGoQuals(const CUser_object& uo);
     void x_ImportQuals(CBioseqContext& ctx);
@@ -382,6 +389,8 @@ public:
     CSourceFeatureItem(const CMappedFeat& feat, CBioseqContext& ctx,
         CRef<feature::CFeatTree> ftree, const CSeq_loc* loc = NULL);
 
+    void SetObject(const CSerialObject& obj) { x_SetObject(obj); }
+
     bool WasDesc(void) const { return m_WasDesc; }
     const CBioSource& GetSource(void) const {
         return m_Feat.GetData().GetBiosrc();
