@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT_ITEMS___FLAT_SEQLOC__HPP
 #define OBJTOOLS_FORMAT_ITEMS___FLAT_SEQLOC__HPP
 
-/*  $Id: flat_seqloc.hpp 533244 2017-04-13 20:40:43Z kans $
+/*  $Id: flat_seqloc.hpp 560984 2018-03-29 19:53:54Z dondosha $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -78,8 +78,12 @@ public:
     };
     typedef EType     TType;
 
+    // ID-4736 : The add_join flag is needed for the GBSeq format
+    // (<GBSeq_contig> node), where the wrapping join(...) is
+    // required even when location consists of a single interval.
     CFlatSeqLoc(const CSeq_loc& loc, CBioseqContext& ctx, 
-        TType type = eType_location, bool show_all_accns = false);
+                TType type = eType_location, bool show_all_accns = false,
+                bool add_join = false);
 
     const string&     GetString(void)    const { return m_String;    }
     
@@ -102,7 +106,7 @@ private:
 
     bool x_Add(const CSeq_loc& loc, CNcbiOstrstream& oss,
         CBioseqContext& ctx, TType type, bool show_comp,
-        bool show_all_accns = false);
+        bool show_all_accns = false, bool join_whole_loc = false);
     bool x_Add(const CSeq_interval& si, CNcbiOstrstream& oss,
         CBioseqContext& ctx, TType type, bool show_comp,
         bool show_all_accns = false);
