@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_FORMAT_ITEMS___LOCUS_ITEM__HPP
 #define OBJTOOLS_FORMAT_ITEMS___LOCUS_ITEM__HPP
 
-/*  $Id: locus_item.hpp 501627 2016-05-17 17:47:33Z kans $
+/*  $Id: locus_item.hpp 554830 2018-01-09 18:47:38Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -73,7 +73,7 @@ public:
     const string& GetDivision (void) const;
     const string& GetDate     (void) const;
 
-    static string GetDivision(const CBioseq_Handle& bsh);
+    static string GetDivision(const CBioseq_Handle& bsh, const CBioseqContext* ctx = 0);
 
 private:
 
