@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS___ALN_READER__HPP
 #define OBJTOOLS_READERS___ALN_READER__HPP
 
-/*  $Id: aln_reader.hpp 505117 2016-06-22 15:06:00Z foleyjp $
+/*  $Id: aln_reader.hpp 534219 2017-04-25 15:59:05Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,7 +34,11 @@
 
 #include <corelib/ncbistd.hpp>
 #include <objects/seqalign/Seq_align.hpp>
+#include <objects/seqalign/Dense_seg.hpp>
 #include <objects/seqset/Seq_entry.hpp>
+#include <objtools/readers/fasta_reader_utils.hpp>
+#include <objects/seq/seq_id_handle.hpp>
+
 
 BEGIN_NCBI_SCOPE
 
@@ -86,7 +90,6 @@ private:
 class NCBI_XOBJREAD_EXPORT CAlnReader
 {
 public:
-
     // alphabets to try
     enum EAlphabet {
         eAlpha_Nucleotide,
@@ -140,7 +143,7 @@ public:
                                                                  SetAllGap(".-"); };
 
     // destructor
-    ~CAlnReader(void);
+    virtual ~CAlnReader(void);
 
 
 
@@ -189,8 +192,6 @@ public:
     /// that would parse the alignment file and create the result data
     void Read(bool guess=false, bool generate_local_ids=false);
 
-
-
     /// Parsed result data accessors
     const vector<string>& GetIds(void)       const {return m_Ids;};
     const vector<string>& GetSeqs(void)      const {return m_Seqs;};
@@ -200,9 +201,10 @@ public:
 
     const TErrorList& GetErrorList(void)     const {return m_Errors;};
     
+    using TFastaFlags = objects::CFastaDeflineReader::TFastaFlags;
     /// Create ASN.1 classes from the parsed alignment
-    CRef<objects::CSeq_align> GetSeqAlign(void);
-    CRef<objects::CSeq_entry> GetSeqEntry(void);
+    CRef<objects::CSeq_align> GetSeqAlign(TFastaFlags fasta_flags=0);
+    CRef<objects::CSeq_entry> GetSeqEntry(TFastaFlags fasta_flags=0);
 
 
 private:
@@ -210,7 +212,11 @@ private:
     CAlnReader(const CAlnReader& value);
     CAlnReader& operator=(const CAlnReader& value);
 
+    int x_GetGCD(const int a, const int b) const;
 
+    bool x_IsReplicatedSequence(const char* sequence_data,
+        int sequence_length,
+        int repeat_interval) const;
 
     /// A bunch of strings listing characters with various
     /// meanings in an alignment file.
@@ -242,6 +248,43 @@ private:
     vector<string>            m_SeqVec; 
     vector<TSeqPos>           m_SeqLen; 
     TErrorList                m_Errors;
+
+    /// characters have different contexts, depending on 
+    /// whether they are before the first non-gap character,
+    /// after the last non-gap character, or between the
+    /// first and last non-gap character. This must be
+    /// precalculated before gap characters can be converted.
+    typedef pair<TSeqPos, TSeqPos> TAlignMiddleInterval;
+    typedef vector<TAlignMiddleInterval> TAlignMiddles;
+    TAlignMiddles m_MiddleSections;
+    void x_CalculateMiddleSections();
+    typedef objects::CDense_seg::TDim TNumrow;
+    bool x_IsGap(TNumrow row, TSeqPos pos, const string& residue);
+    void x_AssignDensegIds(
+        TFastaFlags fasta_flags,
+        objects::CDense_seg& denseg);
+protected:
+    virtual CRef<objects::CSeq_id> GenerateID(const string& fasta_defline, 
+        const TSeqPos& line_number,
+        TFastaFlags fasta_flags);
+
+    using SLineTextAndLoc   = objects::CFastaDeflineReader::SLineTextAndLoc;
+    using TSeqTitles        = objects::CFastaDeflineReader::TSeqTitles;
+    using SDeflineParseInfo = objects::CFastaDeflineReader::SDeflineParseInfo;
+    using TIgnoredProblems  = objects::CFastaDeflineReader::TIgnoredProblems;
+
+    void ParseDefline(const string& defline,
+            const SDeflineParseInfo& info,
+            const TIgnoredProblems& ignoredErrors,
+            list<CRef<objects::CSeq_id>>& ids,
+            bool& hasRange,
+            TSeqPos& rangeStart,
+            TSeqPos& rangeEnd,
+            TSeqTitles& seqTitles,
+            objects::ILineErrorListener* pMessageListener);
+
+protected:
+    objects::CFastaIdHandler m_FastaIdHandler;
 };
 
 
