@@ -1,4 +1,4 @@
-/*  $Id: bed_reader.hpp 532427 2017-04-05 12:45:59Z ludwigf $
+/*  $Id: bed_reader.hpp 572342 2018-10-11 17:14:40Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -46,6 +46,8 @@ BEGIN_NCBI_SCOPE
 
 BEGIN_SCOPE(objects) // namespace ncbi::objects::
 
+class CLinePreBuffer;
+
 //  ----------------------------------------------------------------------------
 class CRawBedRecord
 //  ----------------------------------------------------------------------------
@@ -207,6 +209,17 @@ public:
         ILineErrorListener* =0 );
 
 protected:
+    virtual bool xDetermineLikelyColumnCount(
+        CLinePreBuffer&,
+        ILineErrorListener*);
+
+    virtual bool xSplitColumns(
+        const string&,
+        vector<string>&);
+
+    virtual bool xAddDefaultColumns(
+        vector<string>&);
+
     virtual bool xParseTrackLine(
         const string&,
         ILineErrorListener*);
@@ -254,19 +267,19 @@ protected:
         CRef<CSeq_annot>&,
         ILineErrorListener*);
 
-    bool xAppendFeatureGene(
+    CRef<CSeq_feat> xAppendFeatureGene(
         const vector<string>&,
         CRef<CSeq_annot>&,
         unsigned int,
         ILineErrorListener*);
 
-    bool xAppendFeatureRna(
+    CRef<CSeq_feat> xAppendFeatureRna(
         const vector<string>&,
         CRef<CSeq_annot>&,
         unsigned int,
         ILineErrorListener*);
 
-    bool xAppendFeatureCds(
+    CRef<CSeq_feat> xAppendFeatureCds(
         const vector<string>&,
         CRef<CSeq_annot>&,
         unsigned int,
@@ -326,7 +339,8 @@ protected:
         
     void xSetFeatureBedData(
         CRef<CSeq_feat>&,
-        const vector<string>&);
+        const vector<string>&,
+        ILineErrorListener*);
         
     void xSetFeatureTitle(
         CRef<CSeq_feat>&,
@@ -337,18 +351,21 @@ protected:
         const vector<string>&);
     void xSetFeatureColor(
         CRef<CUser_object>,
-        const vector<string>&);
+        const vector<string>&,
+        ILineErrorListener*);
 
     void xSetFeatureColorFromItemRgb(
         CRef<CUser_object>,
-        const string&);
+        const string&,
+        ILineErrorListener*);
     void xSetFeatureColorFromScore(
         CRef<CUser_object>,
         const string&);
     void xSetFeatureColorByStrand(
         CRef<CUser_object>,
         const string&,
-        ENa_strand);
+        ENa_strand,
+        ILineErrorListener*);
     void xSetFeatureColorDefault(
         CRef<CUser_object>);
 
@@ -401,12 +418,16 @@ protected:
     //
 protected:
     string m_currentId;
-
-    vector<string>::size_type m_columncount;
+    string mColumnSeparator;
+    NStr::TSplitFlags mColumnSplitFlags;
+    vector<string>::size_type mRealColumnCount;
+    vector<string>::size_type mValidColumnCount;
+    bool mAssumeErrorsAreRecordLevel;
     unsigned int m_CurrentFeatureCount;
     bool m_usescore;
     unsigned int m_CurBatchSize;
     const unsigned int m_MaxBatchSize;
+    unique_ptr<CLinePreBuffer> mLinePreBuffer;
 };
 
 END_SCOPE(objects)
