@@ -1,4 +1,4 @@
- /*  $Id: gtf_reader.hpp 543903 2017-08-15 14:52:01Z ivanov $
+ /*  $Id: gtf_reader.hpp 547755 2017-10-03 18:03:15Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -73,24 +73,9 @@ public:
 
     virtual ~CGtfReader();
     
-    virtual void
-    ReadSeqAnnots(
-        TAnnots&,
-        CNcbiIstream&,
-        ILineErrorListener* =0 );
-                        
-    virtual void
-    ReadSeqAnnots(
-        TAnnots&,
-        ILineReader&,
-        ILineErrorListener* =0 );
-
 protected:
     virtual CGff2Record* x_CreateRecord() { return new CGtfReadRecord(); };    
 
-    bool xNeedsNewSeqAnnot(
-        const string&);
-
     virtual bool x_UpdateAnnotFeature(
         const CGff2Record&,
         CRef< CSeq_annot >,
@@ -202,7 +187,6 @@ protected:
     bool x_CdsIsPartial(
         const CGff2Record& );
 
-    string m_CurrentSeqId;
     typedef map< string, CRef< CSeq_feat > > TIdToFeature;
     TIdToFeature m_GeneMap;
     TIdToFeature m_CdsMap;
