@@ -48,7 +48,7 @@ BEGIN_NCBI_SCOPE
 
 BEGIN_objects_SCOPE // namespace ncbi::objects::
 
-class CFeature_table_reader_imp;
+class CFeatureTableReader_Imp;
 class ILineErrorListener;
 class ITableFilter;
 class CSeq_entry;
@@ -76,9 +76,21 @@ public:
 
     CFeature_table_reader(TReaderFlags fReaderFlags = 0);
 
+    CFeature_table_reader(ILineReader& lr, 
+                          ILineErrorListener* pErrors=0);
+
+    ~CFeature_table_reader();
+
+
+    using TAnnots = CReaderBase::TAnnots;
+
     // For CReaderBase
-    CRef<CSerialObject> ReadObject   (ILineReader &lr, ILineErrorListener *pErrors);
-    CRef<CSeq_annot>    ReadSeqAnnot (ILineReader &lr, ILineErrorListener *pErrors);
+    CRef<CSerialObject> ReadObject   (ILineReader &lr, ILineErrorListener *pErrors) override;
+    CRef<CSeq_annot>    ReadSeqAnnot (ILineReader &lr, ILineErrorListener *pErrors) override;
+
+    CRef<CSeq_annot> ReadSequinFeatureTable(const TFlags flags=0,
+                                            ITableFilter* filter=0,
+                                            const string& seqid_prefix = kEmptyStr);
 
     // read 5-column feature table and return Seq-annot
     static CRef<CSeq_annot> ReadSequinFeatureTable(ILineReader& reader,
@@ -158,6 +170,18 @@ public:
         string & out_annotname );
 
 private:
+    static CRef<CSeq_annot> x_ReadFeatureTable(CFeatureTableReader_Imp& reader, 
+                                               const string& seqid,
+                                               const string& annot_name,
+                                               const TFlags flags, 
+                                               ITableFilter* filter);
+
+    static CRef<CSeq_annot> x_ReadFeatureTable(CFeatureTableReader_Imp& reader, 
+                                               const TFlags flags,
+                                               ITableFilter* filter,
+                                               const string& seqid_prefix=kEmptyStr);
+
+    unique_ptr<CFeatureTableReader_Imp> m_pImpl;
 };
 
 
