@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS___SOURCE_MOD_PARSER__HPP
 #define OBJTOOLS_READERS___SOURCE_MOD_PARSER__HPP
 
-/*  $Id: source_mod_parser.hpp 542051 2017-07-25 16:55:29Z ivanov $
+/*  $Id: source_mod_parser.hpp 572183 2018-10-10 12:13:06Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -48,6 +48,7 @@
 #include <objects/seqfeat/Prot_ref.hpp>
 
 #include <objtools/readers/reader_exception.hpp>
+#include <objtools/readers/line_error.hpp>
 
 #include <set>
 #include <map>
@@ -62,6 +63,8 @@ BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects)
 
 class CBioseq;
+class ILineErrorListener;
+class CPCRReactionSet;
 // class CSubmit_block;
 
 /////////////////////////////////////////////////////////////////////////////
@@ -78,6 +81,7 @@ class CBioseq;
 
 template<class _T>
 class CAutoInitDesc;
+class CAutoAddDBLink;
 
 class NCBI_XOBJREAD_EXPORT CSourceModParser
 {
@@ -87,10 +91,25 @@ public:
         // although it may be incomplete if you exit early (e.g. from throw)
         eHandleBadMod_Ignore = 1, // behavior of C toolkit is to ignore
         eHandleBadMod_Throw,
-        eHandleBadMod_PrintToCerr
+        eHandleBadMod_PrintToCerr,
+        eHandleBadMod_ErrorListener
     };
-    CSourceModParser(EHandleBadMod handleBadMod = eHandleBadMod_Ignore) 
-        : m_HandleBadMod(handleBadMod) { }
+
+    /// Construct object providing old-style error handling directives
+    CSourceModParser(
+        EHandleBadMod handleBadMod = eHandleBadMod_Ignore) : 
+        m_HandleBadMod(handleBadMod), 
+        m_pErrorListener(nullptr) 
+    {}
+
+    /// Construct object providing external error handler
+    CSourceModParser(
+            ILineErrorListener* pErrorListener,
+            size_t lineNumber =1) :
+        m_HandleBadMod(eHandleBadMod_ErrorListener), 
+        m_pErrorListener(pErrorListener),
+        m_LineNumber(lineNumber) 
+    {}
 
     /// Extract and store bracketed modifiers from a title string, returning a
     /// stripped version (which may well be empty at that point!)
@@ -113,6 +132,7 @@ public:
     void ApplyTPAMods(CUser_object& tpa);
     void ApplyGenomeProjectsDBMods(CUser_object& gpdb);
     void ApplyPubMods(CBioseq& seq);
+    
 
     static int CompareKeys(const CTempString& lhs, const CTempString& rhs);
 
@@ -175,6 +195,20 @@ public:
             const string & sAllowedValues );
     };
 
+    class CUnkModError : public runtime_error {
+    public:
+        CUnkModError(
+            const SMod& unkMod);
+        ~CUnkModError() THROWS_NONE {};
+
+        const SMod& GetUnkMod() const { return m_UnkMod; };
+
+    private:
+        SMod m_UnkMod;
+        std::string x_CalculateErrorString(
+            const SMod& unkMod );
+    };
+
     /// Used for passing an empty mod to some funcs without having to
     /// constantly recreate an empty one.
     static CSafeStatic<CSourceModParser::SMod> kEmptyMod;
@@ -188,11 +222,12 @@ public:
 
     /// If a modifier with either key is present, mark it as used and
     /// return it; otherwise, return NULL.
-    const SMod* FindMod(const SMod & smod,
-                        const SMod & alt_smod = kEmptyMod.Get());
+
+    const SMod* FindMod(const CTempString& key, const CTempString& alt_key = CTempString());
 
     /// Return all modifiers with the given key (e.g., db_xref), marking them
     /// as used along the way.
+    TModsRange FindAllMods(const CTempString& key, const CTempString& alt_key);
     TModsRange FindAllMods(const CTempString& key);
 
     /// Return all modifiers with the given key (e.g., db_xref), marking them
@@ -235,6 +270,8 @@ private:
     static const unsigned char kKeyCanonicalizationTable[257];
 
     EHandleBadMod m_HandleBadMod;
+    ILineErrorListener* m_pErrorListener;
+    size_t m_LineNumber;
 
     TMods m_Mods;
     TMods m_BadMods;
@@ -250,13 +287,18 @@ private:
     // void x_ApplyMods(CAutoInitRef<CSubmit_block>& sb);
     void x_ApplyTPAMods(CAutoInitRef<CUser_object>& tpa);
     void x_ApplyGenomeProjectsDBMods(CAutoInitRef<CUser_object>& gpdb);
-
+    void x_AddPCRPrimers(CAutoInitRef<CPCRReactionSet>& pcr_reaction_set);
+    void x_ApplyDBLinkMods(CBioseq& bioseq);
 
     // sAllowedValues, enum_values, etc. are combined to produce the final list of
     // allowed values.
     // TModMap is some kind of map whose keys are "const char *".
     void x_HandleBadModValue(
         const SMod& mod );
+    void x_HandleUnkModValue(
+        const SMod& mod );
+    void x_ProcessError(
+        CObjReaderLineException& err);
 };
 
 
