@@ -1,6 +1,6 @@
 #ifndef EXCEPTION__HPP
 #define EXCEPTION__HPP
-/*  $Id: exception.hpp 445781 2014-09-08 17:33:32Z vasilche $
+/*  $Id: exception.hpp 547688 2017-10-03 13:21:50Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -67,7 +67,7 @@ public:
     NCBI_EXCEPTION_DEFAULT(CSerialException,CException);
 
 public:
-    // Combine steram frames info into single message
+    // Combine stream frames info into single message
     void AddFrameInfo(string frame_info);
     virtual void ReportExtra(ostream& out) const;
 
@@ -141,6 +141,28 @@ protected:
     virtual const CException* x_Clone(void) const;
 };
 
+class NCBI_XSERIAL_EXPORT CSerialFacetException : public CSerialException
+{
+public:
+    /// Error codes
+    enum EErrCode {
+        eMinLength,
+        eMaxLength,
+        eLength,
+        ePattern,
+        eInclusiveMinimum,
+        eExclusiveMinimum,
+        eInclusiveMaximum,
+        eExclusiveMaximum,
+        eMultipleOf,
+        eMinItems,
+        eMaxItems,
+        eUniqueItems
+    };
+    virtual const char* GetErrCodeString(void) const;
+    NCBI_EXCEPTION_DEFAULT(CSerialFacetException,CSerialException);
+};
+
 END_NCBI_SCOPE
 
 
