@@ -1,7 +1,7 @@
 #ifndef SERIAL___OBJISTR__HPP
 #define SERIAL___OBJISTR__HPP
 
-/*  $Id: objistr.hpp 507795 2016-07-21 17:24:14Z gouriano $
+/*  $Id: objistr.hpp 546713 2017-09-20 18:52:46Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -482,14 +482,14 @@ public:
     ///
     /// @return
     ///   string
-    virtual string GetStackTrace(void) const;
+    virtual string GetStackTrace(void) const override;
 
     /// Get current stream position as string.
     /// Useful for diagnostic and information messages.
     ///
     /// @return
     ///   string
-    virtual string GetPosition(void) const;
+    virtual string GetPosition(void) const override;
 
 //---------------------------------------------------------------------------
 // Local read hooks
@@ -771,7 +771,7 @@ public:
         eOtherPointer
     };
 
-    void SkipPointer(TTypeInfo declaredType);
+    virtual void SkipPointer(TTypeInfo declaredType);
 
 #ifdef NCBI_STRICT_GI
     void ReadStd(TGi& data);
@@ -817,7 +817,7 @@ public:
     // report unended block
     void Unended(const string& msg);
     // report unended object stack frame
-    virtual void UnendedFrame(void);
+    virtual void UnendedFrame(void) override;
     // report class member errors
     void DuplicatedMember(const CMemberInfo* memberInfo);
     bool ExpectedMember(const CMemberInfo* memberInfo);
@@ -1006,7 +1006,7 @@ protected:
     CObjectIStream(ESerialDataFormat format);
     CObjectIStream(CNcbiIstream& in, bool deleteIn = false);
     // reset state machine
-    virtual void ResetState(void);
+    virtual void ResetState(void) override;
 
     typedef size_t TObjectIndex;
     // low level readers
@@ -1019,7 +1019,7 @@ protected:
     void RegisterObject(TTypeInfo typeInfo);
     void RegisterObject(TObjectPtr object, TTypeInfo typeInfo);
     const CReadObjectInfo& GetRegisteredObject(TObjectIndex index);
-    virtual void x_SetPathHooks(bool set);
+    virtual void x_SetPathHooks(bool set) override;
     bool x_HavePathHooks() const;
     EFixNonPrint x_GetFixCharsMethodDefault(void) const;
     EFixNonPrint x_FixCharsMethod(void) const {
@@ -1030,6 +1030,7 @@ protected:
     bool m_DiscardCurrObject;
     ESerialDataFormat   m_DataFormat;
     EDelayBufferParsing  m_ParseDelayBuffers;
+    TTypeInfo m_TypeAlias;
     
 private:
     static CObjectIStream* CreateObjectIStreamAsn(void);
@@ -1072,7 +1073,7 @@ public:
         eReadAsNormal  = 0,
         eReadAsDefault = 1,
         eReadAsNil     = 2,
-        eReadAsBigInt  = 3
+        eReadAsBigInt  = 4
     };
 private:
     TConstObjectPtr m_MemberDefault;
@@ -1081,13 +1082,27 @@ private:
 
     void SetMemberDefault( TConstObjectPtr def)
     {
-        m_MemberDefault = def;
         m_SpecialCaseUsed = eReadAsNormal;
-        m_SpecialCaseToExpect = def ? eReadAsDefault : eReadAsNormal;
+        m_MemberDefault = def;
+        if (def) {
+            m_SpecialCaseToExpect = m_SpecialCaseToExpect | (int)eReadAsDefault;
+        }
+    }
+    void UnsetMemberSpecialCase(void)
+    {
+        m_SpecialCaseUsed = eReadAsNormal;
+        m_MemberDefault = nullptr;
+        m_SpecialCaseToExpect = eReadAsNormal;
     }
     void SetMemberNillable()
     {
-        m_SpecialCaseToExpect = (m_SpecialCaseToExpect | (int)eReadAsNil);
+        m_SpecialCaseUsed = eReadAsNormal;
+        m_SpecialCaseToExpect = m_SpecialCaseToExpect | (int)eReadAsNil;
+    }
+    void UnsetMemberNillable()
+    {
+        m_SpecialCaseUsed = eReadAsNormal;
+        m_SpecialCaseToExpect = m_SpecialCaseToExpect & ~((int)eReadAsNil);
     }
 public:
     int ExpectSpecialCase(void) const
@@ -1115,6 +1130,7 @@ public:
     CLocalHookSet<CSkipClassMemberHook> m_ClassMemberSkipHookKey;
     CLocalHookSet<CSkipChoiceVariantHook> m_ChoiceVariantSkipHookKey;
 
+    friend class CObjectOStream;
     friend class CObjectStreamCopier;
     friend class CMemberInfoFunctions;
     friend class CClassTypeInfo;
