@@ -1,7 +1,7 @@
 #ifndef SERIAL___RPCBASE_IMPL__HPP
 #define SERIAL___RPCBASE_IMPL__HPP
 
-/*  $Id: rpcbase_impl.hpp 541584 2017-07-19 16:03:23Z ivanov $
+/*  $Id: rpcbase_impl.hpp 554977 2018-01-11 14:18:53Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -115,9 +115,9 @@ private:
 protected:
     string                   m_Service; ///< Used by default Connect().
     string                   m_Args;
-    auto_ptr<CNcbiIostream>  m_Stream; // This must be destroyed after m_In/m_Out.
-    auto_ptr<CObjectIStream> m_In;
-    auto_ptr<CObjectOStream> m_Out;
+    unique_ptr<CNcbiIostream>  m_Stream; // This must be destroyed after m_In/m_Out.
+    unique_ptr<CObjectIStream> m_In;
+    unique_ptr<CObjectOStream> m_Out;
     string                   m_Affinity;
     unsigned int             m_RetryLimit;
     CHttpRetryContext        m_RetryCtx;
