@@ -1,33 +1,28 @@
 #ifndef BMENCODING_H__INCLUDED__
 #define BMENCODING_H__INCLUDED__
 /*
-Copyright(c) 2002-2009 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
+Copyright(c) 2002-2017 Anatoliy Kuznetsov(anatoliy_kuznetsov at yahoo.com)
 
+Licensed under the Apache License, Version 2.0 (the "License");
+you may not use this file except in compliance with the License.
+You may obtain a copy of the License at
 
-Permission is hereby granted, free of charge, to any person 
-obtaining a copy of this software and associated documentation 
-files (the "Software"), to deal in the Software without restriction, 
-including without limitation the rights to use, copy, modify, merge, 
-publish, distribute, sublicense, and/or sell copies of the Software, 
-and to permit persons to whom the Software is furnished to do so, 
-subject to the following conditions:
+    http://www.apache.org/licenses/LICENSE-2.0
 
-The above copyright notice and this permission notice shall be included 
-in all copies or substantial portions of the Software.
-
-THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
-EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
-OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
-IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
-DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
-ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
-OTHER DEALINGS IN THE SOFTWARE.
-
-
-For more information please visit:   http://bmagic.sourceforge.net
+Unless required by applicable law or agreed to in writing, software
+distributed under the License is distributed on an "AS IS" BASIS,
+WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+See the License for the specific language governing permissions and
+limitations under the License.
 
+For more information please visit:  http://bitmagic.io
 */
 
+/*! \file encoding.h
+    \brief Encoding utilities for serialization (internal)
+*/
+
+
 #include <memory.h>
 #include "bmutil.h"
 
@@ -49,7 +44,7 @@ class encoder
 public:
     typedef unsigned char* position_type;
 public:
-    encoder(unsigned char* buf, unsigned size);
+    encoder(unsigned char* buf, size_t size);
     void put_8(unsigned char c);
     void put_16(bm::short_t  s);
     void put_16(const bm::short_t* s, unsigned count);
@@ -61,13 +56,14 @@ public:
     void put_prefixed_array_16(unsigned char c, 
                                const bm::short_t* s, unsigned count,
                                bool encode_count);
+    void memcpy(const unsigned char* src, size_t count);
     unsigned size() const;
     unsigned char* get_pos() const;
     void set_pos(unsigned char* buf_pos);
 private:
     unsigned char*  buf_;
     unsigned char*  start_;
-    unsigned int    size_;
+    size_t          size_;
 };
 
 // ----------------------------------------------------------------
@@ -79,14 +75,21 @@ class decoder_base
 {
 public:
     decoder_base(const unsigned char* buf) { buf_ = start_ = buf; }
+    
     /// Reads character from the decoding buffer. 
-    BMFORCEINLINE unsigned char get_8() { return *buf_++; }
+    unsigned char get_8() { return *buf_++; }
+    
     /// Returns size of the current decoding stream.
-    BMFORCEINLINE 
     unsigned size() const { return (unsigned)(buf_ - start_); }
+    
     /// change current position
-    BMFORCEINLINE
     void seek(int delta) { buf_ += delta; }
+    
+    /// read bytes from the decode buffer
+    void memcpy(unsigned char* dst, size_t count);
+    
+    /// Return current buffer pointer
+    const unsigned char* get_pos() const { return buf_; }
 protected:
    const unsigned char*   buf_;
    const unsigned char*   start_;
@@ -107,6 +110,8 @@ public:
     bm::word_t get_32();
     bm::id64_t get_64();
     void get_32(bm::word_t* w, unsigned count);
+    bool get_32_OR(bm::word_t* w, unsigned count);
+    void get_32_AND(bm::word_t* w, unsigned count);
     void get_16(bm::short_t* s, unsigned count);
 };
 
@@ -136,6 +141,8 @@ public:
     bm::short_t get_16();
     bm::word_t get_32();
     void get_32(bm::word_t* w, unsigned count);
+    bool get_32_OR(bm::word_t* w, unsigned count);
+    void get_32_AND(bm::word_t* w, unsigned count);
     void get_16(bm::short_t* s, unsigned count);
 };
 
@@ -174,15 +181,16 @@ public:
         unsigned acc = accum_;
 
         {
-            unsigned mask = ~0;
+            unsigned mask = ~0u;
             mask >>= (sizeof(accum_) * 8) - count;
             value &= mask;
         }
         for (;count;)
         {  
+            unsigned free_bits = unsigned(sizeof(accum_) * 8) - used;
+            BM_ASSERT(free_bits);
             acc |= value << used;
 
-            unsigned free_bits = (sizeof(accum_) * 8) - used;
             if (count <= free_bits)
             {
                 used += count;
@@ -197,6 +205,11 @@ public:
                 continue;
             }
         }
+        if (used == (sizeof(accum_) * 8))
+        {
+            dest_.put_32(acc);
+            acc = used = 0;
+        }
         used_bits_ = used;
         accum_ = acc;
     }
@@ -207,9 +220,9 @@ public:
             flush_accum();        
     }
 
-    void put_zero_bits(register unsigned count)
+    void put_zero_bits(unsigned count)
     {
-        register unsigned used = used_bits_;
+        unsigned used = used_bits_;
         unsigned free_bits = (sizeof(accum_) * 8) - used;
         if (count >= free_bits)
         {
@@ -239,12 +252,7 @@ public:
     {
         BM_ASSERT(value);
 
-        unsigned logv = 
-        #if defined(BM_x86) && (defined(__GNUG__) || defined(_MSC_VER))
-            bm::bsr_asm32(value);
-        #else
-            bm::ilog2_LUT(value);
-        #endif
+        unsigned logv = bm::bit_scan_reverse32(value);
 
         // Put zeroes + 1 bit
 
@@ -254,29 +262,29 @@ public:
         unsigned free_bits = acc_bits - used;
 
         {
-        unsigned count = logv;
-        if (count >= free_bits)
-        {
-            dest_.put_32(acc);
-            acc = used ^= used;
-            count -= free_bits;
-
-            for ( ;count >= acc_bits; count -= acc_bits)
+            unsigned count = logv;
+            if (count >= free_bits)
             {
-                dest_.put_32(0);
+                dest_.put_32(acc);
+                acc = used ^= used;
+                count -= free_bits;
+
+                for ( ;count >= acc_bits; count -= acc_bits)
+                {
+                    dest_.put_32(0);
+                }
+                used += count;
+            }
+            else
+            {
+                used += count;
+            }
+            acc |= (1 << used);
+            if (++used == acc_bits)
+            {
+                dest_.put_32(acc);
+                acc = used ^= used;
             }
-            used += count; 
-        }
-        else
-        {
-            used += count;
-        }
-        acc |= (1 << used);
-        if (++used == acc_bits)
-        {
-            dest_.put_32(acc);
-            acc = used ^= used;
-        }
         }
 
         // Put the value bits
@@ -431,6 +439,36 @@ public:
         used_bits_ = used;
         return current;
     }
+    
+    unsigned get_bits(unsigned count)
+    {
+        BM_ASSERT(count);
+        
+        unsigned value = 0;
+        unsigned free_bits = unsigned((sizeof(accum_) * 8) - used_bits_);
+        if (count <= free_bits)
+        {
+        take_accum:
+            value =
+                (accum_ & block_set_table<true>::_left[count-1]);
+            accum_ >>= count;
+            used_bits_ += count;
+            return value;
+        }
+        if (used_bits_ == (sizeof(accum_) * 8))
+        {
+            accum_ = src_.get_32();
+            used_bits_ ^= used_bits_;
+            goto take_accum;
+        }
+        value = accum_;
+        accum_ = src_.get_32();
+        used_bits_ = count - free_bits;
+        value |=
+            ((accum_ & block_set_table<true>::_left[used_bits_-1]) << free_bits);
+        accum_ >>= used_bits_;
+        return value;
+    }
 
 
 private:
@@ -518,7 +556,7 @@ private:
     \param buf - memory buffer pointer.
     \param size - size of the buffer
 */
-inline encoder::encoder(unsigned char* buf, unsigned a_size)
+inline encoder::encoder(unsigned char* buf, size_t a_size)
 : buf_(buf), start_(buf)
 {
     size_ = a_size;
@@ -608,6 +646,18 @@ inline void encoder::put_16(const bm::short_t* s, unsigned count)
 #endif
 }
 
+/*!
+    \brief copy bytes into target buffer or just rewind if src is NULL
+*/
+inline
+void encoder::memcpy(const unsigned char* src, size_t count)
+{
+    BM_ASSERT((buf_ + count) < (start_ + size_));
+    if (src)
+        ::memcpy(buf_, src, count);
+    buf_ += count;
+}
+
 
 /*!
    \fn unsigned encoder::size() const
@@ -640,7 +690,7 @@ inline void encoder::set_pos(encoder::position_type buf_pos)
    \brief Puts 32 bits word into encoding buffer.
    \param w - word to encode.
 */
-BMFORCEINLINE void encoder::put_32(bm::word_t w)
+inline void encoder::put_32(bm::word_t w)
 {
 #if (BM_UNALIGNED_ACCESS_OK == 1)
 	*((bm::word_t*) buf_) = w;
@@ -715,6 +765,18 @@ void encoder::put_32(const bm::word_t* w, unsigned count)
 
 // ---------------------------------------------------------------------
 
+
+/*!
+    Load bytes from the decode buffer
+*/
+inline
+void decoder_base::memcpy(unsigned char* dst, size_t count)
+{
+    if (dst)
+        ::memcpy(dst, buf_, count);
+    buf_ += count;
+}
+
 /*!
    \fn decoder::decoder(const unsigned char* buf) 
    \brief Construction
@@ -765,7 +827,7 @@ inline bm::id64_t decoder::get_64()
 #if (BM_UNALIGNED_ACCESS_OK == 1)
 	bm::id64_t a = *((bm::id64_t*)buf_);
 #else
-	bm::word_t a = buf_[0]+
+	bm::id64_t a = buf_[0]+
                    ((bm::id64_t)buf_[1] << 8)  +
                    ((bm::id64_t)buf_[2] << 16) +
                    ((bm::id64_t)buf_[3] << 24) +
@@ -789,12 +851,12 @@ inline void decoder::get_32(bm::word_t* w, unsigned count)
 {
     if (!w) 
     {
-        seek(count * 4);
+        seek(int(count * sizeof(bm::word_t)));
         return;
     }
 #if (BM_UNALIGNED_ACCESS_OK == 1)
-	memcpy(w, buf_, count * sizeof(bm::word_t));
-	seek(count * 4);
+	::memcpy(w, buf_, count * sizeof(bm::word_t));
+	seek(int(count * sizeof(bm::word_t)));
 	return;
 #else
     const unsigned char* buf = buf_;
@@ -809,6 +871,86 @@ inline void decoder::get_32(bm::word_t* w, unsigned count)
     buf_ = (unsigned char*)buf;
 #endif
 }
+ 
+/*!
+   \brief Reads block of 32-bit words from the decoding buffer and ORs
+   to the destination
+   \param w - pointer on memory block to read into
+   \param count - should match bm::set_block_size
+*/
+inline
+bool decoder::get_32_OR(bm::word_t* w, unsigned count)
+{
+    if (!w)
+    {
+        seek(int(count * sizeof(bm::word_t)));
+        return false;
+    }
+#if defined(BMAVX2OPT)
+        __m256i* buf_start = (__m256i*)buf_;
+        seek(int(count * sizeof(bm::word_t)));
+        __m256i* buf_end = (__m256i*)buf_;
+
+        return bm::avx2_or_arr_unal((__m256i*)w, buf_start, buf_end);
+#elif defined(BMSSE42OPT) || defined(BMSSE2OPT)
+        __m128i* buf_start = (__m128i*)buf_;
+        seek(int(count * sizeof(bm::word_t)));
+        __m128i* buf_end = (__m128i*)buf_;
+
+        return bm::sse2_or_arr_unal((__m128i*)w, buf_start, buf_end);
+#else
+        bm::word_t acc = 0;
+        const bm::word_t not_acc = acc = ~acc;
+
+        for (unsigned i = 0; i < count; i+=4)
+        {
+            acc &= (w[i+0] |= get_32());
+            acc &= (w[i+1] |= get_32());
+            acc &= (w[i+2] |= get_32());
+            acc &= (w[i+3] |= get_32());
+        }
+        return acc == not_acc;
+#endif
+}
+
+/*!
+   \brief Reads block of 32-bit words from the decoding buffer and ANDs
+   to the destination
+   \param w - pointer on memory block to read into
+   \param count - should match bm::set_block_size
+*/
+inline
+void decoder::get_32_AND(bm::word_t* w, unsigned count)
+{
+    if (!w)
+    {
+        seek(int(count * sizeof(bm::word_t)));
+        return;
+    }
+#if defined(BMAVX2OPT)
+        __m256i* buf_start = (__m256i*)buf_;
+        seek(int(count * sizeof(bm::word_t)));
+        __m256i* buf_end = (__m256i*)buf_;
+
+        bm::avx2_and_arr_unal((__m256i*)w, buf_start, buf_end);
+#elif defined(BMSSE42OPT) || defined(BMSSE2OPT)
+        __m128i* buf_start = (__m128i*)buf_;
+        seek(int(count * sizeof(bm::word_t)));
+        __m128i* buf_end = (__m128i*)buf_;
+
+        bm::sse2_and_arr_unal((__m128i*)w, buf_start, buf_end);
+#else
+        for (unsigned i = 0; i < count; i+=4)
+        {
+            w[i+0] &= get_32();
+            w[i+1] &= get_32();
+            w[i+2] &= get_32();
+            w[i+3] &= get_32();
+        }
+#endif
+}
+
+
 
 /*!
    \fn void decoder::get_16(bm::short_t* s, unsigned count)
@@ -820,7 +962,7 @@ inline void decoder::get_16(bm::short_t* s, unsigned count)
 {
     if (!s) 
     {
-        seek(count * 2);
+        seek(int(count * sizeof(bm::short_t)));
         return;
     }
 #if (BM_UNALIGNED_ACCESS_OK == 1)
@@ -852,14 +994,18 @@ inline decoder_little_endian::decoder_little_endian(const unsigned char* buf)
 {
 }
 
-BMFORCEINLINE bm::short_t decoder_little_endian::get_16()
+BMFORCEINLINE
+bm::short_t decoder_little_endian::get_16()
 {
-    bm::short_t a = bm::short_t((bm::short_t)buf_[0] << 8) + ((bm::short_t)buf_[1]);
+    bm::short_t v1 = bm::short_t(buf_[0]);
+    bm::short_t v2 = bm::short_t(buf_[1]);
+    bm::short_t a = bm::short_t((v1 << 8) + v2);
     buf_ += sizeof(a);
     return a;
 }
 
-BMFORCEINLINE bm::word_t decoder_little_endian::get_32() 
+BMFORCEINLINE
+bm::word_t decoder_little_endian::get_32()
 {
     bm::word_t a = ((unsigned)buf_[0] << 24)+ ((unsigned)buf_[1] << 16) +
                    ((unsigned)buf_[2] << 8) + ((unsigned)buf_[3]);
@@ -867,11 +1013,12 @@ BMFORCEINLINE bm::word_t decoder_little_endian::get_32()
     return a;
 }
 
-inline void decoder_little_endian::get_32(bm::word_t* w, unsigned count)
+inline
+void decoder_little_endian::get_32(bm::word_t* w, unsigned count)
 {
     if (!w) 
     {
-        seek(count * 4);
+        seek(int(count * sizeof(bm::word_t)));
         return;
     }
 
@@ -887,11 +1034,47 @@ inline void decoder_little_endian::get_32(bm::word_t* w, unsigned count)
     buf_ = (unsigned char*)buf;
 }
 
-inline void decoder_little_endian::get_16(bm::short_t* s, unsigned count)
+inline
+bool decoder_little_endian::get_32_OR(bm::word_t* w, unsigned count)
+{
+    if (!w)
+    {
+        seek(int(count * sizeof(bm::word_t)));
+        return false;
+    }
+
+    bm::word_t acc = 0;
+    const bm::word_t not_acc = acc = ~acc;
+
+    for (unsigned i = 0; i < count; i+=4)
+    {
+        acc &= (w[i+0] |= get_32());
+        acc &= (w[i+1] |= get_32());
+        acc &= (w[i+2] |= get_32());
+        acc &= (w[i+3] |= get_32());
+    }
+    return acc == not_acc;
+}
+
+inline
+void decoder_little_endian::get_32_AND(bm::word_t* w, unsigned count)
+{
+    for (unsigned i = 0; i < count; i+=4)
+    {
+        w[i+0] &= get_32();
+        w[i+1] &= get_32();
+        w[i+2] &= get_32();
+        w[i+3] &= get_32();
+    }
+}
+
+
+inline
+void decoder_little_endian::get_16(bm::short_t* s, unsigned count)
 {
     if (!s) 
     {
-        seek(count * 2);
+        seek(int(count * sizeof(bm::short_t)));
         return;
     }
 
@@ -899,7 +1082,9 @@ inline void decoder_little_endian::get_16(bm::short_t* s, unsigned count)
     const bm::short_t* s_end = s + count;
     do 
     {
-        bm::short_t a = bm::short_t((bm::short_t)buf[0] << 8) + ((bm::short_t)buf[1]);
+        bm::short_t v1 = bm::short_t(buf_[0]);
+        bm::short_t v2 = bm::short_t(buf_[1]);
+        bm::short_t a = bm::short_t((v1 << 8) + v2);
         *s++ = a;
         buf += sizeof(a);
     } while (s < s_end);
