@@ -1,7 +1,7 @@
 #ifndef UTIL___ICACHE__HPP
 #define UTIL___ICACHE__HPP
 
-/*  $Id: icache.hpp 534859 2017-05-03 12:47:35Z ivanov $
+/*  $Id: icache.hpp 554427 2018-01-03 15:35:44Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -446,10 +446,7 @@ public:
     ///
     /// @param access_timeout
     ///    Time in seconds. All objects older than this are deleted.
-    /// @param keep_last_version
-    ///    type of cleaning action
-    virtual void Purge(time_t         access_timeout,
-                       EKeepVersions  keep_last_version = eDropAll) = 0;
+    virtual void Purge(time_t         access_timeout) { Purge(access_timeout, eDropAll); }
 
     /// Delete BLOBs with access time older than specified
     ///
@@ -461,12 +458,16 @@ public:
     ///    BLOB identification subkey
     /// @param access_timeout
     ///    Time in seconds. All objects older than this are deleted.
-    /// @param keep_last_version
-    ///    type of cleaning action
     virtual void Purge(const string&  key,
                        const string&  subkey,
-                       time_t         access_timeout,
-                       EKeepVersions  keep_last_version = eDropAll) = 0;
+                       time_t         access_timeout) { Purge(key, subkey, access_timeout, eDropAll); }
+
+    /// @deprecated EKeepVersions arguments have been no-op in Purge() for long time and are now deprecated.
+    /// If you are using one of these methods, switch to use one the overloads above instead.
+    /// If you have implemented these, remove EKeepVersions (making them overrides for the overloads above).
+    /// The deprecated will be eventually purged (after all implementations have switched to the overloads above).
+    virtual void Purge(time_t access_timeout, EKeepVersions) { Purge(access_timeout); }
+    virtual void Purge(const string& key, const string& subkey, time_t access_timeout, EKeepVersions) { Purge(key, subkey, access_timeout); }
 
 
     virtual ~ICache() {}
