@@ -1,7 +1,7 @@
 #ifndef UTIL_NCBITABLE__HPP
 #define UTIL_NCBITABLE__HPP
 
-/* $Id: ncbi_table.hpp 327691 2011-07-28 15:07:38Z falkrb $
+/* $Id: ncbi_table.hpp 437336 2014-06-05 13:49:06Z ucko $
 * ===========================================================================
 *
 *                            public DOMAIN NOTICE                          
@@ -81,7 +81,8 @@ public:
     CNcbiTable& operator=(const CNcbiTable& table)
     {
         x_Free();
-        x_Copy();
+        x_Copy(table);
+        return *this;
     }
 
     ~CNcbiTable();
