@@ -1,7 +1,7 @@
 #ifndef UTIL___SYNC_QUEUE__HPP
 #define UTIL___SYNC_QUEUE__HPP
 
-/*  $Id: sync_queue.hpp 490388 2016-01-25 16:58:30Z satskyse $
+/*  $Id: sync_queue.hpp 534859 2017-05-03 12:47:35Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -1330,7 +1330,7 @@ void CSyncQueue<Type, Container, Traits>::x_LockAndWait(
     const
 {
    // TODO: change to use CTimeout instead
-    auto_ptr<CTimeSpan> real_timeout;
+    unique_ptr<CTimeSpan> real_timeout;
 
     if (full_tmo) {
         real_timeout.reset(new CTimeSpan(*full_tmo));
