@@ -5,12 +5,12 @@ SCRIPTDIR=$2
 BLAST_VERSION=$3
 PRODUCT="ncbi-igblast-$BLAST_VERSION+"
 
-INSTALL_LOCATION1=/usr/local/ncbi/blast
+INSTALL_LOCATION1=/usr/local/ncbi/igblast
 INSTALL_LOCATION2=/etc/paths.d
 STAGE_DIR1=_stage1
 STAGE_DIR2=_stage2
 RESOURCES_DIR=Resources
-ID=gov.nlm.nih.ncbi.blast
+ID=gov.nlm.nih.ncbi.igblast
 
 if [ $# -ne 3 ] ; then
     echo "Usage: ncbi-igblast.sh [installation directory] [MacOSX post-build script directory] [BLAST version]";
@@ -41,7 +41,7 @@ prep_binary_component_package()
 
 prep_paths_component_package()
 {
-    echo /usr/local/ncbi/igblast/bin > $STAGE_DIR2/ncbi_blast
+    echo /usr/local/ncbi/igblast/bin > $STAGE_DIR2/ncbi_igblast
     /usr/bin/pkgbuild --root $STAGE_DIR2 --identifier $ID.paths --version \
         $BLAST_VERSION --install-location $INSTALL_LOCATION2 paths.pkg
 }
