@@ -1,6 +1,6 @@
 #! /usr/bin/env python3
 """Script to create the Windows installer for BLAST command line applications"""
-# $Id: make_win.py 547285 2017-09-27 19:34:45Z boratyng $
+# $Id: make_win.py 563079 2018-05-01 21:34:28Z camacho $
 #
 # Author: Christiam camacho
 
@@ -42,14 +42,6 @@ def main():
     VERBOSE = options.VERBOSE
     
     apps = [ "igblastn.exe", "igblastp.exe",
-             "libgcc_s_seh-1.dll",
-             "libgmp-10.dll",
-             "libgnutls-30.dll",
-             "libhogweed-4-2.dll",
-             "libnettle-6-2.dll",
-             "libp11-kit-0.dll",
-             "msvcp120.dll",
-             "msvcr120.dll",
              "ncbi-vdb-md.dll"
             ]
     
