@@ -1,4 +1,4 @@
-/*  $Id: blast_aux.cpp 418171 2013-11-14 22:19:34Z rafanovi $
+/*  $Id: blast_aux.cpp 519527 2016-11-16 14:19:45Z camacho $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -53,6 +53,11 @@
 #include "blast_setup.hpp"
 #include "blast_aux_priv.hpp"
 
+#include <objects/seq/Seq_ext.hpp>
+#include <objects/seq/Delta_seq.hpp>
+#include <objects/seq/Delta_ext.hpp>
+#include <objmgr/seq_map_ci.hpp>
+
 #include <algorithm>
 #include <sstream>
 
@@ -227,6 +232,7 @@ CBlastHitSavingOptions::DebugDump(CDebugDumpContext ddc, unsigned int /*depth*/)
 
     ddc.Log("hitlist_size", m_Ptr->hitlist_size);
     ddc.Log("hsp_num_max", m_Ptr->hsp_num_max);
+    ddc.Log("max_hsps_per_subject", m_Ptr->max_hsps_per_subject);
     ddc.Log("total_hsp_limit", m_Ptr->total_hsp_limit);
     ddc.Log("culling_limit", m_Ptr->culling_limit);
     ddc.Log("expect_value", m_Ptr->expect_value);
@@ -655,7 +661,7 @@ void CAutomaticGenCodeSingleton::AddGeneticCode(int genetic_code)
 
 void ThrowIfInvalidTask(const string& task)
 {
-    static set<string> valid_tasks;
+    set<string> valid_tasks;
     if (valid_tasks.empty()) {
         valid_tasks = CBlastOptionsFactory::GetTasks();
     }
@@ -686,6 +692,7 @@ string EProgramToTaskName(EProgram p)
     case ePHIBlastn:        retval.assign("phiblastn"); break;
     case eDeltaBlast:       retval.assign("deltablast"); break;
     case eVecScreen:        retval.assign("vecscreen"); break;
+    case eMapper:           retval.assign("mapr2g"); break;
     default:
         cerr << "Invalid EProgram value: " << (int)p << endl;
         abort();
@@ -707,6 +714,9 @@ EProgramToEBlastProgramType(EProgram p)
     case eDiscMegablast:
     case eVecScreen:
         return eBlastTypeBlastn;
+
+    case eMapper:
+        return eBlastTypeMapping;
         
     case eBlastp:
         return eBlastTypeBlastp;
@@ -784,6 +794,16 @@ EProgram ProgramNameToEnum(const std::string& program_name)
         return eDeltaBlast;
     } else if (lowercase_program_name == "vecscreen") {
         return eVecScreen;
+    // FIXME: mapper is used in core as a single program name for all tasks,
+    // we may need a better approach to mapping tasks with fewer program names
+    } else if (lowercase_program_name == "mapper") {
+        return eMapper;
+    } else if (lowercase_program_name == "mapr2g") {
+        return eMapper;
+    } else if (lowercase_program_name == "mapr2r") {
+        return eMapper;
+    } else if (lowercase_program_name == "mapg2g") {
+        return eMapper;
     } else {
         NCBI_THROW(CBlastException, eNotSupported, 
                    "Program type '" + program_name + "' not supported");
@@ -898,7 +918,7 @@ Blast_GetSeqLocInfoVector(EBlastProgramType program,
         NCBI_THROW(CBlastException, eInvalidArgument, msg);
     }
 
-    if (program == eBlastTypeBlastn) {
+    if (program == eBlastTypeBlastn || program == eBlastTypeMapping) {
         s_ConvertBlastnMasks(query_intervals, mask, mask_v);
         return;
     }
@@ -1087,6 +1107,51 @@ IsLocalId(const objects::CSeq_id* seqid)
     return retval;
 }
 
+void
+LoadSequencesToScope(CScope::TIds& ids, vector<TSeqRange>& ranges, CRef<CScope> & scope)
+{
+	CScope::TBioseqHandles bhs = scope->GetBioseqHandles(ids);
+
+    // Per Eugene Vasilchenko's suggestion, via email on 6/8/10:
+    // "With the current API you can make artificial delta sequence
+    // referencing several other sequences and use its CSeqMap to load them
+    // all in one call. There is no straightforward way to do this, sorry."
+
+    // Create virtual delta sequence
+	CRef<CBioseq> top_seq(new CBioseq);
+    CSeq_inst& inst = top_seq->SetInst();
+    inst.SetRepr(CSeq_inst::eRepr_virtual);
+    inst.SetMol(CSeq_inst::eMol_not_set);
+    CDelta_ext& delta = inst.SetExt().SetDelta();
+    int i = 0;
+    ITERATE(CScope::TBioseqHandles, it, bhs) {
+    	CRef<CDelta_seq> seq(new CDelta_seq);
+	    CSeq_interval& interval = seq->SetLoc().SetInt();
+	    interval.SetId(*SerialClone(*it->GetAccessSeq_id_Handle().GetSeqId()));
+	    if (ranges[i].GetFrom() > ranges[i].GetToOpen()) {
+	    	TSeqPos length = it->GetBioseqLength();
+	        interval.SetFrom(length - ranges[i].GetFrom());
+	        interval.SetTo(length - ranges[i].GetTo());
+	    } else {
+	        interval.SetFrom(ranges[i].GetFrom());
+	        interval.SetTo(ranges[i].GetTo());
+	    }
+	    i++;
+	    delta.Set().push_back(seq);
+    }
+
+    // Add it to the scope
+    CBioseq_Handle top_bh = scope->AddBioseq(*top_seq);
+
+    // prepare selector. SetLinkUsedTSE() is necessary for batch loading
+	SSeqMapSelector sel(CSeqMap::fFindAnyLeaf, kInvalidSeqPos);
+	sel.SetLinkUsedTSE(top_bh.GetTSE_Handle());
+
+	// and get all sequence data in batch mode
+	_TRACE("Fetching " << ids.size() << " sequences");
+	top_bh.GetSeqMap().CanResolveRange(&*scope, sel);
+}
+
 END_SCOPE(blast)
 END_NCBI_SCOPE
 
