@@ -1,4 +1,4 @@
-/* $Id: magicblast.cpp 548710 2017-10-17 16:02:42Z ivanov $
+/* $Id: magicblast.cpp 574908 2018-11-20 18:35:55Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -208,7 +208,7 @@ static void s_ComputeBtopAndIdentity(const HSPChain* chain,
     _ASSERT(chain);
     _ASSERT(chain->hsps->hsp);
     const Uint1 kGap = 15;
-    
+
     int num_identical = 0;
     int len = 0;
     int md_matches = 0;
@@ -240,14 +240,14 @@ static void s_ComputeBtopAndIdentity(const HSPChain* chain,
 
                 len += hsp->query.offset - prev->query.end;
             }
-        }        
+        }
 
         int query_pos = hsp->query.offset;
         int num_matches = 0;
         for (int i=0;i < hsp_edits->num_edits;i++) {
             num_matches = hsp_edits->edits[i].query_pos - query_pos;
             query_pos += num_matches;
-            
+
             _ASSERT(num_matches >= 0);
             num_identical += num_matches;
             if (num_matches > 0) {
@@ -289,14 +289,7 @@ static void s_ComputeBtopAndIdentity(const HSPChain* chain,
         num_identical += num_matches;
         if (num_matches > 0) {
             btop += NStr::IntToString(num_matches);
-
-            if (hsp_edits->num_edits > 0) {
-                md_tag += NStr::IntToString(num_matches + md_matches);
-                md_matches = 0;
-            }
-            else {
-                md_matches += num_matches;
-            }
+            md_matches += num_matches;
         }
     }
     if (md_matches > 0) {
@@ -436,7 +429,7 @@ CRef<CSeq_align_set> CMagicBlast::x_BuildSeqAlignSet(
             retval->Set().push_back(it);
         }
     }
-    
+
     return retval;
 }
 
@@ -515,7 +508,7 @@ CRef<CMagicBlastResultSet> CMagicBlast::x_BuildResultSet(
 
         retval->push_back(res);
     }
-    
+
     return retval;
 }
 
@@ -548,6 +541,74 @@ CMagicBlastResults::CMagicBlastResults(CConstRef<CSeq_id> query_id,
 }
 
 
+// Sort alignments so that paired, forward-reversed alignments are first
+struct compare_alignments_fwrev_first
+{
+    bool operator()(const CRef<CSeq_align>& a, const CRef<CSeq_align>& b)
+    {
+        if (a->GetSegs().IsDisc() && b->GetSegs().IsDisc()) {
+            const CSeq_align& a_first = *a->GetSegs().GetDisc().Get().front();
+            const CSeq_align& a_second = *a->GetSegs().GetDisc().Get().back();
+            const CSeq_align& b_first = *b->GetSegs().GetDisc().Get().front();
+            const CSeq_align& b_second = *b->GetSegs().GetDisc().Get().back();
+
+            if (a_first.GetSeqStrand(0) == eNa_strand_plus &&
+                a_second.GetSeqStrand(0) == eNa_strand_minus &&
+                a_first.GetSeqStart(1) <= a_second.GetSeqStart(1) &&
+                (b_first.GetSeqStrand(0) != eNa_strand_plus ||
+                 b_second.GetSeqStrand(0) != eNa_strand_minus ||
+                 b_first.GetSeqStart(1) > b_second.GetSeqStart(1))) {
+
+                return true;
+            }
+
+            return false;
+        }
+        
+        return (a->GetSegs().IsDisc() && !b->GetSegs().IsDisc());
+    }
+};
+
+
+// Sort alignments so that paired, reversed-forward alignments are first
+struct compare_alignments_revfw_first
+{
+    bool operator()(const CRef<CSeq_align>& a, const CRef<CSeq_align>& b)
+    {
+        if (a->GetSegs().IsDisc() && b->GetSegs().IsDisc()) {
+            const CSeq_align& a_first = *a->GetSegs().GetDisc().Get().front();
+            const CSeq_align& a_second = *a->GetSegs().GetDisc().Get().back();
+            const CSeq_align& b_first = *b->GetSegs().GetDisc().Get().front();
+            const CSeq_align& b_second = *b->GetSegs().GetDisc().Get().back();
+
+            if (a_first.GetSeqStrand(0) == eNa_strand_minus &&
+                a_second.GetSeqStrand(0) == eNa_strand_plus &&
+                a_second.GetSeqStart(1) <= a_first.GetSeqStart(1) &&
+                (b_first.GetSeqStrand(0) != eNa_strand_minus ||
+                 b_second.GetSeqStrand(0) != eNa_strand_plus ||
+                 b_second.GetSeqStart(1) > b_first.GetSeqStart(1))) {
+
+                return true;
+            }
+
+            return false;
+        }
+        
+        return (a->GetSegs().IsDisc() && !b->GetSegs().IsDisc());
+    }
+};
+
+void CMagicBlastResults::SortAlignments(CMagicBlastResults::EOrdering order)
+{
+    if (order == eFwRevFirst) {
+        m_Aligns->Set().sort(compare_alignments_fwrev_first());
+    }
+    else {
+        m_Aligns->Set().sort(compare_alignments_revfw_first());
+    }
+}
+
+
 void CMagicBlastResults::x_SetInfo(int first_length,
                              const TMaskedQueryRegions* first_mask,
                              int last_length /* = 0 */,
@@ -555,12 +616,14 @@ void CMagicBlastResults::x_SetInfo(int first_length,
 {
     m_FirstInfo = 0;
     m_LastInfo = 0;
+    m_Concordant = false;
 
     bool first_aligned = false;
     bool last_aligned = false;
 
     if (!m_Paired) {
         first_aligned = !m_Aligns->Get().empty();
+        m_Concordant = true;
     }
     else {
 
@@ -568,6 +631,31 @@ void CMagicBlastResults::x_SetInfo(int first_length,
             if (it->GetSegs().IsDisc()) {
                 first_aligned = true;
                 last_aligned = true;
+
+                const CSeq_align_set::Tdata& sasd =
+                        it->GetSegs().GetDisc().Get();
+                ASSERT(sasd.size() == 2);
+
+                CRef<CSeq_align>   sa_q = sasd.front();
+                ENa_strand str_q = sa_q->GetSeqStrand(0);
+                TSeqPos sp_q     = sa_q->GetSeqStart(1);
+
+                CRef<CSeq_align>   sa_m = sasd.back();
+                ENa_strand str_m = sa_m->GetSeqStrand(0);
+                TSeqPos sp_m     = sa_m->GetSeqStart(1);
+
+                if (str_q == eNa_strand_plus
+                        &&  str_m == eNa_strand_minus) {
+                    if (sp_q <= sp_m) {
+                        m_Concordant = true;
+                    }
+                } else if (str_q == eNa_strand_minus
+                        &&  str_m == eNa_strand_plus) {
+                    if (sp_q >= sp_m) {
+                        m_Concordant = true;
+                    }
+                }
+
                 break;
             }
             else if (it->GetSeq_id(0).Match(*m_QueryId)) {
@@ -602,11 +690,16 @@ void CMagicBlastResults::x_SetInfo(int first_length,
     }
 }
 
-CRef<CSeq_align_set> CMagicBlastResultSet::GetFlatResults(void)
+CRef<CSeq_align_set> CMagicBlastResultSet::GetFlatResults(bool no_discordant)
 {
     CRef<CSeq_align_set> retval(new CSeq_align_set);
 
     for (auto result: *this) {
+
+        if (no_discordant && result->IsPaired() && !result->IsConcordant()) {
+            continue;
+        }
+
         for (auto it: result->GetSeqAlign()->Get()) {
             retval->Set().push_back(it);
         }
