@@ -1,7 +1,3 @@
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] =
-    "$Id: objmgr_query_data.cpp 382127 2012-12-03 19:47:11Z rafanovi $";
-#endif /* SKIP_DOXYGEN_PROCESSING */
 
 /* ===========================================================================
  *
