@@ -1,7 +1,3 @@
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] =
-    "$Id: psiblast_aux_priv.cpp 488748 2016-01-05 16:40:40Z ivanov $";
-#endif /* SKIP_DOXYGEN_PROCESSING */
 
 /* ===========================================================================
  *
