@@ -1,4 +1,4 @@
-/*  $Id: blast_fasta_input.cpp 547030 2017-09-25 17:24:51Z ivanov $
+/*  $Id: blast_fasta_input.cpp 573648 2018-10-30 18:13:44Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -486,7 +486,7 @@ CShortReadFastaInputSource::CShortReadFastaInputSource(CNcbiIstream& infile,
       m_ParseSeqIds(false)
 {
     // allocate sequence buffer
-    m_Sequence.reserve(m_SeqBuffLen + 1);
+    m_Sequence.resize(m_SeqBuffLen + 1);
 
     // read the first line for FASTA input
     if (m_Format == eFasta) {
@@ -523,7 +523,7 @@ CShortReadFastaInputSource::CShortReadFastaInputSource(CNcbiIstream& infile1,
     }
 
     // allocate sequence buffer
-    m_Sequence.reserve(m_SeqBuffLen + 1);
+    m_Sequence.resize(m_SeqBuffLen + 1);
 
     // read the first line for FASTA input
     if (m_Format == eFasta) {
@@ -737,7 +737,8 @@ CShortReadFastaInputSource::x_ReadFastc(CBioseq_set& bioseq_set)
             CBioseq& bioseq = seq_entry->SetSeq();
             bioseq.SetId().clear();
             if (m_ParseSeqIds) {
-                CRef<CSeq_id> seqid(new CSeq_id(id + ".1"));
+                CRef<CSeq_id> seqid(new CSeq_id(id + ".1",
+                                                CSeq_id::fParse_AnyLocal));
                 bioseq.SetId().push_back(seqid);
             }
             else {
@@ -762,7 +763,8 @@ CShortReadFastaInputSource::x_ReadFastc(CBioseq_set& bioseq_set)
             CBioseq& bioseq = seq_entry->SetSeq();
             bioseq.SetId().clear();
             if (m_ParseSeqIds) {
-                CRef<CSeq_id> seqid(new CSeq_id(id + ".2"));
+                CRef<CSeq_id> seqid(new CSeq_id(id + ".2",
+                                                CSeq_id::fParse_AnyLocal));
                 bioseq.SetId().push_back(seqid);
             }
             else {
@@ -809,7 +811,7 @@ CShortReadFastaInputSource::x_ReadFastaOneSeq(CRef<ILineReader> line_reader)
         if (start + line.length() + 1 > m_SeqBuffLen) {
             string tmp;
             m_SeqBuffLen = 2 * (start + line.length() + 1);
-            tmp.reserve(m_SeqBuffLen);
+            tmp.resize(m_SeqBuffLen);
             memcpy(&tmp[0], &m_Sequence[0], start);
             m_Sequence.swap(tmp);
         }
@@ -831,7 +833,8 @@ CShortReadFastaInputSource::x_ReadFastaOneSeq(CRef<ILineReader> line_reader)
         CBioseq& bioseq = seq_entry->SetSeq();
         bioseq.SetId().clear();
         if (m_ParseSeqIds) {
-            CRef<CSeq_id> seqid(new CSeq_id(defline_id));
+            CRef<CSeq_id> seqid(new CSeq_id(defline_id,
+                                            CSeq_id::fParse_AnyLocal));
             bioseq.SetId().push_back(seqid);
             bioseq.SetDescr();
         }
@@ -895,7 +898,8 @@ CShortReadFastaInputSource::x_ReadFastqOneSeq(CRef<ILineReader> line_reader)
         CBioseq& bioseq = seq_entry->SetSeq();
         bioseq.SetId().clear();
         if (m_ParseSeqIds) {
-            CRef<CSeq_id> seqid(new CSeq_id(defline_id));
+            CRef<CSeq_id> seqid(new CSeq_id(defline_id,
+                                            CSeq_id::fParse_AnyLocal));
             bioseq.SetId().push_back(seqid);
             bioseq.SetDescr();
         }
