@@ -1,4 +1,4 @@
-/*  $Id: blastp_args.cpp 500404 2016-05-04 14:59:01Z camacho $
+/*  $Id: blastp_args.cpp 565102 2018-06-06 17:25:02Z rackerst $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -67,7 +67,8 @@ CBlastpAppArgs::CBlastpAppArgs()
     arg.Reset(m_StdCmdLineArgs);
     m_Args.push_back(arg);
 
-    arg.Reset(new CGenericSearchArgs(kQueryIsProtein));
+    arg.Reset(new CGenericSearchArgs(kQueryIsProtein, false, false,
+                                     false, false, true));
     m_Args.push_back(arg);
 
     arg.Reset(new CFilteringArgs(kQueryIsProtein, kFilterByDefault));
