@@ -1,4 +1,4 @@
-/*  $Id: cmdline_flags.cpp 546749 2017-09-21 11:22:02Z ivanov $
+/*  $Id: cmdline_flags.cpp 562577 2018-04-24 15:51:23Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -128,6 +128,8 @@ const double kDfltArgBestHitOverhang = kBestHit_OverhangDflt;
 const string kArgBestHitScoreEdge("best_hit_score_edge");
 const double kDfltArgBestHitScoreEdge = kBestHit_ScoreEdgeDflt;
 
+const string kArgSubjectBestHit("subject_besthit");
+
 const string kArgFrameShiftPenalty("frame_shift_penalty");
 
 const string kArgGapTrigger("gap_trigger");
@@ -199,6 +201,12 @@ const string kArgPercentIdentity("perc_identity");
 const string kArgNoGreedyExtension("no_greedy");
 const string kArgDMBTemplateType("template_type");
 const string kArgDMBTemplateLength("template_length");
+const string kArgNoDiscordant("no_discordant");
+const string kArgFwdRev("fr");
+const string kArgRevFwd("rf");
+const string kArgFwdOnly("f");
+const string kArgRevOnly("r");
+const string kArgOnlyStrandSpecific("only_strand_specific");
 
 const string kArgInputSearchStrategy("import_search_strategy");
 const string kArgOutputSearchStrategy("export_search_strategy");
@@ -228,6 +236,10 @@ const string kArgArchive("archive");
 
 const string kArgQueryCovHspPerc("qcov_hsp_perc");
 const string kArgLineLength("line_length");
+const string kArgTaxIdList("taxids");
+const string kArgTaxIdListFile("taxidlist");
+const string kArgNegativeTaxIdList("negative_taxids");
+const string kArgNegativeTaxIdListFile("negative_taxidlist");
 
 const string kArgPaired("paired");
 const string kArgScore("score");
@@ -244,6 +256,7 @@ const string kArgNoReadIdTrim("no_query_id_trim");
 const string kArgNoUnaligned("no_unaligned");
 const string kArgEnableSraCache("sra_cache");
 const string kArgMaxEditDist("max_edit_dist");
+const string kArgMaxDbWordCount("max_db_word_count");
 
 END_SCOPE(blast)
 END_NCBI_SCOPE
