@@ -1,4 +1,4 @@
-/*  $Id: deltablast_args.cpp 483573 2015-11-02 17:48:31Z boratyng $
+/*  $Id: deltablast_args.cpp 500404 2016-05-04 14:59:01Z camacho $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -30,11 +30,6 @@
 /** @file deltablast_args.cpp
  * Implementation of the DELTA-BLAST command line arguments
  */
-
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = "$Id: deltablast_args.cpp 483573 2015-11-02 17:48:31Z boratyng $";
-#endif
-
 #include <ncbi_pch.hpp>
 #include <algo/blast/blastinput/deltablast_args.hpp>
 #include <algo/blast/api/deltablast_options.hpp>
