@@ -1,4 +1,4 @@
-/* $Id: blast_engine.c 548814 2017-10-18 14:11:31Z ivanov $
+/* $Id: blast_engine.c 574995 2018-11-21 15:39:18Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -79,9 +79,9 @@
 #define CONV_NUCL2PROT_COORDINATES(length) (length) / CODON_LENGTH
 
 NCBI_XBLAST_EXPORT const int   kBlastMajorVersion = 2;
-NCBI_XBLAST_EXPORT const int   kBlastMinorVersion = 7;
+NCBI_XBLAST_EXPORT const int   kBlastMinorVersion = 8;
 NCBI_XBLAST_EXPORT const int   kBlastPatchVersion = 1;
-NCBI_XBLAST_EXPORT const char* kBlastReleaseDate = "October-23-2017";
+NCBI_XBLAST_EXPORT const char* kBlastReleaseDate = "December-17-2018";
 
 /** Structure to be passed to s_BlastSearchEngineCore, containing pointers
     to various preallocated structures and arrays. */
@@ -636,11 +636,11 @@ s_BlastSearchEngineOneContext(EBlastProgramType program_number,
                    coordinates */
                 s_TranslateHSPsToDNAPCoord(program_number, init_hitlist,
                        query_info, subject->frame, orig_length, backup.offset);
-            if (kTranslatedSubject) {
-                prot_length = subject->length;
-                subject->length = orig_length;
+                if (kTranslatedSubject) {
+                    prot_length = subject->length;
+                    subject->length = orig_length;
+                }
             }
-        }
         /** NB: If queries are concatenated, HSP offsets must be adjusted
           * inside the following function call, so coordinates are
           * relative to the individual contexts (i.e. queries, strands or
@@ -675,12 +675,15 @@ s_BlastSearchEngineOneContext(EBlastProgramType program_number,
 
             /* For nucleotide search, if match score is = 2, the odd scores
                are rounded down to the nearest even number. */
+#if 0
             Blast_HSPListAdjustOddBlastnScores(hsp_list, score_options->gapped_calculation, gap_align->sbp);
+#endif
 
         }
 
         Blast_HSPListSortByScore(hsp_list);
 
+
         if (score_options->is_ooframe && kTranslatedSubject)
             subject->length = prot_length;
         } else {
@@ -709,6 +712,13 @@ s_BlastSearchEngineOneContext(EBlastProgramType program_number,
                      overlap, score_options->gapped_calculation,
                      Blast_ProgramIsMapping(program_number));
 
+        if((hit_params->options->hsp_filt_opt != NULL) &&
+           (hit_params->options->hsp_filt_opt->subject_besthit_opts != NULL)) {
+           	Blast_HSPListSubjectBestHit(program_number,
+           								hit_params->options->hsp_filt_opt->subject_besthit_opts,
+           								query_info, combined_hsp_list);
+        }
+
         if (getenv("MAPPER_WRITE_SUBJECT_CHUNK")) {
             s_WriteHSPsForChunk(combined_hsp_list, &backup, hit_params,
                                 hsp_stream, subject->length,
