@@ -1,4 +1,4 @@
-/* $Id: blast_itree.c 358499 2012-04-03 14:48:04Z coulouri $
+/* $Id: blast_itree.c 500404 2016-05-04 14:59:01Z camacho $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,11 +31,6 @@
  * Functions that implement an interval tree for fast HSP containment tests
  */
 
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = 
-    "$Id: blast_itree.c 358499 2012-04-03 14:48:04Z coulouri $";
-#endif /* SKIP_DOXYGEN_PROCESSING */
-
 #include "blast_itree.h"
 #include "blast_gapalign_priv.h"
 #include "blast_hits_priv.h"
