@@ -1,4 +1,4 @@
-/* $Id: greedy_align.c 505944 2016-06-30 12:29:20Z madden $
+/* $Id: greedy_align.c 567051 2018-07-12 13:09:32Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -571,6 +571,9 @@ Int4 BLAST_GreedyAlign(const Uint1* seq1, Int4 len1,
             index = s_FindFirstMismatch(seq1, seq2, len1, len2, 
                                         seq1_index, seq2_index,
                                         fence_hit, reverse, rem);
+            if(fence_hit && *fence_hit){
+            	return 0;
+            }
 
             if (index > longest_match_run) {
                 seed->start_q = seq1_index;
