@@ -901,17 +901,17 @@ CBlastFormat::x_PrintIgTabularReport(const blast::CIgBlastResults& results,
                                 annots->m_ChainType[0], 
                                 annots->m_ChainTypeToShow, 
                                 &m_ScoringMatrix);
-        tabinfo.SetIgAnnotation(annots, m_IgOptions);
+        tabinfo.SetIgAnnotation(annots, m_IgOptions, aln_set, *m_Scope);
         if (fill_clone_info) {
             s_SetCloneInfo(tabinfo, bhandle, clone_info);
         }
         tabinfo.PrintHeader(strProgVersion, *(bhandle.GetBioseqCore()),
-                                m_DbName, 
-                                m_IgOptions->m_DomainSystem,
-                                results.GetRID(), 
-                                numeric_limits<unsigned int>::max(),
-                                aln_set, subject_bioseq);
- 
+                            m_DbName, 
+                            m_IgOptions->m_DomainSystem,
+                            results.GetRID(), 
+                            numeric_limits<unsigned int>::max(),
+                            aln_set, subject_bioseq);
+        
         int j = 1;
         for (; itr != aln_set->Get().end(); ++itr) {
             tabinfo.SetFields(**itr, *m_Scope, 
@@ -933,6 +933,56 @@ CBlastFormat::x_PrintIgTabularReport(const blast::CIgBlastResults& results,
     }
 }
 
+
+void CBlastFormat::x_PrintAirrRearrangement(const blast::CIgBlastResults& results,
+                                            SClone& clone_info,
+                                            bool fill_clone_info,
+                                            bool print_airr_format_header)
+{
+    CConstRef<CSeq_align_set> aln_set = results.GetSeqAlign();
+
+    // other output types will need a bioseq handle
+    CBioseq_Handle bhandle = m_Scope->GetBioseqHandle(*results.GetSeqId(),
+                                                      CScope::eGetBioseq_All);
+
+    // tabular formatting just prints each alignment in turn
+    // (plus a header)
+
+    const CBlastTabularInfo::EFieldDelimiter kDelim =
+            (m_FormatType == CFormattingArgs::eCommaSeparatedValues
+             ? CBlastTabularInfo::eComma : CBlastTabularInfo::eTab);
+
+    CIgBlastTabularInfo tabinfo(m_Outfile, m_CustomOutputFormatSpec, kDelim);
+    tabinfo.SetParseLocalIds(m_BelieveQuery);
+
+    string strProgVersion =
+        "IG" + NStr::ToUpper(m_Program);
+    CConstRef<CBioseq> subject_bioseq = x_CreateSubjectBioseq();
+
+    CRef<CIgAnnotation> annots(null);
+    if (results.HasAlignments()) {
+        annots = results.GetIgAnnotation();
+        tabinfo.SetIgAnnotation(annots, m_IgOptions, aln_set, *m_Scope);
+        if (fill_clone_info) {
+            s_SetCloneInfo(tabinfo, bhandle, clone_info);
+        }
+    }
+    tabinfo.SetAirrFormatData(*m_Scope, annots, 
+                              bhandle, aln_set, m_IgOptions);
+    
+    
+    tabinfo.PrintAirrRearrangement(*m_Scope, annots, strProgVersion,
+                                   *(bhandle.GetBioseqCore()),
+                                   m_DbName, 
+                                   m_IgOptions->m_DomainSystem,
+                                       results.GetRID(), 
+                                       numeric_limits<unsigned int>::max(),
+                                       aln_set, subject_bioseq, &m_ScoringMatrix,
+                                       print_airr_format_header,
+                                       m_IgOptions);
+
+}
+
 CConstRef<objects::CBioseq> CBlastFormat::x_CreateSubjectBioseq()
 {
     if ( !m_IsBl2Seq && !m_IsDbScan) {
@@ -1207,6 +1257,7 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
                                 CConstRef<blast::CBlastQueryVector> queries, 
                                 SClone& clone_info,
                                 bool fill_clone_info,
+                                bool print_airr_format_header,
                                 int index)
 {
     clone_info.na = NcbiEmptyString;
@@ -1263,6 +1314,16 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
         x_PrintIgTabularReport(results, clone_info, fill_clone_info);
         return;
     }
+    
+    if (m_FormatType == CFormattingArgs::eAirrRearrangement) {
+        
+        if (m_Program == "blastn" || m_Program == "BLASTN") {
+            x_PrintAirrRearrangement(results, clone_info, fill_clone_info, print_airr_format_header);
+        } else {
+            m_Outfile << "The AIRR format is only available for nucleotide sequence search" << endl;
+        }
+        return;
+    }
 
     if (m_FormatType == CFormattingArgs::eTaxFormat) {
         string reportCaption = "Tax BLAST report";
@@ -1333,7 +1394,7 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
                                 annots->m_ChainType[0], 
                                 annots->m_ChainTypeToShow, 
                                 &m_ScoringMatrix);
-        tabinfo.SetIgAnnotation(annots, m_IgOptions);
+        tabinfo.SetIgAnnotation(annots, m_IgOptions, aln_set, *m_Scope);
         if (fill_clone_info) {
             s_SetCloneInfo(tabinfo, bhandle, clone_info);
         }
@@ -1457,8 +1518,9 @@ CBlastFormat::PrintOneResultSet(blast::CIgBlastResults& results,
     }
     display.SetSequencePropertyLabel(&chain_type_list);
     // set the alignment flags
+   
     display.SetAlignOption(flags);
-    if (m_Program == "blastn") {
+    if (m_Program == "blastn" || m_Program == "BLASTN") {
         display.SetAlignType(CDisplaySeqalign::eNuc);
     } else {
         display.SetAlignType(CDisplaySeqalign::eProt);
