@@ -57,6 +57,7 @@ static int max_allowed_VD_distance = 55;
 static int extend_length = 30;
 static int max_allowed_V_end_to_J_end =150;
 static int max_v_j_overlap = 7;
+static int j_wordsize = 7;
 
 static void s_ReadLinesFromFile(const string& fn, vector<string>& lines)
 {
@@ -419,7 +420,7 @@ void CIgBlast::x_SetupDJSearch(const vector<CRef <CIgAnnotation> > &annots,
     CBlastOptions & opts = opts_hndl->SetOptions();
     opts.SetMatchReward(1);
     if (db_type == 2){ //J genes are longer so if can afford more reliable identification
-        opts.SetWordSize(7);
+        opts.SetWordSize(j_wordsize);
         opts.SetMismatchPenalty(m_IgOptions->m_J_penalty);
     } else {
         opts.SetWordSize(m_IgOptions->m_Min_D_match);
@@ -461,12 +462,12 @@ void CIgBlast::x_SetupDJSearch(const vector<CRef <CIgAnnotation> > &annots,
               (*annot)->m_GeneInfo[0] - max_allowed_V_end_to_J_end: (*annot)->m_GeneInfo[1] - 1 - v_overlap;
             int end = (ms)? 
               (*annot)->m_GeneInfo[0] + v_overlap: (*annot)->m_GeneInfo[1] + max_allowed_V_end_to_J_end;
-            if (begin > 0) {
+            if (begin > 0 && begin <= len-1) {
                 CRef<CSeqLocInfo> mask(
                   new CSeqLocInfo(new CSeq_interval(*q_id, 0, begin), 0));
                 m_Query->AddMask(iq, mask);
             }
-            if (end < len -1 && end > 0) {
+            if (end < len -1 && end >= 0) {
                 CRef<CSeqLocInfo> mask(
                   new CSeqLocInfo(new CSeq_interval(*q_id, end, len-1), 0));
                 m_Query->AddMask(iq, mask);
@@ -603,6 +604,30 @@ static string s_RemoveLocalPrefix(const string & sid)
     return sid;
 }
 
+static string s_MakeTopHitsId(const CSeq_align_set::Tdata& align_list, int num_align) {
+    string ids = NcbiEmptyString;
+    CRef<CSeq_align> align = align_list.front();
+    int count = 0;
+    ITERATE(CSeq_align_set::Tdata, it, align_list) {
+        
+        if (count < num_align && s_IsSeqAlignAsGood(align, (*it))) {
+            string this_id = s_RemoveLocalPrefix((*it)->GetSeq_id(1).AsFastaString());
+            if (ids.find(this_id) == string::npos) {
+                
+                //no redundant id
+                if (ids != NcbiEmptyString) {
+                    ids += ",";
+                }
+                ids += this_id;
+                count ++;
+            }
+        } else {
+            break;
+        }
+    }
+    return ids;
+}
+
 void CIgBlast::x_AnnotateV(CRef<CSearchResultSet>        &results, 
                            vector<CRef <CIgAnnotation> > &annots)
 {
@@ -616,15 +641,7 @@ void CIgBlast::x_AnnotateV(CRef<CSearchResultSet>        &results,
             CRef<CSeq_align> align = align_list.front();
             annot->m_GeneInfo[0] = align->GetSeqStart(0);
             annot->m_GeneInfo[1] = align->GetSeqStop(0)+1;
-            annot->m_TopGeneIds[0] = "";
-
-            int ii=0;
-            ITERATE(CSeq_align_set::Tdata, it, align_list) {
-                if (ii++ < m_IgOptions->m_NumAlign[0] && s_IsSeqAlignAsGood(align, (*it))) {
-                    if (annot->m_TopGeneIds[0] != "") annot->m_TopGeneIds[0] += ",";
-                    annot->m_TopGeneIds[0] += s_RemoveLocalPrefix((*it)->GetSeq_id(1).AsFastaString());
-                } else break;
-            }
+            annot->m_TopGeneIds[0] = s_MakeTopHitsId(align_list, m_IgOptions->m_NumAlign[0]);
         } 
     }
 };
@@ -876,9 +893,9 @@ void CIgBlast::x_FindDJAln(CRef<CSeq_align_set>& align_D,
                 int q_js = (*it)->GetSeqStart(0);
                 int q_je = (*it)->GetSeqStop(0);
                 if (q_ms) { 
-                    if (q_je < q_ve - allowed_VJ_distance  || q_js > q_ve) keep = false;
+                    if (q_je < q_ve - allowed_VJ_distance  || q_js > q_ve - j_wordsize) keep = false;
                 } else {
-                    if (q_js > q_ve + allowed_VJ_distance || q_je < q_ve) keep = false;
+                    if (q_js > q_ve + allowed_VJ_distance || q_je < q_ve + j_wordsize) keep = false;
                 }
                 /* remove failed seq_align */
                 if (!keep) it = align_list.erase(it);
@@ -1175,15 +1192,7 @@ void CIgBlast::x_AnnotateD(CRef<CSearchResultSet>        &results_D,
             CRef<CSeq_align> align = align_list.front();
             (*annot)->m_GeneInfo[2] = align->GetSeqStart(0);
             (*annot)->m_GeneInfo[3] = align->GetSeqStop(0)+1;
-            (*annot)->m_TopGeneIds[1] = "";
-
-            int ii=0;
-            ITERATE(CSeq_align_set::Tdata, it, align_list) {
-                if (ii++ < m_IgOptions->m_NumAlign[1] && s_IsSeqAlignAsGood(align, (*it))) {
-                    if ((*annot)->m_TopGeneIds[1] != "") (*annot)->m_TopGeneIds[1] += ",";
-                    (*annot)->m_TopGeneIds[1] += s_RemoveLocalPrefix((*it)->GetSeq_id(1).AsFastaString());
-                } else break;
-            }
+            (*annot)->m_TopGeneIds[1] = s_MakeTopHitsId(align_list, m_IgOptions->m_NumAlign[1]);
         }
 
      
@@ -1219,15 +1228,7 @@ void CIgBlast::x_AnnotateJ(CRef<CSearchResultSet>        &results_J,
                                            align->GetSeqStop(0)  + frame_adj 
                                          : align->GetSeqStart(0) - frame_adj;
             } 
-            (*annot)->m_TopGeneIds[2] = "";
-
-            int ii=0;
-            ITERATE(CSeq_align_set::Tdata, it, align_list) {
-                if (ii++ < m_IgOptions->m_NumAlign[2] && s_IsSeqAlignAsGood(align, (*it))) {
-                    if ((*annot)->m_TopGeneIds[2] != "") (*annot)->m_TopGeneIds[2] += ",";
-                    (*annot)->m_TopGeneIds[2] += s_RemoveLocalPrefix((*it)->GetSeq_id(1).AsFastaString());
-                } else break;
-            }
+            (*annot)->m_TopGeneIds[2] = s_MakeTopHitsId(align_list, m_IgOptions->m_NumAlign[2]);
         }
      
         /* next set of results */
@@ -1256,15 +1257,8 @@ void CIgBlast::x_AnnotateDJ(CRef<CSearchResultSet>        &results_D,
             CRef<CSeq_align> align = align_list.front();
             (*annot)->m_GeneInfo[2] = align->GetSeqStart(0);
             (*annot)->m_GeneInfo[3] = align->GetSeqStop(0)+1;
-            (*annot)->m_TopGeneIds[1] = "";
+            (*annot)->m_TopGeneIds[1] = s_MakeTopHitsId(align_list, m_IgOptions->m_NumAlign[1]);
 
-            int ii=0;
-            ITERATE(CSeq_align_set::Tdata, it, align_list) {
-                if (ii++ < m_IgOptions->m_NumAlign[1] && s_IsSeqAlignAsGood(align, (*it))) {
-                    if ((*annot)->m_TopGeneIds[1] != "") (*annot)->m_TopGeneIds[1] += ",";
-                    (*annot)->m_TopGeneIds[1] += s_RemoveLocalPrefix((*it)->GetSeq_id(1).AsFastaString());
-                } else break;
-            }
         }
             
         /* annotate J */    
@@ -1282,15 +1276,7 @@ void CIgBlast::x_AnnotateDJ(CRef<CSearchResultSet>        &results_D,
                                            align->GetSeqStop(0)  + frame_adj 
                                          : align->GetSeqStart(0) - frame_adj;
             } 
-            (*annot)->m_TopGeneIds[2] = "";
-
-            int ii=0;
-            ITERATE(CSeq_align_set::Tdata, it, align_list) {
-                if (ii++ < m_IgOptions->m_NumAlign[2] && s_IsSeqAlignAsGood(align, (*it))) {
-                    if ((*annot)->m_TopGeneIds[2] != "") (*annot)->m_TopGeneIds[2] += ",";
-                    (*annot)->m_TopGeneIds[2] += s_RemoveLocalPrefix((*it)->GetSeq_id(1).AsFastaString());
-                } else break;
-            }
+            (*annot)->m_TopGeneIds[2] = s_MakeTopHitsId(align_list, m_IgOptions->m_NumAlign[2]);
         }
      
         /* next set of results */
