@@ -1,4 +1,4 @@
-/*  $Id: pssmenginefreqratios_unit_test.cpp 464351 2015-04-08 18:18:59Z ivanov $
+/*  $Id: pssmenginefreqratios_unit_test.cpp 485863 2015-11-27 16:39:56Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -144,15 +144,15 @@ BOOST_AUTO_TEST_CASE(FreqRatiosFromMsa)
     string tmp_line;
     getline(raw_data, tmp_line);
     list<string>  ref_data;
-    NStr::Split(tmp_line,  " ", ref_data);
+    NStr::Split(tmp_line,  " ", ref_data, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
     list<string>::iterator ref_pt=ref_data.begin();
     ITERATE(list<double>, pt, freq_ratios) {
     	if (*pt == 0) {
     		continue;
     	}
     	else {
-    		double ref_value = NStr::StringToDouble(*ref_pt);
     		BOOST_REQUIRE(ref_pt != ref_data.end());
+    		double ref_value = NStr::StringToDouble(*ref_pt);
     		BOOST_REQUIRE(fabs(ref_value - *pt) < 0.001);
     		ref_pt ++;
     	}
@@ -160,6 +160,5 @@ BOOST_AUTO_TEST_CASE(FreqRatiosFromMsa)
     }
 
 
-
 }
 BOOST_AUTO_TEST_SUITE_END()
