@@ -1,4 +1,4 @@
-/*  $Id: remote_blast_unit_test.cpp 454692 2014-12-17 15:16:27Z camacho $
+/*  $Id: remote_blast_unit_test.cpp 507726 2016-07-21 14:42:04Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -206,7 +206,7 @@ BOOST_AUTO_TEST_CASE(CheckRemoteRPSBlastOptionsHandle) {
     rmt_blaster.SetDatabase("cdd");
     CRemoteBlast::TSeqLocList query_seqlocs;
     CRef<CSeq_loc> sl(new CSeq_loc);
-    sl->SetWhole().SetGi(GI_FROM(TIntId, 129295));
+    sl->SetWhole().SetGi(GI_CONST(129295));
     query_seqlocs.push_back(sl);
     rmt_blaster.SetQueries(query_seqlocs);
 
