@@ -1,4 +1,4 @@
-/*  $Id: blast_app_util.cpp 509107 2016-08-03 14:09:45Z fongah2 $
+/*  $Id: blast_app_util.cpp 571364 2018-09-25 16:53:32Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -54,6 +54,10 @@
 #include <algo/blast/format/blast_format.hpp>
 #include <objtools/align_format/format_flags.hpp>
 
+#if defined(NCBI_OS_LINUX) && HAVE_MALLOC_H
+#include <malloc.h>
+#endif
+
 BEGIN_NCBI_SCOPE
 USING_SCOPE(objects);
 USING_SCOPE(blast);
@@ -151,7 +155,7 @@ InitializeQueryDataLoaderConfiguration(bool query_is_protein,
         (query_is_protein == db_adapter->IsProtein())) { /* the same database type is used for both queries and subjects */
         // Make sure we don't add the same database more than once
         vector<string> default_dbs;
-        NStr::Split(retval.m_BlastDbName, " ", default_dbs, NStr::fSplit_NoMergeDelims);
+        NStr::Split(retval.m_BlastDbName, " ", default_dbs);
         if (default_dbs.size() &&
             (find(default_dbs.begin(), default_dbs.end(),
                  db_adapter->GetDatabaseName()) == default_dbs.end())) {
@@ -791,7 +795,7 @@ CheckForFreqRatioFile(const string& rps_dbname, CRef<CBlastOptionsHandle>  & opt
     bool use_cbs = (opt_handle->GetOptions().GetCompositionBasedStats() == eNoCompositionBasedStats) ? false : true;
     if(use_cbs) {
         vector<string> db;
-        NStr::Split(rps_dbname, " ", db, NStr::fSplit_NoMergeDelims);
+        NStr::Split(rps_dbname, " ", db);
         list<string> failed_db;
         for (unsigned int i=0; i < db.size(); i++) {
     	    string path;
@@ -942,4 +946,13 @@ void PrintErrorArchive(const CArgs & a, const list<CRef<CBlast4_error> > & msg)
 	} catch (...) {}
 }
 
+void QueryBatchCleanup()
+{
+#if defined(NCBI_OS_LINUX) && HAVE_MALLOC_H
+	malloc_trim(0);
+#endif
+    return;
+
+}
+
 END_NCBI_SCOPE
