@@ -1,4 +1,4 @@
-/*  $Id: blastn_app.cpp 529178 2017-03-01 19:10:55Z fongah2 $
+/*  $Id: blastn_app.cpp 571362 2018-09-25 16:52:58Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -179,7 +179,7 @@ int CBlastnApp::Run(void)
             }
             input.SetBatchSize(mixer.GetBatchSize());
         }
-        for (; !input.End(); formatter.ResetScopeHistory()) {
+        for (; !input.End(); formatter.ResetScopeHistory(), QueryBatchCleanup()) {
 
             CRef<CBlastQueryVector> query_batch(input.GetNextSeqBatch(*scope));
             CRef<IQueryFactory> queries(new CObjMgr_QueryFactory(*query_batch));
