@@ -1,4 +1,4 @@
-/*  $Id: blastp_app.cpp 529178 2017-03-01 19:10:55Z fongah2 $
+/*  $Id: blastp_app.cpp 571365 2018-09-25 16:53:46Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -160,7 +160,7 @@ int CBlastpApp::Run(void)
         formatter.PrintProlog();
 
         /*** Process the input ***/
-        for (; !input.End(); formatter.ResetScopeHistory()) {
+        for (; !input.End(); formatter.ResetScopeHistory(), QueryBatchCleanup()) {
 
             CRef<CBlastQueryVector> query_batch(input.GetNextSeqBatch(*scope));
             CRef<IQueryFactory> queries(new CObjMgr_QueryFactory(*query_batch));
