@@ -1,4 +1,4 @@
-/*  $Id: rpsblast_app.cpp 509107 2016-08-03 14:09:45Z fongah2 $
+/*  $Id: rpsblast_app.cpp 571365 2018-09-25 16:53:46Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -164,7 +164,7 @@ int CRPSBlastApp::Run(void)
         formatter.PrintProlog();
 
         /*** Process the input ***/
-        for (; !input.End(); formatter.ResetScopeHistory()) {
+        for (; !input.End(); formatter.ResetScopeHistory(), QueryBatchCleanup()) {
 
             CRef<CBlastQueryVector> query_batch(input.GetNextSeqBatch(*scope));
             CRef<IQueryFactory> queries(new CObjMgr_QueryFactory(*query_batch));
