@@ -1,4 +1,4 @@
-/*  $Id: blastdbcmd.cpp 536284 2017-05-17 12:39:01Z ivanov $
+/*  $Id: blastdbcmd.cpp 571655 2018-10-01 12:46:33Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,13 +36,14 @@
 #include <corelib/ncbiapp.hpp>
 #include <algo/blast/api/version.hpp>
 #include <objtools/blast/seqdb_reader/seqdbexpert.hpp>
+#include <objtools/blast/seqdb_reader/impl/seqdbtax.hpp>
 #include <algo/blast/api/blast_exception.hpp>
 #include <algo/blast/blastinput/blast_input_aux.hpp>
 #include <objtools/blast/blastdb_format/seq_formatter.hpp>
 #include <objtools/blast/blastdb_format/blastdb_formatter.hpp>
 #include <objtools/blast/blastdb_format/blastdb_seqid.hpp>
-
 #include <algo/blast/blastinput/blast_input.hpp>
+#include <objects/seqloc/PDB_seq_id.hpp>
 #include "../blast/blast_app_util.hpp"
 #include <iomanip>
 
@@ -83,8 +84,11 @@ private:
 
     CBlastDB_FormatterConfig m_Config;
 
-    /// Initializes the application's data members
-    void x_InitApplicationData();
+    set<Int4> m_TaxIdList;
+
+    /// Initializes Blast DB
+    void x_InitBlastDB();
+    void x_InitBlastDB_TaxIdList();
 
     string x_InitSearchRequest();
 
@@ -102,24 +106,59 @@ private:
     /// @return 0 on sucess; 1 if some queries were not processed
     int x_ProcessBatchEntry(CBlastDB_Formatter & seq_fmt);
 
+    int x_ProcessBatchEntry_NoDup(CBlastDB_Formatter & fmt);
+
     /// Process entry with range, strand and filter id
     /// @param args program input args
     /// @param seq_fmt sequence formatter object
     /// @return 0 on sucess; 1 if some queries were not processed
     int x_ProcessEntry(CBlastDB_Formatter & fmt);
 
+    int x_ProcessTaxIdList(CBlastDB_Formatter & fmt);
+
     int x_ProcessSearchType(CBlastDB_Formatter & fmt);
 
     bool x_GetOids(const string & acc, vector<int> & oids);
 
-    int x_ModifyConfigForBatchEntry(const vector<string> & tmp);
+    int x_ModifyConfigForBatchEntry(const string & config);
 
     bool x_UseLongSeqIds();
+
+    void x_PrintBlastDatabaseTaxInformation();
+
 };
 
-bool
-CBlastDBCmdApp::x_GetOids(const string & acc, vector<int> & oids)
+string s_PreProcessAccessionsForDBv5(const string & id)
 {
+	string rv = id;
+	if ((id.find('|') != NPOS) || (id.find('_') != NPOS)) {
+
+		CRef<CSeq_id> seqid;
+		try {
+			seqid = new CSeq_id(id, CSeq_id::fParse_RawText | CSeq_id::fParse_AnyLocal | CSeq_id::fParse_PartialOK);
+		}
+		catch(...) {
+		}
+
+		if(seqid.NotEmpty()) {
+			if(seqid->IsPir() || seqid->IsPrf()) {
+				return seqid->AsFastaString();
+			}
+			rv = seqid->GetSeqIdString(true);
+		}
+	}
+
+	return NStr::ToUpper(rv);
+
+}
+
+bool
+CBlastDBCmdApp::x_GetOids(const string & id, vector<int> & oids)
+{
+	string acc = id;
+	if(m_BlastDb->GetBlastDbVersion() == EBlastDbVersion::eBDB_Version5) {
+		acc = s_PreProcessAccessionsForDBv5(id);
+	}
 	Int8 num_id = NStr::StringToNumeric<Int8>(acc, NStr::fConvErr_NoThrow);
 	if(!errno) {
 		int gi_oid = -1;
@@ -130,7 +169,7 @@ CBlastDBCmdApp::x_GetOids(const string & acc, vector<int> & oids)
 		else {
 			oids.push_back(gi_oid);
 		}
-			
+
 	}
 	else {
 		m_BlastDb->AccessionToOids(acc, oids);
@@ -159,7 +198,7 @@ CBlastDBCmdApp::x_ProcessEntry(CBlastDB_Formatter & fmt)
 
     	vector<string> queries;
        	if (entry.find(kDelim[0]) != string::npos) {
-           	NStr::Split(entry, kDelim, queries, NStr::fSplit_NoMergeDelims);
+           	NStr::Split(entry, kDelim, queries);
        	} else {
        		queries.resize(1);
        		queries[0]  = entry;
@@ -167,18 +206,13 @@ CBlastDBCmdApp::x_ProcessEntry(CBlastDB_Formatter & fmt)
    		for(unsigned int i=0; i < queries.size(); i++) {
      		vector<CSeqDB::TOID> oids;
      		if(x_GetOids(queries[i], oids)) {
-     			if (m_GetDuplicates) {
-     				for(unsigned int j=0; j < oids.size(); j++) {
-     					fmt.Write(oids[j], m_Config);
-     				}
-     			}
-     			else {
-     				if(m_TargetOnly) {
-     					fmt.Write(oids[0], m_Config, queries[i]);
-     				}
-     				else {
-     					fmt.Write(oids[0], m_Config);
-     				}
+   				for(unsigned int j=0; j < oids.size(); j++) {
+   					if(m_TargetOnly) {
+   						fmt.Write(oids[j], m_Config, queries[i]);
+   					}
+   					else {
+   						fmt.Write(oids[j], m_Config);
+   					}
      			}
      		}
      		else {
@@ -206,7 +240,7 @@ bool s_IsMaskAlgoIdValid(CSeqDB & blastdb, int id)
 	return true;
 }
 
-int CBlastDBCmdApp::x_ModifyConfigForBatchEntry(const vector<string> & tmp)
+int CBlastDBCmdApp::x_ModifyConfigForBatchEntry(const string & format)
 {
 	int status = 0;
 	if (!m_DbIsProtein) {
@@ -214,25 +248,140 @@ int CBlastDBCmdApp::x_ModifyConfigForBatchEntry(const vector<string> & tmp)
 	}
    	m_Config.m_SeqRange = TSeqRange::GetEmpty();
    	m_Config.m_FiltAlgoId = -1;
-   	for(unsigned int i=1; i < tmp.size(); i++) {
-   		if(tmp[i].find('-')!= string::npos)
-    	{
-    		try {
-    			m_Config.m_SeqRange = ParseSequenceRangeOpenEnd(tmp[i]);
-    		} catch (...) {
-    		}
-    	}
-    	else if (!m_DbIsProtein && NStr::EqualNocase(tmp[i].c_str(), "minus")) {
-    		m_Config.m_Strand = eNa_strand_minus;
-    	}
-    	else {
-    		m_Config.m_FiltAlgoId = NStr::StringToNonNegativeInt(tmp[i]);
-    		if(!s_IsMaskAlgoIdValid(*m_BlastDb, m_Config.m_FiltAlgoId)){
-    			status = 1;
-    		}
+   	if(!format.empty()) {
+   		vector<string> tmp;
+   		NStr::Split(format, " \t", tmp, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
+   		for(unsigned int i=0; i < tmp.size(); i++) {
+   			if(tmp[i].find('-')!= string::npos) {
+   				try {
+   					m_Config.m_SeqRange = ParseSequenceRangeOpenEnd(tmp[i]);
+   				} catch (...) {
+   				}
+   			}
+   			else if (!m_DbIsProtein && NStr::EqualNocase(tmp[i].c_str(), "minus")) {
+   				m_Config.m_Strand = eNa_strand_minus;
+   			}
+   			else {
+   				m_Config.m_FiltAlgoId = NStr::StringToNonNegativeInt(tmp[i]);
+   				if(!s_IsMaskAlgoIdValid(*m_BlastDb, m_Config.m_FiltAlgoId)){
+   					status = 1;
+   				}
+   			}
+   		}
+   	}
+   	return status;
+}
+
+int
+CBlastDBCmdApp::x_ProcessTaxIdList(CBlastDB_Formatter & fmt)
+{
+    vector<blastdb::TOid> oids;
+    m_BlastDb->TaxIdsToOids(m_TaxIdList, oids);
+    if(oids.size() == 0) {
+		ERR_POST (Error << "No seq found in db for taxonomy list");
+		return 1;
+    }
+    for(unsigned i=0; i < oids.size(); i++) {
+    	fmt.Write(oids[i], m_Config);
+    }
+    return  0;
+}
+
+
+void
+CBlastDBCmdApp::x_InitBlastDB_TaxIdList()
+{
+   	const CArgs& args = GetArgs();
+    vector<string> ids;
+    if(args[kArgTaxIdList].HasValue()) {
+    	string input = args[kArgTaxIdList].AsString();
+    	NStr::Split(input, ",", ids);
+    }
+    else {
+    	CNcbiIstream& input = args[kArgTaxIdListFile].AsInputFile();
+    	while (input) {
+    		string line;
+    	    NcbiGetlineEOL(input, line);
+    	    if ( !line.empty() ) {
+    	       	ids.push_back(line);
+    	    }
     	}
     }
-   	return status;
+    for(unsigned int i=0; i < ids.size(); i++) {
+    	m_TaxIdList.insert(NStr::StringToInt(ids[i], NStr::fAllowLeadingSpaces | NStr::fAllowTrailingSpaces));
+    }
+
+    CSeqDB::ESeqType seqtype = ParseMoleculeTypeString(args[kArgDbType].AsString());
+    m_DbIsProtein = static_cast<bool>(seqtype == CSeqDB::eProtein);
+    m_TargetOnly = args["target_only"];
+   	if(m_TargetOnly) {
+    	CRef<CSeqDBGiList> taxid_list(new CSeqDBGiList());
+    	taxid_list->AddTaxIds(m_TaxIdList);
+   		m_BlastDb.Reset(new CSeqDBExpert(args[kArgDb].AsString(), seqtype, taxid_list.GetPointer()));
+    }
+   	else {
+   		m_BlastDb.Reset(new CSeqDBExpert(args[kArgDb].AsString(), seqtype));
+   	}
+}
+
+
+int
+CBlastDBCmdApp::x_ProcessBatchEntry_NoDup(CBlastDB_Formatter & fmt)
+{
+	int err_found = 0;
+   	const CArgs& args = GetArgs();
+    CNcbiIstream& input = args["entry_batch"].AsInputFile();
+    vector<string> ids, formats;
+    vector<CSeqDB::TOID> oids;
+    while (input) {
+        string line;
+        NcbiGetlineEOL(input, line);
+        if ( !line.empty() ) {
+        	string id, format;
+        	NStr::SplitInTwo(line, " \t", id, format, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
+        	if(id.empty()) {
+        		continue;
+        	}
+        	ids.push_back(id);
+        	formats.push_back(format);
+        }
+    }
+
+    if(m_BlastDb->GetBlastDbVersion() == EBlastDbVersion::eBDB_Version5) {
+    	for(unsigned int i=0; i < ids.size(); i++) {
+    		ids[i] = s_PreProcessAccessionsForDBv5(ids[i]);
+    	}
+    }
+    m_BlastDb->AccessionsToOids(ids, oids);
+    for(unsigned i=0; i < ids.size(); i++) {
+    	if(oids[i] == kSeqDBEntryNotFound) {
+    		Int8 num_id = NStr::StringToNumeric<Int8>(ids[i], NStr::fConvErr_NoThrow);
+    		if(!errno) {
+    			int gi_oid = -1;
+    			m_BlastDb->GiToOidwFilterCheck(num_id, gi_oid);
+    			if(gi_oid >= 0) {
+    				oids[i] = gi_oid;
+    			}
+    		}
+    		if(oids[i] == kSeqDBEntryNotFound) {
+    			err_found ++;
+    			ERR_POST (Error << "Skipped " << ids[i]);
+    			continue;
+    		}
+    	}
+    	if(x_ModifyConfigForBatchEntry(formats[i]))  {
+    		err_found ++;
+    		ERR_POST (Error << "Skipped " << ids[i]);
+    		continue;
+    	}
+    	if(m_TargetOnly) {
+    		fmt.Write(oids[i], m_Config, ids[i]);
+    	}
+    	else {
+    	   	fmt.Write(oids[i], m_Config);
+    	}
+    }
+    return (err_found) ? 1 : 0;
 }
 
 int
@@ -246,20 +395,20 @@ CBlastDBCmdApp::x_ProcessBatchEntry(CBlastDB_Formatter & fmt)
         string line;
         NcbiGetlineEOL(input, line);
         if ( !line.empty() ) {
-        	vector<string> tmp;
-        	NStr::Split(line, " \t", tmp, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
-        	if(tmp.empty()) {
+        	string id, format;
+        	NStr::SplitInTwo(line, " \t", id, format, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
+        	if(id.empty()) {
         		continue;
         	}
-        	if(x_ModifyConfigForBatchEntry(tmp))  {
+        	if(x_ModifyConfigForBatchEntry(format))  {
         		err_found ++;
-        		ERR_POST (Error << "Skipped " << tmp[0]);
+        		ERR_POST (Error << "Skipped " << id);
         		continue;
         	}
    			vector<int> oids;
-         	if(!x_GetOids(tmp[0], oids)) {
+         	if(!x_GetOids(id, oids)) {
          		err_found ++;
-        		ERR_POST (Error << "Skipped " << tmp[0]);
+        		ERR_POST (Error << "Skipped " << id);
         		continue;
          	}
 
@@ -270,7 +419,7 @@ CBlastDBCmdApp::x_ProcessBatchEntry(CBlastDB_Formatter & fmt)
            	}
          	else {
          		if(m_TargetOnly) {
-         			fmt.Write(oids[0], m_Config, tmp[0]);
+         			fmt.Write(oids[0], m_Config, id);
          		}
          		else {
          			fmt.Write(oids[0], m_Config);
@@ -283,7 +432,7 @@ CBlastDBCmdApp::x_ProcessBatchEntry(CBlastDB_Formatter & fmt)
 
 
 void
-CBlastDBCmdApp::x_InitApplicationData()
+CBlastDBCmdApp::x_InitBlastDB()
 {
     const CArgs& args = GetArgs();
 
@@ -298,6 +447,7 @@ CBlastDBCmdApp::x_PrintBlastDatabaseInformation()
     _ASSERT(m_BlastDb.NotEmpty());
     static const NStr::TNumToStringFlags kFlags = NStr::fWithCommas;
     const string kLetters = m_DbIsProtein ? "residues" : "bases";
+    const string kVersion = (m_BlastDb->GetBlastDbVersion() == EBlastDbVersion::eBDB_Version5) ? "5":"4";
     const CArgs& args = GetArgs();
 
     CNcbiOstream& out = args[kArgOutput].AsOutputFile();
@@ -314,7 +464,9 @@ CBlastDBCmdApp::x_PrintBlastDatabaseInformation()
         << "Date: " << m_BlastDb->GetDate()
         << "\tLongest sequence: "
         << NStr::IntToString(m_BlastDb->GetMaxLength(), kFlags) << " "
-        << kLetters << endl;
+        << kLetters << endl << endl;
+
+    out << "BLASTDB Version: " << kVersion << endl;
 
 #if ((!defined(NCBI_COMPILER_WORKSHOP) || (NCBI_COMPILER_VERSION  > 550)) && \
      (!defined(NCBI_COMPILER_MIPSPRO)) )
@@ -331,6 +483,131 @@ CBlastDBCmdApp::x_PrintBlastDatabaseInformation()
     }
 }
 
+class CPrintTaxFields
+{
+private:
+	enum {
+		eTaxID,
+		eSciName,
+		eCommonName,
+		eSuperKingdom,
+		eBlastName,
+		eMaxFields
+	};
+	CNcbiOstream & m_Out;
+	vector<int> m_Fields;
+	vector<string> m_Seperators;
+public:
+	CPrintTaxFields(CNcbiOstream & out, const string & fmt): m_Out(out) {
+		vector<string> fields;
+        string sp = kEmptyStr;
+		if(fmt == "%f") {
+			m_Seperators.push_back(sp);
+			for(unsigned int i=eTaxID; i < eMaxFields; i++){
+				m_Fields.push_back(i);
+				m_Seperators.push_back("\t");
+			}
+			return;
+		}
+
+	    for (unsigned int i = 0; i < fmt.size(); i++) {
+	    	if (fmt[i] == '%') {
+		        if (fmt[i+1] == '%') {
+		            sp += fmt[i];
+		            continue;
+		        }
+		        i++;
+		        switch (fmt[i]) {
+		        case 'T' :
+       				m_Fields.push_back(eTaxID);
+       			break;
+		        case 'S' :
+		        	m_Fields.push_back(eSciName);
+                break;
+		        case 'L' :
+		        	m_Fields.push_back(eCommonName);
+		        break;
+		        case 'K' :
+		        	m_Fields.push_back(eSuperKingdom);
+                break;
+		        case 'B' :
+		        	m_Fields.push_back(eBlastName);
+		        break;
+		        default:
+	                sp += fmt[i-1];
+	                sp += fmt[i];
+	                continue;
+	            break;
+		        }
+	            m_Seperators.push_back(sp);
+	            sp = kEmptyStr;
+	    	}
+	    	else {
+		        sp += fmt[i];
+	    	}
+	    }
+		m_Seperators.push_back(sp);
+
+		if(m_Fields.empty()) {
+			NCBI_THROW(CInputException, eInvalidInput,
+				       "Invalid format options for tax_info.");
+		}
+	}
+
+	void PrintEntry(const SSeqDBTaxInfo & t){
+		for(unsigned int i=0; i < m_Fields.size(); i++) {
+			m_Out << m_Seperators[i];
+			switch (m_Fields[i]){
+				case eTaxID:
+					m_Out << t.taxid;
+				break;
+				case eSciName:
+					m_Out << t.scientific_name;
+				break;
+				case eCommonName:
+					m_Out << t.common_name;
+				break;
+				case eSuperKingdom:
+					m_Out << t.s_kingdom;
+				break;
+				case eBlastName:
+					m_Out << t.blast_name;
+				break;
+				default:
+					NCBI_THROW(CInputException, eInvalidInput,
+					           "Invalid format options for tax_info.");
+				break;
+			}
+		}
+		m_Out << m_Seperators.back();
+		m_Out << "\n";
+	}
+
+};
+
+
+void
+CBlastDBCmdApp::x_PrintBlastDatabaseTaxInformation()
+{
+    _ASSERT(m_BlastDb.NotEmpty());
+    const CArgs& args = GetArgs();
+
+    CNcbiOstream& out = args[kArgOutput].AsOutputFile();
+    const string& kFmt = args["outfmt"].AsString();
+    CPrintTaxFields tf(out, kFmt);
+    set<Int4> tax_ids;
+    m_BlastDb->GetDBTaxIds(tax_ids);
+    // Print basic database information
+    out << "# of Tax IDs in Database: " << tax_ids.size() << endl;
+	SSeqDBTaxInfo info;
+    ITERATE(set<Int4>, itr, tax_ids) {
+    	SSeqDBTaxInfo info;
+   		CSeqDBTaxInfo::GetTaxNames(*itr, info);
+   		tf.PrintEntry(info);
+    }
+}
+
+
 string
 CBlastDBCmdApp::x_InitSearchRequest()
 {
@@ -423,11 +700,20 @@ CBlastDBCmdApp::x_ProcessSearchType(CBlastDB_Formatter & fmt)
 		fmt.DumpAll(m_Config);
 	}
 	else if (args["entry_batch"].HasValue()) {
-		return x_ProcessBatchEntry(fmt);
+		if(m_GetDuplicates) {
+			return x_ProcessBatchEntry(fmt);
+		}
+		else {
+			return x_ProcessBatchEntry_NoDup(fmt);
+		}
 	}
 	else if (args["entry"].HasValue() || args["pig"].HasValue()) {
 		return x_ProcessEntry(fmt);
 	}
+	else if(args[kArgTaxIdList].HasValue()||
+			args[kArgTaxIdListFile].HasValue()) {
+		 return x_ProcessTaxIdList(fmt);
+	}
 	else {
 		NCBI_THROW(CInputException, eInvalidInput,
 		       	   "Must specify query type: one of 'entry', 'entry_batch', or 'pig'");
@@ -527,17 +813,49 @@ void CBlastDBCmdApp::Init()
     arg_desc->SetDependency("pig", CArgDescriptions::eExcludes, "entry_batch");
     arg_desc->SetDependency("pig", CArgDescriptions::eExcludes, "target_only");
 
+    arg_desc->AddOptionalKey(kArgTaxIdList, "taxonomy_ids",
+    						"Comma-delimited taxonomy identifiers", CArgDescriptions::eString);
+    arg_desc->SetDependency(kArgTaxIdList, CArgDescriptions::eExcludes, "entry");
+    arg_desc->SetDependency(kArgTaxIdList, CArgDescriptions::eExcludes, "entry_batch");
+    arg_desc->SetDependency(kArgTaxIdList, CArgDescriptions::eExcludes, "pig");
+
+    arg_desc->AddOptionalKey(kArgTaxIdListFile, "input_file",
+    						"Input file for taxonomy identifiers", CArgDescriptions::eInputFile);
+    arg_desc->SetDependency(kArgTaxIdListFile, CArgDescriptions::eExcludes, "entry");
+    arg_desc->SetDependency(kArgTaxIdListFile, CArgDescriptions::eExcludes, "entry_batch");
+    arg_desc->SetDependency(kArgTaxIdListFile, CArgDescriptions::eExcludes, "pig");
+    arg_desc->SetDependency(kArgTaxIdListFile, CArgDescriptions::eExcludes, kArgTaxIdList);
+
     arg_desc->AddFlag("info", "Print BLAST database information", true);
     // All other options to this program should be here
     const char* exclusions[]  = { "entry", "entry_batch", "outfmt", "strand",
         "target_only", "ctrl_a", "get_dups", "pig", "range",
         "mask_sequence", "list", "remove_redundant_dbs", "recursive",
-        "list_outfmt" };
+        "list_outfmt", kArgTaxIdListFile.c_str(), kArgTaxIdList.c_str()};
     for (size_t i = 0; i < sizeof(exclusions)/sizeof(*exclusions); i++) {
         arg_desc->SetDependency("info", CArgDescriptions::eExcludes,
                                 string(exclusions[i]));
     }
 
+    arg_desc->AddFlag("tax_info",
+    		          "Print taxonomic information contained in this BLAST database.\n"
+    		          "Use -outfmt to customize output. Format specifiers supported are:\n"
+    		          "\t\t%T means taxid\n"
+    		          "\t\t%L means common taxonomic name\n"
+    		          "\t\t%S means scientific name\n"
+    		          "\t\t%K means taxonomic super kingdom\n"
+    		          "\t\t%B means BLAST name\n"
+    		          "By default it prints: '%T %S %L %K %B'\n", true);
+    // All other options to this program should be here
+    const char* tax_info_exclusions[]  = { "info", "entry", "entry_batch", "strand",
+        "target_only", "ctrl_a", "get_dups", "pig", "range",
+        "mask_sequence", "list", "remove_redundant_dbs", "recursive",
+        "list_outfmt", kArgTaxIdListFile.c_str(), kArgTaxIdList.c_str() };
+    for (size_t i = 0; i < sizeof(tax_info_exclusions)/sizeof(*tax_info_exclusions); i++) {
+        arg_desc->SetDependency("tax_info", CArgDescriptions::eExcludes,
+                                string(tax_info_exclusions[i]));
+    }
+
     arg_desc->SetCurrentGroup("Sequence retrieval configuration options");
     arg_desc->AddOptionalKey("range", "numbers",
                          "Range of sequence to extract in 1-based offsets "
@@ -715,15 +1033,26 @@ int CBlastDBCmdApp::Run(void)
             return status;
         }
 
-        x_InitApplicationData();
         if (args["info"]) {
+        	x_InitBlastDB();
             x_PrintBlastDatabaseInformation();
-        } else {
-        	x_InitSearchRequest();
+        }
+        else if (args["tax_info"]) {
+        	x_InitBlastDB();
+            x_PrintBlastDatabaseTaxInformation();
+        }
+        else if(args[kArgTaxIdList].HasValue() ||
+                args[kArgTaxIdListFile].HasValue()) {
+    		x_InitBlastDB_TaxIdList();
+       		status = x_ProcessSearchRequest();
+    	}
+        else {
+        	x_InitBlastDB();
        		status = x_ProcessSearchRequest();
         }
 
     } CATCH_ALL(status)
+
     return status;
 }
 
