@@ -1,4 +1,4 @@
-/*  $Id: makeblastdb.cpp 515237 2016-09-29 12:32:30Z camacho $
+/*  $Id: makeblastdb.cpp 560817 2018-03-27 17:24:45Z rackerst $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -276,6 +276,12 @@ void CMakeBlastDBApp::Init()
                              "Required if multiple file(s)/database(s) are "
                              "provided as input",
                              CArgDescriptions::eString);
+    arg_desc->AddDefaultKey("blastdb_version", "version",
+                             "Version of BLAST database to be created",
+                             CArgDescriptions::eInteger,
+                             NStr::NumericToString(static_cast<int>(eBDB_Version4)));
+    arg_desc->SetConstraint("blastdb_version",
+                            new CArgAllow_Integers(eBDB_Version4, eBDB_Version5));
     arg_desc->AddDefaultKey("max_file_sz", "number_of_bytes",
                             "Maximum file size for BLAST database files",
                             CArgDescriptions::eString, "1GB");
@@ -609,7 +615,9 @@ CRawSeqDBSource::CRawSeqDBSource(const string & name, bool protein, CBuildDataba
         objects::EBlast_filter_program algo;
         string algo_opts, algo_name;
         m_Source->GetMaskAlgorithmDetails(*algo_id, algo, algo_name, algo_opts);
-        algo_name += NStr::IntToString(*algo_id);
+        if (algo != EBlast_filter_program::eBlast_filter_program_other) {
+            algo_name += NStr::IntToString(*algo_id);
+        }
         m_MaskIdMap[*algo_id] = outdb->RegisterMaskingAlgorithm(algo, algo_opts, algo_name);
     }
     // Process columns
@@ -811,16 +819,14 @@ void CMakeBlastDBApp::x_ProcessMaskData()
     vector<string> gi_mask_names;
 
     if (! files.HasValue()) return;
-    NStr::Split(NStr::TruncateSpaces(files.AsString()), ",", file_list,
-                   NStr::eNoMergeDelims);
+    NStr::Split(NStr::TruncateSpaces(files.AsString()), ",", file_list);
     if (! file_list.size()) {
         NCBI_THROW(CInvalidDataException, eInvalidInput,
                 "mask_data option found, but no files were specified.");
     }
 
     if (ids.HasValue()) {
-        NStr::Split(NStr::TruncateSpaces(ids.AsString()), ",", id_list,
-                   NStr::eNoMergeDelims);
+        NStr::Split(NStr::TruncateSpaces(ids.AsString()), ",", id_list);
         if (file_list.size() != id_list.size()) {
             NCBI_THROW(CInvalidDataException, eInvalidInput,
                 "the size of mask_id does not match that of mask_data.");
@@ -836,8 +842,7 @@ void CMakeBlastDBApp::x_ProcessMaskData()
     }
 
     if (descs.HasValue()) {
-        NStr::Split(NStr::TruncateSpaces(descs.AsString()), ",", desc_list,
-                   NStr::eNoMergeDelims);
+        NStr::Split(NStr::TruncateSpaces(descs.AsString()), ",", desc_list);
         if (file_list.size() != desc_list.size()) {
             NCBI_THROW(CInvalidDataException, eInvalidInput,
                 "the size of mask_desc does not match that of mask_data.");
@@ -849,8 +854,7 @@ void CMakeBlastDBApp::x_ProcessMaskData()
     }
 
     if (gi_names.HasValue()) {
-        NStr::Split(NStr::TruncateSpaces(gi_names.AsString()), ",", gi_mask_names,
-                   NStr::eNoMergeDelims);
+        NStr::Split(NStr::TruncateSpaces(gi_names.AsString()), ",", gi_mask_names);
         if (file_list.size() != gi_mask_names.size()) {
             NCBI_THROW(CInvalidDataException, eInvalidInput,
                 "the size of gi_mask_name does not match that of mask_data.");
@@ -946,6 +950,7 @@ void CMakeBlastDBApp::x_ProcessInputData(const string & paths,
     vector<CTempString> names;
     SeqDB_SplitQuoted(paths, names);
     vector<string> blastdb;
+    TIdToLeafs leafTaxIds;
 
     ESupportedInputFormats input_fmt = x_GetUserInputTypeHint();
     CMakeBlastDBApp::TFormat build_fmt = x_ConvertToCFormatGuessType(input_fmt);
@@ -982,7 +987,6 @@ void CMakeBlastDBApp::x_ProcessInputData(const string & paths,
             }
             final_blastdb.push_back(blastdb[0]);
         } else {
-
             ITERATE(vector<string>, iter, blastdb) {
                 const string & s = *iter;
 
@@ -1000,7 +1004,31 @@ void CMakeBlastDBApp::x_ProcessInputData(const string & paths,
         if (final_blastdb.size()) {
             string quoted;
             SeqDB_CombineAndQuote(final_blastdb, quoted);
-            CRef<IRawSequenceSource> raw(new CRawSeqDBSource(quoted, is_protein, m_DB));
+
+            CRef<CSeqDB> indb(new CSeqDB(
+                    quoted,
+                    is_protein ? CSeqDB::eProtein : CSeqDB::eNucleotide
+            ));
+            const int numoids = indb->GetNumOIDs();
+            for (int oid = 0; oid < numoids; ++oid) {
+                CRef<CBlast_def_line_set> hdr = indb->GetHdr(oid);
+                ITERATE(CBlast_def_line_set::Tdata, itr, hdr->Get()) {
+                    CRef<CBlast_def_line> bdl = *itr;
+                    CBlast_def_line::TTaxIds leafs = bdl->GetLeafTaxIds();
+                    if (!leafs.empty()) {
+                        const string id =
+                                bdl->GetSeqid().front()->AsFastaString();
+                        set<int> ids = leafTaxIds[id];
+                        ids.insert(leafs.begin(), leafs.end());
+                        leafTaxIds[id] = ids;
+                    }
+                }
+            }
+
+            m_DB->SetLeafTaxIds(leafTaxIds, true);
+            CRef<IRawSequenceSource> raw(
+                    new CRawSeqDBSource(quoted, is_protein, m_DB)
+            );
             m_DB->AddSequences(*raw);
         } else {
             NCBI_THROW(CInvalidDataException, eInvalidInput,
@@ -1035,7 +1063,7 @@ void CMakeBlastDBApp::x_BuildDatabase()
     }
 
     vector<string> input_files;
-    NStr::Split(dbname, kInputSeparators, input_files, NStr::fSplit_NoMergeDelims);
+    NStr::Split(dbname, kInputSeparators, input_files);
     if (dbname == "-" || input_files.size() > 1) {
         NCBI_THROW(CInvalidDataException, eInvalidInput,
             "Please provide a database name using -" + kOutput);
@@ -1091,13 +1119,17 @@ void CMakeBlastDBApp::x_BuildDatabase()
         long_seqids = (registry.Get("BLAST", "LONG_SEQID") == "1");
     }
 
+    const EBlastDbVersion dbver =
+        static_cast<EBlastDbVersion>(args["blastdb_version"].AsInteger());
+
     m_DB.Reset(new CBuildDatabase(dbname,
                                   title,
                                   is_protein,
                                   indexing,
                                   use_gi_mask,
                                   m_LogFile,
-                                  long_seqids));
+                                  long_seqids,
+                                  dbver));
 
 #if _BLAST_DEBUG
     if (args["verbose"]) {
@@ -1123,9 +1155,10 @@ void CMakeBlastDBApp::x_BuildDatabase()
     // Max file size
 
     Uint8 bytes = NStr::StringToUInt8_DataSize(args["max_file_sz"].AsString());
-    if (bytes >= (1L << 31)) {
+    static const Uint8 MAX_VOL_FILE_SIZE = 0x100000000;
+    if (bytes >= MAX_VOL_FILE_SIZE) {
         NCBI_THROW(CInvalidDataException, eInvalidInput,
-                "max_file_sz must be < 2 GiB");
+                "max_file_sz must be < 4 GiB");
     }
     *m_LogFile << "Maximum file size: "
                << Uint8ToString_DataSize(bytes) << endl;
