@@ -1,4 +1,4 @@
-/* $Id: mac_prj_generator.cpp 485908 2015-11-30 14:28:08Z gouriano $
+/* $Id: mac_prj_generator.cpp 570296 2018-09-06 16:51:11Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -386,7 +386,7 @@ void CMacProjectGenerator::Save(const string& solution_name, CPlist& xproj)
         solution_file += ".candidate";
     }
     {
-        auto_ptr<CObjectOStream> out(CObjectOStream::Open(solution_file, eSerial_Xml));
+        unique_ptr<CObjectOStream> out(CObjectOStream::Open(solution_file, eSerial_Xml));
         CObjectOStreamXml *ox = dynamic_cast<CObjectOStreamXml*>(out.get());
         ox->SetReferenceDTD(true);
         ox->SetDTDPublicId("-//Apple//DTD PLIST 1.0//EN");
@@ -638,18 +638,28 @@ string CMacProjectGenerator::CreateProjectScriptPhase(
         ITERATE ( list<string>, f, prj_files.GetDataSpecs()) {
             CDirEntry entry(*f);
             string spec_base( CDirEntry(GetRelativePath(*f)).GetDir() + entry.GetBase());
+            string spec_ext = entry.GetExt();
             AddString( *inputs, GetRelativePath(*f));
-            AddString( *inputs, spec_base + ".def");
-            AddString( *outputs, spec_base + ".files");
-            AddString( *outputs, spec_base + "__.cpp");
-            AddString( *outputs, spec_base + "___.cpp");
+            if (spec_ext == ".proto") {
+                AddString( *outputs, spec_base + ".pb.cc");
+                AddString( *outputs, spec_base + ".grpc.pb.cc");
+            } else {
+                AddString( *inputs, spec_base + ".def");
+                AddString( *outputs, spec_base + ".files");
+                AddString( *outputs, spec_base + "__.cpp");
+                AddString( *outputs, spec_base + "___.cpp");
+            }
 #if 0
-            script += "echo Using datatool to create a C++ objects from ASN/DTD/Schema " + entry.GetName() + "\n";
+            script += "echo Generating C++ classes from " + entry.GetName() + "\n";
             script += m_OutputDir + GetApp().GetDatatoolPathForApp();
 #else
             if (dataspec_first) {
                 script += "export PTB_PLATFORM=\"$ARCHS\"\n";
-                script += "export DATATOOL_PATH=" + m_OutputDir + "../static/bin/ReleaseDLL\n";
+                if (spec_ext == ".proto") {
+                    script += "export GENERATOR_PATH=" + GetApp().GetSite().GetConfigureEntry("XCode_CustomCodeGenerator" + spec_ext) + "\n";
+                } else {
+                    script += "export DATATOOL_PATH=" + m_OutputDir + "../static/bin/ReleaseDLL\n";
+                }
                 script += "export TREE_ROOT=" +
                     CDirEntry::DeleteTrailingPathSeparator( GetRelativePath( GetApp().m_Root)) + "\n";
                 script += "export BUILD_TREE_ROOT=" +
@@ -659,7 +669,11 @@ string CMacProjectGenerator::CreateProjectScriptPhase(
                         GetApp().GetRegSettings().m_CompilersSubdir)))) + "\n";
                 dataspec_first = false;
             }
-            script +=  "\"$BUILD_TREE_ROOT/datatool.sh\"";
+            if (spec_ext == ".proto") {
+                script +=  "\"$BUILD_TREE_ROOT/protoc.sh\"";
+            } else {
+                script +=  "\"$BUILD_TREE_ROOT/datatool.sh\"";
+            }
 #endif
             script += " " + GetApp().GetDatatoolCommandLine() + " -pch " + pch_name;
             script += " -m " + GetRelativePath( entry.GetPath(), &GetApp().GetProjectTreeInfo().m_Src);
@@ -1464,6 +1478,8 @@ string CMacProjectGenerator::AddFile(CDict& dict, const string& name, bool style
         } else {
             filetype = string("sourcecode") + ext + ext;
         }
+    } else if (ext == ".cc" || ext == ".cxx") {
+        filetype = "sourcecode.cpp.cpp";
     } else if (ext == ".hpp" || ext == ".inl") {
         filetype = "sourcecode.cpp.h";
     } else if (ext == ".h") {
@@ -1472,6 +1488,8 @@ string CMacProjectGenerator::AddFile(CDict& dict, const string& name, bool style
         filetype = "text.xml";
     } else if (ext == ".wsdl") {
         filetype = "text.xml";
+    } else if (ext == ".jsd") {
+        filetype = "text.json";
     } else {
         filetype = "text";
     }
