@@ -535,7 +535,7 @@ void CMsvcProjectGenerator::Generate(CProjItem& prj)
     }}
     
     // Insert sources, headers, inlines:
-    auto_ptr<IFilesToProjectInserter> inserter;
+    unique_ptr<IFilesToProjectInserter> inserter;
 
     if (prj.m_ProjType == CProjKey::eDll) {
         inserter.reset(new CDllProjectFilesInserter
@@ -683,6 +683,8 @@ void s_CreateDatatoolCustomBuildInfo(const CProjItem&              prj,
     //SourceFile
     build_info->m_SourceFile = 
         CDirEntry::ConcatPath(src.m_SourceBaseDir, src.m_SourceFile);
+    string ext;
+    CDirEntry::SplitPath(build_info->m_SourceFile, nullptr, nullptr, &ext);
 
     CMsvc7RegSettings::EMsvcVersion eVer = CMsvc7RegSettings::GetMsvcVersion();
     if (eVer > CMsvc7RegSettings::eMsvc710 &&
@@ -734,24 +736,34 @@ void s_CreateDatatoolCustomBuildInfo(const CProjItem&              prj,
             }
         }
 
-        build_info->m_CommandLine  =  "set DATATOOL_PATH=" + dt_path + "\n";
+        if (ext == ".proto") {
+            dt_path = GetApp().GetSite().GetConfigureEntry("CustomCodeGenerator" + ext);
+            build_info->m_CommandLine  =  "set GENERATOR_PATH=" + dt_path + "\n";
+        } else {
+            build_info->m_CommandLine  =  "set DATATOOL_PATH=" + dt_path + "\n";
+        }
         build_info->m_CommandLine +=  "set TREE_ROOT=" + tree_root + "\n";
         build_info->m_CommandLine +=  "set PTB_PLATFORM=$(PlatformName)\n";
         build_info->m_CommandLine +=  "set BUILD_TREE_ROOT=" + build_root + "\n";
-        build_info->m_CommandLine +=  "call \"%BUILD_TREE_ROOT%\\datatool.bat\" " + tool_cmd + "\n";
+        if (ext == ".proto") {
+            build_info->m_CommandLine +=  "call \"%BUILD_TREE_ROOT%\\protoc.bat\" " + tool_cmd + "\n";
+        } else {
+            build_info->m_CommandLine +=  "call \"%BUILD_TREE_ROOT%\\datatool.bat\" " + tool_cmd + "\n";
+        }
         build_info->m_CommandLine +=  "if errorlevel 1 exit 1";
-        string tool_exe_location("\"");
-        tool_exe_location += dt_path + "datatool.exe" + "\"";
 
         //Description
         build_info->m_Description = 
-            "Using datatool to generate C++ classes from ASN/DTD/Schema $(InputPath)";
+            "Generating C++ classes from $(InputPath)";
 
         //Outputs
-        build_info->m_Outputs = "$(InputDir)$(InputName).files;$(InputDir)$(InputName)__.cpp;$(InputDir)$(InputName)___.cpp";
+        if (ext == ".proto") {
+            build_info->m_Outputs = "$(InputDir)$(InputName).pb.cc;$(InputDir)$(InputName).grpc.pb.cc";
+        } else {
+            build_info->m_Outputs = "$(InputDir)$(InputName).files;$(InputDir)$(InputName)__.cpp;$(InputDir)$(InputName)___.cpp";
+            build_info->m_AdditionalDependencies = "$(InputDir)$(InputName).def;";
+        }
 
-        //Additional Dependencies
-        build_info->m_AdditionalDependencies = "$(InputDir)$(InputName).def;";
     } else {
         //exe location - path is supposed to be relative encoded
         string tool_exe_location("\"");
@@ -949,7 +961,11 @@ void CMsvcProjectGenerator::GeneratePropertySheets()
         {
             //default is very simple and empty
             msbuild::CProject project;
-            project.SetAttlist().SetToolsVersion("4.0");
+            string tmp = GetApp().GetMetaMakefile().GetConfigurationOpt( "ToolsVersion", SConfigInfo());
+            if (tmp.empty()) {
+                tmp = "4.0";
+            }
+            project.SetAttlist().SetToolsVersion(tmp);
 
             // NCBI base prop sheet
             {
@@ -1012,7 +1028,11 @@ void CMsvcProjectGenerator::GeneratePropertySheets()
     {
         //default is very simple and empty
         msbuild::CProject project;
-        project.SetAttlist().SetToolsVersion("4.0");
+        string tmp = GetApp().GetMetaMakefile().GetConfigurationOpt( "ToolsVersion", SConfigInfo());
+        if (tmp.empty()) {
+            tmp = "4.0";
+        }
+        project.SetAttlist().SetToolsVersion(tmp);
 
         // NCBI base prop sheet
         {
@@ -1078,7 +1098,11 @@ void CMsvcProjectGenerator::GeneratePropertySheets()
     string file_base(CDirEntry::ConcatPath(ps_dir, ps_base));
     {
         msbuild::CProject project;
-        project.SetAttlist().SetToolsVersion("4.0");
+        string tmp = GetApp().GetMetaMakefile().GetConfigurationOpt( "ToolsVersion", SConfigInfo());
+        if (tmp.empty()) {
+            tmp = "4.0";
+        }
+        project.SetAttlist().SetToolsVersion(tmp);
 
         // NCBI base prop sheet
         {
@@ -1194,7 +1218,12 @@ void CMsvcProjectGenerator::GenerateMsbuild(
 {
     msbuild::CProject project;
     project.SetAttlist().SetDefaultTargets("Build");
-    project.SetAttlist().SetToolsVersion("4.0");
+    string tmp;
+    tmp = GetApp().GetMetaMakefile().GetConfigurationOpt( "ToolsVersion", SConfigInfo());
+    if (tmp.empty()) {
+        tmp = "4.0";
+    }
+    project.SetAttlist().SetToolsVersion(tmp);
 #if __USE_DISABLED_CFGS__
     const list<SConfigInfo>& all_cfgs = GetApp().GetRegSettings().m_ConfigInfo;
 #else
@@ -1214,6 +1243,14 @@ void CMsvcProjectGenerator::GenerateMsbuild(
         } else {
             __SET_PROPGROUP_ELEMENT( t, "ProjectName", project_context.ProjectName());
         }
+        tmp = GetApp().GetMetaMakefile().GetConfigurationOpt( "VCProjectVersion", SConfigInfo());
+        if (!tmp.empty()) {
+            __SET_PROPGROUP_ELEMENT( t, "VCProjectVersion", tmp);
+        }
+        tmp = GetApp().GetMetaMakefile().GetConfigurationOpt( "WindowsTargetPlatformVersion", SConfigInfo());
+        if (!tmp.empty()) {
+            __SET_PROPGROUP_ELEMENT( t, "WindowsTargetPlatformVersion", tmp);
+        }
     }
     {
         // project configurations
@@ -1900,7 +1937,11 @@ void CMsvcProjectGenerator::GenerateMsbuildFilters(
     CMsvcPrjProjectContext& project_context, CProjItem& prj)
 {
     msbuild::CProject filters;
-    filters.SetAttlist().SetToolsVersion("4.0");
+    string tmp = GetApp().GetMetaMakefile().GetConfigurationOpt( "ToolsVersion", SConfigInfo());
+    if (tmp.empty()) {
+        tmp = "4.0";
+    }
+    filters.SetAttlist().SetToolsVersion(tmp);
     string project_dir(project_context.ProjectDir());
 #if __USE_DISABLED_CFGS__
     const list<SConfigInfo>& all_cfgs = GetApp().GetRegSettings().m_ConfigInfo;
@@ -1915,7 +1956,7 @@ void CMsvcProjectGenerator::GenerateMsbuildFilters(
     if (!collector.SourceFiles().empty()) {
         string tag_name("ClCompile");
         string filter_name("Source Files");
-        CMsbuildFileFilter filter( filters, filter_list, "cpp;c;cxx",
+        CMsbuildFileFilter filter( filters, filter_list, "cpp;c;cxx;cc",
             tag_name, filter_name, project_dir, collector, prj);
         ITERATE(list<string>, f, collector.SourceFiles()) {
             const string& rel_source_file = *f;
@@ -2059,7 +2100,11 @@ void CMsvcProjectGenerator::GenerateMsbuildUser(
     const list<SConfigInfo>& all_cfgs = m_project_configs;
 #endif
     msbuild::CProject project;
-    project.SetAttlist().SetToolsVersion("4.0");
+    string tmp = GetApp().GetMetaMakefile().GetConfigurationOpt( "ToolsVersion", SConfigInfo());
+    if (tmp.empty()) {
+        tmp = "4.0";
+    }
+    project.SetAttlist().SetToolsVersion(tmp);
 
     ITERATE(list<SConfigInfo>, c , all_cfgs) {
         const SConfigInfo& cfg_info = *c;
