@@ -1,4 +1,4 @@
-/* $Id: msvc_prj_utils.cpp 493103 2016-02-24 13:52:12Z gouriano $
+/* $Id: msvc_prj_utils.cpp 568727 2018-08-09 18:33:41Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -50,13 +50,13 @@ BEGIN_NCBI_SCOPE
 
 CVisualStudioProject* LoadFromXmlFile(const string& file_path)
 {
-    auto_ptr<CObjectIStream> in(CObjectIStream::Open(eSerial_Xml, 
+    unique_ptr<CObjectIStream> in(CObjectIStream::Open(eSerial_Xml, 
                                                     file_path, 
                                                     eSerial_StdWhenAny));
     if ( in->fail() )
         NCBI_THROW(CProjBulderAppException, eFileOpen, file_path);
     
-    auto_ptr<CVisualStudioProject> prj(new CVisualStudioProject());
+    unique_ptr<CVisualStudioProject> prj(new CVisualStudioProject());
     in->Read(prj.get(), prj->GetThisTypeInfo());
     return prj.release();
 }
@@ -423,9 +423,11 @@ string SourceFileExt(const string& file_path)
     if (explicit_c  &&  CFile(file_path).Exists()) {
         return ".c";
     }
-    bool explicit_cpp = NStr::CompareNocase(ext, ".cpp")== 0;
+    bool explicit_cpp = NStr::CompareNocase(ext, ".cpp")== 0
+                    ||  NStr::CompareNocase(ext, ".cxx")== 0
+                    ||  NStr::CompareNocase(ext, ".cc")== 0;
     if (explicit_cpp  &&  CFile(file_path).Exists()) {
-        return ".cpp";
+        return ext;
     }
     string file = file_path + ".cpp";
     if ( CFile(file).Exists() ) {
@@ -695,8 +697,11 @@ void CMsvc7RegSettings::IdentifyPlatform()
             sm_MsvcVersion = eMsvc1400;
             sm_MsvcVersionName = NStr::NumericToString(ide);
             break;
+        case 1500:
         default:
-            NCBI_THROW(CProjBulderAppException, eBuildConfiguration, "Unsupported IDE version");
+            sm_MsvcVersion = eMsvc1500;
+            sm_MsvcVersionName = NStr::NumericToString(ide);
+//            NCBI_THROW(CProjBulderAppException, eBuildConfiguration, "Unsupported IDE version");
             break;
         }
     }    
@@ -783,7 +788,7 @@ string CMsvc7RegSettings::GetSolutionFileFormatVersion(void)
         return "12.00\n# Visual Studio 2012";
     } else if (GetMsvcVersion() == eMsvc1200) {
         return "12.00\n# Visual Studio 2013";
-    } else if (GetMsvcVersion() == eMsvc1400) {
+    } else if (GetMsvcVersion() >= eMsvc1400) {
         return GetApp().GetRegSettings().m_Version;
     }
     return "";
@@ -1685,6 +1690,7 @@ IsProducedByDatatool(const string&    src_path_abs,
     string dtd_name = asn_base + ".dtd";
     string xsd_name = asn_base + ".xsd";
     string wsdl_name = asn_base + ".wsdl";
+    string jsd_name = asn_base + ".jsd";
 
     //try to find this name in datatool generated sources container
     ITERATE(list<CDataToolGeneratedSrc>, p, project.m_DatatoolSources) {
@@ -1692,7 +1698,9 @@ IsProducedByDatatool(const string&    src_path_abs,
         if ((asn.m_SourceFile == asn_name) ||
             (asn.m_SourceFile == dtd_name) ||
             (asn.m_SourceFile == xsd_name) ||
-            (asn.m_SourceFile == wsdl_name)) {
+            (asn.m_SourceFile == wsdl_name) ||
+            (asn.m_SourceFile == jsd_name)
+            ) {
             return &(*p);
         }
     }
