@@ -1,4 +1,4 @@
-/* $Id: proj_tree.cpp 524100 2017-01-09 19:49:55Z gouriano $
+/* $Id: proj_tree.cpp 568727 2018-08-09 18:33:41Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -187,7 +187,9 @@ void CProjectItemsTree::CreateFrom(const string& root_src,
                 else if (info.m_Type == SMakeProjectT::SMakeInInfo::eASN ||
                          info.m_Type == SMakeProjectT::SMakeInInfo::eDTD ||
                          info.m_Type == SMakeProjectT::SMakeInInfo::eXSD ||
-                         info.m_Type == SMakeProjectT::SMakeInInfo::eWSDL) {
+                         info.m_Type == SMakeProjectT::SMakeInInfo::eWSDL ||
+                         info.m_Type == SMakeProjectT::SMakeInInfo::eJSD ||
+                         info.m_Type == SMakeProjectT::SMakeInInfo::eProtobuf) {
 
                     SAsnProjectT::DoCreate(source_base_dir, 
                                            proj_name, 
