@@ -1,7 +1,7 @@
 #ifndef PROJECT_TREE_BUILDER__PROJ_TREE_BUILDER__HPP
 #define PROJECT_TREE_BUILDER__PROJ_TREE_BUILDER__HPP
 
-/* $Id: proj_tree_builder.hpp 426461 2014-02-10 18:07:55Z gouriano $
+/* $Id: proj_tree_builder.hpp 568727 2018-08-09 18:33:41Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -66,6 +66,8 @@ struct SMakeProjectT
             eDTD,
             eXSD,
             eWSDL,
+            eJSD,
+            eProtobuf,
             eMsvc
         } TMakeinType;
 
