@@ -1,7 +1,7 @@
 #ifndef PROJECT_TREE_BUILDER__RESOLVER__HPP
 #define PROJECT_TREE_BUILDER__RESOLVER__HPP
 
-/* $Id: resolver.hpp 281406 2011-05-04 12:17:05Z gouriano $
+/* $Id: resolver.hpp 426194 2014-02-06 17:22:06Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -44,6 +44,34 @@
 BEGIN_NCBI_SCOPE
 
 
+class CSymResolver;
+class CExpansionRule
+{
+public:
+    enum EExpRule {
+        eNoop,
+        eReplace,
+        ePattern
+    };
+
+    CExpansionRule(void);
+    CExpansionRule(const string& textrule);
+    CExpansionRule(const CExpansionRule& other);
+    CExpansionRule& operator= (const CExpansionRule& other);
+
+    void Reset();
+    void Init(const string& textrule,
+        CSymResolver* resolver=NULL,  const CSimpleMakeFileContents* data=NULL);
+
+    string ApplyRule( const string& value) const;
+    void  ApplyRule( list<string>& value) const;
+
+private:
+    EExpRule m_Rule;
+    string m_Lvalue;
+    string m_Rvalue;
+};
+
 class CSymResolver
 {
 public:
@@ -69,9 +97,13 @@ public:
 
     bool IsEmpty(void) const;
 
+    static bool   StripSuffix(string& libname, string* suffix=nullptr);
     static bool   IsDefine   (const string& param);
     static bool   HasDefine   (const string& param);
     static string StripDefine(const string& define);
+    static string TrimDefine(const string& define);
+    string FilterDefine(const string& define, CExpansionRule& rule,
+                        const CSimpleMakeFileContents& data);
 
 private:
     void Clear(void);
