@@ -1,4 +1,4 @@
-/* $Id: ncbi_conn_stream.cpp 547023 2017-09-25 17:18:32Z ivanov $
+/* $Id: ncbi_conn_stream.cpp 569142 2018-08-16 16:45:35Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -61,10 +61,15 @@ CConn_IOStream::CConn_IOStream(const TConn_Pair& connpair,
                                 timeout, buf_size, flgs, ptr, size));
     CONN conn = csb->GetCONN();
     if (conn) {
-        SOCK s/*dummy*/;
-        // NB: CONN_Write(0 bytes) could have caused the same effect as GetSOCK
-        (void) CONN_GetSOCK(conn, &s);  // Prompt CONN to actually open
-        if (CONN_Status(conn, eIO_Open) == eIO_Success) {
+        EIO_Status status;
+        if (!(flgs & fConn_DelayOpen)) {
+            SOCK s/*dummy*/;
+            // NB: CONN_Write(0 bytes) could have caused the same effect
+            (void) CONN_GetSOCK(conn, &s);  // Prompt CONN to actually open
+            status  = CONN_Status(conn, eIO_Open);
+        } else
+            status  = eIO_Success;
+        if (status == eIO_Success) {
             init(csb.get());
             m_CSb = csb.release();
             return;
@@ -81,12 +86,18 @@ CConn_IOStream::CConn_IOStream(CONN conn, bool close,
     : CNcbiIostream(0), m_CSb(0)
 {
     auto_ptr<CConn_Streambuf>
-        csb(new CConn_Streambuf(conn, close, timeout, buf_size, flgs, ptr, size));
+        csb(new CConn_Streambuf(conn, close,
+                                timeout, buf_size, flgs, ptr, size));
     if (conn) {
-        SOCK s/*dummy*/;
-        // NB: CONN_Write(0 bytes) could have caused the same effect as GetSOCK
-        (void) CONN_GetSOCK(conn, &s);  // Prompt CONN to actually open
-        if (CONN_Status(conn, eIO_Open) == eIO_Success) {
+        EIO_Status status;
+        if (!(flgs & fConn_DelayOpen)) {
+            SOCK s/*dummy*/;
+            // NB: CONN_Write(0 bytes) could have caused the same effect
+            (void) CONN_GetSOCK(conn, &s);  // Prompt CONN to actually open
+            status  = CONN_Status(conn, eIO_Open);
+        } else
+            status  = eIO_Success;
+        if (status == eIO_Success) {
             init(csb.get());
             m_CSb = csb.release();
             return;
@@ -152,6 +163,11 @@ EIO_Status CConn_IOStream::Status(EIO_Event dir) const
 }
 
 
+EIO_Status CConn_IOStream::Pushback(const CT_CHAR_TYPE* data, streamsize size)
+{
+    return m_CSb ? m_CSb->Pushback(data, size) : eIO_NotSupported;
+}
+
 EIO_Status CConn_IOStream::Close(void)
 {
     return m_CSb ? m_CSb->Close() : eIO_Closed;
@@ -295,7 +311,7 @@ s_SocketConnectorBuilder(const SConnNetInfo* net_info,
             init.size = size;
             init.cred = net_info->credentials;
             SOCK s;
-            status  = SOCK_CreateOnTopInternal(sock, 0, &s, &init, flgs);
+            status = SOCK_CreateOnTopInternal(sock, 0, &s, &init, flgs);
             _ASSERT(!s ^ !(status != eIO_Success));
             SOCK_Destroy(sock);
             sock = s;
@@ -411,7 +427,7 @@ struct Deleter<SConnNetInfo>
 
 static CConn_IOStream::TConn_Pair
 s_HttpConnectorBuilder(const SConnNetInfo* net_info,
-                       TReqMethod          method,
+                       EReqMethod          method,
                        const char*         url,
                        const char*         host,
                        unsigned short      port,
@@ -769,7 +785,8 @@ CConn_ServiceStream::CConn_ServiceStream(const string&         service,
                                                extra  &&  extra->get_next_info
                                                ? x_GetNextInfo : 0,
                                                timeout),
-                     timeout, buf_size)
+                     timeout, buf_size,
+                     types & fSERV_DelayOpen ? fConn_DelayOpen : 0)
 {
     return;
 }
@@ -797,7 +814,8 @@ CConn_ServiceStream::CConn_ServiceStream(const string&         service,
                                                extra  &&  extra->get_next_info
                                                ? x_GetNextInfo : 0,
                                                timeout),
-                     timeout, buf_size)
+                     timeout, buf_size,
+                     types & fSERV_DelayOpen ? fConn_DelayOpen : 0)
 {
     return;
 }
@@ -891,8 +909,7 @@ CConn_MemoryStream::CConn_MemoryStream(BUF        buf,
                                                          ? 1/*true*/
                                                          : 0/*false*/),
                                 eIO_Unknown),
-                     0, buf_size,
-                     fConn_ReadBuffered | fConn_WriteBuffered,
+                     0, buf_size, 0,
                      0, BUF_Size(buf)),
       m_Ptr(0)
 {
@@ -905,8 +922,7 @@ CConn_MemoryStream::CConn_MemoryStream(const void* ptr,
                                        EOwnership  owner,
                                        size_t      buf_size)
     : CConn_IOStream(TConn_Pair(MEMORY_CreateConnector(), eIO_Unknown),
-                     0, buf_size,
-                     fConn_ReadBuffered | fConn_WriteBuffered,
+                     0, buf_size, 0,
                      (CT_CHAR_TYPE*) ptr, size),
       m_Ptr(owner == eTakeOwnership ? ptr : 0)
 {
@@ -1051,8 +1067,8 @@ s_FtpConnectorBuilder(const SConnNetInfo*  net_info,
  
 
 /* For data integrity and unambigous interpretation, FTP streams are not
- * buffered at the level of C++ STL streambuf because of the way they execute
- * read / write operations on the mix of FTP commands and data.
+ * buffered at the level of the C++ STL streambuf because of the way they
+ * execute read / write operations on the mix of FTP commands and data.
  * There should be a little impact on performance of byte-by-byte I/O (such as
  * formatted input, which is not expected very often for this kind of streams,
  * anyways), and almost none for block I/O (such as read / readsome / write).
@@ -1075,7 +1091,7 @@ CConn_FtpStream::CConn_FtpStream(const string&        host,
                                                           cmcb),
                                 eIO_Unknown),
                      timeout, buf_size,
-                     fConn_Untie | fConn_ReadBuffered)
+                     fConn_Untie | fConn_WriteUnbuffered)
 {
     return;
 }
@@ -1091,7 +1107,7 @@ CConn_FtpStream::CConn_FtpStream(const SConnNetInfo&  net_info,
                                            cmcb,
                                            timeout),
                      timeout, buf_size,
-                     fConn_Untie | fConn_ReadBuffered)
+                     fConn_Untie | fConn_WriteUnbuffered)
 {
     return;
 }
@@ -1167,7 +1183,7 @@ CConn_FTPDownloadStream::CConn_FTPDownloadStream(const SConnNetInfo&  net_info,
 void CConn_FTPDownloadStream::x_InitDownload(const string& file, Uint8 offset)
 {
     // Use '\n' here instead of NcbiFlush to avoid (and thus make silent)
-    // flush errors on retrieval of inexistent (or bad) files / directories..
+    // flush errors on retrieval of nonexistent (or bad) files / directories...
     EIO_Status status;
     if (offset) {
         write("REST ", 5) << NStr::UInt8ToString(offset) << '\n';
@@ -1245,7 +1261,7 @@ public:
 };
 
 
-static bool s_IsIdentifier(const string& str)
+static bool x_IsIdentifier(const string& str)
 {
     const char* s = str.c_str();
     if (!isalpha((unsigned char)(*s)))
@@ -1267,7 +1283,7 @@ CConn_IOStream* NcbiOpenURL(const string& url, size_t buf_size)
         {
         } conn_initer;  /*NCBI_FAKE_WARNING*/
     }
-    bool svc = s_IsIdentifier(url);
+    bool svc = x_IsIdentifier(url);
 
     AutoPtr<SConnNetInfo> net_info
         (ConnNetInfo_Create(svc ? url.c_str() : 0));
@@ -1338,11 +1354,11 @@ const char* CIO_Exception::GetErrCodeString(void) const
 {
     switch (GetErrCode()) {
     case eTimeout:       return "eIO_Timeout";
-    case eClosed:        return "eIO_Closed";
     case eInterrupt:     return "eIO_Interrupt";
     case eInvalidArg:    return "eIO_InvalidArg";
     case eNotSupported:  return "eIO_NotSupported";
     case eUnknown:       return "eIO_Unknown";
+    case eClosed:        return "eIO_Closed";
     default:             break;
     }
     return CException::GetErrCodeString();
