@@ -1,4 +1,4 @@
-/* $Id: ncbi_core_cxx.cpp 532418 2017-04-05 03:37:11Z lavr $
+/* $Id: ncbi_core_cxx.cpp 565391 2018-06-11 18:09:29Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,6 +36,7 @@
 #include <ncbi_pch.hpp>
 #include "ncbi_ansi_ext.h"
 #include "ncbi_priv.h"
+#include "ncbi_socketp.h"
 #include <corelib/ncbiapp.hpp>
 #include <corelib/request_ctx.hpp>
 #include <connect/error_codes.hpp>
@@ -412,7 +413,7 @@ static void s_SetMonkeyHooks(EMonkeyHookSwitch hook_switch)
 #endif //NCBI_MONKEY
 
 
-static enum EConnectInit {
+static volatile enum EConnectInit {
     eConnectInit_Weak     = -1,  ///< CConn_Initer
     eConnectInit_Intact   =  0,  ///< Not yet visited
     eConnectInit_Strong   =  1,  ///< User init detected
@@ -460,7 +461,7 @@ static void s_Init(const IRWRegistry* reg  = 0,
 
     TCORE_Set set = 0;
     if (!(g_CORE_Set & eCORE_SetLOCK)) {
-        CORE_SetLOCK(MT_LOCK_cxx2c(lock, flag & eConnectInit_OwnLock));
+        CORE_SetLOCK(MT_LOCK_cxx2c(lock, !!(flag & eConnectInit_OwnLock)));
         set |= eCORE_SetLOCK;
     }
     if (!(g_CORE_Set & eCORE_SetLOG)) {
@@ -468,18 +469,19 @@ static void s_Init(const IRWRegistry* reg  = 0,
         set |= eCORE_SetLOG;
     }
     if (!(g_CORE_Set & eCORE_SetREG)) {
-        CORE_SetREG(REG_cxx2c(reg, flag & eConnectInit_OwnRegistry));
+        CORE_SetREG(REG_cxx2c(reg, !!(flag & eConnectInit_OwnRegistry)));
         set |= eCORE_SetREG;
     }
     if (!(g_CORE_Set & eCORE_SetSSL)) {
-        SOCK_SetupSSL(ssl);
+        SOCK_SetupSSLInternal(ssl, 1/*init*/);
         set |= ssl ? eCORE_SetSSL : 0;
     }
     g_CORE_Set &= ~set;
     s_CORE_Set |=  set;
 
     if (s_ConnectInit == eConnectInit_Intact) {
-        g_NCBI_ConnectRandomSeed  = (int) time(0) ^ NCBI_CONNECT_SRAND_ADDEND;
+        g_NCBI_ConnectRandomSeed
+            = (unsigned int) time(0) ^ NCBI_CONNECT_SRAND_ADDEND;
         srand(g_NCBI_ConnectRandomSeed);
         atexit(s_Fini);
     }
