@@ -1,4 +1,4 @@
-/* $Id: ncbi_heapmgr.c 527595 2017-02-14 18:42:16Z lavr $
+/* $Id: ncbi_heapmgr.c 565397 2018-06-11 19:08:08Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -157,8 +157,9 @@ typedef struct HEAP_PACKED {
 struct SHEAP_tag {
     SHEAP_HeapBlock* base;    /* Base of heap extent:         !base == !size */
     TNCBI_Size       size;    /* # blocks in the heap extent: !base == !size */
+    TNCBI_Size       used;    /* # of blocks used (size - used = free blocks)*/
     TNCBI_Size       free;    /* Index of the largest free block (OOB=none)  */
-    TNCBI_Size       last;    /* Index of the last block (RW heap only)      */
+    TNCBI_Size       last;    /* Index of the last block (RW heap, else OOB) */
     TNCBI_Size       chunk;   /* Aligned (bytes);  0 when the heap is RO     */
     FHEAP_Resize     resize;  /* != NULL when resizeable (RW heap only)      */
     void*            auxarg;  /* Auxiliary argument to pass to "resize"      */
@@ -171,20 +172,19 @@ static int/*bool*/ s_HEAP_fast = 1/*true*/;
 
 
 #define _HEAP_ALIGN_EX(a, b)  ((((unsigned long)(a) + ((b) - 1)) / (b)) * (b))
-#define _HEAP_ALIGN(a, b)     (( (unsigned long)(a) + ((b) - 1)) & ~((b) - 1))
+#define _HEAP_ALIGN_2(a, b)   (( (unsigned long)(a) + ((b) - 1)) \
+                               & (unsigned long)(~((b) - 1)))
 #define _HEAP_SIZESHIFT       4
 #define HEAP_BLOCKS(s)        ((s) >> _HEAP_SIZESHIFT)
 #define HEAP_EXTENT(b)        ((b) << _HEAP_SIZESHIFT)
-#define HEAP_ALIGN(a)         _HEAP_ALIGN(a, HEAP_EXTENT(1))
+#define HEAP_ALIGN(a)         _HEAP_ALIGN_2(a, HEAP_EXTENT(1))
 #define HEAP_MASK             (~(HEAP_EXTENT(1) - 1))
-#define _HEAP_OLDLAST         0x80000000UL
 #define HEAP_PREV_BIT         8
 #define HEAP_NEXT_BIT         4
-#define HEAP_LAST             (2 | _HEAP_OLDLAST)
+#define HEAP_LAST             2
 #define HEAP_USED             1
-#define HEAP_FREE             0
-#define HEAP_SIZE(s)          ((s) &  HEAP_MASK)
-#define HEAP_FLAG(s)          ((s) & ~HEAP_MASK)
+#define HEAP_SIZE(s)          ((s) & (unsigned long)  HEAP_MASK)
+#define HEAP_FLAG(s)          ((s) & (unsigned long)(~HEAP_MASK))
 #define HEAP_NEXT(b)          ((SHEAP_HeapBlock*)                       \
                                ((char*)(b) +           (b)->head.size))
 #define HEAP_PREV(b)          ((SHEAP_HeapBlock*)                       \
@@ -193,11 +193,16 @@ static int/*bool*/ s_HEAP_fast = 1/*true*/;
 #define HEAP_ISLAST(b)        ((b)->head.flag & HEAP_LAST)
 #define HEAP_ISUSED(b)        ((b)->head.flag & HEAP_USED)
 
+#define HEAP_CHECK(heap)                                            \
+    assert(!heap->base == !heap->size);                             \
+    assert(heap->used <= heap->size);                               \
+    assert(heap->free <= heap->size);                               \
+    assert(heap->last <= heap->size);                               \
+    assert(heap->used == heap->size  ||  heap->free < heap->size)
 
-#if HEAP_MASK != ~((HEAP_PREV_BIT | HEAP_NEXT_BIT |                     \
-                    HEAP_FREE | HEAP_USED | HEAP_LAST) & ~_HEAP_OLDLAST) \
+#if ~HEAP_MASK != (HEAP_PREV_BIT | HEAP_NEXT_BIT |  HEAP_LAST | HEAP_USED) \
     ||  HEAP_BLOCKS(~HEAP_MASK) != 0
-#  error "HEAP_MASK invalid"
+#  error "HEAP_MASK is invalid!"
 #endif
 
 
@@ -205,20 +210,17 @@ static int/*bool*/ s_HEAP_fast = 1/*true*/;
  * 1. flag is to keep byte-size of the previous block (instead of the magic);
  * 2. since sizes always have last nibble zero, use that in the flag field as
  *    the following:
- *    bit 0 -- set for used, unset for free block;
- *    bit 1 -- set for last block in the heap.
+ *    bit 0 -- set for used, unset for a free block;
+ *    bit 1 -- set for the last block in the heap.
  * 3. bits 2 & 3 can be used as a parity checks for each size to compensate
  *    for discontinued use of the magic value (at least for used blocks).
  * With this scheme, block freeing will no longer need a lookup.
  * Yet keeping current size clean will still allow fast forward moves.
- * Remove:
- *   HEAP_FREE
- *   _HEAP_OLDLAST
- * Change:
- *   HEAP_LAST -> 2
- *   Use HEAP_ISLAST() return value instead of the ternary op in assgn: faster
  */
 
+#ifdef __GNUC__
+inline
+#endif /*__GNUC__*/
 static int/*bool*/ x_Parity(unsigned int v)
 {
 #if 0
@@ -227,10 +229,10 @@ static int/*bool*/ x_Parity(unsigned int v)
     v  = (v & 0x11111111U) * 0x11111111U;
     return (v >> 28) & 1;
 #else
-    v ^= v >> 16;
-    v ^= v >> 8;
-    v ^= v >> 4;
-    v &= 0xF;
+    v ^=  v >> 16;
+    v ^=  v >> 8;
+    v ^=  v >> 4;
+    v &=  0xF;
     return (0x6996 >> v) & 1;
 #endif
 }
@@ -254,14 +256,14 @@ static unsigned int x_PrevBit(TNCBI_Size size)
 }
 
 
-HEAP HEAP_Create(void*      base,  TNCBI_Size   size,
-                 TNCBI_Size chunk, FHEAP_Resize resize, void* auxarg)
+extern HEAP HEAP_Create(void*      base,  TNCBI_Size   size,
+                        TNCBI_Size chunk, FHEAP_Resize resize, void* auxarg)
 {
     HEAP heap;
 
     assert(HEAP_EXTENT(1) == sizeof(SHEAP_HeapBlock));
     assert(_HEAP_ALIGN_EX(1, sizeof(SHEAP_Block)) ==
-           _HEAP_ALIGN   (1, sizeof(SHEAP_Block)));
+           _HEAP_ALIGN_2 (1, sizeof(SHEAP_Block)));
 
     if (!base != !size)
         return 0;
@@ -276,23 +278,24 @@ HEAP HEAP_Create(void*      base,  TNCBI_Size   size,
         return 0;
     heap->base   = (SHEAP_HeapBlock*) base;
     heap->size   = HEAP_BLOCKS(size);
+    heap->used   = 0;
     heap->free   = 0;
     heap->last   = 0;
-    heap->chunk  = chunk        ? HEAP_ALIGN(chunk) : 0;
-    heap->resize = heap->chunk  ? resize            : 0;
-    heap->auxarg = heap->resize ? auxarg            : 0;
+    heap->chunk  = chunk        ? (TNCBI_Size) HEAP_ALIGN(chunk) : 0;
+    heap->resize = heap->chunk  ? resize                         : 0;
+    heap->auxarg = heap->resize ? auxarg                         : 0;
     heap->refcnt = 0/*original*/;
     heap->serial = 0;
     if (base) {
         SHEAP_HeapBlock* b = heap->base;
         /* Reformat the pre-allocated heap */
-        if (_HEAP_ALIGN(base, sizeof(SHEAP_Block)) != (unsigned long) base) {
+        if (_HEAP_ALIGN_2(base, sizeof(SHEAP_Block)) != (unsigned long) base) {
             CORE_LOGF_X(2, eLOG_Warning,
                         ("Heap Create: Unaligned base (0x%08lX)",
                          (long) base));
         }
-        b->head.flag = HEAP_FREE | HEAP_LAST;
-        b->head.size = HEAP_SIZE(size);
+        b->head.flag = HEAP_LAST;
+        b->head.size = (TNCBI_Size) HEAP_SIZE(size);
         b->nextfree  = 0;
         b->prevfree  = 0;
     }
@@ -300,22 +303,23 @@ HEAP HEAP_Create(void*      base,  TNCBI_Size   size,
 }
 
 
-HEAP HEAP_AttachFast(const void* base, TNCBI_Size size, int serial)
+extern HEAP HEAP_AttachFast(const void* base, TNCBI_Size size, int serial)
 {
     HEAP heap;
 
     assert(HEAP_EXTENT(1) == sizeof(SHEAP_HeapBlock));
     assert(_HEAP_ALIGN_EX(1, sizeof(SHEAP_Block)) ==
-           _HEAP_ALIGN   (1, sizeof(SHEAP_Block)));
+           _HEAP_ALIGN_2 (1, sizeof(SHEAP_Block)));
 
     if (!base != !size  ||  !(heap = (HEAP) calloc(1, sizeof(*heap))))
         return 0;
-    if (_HEAP_ALIGN(base, sizeof(SHEAP_Block)) != (unsigned long) base) {
+    if (_HEAP_ALIGN_2(base, sizeof(SHEAP_Block)) != (unsigned long) base) {
         CORE_LOGF_X(3, eLOG_Warning,
                     ("Heap Attach: Unaligned base (0x%08lX)", (long) base));
     }
     heap->base   = (SHEAP_HeapBlock*) base;
     heap->size   = HEAP_BLOCKS(size);
+    heap->used   = heap->size;
     heap->free   = heap->size;
     heap->last   = heap->size;
     heap->serial = serial;
@@ -329,7 +333,7 @@ HEAP HEAP_AttachFast(const void* base, TNCBI_Size size, int serial)
 }
 
 
-HEAP HEAP_Attach(const void* base, TNCBI_Size maxsize, int serial)
+extern HEAP HEAP_Attach(const void* base, TNCBI_Size maxsize, int serial)
 {
     TNCBI_Size size = 0;
 
@@ -387,23 +391,21 @@ static const char* s_HEAP_Id(char* buf, HEAP h)
 
 /* Scan the heap and return a free block, whose size is the minimal to fit the
  * requested one (i.e. its previous free block, if any, is smaller).  The block
- * returned is still linked into the list of free blocks.  Iff no suitable free
- * block can be found, *size gets updated with the total free space available.
+ * returned is still linked into the list of free blocks.  "hint" (if provided)
+ * is from where to start searching downto the start of the free block chain.
  */
 static SHEAP_HeapBlock* s_HEAP_Find(HEAP             heap,
-                                    TNCBI_Size*      size,
+                                    TNCBI_Size       need,
                                     SHEAP_HeapBlock* hint)
 {
     SHEAP_HeapBlock *f, *b, *e = heap->base + heap->free;
-    TNCBI_Size find = size ? *size : 0;
 
     assert(!hint  ||  !HEAP_ISUSED(hint));
     assert(heap->free < heap->size  &&  !HEAP_ISUSED(e));
-    if (!hint  &&  find < (e->head.size >> 1)) {
+    if (!hint  &&  need < (e->head.size >> 1)) {
         /* begin from the smallest block */
         for (b = heap->base + e->nextfree;  ; b = heap->base + b->nextfree) {
-            f = b;
-            if (!s_HEAP_fast) {
+            if (unlikely(!s_HEAP_fast)) {
                 if (b < heap->base  ||  heap->base + heap->size <= b) {
                     b = 0;
                     goto err;
@@ -411,15 +413,18 @@ static SHEAP_HeapBlock* s_HEAP_Find(HEAP             heap,
                 if (HEAP_ISUSED(b))
                     goto err;
             }
-            if (find <= b->head.size  ||  b == e)
+            if (need <= b->head.size) {
+                f = b;
                 break;
+            }
+            assert(b != e);
         }
         assert(f);
     } else {
-        TNCBI_Size free = (b = hint ? hint : e)->head.size;
-        f = free < find ? 0 : b;  /* a bigger free block */
+        b = hint ? hint : e;
+        f = b->head.size < need ? 0 : b; /*a bigger free block*/
         for (b = heap->base + b->prevfree;  ; b = heap->base + b->prevfree) {
-            if (!s_HEAP_fast) {
+            if (unlikely(!s_HEAP_fast)) {
                 if (b < heap->base  ||  heap->base + heap->size <= b) {
                     b = 0;
                     goto err;
@@ -427,14 +432,7 @@ static SHEAP_HeapBlock* s_HEAP_Find(HEAP             heap,
                 if (HEAP_ISUSED(b))
                     goto err;
             }
-            if (!f) {
-                if (b != e  &&  (free += b->head.size) < find)
-                    continue;
-                assert(size);
-                *size = free;
-                break;
-            }
-            if (b == e  ||  b->head.size < find)
+            if (b == e  ||  b->head.size < need)
                 break;
             f = b;
         }
@@ -453,8 +451,6 @@ static SHEAP_HeapBlock* s_HEAP_Find(HEAP             heap,
                      s_HEAP_Id(_id, heap), HEAP_INDEX(b, heap->base),
                      heap->size, msg));
     }}
-    if (size)
-        *size = 0;
     return 0;
 }
 
@@ -484,10 +480,9 @@ static void s_HEAP_Link(HEAP heap, SHEAP_HeapBlock* f, SHEAP_HeapBlock* hint)
         b->nextfree = free;
         heap->free  = free;
     } else {
-        TNCBI_Size size = f->head.size;
-        /* find a block "b" that is just bigger than "f" */
-        assert(!hint  ||  size <= hint->head.size);
-        b = s_HEAP_Find(heap, &size, hint);
+        /* Find a block "b" that is just bigger than "f" */
+        assert(!hint  ||  f->head.size <= hint->head.size);
+        b = s_HEAP_Find(heap, f->head.size, hint);
         assert(b  &&  !HEAP_ISUSED(b));
         /* Link in BEFORE b (so that f <= b) */
         f->nextfree = HEAP_INDEX(b, heap->base);
@@ -508,7 +503,7 @@ static void s_HEAP_Unlink(SHEAP_HeapBlock* b)
     b->prevfree = b->nextfree = ~((TNCBI_Size) 0);
 }
 #else
-#define s_HEAP_Unlink(b)  ((void) 0)
+#  define s_HEAP_Unlink(b)  /*void*/
 #endif /*_DEBUG && !NDEBUG*/
 
 
@@ -532,7 +527,7 @@ static SHEAP_HeapBlock* s_HEAP_Collect(HEAP heap, TNCBI_Size need)
         SHEAP_HeapBlock* n = b == e ? 0 : HEAP_NEXT(b);
         assert(!n  ||  HEAP_SIZE(b->head.size) == b->head.size);
         if (n)
-            last = HEAP_ISLAST(b) ? HEAP_LAST : 0;
+            last = HEAP_ISLAST(b);
         if (!n  ||  !HEAP_ISUSED(b)) {
             if (n) {
                 assert(!need  ||  b->head.size < need);
@@ -568,9 +563,9 @@ static SHEAP_HeapBlock* s_HEAP_Collect(HEAP heap, TNCBI_Size need)
                     memmove(f, u, HEAP_EXTENT(used));
                     assert(p);
                     p -= size;
-                    p->head.flag &= ~HEAP_LAST;
+                    p->head.flag &= (unsigned int)(~HEAP_LAST);
                     f += used;
-                    f->head.flag = HEAP_FREE | last;
+                    f->head.flag = last;
                     f->head.size = free;
                     s_HEAP_Unlink(f);
                     if (last)
@@ -614,12 +609,12 @@ static SHEAP_Block* s_HEAP_Take(HEAP heap,
     assert(HEAP_SIZE(size) == size);
     assert(!HEAP_ISUSED(b)  &&  size <= b->head.size);
     if (size + HEAP_EXTENT(1) <= b->head.size) {
-        /* the block is to used partially */
-        unsigned int last = HEAP_ISLAST(b) ? HEAP_LAST : 0;
+        /* the block is to be split */
+        unsigned int last = HEAP_ISLAST(b);
         SHEAP_HeapBlock* f;
         if (tail) {
             f = b;
-            f->head.flag &= ~HEAP_LAST;
+            f->head.flag &= (unsigned int)(~HEAP_LAST);
             f->head.size -= size;
             b = HEAP_NEXT(f);
             b->head.flag  = HEAP_USED | last;
@@ -640,15 +635,16 @@ static SHEAP_Block* s_HEAP_Take(HEAP heap,
         s_HEAP_Link(heap, f, n);
     } else
         b->head.flag |= HEAP_USED;
+    heap->used += HEAP_BLOCKS(size);
     if (size -= need)
         memset((char*) b + need, 0, size); /* block padding (if any) zeroed */
     return &b->head;
 }
 
 
-SHEAP_Block* HEAP_Alloc(HEAP        heap,
-                        TNCBI_Size  size,
-                        int/*bool*/ tail)
+extern SHEAP_Block* HEAP_Alloc(HEAP        heap,
+                               TNCBI_Size  size,
+                               int/*bool*/ tail)
 {
     SHEAP_HeapBlock *f, *n;
     TNCBI_Size need;
@@ -658,7 +654,7 @@ SHEAP_Block* HEAP_Alloc(HEAP        heap,
         CORE_LOG_X(6, eLOG_Warning, "Heap Alloc: NULL heap");
         return 0;
     }
-    assert(!heap->base == !heap->size);
+    HEAP_CHECK(heap);
 
     if (unlikely(!heap->chunk)) {
         CORE_LOGF_X(7, eLOG_Error,
@@ -669,11 +665,11 @@ SHEAP_Block* HEAP_Alloc(HEAP        heap,
         return 0;
 
     size += (TNCBI_Size) sizeof(SHEAP_Block);
-    need  = HEAP_ALIGN(size);
+    need  = (TNCBI_Size) HEAP_ALIGN(size);
 
-    if (likely(heap->free < heap->size)) {
-        TNCBI_Size free = need;
-        if (likely((f = s_HEAP_Find(heap, &free, 0)) != 0)) {
+    if (need <= HEAP_EXTENT(heap->size - heap->used)) {
+        assert(heap->free < heap->size);
+        if (likely((f = s_HEAP_Find(heap, need, 0)) != 0)) {
             /*NB: f is still linked -- unlink*/
             n = heap->base + f->nextfree;
             if (n == f) {
@@ -690,24 +686,24 @@ SHEAP_Block* HEAP_Alloc(HEAP        heap,
                 }
             }
             s_HEAP_Unlink(f);
-        } else if (unlikely(need <= free)) {
-            /*NB: here f returns unlinked*/
+        } else {
+            /*NB: here f is returned unlinked*/
             f = s_HEAP_Collect(heap, need);
             assert(f  &&  !HEAP_ISUSED(f)  &&  need <= f->head.size);
             if (unlikely(f->head.flag & HEAP_LAST))
-                f->head.flag = HEAP_FREE | HEAP_LAST;
+                f->head.flag = HEAP_LAST;
             n = 0;
-        } else
-            f = n = 0;
+        }
     } else
         f = n = 0;
     if (unlikely(!f)) {
-        TNCBI_Size dsize = HEAP_EXTENT(heap->size);
-        TNCBI_Size hsize = _HEAP_ALIGN_EX(dsize + need, heap->chunk);
+        TNCBI_Size dsize = (TNCBI_Size) HEAP_EXTENT(heap->size);
+        TNCBI_Size hsize = (TNCBI_Size) _HEAP_ALIGN_EX(dsize + need,
+                                                       heap->chunk);
         SHEAP_HeapBlock* base = (SHEAP_HeapBlock*)
             heap->resize(heap->base, hsize, heap->auxarg);
         if (unlikely
-            (_HEAP_ALIGN(base, sizeof(SHEAP_Block)) != (unsigned long) base)) {
+            (_HEAP_ALIGN_2(base,sizeof(SHEAP_Block)) != (unsigned long) base)){
             CORE_LOGF_X(9, eLOG_Warning,
                         ("Heap Alloc%s: Unaligned base (0x%08lX)",
                          s_HEAP_Id(_id, heap), (long) base));
@@ -719,7 +715,7 @@ SHEAP_Block* HEAP_Alloc(HEAP        heap,
         f = base + heap->last;
         if (unlikely(!heap->base)) {
             assert(!heap->last  &&  !heap->free);
-            f->head.flag = HEAP_FREE | HEAP_LAST;
+            f->head.flag = HEAP_LAST;
             f->head.size = hsize;
             hsize = HEAP_BLOCKS(hsize);
             heap->free = hsize;
@@ -727,11 +723,11 @@ SHEAP_Block* HEAP_Alloc(HEAP        heap,
             assert(base <= f  &&  f < base + heap->size  &&  HEAP_ISLAST(f));
             hsize = HEAP_BLOCKS(hsize);
             if (unlikely(HEAP_ISUSED(f))) {
-                f->head.flag &= ~HEAP_LAST;
+                f->head.flag &= (unsigned int)(~HEAP_LAST);
                 /* New block is at the very top of the heap */
                 heap->last = heap->size;
                 f = base + heap->size;
-                f->head.flag  = HEAP_FREE | HEAP_LAST;
+                f->head.flag  = HEAP_LAST;
                 f->head.size  = dsize;
                 if (heap->free == heap->size)
                     heap->free  = hsize;
@@ -768,14 +764,16 @@ static void s_HEAP_Free(HEAP             heap,
     /* NB: in order to maintain HEAP_Walk() "b" must have "size" updated
      * so that the next heap block could be located correctly, and also
      * "flag" must keep its HEAP_LAST bit so that it can be verified. */
-    unsigned int last = HEAP_ISLAST(b) ? HEAP_LAST : 0;
+    unsigned int last = HEAP_ISLAST(b);
 
+    assert(HEAP_ISUSED(b));
     assert(p < b  &&  b < n);
     assert((!p  ||  HEAP_NEXT(p) == b)  &&  b  &&  HEAP_NEXT(b) == n);
     assert((!p  ||  heap->base <= p)  &&  n <= heap->base + heap->size);
 
     s_HEAP_Unlink(b);
-    b->head.flag = HEAP_FREE | last;
+    b->head.flag = last;
+    heap->used -= HEAP_BLOCKS(b->head.size);
     if (!last  &&  !HEAP_ISUSED(n)) {
         assert((n->nextfree | n->prevfree) != (TNCBI_Size)(~0));
         b->head.size += n->head.size;
@@ -823,7 +821,7 @@ static void s_HEAP_Free(HEAP             heap,
 }
 
 
-void HEAP_Free(HEAP heap, SHEAP_Block* ptr)
+extern void HEAP_Free(HEAP heap, SHEAP_Block* ptr)
 {
     const SHEAP_HeapBlock* e;
     SHEAP_HeapBlock *b, *p;
@@ -833,7 +831,7 @@ void HEAP_Free(HEAP heap, SHEAP_Block* ptr)
         CORE_LOG_X(10, eLOG_Warning, "Heap Free: NULL heap");
         return;
     }
-    assert(!heap->base == !heap->size);
+    HEAP_CHECK(heap);
 
     if (unlikely(!heap->chunk)) {
         CORE_LOGF_X(11, eLOG_Error,
@@ -875,16 +873,16 @@ void HEAP_Free(HEAP heap, SHEAP_Block* ptr)
 
 
 /*FIXME: to remove*/
-void HEAP_FreeFast(HEAP heap, SHEAP_Block* ptr, const SHEAP_Block* prev)
+extern void HEAP_FreeFast(HEAP heap, SHEAP_Block* ptr, const SHEAP_Block* prev)
 {
-    SHEAP_HeapBlock *b, *p, *n;
+    SHEAP_HeapBlock *b, *p, *n, *t;
     char _id[32];
 
     if (unlikely(!heap)) {
         CORE_LOG_X(15, eLOG_Warning, "Heap Free: NULL heap");
         return;
     }
-    assert(!heap->base == !heap->size);
+    HEAP_CHECK(heap);
 
     if (unlikely(!heap->chunk)) {
         CORE_LOGF_X(16, eLOG_Error,
@@ -898,6 +896,11 @@ void HEAP_FreeFast(HEAP heap, SHEAP_Block* ptr, const SHEAP_Block* prev)
     b = (SHEAP_HeapBlock*) ptr;
     n = HEAP_NEXT(b);
 
+    if (likely(p  &&  HEAP_ISUSED(p))  &&  unlikely((t = HEAP_NEXT(p)) != b)
+        &&  likely(!HEAP_ISUSED(t)  &&  HEAP_NEXT(t) == b)) {
+        p = t;
+    }
+
     if (unlikely(!s_HEAP_fast)) {
         const SHEAP_HeapBlock* e = heap->base + heap->size;
         if (unlikely(b < heap->base)  ||  unlikely(e < n)) {
@@ -930,7 +933,7 @@ void HEAP_FreeFast(HEAP heap, SHEAP_Block* ptr, const SHEAP_Block* prev)
 }
 
 
-HEAP HEAP_Trim(HEAP heap)
+extern HEAP HEAP_Trim(HEAP heap)
 {
     TNCBI_Size hsize, size, prev;
     SHEAP_HeapBlock* b;
@@ -938,22 +941,22 @@ HEAP HEAP_Trim(HEAP heap)
 
     if (!heap)
         return 0;
-    assert(!heap->base == !heap->size);
+    HEAP_CHECK(heap);
  
     if (!heap->chunk) {
         CORE_LOGF_X(30, eLOG_Error,
                     ("Heap Trim%s: Heap read-only", s_HEAP_Id(_id, heap)));
         return 0;
     }
-    if (s_HEAP_fast  &&  heap->size == heap->free)
+    if (likely(s_HEAP_fast)  &&  heap->used == heap->size)
         return heap/*no free space, nothing to do*/;
 
     b = s_HEAP_Collect(heap, 0);
 
     if (b) {
         assert(!HEAP_ISUSED(b)  &&  HEAP_ISLAST(b));
-        prev = HEAP_BLOCKS(b->head.flag & ~_HEAP_OLDLAST);
-        b->head.flag = HEAP_FREE | HEAP_LAST;
+        prev = HEAP_BLOCKS(b->head.flag);
+        b->head.flag = HEAP_LAST;
     } else
         prev = 0;
     if (!b  ||  b->head.size < heap->chunk) {
@@ -975,7 +978,7 @@ HEAP HEAP_Trim(HEAP heap)
             assert(!base);
         else if (!base)
             return 0;
-        if (_HEAP_ALIGN(base, sizeof(SHEAP_Block)) != (unsigned long) base) {
+        if (_HEAP_ALIGN_2(base, sizeof(SHEAP_Block)) != (unsigned long) base) {
             CORE_LOGF_X(31, eLOG_Warning,
                         ("Heap Trim%s: Unaligned base (0x%08lX)",
                          s_HEAP_Id(_id, heap), (long) base));
@@ -1198,13 +1201,13 @@ static SHEAP_Block* s_HEAP_Walk(const HEAP heap, const SHEAP_Block* ptr)
 }
 
 
-SHEAP_Block* HEAP_Walk(const HEAP heap, const SHEAP_Block* ptr)
+extern SHEAP_Block* HEAP_Walk(const HEAP heap, const SHEAP_Block* ptr)
 {
     if (unlikely(!heap)) {
         CORE_LOG_X(29, eLOG_Warning, "Heap Walk: NULL heap");
         return 0;
     }
-    assert(!heap->base == !heap->size);
+    HEAP_CHECK(heap);
 
     if (likely(s_HEAP_fast)) {
         SHEAP_HeapBlock* b;
@@ -1218,7 +1221,7 @@ SHEAP_Block* HEAP_Walk(const HEAP heap, const SHEAP_Block* ptr)
 }
 
 
-SHEAP_Block* HEAP_Next(const HEAP heap, const SHEAP_Block* ptr)
+extern SHEAP_Block* HEAP_Next(const HEAP heap, const SHEAP_Block* ptr)
 {
     SHEAP_Block* next;
     for (next = HEAP_Walk(heap, ptr);  next;  next = HEAP_Walk(heap, next)) {
@@ -1229,17 +1232,17 @@ SHEAP_Block* HEAP_Next(const HEAP heap, const SHEAP_Block* ptr)
 }
 
 
-HEAP HEAP_Copy(const HEAP heap, size_t extra, int serial)
+extern HEAP HEAP_Copy(const HEAP heap, size_t extra, int serial)
 {
     HEAP       copy;
     TNCBI_Size size;
 
     assert(_HEAP_ALIGN_EX(1, sizeof(SHEAP_Block)) ==
-           _HEAP_ALIGN   (1, sizeof(SHEAP_Block)));
+           _HEAP_ALIGN_2 (1, sizeof(SHEAP_Block)));
 
     if (!heap)
         return 0;
-    assert(!heap->base == !heap->size);
+    HEAP_CHECK(heap);
 
     size = HEAP_EXTENT(heap->size);
     copy = (HEAP) malloc(sizeof(*copy) +
@@ -1248,14 +1251,15 @@ HEAP HEAP_Copy(const HEAP heap, size_t extra, int serial)
         return 0;
     if (size) {
         char* base = (char*) copy + sizeof(*copy);
-        base += _HEAP_ALIGN(base, sizeof(SHEAP_Block)) - (unsigned long) base;
-        assert(_HEAP_ALIGN(base, sizeof(SHEAP_Block)) == (unsigned long) base);
+        base +=_HEAP_ALIGN_2(base,sizeof(SHEAP_Block)) - (unsigned long) base;
+        assert(_HEAP_ALIGN_2(base,sizeof(SHEAP_Block)) ==(unsigned long) base);
         copy->base = (SHEAP_HeapBlock*) base;
     } else
         copy->base = 0;
     copy->size   = heap->size;
-    copy->free   = heap->size;
-    copy->last   = heap->size;
+    copy->free   = heap->free;
+    copy->used   = heap->used;
+    copy->last   = heap->last;
     copy->chunk  = 0/*read-only*/;
     copy->resize = 0;
     copy->auxarg = 0;
@@ -1269,11 +1273,11 @@ HEAP HEAP_Copy(const HEAP heap, size_t extra, int serial)
 }
 
 
-unsigned int HEAP_AddRef(HEAP heap)
+extern unsigned int HEAP_AddRef(HEAP heap)
 {
     if (!heap)
         return 0;
-    assert(!heap->base == !heap->size);
+    HEAP_CHECK(heap);
     if (heap->refcnt) {
         heap->refcnt++;
         assert(heap->refcnt);
@@ -1282,11 +1286,11 @@ unsigned int HEAP_AddRef(HEAP heap)
 }
 
 
-unsigned int HEAP_Detach(HEAP heap)
+extern unsigned int HEAP_Detach(HEAP heap)
 {
     if (!heap)
         return 0;
-    assert(!heap->base == !heap->size);
+    HEAP_CHECK(heap);
     if (heap->refcnt  &&  --heap->refcnt)
         return heap->refcnt;
     memset(heap, 0, sizeof(*heap));
@@ -1295,13 +1299,13 @@ unsigned int HEAP_Detach(HEAP heap)
 }
 
 
-unsigned int HEAP_Destroy(HEAP heap)
+extern unsigned int HEAP_Destroy(HEAP heap)
 {
     char _id[32];
 
     if (!heap)
         return 0;
-    assert(!heap->base == !heap->size);
+    HEAP_CHECK(heap);
     if (!heap->chunk  &&  !heap->refcnt) {
         CORE_LOGF_X(33, eLOG_Error,
                     ("Heap Destroy%s: Heap read-only", s_HEAP_Id(_id, heap)));
@@ -1311,29 +1315,68 @@ unsigned int HEAP_Destroy(HEAP heap)
 }
 
 
-void* HEAP_Base(const HEAP heap)
+extern void* HEAP_Base(const HEAP heap)
 {
     if (!heap)
         return 0;
-    assert(!heap->base == !heap->size);
+    HEAP_CHECK(heap);
     return heap->base;
 }
 
 
-TNCBI_Size HEAP_Size(const HEAP heap)
+extern TNCBI_Size HEAP_Size(const HEAP heap)
 {
     if (!heap)
         return 0;
-    assert(!heap->base == !heap->size);
+    HEAP_CHECK(heap);
     return HEAP_EXTENT(heap->size);
 }
 
 
-int HEAP_Serial(const HEAP heap)
+extern TNCBI_Size HEAP_Used(const HEAP heap)
 {
     if (!heap)
         return 0;
-    assert(!heap->base == !heap->size);
+    HEAP_CHECK(heap);
+    return HEAP_EXTENT(heap->used);
+}
+
+
+extern TNCBI_Size HEAP_Idle(const HEAP heap)
+{
+    TNCBI_Size size = 0;
+    if (heap) {
+        HEAP_CHECK(heap);
+        if (heap->free < heap->size) {
+            const SHEAP_HeapBlock* f = heap->base + heap->free;
+            const SHEAP_HeapBlock* b = f;
+            do {
+                size += b->head.size;
+                b = heap->base + b->prevfree;
+            } while (b != f);
+#if defined(_DEBUG)  &&  !defined(NDEBUG)
+            {{
+                TNCBI_Size alt_size = 0;
+                do {
+                    alt_size += b->head.size;
+                    b = heap->base + b->nextfree;
+                } while (b != f);
+                assert(alt_size == size);
+            }}
+#endif /*_DEBUG && !NDEBUG*/
+        }
+        assert(size == HEAP_SIZE(size));
+        assert(size == HEAP_EXTENT(heap->size - heap->used));
+    }
+    return size;
+}
+
+
+extern int HEAP_Serial(const HEAP heap)
+{
+    if (!heap)
+        return 0;
+    HEAP_CHECK(heap);
     return heap->serial;
 }
 
