@@ -1,4 +1,4 @@
-/* $Id: ncbi_http_session.cpp 532795 2017-04-09 16:38:29Z lavr $
+/* $Id: ncbi_http_session.cpp 567636 2018-07-21 15:53:54Z mcelhany $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -35,8 +35,10 @@
 #include <corelib/ncbifile.hpp>
 #include <corelib/request_ctx.hpp>
 #include <corelib/ncbimtx.hpp>
+#include <corelib/ncbistr.hpp>
 #include <connect/ncbi_http_session.hpp>
 #include <stdlib.h>
+#include "ncbi_ansi_ext.h"
 
 
 BEGIN_NCBI_SCOPE
@@ -46,7 +48,7 @@ BEGIN_NCBI_SCOPE
 //  CHttpHeaders::
 //
 
-static const char* s_HttpHeaderNames[] = {
+static const char* kHttpHeaderNames[] = {
     "Cache-Control",
     "Content-Length",
     "Content-Type",
@@ -57,7 +59,8 @@ static const char* s_HttpHeaderNames[] = {
     "Range",
     "Referer",
     "Set-Cookie",
-    "User-Agent"
+    "User-Agent",
+    "Host"
 };
 
 
@@ -73,8 +76,8 @@ static const char kHttpHeaderDelimiter = ':';
 
 const char* CHttpHeaders::GetHeaderName(EHeaderName name)
 {
-    _ASSERT(size_t(name) < sizeof(s_HttpHeaderNames)/sizeof(s_HttpHeaderNames[0]));
-    return s_HttpHeaderNames[name];
+    _ASSERT(size_t(name)<sizeof(kHttpHeaderNames)/sizeof(kHttpHeaderNames[0]));
+    return kHttpHeaderNames[name];
 }
 
 
@@ -227,7 +230,7 @@ void CHttpFormData::SetContentType(EContentType content_type)
 {
     if (!m_Providers.empty()  &&  content_type != eMultipartFormData) {
         NCBI_THROW(CHttpSessionException, eBadContentType,
-            "The requested Content-Type can not be used with the form data.");
+            "The requested Content-Type cannot be used with the form data.");
     }
     m_ContentType = content_type;
 }
@@ -239,7 +242,7 @@ void CHttpFormData::AddEntry(CTempString entry_name,
 {
     if ( entry_name.empty() ) {
         NCBI_THROW(CHttpSessionException, eBadFormDataName,
-            "Form data entry name can not be empty.");
+            "Form data entry name must not be empty.");
     }
     TValues& values = m_Entries[entry_name];
     SFormData entry;
@@ -254,7 +257,7 @@ void CHttpFormData::AddProvider(CTempString             entry_name,
 {
     if ( entry_name.empty() ) {
         NCBI_THROW(CHttpSessionException, eBadFormDataName,
-            "Form data entry name can not be empty.");
+            "Form data entry name must not be empty.");
     }
     m_ContentType = eMultipartFormData;
     m_Providers[entry_name].push_back(Ref(provider));
@@ -532,15 +535,15 @@ unsigned short SGetHttpDefaultRetries::operator()(void) const
     ConnNetInfo_GetValue(0, REG_CONN_MAX_TRY, buf, sizeof(buf), STR(DEF_CONN_MAX_TRY));
 #undef   STR
 #undef  _STR
-    unsigned short maxtry = atoi(buf);
-    return maxtry ? maxtry - 1 : 0;
+    int maxtry = atoi(buf);
+    return (unsigned short)(maxtry ? maxtry - 1 : 0);
 }
 
 
 inline
 unsigned short x_RetriesToMaxtry(unsigned short retries)
 {
-    return ++retries ? retries : retries - 1;
+    return (unsigned short)(++retries ? retries : retries - 1);
 }
 
 
@@ -549,7 +552,7 @@ CHttpRequest::CHttpRequest(CHttpSession& session,
                            EReqMethod    method)
     : m_Session(&session),
       m_Url(url),
-      m_IsService(false),
+      m_IsService(url.IsService()),
       m_Method(method),
       m_Headers(new CHttpHeaders),
       m_Timeout(CTimeout::eDefault),
@@ -748,12 +751,22 @@ void CHttpRequest::x_InitConnection(bool use_form_data)
         NCBI_THROW(CHttpSessionException, eBadRequest,
             "An attempt to execute HTTP request already being executed");
     }
-    SConnNetInfo* net_info = ConnNetInfo_Create(0);
+    SConnNetInfo* net_info = ConnNetInfo_Create(
+        m_Url.IsService() ? m_Url.GetService().c_str() : 0);
     if (m_Session->GetProtocol() == CHttpSession::eHTTP_11) {
         net_info->version = 1;
     }
     net_info->req_method = m_Method;
 
+    // Set scheme if given in URL (only if http(s) since this is CHttpRequest).
+    string url_scheme(m_Url.GetScheme());
+    if (NStr::EqualNocase(url_scheme, "https")) {
+        net_info->scheme = eURL_Https;
+    }
+    else if (NStr::EqualNocase(url_scheme, "http")) {
+        net_info->scheme = eURL_Http;
+    }
+
     // Save headers set automatically (e.g. from CONN_HTTP_USER_HEADER).
     if (net_info->http_user_header) {
         CHttpHeaders usr_hdr;
@@ -761,7 +774,7 @@ void CHttpRequest::x_InitConnection(bool use_form_data)
         m_Headers->Merge(usr_hdr);
     }
 
-    x_AddCookieHeader(m_Url);
+    x_AddCookieHeader(m_Url, true);
     if (use_form_data) {
         m_Headers->SetValue(CHttpHeaders::eContentType,
             m_FormData->GetContentTypeStr());
@@ -795,6 +808,10 @@ void CHttpRequest::x_InitConnection(bool use_form_data)
     else {
         // Try to resolve service name.
         m_IsService = true;
+        const string& path = m_Url.GetPath();
+        if (!path.empty()) {
+            strncpy0(net_info->path, path.c_str(), sizeof(net_info->path) - 1);
+        }
         ConnNetInfo_SetUserHeader(net_info, headers.c_str());
         SSERVICE_Extra x_extra;
         memset(&x_extra, 0, sizeof(x_extra));
@@ -818,11 +835,11 @@ bool CHttpRequest::x_CanSendData(void) const
 }
 
 
-void CHttpRequest::x_AddCookieHeader(const CUrl& url)
+void CHttpRequest::x_AddCookieHeader(const CUrl& url, bool initial)
 {
     if ( !m_Session ) return;
     string cookies = m_Session->x_GetCookies(url);
-    if ( !cookies.empty() ) {
+    if ( !cookies.empty()  ||  !initial ) {
         m_Headers->SetValue(CHttpHeaders::eCookie, cookies);
     }
 }
@@ -850,9 +867,9 @@ EHTTP_HeaderParse CHttpRequest::sx_ParseHeader(const char* http_header,
 
 // CConn_HttpStream callback for handling retries and redirects.
 // user_data must contain CHttpRequest*.
-int CHttpRequest::sx_Adjust(SConnNetInfo* net_info,
-                            void*         user_data,
-                            unsigned int  failure_count)
+int/*bool*/ CHttpRequest::sx_Adjust(SConnNetInfo* net_info,
+                                    void*         user_data,
+                                    unsigned int  failure_count)
 {
     if ( !user_data ) return 1;
     // Reset and re-fill headers on redirects (failure_count == 0).
@@ -869,7 +886,7 @@ int CHttpRequest::sx_Adjust(SConnNetInfo* net_info,
     case 405:
     case 406:
     case 410:
-        return 0;
+        return 0; // false
     default:
         break;
     }
@@ -884,7 +901,9 @@ int CHttpRequest::sx_Adjust(SConnNetInfo* net_info,
                 adjust = req->m_AdjustUrl->AdjustUrl(url);
             }
             else {
-                url.Adjust(req->m_Url, CUrl::fPath_Append | CUrl::fArgs_Merge);
+                url.Adjust(req->m_Url, CUrl::fScheme_Replace |
+                                       CUrl::fPath_Append    |
+                                       CUrl::fArgs_Merge);
             }
             if ( adjust ) {
                 ConnNetInfo_ParseURL(net_info, url.ComposeUrl(CUrlArgs::eAmp_Char).c_str());
@@ -897,9 +916,9 @@ int CHttpRequest::sx_Adjust(SConnNetInfo* net_info,
         free(loc);
     }
     // Discard old cookies, add those for the new location.
-    req->x_AddCookieHeader(resp->m_Location);
+    req->x_AddCookieHeader(resp->m_Location, false);
     string headers = req->m_Headers->GetHttpHeader();
-    ConnNetInfo_SetUserHeader(net_info, headers.c_str());
+    ConnNetInfo_OverrideUserHeader(net_info, headers.c_str());
     return 1; // true
 }
 
