@@ -1,4 +1,4 @@
-/* $Id: ncbi_ipv6.c 545913 2017-09-12 14:25:49Z ivanov $
+/* $Id: ncbi_ipv6.c 565391 2018-06-11 18:09:29Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -49,41 +49,52 @@ static const struct SIPDNSsfx kIPv6DNS = { ".ip6.arpa",      9 };
 static const struct SIPDNSsfx kIPv4DNS = { ".in-addr.arpa", 13 };
 
 
-static int/*bool*/ x_NcbiIsIPv4(const TNCBI_IPv6Addr* addr, int/*bool*/ mapped)
+static int/*bool*/ x_NcbiIsIPv4(const TNCBI_IPv6Addr* addr, int/*bool*/ compat)
 {
     /* RFC 4291 2.1, 3
        NB: 2.5.5.1 and 2.5.5.2 - both obsoleted by RFC 6052 2.1 */
     unsigned short word;
     size_t i;
     for (i = 0;  i < 5;  ++i) {
-        memcpy(&word, addr->octet + (i << 1), sizeof(word));
+        memcpy(&word, addr->octet + i * sizeof(word), sizeof(word));
         if (word)
             return 0/*false*/;
     }
-    memcpy(&word, addr->octet + (5 << 1), sizeof(word));
-    if    (word == 0x0000  &&  mapped >= 0) {
+    memcpy(&word, addr->octet + i * sizeof(word), sizeof(word));
+    if    (word == 0x0000) {
         /* IPv4-compatible IPv6 */
-        unsigned int temp;
-        memcpy(&temp, addr->octet + sizeof(addr->octet) - sizeof(temp),
-               sizeof(temp));
-        return SOCK_NetToHostLong(temp) & 0xFF000000 ? 1/*T*/ : 0/*F*/;
+        if (compat) {
+            unsigned int temp;
+            memcpy(&temp, addr->octet + sizeof(addr->octet) - sizeof(temp),
+                   sizeof(temp));
+            if (SOCK_NetToHostLong(temp) & 0xFF000000)
+                return 1/*true*/;
+        }
+        return 0/*false*/;
     }
-    /* IPv6 mapped IPv4 */
-    return word == 0xFFFF  &&  mapped ? 1/*true*/ : 0/*false*/;
+    /* mapped IPv4 */
+    return word == 0xFFFF ? 1/*true*/ : 0/*false*/;
 }
 
 
 extern int/*bool*/ NcbiIsEmptyIPv6(const TNCBI_IPv6Addr* addr)
 {
-    return !memcchr(addr->octet, '\0', sizeof(addr->octet))
-        ||  (x_NcbiIsIPv4(addr, 1/*mapped*/)  &&  !NcbiIPv6ToIPv4(addr, 0))
+    return !addr
+        ||  !memcchr(addr->octet, 0, sizeof(addr->octet))
+        ||  (x_NcbiIsIPv4(addr, 0/*mapped*/)  &&  !NcbiIPv6ToIPv4(addr, 0))
         ? 1/*true*/ : 0/*false*/;
 }
 
 
 extern int/*bool*/ NcbiIsIPv4(const TNCBI_IPv6Addr* addr)
 {
-    return x_NcbiIsIPv4(addr, 1/*mapped*/);
+    return addr  &&  x_NcbiIsIPv4(addr, 0/*mapped*/) ? 1/*true*/ : 0/*false*/;
+}
+
+
+extern int/*bool*/ NcbiIsIPv4Ex(const TNCBI_IPv6Addr* addr, int/*bool*/ compat)
+{
+    return addr  &&  x_NcbiIsIPv4(addr, compat) ? 1/*true*/ : 0/*false*/;
 }
 
 
@@ -91,8 +102,10 @@ extern unsigned int NcbiIPv6ToIPv4(const TNCBI_IPv6Addr* addr, size_t pfxlen)
 {
     unsigned int ipv4;
     static const size_t size = sizeof(ipv4);
+    if (!addr)
+        return 0;
     if (pfxlen == 0) {
-        if (!x_NcbiIsIPv4(addr, 1/*mapped*/))
+        if (!x_NcbiIsIPv4(addr, 1/*compat*/))
             return 0;
         pfxlen  = 96;
     }
@@ -120,7 +133,7 @@ extern unsigned int NcbiIPv6ToIPv4(const TNCBI_IPv6Addr* addr, size_t pfxlen)
         break;
     default:
         assert(0);
-        return (unsigned int)(-1L)/*failure*/;
+        return (unsigned int)(-1)/*failure*/;
     }
     return ipv4;
 }
@@ -130,6 +143,8 @@ extern int/*bool*/ NcbiIPv4ToIPv6(TNCBI_IPv6Addr* addr,
                                   unsigned int ipv4, size_t pfxlen)
 {
     static const size_t size = sizeof(ipv4);
+    if (!addr)
+        return 0/*failure*/;
     if (pfxlen == 0) {
         /* creates IPv6 mapped */
         memset(addr, 0, sizeof(*addr));
@@ -181,15 +196,15 @@ static const char* x_StringToIPv4(unsigned int* dst,
 
     *ptr = 0;
     for (n = 0;  n < len;  ++n) {
-        unsigned char c = str[n];
+        char c = str[n];
         if ('0' <= c  &&  c <= '9') {
             unsigned int val;
             if (was_digit  &&  !*ptr)
                 return 0/*leading "0" in octet*/;
-            val = *ptr * 10 + (c - '0');
+            val = (unsigned int)(*ptr * 10 + (c - '0'));
             if (val > 255)
                 return 0;
-            *ptr = val;
+            *ptr = (unsigned char) val;
             if (!was_digit) {
                 ++octets;
                 assert(octets <= 4);
@@ -239,7 +254,8 @@ static const char* x_StringToIPv6(TNCBI_IPv6Addr* addr,
 
     if (len < 2  ||  (str[n = 0] == ':'  &&  str[++n] != ':'))
         return 0/*failure*/;
-    gap = ipv4 = 0/*false*/;
+    gap = 0;
+    ipv4 = 0/*false*/;
     token[t = 0].ptr = str + n;
     while (n <= len) {
         assert(t <= maxt);
@@ -378,7 +394,7 @@ static char* x_IPv6ToString(char* buf, size_t bufsize,
     size_t i, n, z, zlen;
     char* ptr = ipv6;
 
-    if (x_NcbiIsIPv4(addr, 1/*mapped*/)) {
+    if (x_NcbiIsIPv4(addr, 1/*compat*/)) {
         unsigned int ip;
         n = sizeof(addr->octet) - sizeof(ip);
         memcpy(&ip, addr->octet + n, sizeof(ip));
@@ -469,6 +485,8 @@ extern char* NcbiIPv6ToString(char* buf, size_t bufsize,
     if (!buf  ||  !bufsize)
         return 0;
     *buf = '\0';
+    if (!addr)
+        return 0;
 
     return x_IPv6ToString(buf, bufsize, addr);
 }
@@ -480,8 +498,10 @@ extern char* NcbiAddrToString(char* buf, size_t bufsize,
     if (!buf  ||  !bufsize)
         return 0;
     *buf = '\0';
+    if (!addr)
+        return 0;
 
-    if (x_NcbiIsIPv4(addr, -1/*mapped only*/)) {
+    if (x_NcbiIsIPv4(addr, 0/*mapped*/)) {
         unsigned int ipv4 = NcbiIPv6ToIPv4(addr, 0);
         return x_IPv4ToString(buf, bufsize, &ipv4);
     }
@@ -500,10 +520,12 @@ extern const char* NcbiAddrToDNS(char* buf, size_t bufsize,
     if (!buf  ||  !bufsize)
         return 0;
     *buf = '\0';
+    if (!addr)
+        return 0;
 
     len = 0;
     src = addr->octet + sizeof(addr->octet) - 1;
-    if (x_NcbiIsIPv4(addr, -1/*mapped only*/)) {
+    if (x_NcbiIsIPv4(addr, 0/*mapped*/)) {
         sfx = &kIPv4DNS;
         for (n = 0;  n < sizeof(unsigned int);  ++n) {
             size_t off = (size_t)sprintf(dst, "%d.",    *src--);
@@ -625,19 +647,19 @@ static const char* s_StringToAddr(TNCBI_IPv6Addr* addr,
     if (dns) {
         dns = str[--n] == '.' ? 1/*true*/ : 0/*false*/;
         if (len > kIPv4DNS.len
-            &&  strncasecmp(tmp = str + len - (kIPv4DNS.len + dns),
+            &&  strncasecmp(tmp = str + len - (kIPv4DNS.len + !!dns),
                             kIPv4DNS.sfx, kIPv4DNS.len) == 0) {
-            if (x_DNSToIPv4(&ipv4, str, len - (kIPv4DNS.len + dns)) == tmp) {
+            if (x_DNSToIPv4(&ipv4, str, len - (kIPv4DNS.len + !!dns)) == tmp) {
                 NcbiIPv4ToIPv6(addr, ipv4, 0);
-                return tmp + (kIPv4DNS.len + dns);
+                return tmp + (kIPv4DNS.len + !!dns);
             } else if (dns)
                 return 0/*failure*/;
         }
         if (len > kIPv6DNS.len
-            &&  strncasecmp(tmp = str + len - (kIPv6DNS.len + dns),
+            &&  strncasecmp(tmp = str + len - (kIPv6DNS.len + !!dns),
                             kIPv6DNS.sfx, kIPv6DNS.len) == 0) {
-            if (x_DNSToIPv6(addr,  str, len - (kIPv6DNS.len + dns)) == tmp)
-                return tmp + (kIPv6DNS.len + dns);
+            if (x_DNSToIPv6(addr,  str, len - (kIPv6DNS.len + !!dns)) == tmp)
+                return tmp + (kIPv6DNS.len + !!dns);
             else if (dns)
                 return 0/*failure*/;
         }
@@ -665,25 +687,28 @@ extern const char* NcbiStringToAddr(TNCBI_IPv6Addr* addr,
 }
 
 
-int/*bool*/ NcbiIsInIPv6Network(const TNCBI_IPv6Addr* base,
-                                unsigned int          bits,
-                                const TNCBI_IPv6Addr* addr)
+extern int/*bool*/ NcbiIsInIPv6Network(const TNCBI_IPv6Addr* base,
+                                       unsigned int          bits,
+                                       const TNCBI_IPv6Addr* addr)
 {
     size_t n;
 
+    if (!base  ||  !addr)
+        return 0/*false*/;
+
     if (bits > (sizeof(base->octet) << 3))
         return 0/*false*/;
 
     for (n = 0;  n < sizeof(addr->octet);  ++n) {
         unsigned char mask;
         if (!bits)
-            mask  =  0;
+            mask  =                  0;
         else if (8 > bits) {
-            mask  = ~0 << (8 - bits);
-            bits  =  0;
+            mask  = (unsigned char)(~0 << (8 - bits));
+            bits  =                  0;
         } else {
-            mask  = ~0;
-            bits -=  8;
+            mask  = (unsigned char)(~0);
+            bits -=                  8;
         }
         if ((addr->octet[n] & mask) ^ base->octet[n])
             return 0/*false*/;
@@ -691,3 +716,29 @@ int/*bool*/ NcbiIsInIPv6Network(const TNCBI_IPv6Addr* base,
 
     return 1/*true*/;
 }
+
+
+extern int/*bool*/ NcbiIPv6Subnet(TNCBI_IPv6Addr* addr, unsigned int bits)
+{
+    int/*bool*/ zero = 1/*true*/;
+
+    if (addr) {
+        size_t n;
+        for (n = 0;  n < sizeof(addr->octet);  ++n) {
+            if (!bits)
+                addr->octet[n] = 0;
+            else if (8 > bits) {
+                unsigned char mask = (unsigned char)(~0 << (8 - bits));
+                if (addr->octet[n] &= mask)
+                    zero = 0/*false*/;
+                bits  = 0;
+            } else {
+                bits -= 8;
+                if (addr->octet[n])
+                    zero = 0/*false*/;
+            }
+        }
+    }
+
+    return !zero;
+}
