@@ -1,4 +1,4 @@
-/* $Id: ncbi_mbedtls.c 545593 2017-09-07 18:06:33Z ivanov $
+/* $Id: ncbi_mbedtls.c 565910 2018-06-20 19:14:14Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -152,8 +152,8 @@ static int                      s_MbedTlsLogLevel;
 static mbedtls_entropy_context  s_MbedTlsEntropy;
 static mbedtls_ctr_drbg_context s_MbedTlsCtrDrbg;
 static mbedtls_ssl_config       s_MbedTlsConf;
-static FSSLPull                 s_Pull;
-static FSSLPush                 s_Push;
+static volatile FSSLPull        s_Pull;
+static volatile FSSLPush        s_Push;
 
 
 /*ARGSUSED*/
@@ -229,14 +229,14 @@ static EIO_Status x_ErrorToStatus(int error, mbedtls_ssl_context* session,
     case MBEDTLS_ERR_NET_RECV_FAILED:
         if (sock->r_status != eIO_Success  &&
             sock->r_status != eIO_Unknown) {
-            status = sock->r_status;
+            status = (EIO_Status) sock->r_status;
         } else
             status = eIO_Unknown;
         break;
     case MBEDTLS_ERR_NET_SEND_FAILED:
         if (sock->w_status != eIO_Success  &&
             sock->w_status != eIO_Unknown) {
-            status = sock->w_status;
+            status = (EIO_Status) sock->w_status;
         } else
             status = eIO_Unknown;
         break;
@@ -247,8 +247,8 @@ static EIO_Status x_ErrorToStatus(int error, mbedtls_ssl_context* session,
         break;
     }
 
-    CORE_LOGF(eLOG_Trace, ("MBEDTLS error %d -> CONNECT MBEDTLS status %s",
-                           error, IO_StatusStr(status)));
+    CORE_TRACEF(("MBEDTLS error %d -> CONNECT MBEDTLS status %s",
+                 error, IO_StatusStr(status)));
 
     return status;
 }
@@ -268,9 +268,6 @@ static int x_StatusToError(EIO_Status status, SOCK sock, EIO_Event direction)
     case eIO_Timeout:
         error = EAGAIN;
         break;
-    case eIO_Closed:
-        error = SOCK_ENOTCONN;
-        break;
     case eIO_Interrupt:
         error = SOCK_EINTR;
         break;
@@ -280,6 +277,9 @@ static int x_StatusToError(EIO_Status status, SOCK sock, EIO_Event direction)
     case eIO_Unknown:
         error = 0/*keep*/;
         break;
+    case eIO_Closed:
+        error = SOCK_ENOTCONN;
+        break;
     default:
         /*NB:eIO_InvalidArg*/
         error = EINVAL;
@@ -287,12 +287,11 @@ static int x_StatusToError(EIO_Status status, SOCK sock, EIO_Event direction)
     }
 
     {{
-        const char* x_what = error ? "error" : "errno";
-        int        x_error = error ?  error  :  errno;
-        CORE_LOGF(eLOG_Trace, ("CONNECT MBEDTLS status %s -> %s %d",
-                               IO_StatusStr(status), x_what, x_error));
+        int x_err = error ? error : errno;
+        CORE_TRACEF(("CONNECT MBEDTLS status %s -> %s %d",
+                     IO_StatusStr(status), error ? "error" : "errno", x_err));
         if (!error)
-            errno = x_error; /* restore errno that may be clobbered by log */
+            errno = x_err; /* restore errno that may be clobbered by log */
     }}
 
     if (error)
@@ -433,9 +432,9 @@ static int x_MbedTlsPush(void* ctx, const unsigned char* data, size_t size)
             } else {
                 assert(status == eIO_Success);
                 assert(x_written <= size);
-                n_written += x_written;
-                size      -= x_written;
-                data       = data + x_written;
+                n_written += (ssize_t) x_written;
+                size      -=           x_written;
+                data       = data  +   x_written;
             }
         } while (size);
         return (int) n_written;
@@ -504,8 +503,9 @@ static EIO_Status s_MbedTlsRead(void* session, void* buf, size_t n_todo,
 
     assert(session);
 
-    x_read = mbedtls_ssl_read((mbedtls_ssl_context*) session, buf, n_todo);
-    assert(x_read < 0  ||  x_read <= n_todo);
+    x_read = mbedtls_ssl_read((mbedtls_ssl_context*) session,
+                              (unsigned char*) buf, n_todo);
+    assert(x_read < 0  ||  (size_t) x_read <= n_todo);
 
     if (x_read <= 0) {
         status = x_ErrorToStatus(x_read, (mbedtls_ssl_context*) session,
@@ -515,7 +515,7 @@ static EIO_Status s_MbedTlsRead(void* session, void* buf, size_t n_todo,
     } else
         status = eIO_Success;
 
-    *n_done = x_read;
+    *n_done = (size_t) x_read;
     return status;
 }
 
@@ -528,8 +528,9 @@ static EIO_Status x_MbedTlsWrite(void* session, const void* data,
 
     assert(session);
 
-    x_written = mbedtls_ssl_write((mbedtls_ssl_context*)session, data, n_todo);
-    assert(x_written < 0  ||  x_written <= n_todo);
+    x_written = mbedtls_ssl_write((mbedtls_ssl_context*) session,
+                                  (const unsigned char*) data, n_todo);
+    assert(x_written < 0  ||  (size_t) x_written <= n_todo);
 
     if (x_written <= 0) {
         status = x_ErrorToStatus(x_written, (mbedtls_ssl_context*) session,
@@ -539,7 +540,7 @@ static EIO_Status x_MbedTlsWrite(void* session, const void* data,
     } else
         status = eIO_Success;
 
-    *n_done = x_written;
+    *n_done = (size_t) x_written;
     return status;
 }
 
@@ -624,7 +625,9 @@ static EIO_Status s_MbedTlsInit(FSSLPull pull, FSSLPush push)
                                 MBEDTLS_SSL_PRESET_DEFAULT);
     mbedtls_ssl_conf_authmode(&s_MbedTlsConf, MBEDTLS_SSL_VERIFY_NONE);
 
-    val = ConnNetInfo_GetValue(0, "TLS_LOGLEVEL", buf, sizeof(buf), 0);
+    val = ConnNetInfo_GetValue(0, "MBEDTLS_LOGLEVEL", buf, sizeof(buf), 0);
+    if (!val  ||  !*val)
+        val = ConnNetInfo_GetValue(0, "TLS_LOGLEVEL", buf, sizeof(buf), 0);
     CORE_LOCK_READ;
     if (val  &&  *val) {
         ELOG_Level level;
@@ -638,8 +641,7 @@ static EIO_Status s_MbedTlsInit(FSSLPull pull, FSSLPush push)
             level = eLOG_Trace;
         CORE_LOGF(level, ("%s V%s (LogLevel=%d)",
                           kMbedTls, version, s_MbedTlsLogLevel));
-	}
-	else
+	} else
         CORE_UNLOCK;
 
     if ((status = x_InitLocking()) != eIO_Success) {
