@@ -1,4 +1,4 @@
-/* $Id: ncbi_namerd.c 539642 2017-06-26 13:22:31Z ivanov $
+/* $Id: ncbi_namerd.c 570469 2018-09-10 13:46:13Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,6 +34,7 @@
 #include "ncbi_ansi_ext.h"
 #include "ncbi_comm.h"
 #include "ncbi_lb.h"
+#include "ncbi_linkerd.h"
 #include "ncbi_namerd.h"
 #include "ncbi_once.h"
 #include "ncbi_priv.h"
@@ -43,6 +44,7 @@
 #include <connect/ncbi_connutil.h>
 #include <connect/ncbi_http_connector.h>
 #include <connect/ncbi_memory_connector.h>
+#include <connect/ncbi_server_info.h>
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -105,11 +107,11 @@ enum ENAMERD_Subcodes {
 #define REG_NAMERD_PROXY_HOST_DEF   "linkerd"
 #else
 #define REG_NAMERD_PROXY_HOST_DEF   \
-    "linkerd-proxy.service.bethesda-dev.consul.ncbi.nlm.nih.gov"
+    "pool.linkerd-proxy.service.bethesda-dev.consul.ncbi.nlm.nih.gov"
 #endif
 
 #define REG_NAMERD_PROXY_PORT_KEY   "PROXY_PORT"
-#define REG_NAMERD_PROXY_PORT_DEF   4140
+#define REG_NAMERD_PROXY_PORT_DEF   "4140"
 
 #define REG_NAMERD_API_HOST_KEY     "API_HOST"
 #define REG_NAMERD_API_HOST_DEF     "namerd-api.linkerd.ncbi.nlm.nih.gov"
@@ -132,6 +134,9 @@ enum ENAMERD_Subcodes {
 #define REG_NAMERD_DTAB_KEY         "DTAB"
 #define REG_NAMERD_DTAB_DEF         ""
 
+/* Default rate increase 20% if svc runs locally */
+#define NAMERD_LOCAL_BONUS          1.2
+
 
 #ifdef __cplusplus
 extern "C" {
@@ -429,34 +434,52 @@ static EBURLScheme s_GetScheme()
 }
 
 
-static unsigned short s_GetProxyPort()
+static int s_GetHttpProxy(char* host, size_t host_size, unsigned short* port_p)
 {
-    char val[24];
-    char reg_def_port[24];
-    unsigned short port;
+    char port_str[24];
+
+    /* Highest precedence is $http_proxy environment variable */
+    switch (LINKERD_GetHttpProxy(host, host_size, port_p)) {
+        case eLGHP_Success:
+            return 1;
+        case eLGHP_Fail:
+            CORE_LOG_X(eNSub_BadData, eLOG_Critical,
+                       "Couldn't get Linkerd http_proxy.");
+            return 0;
+        case eLGHP_NotSet:
+            break;
+    }
 
-    sprintf(reg_def_port, "%hu", REG_NAMERD_PROXY_PORT_DEF);
     if ( ! ConnNetInfo_GetValue(REG_NAMERD_SECTION,
-        REG_NAMERD_PROXY_PORT_KEY, val, sizeof(val)-1,
-        reg_def_port))
+        REG_NAMERD_PROXY_HOST_KEY, host, host_size - 1,
+        REG_NAMERD_PROXY_HOST_DEF)  ||  ! *host)
     {
         CORE_LOG_X(eNSub_Alloc, eLOG_Critical,
-                   "Couldn't alloc for proxy port.");
+                   "Couldn't set default http proxy host.");
         return 0;
     }
-    if ( ! *val)
-        return 0;
-    if (sscanf(val, "%hu", &port) == 1)
-        return port;
 
-    return 0;
+    if ( ! ConnNetInfo_GetValue(REG_NAMERD_SECTION,
+            REG_NAMERD_PROXY_PORT_KEY, port_str, sizeof(port_str) - 1,
+            REG_NAMERD_PROXY_PORT_DEF)  ||
+         ! *port_str  ||
+         sscanf(port_str, "%hu", port_p) != 1)
+    {
+        CORE_LOG_X(eNSub_Alloc, eLOG_Critical,
+                   "Couldn't set default http proxy port.");
+        return 0;
+    }
+
+    return 1;
 }
 
 
 static void s_RemoveCand(struct SNAMERD_Data* data, size_t n, int free_info)
 {
-    CORE_TRACE("s_RemoveCand() Removing endpoint.");
-    assert(n >= 0  &&  n < data->n_cand);
+    CORE_TRACEF(("s_RemoveCand() Removing info " FMT_SIZE_T ": %p",
+                 n, data->cand[n].info));
+
+    assert(n < data->n_cand);
     if (free_info)
         free((void*) data->cand[n].info);
     if (n < --data->n_cand) {
@@ -471,7 +494,8 @@ static void s_RemoveCand(struct SNAMERD_Data* data, size_t n, int free_info)
 static void s_RemoveStandby(struct SNAMERD_Data* data)
 {
     double  max_rate = 0.0;
-    int     all_standby = 1, i;
+    int     all_standby = 1;
+    size_t  i;
 
     /*  basic logic:
         if any endpoints have rate >= LBSM_STANDBY_THRESHOLD
@@ -480,7 +504,7 @@ static void s_RemoveStandby(struct SNAMERD_Data* data)
             discard all endpoints with rate < highest rate
     */
 
-    for (i = 0;  i < (int)data->n_cand;  ++i) {
+    for (i = 0;  i < data->n_cand;  ++i) {
 
         if (max_rate < data->cand[i].info->rate)
             max_rate = data->cand[i].info->rate;
@@ -489,10 +513,13 @@ static void s_RemoveStandby(struct SNAMERD_Data* data)
             all_standby = 0;
     }
 
-    for (i = (int)data->n_cand - 1;  i >= 0;  --i) {
+    /* Loop from highest index to lowest to ensure well-defined behavior when
+        candidates are removed and to avoid memmove in s_RemoveCand(). */
+    for (i = data->n_cand;  i > 0;  ) {
+        --i; /* decrement here to avoid unsigned underflow */
         if (data->cand[i].info->rate <
                 (all_standby ? max_rate : LBSM_STANDBY_THRESHOLD))
-            s_RemoveCand(data, (size_t)i, 1);
+            s_RemoveCand(data, i, 1);
     }
 }
 
@@ -502,7 +529,7 @@ static int/*bool*/ s_AddServerInfo(struct SNAMERD_Data* data, SSERV_Info* info)
     const char* name = SERV_NameOfInfo(info);
     size_t i;
 
-    /* First check that the new server info updates an existing one */
+    /* First check if the new server info updates an existing one */
     for (i = 0;  i < data->n_cand;  ++i) {
         if (strcasecmp(name, SERV_NameOfInfo(data->cand[i].info)) == 0
             &&  SERV_EqualInfo(info, data->cand[i].info))
@@ -537,8 +564,9 @@ static int/*bool*/ s_AddServerInfo(struct SNAMERD_Data* data, SSERV_Info* info)
     {{
         char hostbuf[40];
         SOCK_ntoa(info->host, hostbuf, sizeof(hostbuf));
-        CORE_TRACEF(("Added candidate %s:%hu with svc_type '%s'.",
-            hostbuf, info->port, SERV_TypeStr(info->type)));
+        CORE_TRACEF(("Added candidate " FMT_SIZE_T
+            ":   %s:%hu with svc_type '%s'.",
+            data->n_cand, hostbuf, info->port, SERV_TypeStr(info->type)));
     }}
 #endif
     data->cand[data->n_cand].info   = info;
@@ -559,37 +587,35 @@ static int/*bool*/ s_AddServerInfo(struct SNAMERD_Data* data, SSERV_Info* info)
     */
 static TNCBI_Time s_ParseExpires(time_t tt_now, const char* expires)
 {
-    int     tm_year, tm_mon, tm_mday, tm_hour, tm_min, tm_sec;
-    char    tm_zulu;
+    int     exp_year, exp_mon, exp_mday, exp_hour, exp_min, exp_sec;
+    char    exp_zulu;
 
     assert(expires);
 
-    if (sscanf(expires, "%d-%d-%dT%d:%d:%d%c", &tm_year, &tm_mon, &tm_mday,
-            &tm_hour, &tm_min, &tm_sec, &tm_zulu) != 7  ||
-        tm_year < 2017  ||
-        (tm_mon < 0  || tm_mon > 11)  ||
-        (tm_mday < 1  || tm_mday > 31)  ||
-        (tm_hour < 0  || tm_hour > 23)  ||
-        (tm_min < 0  || tm_min > 59)  ||
-        (tm_sec < 0  || tm_sec > 60)  ||
-        tm_zulu != 'Z')
+    if (sscanf(expires, "%d-%d-%dT%d:%d:%d%c", &exp_year, &exp_mon, &exp_mday,
+            &exp_hour, &exp_min, &exp_sec, &exp_zulu) != 7  ||
+        (exp_year < 2017  ||  exp_year > 9999)  ||
+        (exp_mon  < 1     ||  exp_mon  > 12)    ||
+        (exp_mday < 1     ||  exp_mday > 31)    ||
+        (exp_hour < 0     ||  exp_hour > 23)    ||
+        (exp_min  < 0     ||  exp_min  > 59)    ||
+        (exp_sec  < 0     ||  exp_sec  > 60)    ||  /* 60 for leap second */
+        exp_zulu != 'Z')
     {
         CORE_LOGF_X(eNSub_Json, eLOG_Error,
             ("Unexpected JSON {\"addrs[i].meta.expires\"} "
              "value '%s'.", expires));
         return 0;
     }
-    tm_year -= 1900;    /* years since 1900 */
-    tm_mon  -= 1;       /* months since January: 0-11 */
 
     /* Get the UTC epoch time for the expires value. */
     struct tm   tm_expires;
-    tm_expires.tm_year  = tm_year;
-    tm_expires.tm_mon   = tm_mon;
-    tm_expires.tm_mday  = tm_mday;
-    tm_expires.tm_hour  = tm_hour;
-    tm_expires.tm_min   = tm_min;
-    tm_expires.tm_sec   = tm_sec;
+    tm_expires.tm_year  = exp_year - 1900;    /* years since 1900 */
+    tm_expires.tm_mon   = exp_mon - 1;        /* months since January: 0-11 */
+    tm_expires.tm_mday  = exp_mday;
+    tm_expires.tm_hour  = exp_hour;
+    tm_expires.tm_min   = exp_min;
+    tm_expires.tm_sec   = exp_sec;
     tm_expires.tm_isdst = 0;
     tm_expires.tm_wday  = 0;
     tm_expires.tm_yday  = 0;
@@ -604,9 +630,49 @@ static TNCBI_Time s_ParseExpires(time_t tt_now, const char* expires)
     CORE_UNLOCK;
     tdiff = difftime(mktime(&tm_expires), mktime(&tm_now));
     if (tdiff < -14.0 * 3600.0  ||  tdiff > 12.0 * 3600.0) {
+        int     now_year = tm_now.tm_year + 1900;
+        int     now_mon  = tm_now.tm_mon + 1;
+        int     now_mday = tm_now.tm_mday;
+        int     now_hour = tm_now.tm_hour;
+        int     now_min  = tm_now.tm_min;
+        int     now_sec  = tm_now.tm_sec;
+        char    now_str[32]; /* need 21 for 'yyyy-mm-ddThh:mm:ssZ' */
+        double  td_diff, td_hour, td_min, td_sec;
+        const char* td_sign;
+
+        if ((now_year < 2017  ||  now_year > 9999)  ||
+            (now_mon  < 1     ||  now_mon  > 12)    ||
+            (now_mday < 1     ||  now_mday > 31)    ||
+            (now_hour < 0     ||  now_hour > 23)    ||
+            (now_min  < 0     ||  now_min  > 59)    ||
+            (now_sec  < 0     ||  now_sec  > 60))  /* 60 for leap second */
+        {
+            CORE_LOGF_X(eNSub_Json, eLOG_Error,
+                ("Invalid 'struct tm' for current time (adjusted to UTC): {"
+                 "tm_year = %d, tm_mon = %d, tm_mday = %d, "
+                 "tm_hour = %d, tm_min = %d, tm_sec = %d, }.",
+                 tm_now.tm_year + 1900, tm_now.tm_mon + 1, tm_now.tm_mday,
+                 tm_now.tm_hour, tm_now.tm_min, tm_now.tm_sec));
+            return 0;
+        }
+        if (tdiff < 0.0) {
+            td_sign = "-";
+            td_diff = -tdiff;
+        } else {
+            td_sign = "";
+            td_diff = tdiff;
+        }
+        td_hour  = td_diff / 3600.0;
+        td_diff -= td_hour * 3600.0;
+        td_min   = td_diff / 60.0;
+        td_sec   = td_diff - td_min * 60.0;
+        sprintf(now_str, "%d-%02d-%02dT%02d:%02d:%02dZ",
+                now_year, now_mon, now_mday, now_hour, now_min, now_sec);
         CORE_LOGF_X(eNSub_Json, eLOG_Error,
             ("Unexpected JSON {\"addrs[i].meta.expires\"} "
-             "value '%s' - tdiff = %lf", expires, tdiff));
+             "value '%s' - excessive difference (%.0lfs = %s%.0lf:%.0lf:%.0lf) "
+             "from current time '%s'",
+             expires, tdiff, td_sign, td_hour, td_min, td_sec, now_str));
         return 0;
     }
     TNCBI_Time  timeval = (TNCBI_Time)((double)tt_now + tdiff);
@@ -633,7 +699,7 @@ static EIO_Status s_ReadFullResponse(CONN conn, char** bufp,
     int             max_steps = sizeof(buf_len_steps)/sizeof(buf_len_steps[0]);
     size_t          waste_threshold = 50100;
     size_t          total_got = 0;
-    size_t          buf_len, step_max, step_got;
+    size_t          buf_len = 0, step_max = 0, step_got = 0;
     char*           new_buf;
     int             num_steps;
     EIO_Status      status = eIO_Unknown;
@@ -872,12 +938,18 @@ static int/*bool*/ s_ParseResponse(SERV_ITER iter, CONN conn)
 
             /* Make sure the service type matches an allowed type. */
             ESERV_Type  type;
-            if ( ! SERV_ReadType(svc_type, &type)  ||
-                 ! (iter->types & type))
+            if ( ! SERV_ReadType(svc_type, &type))
+            {
+                CORE_LOGF_X(eNSub_Json, eLOG_Error,
+                    ("Unrecognized service type '%s'.", svc_type));
+                goto out;
+            }
+            if (iter->types != fSERV_Any  &&  !(iter->types & type))
             {
                 CORE_TRACEF((
-                    "Ignoring endpoint %s:%d with unallowed svc_type '%s'.",
-                    host, port, svc_type));
+                    "Ignoring endpoint %s:%d with unallowed svc_type '%s'"
+                    " - allowed types = 0x%lx.",
+                    host, port, svc_type, (unsigned long)iter->types));
                 continue;
             }
             CORE_TRACEF((
@@ -903,6 +975,14 @@ static int/*bool*/ s_ParseResponse(SERV_ITER iter, CONN conn)
                 stateful = "";
             }
 
+            /* Make sure the JSON is stateless for a stateless SERV_ITER. */
+            if ((iter->types & fSERV_Stateless)  &&  stateful[0] != '\0') {
+                CORE_TRACEF((
+                    "Ignoring stateful endpoint %s:%d in stateless search.",
+                    host, port));
+                continue;
+            }
+
             /* SSERV_Info.site <=== addrs[i].meta.mode */
             local = "NO";
             priv = "";
@@ -934,10 +1014,10 @@ static int/*bool*/ s_ParseResponse(SERV_ITER iter, CONN conn)
             {
                 rate = x_json_object_dotget_number(address, "meta.rate");
             } else {
-                rate = 0.0;
+                rate = LBSM_DEFAULT_RATE;
             }
             /* verify magnitude */
-            if (rate < 0.0  ||  rate > 10000.0) {
+            if (rate < SERV_MINIMAL_RATE  ||  rate > SERV_MAXIMAL_RATE) {
                 CORE_LOGF_X(eNSub_Json, eLOG_Error,
                     ("Unexpected JSON {\"addrs[i].meta.rate\"} value '%lf'.",
                      rate));
@@ -1028,14 +1108,15 @@ static int/*bool*/ s_ParseResponse(SERV_ITER iter, CONN conn)
                 continue;
             }
             free(server_description);
+            CORE_TRACEF(("Created info: %p", info));
 
             /* Add new info to collection */
             if (s_AddServerInfo(data, info)) {
                 retval = 1; /* at least one info added */
-                /* Note: successful s_AddServerInfo() frees 'info' */
             } else {
                 CORE_LOG_X(eNSub_Alloc, eLOG_Critical,
                     "Unable to add server info.");
+                CORE_TRACEF(("Freeing info: %p", info));
                 free(info); /* not freed by failed s_AddServerInfo() */
                 goto out;
             }
@@ -1074,14 +1155,14 @@ static char* s_GetDtabHeaderFromBuf(const char* buf)
         while (*end  &&  *end != '\r'  &&  *end != '\n') ++end;
 
         /* clone the header value */
-        dup_hdr = (char*)malloc(end - start + 1);
+        dup_hdr = (char*)malloc((size_t)(end - start + 1));
         if ( ! dup_hdr) {
             CORE_LOG_X(eNSub_Alloc, eLOG_Critical,
                        "Couldn't alloc for dtab header value.");
             TOUT("s_GetDtabHeaderFromBuf() -- bad alloc");
             return NULL;
         }
-        memcpy(dup_hdr, start, end - start);
+        memcpy(dup_hdr, start, (size_t)(end - start));
         dup_hdr[end - start] = NIL;
         TOUT1("s_GetDtabHeaderFromBuf() -- got dtab header \"%s\"", dup_hdr);
         return dup_hdr;
@@ -1236,7 +1317,6 @@ static int/*bool*/ s_Resolve(SERV_ITER iter)
 
     TIN("s_Resolve()");
     assert( ! (data->eof | data->fail));
-    assert( !! net_info->stateless ==  !! iter->stateless);
 
     /* Handle DTAB, if present. */
     s_ProcessDtab(net_info);
@@ -1263,9 +1343,13 @@ static int/*bool*/ s_IsUpdateNeeded(TNCBI_Time now, struct SNAMERD_Data *data)
             *   there are no candidates; or
             *   any candidates are expired.
         */
-    int any_expired = 0, i;
+    int     any_expired = 0;
+    size_t  i;
 
-    for (i = (int)data->n_cand - 1;  i >= 0;  --i) {
+    /* Loop from highest index to lowest to ensure well-defined behavior when
+        candidates are removed and to avoid memmove in s_RemoveCand(). */
+    for (i = data->n_cand;  i > 0;  ) {
+        --i; /* decrement here to avoid unsigned underflow */
         const SSERV_Info* info = data->cand[i].info;
         if (info->time < now) {
 #if defined(_DEBUG)  &&  ! defined(NDEBUG)
@@ -1279,7 +1363,7 @@ static int/*bool*/ s_IsUpdateNeeded(TNCBI_Time now, struct SNAMERD_Data *data)
                 now - info->time, tnow - now));
 #endif
             any_expired = 1;
-            s_RemoveCand(data, (size_t)i, 1);
+            s_RemoveCand(data, i, 1);
         }
     }
 
@@ -1299,6 +1383,13 @@ static int/*bool*/ s_Update(SERV_ITER iter, const char* text, int code)
 }
 
 
+static SLB_Candidate* s_GetCandidate(void* user_data, size_t n)
+{
+    struct SNAMERD_Data* data = (struct SNAMERD_Data*) user_data;
+    return n < data->n_cand ? &data->cand[n] : 0;
+}
+
+
 static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
 {
     struct SNAMERD_Data* data = (struct SNAMERD_Data*) iter->data;
@@ -1310,19 +1401,23 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER iter, HOST_INFO* host_info)
 
     if (data->n_cand < 1  &&  data->done) {
         data->done = 0;
-        TOUT("s_GetNextInfo() -- no cand");
+        TOUT("s_GetNextInfo() -- end of candidates");
         return NULL;
     }
 
     if (s_IsUpdateNeeded(iter->time, data)) {
-        if ( ! (data->eof | data->fail))
+        if ( ! (data->eof | data->fail)) {
             s_Resolve(iter);
+            if (data->n_cand < 1) {
+                data->done = 1;
+                TOUT("s_GetNextInfo() -- resolved no candidates");
+                return NULL;
+            }
+        }
     }
 
-    /* No need to load-balance - that's already been done by the namerd service.
-     * Just take the first one.
-     */
-    n = 0;
+    /* Pick a randomized candidate. */
+    n = LB_Select(iter, data, s_GetCandidate, NAMERD_LOCAL_BONUS);
     info       = (SSERV_Info*) data->cand[n].info;
     info->rate =               data->cand[n].status;
 
@@ -1348,8 +1443,11 @@ static void s_Reset(SERV_ITER iter)
         if (data->cand) {
             size_t i;
             assert(data->n_cand <= data->a_cand);
-            for (i = 0;  i < data->n_cand;  ++i)
+            for (i = 0;  i < data->n_cand;  ++i) {
+                CORE_TRACEF(("Freeing available info " FMT_SIZE_T ": %p",
+                             i, data->cand[i].info));
                 free((void*) data->cand[i].info);
+            }
             data->n_cand = 0;
         }
     }
@@ -1364,11 +1462,8 @@ static void s_Close(SERV_ITER iter)
 
     TIN("s_Close()");
 
-    /* Make sure s_Reset() has been called. */
-    assert(data);
-    if (data->n_cand)
-        s_Reset(iter);
-    assert( ! data->n_cand); /* s_Reset() clears candidates count */
+    /* Make sure s_Reset() has been called - it frees info structs. */
+    s_Reset(iter);
 
     if (data->cand)
         free(data->cand);
@@ -1397,7 +1492,6 @@ extern const SSERV_VTable* SERV_NAMERD_Open(SERV_ITER           iter,
     s_Init();
 
     assert(iter);
-    assert(net_info);
 
     /* Check that service name is provided - otherwise there is nothing to
      * search for. */
@@ -1435,12 +1529,29 @@ extern const SSERV_VTable* SERV_NAMERD_Open(SERV_ITER           iter,
     }
     iter->data = data;
 
-    if ( ! (data->net_info = ConnNetInfo_Clone(net_info))) {
+    /* Make sure a net_info exists. */
+    SConnNetInfo* new_net_info = NULL;
+    if ( ! net_info) {
+        new_net_info = ConnNetInfo_Create(0);
+        if ( ! new_net_info) {
+            CORE_LOG_X(eNSub_Alloc, eLOG_Critical, "Couldn't create net_info.");
+            s_Close(iter);
+            TOUT("SERV_NAMERD_Open() -- fail");
+            return NULL;
+        }
+        data->net_info = ConnNetInfo_Clone(new_net_info);
+    } else {
+        data->net_info = ConnNetInfo_Clone(net_info);
+    }
+    if ( ! data->net_info) {
         CORE_LOG_X(eNSub_Alloc, eLOG_Critical, "Couldn't clone net_info.");
         s_Close(iter);
         TOUT("SERV_NAMERD_Open() -- fail");
         return NULL;
     }
+    if (new_net_info) {
+        ConnNetInfo_Destroy(new_net_info);
+    }
 
     if ( ! ConnNetInfo_SetupStandardArgs(data->net_info, iter->name)) {
         CORE_LOG_X(eNSub_BadData, eLOG_Critical,
@@ -1455,21 +1566,22 @@ extern const SSERV_VTable* SERV_NAMERD_Open(SERV_ITER           iter,
         srand(g_NCBI_ConnectRandomSeed);
     }
 
-    data->net_info->http_proxy_port = s_GetProxyPort();
-    data->net_info->req_method      = s_GetReqMethod();
-    data->net_info->scheme          = s_GetScheme();
-    data->net_info->port            = 0; /* namerd doesn't support a port */
-    data->net_info->host[0]         = NIL;
-    data->net_info->path[0]         = NIL;
-    data->net_info->args[0]         = NIL;
+    data->net_info->req_method  = s_GetReqMethod();
+    data->net_info->scheme      = s_GetScheme();
+    data->net_info->port        = 0; /* namerd doesn't support a port */
+    data->net_info->host[0]     = NIL;
+    data->net_info->path[0]     = NIL;
+    data->net_info->args[0]     = NIL;
 
-    if ( ! ConnNetInfo_GetValue(REG_NAMERD_SECTION,
-        REG_NAMERD_PROXY_HOST_KEY, data->net_info->http_proxy_host,
-        sizeof(data->net_info->http_proxy_host) - 1,
-        REG_NAMERD_PROXY_HOST_DEF))
+    if ( ! s_GetHttpProxy(data->net_info->http_proxy_host,
+        sizeof(data->net_info->http_proxy_host),
+        &data->net_info->http_proxy_port))
     {
-        data->net_info->http_proxy_host[0] = NIL;
+        s_Close(iter);
+        TOUT("SERV_NAMERD_Open() -- fail");
+        return NULL;
     }
+
     if ( ! ConnNetInfo_GetValue(REG_NAMERD_SECTION,
         REG_NAMERD_API_HOST_KEY, data->net_info->host, sizeof(net_info->host)-1,
         REG_NAMERD_API_HOST_DEF))
@@ -1502,19 +1614,18 @@ extern const SSERV_VTable* SERV_NAMERD_Open(SERV_ITER           iter,
         strcat(data->net_info->args, iter->name);
     }
 
-    if (iter->stateless)
+    if (iter->types & fSERV_Stateless)
         data->net_info->stateless = 1/*true*/;
-    if ((iter->types & fSERV_Firewall)  &&   ! data->net_info->firewall)
+    if ((iter->types & fSERV_Firewall)  &&  !data->net_info->firewall)
         data->net_info->firewall = eFWMode_Adaptive;
 
     iter->op = &s_op; /*SERV_Update() [from HTTP callback] expects*/
     s_Resolve(iter);
     iter->op = NULL;
 
-    if ( ! data->n_cand  &&  (data->fail  ||
-                             ! (data->net_info->stateless  &&
-                              data->net_info->firewall)))
-    {
+    if (!data->n_cand  &&  (data->fail
+                            ||  !(data->net_info->stateless  &&
+                                  data->net_info->firewall))) {
         s_Close(iter);
         TOUT("SERV_NAMERD_Open() -- fail");
         return NULL;
