@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_PRIV__H
 #define CONNECT___NCBI_PRIV__H
 
-/* $Id: ncbi_priv.h 537410 2017-05-31 16:20:20Z lavr $
+/* $Id: ncbi_priv.h 568534 2018-08-07 14:23:31Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,13 +32,13 @@
  *    Private aux. code for the "ncbi_*.[ch]"
  *
  *********************************
- * Tracing and logging
+ * Error handling and logging (including tracing):
  *    C error codes:   NCBI_C_DEFINE_ERRCODE_X, NCBI_C_ERRCODE_X
  *    private global:  g_CORE_Log
  *    macros:          CORE_TRACE[F], CORE_LOG[F][_[EX]X],
  *                     CORE_LOG[F]_ERRNO[_[EX]X](), CORE_DATA[F][_[EX]X]
- * Critical section (basic multi-thread synchronization)
- *    private global:  g_CORE_MT_Lock
+ * Multi-thread safety (critical section basic MT synchronization):
+ *    private globals: g_CORE_MT_Lock[_default]
  *    macros:          CORE_LOCK_WRITE, CORE_LOCK_READ, CORE_UNLOCK
  * Registry:
  *    private global:  g_CORE_Registry
@@ -96,17 +96,18 @@ extern "C" {
 NCBI_C_DEFINE_ERRCODE_X(Connect_Conn,     301,  36);
 NCBI_C_DEFINE_ERRCODE_X(Connect_Socket,   302, 163);
 NCBI_C_DEFINE_ERRCODE_X(Connect_Util,     303,   9);
-NCBI_C_DEFINE_ERRCODE_X(Connect_LBSM,     304,  33);
+NCBI_C_DEFINE_ERRCODE_X(Connect_LBSM,     304,  30);
 NCBI_C_DEFINE_ERRCODE_X(Connect_FTP,      305,  13);
 NCBI_C_DEFINE_ERRCODE_X(Connect_SMTP,     306,  33);
 NCBI_C_DEFINE_ERRCODE_X(Connect_HTTP,     307,  24);
-NCBI_C_DEFINE_ERRCODE_X(Connect_Service,  308,  11);
+NCBI_C_DEFINE_ERRCODE_X(Connect_Service,  308,  10);
 NCBI_C_DEFINE_ERRCODE_X(Connect_HeapMgr,  309,  33);
 NCBI_C_DEFINE_ERRCODE_X(Connect_LBOS,     310, 600); /*safe upper bound*/
 NCBI_C_DEFINE_ERRCODE_X(Connect_Mghbn,    311,  16);
 NCBI_C_DEFINE_ERRCODE_X(Connect_Crypt,    312,   5);
 NCBI_C_DEFINE_ERRCODE_X(Connect_LocalIP,  313,   5);
 NCBI_C_DEFINE_ERRCODE_X(Connect_Namerd,   314,   9);
+NCBI_C_DEFINE_ERRCODE_X(Connect_Linkerd,  315,   5);
 
 /** Make one identifier from 2 parts */
 #define NCBI_C_CONCAT_IDENTIFIER(prefix, postfix)  prefix##postfix
@@ -249,12 +250,12 @@ extern NCBI_XCONNECT_EXPORT LOG g_CORE_Log;
 extern NCBI_XCONNECT_EXPORT const char* g_CORE_Sprintf(const char* fmt, ...)
 #ifdef __GNUC__
     __attribute__((format(printf,1,2)))
-#endif
+#endif /*__GNUC__*/
 ;
 
 
 /******************************************************************************
- *  Multi-Thread SAFETY
+ *  Multi-thread safety
  */
 
 extern struct MT_LOCK_tag g_CORE_MT_Lock_default;
@@ -266,12 +267,12 @@ extern NCBI_XCONNECT_EXPORT MT_LOCK g_CORE_MT_Lock;
  * do not access/change it directly!
  */
 
-#define CORE_LOCK_WRITE  verify(CORE_CHECK_LOCK  &&                     \
-                                MT_LOCK_Do(g_CORE_MT_Lock, eMT_Lock    ))
-#define CORE_LOCK_READ   verify(CORE_CHECK_LOCK  &&                     \
-                                MT_LOCK_Do(g_CORE_MT_Lock, eMT_LockRead))
-#define CORE_UNLOCK      verify(CORE_CHECK_UNLOCK  &&                   \
-                                MT_LOCK_Do(g_CORE_MT_Lock, eMT_Unlock  ))
+#define CORE_LOCK_WRITE  verify(CORE_CHECK_LOCK   != 0  &&              \
+                                MT_LOCK_Do(g_CORE_MT_Lock, eMT_Lock    ) != 0)
+#define CORE_LOCK_READ   verify(CORE_CHECK_LOCK   != 0  &&              \
+                                MT_LOCK_Do(g_CORE_MT_Lock, eMT_LockRead) != 0)
+#define CORE_UNLOCK      verify(CORE_CHECK_UNLOCK != 0  &&              \
+                                MT_LOCK_Do(g_CORE_MT_Lock, eMT_Unlock  ) != 0)
 
 #ifdef _DEBUG
 extern NCBI_XCONNECT_EXPORT int g_NCBI_CoreCheckLock  (void);
@@ -325,8 +326,8 @@ extern NCBI_XCONNECT_EXPORT int/*bool*/ g_CORE_RegistrySET
 
 typedef enum {
     eCORE_SetSSL  = 1,
-    eCORE_SetREG  = 4,
     eCORE_SetLOG  = 2,
+    eCORE_SetREG  = 4,
     eCORE_SetLOCK = 8
 } ECORE_Set;
 typedef unsigned int TCORE_Set;
@@ -339,9 +340,9 @@ extern TCORE_Set g_CORE_Set;
  *  Random generator seeding support
  */
 
-extern NCBI_XCONNECT_EXPORT int    g_NCBI_ConnectRandomSeed;
-extern NCBI_XCONNECT_EXPORT int    g_NCBI_ConnectSrandAddend(void);
-#define NCBI_CONNECT_SRAND_ADDEND  g_NCBI_ConnectSrandAddend()
+extern NCBI_XCONNECT_EXPORT unsigned int g_NCBI_ConnectRandomSeed;
+extern NCBI_XCONNECT_EXPORT unsigned int g_NCBI_ConnectSrandAddend(void);
+#define NCBI_CONNECT_SRAND_ADDEND        g_NCBI_ConnectSrandAddend()
 
 
 /******************************************************************************
