@@ -1,4 +1,4 @@
-/* $Id: ncbi_service.c 533605 2017-04-18 15:12:53Z mcelhany $
+/* $Id: ncbi_service.c 565391 2018-06-11 18:09:29Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -36,6 +36,7 @@
 #include "ncbi_local.h"
 #ifdef NCBI_CXX_TOOLKIT
 #  include "ncbi_lbosp.h"
+#  include "ncbi_linkerd.h"
 #endif
 #include "ncbi_namerd.h"
 #include "ncbi_priv.h"
@@ -179,14 +180,14 @@ static int/*bool*/ s_IsMapperConfigured(const char* service, const char* key)
 {
     char val[32];
     if (s_Fast)
-        return 0;
+        return 0/*false*/;
     ConnNetInfo_GetValue(service, key, val, sizeof(val), 0);
     return ConnNetInfo_Boolean(val);
 }
 
 
 static SERV_ITER x_Open(const char*         service,
-                        unsigned/*bool*/    ismask,
+                        int/*bool*/         ismask,
                         TSERV_Type          types,
                         unsigned int        preferred_host,
                         unsigned short      preferred_port,
@@ -194,7 +195,7 @@ static SERV_ITER x_Open(const char*         service,
                         const SConnNetInfo* net_info,
                         SSERV_InfoCPtr      skip[],
                         size_t              n_skip,
-                        unsigned/*bool*/    external,
+                        int/*bool*/         external,
                         const char*         arg,
                         const char*         val,
                         SSERV_Info**        info,
@@ -203,6 +204,7 @@ static SERV_ITER x_Open(const char*         service,
     int/*bool*/
         do_lbsmd = -1/*unassigned*/,
 #ifdef NCBI_CXX_TOOLKIT
+        do_linkerd = -1/*unassigned*/,
         do_namerd = -1/*unassigned*/,
         do_lbos  = -1/*unassigned*/,
 #endif /*NCBI_CXX_TOOLKIT*/
@@ -229,19 +231,21 @@ static SERV_ITER x_Open(const char*         service,
                                :  0.01 * (preference > 100.0
                                           ? 100.0
                                           : preference));
-    iter->types             = types;
+    iter->types             = (TSERV_TypeOnly) types;
     if (ismask)
         iter->ismask        = 1;
     if (types & fSERV_IncludeDown)
         iter->ok_down       = 1;
+    if (types & fSERV_IncludeStandby)
+        iter->ok_standby    = 1;
+    if (types & fSERV_IncludePrivate)
+        iter->ok_private    = 1;
     if (types & fSERV_IncludeReserved)
         iter->ok_reserved   = 1;
     if (types & fSERV_IncludeSuppressed)
         iter->ok_suppressed = 1;
     if (types & fSERV_ReverseDns)
         iter->reverse_dns   = 1;
-    if (types & fSERV_Stateless)
-        iter->stateless     = 1;
     iter->external          = external ? 1 : 0;
     if (arg  &&  *arg) {
         iter->arg           = arg;
@@ -255,7 +259,7 @@ static SERV_ITER x_Open(const char*         service,
 
     if (n_skip) {
         size_t i;
-        for (i = 0;  i < n_skip;  i++) {
+        for (i = 0;  i < n_skip;  ++i) {
             const char* name = (iter->ismask  ||  skip[i]->type == fSERV_Dns
                                 ? SERV_NameOfInfo(skip[i]) : "");
             SSERV_Info* temp = SERV_CopyInfoEx(skip[i],
@@ -283,7 +287,7 @@ static SERV_ITER x_Open(const char*         service,
         if (net_info->firewall)
             iter->types |= fSERV_Firewall;
         if (net_info->stateless)
-            iter->stateless = 1;
+            iter->types |= fSERV_Stateless;
         if (net_info->lb_disable)
             do_lbsmd = 0/*false*/;
     } else
@@ -302,6 +306,9 @@ static SERV_ITER x_Open(const char*         service,
                                    (service, REG_CONN_DISPD_DISABLE)))
 #ifdef NCBI_CXX_TOOLKIT
                                 &&
+                                !(do_linkerd = s_IsMapperConfigured
+                                  (service, REG_CONN_LINKERD_ENABLE))
+                                &&
                                 !(do_namerd = s_IsMapperConfigured
                                   (service, REG_CONN_NAMERD_ENABLE))
                                 &&
@@ -312,6 +319,11 @@ static SERV_ITER x_Open(const char*         service,
 
 #ifdef NCBI_CXX_TOOLKIT
         &&
+        (!do_linkerd                                                         ||
+         (do_linkerd < 0  &&  !(do_linkerd = s_IsMapperConfigured
+                                (service, REG_CONN_LINKERD_ENABLE)))         ||
+         !(op = SERV_LINKERD_Open(iter, net_info, info)))
+        &&
         (!do_namerd                                                          ||
          (do_namerd < 0  &&  !(do_namerd = s_IsMapperConfigured
                                (service, REG_CONN_NAMERD_ENABLE)))           ||
@@ -323,13 +335,13 @@ static SERV_ITER x_Open(const char*         service,
          !(op = SERV_LBOS_Open(iter, net_info, info)))
 #endif /*NCBI_CXX_TOOLKIT*/
 
-        && 
+        &&
         (!do_dispd                                                           ||
          (do_dispd < 0  &&  !(do_dispd = !s_IsMapperConfigured
                               (service, REG_CONN_DISPD_DISABLE)))            ||
          !(op = SERV_DISPD_Open(iter, net_info, info, host_info)))) {
 #ifdef NCBI_CXX_TOOLKIT
-        if (!do_namerd  &&  !do_lbsmd  &&  !do_dispd) {
+        if (!do_linkerd  &&  !do_namerd  &&  !do_lbsmd  &&  !do_dispd) {
 #else
         if (!do_lbsmd  &&  !do_dispd) {
 #endif /*NCBI_CXX_TOOLKIT*/
@@ -370,6 +382,178 @@ static void s_SkipSkip(SERV_ITER iter)
 }
 
 
+#if defined(_DEBUG)  &&  !defined(NDEBUG)
+
+#define RETURN(retval)  return x_Return(str, retval)
+
+
+#  ifdef __GNUC__
+inline
+#  endif /*__GNUC__*/
+static int/*bool*/ x_Return(const char* str, int/*bool*/ retval)
+{
+    if (str)
+        free((void*) str);
+    return retval;
+}
+
+
+/* Do some basic set of consistency checks for the returned server info */
+static int/*bool*/ x_ConsistencyCheck(SERV_ITER iter, const SSERV_Info* info)
+{
+    const char* str = SERV_WriteInfo(info);
+    TSERV_TypeOnly types;
+
+    if (!info->host  ||  !info->port) {
+        if (info->type != fSERV_Dns) {
+            CORE_LOGF(eLOG_Critical,
+                      ("[%s]  Non-DNS server with empty %s:\n%s", iter->name,
+                       !(info->host | info->port) ? "host:port"
+                       : info->host ? "port" : "host",
+                       str ? str : "<NULL>"));
+            return x_Return(str, 0/*failure*/);
+        }
+        if (!info->host  &&  (iter->last  ||   iter->ismask)) {
+            CORE_LOGF(eLOG_Critical,
+                      ("[%s]  Interim DNS server w/o host:\n%s", iter->name,
+                       str ? str : "<NULL>"));
+            RETURN(0/*failure*/);
+        }
+    }
+
+    if (info->type == fSERV_Firewall) {
+        if (info->u.firewall.type == fSERV_Dns) {
+            CORE_LOGF(eLOG_Critical,
+                      ("[%s]  Firewall DNS entry not allowed:\n%s", iter->name,
+                       str ? str : "<NULL>"));
+            RETURN(0/*failure*/);
+        }
+        if (info->vhost  ||  info->extra) {
+            CORE_LOGF(eLOG_Critical,
+                      ("[%s]  Firewall entry with extra(s):\n%s", iter->name,
+                       str ? str : "<NULL>"));
+            RETURN(0/*failure*/);
+        }
+    }
+    if (info->type == fSERV_Dns) {
+        if (info->site & fSERV_Private) {
+            CORE_LOGF(eLOG_Critical,
+                      ("[%s]  DNS entry cannot be private:\n%s", iter->name,
+                       str ? str : "<NULL>"));
+            RETURN(0/*failure*/);
+        }
+        if (info->mode & fSERV_Stateful) {
+            CORE_LOGF(eLOG_Critical,
+                      ("[%s]  DNS entry cannot be stateful:\n%s", iter->name,
+                       str ? str : "<NULL>"));
+            RETURN(0/*failure*/);
+        }
+        if (info->mime_t != eMIME_T_Undefined  ||
+            info->mime_s != eMIME_Undefined    ||
+            info->mime_e != eENCOD_None) {
+            CORE_LOGF(eLOG_Critical,
+                      ("[%s]  DNS entry with have MIME type:\n%s", iter->name,
+                       str ? str : "<NULL>"));
+            RETURN(0/*failure*/);
+        }
+    }
+    if ((info->type & fSERV_Http)  &&  (info->mode & fSERV_Stateful)) {
+        CORE_LOGF(eLOG_Critical,
+                  ("[%s]  HTTP entry cannot be stateful:\n%s", iter->name,
+                   str ? str : "<NULL>"));
+        RETURN(0/*failure*/);
+    }
+
+    if (!(types = iter->types &
+          (TSERV_TypeOnly)(~(fSERV_Stateless | fSERV_Firewall)))) {
+        if (info->type == fSERV_Dns  &&  !iter->reverse_dns) {
+            CORE_LOGF(eLOG_Critical,
+                      ("[%s]  DNS entry unwarranted:\n%s", iter->name,
+                       str ? str : "<NULL>"));
+            RETURN(0/*failure*/);
+        }
+    } else if ((info->type != fSERV_Firewall
+                &&  !(types & info->type))  ||
+               (info->type == fSERV_Firewall
+                &&  !(types & info->u.firewall.type))) {
+        if (info->type != fSERV_Dns  ||  !iter->reverse_dns) {
+            CORE_LOGF(eLOG_Critical,
+                      ("[%s]  Mismatched type 0x%X vs 0x%X:\n%s", iter->name,
+                       (int)(info->type == fSERV_Firewall
+                             ? info->u.firewall.type
+                             : info->type), (int) types,
+                       str ? str : "<NULL>"));
+            RETURN(0/*failure*/);
+        }
+    }
+
+    if (iter->external  &&  (info->site & (fSERV_Local | fSERV_Private))) {
+        CORE_LOGF(eLOG_Critical,
+                  ("[%s]  Local/private entry for external:\n%s", iter->name,
+                   str ? str : "<NULL>"));
+        RETURN(0/*failure*/);
+    }
+    if ((info->site & fSERV_Private)  &&  !iter->ok_private
+        &&  iter->localhost  &&  info->host != iter->localhost) {
+        CORE_LOGF(eLOG_Critical,
+                  ("[%s]  Private entry unwarranted:\n%s", iter->name,
+                   str ? str : "<NULL>"));
+        RETURN(0/*failure*/);
+    }
+    if ((iter->types & fSERV_Stateless)  &&  (info->mode & fSERV_Stateful)) {
+        CORE_LOGF(eLOG_Critical,
+                  ("[%s]  Steteful entry in stateless search:\n%s", iter->name,
+                   str ? str : "<NULL>"));
+        RETURN(0/*failure*/);
+    }
+
+    if (info->type != fSERV_Dns  ||  info->host) {
+        if (!info->time  &&  !info->rate) {
+            CORE_LOGF(eLOG_Critical,
+                      ("[%s]  Off entry returned:\n%s", iter->name,
+                       str ? str : "<NULL>"));
+            RETURN(0/*failure*/);
+        }
+        if (SERV_IsDown(info)  &&  !iter->ok_down) {
+            CORE_LOGF(eLOG_Critical,
+                      ("[%s]  Down entry unwarranted:\n%s", iter->name,
+                       str ? str : "<NULL>"));
+            RETURN(0/*failure*/);
+        }
+        if (SERV_IfSuppressed(info)  &&  !iter->ok_suppressed) {
+            CORE_LOGF(eLOG_Critical,
+                      ("[%s]  Suppressed entry unwarranted:\n%s", iter->name,
+                       str ? str : "<NULL>"));
+            RETURN(0/*failure*/);
+        }
+        if (SERV_IsReserved(info)  &&  !iter->ok_reserved) {
+            CORE_LOGF(eLOG_Critical,
+                      ("[%s]  Reserved entry unwarranted:\n%s", iter->name,
+                       str ? str : "<NULL>"));
+            RETURN(0/*failure*/);
+        }
+        if (SERV_IsStandby(info)  &&  !iter->ok_standby) {
+            size_t n;
+            for (n = 0;  n < iter->n_skip;  ++n) {
+                if (!SERV_IsStandby(iter->skip[n])) {
+                    CORE_LOGF(eLOG_Critical,
+                              ("[%s]  Standby entry unwarranted:\n%s",
+                               iter->name, str ? str : "<NULL>"));
+                    RETURN(0/*failure*/);
+                }
+            }
+        }
+    }
+
+    RETURN(1/*success*/);
+}
+
+
+#undef RETURN
+
+#endif /*_DEBUG && !NDEBUG*/
+
+
 static SSERV_Info* s_GetNextInfo(SERV_ITER   iter,
                                  HOST_INFO*  host_info,
                                  int/*bool*/ internal)
@@ -383,9 +567,16 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER   iter,
         }
         /* Obtain a fresh entry from the actual mapper */
         while ((info = iter->op->GetNextInfo(iter, host_info)) != 0) {
-            /* This should never actually be used for LBSMD dispatcher,
+            int/*bool*/ go;
+#if defined(_DEBUG)  &&  !defined(NDEBUG)
+            if (!x_ConsistencyCheck(iter, info)) {
+                free(info);
+                return 0;
+            }
+#endif /*_DEBUG && !NDEBUG*/
+            /* This should never be actually used for LBSMD dispatcher,
              * as all exclusion logic is already done in it internally. */
-            int/*bool*/ go =
+            go =
                 !info->host  ||  iter->pref >= 0.0  ||
                 !iter->host  ||  (iter->host == info->host  &&
                                   (!iter->port  ||  iter->port == info->port));
@@ -406,7 +597,7 @@ static SSERV_Info* s_GetNextInfo(SERV_ITER   iter,
 
 
 static SERV_ITER s_Open(const char*         service,
-                        unsigned/*bool*/    ismask,
+                        int/*bool*/         ismask,
                         TSERV_Type          types,
                         unsigned int        preferred_host,
                         unsigned short      preferred_port,
@@ -414,7 +605,7 @@ static SERV_ITER s_Open(const char*         service,
                         const SConnNetInfo* net_info,
                         SSERV_InfoCPtr      skip[],
                         size_t              n_skip,
-                        int /*bool*/        external,
+                        int/*bool*/         external,
                         const char*         arg,
                         const char*         val,
                         SSERV_Info**        info,
@@ -754,27 +945,28 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
     static const char kSkipInfo[] = "Skip-Info-%u: ";
     static const char kAffinity[] = "Affinity: ";
     char buffer[128], *str;
-    TBSERV_TypeOnly t;
     size_t buflen, i;
     BUF buf = 0;
 
     /* Put client version number */
-    buflen = sprintf(buffer, kClientRevision,
-                     SERV_CLIENT_REVISION_MAJOR, SERV_CLIENT_REVISION_MINOR);
+    buflen = (size_t) sprintf(buffer, kClientRevision,
+                              SERV_CLIENT_REVISION_MAJOR,
+                              SERV_CLIENT_REVISION_MINOR);
     assert(buflen < sizeof(buffer));
     if (!BUF_Write(&buf, buffer, buflen)) {
         BUF_Destroy(buf);
         return 0;
     }
     if (iter) {
-        assert(iter->op);
+        TSERV_TypeOnly t, types
+            = iter->types & (TSERV_TypeOnly)(~fSERV_Stateless);
         if (net_info  &&  !net_info->http_referer  &&  iter->op->mapper)
             s_SetDefaultReferer(net_info, iter);
         /* Accepted server types */
         buflen = sizeof(kAcceptedServerTypes) - 1;
         memcpy(buffer, kAcceptedServerTypes, buflen);
-        for (t = 1;  t;  t <<= 1) {
-            if (iter->types & t) {
+        for (t = 1;  t;  t = (TSERV_TypeOnly)(t << 1)) {
+            if (types & t) {
                 const char* name = SERV_TypeStr((ESERV_Type) t);
                 size_t namelen = strlen(name);
                 if (!namelen  ||  buflen + 1 + namelen + 2 >= sizeof(buffer))
@@ -782,7 +974,7 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
                 buffer[buflen++] = ' ';
                 memcpy(buffer + buflen, name, namelen);
                 buflen += namelen;
-            } else if (iter->types < t)
+            } else if (types < t)
                 break;
         }
         if (buffer[buflen - 1] != ':') {
@@ -800,7 +992,7 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
                 return 0;
             }
         }
-        if (iter->types & fSERV_Firewall) {
+        if (types & fSERV_Firewall) {
             /* Firewall */
             EFWMode mode
                 = net_info ? (EFWMode) net_info->firewall : eFWMode_Legacy;
@@ -852,7 +1044,7 @@ char* SERV_Print(SERV_ITER iter, SConnNetInfo* net_info, int/*bool*/ but_last)
                 buflen = sizeof(kUsedServerInfo) - 1;
                 memcpy(buffer, kUsedServerInfo, buflen);
             } else
-                buflen = sprintf(buffer, kSkipInfo, (unsigned) i + 1);
+                buflen = (size_t) sprintf(buffer, kSkipInfo, (unsigned) i + 1);
             assert(buflen < sizeof(buffer) - 1);
             if (!BUF_Write(&buf, buffer, buflen)                ||
                 (namelen  &&  !BUF_Write(&buf, name, namelen))  ||
