@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_SOCKETP__H
 #define CONNECT___NCBI_SOCKETP__H
 
-/* $Id: ncbi_socketp.h 488580 2016-01-01 00:39:21Z lavr $
+/* $Id: ncbi_socketp.h 563003 2018-05-01 01:21:26Z lavr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -292,8 +292,8 @@ typedef struct SOCK_tag {
     /* aux I/O data */
     BUF              r_buf;     /* read  buffer                              */
     BUF              w_buf;     /* write buffer                              */
-    size_t           r_len;     /* DSOCK: size of last message received      */
-    size_t           w_len;     /* SOCK: how much data is pending for output */
+    size_t           r_len;     /* SOCK: unused; DSOCK: sz of last msg recv'd*/
+    size_t           w_len;     /* SOCK: w_buf;  DSOCK: sz of last msg sent  */
 
     /* statistics */
     TNCBI_BigCount   n_read;    /* DSOCK: total # of bytes read (in all msgs)
@@ -390,6 +390,9 @@ const char* SOCK_StringToHostPortEx(const char*     str,
                                     int/*bool*/     flag);
 
 
+void SOCK_SetupSSLInternal(FSSLSetup setup, int/*bool*/ init);
+
+
 #ifdef __cplusplus
 } /* extern "C" */
 #endif /*__cplusplus*/
