@@ -1,4 +1,4 @@
-/* $Id: server.cpp 508624 2016-07-29 15:32:34Z satskyse $
+/* $Id: server.cpp 565441 2018-06-12 15:57:02Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -532,10 +532,6 @@ bool CServer_Connection::IsOpen(void)
 
 void CServer_Connection::OnSocketEvent(EServIO_Event event)
 {
-    if (event == (EServIO_Event) -1) {
-        m_Handler->OnTimer();
-        return;
-    }
     switch (event) {
     case eServIO_Open:
         m_Handler->OnOpen();
@@ -556,6 +552,9 @@ void CServer_Connection::OnSocketEvent(EServIO_Event event)
     case eServIO_Delete:
         delete this;
         break;
+    case eServIO_Alarm:
+        m_Handler->OnTimer();
+        break;
     default:
         if (eServIO_Read & event)
             m_Handler->OnRead();
@@ -740,7 +739,7 @@ void CServer::x_DoRun(void)
                 {
                     IServer_ConnectionBase* conn_base = *it;
                     CRef<CStdRequest> req(conn_base->CreateRequest(
-                                          (EServIO_Event)-1, *m_ConnectionPool, timeout));
+                                          eServIO_Alarm, *m_ConnectionPool, timeout));
                     m_ThreadPool->AcceptRequest(req);
                 }
             }
