@@ -1,4 +1,4 @@
-/*  $Id: json_over_uttp.cpp 502217 2016-05-23 15:32:11Z sadyrovr $
+/*  $Id: json_over_uttp.cpp 527718 2017-02-15 17:27:37Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -97,8 +97,7 @@ const char* CJsonException::GetErrCodeString() const
     }
 }
 
-typedef CRef<SJsonNodeImpl,
-    CNetComponentCounterLocker<SJsonNodeImpl> > TJsonNodeRef;
+typedef CNetRef<SJsonNodeImpl> TJsonNodeRef;
 
 struct SJsonObjectElement {
     SJsonObjectElement(const string& key, SJsonNodeImpl* node_impl) :
@@ -300,24 +299,34 @@ CJsonNode CJsonNode::NewArrayNode()
     return new SJsonArrayNodeImpl;
 }
 
-CJsonNode CJsonNode::NewStringNode(const string& value)
+CJsonNode::CJsonNode(const string& value)
+    : m_Impl(new SJsonStringNodeImpl(value))
 {
-    return new SJsonStringNodeImpl(value);
 }
 
-CJsonNode CJsonNode::NewIntegerNode(Int8 value)
+CJsonNode::CJsonNode(const char* value)
+    : m_Impl(new SJsonStringNodeImpl(value))
 {
-    return new SJsonFixedSizeNodeImpl(value);
 }
 
-CJsonNode CJsonNode::NewDoubleNode(double value)
+CJsonNode::CJsonNode(int value)
+    : CJsonNode(new SJsonFixedSizeNodeImpl(Int8(value)))
 {
-    return new SJsonFixedSizeNodeImpl(value);
 }
 
-CJsonNode CJsonNode::NewBooleanNode(bool value)
+CJsonNode::CJsonNode(Int8 value)
+    : CJsonNode(new SJsonFixedSizeNodeImpl(value))
+{
+}
+
+CJsonNode::CJsonNode(double value)
+    : CJsonNode(new SJsonFixedSizeNodeImpl(value))
+{
+}
+
+CJsonNode::CJsonNode(bool value)
+    : CJsonNode(new SJsonFixedSizeNodeImpl(value))
 {
-    return new SJsonFixedSizeNodeImpl(value);
 }
 
 CJsonNode CJsonNode::NewNullNode()
@@ -384,11 +393,38 @@ CJsonNode CJsonNode::GuessType(const CTempString& value)
             NewStringNode(value);
 }
 
+CJsonNode::TInstance s_CreateImpl(CJsonNode::ENodeType type)
+{
+    switch (type)
+    {
+    case CJsonNode::eObject:    return new SJsonObjectNodeImpl;
+    case CJsonNode::eArray:     return new SJsonArrayNodeImpl;
+    case CJsonNode::eString:    return new SJsonStringNodeImpl(string());
+    case CJsonNode::eInteger:   return new SJsonFixedSizeNodeImpl(Int8());
+    case CJsonNode::eDouble:    return new SJsonFixedSizeNodeImpl(double());
+    case CJsonNode::eBoolean:   return new SJsonFixedSizeNodeImpl(bool());
+    case CJsonNode::eNull:      return new SJsonFixedSizeNodeImpl();
+    }
+
+    NCBI_THROW_FMT(CJsonException, eInvalidNodeType, "Unknown type: " << type);
+    return nullptr; // Not reached
+}
+
+CJsonNode::CJsonNode(ENodeType type) :
+    m_Impl(s_CreateImpl(type))
+{
+}
+
 CJsonNode::ENodeType CJsonNode::GetNodeType() const
 {
     return m_Impl->m_NodeType;
 }
 
+string CJsonNode::GetTypeName() const
+{
+    return m_Impl->GetTypeName();
+}
+
 struct SJsonObjectKeyIterator : public SJsonIteratorImpl
 {
     SJsonObjectKeyIterator(SJsonObjectNodeImpl* container) :
@@ -402,8 +438,7 @@ struct SJsonObjectKeyIterator : public SJsonIteratorImpl
     virtual bool Next();
     virtual bool IsValid() const;
 
-    CRef<SJsonObjectNodeImpl,
-            CNetComponentCounterLocker<SJsonObjectNodeImpl> > m_Container;
+    CNetRef<SJsonObjectNodeImpl> m_Container;
     TJsonObjectElements::iterator m_Iterator;
 };
 
@@ -443,8 +478,7 @@ struct SJsonObjectElementOrderIterator : public SJsonIteratorImpl
     virtual bool Next();
     virtual bool IsValid() const;
 
-    CRef<SJsonObjectNodeImpl,
-            CNetComponentCounterLocker<SJsonObjectNodeImpl> > m_Container;
+    CNetRef<SJsonObjectNodeImpl> m_Container;
     TJsonObjectElementOrder::iterator m_Iterator;
 };
 
@@ -484,8 +518,7 @@ struct SJsonArrayIterator : public SJsonIteratorImpl
     virtual bool Next();
     virtual bool IsValid() const;
 
-    CRef<SJsonArrayNodeImpl,
-            CNetComponentCounterLocker<SJsonArrayNodeImpl> > m_Container;
+    CNetRef<SJsonArrayNodeImpl> m_Container;
     TJsonNodeVector::iterator m_Iterator;
 };
 
