@@ -1,4 +1,4 @@
-/*  $Id: netcache_api.cpp 535008 2017-05-04 16:02:37Z sadyrovr $
+/*  $Id: netcache_api.cpp 573297 2018-10-25 16:49:18Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -44,6 +44,7 @@
 
 #include <connect/services/srv_connections_expt.hpp>
 #include <connect/services/error_codes.hpp>
+#include <connect/services/impl/netcache_api_int.hpp>
 
 #include <connect/ncbi_conn_exception.hpp>
 
@@ -63,9 +64,9 @@
 BEGIN_NCBI_SCOPE
 
 static bool s_FallbackServer_Initialized = false;
-static CSafeStatic<auto_ptr<SServerAddress> > s_FallbackServer;
+static CSafeStatic<auto_ptr<CNetServer::SAddress> > s_FallbackServer;
 
-static SServerAddress* s_GetFallbackServer()
+static CNetServer::SAddress* s_GetFallbackServer()
 {
     if (s_FallbackServer_Initialized)
         return s_FallbackServer->get();
@@ -73,7 +74,7 @@ static SServerAddress* s_GetFallbackServer()
         string host, port;
         if (NStr::SplitInTwo(TCGI_NetCacheFallbackServer::GetDefault(),
                 ":", host, port)) {
-            s_FallbackServer->reset(new SServerAddress(host,
+            s_FallbackServer->reset(new CNetServer::SAddress(host,
                             (unsigned short) NStr::StringToInt(port)));
         }
     } catch (...) {
@@ -82,40 +83,20 @@ static SServerAddress* s_GetFallbackServer()
     return s_FallbackServer->get();
 }
 
-CRef<INetServerProperties> CNetCacheServerListener::AllocServerProperties()
+INetServerConnectionListener::TPropCreator CNetCacheServerListener::GetPropCreator() const
 {
-    return CRef<INetServerProperties>(new SNetCacheServerProperties);
+    return [] { return new SNetCacheServerProperties; };
 }
 
-void CNetCacheServerListener::OnPreInit(CObject* api_impl, ISynRegistry& registry, SRegSynonyms& sections, string& client_name)
+INetServerConnectionListener* CNetCacheServerListener::Clone()
 {
-    SNetCacheAPIImpl* nc_impl = static_cast<SNetCacheAPIImpl*>(api_impl);
-
-    _ASSERT(nc_impl);
-
-    if (CNetScheduleAPI api = nc_impl->m_NetScheduleAPI) {
-        // Use client name from NetSchedule if it's not provided for NetCache
-        if (client_name.empty()) {
-            client_name = api.GetExecutor().GetClientName();
-        }
-
-        CNetScheduleConfigLoader loader(registry, sections, false);
-
-        loader(api);
-    }
+    return new CNetCacheServerListener(*this);
 }
 
-void CNetCacheServerListener::OnInit(CObject* api_impl, ISynRegistry& registry, SRegSynonyms& sections)
+void SNetCacheAPIImpl::Init(CSynRegistry& registry, const SRegSynonyms& sections)
 {
-    SNetCacheAPIImpl* nc_impl = static_cast<SNetCacheAPIImpl*>(api_impl);
+    GetListener()->SetAuthString(m_Service->MakeAuthString());
 
-    m_Auth = nc_impl->m_Service->MakeAuthString();
-
-    nc_impl->Init(registry, sections);
-}
-
-void SNetCacheAPIImpl::Init(ISynRegistry& registry, const SRegSynonyms& sections)
-{
     if (m_Service->GetClientName().length() < 3) {
         NCBI_THROW(CNetCacheException,
             eAuthenticationError, "Client name is too short or empty");
@@ -155,8 +136,7 @@ void SNetCacheAPIImpl::Init(ISynRegistry& registry, const SRegSynonyms& sections
 
 void CNetCacheServerListener::OnConnected(CNetServerConnection& connection)
 {
-    CRef<SNetCacheServerProperties> server_props(
-            x_GetServerProperties(connection->m_Server));
+    auto server_props = connection->m_Server->Get<SNetCacheServerProperties>();
 
     CFastMutexGuard guard(server_props->m_Mutex);
 
@@ -181,7 +161,7 @@ void CNetCacheServerListener::OnConnected(CNetServerConnection& connection)
     }
 }
 
-void CNetCacheServerListener::OnError(const string& err_msg, CNetServer& server)
+void CNetCacheServerListener::OnErrorImpl(const string& err_msg, CNetServer& server)
 {
     static const char s_BlobNotFoundMsg[] = "BLOB not found";
     if (NStr::strncmp(err_msg.c_str(), s_BlobNotFoundMsg,
@@ -208,50 +188,47 @@ void CNetCacheServerListener::OnError(const string& err_msg, CNetServer& server)
     CONNSERV_THROW_FMT(CNetCacheException, eServerError, server, err_msg);
 }
 
-void CNetCacheServerListener::OnWarning(const string& warn_msg,
+void CNetCacheServerListener::OnWarningImpl(const string& warn_msg,
         CNetServer& server)
 {
-    if (m_EventHandler)
-        m_EventHandler->OnWarning(warn_msg, server);
-    else {
         LOG_POST(Warning << "NetCache server at "
                 << server->m_ServerInPool->m_Address.AsString() <<
                 ": WARNING: " << warn_msg);
-    }
-}
-
-CRef<SNetCacheServerProperties> CNetCacheServerListener::x_GetServerProperties(
-        SNetServerImpl* server_impl)
-{
-    return CRef<SNetCacheServerProperties>(
-            static_cast<SNetCacheServerProperties*>(
-                    server_impl->m_ServerInPool->
-                            m_ServerProperties.GetPointerOrNull()));
 }
 
 const char* const kNetCacheAPIDriverName = "netcache_api";
 
-SNetCacheAPIImpl::SNetCacheAPIImpl(SConfigOrRegistry conf_or_reg, const string& section,
+SNetCacheAPIImpl::SNetCacheAPIImpl(CSynRegistryBuilder registry_builder, const string& section,
         const string& service, const string& client_name,
         CNetScheduleAPI::TInstance ns_api) :
-    m_Service(new SNetServiceImpl("NetCacheAPI", service, client_name,
-        new CNetCacheServerListener)),
     m_NetScheduleAPI(ns_api),
     m_DefaultParameters(eVoid)
 {
-    m_Service->Init(this, conf_or_reg, { section, kNetCacheAPIDriverName, "netcache_client", "netcache" });
+    SRegSynonyms sections{ section, kNetCacheAPIDriverName, "netcache_client", "netcache" };
+
+    string ns_client_name;
+
+    if (ns_api) {
+        ns_client_name = ns_api->m_Service->GetClientName();
+
+        CNetScheduleConfigLoader loader(registry_builder, sections, false);
+        loader(ns_api);
+    }
+
+    m_Service = SNetServiceImpl::Create("NetCacheAPI", service, client_name,
+            new CNetCacheServerListener,
+            registry_builder, sections, ns_client_name);
+    Init(registry_builder, sections);
 }
 
-SNetCacheAPIImpl::SNetCacheAPIImpl(const string& api_name, const string& service,
-        const string& client_name, INetServerConnectionListener* listener) :
-    m_Service(new SNetServiceImpl(api_name, service, client_name, listener)),
+SNetCacheAPIImpl::SNetCacheAPIImpl() :
     m_DefaultParameters(eVoid)
 {
 }
 
 SNetCacheAPIImpl::SNetCacheAPIImpl(SNetServerInPool* server,
         SNetCacheAPIImpl* parent) :
-    m_Service(new SNetServiceImpl(server, parent->m_Service)),
+    m_Service(SNetServiceImpl::Clone(server, parent->m_Service)),
     m_ServiceMap(parent->m_ServiceMap),
     m_TempDir(parent->m_TempDir),
     m_CacheInput(parent->m_CacheInput),
@@ -261,25 +238,24 @@ SNetCacheAPIImpl::SNetCacheAPIImpl(SNetServerInPool* server,
 {
 }
 
-void s_AppendClientIPSessionID(string* cmd, const CRequestContext& req)
+void SNetCacheAPIImpl::AppendClientIPSessionID(string* cmd, CRequestContext& req)
 {
-    cmd->append(" ip=\"");
-    cmd->append(req.GetClientIP());
-    cmd->append("\" sid=\"");
-    cmd->append(req.GetSessionID());
-    cmd->append(1, '\"');
-}
+    _ASSERT(cmd);
 
-void SNetCacheAPIImpl::AppendClientIPSessionID(string* cmd)
-{
-    s_AppendClientIPSessionID(cmd, CDiagContext::GetRequestContext());
+    // XXX: A workaround for IP always required
+    // TODO: Remove after all NetCache servers upgraded to include CXX-9580 and CXX-9720
+    if (!req.IsSetClientIP()) *cmd += " ip=\"\"";
+
+    g_AppendClientIPAndSessionID(*cmd, req);
 }
 
 void SNetCacheAPIImpl::AppendClientIPSessionIDPasswordAgeHitID(string* cmd,
         const CNetCacheAPIParameters* parameters)
 {
+    _ASSERT(cmd);
+
     CRequestContext& req = CDiagContext::GetRequestContext();
-    s_AppendClientIPSessionID(cmd, req);
+    AppendClientIPSessionID(cmd, req);
 
     string password(parameters->GetPassword());
     if (!password.empty()) {
@@ -294,10 +270,21 @@ void SNetCacheAPIImpl::AppendClientIPSessionIDPasswordAgeHitID(string* cmd,
         cmd->append(NStr::NumericToString(max_age));
     }
 
-    cmd->append(" ncbi_phid=\"");
-    cmd->append(m_UseNextSubHitID ?
-            req.GetNextSubHitID() : req.GetCurrentSubHitID());
-    cmd->append(1, '\"');
+    AppendHitID(cmd, req);
+}
+
+void SNetCacheAPIImpl::AppendHitID(string* cmd, CRequestContext& req)
+{
+    _ASSERT(cmd);
+
+    g_AppendHitID(*cmd, req, m_UseNextSubHitID);
+}
+
+void SNetCacheAPIImpl::AppendClientIPSessionIDHitID(string* cmd)
+{
+    CRequestContext& req = CDiagContext::GetRequestContext();
+    AppendClientIPSessionID(cmd, req);
+    AppendHitID(cmd, req);
 }
 
 string SNetCacheAPIImpl::MakeCmd(const char* cmd_base, const CNetCacheKey& key,
@@ -348,33 +335,32 @@ CNetServer::SExecResult SNetCacheAPIImpl::ExecMirrorAware(
     const CNetCacheKey& key, const string& cmd,
     bool multiline_output,
     const CNetCacheAPIParameters* parameters,
-    SNetServiceImpl::EServerErrorHandling error_handling,
-    INetServerConnectionListener* conn_listener)
+    SNetServiceImpl::EServerErrorHandling error_handling)
 {
-    const string& service_name(key.GetServiceName());
+    const string& key_service_name = key.GetServiceName();
 
-    bool service_name_is_defined = !key.GetServiceName().empty();
+    bool key_has_service_name = !key_service_name.empty();
 
     CNetService service(m_Service);
 
-    if (service_name_is_defined && service_name != service.GetServiceName()) {
+    if (key_has_service_name && key_service_name != service.GetServiceName()) {
         // NB: Configured service is always allowed
 
-        if (!m_ServiceMap.IsAllowed(service_name)) {
+        if (!m_ServiceMap.IsAllowed(key_service_name)) {
             NCBI_THROW_FMT(CNetCacheException, eAccessDenied, "Service " <<
-                    service_name << " is not in the allowed services");
+                    key_service_name << " is not in the allowed services");
         }
 
-        service = m_ServiceMap.GetServiceByName(service_name, m_Service);
+        service = m_ServiceMap.GetServiceByName(key_service_name, m_Service);
     }
 
-    bool key_is_mirrored = service_name_is_defined &&
+    bool mirroring_allowed =
             !key.GetFlag(CNetCacheKey::fNCKey_SingleServer) &&
             parameters->GetMirroringMode() != CNetCacheAPI::eMirroringDisabled;
 
     if (key.GetVersion() == 3) {
         /* Version 3 - no server address, only CRC32 of it: */
-        if (service.GetServiceType() != CNetService::eLoadBalancedService) {
+        if (!service.IsLoadBalanced()) {
             NCBI_THROW_FMT(CNetSrvConnException, eLBNameNotFound,
                 key.GetKey() << ": NetCache key version 3 "
                 "requires an LBSM service name.");
@@ -395,9 +381,8 @@ CNetServer::SExecResult SNetCacheAPIImpl::ExecMirrorAware(
                 // TODO: cache the calculated checksums to resolve
                 // them into host:port immediately.
 
-                if (!key_is_mirrored)
-                    return server.ExecWithRetry(cmd, multiline_output,
-                            conn_listener);
+                if (!mirroring_allowed)
+                    return server.ExecWithRetry(cmd, multiline_output);
 
                 CNetServer::SExecResult exec_result;
 
@@ -407,13 +392,18 @@ CNetServer::SExecResult SNetCacheAPIImpl::ExecMirrorAware(
                                         through this service */);
 
                 service->IterateUntilExecOK(cmd, multiline_output,
-                        exec_result, &mirror_traversal, error_handling,
-                        conn_listener);
+                        exec_result, &mirror_traversal, error_handling);
 
                 return exec_result;
             }
         }
 
+        if (mirroring_allowed) {
+            // Original server either is down or has a different port (thus crc32 mismatch),
+            // will use any server from this service
+            return service->FindServerAndExec(cmd, multiline_output);
+        }
+
         NCBI_THROW_FMT(CNetSrvConnException, eServerNotInService,
                 key.GetKey() << ": unable to find a NetCache server "
                 "by the checksum from this key.");
@@ -427,14 +417,14 @@ CNetServer::SExecResult SNetCacheAPIImpl::ExecMirrorAware(
         server_check = key.GetFlag(CNetCacheKey::fNCKey_NoServerCheck) ?
                 eOff : eOn;
 
-    if (key_is_mirrored) {
+    if (key_has_service_name && mirroring_allowed) {
         CNetServer::SExecResult exec_result;
 
         SNetCacheMirrorTraversal mirror_traversal(service,
                 primary_server, server_check);
 
         service->IterateUntilExecOK(cmd, multiline_output, exec_result,
-                &mirror_traversal, error_handling, conn_listener);
+                &mirror_traversal, error_handling);
 
         return exec_result;
     }
@@ -443,7 +433,7 @@ CNetServer::SExecResult SNetCacheAPIImpl::ExecMirrorAware(
     if (server_check != eOff && !service->IsInService(primary_server)) {
 
         // Service name is known, no need to check other services
-        if (service_name_is_defined) {
+        if (key_has_service_name) {
             NCBI_THROW_FMT(CNetSrvConnException, eServerNotInService,
                     key.GetKey() << ": NetCache server " <<
                     primary_server.GetServerAddress() << " could not be "
@@ -461,7 +451,7 @@ CNetServer::SExecResult SNetCacheAPIImpl::ExecMirrorAware(
 
     }
 
-    return primary_server.ExecWithRetry(cmd, multiline_output, conn_listener);
+    return primary_server.ExecWithRetry(cmd, multiline_output);
 }
 
 CNetCacheAPI::CNetCacheAPI(CNetCacheAPI::EAppRegistry /* use_app_reg */,
@@ -543,7 +533,7 @@ CNetServerConnection SNetCacheAPIImpl::InitiateWriteCmd(
         try {
             exec_result = m_Service.FindServerAndExec(cmd, false);
         } catch (CNetSrvConnException& e) {
-            SServerAddress* backup = s_GetFallbackServer();
+            CNetServer::SAddress* backup = s_GetFallbackServer();
 
             if (backup == NULL) {
                 LOG_POST(Info << "Fallback server address is not configured.");
@@ -594,8 +584,7 @@ CNetServerConnection SNetCacheAPIImpl::InitiateWriteCmd(
             case CNetCacheAPI::eMirroringEnabled:
                 break;
             default:
-                if (!CNetCacheServerListener::x_GetServerProperties(
-                        exec_result.conn->m_Server)->mirrored)
+                if (!exec_result.conn->m_Server->Get<SNetCacheServerProperties>()->mirrored)
                     key_flags |= CNetCacheKey::fNCKey_SingleServer;
             }
 
@@ -667,19 +656,10 @@ bool CNetCacheAPI::HasBlob(const string& blob_id,
 
     parameters.LoadNamedParameters(optional);
 
-    try {
-        return m_Impl->ExecMirrorAware(key,
-                m_Impl->MakeCmd("HASB ", key, &parameters),
-                false,
-                &parameters).response[0] == '1';
-    }
-    // Workaround for a bug in NC v6.6.1 (CXX-4095)
-    // TODO: Throw away after all NC servers are upgraded to v6.6.2+
-    catch (CNetCacheException& e) {
-        if (e.GetErrCode() == CNetCacheException::eBlobNotFound)
-            return false;
-        throw;
-    }
+    return m_Impl->ExecMirrorAware(key,
+            m_Impl->MakeCmd("HASB ", key, &parameters),
+            false,
+            &parameters).response[0] == '1';
 }
 
 
@@ -737,6 +717,8 @@ CNetServerMultilineCmdOutput CNetCacheAPI::GetBlobInfo(const string& blob_id,
 
     parameters.LoadNamedParameters(optional);
 
+    m_Impl->AppendClientIPSessionIDHitID(&cmd);
+
     CNetServerMultilineCmdOutput output(m_Impl->ExecMirrorAware(key,
             cmd, true, &parameters));
 
@@ -940,16 +922,11 @@ CNetService CNetCacheAPI::GetService()
     return m_Impl->m_Service;
 }
 
-CNetCacheAPI CNetCacheAPI::GetServer(CNetServer::TInstance server)
+CNetCacheAPIExt CNetCacheAPIExt::GetServer(CNetServer::TInstance server)
 {
     return new SNetCacheAPIImpl(server->m_ServerInPool, m_Impl);
 }
 
-void CNetCacheAPI::SetEventHandler(INetEventHandler* event_handler)
-{
-    m_Impl->m_Service->SetEventHandler(event_handler);
-}
-
 CCompoundIDPool CNetCacheAPI::GetCompoundIDPool()
 {
     return m_Impl->m_CompoundIDPool;
