@@ -1,4 +1,4 @@
-/*  $Id: netschedule_api_submitter.cpp 537390 2017-05-31 15:32:22Z sadyrovr $
+/*  $Id: netschedule_api_submitter.cpp 551837 2017-11-24 16:29:55Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -37,17 +37,9 @@
 #include <connect/services/netschedule_api.hpp>
 #include <connect/services/util.hpp>
 
-#include <connect/ncbi_conn_reader_writer.hpp>
-
-#include <util/transmissionrw.hpp>
-
 #include <stdio.h>
 #include <cmath>
 #include <array>
-#include <condition_variable>
-#include <deque>
-#include <mutex>
-#include <thread>
 
 BEGIN_NCBI_SCOPE
 
@@ -65,7 +57,7 @@ static void s_SerializeJob(string& cmd, const CNetScheduleNewJob& job,
     cmd.append(NStr::PrintableString(job.input));
     cmd.push_back('"');
 
-    if (udp_port != 0) {
+    if (udp_port && wait_time) {
         cmd.append(" port=");
         cmd.append(NStr::UIntToString(udp_port));
         cmd.append(" timeout=");
@@ -110,7 +102,7 @@ void SNetScheduleSubmitterImpl::AppendClientIPSessionIDHitID(string& cmd, const
     }
 
     m_UseNextSubHitID.ProperCommand();
-    g_AppendHitID(cmd, m_UseNextSubHitID ? req.GetNextSubHitID() : req.GetCurrentSubHitID());
+    g_AppendHitID(cmd, req, m_UseNextSubHitID);
 }
 
 string SNetScheduleSubmitterImpl::SubmitJobImpl(CNetScheduleNewJob& job,
@@ -379,191 +371,37 @@ void CNetScheduleNotificationHandler::SubmitJob(
     submitter->SubmitJobImpl(job, GetPort(), wait_time, server);
 }
 
-static const char* const s_JobStatusAttrNames[3] =
-        {"job_key", "job_status", "last_event_index"};
-
 bool CNetScheduleNotificationHandler::CheckJobStatusNotification(
         const string& job_id, CNetScheduleAPI::EJobStatus* job_status,
         int* last_event_index /*= NULL*/)
 {
-    string attr_values[3];
+    _ASSERT(job_status);
 
-    if (last_event_index == NULL)
-        g_ParseNSOutput(m_Message, s_JobStatusAttrNames, attr_values, 2);
-    else {
-        g_ParseNSOutput(m_Message, s_JobStatusAttrNames, attr_values, 3);
+    SNetScheduleOutputParser parser(m_Receiver.message);
 
-        if (!attr_values[2].empty())
-            *last_event_index = NStr::StringToInt(attr_values[2],
-                    NStr::fConvErr_NoThrow);
-    }
+    if (parser("job_key") != job_id) return false;
 
-    return (*job_status = CNetScheduleAPI::StringToStatus(attr_values[1])) !=
-            CNetScheduleAPI::eJobNotFound && attr_values[0] == job_id;
+    *job_status = CNetScheduleAPI::StringToStatus(parser("job_status"));
+
+    if (last_event_index) *last_event_index = NStr::StringToInt(parser("last_event_index"), NStr::fConvErr_NoThrow);
+
+    return *job_status != CNetScheduleAPI::eJobNotFound;
 }
 
-bool CNetScheduleNotificationHandler::CheckJobStatusNotification(CNetScheduleAPI ns_api, CNetScheduleJob& job,
-        time_t* job_exptime, CNetScheduleAPI::EJobStatus& job_status, pair<bool*, CNcbiOstream*> receiver)
+bool CNetScheduleNotificationHandler::GetJobDetailsIfCompleted(CNetScheduleAPI ns_api, CNetScheduleJob& job,
+        time_t* job_exptime, CNetScheduleAPI::EJobStatus& job_status)
 {
-    const char* const attr_names[] = {"job_key", "job_status", "worker_node_host", "worker_node_port"};
-    array<string, 4> attr_values;
-    const string& received_job_id = attr_values[0];
-    const string& received_job_status = attr_values[1];
-    const string& worker_node_host = attr_values[2];
-    const string& worker_node_port = attr_values[3];
+    SNetScheduleOutputParser parser(m_Receiver.message);
 
-    g_ParseNSOutput(m_Message, attr_names, attr_values.data(), attr_values.size());
+    if (parser("job_key") != job.job_id) return false;
 
-    if (received_job_id != job.job_id) return false;
+    job_status = CNetScheduleAPI::StringToStatus(parser("job_status"));
+    if (job_status <= CNetScheduleAPI::eRunning) return false;
 
-    const auto& timeout = ns_api->GetConfig().direct_output_connect_timeout;
-
-    if (ReadOutput(job_status, receiver, worker_node_host, worker_node_port, timeout)) return true;
-
-    switch (CNetScheduleAPI::StringToStatus(received_job_status)) {
-    case CNetScheduleAPI::eJobNotFound:
-    case CNetScheduleAPI::eRunning:
-    case CNetScheduleAPI::ePending:
-        return false;
-
-    default:
-        job_status = ns_api.GetJobDetails(job, job_exptime);
-        return true;
-    }
+    job_status = ns_api.GetJobDetails(job, job_exptime);
+    return job_status > CNetScheduleAPI::eRunning;
 }
 
-#ifdef NCBI_THREADS
-bool CNetScheduleNotificationHandler::ReadOutput(CNetScheduleAPI::EJobStatus& job_status,
-        pair<bool*, CNcbiOstream*> receiver, const string& worker_node_host, const string& worker_node_port,
-        const STimeout& timeout)
-{
-    if (!receiver.first || !receiver.second) return false;
-
-    if (worker_node_host.empty() || worker_node_port.empty()) {
-        *receiver.first = false;
-        return false;
-    }
-
-    const auto host = static_cast<unsigned>(stoul(worker_node_host));
-    const auto port = static_cast<unsigned short>(stoul(worker_node_port));
-    unique_ptr<CSocket> socket;
-
-    try {
-        // There is no CSocket::Connect(unsigned host...) and no CSocket::CSocket(const CSocket&),
-        // have to create on heap.
-        socket.reset(new CSocket(host, port, &timeout));
-    }
-    catch (...) {
-        return false;
-    }
-
-    mutex m;
-    condition_variable cv;
-    deque<char> d;
-    bool done = false;
-
-    auto reader = [&] {
-        CSocketReaderWriter socket_reader(socket.get());
-        CTransmissionReader reader(&socket_reader);
-        array<char, 1024 * 1024> buf;
-        bool eof = false;
-
-        while (!eof) {
-            char* data = buf.data();
-            size_t to_read = buf.size();
-
-            while (!eof && to_read) {
-                ERW_Result status = eRW_Error;
-                size_t bytes_read = 0;
-
-                try {
-                    status = reader.Read(data, to_read, &bytes_read);
-                }
-                catch (...) {
-                }
-
-                switch (status) {
-                    case eRW_Success:
-                        break;
-
-                    case eRW_Eof:
-                        eof = true;
-                        break;
-
-                    default:
-                        eof = true;
-                        bytes_read = 0;
-                        break;
-                }
-
-                data += bytes_read;
-                to_read -= bytes_read;
-            }
-
-            unique_lock<mutex> lock(m);
-            d.insert(d.end(), buf.cbegin(), buf.cbegin() + (buf.size() - to_read));
-
-            if (done) eof = true;
-
-            done = eof;
-            lock.unlock();
-            cv.notify_one();
-        }
-    };
-
-    auto writer = [&] {
-        array<char, 1024 * 1024> buf;
-        bool eof = false;
-
-        while (!eof) {
-            char* data = buf.data();
-            size_t to_write = 0;
-
-            {
-                unique_lock<mutex> lock(m);
-
-                if (!receiver.second->good()) {
-                    done = true;
-                    return;
-                }
-
-                cv.wait(lock, [&]{ return d.size() || done; });
-
-                to_write = min(buf.size(), d.size());
-
-                if (to_write) {
-                    copy_n(d.cbegin(), to_write, buf.begin());
-                    d.erase(d.begin(), d.begin() + to_write);
-                }
-
-                eof = done;
-            }
-
-            try {
-                receiver.second->write(data, to_write);
-            }
-            catch (...) {
-                receiver.second->setstate(ios_base::badbit);
-            }
-        }
-    };
-
-    thread t(reader);
-    writer();
-    t.join();
-
-    job_status = CNetScheduleAPI::eDone;
-    *receiver.first = true;
-    return true;
-}
-#else
-bool CNetScheduleNotificationHandler::ReadOutput(CNetScheduleAPI::EJobStatus&,
-        pair<bool*, CNcbiOstream*>, const string&, const string&, const STimeout&)
-{
-    return false;
-}
-#endif
-
 CNetScheduleAPI::EJobStatus
 CNetScheduleSubmitter::SubmitJobAndWait(CNetScheduleJob& job,
                                         unsigned       wait_time)
@@ -572,17 +410,16 @@ CNetScheduleSubmitter::SubmitJobAndWait(CNetScheduleJob& job,
 }
 
 CNetScheduleAPI::EJobStatus
-SNetScheduleSubmitterImpl::SubmitJobAndWait(CNetScheduleJob& job,
-        unsigned wait_time, time_t* job_exptime, pair<bool*, CNcbiOstream*> receiver)
+SNetScheduleSubmitterImpl::SubmitJobAndWait(CNetScheduleJob& job, unsigned wait_time, time_t* job_exptime)
 {
     CDeadline deadline(wait_time, 0);
 
     CNetScheduleNotificationHandler submit_job_handler;
 
-    submit_job_handler.SubmitJob(this, job, wait_time);
+    SubmitJobImpl(job, submit_job_handler.GetPort(), wait_time);
+    if (!wait_time) return CNetScheduleAPI::ePending;
 
-    return submit_job_handler.WaitForJobCompletion(job,
-            deadline, m_API, job_exptime, receiver);
+    return submit_job_handler.WaitForJobCompletion(job, deadline, m_API, job_exptime);
 }
 
 CNetScheduleAPI::EJobStatus
@@ -603,24 +440,21 @@ CNetScheduleNotificationHandler::WaitForJobCompletion(
         CNetScheduleJob& job,
         CDeadline& deadline,
         CNetScheduleAPI ns_api,
-        time_t* job_exptime,
-        pair<bool*, CNcbiOstream*> receiver)
+        time_t* job_exptime)
 {
     CNetScheduleAPI::EJobStatus status = CNetScheduleAPI::ePending;
 
     unsigned wait_sec = FORCED_SST_INTERVAL_SEC;
 
-    bool last_wait = false;
     for (;;) {
         CDeadline timeout(wait_sec, FORCED_SST_INTERVAL_NANOSEC);
 
-        if (!(timeout < deadline)) {
+        if (deadline < timeout) {
             timeout = deadline;
-            last_wait = true;
         }
 
         if (WaitForNotification(timeout)) {
-            if (CheckJobStatusNotification(ns_api, job, job_exptime, status, receiver)) return status;
+            if (GetJobDetailsIfCompleted(ns_api, job, job_exptime, status)) return status;
         } else {
             // The wait has timed out - query the server directly.
 
@@ -628,7 +462,7 @@ CNetScheduleNotificationHandler::WaitForJobCompletion(
             try {
                 status = ns_api.GetJobDetails(job, job_exptime);
                 if ((status != CNetScheduleAPI::eRunning &&
-                        status != CNetScheduleAPI::ePending) || last_wait)
+                        status != CNetScheduleAPI::ePending) || deadline.IsExpired())
                     return status;
                 // The job is still running - next time, wait one second
                 // longer before querying the server again.
@@ -639,7 +473,7 @@ CNetScheduleNotificationHandler::WaitForJobCompletion(
                 continue;
             }
             catch (CNetScheduleException& e) {
-                if (last_wait)
+                if (deadline.IsExpired())
                     throw;
                 switch (e.GetErrCode()) {
                 case CNetScheduleException::eJobNotFound:
@@ -649,7 +483,7 @@ CNetScheduleNotificationHandler::WaitForJobCompletion(
                 err_msg = e.GetMsg();
             }
             catch (CException& e) {
-                if (last_wait)
+                if (deadline.IsExpired())
                     throw;
                 err_msg = e.GetMsg();
             }
@@ -672,6 +506,19 @@ bool CNetScheduleNotificationHandler::RequestJobWatching(
         const CDeadline& deadline,
         CNetScheduleAPI::EJobStatus* job_status,
         int* last_event_index)
+{
+    _ASSERT(job_status);
+    _ASSERT(last_event_index);
+
+    tie(*job_status, *last_event_index, ignore) = RequestJobWatching(ns_api, job_id, deadline);
+
+    return *job_status != CNetScheduleAPI::eJobNotFound;
+}
+
+CNetScheduleNotificationHandler::TJobInfo CNetScheduleNotificationHandler::RequestJobWatching(
+        CNetScheduleAPI::TInstance ns_api,
+        const string& job_id,
+        const CDeadline& deadline)
 {
     double remaining_seconds = ceil(deadline.GetRemainingTime().GetAsDouble());
 
@@ -684,18 +531,15 @@ bool CNetScheduleNotificationHandler::RequestJobWatching(
 
     g_AppendClientIPSessionIDHitID(cmd);
 
-    m_Message = ns_api->GetServer(job_id).ExecWithRetry(cmd, false).response;
+    cmd += " need_progress_msg=1";
+    m_Receiver.message = ns_api->GetServer(job_id).ExecWithRetry(cmd, false).response;
 
-    string attr_values[2];
+    SNetScheduleOutputParser parser(m_Receiver.message);
+    const auto job_status = CNetScheduleAPI::StringToStatus(parser("job_status"));
+    const auto last_event_index = NStr::StringToInt(parser("last_event_index"), NStr::fConvErr_NoThrow);
+    const auto progress_message = parser("msg");
 
-    g_ParseNSOutput(m_Message, s_JobStatusAttrNames + 1, attr_values, 2);
-
-    if (!attr_values[1].empty())
-        *last_event_index = NStr::StringToInt(attr_values[1],
-                NStr::fConvErr_NoThrow);
-
-    return (*job_status = CNetScheduleAPI::StringToStatus(attr_values[0])) !=
-        CNetScheduleAPI::eJobNotFound;
+    return make_tuple(job_status, last_event_index, progress_message);
 }
 
 CNetScheduleAPI::EJobStatus
@@ -713,29 +557,30 @@ CNetScheduleNotificationHandler::WaitForJobEvent(
 
     unsigned wait_sec = FORCED_SST_INTERVAL_SEC;
 
-    bool last_wait = false;
     do {
-        CDeadline timeout(wait_sec++, FORCED_SST_INTERVAL_NANOSEC);
+        CDeadline timeout(wait_sec, FORCED_SST_INTERVAL_NANOSEC);
 
-        if (!(timeout < deadline)) {
+        if (deadline < timeout) {
             timeout = deadline;
-            last_wait = true;
         }
 
-        if (RequestJobWatching(ns_api, job_key, timeout, &job_status, &index) &&
+        tie(job_status, index, ignore) = RequestJobWatching(ns_api, job_key, timeout);
+
+        if ((job_status != CNetScheduleAPI::eJobNotFound) &&
                 ((status_mask & (1 << job_status)) != 0 ||
                 index > last_event_index))
             break;
 
-        if (deadline.GetRemainingTime().IsZero())
+        if (deadline.IsExpired())
             break;
 
-        if (WaitForNotification(timeout) &&
-                CheckJobStatusNotification(job_key, &job_status, &index) &&
+        if (!WaitForNotification(timeout))
+            ++wait_sec;
+        else if (CheckJobStatusNotification(job_key, &job_status, &index) &&
                 ((status_mask & (1 << job_status)) != 0 ||
                 index > last_event_index))
             break;
-    } while (!last_wait);
+    } while (!deadline.IsExpired());
 
     if (new_event_index) {
         *new_event_index = index;
