@@ -1,7 +1,7 @@
 #ifndef CONNECT_SERVICES__NETSTORAGE_RPC__HPP
 #define CONNECT_SERVICES__NETSTORAGE_RPC__HPP
 
-/*  $Id: netstorage_rpc.hpp 505978 2016-06-30 15:57:38Z sadyrovr $
+/*  $Id: netstorage_rpc.hpp 528688 2017-02-24 15:08:31Z sadyrovr $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,43 +43,24 @@ BEGIN_NCBI_SCOPE
 struct SNetStorageRPC : public SNetStorageImpl
 {
     SNetStorageRPC(const TConfig& config, TNetStorageFlags default_flags);
+    SNetStorageRPC(SNetServerInPool* server, SNetStorageRPC* parent);
 
-    virtual CNetStorageObject Create(TNetStorageFlags flags);
-    virtual CNetStorageObject Open(const string& object_loc);
-    virtual string Relocate(const string& object_loc, TNetStorageFlags flags,
-            TNetStorageProgressCb cb);
-    virtual bool Exists(const string& object_loc);
-    virtual ENetStorageRemoveResult Remove(const string& object_loc);
+    SNetStorageObjectImpl* Create(TNetStorageFlags flags) override;
+    SNetStorageObjectImpl* Open(const string& object_loc) override;
 
     CJsonNode Exchange(CNetService service,
             const CJsonNode& request,
             CNetServerConnection* conn = NULL,
             CNetServer::TInstance server_to_use = NULL) const;
 
-    static void x_SetStorageFlags(CJsonNode& node, TNetStorageFlags flags);
     CJsonNode MkStdRequest(const string& request_type) const;
     CJsonNode MkObjectRequest(const string& request_type,
             const string& object_loc) const;
     CJsonNode MkObjectRequest(const string& request_type,
             const string& unique_key, TNetStorageFlags flags) const;
-    void x_InitNetCacheAPI();
-    bool x_NetCacheMode(const string& object_loc);
+    EVoid x_InitNetCacheAPI();
 
-    CNetService GetServiceFromLocator(const string& object_loc)
-    {
-        CNetStorageObjectLoc locator_struct(m_CompoundIDPool, object_loc);
-        string service_name = locator_struct.GetServiceName();
-
-        if (service_name.empty())
-            return m_Service;
-
-        // Clone will return itself if it has the same name and
-        // insert would not do anything if such entry already exists,
-        // so no need to check for existence.
-        CNetService service(m_Service.Clone(service_name));
-        m_ServiceMap.insert(make_pair(service_name, service));
-        return service;
-    }
+    CNetService GetServiceIfLocator(const string& object_loc);
 
     TNetStorageFlags GetFlags(TNetStorageFlags flags) const
     {
@@ -104,18 +85,6 @@ public:
 
 private:
     map<string, CNetService> m_ServiceMap;
-
-public:
-#ifdef NCBI_GRID_XSITE_CONN_SUPPORT
-    void AllowXSiteConnections()
-    {
-        m_AllowXSiteConnections = true;
-        m_Service.AllowXSiteConnections();
-        if (m_NetCacheAPI) m_NetCacheAPI.GetService().AllowXSiteConnections();
-    }
-
-    bool m_AllowXSiteConnections;
-#endif
 };
 
 END_NCBI_SCOPE
