@@ -1,7 +1,7 @@
 #ifndef CORELIB___NCBI_OS_UNIX_P__HPP
 #define CORELIB___NCBI_OS_UNIX_P__HPP
 
-/* $Id: ncbi_os_unix_p.hpp 425984 2014-02-05 03:53:01Z lavr $
+/* $Id: ncbi_os_unix_p.hpp 458482 2015-02-05 15:48:25Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -38,7 +38,7 @@
 
 #include <ncbiconf.h>
 #if !defined(NCBI_OS_UNIX)
-#  error "ncbi_os_mswin_p.hpp can be used on MS Windows platforms only"
+#  error "ncbi_os_unix_p.hpp can be used on UNIX platforms only"
 #endif
 
 #include <string>
