@@ -1,4 +1,4 @@
-/*  $Id: ncbi_safe_static.cpp 441144 2014-07-21 18:45:59Z vasilche $
+/*  $Id: ncbi_safe_static.cpp 536051 2017-05-15 16:21:49Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -80,21 +80,20 @@ CSafeStaticLifeSpan& CSafeStaticLifeSpan::GetDefault(void)
 
 // Protective mutex and the owner thread ID to avoid
 // multiple initializations and deadlocks
-DEFINE_CLASS_STATIC_MUTEX(CSafeStaticPtr_Base::sm_Mutex);
+DEFINE_CLASS_STATIC_MUTEX(CSafeStaticPtr_Base::sm_ClassMutex);
 
 
 int CSafeStaticPtr_Base::x_GetCreationOrder(void)
 {
     static CAtomicCounter s_CreationOrder;
-    return s_CreationOrder.Add(1);
+    return int(s_CreationOrder.Add(1));
 }
 
 
 CSafeStaticPtr_Base::~CSafeStaticPtr_Base(void)
 {
     if ( x_IsStdStatic() ) {
-        CMutexGuard guard(sm_Mutex);
-        x_Cleanup(guard);
+        x_Cleanup();
     }
 }
 
@@ -112,6 +111,16 @@ CSafeStaticGuard::TStack* CSafeStaticGuard::sm_Stack;
 int CSafeStaticGuard::sm_RefCount;
 
 
+// CSafeStaticGuard reference counter
+// On MSWin threads are killed before destruction of static objects in DLLs, so in
+// most cases this check is useless.
+#if defined(NCBI_WIN32_THREADS)
+bool CSafeStaticGuard::sm_ChildThreadsCheck = false;
+#else
+bool CSafeStaticGuard::sm_ChildThreadsCheck = true;
+#endif
+
+
 CSafeStaticGuard::CSafeStaticGuard(void)
 {
     // Initialize the guard only once
@@ -122,13 +131,20 @@ CSafeStaticGuard::CSafeStaticGuard(void)
     sm_RefCount++;
 }
 
+
+void CSafeStaticGuard::DisableChildThreadsCheck()
+{
+    sm_ChildThreadsCheck = false;
+}
+
+
 static CSafeStaticGuard* sh_CleanupGuard;
 
 
 
 CSafeStaticGuard::~CSafeStaticGuard(void)
 {
-    CMutexGuard guard(CSafeStaticPtr_Base::sm_Mutex);
+    CMutexGuard guard(CSafeStaticPtr_Base::sm_ClassMutex);
 
     // Protect CSafeStaticGuard destruction
     if ( sh_CleanupGuard ) {
@@ -142,14 +158,23 @@ CSafeStaticGuard::~CSafeStaticGuard(void)
     }
     assert(sm_RefCount == 0);
 
+    if (sm_ChildThreadsCheck) {
+        if (const auto tc = CThread::GetThreadsCount()) {
+            string msg = "On static data destruction, child thread(s) still running: " + to_string(tc);
+            ERR_POST_X(1, Error << msg);
+            _ASSERT(CThread::GetThreadsCount() == 0);
+        }
+    }
+
     for ( int pass = 0; pass < 3; ++pass ) {
         // Call Cleanup() for all variables registered
         TStack cur_Stack;
         swap(cur_Stack, *sm_Stack);
+        guard.Release();
         NON_CONST_ITERATE(TStack, it, cur_Stack) {
-            (*it)->x_Cleanup(guard);
-            guard.Guard(CSafeStaticPtr_Base::sm_Mutex);
+            (*it)->x_Cleanup();
         }
+        guard.Guard(CSafeStaticPtr_Base::sm_ClassMutex);
     }
 
     delete sm_Stack;
