@@ -1,4 +1,4 @@
-/*  $Id: ncbiapp.cpp 536050 2017-05-15 16:18:35Z grichenk $
+/*  $Id: ncbiapp.cpp 563865 2018-05-16 15:00:13Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -32,6 +32,7 @@
  */
 
 #include <ncbi_pch.hpp>
+#include <common/ncbi_source_ver.h>
 #include <corelib/ncbiapp.hpp>
 #include <corelib/ncbifile.hpp>
 #include <corelib/ncbi_system.hpp>
@@ -39,9 +40,7 @@
 #include <corelib/syslog.hpp>
 #include <corelib/error_codes.hpp>
 #include <corelib/ncbi_safe_static.hpp>
-#ifdef HAVE_COMMON_NCBI_BUILD_VER_H
-#  include <common/ncbi_build_ver.h>
-#endif
+#include <corelib/request_ctx.hpp>
 #include "ncbisys.hpp"
 
 #if defined(NCBI_OS_MSWIN)
@@ -128,7 +127,11 @@ CNcbiApplication::CNcbiApplication(const SBuildInfo& build_info)
 
     // Create empty version info
     m_Version.Reset(new CVersion(build_info));
-
+#if NCBI_SC_VERSION_PROXY != 0
+    m_Version->AddComponentVersion("NCBI C++ Toolkit",
+        NCBI_SC_VERSION_PROXY, 0, NCBI_SUBVERSION_REVISION_PROXY,
+        NCBI_TEAMCITY_PROJECT_NAME_PROXY, NCBI_APP_SBUILDINFO_DEFAULT());
+#endif
     // Create empty application arguments & name
     m_Arguments.reset(new CNcbiArguments(0,0));
 
@@ -144,6 +147,13 @@ CNcbiApplication::CNcbiApplication(const SBuildInfo& build_info)
 
 CNcbiApplication::~CNcbiApplication(void)
 {
+    CThread::sm_IsExiting = true;
+    // Execute exit actions before waiting for all threads to stop.
+    m_OnExitActions.ExecuteActions();
+#if defined(NCBI_THREADS)
+    CThread::WaitForAllThreads();
+#endif
+
     {
         CMutexGuard guard(GetInstanceMutex());
         m_Instance = 0;
@@ -508,6 +518,13 @@ void CNcbiApplication::x_TryMain(EAppDiagStream diag,
                 NCBI_RETHROW_SAME(e, "Application's execution failed");
             }
             catch (CException& e) {
+                CRef<CRequestContext> cur_ctx(&GetDiagContext().GetRequestContext());
+                if (cur_ctx != &e.GetRequestContext()) {
+                    GetDiagContext().SetRequestContext(&e.GetRequestContext());
+                    NCBI_REPORT_EXCEPTION_X(16,
+                                            "CException thrown", e);
+                    GetDiagContext().SetRequestContext(cur_ctx);
+                }
                 NCBI_REPORT_EXCEPTION_X(16,
                                         "Application's execution failed", e);
                 *got_exception = true;
@@ -841,7 +858,7 @@ int CNcbiApplication::AppMain
     }
 
     if (!diag_context.IsSetExitCode()) {
-        diag_context.SetExitCode(got_exception ? 500 : exit_code);
+        diag_context.SetExitCode(exit_code);
     }
 
     if (m_ExitCodeCond == eAllExits
@@ -864,24 +881,29 @@ void CNcbiApplication::SetEnvironment(const string& name, const string& value)
     SetEnvironment().Set(name, value);
 }
 
+#if 0
 void CNcbiApplication::SetVersionByBuild(int major)
 {
-    int minor =
-#if defined(NCBI_SC_VERSION)
-        NCBI_SC_VERSION;
-#else
-        0;
-#endif
-    int build_num =
-#if defined(NCBI_TEAMCITY_BUILD_NUMBER)
-        NCBI_TEAMCITY_BUILD_NUMBER;
-#else
-        0;
-#endif
-
-    SetVersion(CVersionInfo(major, minor, build_num));
+    SetVersion(major, NStr::StringToInt(m_Version->GetBuildInfo().GetExtraValue(SBuildInfo::eStableComponentsVersion, "0")));
 }
 
+void CNcbiApplication::SetVersion(int major, int minor)
+{
+    SetVersion(major, minor, NStr::StringToInt(m_Version->GetBuildInfo().GetExtraValue(SBuildInfo::eTeamCityBuildNumber, "0")));
+}
+
+void CNcbiApplication::SetVersion(int major, int minor, int patch)
+{
+    m_Version->SetVersionInfo(major, minor, patch,
+        m_Version->GetBuildInfo().GetExtraValue(SBuildInfo::eTeamCityProjectName));
+}
+#else
+void CNcbiApplication::SetVersionByBuild(int major)
+{
+    m_Version->SetVersionInfo(major, NCBI_SC_VERSION_PROXY, NCBI_TEAMCITY_BUILD_NUMBER_PROXY);
+}
+#endif
+
 
 void CNcbiApplication::SetVersion(const CVersionInfo& version)
 {
@@ -1404,6 +1426,8 @@ void CNcbiApplication::x_HonorStandardSettings( IRegistry* reg)
             return;
     }
 
+    CStackTrace::s_HonorSignalHandlingConfiguration();
+    
     // [NCBI.MEMORY_FILL]
     CObject::SetAllocFillMode(reg->Get("NCBI", "MEMORY_FILL"));
 
