@@ -1,4 +1,4 @@
-/*  $Id: ncbimtx.cpp 534857 2017-05-03 12:26:07Z ivanov $
+/*  $Id: ncbimtx.cpp 560489 2018-03-23 01:26:00Z ucko $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -40,6 +40,7 @@
 #include "ncbidbg_p.hpp"
 #include <stdio.h>
 #include <algorithm>
+#include <chrono>
 
 //#define LOG_MUTEX_EVENTS
 
@@ -138,11 +139,14 @@ void SSystemFastMutex::InitializeHandle(void)
 
     // Create platform-dependent mutex handle
 #if defined(NCBI_WIN32_THREADS)
-#  if defined(NCBI_USE_CRITICAL_SECTION)
+#  if defined(NCBI_FASTMUTEX_USE_NEW)
+    InitializeSRWLock(&m_Handle);
+#  elif defined(NCBI_USE_CRITICAL_SECTION)
     InitializeCriticalSection(&m_Handle);
 #  else
-    xncbi_Validate((m_Handle = CreateMutex(NULL, FALSE, NULL)) != NULL,
-                   "Mutex creation failed");
+    xncbi_ValidateAndErrnoReport(
+        (m_Handle = CreateMutex(NULL, FALSE, NULL)) != NULL,
+        "Mutex creation failed");
 #  endif
 #elif defined(NCBI_POSIX_THREADS)
 #  if defined(NCBI_OS_CYGWIN)
@@ -153,8 +157,8 @@ void SSystemFastMutex::InitializeHandle(void)
         // (just looked at the source code; never tested) in the early 2006.
         memset(&m_Handle, 0, sizeof(m_Handle));
 #  endif  /* NCBI_OS_CYGWIN */
-        xncbi_Validate(pthread_mutex_init(&m_Handle, 0) == 0,
-                       "Mutex creation failed");
+        xncbi_ValidatePthread(pthread_mutex_init(&m_Handle, 0), 0,
+                              "Mutex creation failed");
 #  if defined(NCBI_OS_CYGWIN)
     }
 #  endif  /* NCBI_OS_CYGWIN */
@@ -167,13 +171,15 @@ void SSystemFastMutex::DestroyHandle(void)
 
     // Destroy system mutex handle
 #if defined(NCBI_WIN32_THREADS)
-#  if defined(NCBI_USE_CRITICAL_SECTION)
+#  if defined(NCBI_FASTMUTEX_USE_NEW)
+// noop
+#  elif defined(NCBI_USE_CRITICAL_SECTION)
     DeleteCriticalSection(&m_Handle);
 #  else
-    xncbi_Verify(CloseHandle(m_Handle) != 0);
+    xncbi_VerifyAndErrorReport(CloseHandle(m_Handle) != 0);
 #  endif
 #elif defined(NCBI_POSIX_THREADS)
-    xncbi_Verify(pthread_mutex_destroy(&m_Handle) == 0);
+    xncbi_VerifyAndErrorReport(pthread_mutex_destroy(&m_Handle) == 0);
 #endif
 }
 
@@ -259,7 +265,9 @@ void SSystemFastMutex::Lock(ELockSemantics lock /*= eNormal*/)
 
     // Acquire system mutex
 #  if defined(NCBI_WIN32_THREADS)
-#    if defined(NCBI_USE_CRITICAL_SECTION)
+#    if defined(NCBI_FASTMUTEX_USE_NEW)
+    AcquireSRWLockExclusive(&m_Handle);
+#    elif defined(NCBI_USE_CRITICAL_SECTION)
     EnterCriticalSection(&m_Handle);
 #    else
     if (WaitForSingleObject(m_Handle, INFINITE) != WAIT_OBJECT_0) {
@@ -283,7 +291,9 @@ bool SSystemFastMutex::TryLock(void)
     // Check if the system mutex is acquired.
     // If not, acquire for the current thread.
 #  if defined(NCBI_WIN32_THREADS)
-#    if defined(NCBI_USE_CRITICAL_SECTION)
+#    if defined(NCBI_FASTMUTEX_USE_NEW)
+    return TryAcquireSRWLockExclusive(&m_Handle) != 0;
+#    elif defined(NCBI_USE_CRITICAL_SECTION)
     return TryEnterCriticalSection(&m_Handle) != 0;
 #    else
     DWORD status = WaitForSingleObject(m_Handle, 0);
@@ -323,7 +333,9 @@ void SSystemFastMutex::Unlock(ELockSemantics lock /*= eNormal*/)
 
     // Release system mutex
 # if defined(NCBI_WIN32_THREADS)
-#    if defined(NCBI_USE_CRITICAL_SECTION)
+#    if defined(NCBI_FASTMUTEX_USE_NEW)
+    ReleaseSRWLockExclusive(&m_Handle);
+#    elif defined(NCBI_USE_CRITICAL_SECTION)
     LeaveCriticalSection(&m_Handle);
 #    else
     if ( !ReleaseMutex(m_Handle) ) { // error
@@ -420,7 +432,7 @@ static TSystemMutex s_GetInitMutexHandle(void)
     TSystemMutex init_mutex = s_InitMutexHandle;
     if ( !init_mutex ) {
         init_mutex = CreateMutex(NULL, FALSE, kInitMutexName);
-        xncbi_Verify(init_mutex);
+        xncbi_VerifyAndErrorReport(init_mutex);
         assert(!s_InitMutexHandle || s_InitMutexHandle == init_mutex);
         s_InitMutexHandle = init_mutex;
     }
@@ -437,7 +449,7 @@ static inline void s_ReleaseInitMutexHandle(TSystemMutex _DEBUG_ARG(mutex))
 static inline HANDLE s_GetInitMutexHandle(void)
 {
     HANDLE init_mutex = CreateMutexA(NULL, FALSE, kInitMutexName);
-    xncbi_Verify(init_mutex);
+    xncbi_VerifyAndErrorReport(init_mutex);
     return init_mutex;
 }
 
@@ -454,11 +466,12 @@ void CAutoInitializeStaticFastMutex::Initialize(void)
         return;
     }
     HANDLE init_mutex = s_GetInitMutexHandle();
-    xncbi_Verify(WaitForSingleObject(init_mutex, INFINITE) == WAIT_OBJECT_0);
+    xncbi_VerifyAndErrorReport(
+        WaitForSingleObject(init_mutex, INFINITE) == WAIT_OBJECT_0);
     if ( !m_Mutex.IsInitialized() ) {
         m_Mutex.InitializeStatic();
     }
-    xncbi_Verify(ReleaseMutex(init_mutex));
+    xncbi_VerifyAndErrorReport(ReleaseMutex(init_mutex));
     assert(m_Mutex.IsInitialized());
     s_ReleaseInitMutexHandle(init_mutex);
 }
@@ -469,11 +482,12 @@ void CAutoInitializeStaticMutex::Initialize(void)
         return;
     }
     HANDLE init_mutex = s_GetInitMutexHandle();
-    xncbi_Verify(WaitForSingleObject(init_mutex, INFINITE) == WAIT_OBJECT_0);
+    xncbi_VerifyAndErrorReport(
+        WaitForSingleObject(init_mutex, INFINITE) == WAIT_OBJECT_0);
     if ( !m_Mutex.IsInitialized() ) {
         m_Mutex.InitializeStatic();
     }
-    xncbi_Verify(ReleaseMutex(init_mutex));
+    xncbi_VerifyAndErrorReport(ReleaseMutex(init_mutex));
     assert(m_Mutex.IsInitialized());
     s_ReleaseInitMutexHandle(init_mutex);
 }
@@ -496,7 +510,7 @@ const char* CMutexException::GetErrCodeString(void) const
 //  CInternalRWLock::
 //
 
-#if defined(NCBI_WIN32_THREADS)
+#if defined(NCBI_WIN32_THREADS) && !NCBI_SRWLOCK_USE_NEW
 
 class CWindowsHandle
 {
@@ -552,8 +566,8 @@ public:
     LONG Release(LONG add = 1)
     {
         LONG prev_sema;
-        xncbi_Validate(ReleaseSemaphore(*this, add, &prev_sema),
-                       "CWindowsSemaphore::Release() failed");
+        xncbi_ValidateAndErrnoReport(ReleaseSemaphore(*this, add, &prev_sema),
+                                     "CWindowsSemaphore::Release() failed");
         return prev_sema;
     }
 };
@@ -586,6 +600,7 @@ protected:
 
 #endif
 
+#if !NCBI_SRWLOCK_USE_NEW
 class CInternalRWLock
 {
 public:
@@ -618,11 +633,26 @@ CInternalRWLock::CInternalRWLock(void)
     m_Mutex.Set(CreateMutex(NULL, FALSE, NULL));
 #endif
 }
-
+#endif
 /////////////////////////////////////////////////////////////////////////////
 //  CRWLock::
 //
 
+#if NCBI_SRWLOCK_USE_NEW
+CRWLock::CRWLock(TFlags flags)
+    : m_Owner(0), m_Count(0), m_WaitingWriters(0), m_TrackReaders(false)
+{
+#if defined(_DEBUG)
+    m_TrackReaders = true;
+    m_FavorWriters = (flags & fFavorWriters) != 0;
+#else
+    m_TrackReaders = m_FavorWriters = (flags & fFavorWriters) != 0;
+#endif
+    if (m_TrackReaders) {
+        m_Readers.reserve(16);
+    }
+}
+#else
 CRWLock::CRWLock(TFlags flags)
     : m_Flags(flags),
       m_RW(new CInternalRWLock),
@@ -640,13 +670,55 @@ CRWLock::CRWLock(TFlags flags)
         m_Readers.reserve(16);
     }
 }
+#endif
 
 
 CRWLock::~CRWLock(void)
 {
+    _ASSERT(m_Count == 0);
 }
 
 
+#if NCBI_SRWLOCK_USE_NEW
+inline
+vector<TThreadSystemID>::const_iterator
+CRWLock::x_FindReader(TThreadSystemID self_id)
+{
+    return find(m_Readers.begin(), m_Readers.end(), self_id);
+}
+
+inline
+bool 
+CRWLock::x_HasReader(TThreadSystemID self_id)
+{
+    return m_Readers.empty() ? false : x_FindReader(self_id) != m_Readers.end();
+}
+
+inline
+bool CRWLock::x_MayAcquireForReading(TThreadSystemID self_id)
+{
+    return (m_WaitingWriters == 0 && m_Count >= 0) || (m_TrackReaders && x_HasReader(self_id));
+}
+
+inline 
+bool CRWLock::x_TryWriteLock()
+{
+    long expected = 0;
+    return m_Count.compare_exchange_strong(expected, expected-1);
+}
+
+inline 
+bool CRWLock::x_TryReadLock()
+{
+    long expected = 0;
+    do {
+        if (m_Count.compare_exchange_weak(expected, expected+1)) {
+            return true;
+        }
+    } while (expected >= 0);
+    return false;
+}
+#else
 inline bool CRWLock::x_MayAcquireForReading(TThreadSystemID self_id)
 {
     _ASSERT(self_id == GetCurrentThreadSystemID());
@@ -662,9 +734,10 @@ inline bool CRWLock::x_MayAcquireForReading(TThreadSystemID self_id)
         return !m_WaitingWriters;
     }
 }
+#endif
 
 
-#if defined(NCBI_USE_CRITICAL_SECTION)
+#if defined(NCBI_USE_CRITICAL_SECTION) && !NCBI_SRWLOCK_USE_NEW
 
 // Need special guard for system handle since mutex uses critical section
 class CWin32MutexHandleGuard
@@ -690,7 +763,6 @@ CWin32MutexHandleGuard::~CWin32MutexHandleGuard(void)
 {
     ReleaseMutex(m_Handle);
 }
-
 #endif
 
 
@@ -791,6 +863,28 @@ void CRWLock::ReadLock(void)
     return;
 #else
 
+#if NCBI_SRWLOCK_USE_NEW
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
+    if (m_Owner == self_id) {
+        --m_Count;
+    } else {
+        if (!m_TrackReaders && x_TryReadLock()) {
+            return;
+        }
+        unique_lock<mutex> lck( m_Mtx);
+        do {
+            if ( !x_MayAcquireForReading(self_id) ) {
+                while (m_WaitingWriters > 0 || m_Count < 0) {
+                    m_Cv.wait(lck);
+                }
+            }
+        } while (!x_TryReadLock());
+        if (m_TrackReaders) {
+            m_Readers.push_back(self_id);
+        }
+    }
+#else
+
 #if defined(NCBI_WIN32_THREADS)
     if ((m_Flags & fTrackReaders) == 0) {
         // Try short way if there are other readers.
@@ -818,8 +912,9 @@ void CRWLock::ReadLock(void)
             obj[0] = m_RW->m_Mutex.GetHandle();
             obj[1] = m_RW->m_Rsema;
             obj[2] = m_RW->m_Rsema2;
-            xncbi_Validate(ReleaseMutex(m_RW->m_Mutex.GetHandle()),
-                           "CRWLock::ReadLock() - release mutex error");
+            xncbi_ValidateAndErrnoReport(
+                ReleaseMutex(m_RW->m_Mutex.GetHandle()),
+                "CRWLock::ReadLock() - release mutex error");
             DWORD wait_res =
                 WaitForMultipleObjects(3, obj, TRUE, INFINITE)-WAIT_OBJECT_0;
             xncbi_Validate(wait_res < 3,
@@ -836,10 +931,10 @@ void CRWLock::ReadLock(void)
             }
 #elif defined(NCBI_POSIX_THREADS)
             while ( !x_MayAcquireForReading(self_id) ) {
-                xncbi_Validate(pthread_cond_wait(m_RW->m_Rcond,
-                                                 m_RW->m_Mutex.GetHandle())
-                               == 0,
-                               "CRWLock::ReadLock() - R-lock waiting error");
+                xncbi_ValidatePthread(
+                    pthread_cond_wait(m_RW->m_Rcond,
+                                      m_RW->m_Mutex.GetHandle()), 0,
+                    "CRWLock::ReadLock() - R-lock waiting error");
             }
 #else
             // Can not be already W-locked by another thread without MT
@@ -871,6 +966,7 @@ void CRWLock::ReadLock(void)
         m_Readers.push_back(self_id);
     }
 #endif
+#endif
 }
 
 
@@ -880,6 +976,28 @@ bool CRWLock::TryReadLock(void)
     return true;
 #else
 
+#if NCBI_SRWLOCK_USE_NEW
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
+    if (m_Owner == self_id) {
+        --m_Count;
+    } else {
+        if (!m_TrackReaders && x_TryReadLock()) {
+            return true;
+        }
+        unique_lock<mutex> lck( m_Mtx);
+        if (!x_MayAcquireForReading(self_id)) {
+            return false;
+        }
+        if (!x_TryReadLock()) {
+            return false;
+        }
+        if (m_TrackReaders) {
+            m_Readers.push_back(self_id);
+        }
+    }
+    return true;
+#else
+
 #if defined(NCBI_WIN32_THREADS)
     if ((m_Flags & fTrackReaders) == 0) {
         if (interlocked_inc_min(&m_Count, 0)) return true;
@@ -922,6 +1040,7 @@ bool CRWLock::TryReadLock(void)
     }
     return true;
 #endif
+#endif
 }
 
 
@@ -939,6 +1058,41 @@ bool CRWLock::TryReadLock(const CTimeout& timeout)
         return TryReadLock();
     }
 
+#if NCBI_SRWLOCK_USE_NEW
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
+    if (m_Owner == self_id) {
+        --m_Count;
+    } else {
+        if (!m_TrackReaders && x_TryReadLock()) {
+            return true;
+        }
+        unsigned int sec=0, nanosec=0;
+        timeout.GetNano(&sec, &nanosec);
+        chrono::time_point<chrono::steady_clock> to = chrono::steady_clock::now() + chrono::seconds(sec) + chrono::nanoseconds(nanosec);
+        cv_status res = cv_status::no_timeout;
+        bool ok = true;
+
+        unique_lock<mutex> lck( m_Mtx);
+        do {
+            if (res == cv_status::timeout) {
+                return false;
+            }
+            if (!x_MayAcquireForReading(self_id)) {
+                while (!(ok = m_WaitingWriters == 0 && m_Count >= 0) && res == cv_status::no_timeout) {
+                    res = m_Cv.wait_until(lck, to);
+                }
+            }
+            if (!ok) {
+                return false;
+            }
+        } while (!x_TryReadLock());
+        if (m_TrackReaders) {
+            m_Readers.push_back(self_id);
+        }
+    }
+    return true;
+#else
+
 #if defined(NCBI_WIN32_THREADS)
     if ((m_Flags & fTrackReaders) == 0) {
         // Try short way if there are other readers.
@@ -966,8 +1120,9 @@ bool CRWLock::TryReadLock(const CTimeout& timeout)
             obj[0] = m_RW->m_Mutex.GetHandle();
             obj[1] = m_RW->m_Rsema;
             obj[2] = m_RW->m_Rsema2;
-            xncbi_Validate(ReleaseMutex(m_RW->m_Mutex.GetHandle()),
-                           "CRWLock::TryReadLock() - release mutex error");
+            xncbi_ValidateAndErrnoReport(
+                ReleaseMutex(m_RW->m_Mutex.GetHandle()),
+                "CRWLock::TryReadLock() - release mutex error");
             DWORD timeout_msec = timeout.GetAsMilliSeconds();
             DWORD wait_res =
                 WaitForMultipleObjects(3, obj, TRUE, timeout_msec);
@@ -1034,8 +1189,9 @@ bool CRWLock::TryReadLock(const CTimeout& timeout)
     if ((m_Flags & fTrackReaders) != 0  &&  m_Count > 0) {
         m_Readers.push_back(self_id);
     }
-#endif
     return true;
+#endif
+#endif
 }
 
 
@@ -1045,6 +1201,30 @@ void CRWLock::WriteLock(void)
     return;
 #else
 
+#if NCBI_SRWLOCK_USE_NEW
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
+    if (m_Owner == self_id) {
+        --m_Count;
+    } else {
+        if (x_TryWriteLock()) {
+            m_Owner = self_id;
+            return;
+        }
+        unique_lock<mutex> lck( m_Mtx);
+        _ASSERT(!m_TrackReaders || !x_HasReader(self_id));
+        if (m_FavorWriters) {
+            ++m_WaitingWriters;
+        }
+        while (!x_TryWriteLock()) {
+            m_Cv.wait(lck);
+        }
+        m_Owner = self_id;
+        if (m_FavorWriters) {
+            --m_WaitingWriters;
+        }
+    }
+#else
+
     TThreadSystemID self_id = GetCurrentThreadSystemID();
 
 #if defined(NCBI_USE_CRITICAL_SECTION)
@@ -1089,8 +1269,9 @@ void CRWLock::WriteLock(void)
                                    "error locking R-semaphore 2");
                 }
             }
-            xncbi_Validate(ReleaseMutex(m_RW->m_Mutex.GetHandle()),
-                           "CRWLock::WriteLock() - release mutex error");
+            xncbi_ValidateAndErrnoReport(
+                ReleaseMutex(m_RW->m_Mutex.GetHandle()),
+                "CRWLock::WriteLock() - release mutex error");
             DWORD wait_res =
                 WaitForMultipleObjects(3, obj, TRUE, INFINITE)-WAIT_OBJECT_0;
             xncbi_Validate(wait_res < 3,
@@ -1111,10 +1292,10 @@ void CRWLock::WriteLock(void)
             ++m_WaitingWriters;
         }
         while (m_Count != 0) {
-            xncbi_Validate(pthread_cond_wait(m_RW->m_Wcond,
-                                             m_RW->m_Mutex.GetHandle()) == 0,
-                           "CRWLock::WriteLock() - "
-                           "error locking R&W-conditionals");
+            xncbi_ValidatePthread(
+                pthread_cond_wait(m_RW->m_Wcond,
+                                  m_RW->m_Mutex.GetHandle()), 0,
+                "CRWLock::WriteLock() - error locking R&W-conditionals");
         }
         if (m_Flags & fFavorWriters) {
             --m_WaitingWriters;
@@ -1129,6 +1310,7 @@ void CRWLock::WriteLock(void)
     // No readers allowed
     _ASSERT(m_Readers.empty());
 #endif
+#endif
 }
 
 
@@ -1138,6 +1320,19 @@ bool CRWLock::TryWriteLock(void)
     return true;
 #else
 
+#if NCBI_SRWLOCK_USE_NEW
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
+    if (m_Owner == self_id) {
+        --m_Count;
+    } else {
+        if (!x_TryWriteLock()) {
+            return false;
+        }
+        m_Owner = self_id;
+    }
+    return true;
+#else
+
     TThreadSystemID self_id = GetCurrentThreadSystemID();
 
 #if defined(NCBI_USE_CRITICAL_SECTION)
@@ -1181,6 +1376,7 @@ bool CRWLock::TryWriteLock(void)
 
     return true;
 #endif
+#endif
 }
 
 
@@ -1198,6 +1394,43 @@ bool CRWLock::TryWriteLock(const CTimeout& timeout)
         return TryWriteLock();
     }
 
+#if NCBI_SRWLOCK_USE_NEW
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
+    if (m_Owner == self_id) {
+        --m_Count;
+    } else {
+        if (x_TryWriteLock()) {
+            m_Owner = self_id;
+            return true;
+        }
+        unsigned int sec=0, nanosec=0;
+        timeout.GetNano(&sec, &nanosec);
+        chrono::time_point<chrono::steady_clock> to = chrono::steady_clock::now() + chrono::seconds(sec) + chrono::nanoseconds(nanosec);
+        cv_status res = cv_status::no_timeout;
+        bool ok = true;
+
+        unique_lock<mutex> lck( m_Mtx);
+        do {
+            if (res == cv_status::timeout) {
+                return false;
+            }
+            if (m_FavorWriters) {
+                ++m_WaitingWriters;
+            }
+            while (!(ok = m_Count == 0) && res == cv_status::no_timeout) {
+                res = m_Cv.wait_until(lck, to);
+            }
+            if (m_FavorWriters) {
+                --m_WaitingWriters;
+            }
+            if (!ok) {
+                return false;
+            }
+        } while (!x_TryWriteLock());
+        m_Owner = self_id;
+    }
+    return true;
+#else
     TThreadSystemID self_id = GetCurrentThreadSystemID();
 
 #if defined(NCBI_USE_CRITICAL_SECTION)
@@ -1242,8 +1475,9 @@ bool CRWLock::TryWriteLock(const CTimeout& timeout)
                                    "error locking R-semaphore 2");
                 }
             }
-            xncbi_Validate(ReleaseMutex(m_RW->m_Mutex.GetHandle()),
-                           "CRWLock::TryWriteLock() - release mutex error");
+            xncbi_ValidateAndErrnoReport(
+                ReleaseMutex(m_RW->m_Mutex.GetHandle()),
+                "CRWLock::TryWriteLock() - release mutex error");
             DWORD timeout_msec = timeout.GetAsMilliSeconds();
             DWORD wait_res =
                 WaitForMultipleObjects(3, obj, TRUE, timeout_msec);
@@ -1312,8 +1546,9 @@ bool CRWLock::TryWriteLock(const CTimeout& timeout)
 
     // No readers allowed
     _ASSERT(m_Readers.empty());
-#endif
     return true;
+#endif
+#endif
 }
 
 
@@ -1323,6 +1558,29 @@ void CRWLock::Unlock(void)
     return;
 #else
 
+#if NCBI_SRWLOCK_USE_NEW
+    TThreadSystemID self_id = GetCurrentThreadSystemID();
+    if (m_Owner == self_id && m_Count < -1) {
+        ++m_Count;
+        return;
+    }
+    unique_lock<mutex> lck( m_Mtx/*, defer_lock*/);
+    if (m_Owner == self_id) {
+        m_Owner = 0;
+        ++m_Count;
+//        lck.lock();
+    } else {
+        long cnt = --m_Count;
+        if (m_TrackReaders) {
+//            lck.lock();
+            m_Readers.erase( x_FindReader(self_id) );
+        } else if (cnt != 0) {
+            return;
+        }
+    }
+    m_Cv.notify_all();
+#else
+
 #if defined(NCBI_WIN32_THREADS)
     if ((m_Flags & fTrackReaders) == 0) {
         if (interlocked_dec_min(&m_Count, 1)) return;
@@ -1352,11 +1610,13 @@ void CRWLock::Unlock(void)
                            "CRWLock::Unlock() - invalid R-semaphore state");
 #elif defined(NCBI_POSIX_THREADS)
             if ( !m_WaitingWriters ) {
-                xncbi_Validate(pthread_cond_broadcast(m_RW->m_Rcond) == 0,
-                               "CRWLock::Unlock() - error signalling unlock");
+                xncbi_ValidatePthread(
+                    pthread_cond_broadcast(m_RW->m_Rcond), 0,
+                    "CRWLock::Unlock() - error signalling unlock");
             }
-            xncbi_Validate(pthread_cond_signal(m_RW->m_Wcond) == 0,
-                           "CRWLock::Unlock() - error signalling unlock");
+            xncbi_ValidatePthread(
+                pthread_cond_signal(m_RW->m_Wcond), 0,
+                "CRWLock::Unlock() - error signalling unlock");
 #endif
         }
         if (m_Flags & fTrackReaders) {
@@ -1375,8 +1635,9 @@ void CRWLock::Unlock(void)
             xncbi_Validate(m_RW->m_Wsema.Release() == 0,
                            "CRWLock::Unlock() - invalid W-semaphore state");
 #elif defined(NCBI_POSIX_THREADS)
-            xncbi_Validate(pthread_cond_signal(m_RW->m_Wcond) == 0,
-                           "CRWLock::Unlock() - error signaling unlock");
+            xncbi_ValidatePthread(
+                pthread_cond_signal(m_RW->m_Wcond), 0,
+                "CRWLock::Unlock() - error signaling unlock");
 #endif
         }
         if (m_Flags & fTrackReaders) {
@@ -1391,6 +1652,7 @@ void CRWLock::Unlock(void)
         }
     }
 #endif
+#endif
 }
 
 
@@ -1420,8 +1682,20 @@ struct SSemaphore
 };
 
 
+#if NCBI_SEMAPHORE_USE_NEW
+CSemaphore::CSemaphore(unsigned int init_count, unsigned int max_count)
+    : m_Max(max_count), m_Count(init_count)
+{
+    _ASSERT(max_count != 0);
+    _ASSERT(init_count <= max_count);
+}
+#else
 CSemaphore::CSemaphore(unsigned int init_count, unsigned int max_count)
 {
+#if defined(NCBI_WIN32_THREADS)
+    // On Windows max_count is LONG and can not exceed LONG_MAX.
+    max_count = min(max_count, (unsigned int)LONG_MAX);
+#endif
     xncbi_Validate(max_count != 0,
                    "CSemaphore::CSemaphore() - max_count passed zero");
     xncbi_Validate(init_count <= max_count,
@@ -1440,8 +1714,9 @@ CSemaphore::CSemaphore(unsigned int init_count, unsigned int max_count)
     if (pthread_mutex_init(&m_Sem->mutex, 0) != 0) {
         memset(&m_Sem->mutex, 0, sizeof(m_Sem->mutex));
 #  endif  /* NCBI_OS_CYGWIN */
-        xncbi_Validate(pthread_mutex_init(&m_Sem->mutex, 0) == 0,
-                       "CSemaphore::CSemaphore() - pthread_mutex_init() failed");
+        xncbi_ValidatePthread(
+            pthread_mutex_init(&m_Sem->mutex, 0), 0,
+            "CSemaphore::CSemaphore() - pthread_mutex_init() failed");
 #  if defined(NCBI_OS_CYGWIN)
     }
 #  endif  /* NCBI_OS_CYGWIN */
@@ -1450,17 +1725,23 @@ CSemaphore::CSemaphore(unsigned int init_count, unsigned int max_count)
     if (pthread_cond_init(&m_Sem->cond, 0) != 0) {
         memset(&m_Sem->cond, 0, sizeof(m_Sem->cond));
 #  endif  /* NCBI_OS_CYGWIN */
-        xncbi_Validate(pthread_cond_init(&m_Sem->cond, 0) == 0,
-                       "CSemaphore::CSemaphore() - pthread_cond_init() failed");
+        xncbi_ValidatePthread(
+            pthread_cond_init(&m_Sem->cond, 0), 0,
+            "CSemaphore::CSemaphore() - pthread_cond_init() failed");
 #  if defined(NCBI_OS_CYGWIN)
     }
 #  endif  /* NCBI_OS_CYGWIN */
 
 #elif defined(NCBI_WIN32_THREADS)
     m_Sem->sem = CreateSemaphore(NULL, init_count, max_count, NULL);
+#ifdef _DEBUG
+    if (m_Sem->sem == NULL) {
+        DWORD dwErr = GetLastError();
+        ERR_POST(Error << "GetLastError returns " << dwErr);
+    }
+#endif
     xncbi_Validate(m_Sem->sem != NULL,
                    "CSemaphore::CSemaphore() - CreateSemaphore() failed");
-
 #else
     m_Sem->max_count = max_count;
     m_Sem->count     = init_count;
@@ -1468,28 +1749,54 @@ CSemaphore::CSemaphore(unsigned int init_count, unsigned int max_count)
 
     auto_sem.release();
 }
+#endif
 
 
 CSemaphore::~CSemaphore(void)
 {
+#if NCBI_SEMAPHORE_USE_NEW
+#else
 #if defined(NCBI_POSIX_THREADS)
     _ASSERT(m_Sem->wait_count == 0);
-    xncbi_Verify(pthread_mutex_destroy(&m_Sem->mutex) == 0);
-    xncbi_Verify(pthread_cond_destroy(&m_Sem->cond)  == 0);
+    xncbi_VerifyAndErrorReport(pthread_mutex_destroy(&m_Sem->mutex) == 0);
+    xncbi_VerifyAndErrorReport(pthread_cond_destroy(&m_Sem->cond)  == 0);
 
 #elif defined(NCBI_WIN32_THREADS)
-    xncbi_Verify( CloseHandle(m_Sem->sem) );
+    xncbi_VerifyAndErrorReport( CloseHandle(m_Sem->sem) );
 #endif
 
     delete m_Sem;
+#endif
 }
 
+#if NCBI_SEMAPHORE_USE_NEW
+inline
+bool CSemaphore::x_TryAcquire(void)
+{
+    unsigned int expected = 1;
+    do {
+        if (m_Count.compare_exchange_weak(expected, expected-1)) {
+            return true;
+        }
+    } while (expected > 0);
+    return false;
+}
+#endif
 
 void CSemaphore::Wait(void)
 {
+#if NCBI_SEMAPHORE_USE_NEW
+    if (!x_TryAcquire()) {
+        unique_lock<mutex> lck( m_Mtx);
+        while (!x_TryAcquire()) {
+            m_Cv.wait(lck);
+        }
+    }
+#else
 #if defined(NCBI_POSIX_THREADS)
-    xncbi_Validate(pthread_mutex_lock(&m_Sem->mutex) == 0,
-                   "CSemaphore::Wait() - pthread_mutex_lock() failed");
+    xncbi_ValidatePthread(
+        pthread_mutex_lock(&m_Sem->mutex), 0,
+        "CSemaphore::Wait() - pthread_mutex_lock() failed");
 
     if (m_Sem->count != 0) {
         m_Sem->count--;
@@ -1499,10 +1806,10 @@ void CSemaphore::Wait(void)
         do {
             int status = pthread_cond_wait(&m_Sem->cond, &m_Sem->mutex);
             if (status != 0  &&  status != EINTR) {
-                xncbi_Validate(pthread_mutex_unlock(&m_Sem->mutex) == 0,
-                               "CSemaphore::Wait() - "
-                               "pthread_cond_wait() and "
-                               "pthread_mutex_unlock() failed");
+                xncbi_ValidatePthread(
+                    pthread_mutex_unlock(&m_Sem->mutex), 0,
+                    "CSemaphore::Wait() - pthread_cond_wait() and "
+                    "pthread_mutex_unlock() failed");
                 xncbi_Validate(0,
                                "CSemaphore::Wait() - "
                                "pthread_cond_wait() failed");
@@ -1512,8 +1819,9 @@ void CSemaphore::Wait(void)
         m_Sem->count--;
     }
 
-    xncbi_Validate(pthread_mutex_unlock(&m_Sem->mutex) == 0,
-                   "CSemaphore::Wait() - pthread_mutex_unlock() failed");
+    xncbi_ValidatePthread(
+        pthread_mutex_unlock(&m_Sem->mutex), 0,
+        "CSemaphore::Wait() - pthread_mutex_unlock() failed");
 
 #elif defined(NCBI_WIN32_THREADS)
     xncbi_Validate(WaitForSingleObject(m_Sem->sem, INFINITE) == WAIT_OBJECT_0,
@@ -1525,6 +1833,7 @@ void CSemaphore::Wait(void)
                    "wait with zero count in one-thread mode(?!)");
     m_Sem->count--;
 #endif
+#endif
 }
 
 #if defined(NCBI_NO_THREADS)
@@ -1536,9 +1845,32 @@ void CSemaphore::Wait(void)
 bool CSemaphore::TryWait(unsigned int NCBI_THREADS_ARG(timeout_sec),
                          unsigned int NCBI_THREADS_ARG(timeout_nsec))
 {
+#if NCBI_SEMAPHORE_USE_NEW
+#if defined(NCBI_NO_THREADS)
+    return x_TryAcquire();
+#else
+    if (x_TryAcquire()) {
+        return true;
+    }
+    if (timeout_sec == 0 && timeout_nsec == 0) {
+        return x_TryAcquire();
+    }
+    chrono::time_point<chrono::steady_clock> to = chrono::steady_clock::now() + chrono::seconds(timeout_sec) + chrono::nanoseconds(timeout_nsec);
+    cv_status res = cv_status::no_timeout;
+    unique_lock<mutex> lck( m_Mtx);
+    while (!x_TryAcquire()) {
+        res = m_Cv.wait_until(lck, to);
+        if (res == cv_status::timeout) {
+            return false;
+        }
+    }
+    return true;
+#endif
+#else
 #if defined(NCBI_POSIX_THREADS)
-    xncbi_Validate(pthread_mutex_lock(&m_Sem->mutex) == 0,
-                   "CSemaphore::TryWait() - pthread_mutex_lock() failed");
+    xncbi_ValidatePthread(
+        pthread_mutex_lock(&m_Sem->mutex), 0,
+        "CSemaphore::TryWait() - pthread_mutex_lock() failed");
 
     bool retval = false;
     if (m_Sem->count != 0) {
@@ -1581,10 +1913,10 @@ bool CSemaphore::TryWait(unsigned int NCBI_THREADS_ARG(timeout_sec),
                 break;
             } else if (status != 0  &&  status != EINTR) {
                 // EINVAL, presumably?
-                xncbi_Validate(pthread_mutex_unlock(&m_Sem->mutex) == 0,
-                               "CSemaphore::TryWait() - "
-                               "pthread_cond_timedwait() and "
-                               "pthread_mutex_unlock() failed");
+                xncbi_ValidatePthread(
+                    pthread_mutex_unlock(&m_Sem->mutex), 0,
+                    "CSemaphore::TryWait() - pthread_cond_timedwait() and "
+                    "pthread_mutex_unlock() failed");
                 xncbi_Validate(0, "CSemaphore::TryWait() - "
                                "pthread_cond_timedwait() failed");
             }
@@ -1596,8 +1928,9 @@ bool CSemaphore::TryWait(unsigned int NCBI_THREADS_ARG(timeout_sec),
         }
     }
 
-    xncbi_Validate(pthread_mutex_unlock(&m_Sem->mutex) == 0,
-                   "CSemaphore::TryWait() - pthread_mutex_unlock() failed");
+    xncbi_ValidatePthread(
+        pthread_mutex_unlock(&m_Sem->mutex), 0,
+        "CSemaphore::TryWait() - pthread_mutex_unlock() failed");
 
     return retval;
 
@@ -1619,6 +1952,7 @@ bool CSemaphore::TryWait(unsigned int NCBI_THREADS_ARG(timeout_sec),
     m_Sem->count--;
     return true;
 #endif
+#endif
 }
 
 
@@ -1640,16 +1974,25 @@ void CSemaphore::Post(unsigned int count)
     if (count == 0)
         return;
 
+#if NCBI_SEMAPHORE_USE_NEW
+    unsigned int cnt_now = m_Count;
+    xncbi_Validate(cnt_now <= kMax_UInt - count && cnt_now + count <= m_Max,
+                   "CSemaphore::Post() - attempt to exceed max_count");
+    m_Count += count;
+    unique_lock<mutex> lck( m_Mtx);
+    m_Cv.notify_all();
+#else
 #if defined (NCBI_POSIX_THREADS)
-    xncbi_Validate(pthread_mutex_lock(&m_Sem->mutex) == 0,
-                   "CSemaphore::Post() - pthread_mutex_lock() failed");
+    xncbi_ValidatePthread(
+        pthread_mutex_lock(&m_Sem->mutex), 0,
+        "CSemaphore::Post() - pthread_mutex_lock() failed");
 
     if (m_Sem->count > kMax_UInt - count  ||
         m_Sem->count + count > m_Sem->max_count) {
-        xncbi_Validate(pthread_mutex_unlock(&m_Sem->mutex) == 0,
-                       "CSemaphore::Post() - "
-                       "attempt to exceed max_count and "
-                       "pthread_mutex_unlock() failed");
+        xncbi_ValidatePthread(
+            pthread_mutex_unlock(&m_Sem->mutex), 0,
+            "CSemaphore::Post() - attempt to exceed max_count and "
+            "pthread_mutex_unlock() failed");
         xncbi_Validate(false,
                        "CSemaphore::Post() - attempt to exceed max_count");
     }
@@ -1673,29 +2016,32 @@ void CSemaphore::Post(unsigned int count)
     // Success
     if (err_code == 0) {
         m_Sem->count += count;
-        xncbi_Validate(pthread_mutex_unlock(&m_Sem->mutex) == 0,
-                       "CSemaphore::Post() - pthread_mutex_unlock() failed");
+        xncbi_ValidatePthread(
+            pthread_mutex_unlock(&m_Sem->mutex), 0,
+            "CSemaphore::Post() - pthread_mutex_unlock() failed");
         return;
     }
 
     // Error
-    xncbi_Validate(pthread_mutex_unlock(&m_Sem->mutex) == 0,
-                   "CSemaphore::Post() - "
-                   "pthread_cond_signal/broadcast() and "
-                   "pthread_mutex_unlock() failed");
+    xncbi_ValidatePthread(
+        pthread_mutex_unlock(&m_Sem->mutex), 0,
+        "CSemaphore::Post() - pthread_cond_signal/broadcast() and "
+        "pthread_mutex_unlock() failed");
     xncbi_Validate(0,
                    "CSemaphore::Post() - "
                    "pthread_cond_signal/broadcast() failed");
 
 #elif defined(NCBI_WIN32_THREADS)
-    xncbi_Validate(ReleaseSemaphore(m_Sem->sem, count, NULL),
-                   "CSemaphore::Post() - ReleaseSemaphore() failed");
+    xncbi_ValidateAndErrnoReport(
+        ReleaseSemaphore(m_Sem->sem, count, NULL),
+        "CSemaphore::Post() - ReleaseSemaphore() failed");
 
 #else
     xncbi_Validate(m_Sem->count + count <= m_Sem->max_count,
                    "CSemaphore::Post() - attempt to exceed max_count");
     m_Sem->count += count;
 #endif
+#endif
 }
 
 
@@ -1703,34 +2049,50 @@ void CSemaphore::Post(unsigned int count)
 void
 CFastRWLock::ReadLock(void)
 {
+#if defined(NCBI_WIN32_THREADS) && defined(NCBI_FASTRWLOCK_USE_NEW)
+    AcquireSRWLockShared(&m_Lock);
+#else
     while (m_LockCount.Add(1) > kWriteLockValue) {
         m_LockCount.Add(-1);
         m_WriteLock.Lock();
         m_WriteLock.Unlock();
     }
+#endif
 }
 
 void
 CFastRWLock::ReadUnlock(void)
 {
+#if defined(NCBI_WIN32_THREADS) && defined(NCBI_FASTRWLOCK_USE_NEW)
+    ReleaseSRWLockShared(&m_Lock);
+#else
     m_LockCount.Add(-1);
+#endif
 }
 
 void
 CFastRWLock::WriteLock(void)
 {
+#if defined(NCBI_WIN32_THREADS) && defined(NCBI_FASTRWLOCK_USE_NEW)
+    AcquireSRWLockExclusive(&m_Lock);
+#else
     m_WriteLock.Lock();
     m_LockCount.Add(kWriteLockValue);
     while (m_LockCount.Get() != kWriteLockValue) {
         NCBI_SCHED_YIELD();
     }
+#endif
 }
 
 void
 CFastRWLock::WriteUnlock(void)
 {
+#if defined(NCBI_WIN32_THREADS) && defined(NCBI_FASTRWLOCK_USE_NEW)
+    ReleaseSRWLockExclusive(&m_Lock);
+#else
     m_LockCount.Add(-kWriteLockValue);
     m_WriteLock.Unlock();
+#endif
 }
 
 
@@ -2138,7 +2500,11 @@ bool CConditionVariable::x_WaitForSignal
 #if defined(NCBI_OS_MSWIN)
     DWORD timeout_msec = deadline.IsInfinite() ?
         INFINITE : deadline.GetRemainingTime().GetAsMilliSeconds();
+#if defined(NCBI_FASTMUTEX_USE_NEW)
+    BOOL res = SleepConditionVariableSRW(&m_ConditionVar, &mutex.m_Handle, timeout_msec, 0);
+#else
     BOOL res = SleepConditionVariableCS(&m_ConditionVar, &mutex.m_Handle, timeout_msec);
+#endif
     s_ThrowIfDifferentMutexes(mutex_guard);
 
     if ( res )
