@@ -1,4 +1,4 @@
-/*  $Id: ncbithr.cpp 536050 2017-05-15 16:18:35Z grichenk $
+/*  $Id: ncbithr.cpp 560980 2018-03-29 19:14:24Z gouriano $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -43,6 +43,7 @@
 
 #include <ncbi_pch.hpp>
 #include <corelib/ncbi_param.hpp>
+#include <corelib/request_ctx.hpp>
 #include <corelib/ncbi_system.hpp>
 #include <corelib/error_codes.hpp>
 #ifdef NCBI_POSIX_THREADS
@@ -121,7 +122,7 @@ void CUsedTlsBases::Register(CTlsBase* tls)
 void CUsedTlsBases::Deregister(CTlsBase* tls)
 {
     CMutexGuard tls_cleanup_guard(s_TlsCleanupMutex);
-    xncbi_Verify(m_UsedTls.erase(tls));
+    xncbi_VerifyAndErrorReport(m_UsedTls.erase(tls));
     if (tls->m_AutoDestroy) {
         tls->RemoveReference();
     }
@@ -200,12 +201,12 @@ void CTlsBase::x_Init(void)
 {
     // Create platform-dependent TLS key (index)
 #if defined(NCBI_WIN32_THREADS)
-    xncbi_Verify((m_Key = TlsAlloc()) != DWORD(-1));
+    xncbi_VerifyAndErrorReport((m_Key = TlsAlloc()) != DWORD(-1));
 #elif defined(NCBI_POSIX_THREADS)
-    xncbi_Verify(pthread_key_create(&m_Key, s_PosixTlsCleanup) == 0);
+    xncbi_VerifyAndErrorReport(pthread_key_create(&m_Key, s_PosixTlsCleanup) == 0);
     // pthread_key_create does not reset the value to 0 if the key has been
     // used and deleted.
-    xncbi_Verify(pthread_setspecific(m_Key, 0) == 0);
+    xncbi_VerifyAndErrorReport(pthread_setspecific(m_Key, 0) == 0);
 #else
     m_Key = 0;
 #endif
@@ -246,7 +247,7 @@ void s_TlsSetValue(TTlsKey& key, void* data, const char* err_message)
 #if defined(NCBI_WIN32_THREADS)
     xncbi_Validate(TlsSetValue(key, data), err_message);
 #elif defined(NCBI_POSIX_THREADS)
-    xncbi_Validate(pthread_setspecific(key, data) == 0, err_message);
+    xncbi_ValidatePthread(pthread_setspecific(key, data), 0, err_message);
 #else
     key = data;
     assert(err_message);  // to get rid of the "unused variable" warning
@@ -513,10 +514,8 @@ CThread::TID CThread::GetSelf(void)
 
 bool CThread::IsMain(void)
 {
-    TID id = sx_ThreadId;
     if (!sm_MainThreadIdInitialized) {
         InitializeMainThreadId();
-        id = sx_ThreadId;
     }
     return sx_ThreadId == sx_GetMainThreadId();
 }
@@ -546,6 +545,11 @@ TWrapperRes CThread::Wrapper(TWrapperArg arg)
 
     bool catch_all = TParamThreadCatchExceptions::GetDefault();
 
+    // Check if parent request context should be used.
+    if ( thread_obj->m_ParentRequestContext ) {
+        CDiagContext::SetRequestContext(thread_obj->m_ParentRequestContext);
+    }
+
     // Run user-provided thread main function here
     if ( catch_all ) {
         try {
@@ -617,7 +621,7 @@ TWrapperRes CThread::Wrapper(TWrapperArg arg)
 
 
 CThread::CThread(void)
-    : m_IsRun(false),
+    : m_Handle(0), m_IsRun(false),
       m_IsDetached(false),
       m_IsJoined(false),
       m_IsTerminated(false),
@@ -642,7 +646,7 @@ CThread::~CThread(void)
 {
 #if defined(NCBI_WIN32_THREADS)
     // close handle if it's not yet closed
-    CFastMutexGuard state_guard(s_ThreadMutex);
+//    CFastMutexGuard state_guard(s_ThreadMutex);
     if ( m_IsRun && m_Handle != NULL ) {
         CloseHandle(m_Handle);
         m_Handle = NULL;
@@ -717,6 +721,10 @@ bool CThread::Run(TRunMode flags)
     ++sm_ThreadsCount;
     try {
 
+        if (flags & fRunCloneRequestContext) {
+            m_ParentRequestContext = CDiagContext::GetRequestContext().Clone();
+        }
+
 #if defined(NCBI_WIN32_THREADS)
         // We need this parameter in WinNT - can not use NULL instead!
         DWORD thread_id;
@@ -929,6 +937,36 @@ void CThread::SetCurrentThreadName(const CTempString&)
 #endif
 
 
+bool CThread::sm_IsExiting = false;
+CTimeout CThread::sm_WaitForThreadsTimeout = CTimeout(0.1);
+
+
+void CThread::SetWaitForAllThreadsTimeout(const CTimeout& timeout)
+{
+    sm_WaitForThreadsTimeout = timeout;
+}
+
+
+bool CThread::WaitForAllThreads(void)
+{
+    if (sm_ThreadsCount == 0) return true;
+    if ( !IsMain() ) return false;
+
+    CStopWatch sw(CStopWatch::eStart);
+    bool infinite = sm_WaitForThreadsTimeout.IsInfinite();
+    unsigned long to = 0;
+    unsigned long q = 10;
+    if ( !infinite ) {
+        to = sm_WaitForThreadsTimeout.GetAsMilliSeconds();
+        if (to < q) q = to;
+    }
+    while (sm_ThreadsCount > 0  &&  (infinite || sw.Elapsed()*1000 < to)) {
+        SleepMilliSec(q);
+    }
+    return sm_ThreadsCount == 0;
+}
+
+
 const char* CThreadException::GetErrCodeString(void) const
 {
     switch (GetErrCode()) {
