@@ -1,4 +1,4 @@
-/*  $Id: so_map.cpp 543910 2017-08-15 14:53:43Z ivanov $
+/*  $Id: so_map.cpp 566742 2018-07-06 13:37:33Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -177,6 +177,7 @@ CSoMap::TYPEMAP CSoMap::mMapSoIdToType = {
     {"SO:0001496", "telomeric_repeat"},
     {"SO:0001649", "nested_repeat"},
     {"SO:0001682", "replication_regulatory_region"},
+    {"SO:0001720", "epigenetically_modified_region"},
     {"SO:0001797", "centromeric_repeat"},
     {"SO:0001833", "V_region"},
     {"SO:0001835", "N_region"},
@@ -189,6 +190,9 @@ CSoMap::TYPEMAP CSoMap::mMapSoIdToType = {
     {"SO:0002094", "non_allelic_homologous_recombination_region"},
     {"SO:0002154", "mitotic_recombination_region"},
     {"SO:0002155", "meiotic_recombination_region"},
+    {"SO:0002190", "enhancer_blocking_element"},
+    {"SO:0002191", "imprinting_control_region"},
+    {"SO:0002205", "response_element"},
     {"SO:0005836", "regulatory_region"},
     {"SO:0005850", "primary_binding_site"},
 
@@ -236,6 +240,8 @@ CSoMap::FEATFUNCMAP CSoMap::mMapFeatFunc = {
     {"direct_repeat",           CSoMap::xFeatureMakeRepeatRegion},
     {"dispersed_repeat",        CSoMap::xFeatureMakeRepeatRegion},
     {"enhancer",                CSoMap::xFeatureMakeRegulatory},
+    {"enhancer_blocking_element", CSoMap::xFeatureMakeRegulatory},
+    {"epigenetically_modified_region", CSoMap::xFeatureMakeRegulatory},
     {"exon",                    CSoMap::xFeatureMakeImp},
     {"five_prime_UTR",          CSoMap::xFeatureMakeImp},
     {"gap",                     CSoMap::xFeatureMakeImp},
@@ -243,6 +249,7 @@ CSoMap::FEATFUNCMAP CSoMap::mMapFeatFunc = {
     {"guide_RNA",               CSoMap::xFeatureMakeNcRna},
     {"hammerhead_ribozyme",     CSoMap::xFeatureMakeNcRna},
     {"iDNA",                    CSoMap::xFeatureMakeImp},
+    {"imprinting_control_region", CSoMap::xFeatureMakeRegulatory},
     {"insulator",               CSoMap::xFeatureMakeRegulatory},
     {"intron",                  CSoMap::xFeatureMakeImp},
     {"inverted_repeat",         CSoMap::xFeatureMakeRepeatRegion},
@@ -294,6 +301,7 @@ CSoMap::FEATFUNCMAP CSoMap::mMapFeatFunc = {
     {"repeat_region",           CSoMap::xFeatureMakeRepeatRegion},
     {"replication_regulatory_region", CSoMap::xFeatureMakeRegulatory},
     {"replication_start_site",  CSoMap::xFeatureMakeMiscFeature},
+    {"response_element",        CSoMap::xFeatureMakeRegulatory},
     {"ribosome_entry_site",     CSoMap::xFeatureMakeRegulatory},
     {"riboswitch",              CSoMap::xFeatureMakeRegulatory},
     {"ribozyme",                CSoMap::xFeatureMakeNcRna},
@@ -374,7 +382,8 @@ string CSoMap::SoTypeToId(
 //  ----------------------------------------------------------------------------
 bool CSoMap::SoTypeToFeature(
     const string& so_type,
-    CSeq_feat& feature)
+    CSeq_feat& feature,
+    bool invalidToRegion)
 //  ----------------------------------------------------------------------------
 {
     string resolved_so_type = ResolveSoAlias(so_type);
@@ -382,7 +391,10 @@ bool CSoMap::SoTypeToFeature(
     if (it != mMapFeatFunc.end()) {
         return (it->second)(resolved_so_type, feature);
     }
-    return xFeatureMakeRegion(so_type, feature);
+    if (invalidToRegion) {
+        return xFeatureMakeRegion(so_type, feature);
+    }
+    return false;
 }
 
 //  ----------------------------------------------------------------------------
@@ -740,6 +752,11 @@ bool CSoMap::FeatureToSoType(
     string& so_type)
 //  ----------------------------------------------------------------------------
 {
+    auto original_type = feature.GetNamedQual("SO_type");
+    if (!original_type.empty()) {
+        so_type = original_type;
+        return true;
+    }
     auto subtype = feature.GetData().GetSubtype();
     TYPEFUNCENTRY cit = mMapTypeFunc.find(subtype);
     if (cit == mMapTypeFunc.end()) {
@@ -820,7 +837,7 @@ bool CSoMap::xMapRegion(
     string& so_type)
 //  ----------------------------------------------------------------------------
 {
-    so_type = "biological_region";
+    so_type = "region";
     return true;
 }
 
@@ -954,11 +971,11 @@ bool CSoMap::xMapMiscRecomb(
         {"meiotic_recombination", "meiotic_recombination_region"},
         {"mitotic_recombination", "mitotic_recombination_region"},
         {"non_allelic_homologous_recombination", "non_allelic_homologous_recombination_region"},
-        {"other", "recombination_region"},
+        {"other", "recombination_feature"},
     };
     string recomb_class = feature.GetNamedQual("recombination_class");
     if (recomb_class.empty()) {
-        so_type = "recombination_region";
+        so_type = "recombination_feature";
         return true;
     }
     auto cit = mapRecombClassToSoType.find(recomb_class);
@@ -966,7 +983,7 @@ bool CSoMap::xMapMiscRecomb(
         auto validClasses = CSeqFeatData::GetRecombinationClassList();
         auto valid = std::find(validClasses.begin(), validClasses.end(), recomb_class);
         if (valid == validClasses.end()) {
-            so_type = "recombination_region";
+            so_type = "recombination_feature";
         }
         else {
             so_type = recomb_class;
@@ -1056,13 +1073,14 @@ bool CSoMap::xMapRegulatory(
 //  ----------------------------------------------------------------------------
 {
     map<string, string> mapRegulatoryClassToSoType = {
-        {"DNase_I_hypersensitive_site", "DNaseI_hypersensitive_site"},
+        {"DNase_I_hypersensitive_site", "DNAseI_hypersensitive_site"},
         {"GC_signal", "GC_rich_promoter_region"},
-        {"enhancer_blocking_element", "regulatory_region"},
-        {"imprinting_control_region", "regulatory_region"},
+        {"enhancer_blocking_element", "enhancer_blocking_element"},
+        {"epigenetically_modified_region", "epigenetically_modified_region"},
+        {"imprinting_control_region", "imprinting_control_region"},
         {"matrix_attachment_region", "matrix_attachment_site"},
         {"other", "regulatory_region"},
-        {"response_element", "regulatory_region"},
+        {"response_element", "response_element"},
         {"ribosome_binding_site", "ribosome_entry_site"},
     };
 
