@@ -1,4 +1,4 @@
-/*  $Id: bioseq_set_handle.cpp 444685 2014-08-27 13:07:18Z vasilche $
+/*  $Id: bioseq_set_handle.cpp 571924 2018-10-04 17:41:25Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -289,6 +289,11 @@ CBioseq_set_Handle::sm_ComplexityTable = {
     4, // eco-set (16)
     4, // gen-prod-set (17)
     4, // wgs-set (18)
+    4, // named-annot (19)
+    4, // named-annot-prod (20)
+    4, // read-set (21)
+    4, // paired-end-reads (22)
+    4, // small-genome-set (23)
     0, // other (255 - processed separately)
 };
 
