@@ -1,4 +1,4 @@
-/*  $Id: scope.cpp 490799 2016-01-28 19:44:01Z vasilche $
+/*  $Id: scope.cpp 554030 2017-12-27 16:11:11Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -328,15 +328,17 @@ void CScope::ResetDataAndHistory(ERemoveDataLoaders)
 }
 
 
-void CScope::RemoveFromHistory(const CBioseq_Handle& bioseq)
+void CScope::RemoveFromHistory(const CBioseq_Handle& bioseq,
+                               EActionIfLocked action)
 {
-    m_Impl->RemoveFromHistory(bioseq.GetTSE_Handle());
+    m_Impl->RemoveFromHistory(bioseq.GetTSE_Handle(), action);
 }
 
 
-void CScope::RemoveFromHistory(const CTSE_Handle& tse)
+void CScope::RemoveFromHistory(const CTSE_Handle& tse,
+                               EActionIfLocked action)
 {
-    m_Impl->RemoveFromHistory(tse);
+    m_Impl->RemoveFromHistory(tse, action);
 }
 
 
@@ -364,7 +366,14 @@ void CScope::RemoveSeq_annot(const CSeq_annot_Handle& annot)
     m_Impl->RemoveTopLevelAnnot(annot);
 }
 
-
+bool CScope::Exists(const CSeq_id& id)
+{
+    return Exists(CSeq_id_Handle::GetHandle(id));
+}
+bool CScope::Exists(const CSeq_id_Handle& id)
+{
+    return m_Impl->Exists(id);
+}
 CScope::TIds CScope::GetIds(const CSeq_id& id, TGetFlags flags)
 {
     return GetIds(CSeq_id_Handle::GetHandle(id), flags);
