@@ -1,4 +1,4 @@
-/*  $Id: scope_info.cpp 545052 2017-08-31 13:54:34Z ivanov $
+/*  $Id: scope_info.cpp 569391 2018-08-21 15:45:28Z vasilche $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -209,6 +209,10 @@ CUnlockedTSEsGuard::~CUnlockedTSEsGuard(void)
             TUnlockedTSEsInternal locks;
             swap(m_UnlockedTSEsInternal, locks);
         }
+        while ( !m_UnlockedTSEsLock.empty() ) {
+            TUnlockedTSEsLock locks;
+            swap(m_UnlockedTSEsLock, locks);
+        }
         st_Guard = 0;
     }
 }
@@ -265,8 +269,83 @@ void CDataSource_ScopeInfo::AddTSE_Lock(const CTSE_Lock& lock)
     _VERIFY(m_TSE_LockSet.AddLock(lock));
 }
 
+/////////////////////////////////////////////////////////////////////////////
+// CScope TSE locking scheme.
 
-CDataSource_ScopeInfo::TTSE_Lock
+// CTSE_ScopeUserLock is a link from user handles, it's also an internal lock.
+//
+// User lock maintains m_UserLockCounter, and when the last handle is deleted
+// the TSE is put into m_TSE_UnlockQueue.
+// The entries pushed out of m_TSE_UnlockQueue lose their internal lock.
+//
+// The TSE with user locks can be forcily released by explicit request:
+// RemoveFromHistory(), RemoveDataLoader(), ResetHistory() etc.
+// In this case all handles become invalidated - disconnected from scope.
+// When a TSE is being forcily released the CTSE_ScopeInfo will remain
+// as it may be referenced from other user-level handles, but the content
+// of CTSE_ScopeInfo is cleared - info->IsAttached() == false.
+
+// CTSE_ScopeInternalLock lock holds CTSE_Lock m_TSE_lock (in CDataSource).
+//
+// Internal locks are either links from user handles,
+// links from another TSE (master to segments, sequence to external annots),
+// or entries in m_TSE_UnlockQueue.
+// When the last internal lock is deleted the m_TSE_lock can be released if
+// the TSE can be reloaded later.
+// When an internal lock is obtained it will be assigned with proper CTSE_Lock
+// if necessary, which could involve reloading of entry from CDataLoader.
+
+// So, possible states of CTSE_ScopeInfo are:
+//   0. detached:
+//     m_DS_Info == null
+//     m_TSE_LockCounter >= 0
+//     m_UserLockCounter >= 0
+//     m_TSE_Lock == null
+//     m_UnloadedInfo == null
+//     m_UsedByTSE, m_UsedTSE_Set: empty
+//     m_ReplacedTSE == null
+//     m_BioseqById, m_ScopeInfoMap: empty
+//     not in m_TSE_UnlockQueue
+//   1. attached, unlocked
+//     m_DS_Info != null
+//     m_TSE_LockCounter == 0
+//     m_UserLockCounter == 0
+//     m_TSE_Lock, m_UnloadedInfo: exactly one of them is null
+//     m_UsedByTSE, m_UsedTSE_Set: empty
+//     m_ReplacedTSE
+//     m_BioseqById, m_ScopeInfoMap
+//     not in m_TSE_UnlockQueue
+//   2. attached, locked, no user handle locks
+//     m_DS_Info != null
+//     m_TSE_LockCounter > 0
+//     m_UserLockCounter == 0
+//     m_TSE_Lock != null
+//     m_UnloadedInfo: any
+//     m_UsedByTSE, m_UsedTSE_Set: any
+//     m_ReplacedTSE: any
+//     m_BioseqById: any
+//     m_ScopeInfoMap: any
+//     may be m_TSE_UnlockQueue
+//   3. attached, locked, with user handle locks
+//     m_DS_Info != null
+//     m_TSE_LockCounter > 0
+//     m_UserLockCounter > 0
+//     m_TSE_Lock != null
+//     m_UnloadedInfo: any
+//     m_UsedByTSE, m_UsedTSE_Set: any
+//     m_ReplacedTSE: any
+//     m_BioseqById: any
+//     m_ScopeInfoMap: any
+//     not in m_TSE_UnlockQueue
+
+// get user lock for a CTSE_Lock (in CDataSource)
+// create CTSE_ScopeInfo if necessary
+// preconditions:
+//   lock != null
+//   m_UserLockCounter >= 0 (any)
+
+
+CTSE_ScopeUserLock
 CDataSource_ScopeInfo::GetTSE_Lock(const CTSE_Lock& lock)
 {
     CTSE_ScopeUserLock ret;
@@ -292,6 +371,7 @@ CDataSource_ScopeInfo::GetTSE_Lock(const CTSE_Lock& lock)
         }
         _ASSERT(info->IsAttached() && &info->GetDSInfo() == this);
         info->m_TSE_LockCounter.Add(1);
+        info->m_UserLockCounter.Add(1);
         {{
             // first remove the TSE from unlock queue
             TTSE_LockSetMutex::TWriteLockGuard guard2(m_TSE_UnlockQueueMutex);
@@ -304,6 +384,7 @@ CDataSource_ScopeInfo::GetTSE_Lock(const CTSE_Lock& lock)
         }}
         info->SetTSE_Lock(lock);
         ret.Reset(info);
+        _VERIFY(info->m_UserLockCounter.Add(-1) > 0);
         _VERIFY(info->m_TSE_LockCounter.Add(-1) > 0);
         _ASSERT(info->GetTSE_Lock() == lock);
     }}
@@ -370,40 +451,14 @@ CDataSource_ScopeInfo::x_FindBestTSEInIndex(const CSeq_id_Handle& idh) const
 }
 
 
-void CDataSource_ScopeInfo::UpdateTSELock(CTSE_ScopeInfo& tse, CTSE_Lock lock)
-{
-    {{
-        // first remove the TSE from unlock queue
-        TTSE_LockSetMutex::TWriteLockGuard guard(m_TSE_UnlockQueueMutex);
-        // TSE must be locked already by caller
-        _ASSERT(tse.m_TSE_LockCounter.Get() > 0);
-        m_TSE_UnlockQueue.Erase(&tse);
-        // TSE must be still locked by caller even after removing it
-        // from unlock queue
-        _ASSERT(tse.m_TSE_LockCounter.Get() > 0);
-    }}
-    if ( !tse.GetTSE_Lock() ) {
-        // OK, we need to update the lock
-        if ( !lock ) { // obtain lock from CDataSource
-            lock = tse.m_UnloadedInfo->LockTSE();
-            _ASSERT(lock);
-        }
-        tse.SetTSE_Lock(lock);
-        _ASSERT(tse.GetTSE_Lock() == lock);
-    }
-    _ASSERT(tse.m_TSE_LockCounter.Get() > 0);
-    _ASSERT(tse.GetTSE_Lock());
-}
-
-
 // Called by destructor of CTSE_ScopeUserLock when lock counter goes to 0
-void CDataSource_ScopeInfo::ReleaseTSELock(CTSE_ScopeInfo& tse)
+void CDataSource_ScopeInfo::ReleaseTSEUserLock(CTSE_ScopeInfo& tse)
 {
     CUnlockedTSEsGuard guard;
     {{
         CTSE_ScopeInternalLock unlocked;
         TTSE_LockSetMutex::TWriteLockGuard tse_guard(m_TSE_UnlockQueueMutex);
-        if ( tse.m_TSE_LockCounter.Get() > 0 ) {
+        if ( tse.m_UserLockCounter.Get() > 0 ) {
             // relocked already
             return;
         }
@@ -411,6 +466,7 @@ void CDataSource_ScopeInfo::ReleaseTSELock(CTSE_ScopeInfo& tse)
             // already unlocked
             return;
         }
+        m_TSE_UnlockQueue.Erase(&tse);
         m_TSE_UnlockQueue.Put(&tse, CTSE_ScopeInternalLock(&tse), &unlocked);
         if ( unlocked ) {
             CUnlockedTSEsGuard::SaveInternal(unlocked);
@@ -419,6 +475,29 @@ void CDataSource_ScopeInfo::ReleaseTSELock(CTSE_ScopeInfo& tse)
 }
 
 
+// Called when lock counter becomes non-zero
+void CDataSource_ScopeInfo::AcquireTSEUserLock(CTSE_ScopeInfo& tse)
+{
+    TTSE_LockSetMutex::TWriteLockGuard tse_guard(m_TSE_UnlockQueueMutex);
+    m_TSE_UnlockQueue.Erase(&tse);
+    if ( !tse.m_TSE_Lock ) {
+        CDataSource_ScopeInfo* ds = tse.m_DS_Info;
+        if ( !ds ) {
+            tse.m_UserLockCounter.Add(-1);
+            NCBI_THROW(CCoreException, eNullPtr,
+                       "CTSE_ScopeInfo is not attached to CScope");
+        }
+        // obtain lock from CDataSource
+        CTSE_Lock lock = tse.m_UnloadedInfo->LockTSE();
+        _ASSERT(lock);
+        tse.SetTSE_Lock(lock);
+        _ASSERT(tse.GetTSE_Lock() == lock);
+        _ASSERT(tse.m_UserLockCounter.Get() > 0);
+    }
+    _ASSERT(tse.m_TSE_Lock);
+}
+
+
 // Called by destructor of CTSE_ScopeInternalLock when lock counter goes to 0
 // CTSE_ScopeInternalLocks are stored in m_TSE_UnlockQueue 
 void CDataSource_ScopeInfo::ForgetTSELock(CTSE_ScopeInfo& tse)
@@ -477,12 +556,24 @@ void CDataSource_ScopeInfo::ResetHistory(int action_if_locked)
         TTSE_InfoMapMutex::TWriteLockGuard guard1(m_TSE_InfoMapMutex);
         tses.reserve(m_TSE_InfoMap.size());
         ITERATE ( TTSE_InfoMap, it, m_TSE_InfoMap ) {
+            if ( it->second->IsUserLocked() ) {
+                if ( action_if_locked == CScope::eKeepIfLocked ) {
+                    // skip locked TSEs
+                    continue;
+                }
+                if ( action_if_locked == CScope::eThrowIfLocked ) {
+                    // there are locked TSEs
+                    NCBI_THROW(CObjMgrException, eLockedData,
+                               "Cannot reset scope's history "
+                               "because TSE is locked");
+                }
+            }
             tses.push_back(it->second);
         }
     }}
     CUnlockedTSEsGuard guard;
     ITERATE ( TTSEs, it, tses ) {
-        it->GetNCObject().RemoveFromHistory(action_if_locked);
+        RemoveFromHistory(it->GetNCObject());
     }
 }
 
@@ -497,7 +588,8 @@ void CDataSource_ScopeInfo::RemoveFromHistory(CTSE_ScopeInfo& tse,
     }
     tse.RestoreReplacedTSE();
     _VERIFY(m_TSE_InfoMap.erase(tse.GetBlobId()));
-    tse.m_TSE_LockCounter.Add(1); // to prevent storing into m_TSE_UnlockQueue
+    // prevent storing into m_TSE_UnlockQueue
+    _VERIFY(tse.m_UserLockCounter.Add(1) > 0);
     // remove TSE lock completely
     {{
         TTSE_LockSetMutex::TWriteLockGuard guard2(m_TSE_UnlockQueueMutex);
@@ -514,7 +606,7 @@ void CDataSource_ScopeInfo::RemoveFromHistory(CTSE_ScopeInfo& tse,
         tse.ResetTSE_Lock();
     }
     tse.x_DetachDS();
-    tse.m_TSE_LockCounter.Add(-1); // restore lock counter
+    _VERIFY(tse.m_UserLockCounter.Add(-1) >= 0); // restore lock counter
     _ASSERT(!tse.GetTSE_Lock());
     _ASSERT(!tse.m_DS_Info);
 }
@@ -916,42 +1008,81 @@ const CTSE_ScopeInfo::TSeqIds& CTSE_ScopeInfo::GetBioseqsIds(void) const
 }
 
 
-void CTSE_ScopeInfo_Base::x_LockTSE(void)
+/////////////////////////////////////////////////////////////////////////////
+// TSE locking support classes
+
+void CTSE_ScopeInfo::x_InternalLockTSE(void)
 {
-    CTSE_ScopeInfo* tse = static_cast<CTSE_ScopeInfo*>(this);
-    if ( !tse->m_TSE_Lock ) {
-        CDataSource_ScopeInfo* ds = tse->m_DS_Info;
-        if ( !ds ) {
-            tse->m_TSE_LockCounter.Add(-1);
-            NCBI_THROW(CCoreException, eNullPtr,
-                       "CTSE_ScopeInfo is not attached to CScope");
+    _VERIFY(m_TSE_LockCounter.Add(1) > 0);
+}
+
+
+void CTSE_ScopeInfo::x_InternalUnlockTSE(void)
+{
+    if ( m_TSE_LockCounter.Add(-1) == 0 ) {
+        _ASSERT(CanBeUnloaded());
+        if ( IsAttached() ) {
+            GetDSInfo().ForgetTSELock(*this);
         }
-        ds->UpdateTSELock(*tse, CTSE_Lock());
     }
-    _ASSERT(tse->m_TSE_Lock);
 }
 
 
-void CTSE_ScopeInfo_Base::x_UserUnlockTSE(void)
+void CTSE_ScopeInfo::x_UserLockTSE(void)
 {
-    CTSE_ScopeInfo* tse = static_cast<CTSE_ScopeInfo*>(this);
-    _ASSERT(tse->CanBeUnloaded());
-    if ( tse->IsAttached() ) {
-        tse->GetDSInfo().ReleaseTSELock(*tse);
+    if ( m_UserLockCounter.Add(1) == 1 || !GetTSE_Lock() ) {
+        //_ASSERT(CanBeUnloaded());
+        if ( IsAttached() ) {
+            GetDSInfo().AcquireTSEUserLock(*this);
+        }
     }
 }
 
 
-void CTSE_ScopeInfo_Base::x_InternalUnlockTSE(void)
+void CTSE_ScopeInfo::x_UserUnlockTSE(void)
 {
-    CTSE_ScopeInfo* tse = static_cast<CTSE_ScopeInfo*>(this);
-    _ASSERT(tse->CanBeUnloaded());
-    if ( tse->IsAttached() ) {
-        tse->GetDSInfo().ForgetTSELock(*tse);
+    if ( m_UserLockCounter.Add(-1) == 0 ) {
+        //_ASSERT(CanBeUnloaded());
+        if ( IsAttached() ) {
+            GetDSInfo().ReleaseTSEUserLock(*this);
+        }
     }
 }
 
 
+void CTSE_ScopeInternalLocker::Lock(CTSE_ScopeInfo* tse) const
+{
+    CObjectCounterLocker::Lock(tse);
+    tse->x_InternalLockTSE();
+}
+
+
+void CTSE_ScopeInternalLocker::Unlock(CTSE_ScopeInfo* tse) const
+{
+    tse->x_InternalUnlockTSE();
+    CObjectCounterLocker::Unlock(tse);
+}
+
+
+void CTSE_ScopeUserLocker::Lock(CTSE_ScopeInfo* tse) const
+{
+    CObjectCounterLocker::Lock(tse);
+    tse->x_InternalLockTSE();
+    tse->x_UserLockTSE();
+}
+
+
+void CTSE_ScopeUserLocker::Unlock(CTSE_ScopeInfo* tse) const
+{
+    tse->x_UserUnlockTSE();
+    tse->x_InternalUnlockTSE();
+    CObjectCounterLocker::Unlock(tse);
+}
+
+// end of TSE locking support classes
+/////////////////////////////////////////////////////////////////////////////
+
+
 bool CTSE_ScopeInfo::x_SameTSE(const CTSE_Info& tse) const
 {
     return m_TSE_LockCounter.Get() > 0 && m_TSE_Lock && &*m_TSE_Lock == &tse;
@@ -962,6 +1093,7 @@ void CTSE_ScopeInfo::ReleaseUsedTSEs(void)
 {
     // release all used TSEs
     TUsedTSE_LockSet used;
+    CTSE_ScopeInternalLock self_lock;
     CFastMutexGuard guard(sx_UsedTSEMutex);
     NON_CONST_ITERATE ( TUsedTSE_LockSet, it, m_UsedTSE_Set ) {
         _ASSERT(it->second->m_UsedByTSE == this);
@@ -969,6 +1101,7 @@ void CTSE_ScopeInfo::ReleaseUsedTSEs(void)
     }
     m_UsedTSE_Set.swap(used);
     if ( m_UsedByTSE ) {
+        self_lock.Reset(this); // prevent recursive deletion
         m_UsedByTSE->m_UsedTSE_Set.erase(ConstRef(this));
         m_UsedByTSE = 0;
     }
@@ -1202,21 +1335,21 @@ int CTSE_ScopeInfo::x_GetDSLocksCount(void) const
 }
 
 
-bool CTSE_ScopeInfo::IsLocked(void) const
+bool CTSE_ScopeInfo::IsUserLocked(void) const
 {
-    return int(m_TSE_LockCounter.Get()) > x_GetDSLocksCount();
+    return int(m_UserLockCounter.Get()) > 0;
 }
 
 
-bool CTSE_ScopeInfo::LockedMoreThanOnce(void) const
+bool CTSE_ScopeInfo::IsUserLockedMoreThanOnce(void) const
 {
-    return int(m_TSE_LockCounter.Get()) > x_GetDSLocksCount() + 1;
+    return int(m_UserLockCounter.Get()) > 1;
 }
 
 
 void CTSE_ScopeInfo::RemoveFromHistory(int action_if_locked, bool drop_from_ds)
 {
-    if ( IsLocked() ) {
+    if ( IsUserLockedMoreThanOnce() ) {
         switch ( action_if_locked ) {
         case CScope::eKeepIfLocked:
             return;
@@ -1463,6 +1596,10 @@ void CTSE_ScopeInfo::RemoveLastInfoLock(CScopeInfo_Base& info)
     CUnlockedTSEsGuard guard;
     {{
         CMutexGuard guard_mtx(m_TSE_LockMutex);
+        if (!info.m_TSE_Handle.m_TSE) {
+            // already unlocked
+            return;
+        }
         if ( info.m_LockCounter.Get() > 0 ) {
             // already locked again
             return;
