@@ -1,4 +1,4 @@
-/*  $Id: autodef.cpp 536624 2017-05-22 10:30:37Z bollin $
+/*  $Id: autodef.cpp 572646 2018-10-17 16:58:07Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -187,10 +187,10 @@ unsigned int CAutoDef::GetNumAvailableModifiers()
 
 
 struct SAutoDefModifierComboSort {
-    bool operator()(const CAutoDefModifierCombo& s1,
-                    const CAutoDefModifierCombo& s2) const
+    bool operator()(const CRef<CAutoDefModifierCombo>& s1,
+                    const CRef<CAutoDefModifierCombo>& s2) const
     {
-        return (s1 < s2);
+        return (*s1 < *s2);
     }
 };
 
@@ -202,7 +202,7 @@ CAutoDefModifierCombo * CAutoDef::FindBestModifierCombo()
     TModifierComboVector  combo_list;
 
     combo_list.clear();
-    combo_list.push_back (new CAutoDefModifierCombo(&m_OrigModCombo));
+    combo_list.emplace_back (new CAutoDefModifierCombo(&m_OrigModCombo));
 
 
     TModifierComboVector tmp, add_list;
@@ -226,7 +226,7 @@ CAutoDefModifierCombo * CAutoDef::FindBestModifierCombo()
             if (!tmp.empty()) {
                 stop = false;
                 for (k = 0; k < tmp.size(); k++) {
-                    add_list.push_back (new CAutoDefModifierCombo(tmp[k]));
+                    add_list.emplace_back (new CAutoDefModifierCombo(tmp[k]));
                 }
                 it = combo_list.erase (it);
             } else {
@@ -235,7 +235,7 @@ CAutoDefModifierCombo * CAutoDef::FindBestModifierCombo()
             tmp.clear();
         }
         for (k = 0; k < add_list.size(); k++) {
-            combo_list.push_back (new CAutoDefModifierCombo(add_list[k]));
+            combo_list.emplace_back (new CAutoDefModifierCombo(add_list[k]));
         }
         add_list.clear();
         std::sort (combo_list.begin(), combo_list.end(), SAutoDefModifierComboSort());
@@ -248,13 +248,7 @@ CAutoDefModifierCombo * CAutoDef::FindBestModifierCombo()
         mod_list.push_back (CAutoDefSourceModifierInfo(*it));
     }
 
-    best = combo_list[0];
-    combo_list[0] = NULL;
-    for (k = 1; k < combo_list.size(); k++) {
-       delete combo_list[k];
-       combo_list[k] = NULL;
-    }
-    return best;
+    return combo_list[0].Release();
 }
 
 
@@ -630,6 +624,23 @@ bool CAutoDef::x_Is5SList(CFeat_CI feat_ci)
 } 
 
 
+bool CAutoDef::x_IsSingleMiscFeat(CFeat_CI feat_ci)
+{
+    if (!feat_ci || 
+        feat_ci->GetData().GetSubtype() != CSeqFeatData::eSubtype_misc_feature ||
+        !feat_ci->IsSetComment()) {
+        return false;
+    }
+    bool is_single = true;
+    ++feat_ci;
+    if (feat_ci) {
+        is_single = false;
+    }
+    feat_ci.Rewind();
+    return is_single;
+}
+
+
 bool s_HasPromoter(CBioseq_Handle bh)
 {
     bool has_promoter = false;
@@ -691,6 +702,8 @@ string CAutoDef::x_GetFeatureClauses(CBioseq_Handle bh)
         return "5S ribosomal RNA gene region";
     }
 
+    bool is_single_misc_feat = x_IsSingleMiscFeat(feat_ci);
+
     while (feat_ci)
     { 
         const CSeq_feat& cf = feat_ci->GetOriginalFeature();
@@ -732,8 +745,9 @@ string CAutoDef::x_GetFeatureClauses(CBioseq_Handle bh)
             } else if (subtype == CSeqFeatData::eSubtype_misc_feature) {
 				// some misc-features may require more parsing
                 new_clause = new CAutoDefFeatureClause(bh, cf, mapped_loc);
-                if (m_Options.GetMiscFeatRule() == CAutoDefOptions::eDelete
-                    || (m_Options.GetMiscFeatRule() == CAutoDefOptions::eNoncodingProductFeat && !new_clause->IsNoncodingProductFeat())) {
+                if (!is_single_misc_feat &&
+                    (m_Options.GetMiscFeatRule() == CAutoDefOptions::eDelete
+                     || (m_Options.GetMiscFeatRule() == CAutoDefOptions::eNoncodingProductFeat && !new_clause->IsNoncodingProductFeat()))) {
                     delete new_clause;
                     new_clause = NULL;
                 } else if (m_Options.GetMiscFeatRule() == CAutoDefOptions::eCommentFeat) {
@@ -810,11 +824,6 @@ string CAutoDef::x_GetFeatureClauses(CBioseq_Handle bh)
     if (m_Options.GetSuppressMobileElementSubfeatures()) {
         main_clause.SuppressMobileElementAndInsertionSequenceSubfeatures();
     }
-
-    // if this is a segment, remove genes, mRNAs, and CDSs
-    // unless they end on this segment or have subclauses that need their protein/gene
-    // information
-    main_clause.RemoveNonSegmentClauses(range);
     
     main_clause.Label(m_Options.GetSuppressAlleles());
 
@@ -1025,6 +1034,7 @@ string CAutoDef::x_GetNonFeatureListEnding()
             end = ", partial genome.";
             break;
         case CAutoDefOptions::eSequence:
+        case CAutoDefOptions::eListAllFeatures:
             end = " sequence.";
             break;
         default:
@@ -1060,59 +1070,70 @@ bool IsInGenProdSet(CBioseq_Handle bh)
 }
 
 
+string CAutoDef::x_GetOneNonFeatureClause(CBioseq_Handle bh, unsigned int genome_val)
+{
+    string feature_clauses = "";
+
+    string organelle = "";
+        
+    if (m_Options.GetFeatureListType() != CAutoDefOptions::eSequence
+        || genome_val == CBioSource::eGenome_apicoplast
+        || genome_val == CBioSource::eGenome_chloroplast
+        || genome_val == CBioSource::eGenome_kinetoplast
+        || genome_val == CBioSource::eGenome_leucoplast
+        || genome_val == CBioSource::eGenome_mitochondrion
+        || genome_val == CBioSource::eGenome_plastid) {
+        organelle = OrganelleByGenome(genome_val);
+    }
+    if (!NStr::IsBlank(organelle)) {
+        feature_clauses = " " + organelle;
+    } else if (m_Options.GetFeatureListType() == CAutoDefOptions::eSequence) {
+        string biomol = "";
+        CSeqdesc_CI mi(bh, CSeqdesc::e_Molinfo);
+        if (mi && mi->GetMolinfo().IsSetBiomol()) {
+            if (mi->GetMolinfo().GetBiomol() == CMolInfo::eBiomol_mRNA) {
+                biomol = "mRNA";
+            } else {
+                biomol = CMolInfo::GetBiomolName(mi->GetMolinfo().GetBiomol());
+            }
+        }
+        if (!NStr::IsBlank(biomol)) {
+            feature_clauses = " " + biomol;
+        }
+    }
+
+    feature_clauses += x_GetNonFeatureListEnding();
+    return feature_clauses;
+}
+
+
 string CAutoDef::GetOneFeatureClauseList(CBioseq_Handle bh, unsigned int genome_val)
 {
     string feature_clauses = "";
     if (m_Options.GetFeatureListType() == CAutoDefOptions::eListAllFeatures ||
         (IsBioseqmRNA(bh) && IsInGenProdSet(bh))) {
         feature_clauses = x_GetFeatureClauses(bh);
-        if (!NStr::IsBlank(feature_clauses)) {
-            feature_clauses = " " + feature_clauses;
-        }
-        string ending = x_GetFeatureClauseProductEnding(feature_clauses, bh);
-        if (m_Options.GetAltSpliceFlag()) {
-            if (NStr::IsBlank(ending)) {
-                ending = "; alternatively spliced";
-            } else {
-                ending += ", alternatively spliced";
-            }
-        }
-        feature_clauses += ending;
         if (NStr::IsBlank(feature_clauses)) {
-            feature_clauses = ".";
+            feature_clauses = x_GetOneNonFeatureClause(bh, genome_val);
         } else {
-            feature_clauses += ".";
-        }
-    } else {
-        string organelle = "";
-        
-        if (m_Options.GetFeatureListType() != CAutoDefOptions::eSequence
-            || genome_val == CBioSource::eGenome_apicoplast
-            || genome_val == CBioSource::eGenome_chloroplast
-            || genome_val == CBioSource::eGenome_kinetoplast
-            || genome_val == CBioSource::eGenome_leucoplast
-            || genome_val == CBioSource::eGenome_mitochondrion
-            || genome_val == CBioSource::eGenome_plastid) {
-            organelle = OrganelleByGenome(genome_val);
-        }
-        if (!NStr::IsBlank(organelle)) {
-            feature_clauses = " " + organelle;
-        } else if (NStr::IsBlank(organelle) && m_Options.GetFeatureListType() == CAutoDefOptions::eSequence) {
-            string biomol = "";
-            CSeqdesc_CI mi(bh, CSeqdesc::e_Molinfo);
-            if (mi && mi->GetMolinfo().IsSetBiomol()) {
-                if (mi->GetMolinfo().GetBiomol() == CMolInfo::eBiomol_mRNA) {
-                    biomol = "mRNA";
+            feature_clauses = " " + feature_clauses;
+            string ending = x_GetFeatureClauseProductEnding(feature_clauses, bh);
+            if (m_Options.GetAltSpliceFlag()) {
+                if (NStr::IsBlank(ending)) {
+                    ending = "; alternatively spliced";
                 } else {
-                    biomol = CMolInfo::GetBiomolName(mi->GetMolinfo().GetBiomol());
+                    ending += ", alternatively spliced";
                 }
             }
-            if (!NStr::IsBlank(biomol)) {
-                feature_clauses = " " + biomol;
+            feature_clauses += ending;
+            if (NStr::IsBlank(feature_clauses)) {
+                feature_clauses = ".";
+            } else {
+                feature_clauses += ".";
             }
         }
-
-        feature_clauses += x_GetNonFeatureListEnding();
+    } else {
+        feature_clauses = x_GetOneNonFeatureClause(bh, genome_val);
     }
     return feature_clauses;
 }
@@ -1193,7 +1214,7 @@ string CAutoDef::GetOneDefLine(CBioseq_Handle bh)
     string org_desc = "Unknown organism";
     unsigned int genome_val = CBioSource::eGenome_unknown;
 
-    CAutoDefModifierCombo *mod_combo = GetEmptyCombo();
+    CRef<CAutoDefModifierCombo> mod_combo(GetEmptyCombo());
     mod_combo->InitFromOptions(m_Options);
 
     for (CSeqdesc_CI dit(bh, CSeqdesc::e_Source); dit; ++dit) {
@@ -1384,6 +1405,109 @@ string CAutoDef::x_GetHumanSTRFeatureClauses(CBioseq_Handle bh, const CUser_obje
 }
 
 
+bool s_ChooseModInModList(bool is_org_mod, int subtype, bool require_all, CAutoDefSourceDescription::TAvailableModifierVector& modifiers)
+{
+    bool rval = false;
+    for (auto & modifier : modifiers) {
+        if (modifier.IsOrgMod() && is_org_mod) {
+            if (modifier.GetOrgModType() == subtype) {
+                if (modifier.AllPresent()) {
+                    rval = true;
+                }
+                else if (modifier.AnyPresent() && !require_all) {
+                    rval = true;
+                }
+                if (rval) {
+                    modifier.SetRequested(true);
+                }
+                break;
+            }
+        }
+        else if (!modifier.IsOrgMod() && !is_org_mod) {
+            if (modifier.GetSubSourceType() == subtype) {
+                if (modifier.AllPresent()) {
+                    rval = true;
+                }
+                else if (modifier.AnyPresent() && !require_all) {
+                    rval = true;
+                }
+                if (rval) {
+                    modifier.SetRequested(true);
+                }
+                break;
+            }
+        }
+    }
+    return rval;
+}
+
+
+CRef<CUser_object> CAutoDef::CreateIDOptions(CSeq_entry_Handle seh)
+{
+    CAutoDef ad;
+    ad.AddSources(seh);
+
+    CAutoDefModifierCombo* src_combo = ad.FindBestModifierCombo();
+    CAutoDefSourceDescription::TAvailableModifierVector modifiers;
+    src_combo->GetAvailableModifiers(modifiers);
+
+    static int subtypes[] = { COrgMod::eSubtype_strain,
+        CSubSource::eSubtype_clone,
+        COrgMod::eSubtype_isolate,
+        CSubSource::eSubtype_haplotype,
+        COrgMod::eSubtype_cultivar,
+        COrgMod::eSubtype_ecotype,
+        COrgMod::eSubtype_breed,
+        COrgMod::eSubtype_specimen_voucher,
+        COrgMod::eSubtype_culture_collection,
+        COrgMod::eSubtype_bio_material };
+    static bool is_orgmod[] = { true, false, true, false, true, true, true, true, true, true };
+    static int num_subtypes = sizeof(subtypes) / sizeof(int);
+
+
+    bool found = false;
+    // first look for best identifier found in all
+    for (int i = 0; i < num_subtypes && !found; i++) {
+        found = s_ChooseModInModList(is_orgmod[i], subtypes[i], true, modifiers);
+    }
+    if (!found) {
+        // if not found in all, use best identifier found in some
+        for (int i = 0; i < num_subtypes && !found; i++) {
+            found = s_ChooseModInModList(is_orgmod[i], subtypes[i], false, modifiers);
+        }
+    }
+    if (!src_combo->AreFeatureClausesUnique()) {
+        // use best
+        for (auto &modifier : modifiers) {
+            if (modifier.AnyPresent()) {
+                if (modifier.IsOrgMod()) {
+                    if (src_combo->HasOrgMod(modifier.GetOrgModType())) {
+                        modifier.SetRequested(true);
+                    }
+                }
+                else if (src_combo->HasSubSource(modifier.GetSubSourceType())) {
+                    modifier.SetRequested(true);
+                }
+            }
+        }
+    }
+
+    CRef<CUser_object> user = ad.GetOptionsObject();
+    CAutoDefOptions options;
+    options.InitFromUserObject(*user);
+    for(const auto &it : modifiers) {
+        if (it.IsRequested()) {
+            if (it.IsOrgMod()) {
+                options.AddOrgMod(it.GetOrgModType());
+            } else {
+                options.AddSubSource(it.GetSubSourceType());
+            }
+        }
+    }
+    user = options.MakeUserObject();
+    return user;
+}
+
 
 END_SCOPE(objects)
 END_NCBI_SCOPE
