@@ -1,4 +1,4 @@
-/*  $Id: autodef_feature_clause.cpp 544439 2017-08-23 11:27:20Z ivanov $
+/*  $Id: autodef_feature_clause.cpp 556773 2018-02-05 14:58:19Z bollin $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -229,13 +229,26 @@ void CAutoDefFeatureClause::x_SetBiomol()
 }
 
 
+bool CAutoDefFeatureClause::IsPseudo(const CSeq_feat& f)
+{
+    bool is_pseudo = false;
+    if (f.CanGetPseudo() && f.IsSetPseudo()) {
+        is_pseudo = true;
+    } else if (f.IsSetQual()) {
+        for (auto& it : f.GetQual()) {
+            if (it->IsSetQual() && NStr::EqualNocase(it->GetQual(), "pseudogene")) {
+                is_pseudo = true;
+                break;
+            }
+        }
+    }
+    return is_pseudo;
+}
+
+
 bool CAutoDefFeatureClause::x_IsPseudo()
 {
-    if (m_MainFeat.CanGetPseudo() && m_MainFeat.IsSetPseudo()) {
-        return true;
-    } else {
-        return false;
-    }
+    return (m_GeneIsPseudo || IsPseudo(m_MainFeat));
 }
 
 
@@ -482,9 +495,7 @@ CAutoDefGeneClause::CAutoDefGeneClause(CBioseq_Handle bh, const CSeq_feat &main_
             m_AlleleName = m_GeneName + m_AlleleName;
         }
     }
-    if (m_MainFeat.CanGetPseudo() && m_MainFeat.GetPseudo()) {
-        m_GeneIsPseudo = true;
-    }
+    m_GeneIsPseudo = IsPseudo(m_MainFeat);
     m_HasGene = true;
 }
 
@@ -600,6 +611,8 @@ string CAutoDefFeatureClause::x_GetGeneName(const CGene_ref& gref, bool suppress
         return gref.GetLocus();
     } else if (!suppress_locus_tag && gref.IsSetLocus_tag() && !NStr::IsBlank(gref.GetLocus_tag())) {
         return gref.GetLocus_tag();
+    } else if (gref.IsSetDesc() && !NStr::IsBlank(gref.GetDesc())) {
+        return gref.GetDesc();
     } else {
         return "";
     }
@@ -996,7 +1009,7 @@ bool CAutoDefFeatureClause::x_GetGenericInterval (string &interval, bool suppres
     }
     
     if (subtype == CSeqFeatData::eSubtype_cdregion
-        && (!m_MainFeat.CanGetPseudo() || !m_MainFeat.GetPseudo())) {
+        && (!x_IsPseudo())) {
         interval += "cds";
         if (m_IsAltSpliced) {
             interval += ", alternatively spliced";
@@ -1202,6 +1215,7 @@ bool CAutoDefFeatureClause::AddGene (CAutoDefFeatureClause_Base *gene_clause, bo
             m_GeneName = gene_clause->GetGeneName();
             m_AlleleName = gene_clause->GetAlleleName();
             m_GeneIsPseudo = gene_clause->GetGeneIsPseudo();
+            m_TypewordChosen = x_GetFeatureTypeWord(m_Typeword);
         }
     }
     
@@ -1386,6 +1400,12 @@ CAutoDefFeatureClause_Base *CAutoDefFeatureClause::FindBestParentClause(CAutoDef
     if (subclause == NULL || subclause == this) {
         return NULL;
     }
+
+	if (!NStr::IsBlank(subclause->GetGeneName()) &&
+		!NStr::IsBlank(this->GetGeneName()) &&
+		!NStr::Equal(subclause->GetGeneName(), this->GetGeneName())) {
+		return NULL;
+	}
     
     best_parent = CAutoDefFeatureClause_Base::FindBestParentClause(subclause, gene_cluster_opp_strand);
     
@@ -1902,7 +1922,8 @@ void CAutoDefParsedClause::SetMiscRNAWord(const string& phrase)
             SetTypewordFirst(false);
             m_Description = phrase.substr(0, NStr::Find(phrase, item_name));
         }
-        if (NStr::EndsWith(phrase, " region")) {
+        if (NStr::EndsWith(phrase, " region") &&
+            (!m_ShowTypewordFirst || m_Description != " region")) {
             SetTypeword(item_name + " region");
         } else {
             SetTypeword(item_name);
