@@ -1,4 +1,4 @@
-/*  $Id: objutil.cpp 544492 2017-08-23 17:44:32Z ivanov $
+/*  $Id: objutil.cpp 573600 2018-10-30 11:55:27Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1210,7 +1210,8 @@ static bool s_IsValidDotVersion(const string& accn)
 
 bool IsValidAccession(const string& accn, EAccValFlag flag)
 {
-    bool valid = s_IsValidAccession(accn);
+    // bool valid = s_IsValidAccession(accn);
+    bool valid = (CSeq_id::IdentifyAccession(accn) != CSeq_id::eAcc_unknown);
     if (valid  &&  flag == eValidateAccDotVer) {
         valid = s_IsValidDotVersion(accn);
     }
@@ -1631,7 +1632,7 @@ NCBI_XOBJEDIT_EXPORT const char* strLinkBaseTransTable =
 NCBI_XOBJEDIT_EXPORT const char* strLinkBasePubmed =
     "https://www.ncbi.nlm.nih.gov/pubmed/";
 NCBI_XOBJEDIT_EXPORT const char* strLinkBaseExpasy =
-    "http://www.expasy.org/enzyme/"; // not government site
+    "https://enzyme.expasy.org/EC/"; // not government site
 NCBI_XOBJEDIT_EXPORT const char* strLinkBaseNucSearch =
     "https://www.ncbi.nlm.nih.gov/sites/entrez?db=Nucleotide&amp;cmd=Search&amp;term=";
 NCBI_XOBJEDIT_EXPORT const char* strLinkBaseGenomePrj =
@@ -1644,6 +1645,8 @@ NCBI_XOBJEDIT_EXPORT const char* strLinkBaseGeneOntologyRef =
     "http://www.geneontology.org/cgi-bin/references.cgi#GO_REF:"; // not government site
 NCBI_XOBJEDIT_EXPORT const char* strLinkBaseUSPTO =
     "http://patft.uspto.gov/netacgi/nph-Parser?patentnumber=";
+NCBI_XOBJEDIT_EXPORT const char* strLinkBaseUniProt =
+    "https://www.uniprot.org/uniprot/";
 
 NCBI_XOBJEDIT_EXPORT const char* strDocLink =
     "https://www.ncbi.nlm.nih.gov/genome/annotation_euk/process/";
