@@ -1,4 +1,4 @@
-/* $Id: showdefline.cpp 529376 2017-03-03 16:10:21Z fongah2 $
+/* $Id: showdefline.cpp 557027 2018-02-08 15:51:06Z zaretska $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -285,9 +285,23 @@ void CShowBlastDefline::x_InitLinkOutInfo(SDeflineInfo* sdl,
                                             int blast_rank,
                                             bool getIdentProteins)
 {
+
+    bool is_mixed_database = (m_IsDbNa == true && m_Ctx)? CAlignFormatUtil::IsMixedDatabase(*m_Ctx): false;
+    if (m_DeflineTemplates && m_DeflineTemplates->advancedView && !is_mixed_database)  return;
+
+
     string linkout_list;
-    TGi cur_gi =  FindGi(cur_id);
-    sdl->linkout = m_LinkoutDB ? m_LinkoutDB->GetLinkout(cur_gi,m_MapViewerBuildName) : 0;    
+    
+    sdl->linkout = CAlignFormatUtil::GetSeqLinkoutInfo(cur_id,                                    
+                                    &m_LinkoutDB, 
+                                    m_MapViewerBuildName,                                    
+                                    sdl->gi);
+    if(!m_LinkoutDB) {
+        m_Option &= ~eLinkout;
+        return;
+    }
+
+
     if(m_LinkoutOrder.empty()) {
         m_ConfigFile.reset(new CNcbiIfstream(".ncbirc"));
         m_Reg.reset(new CNcbiRegistry(*m_ConfigFile));        
@@ -352,8 +366,7 @@ void CShowBlastDefline::x_FillDeflineAndId(const CBioseq_Handle& handle,
     sdl->linkout = 0;
     sdl->is_new = false;
     sdl->was_checked = false;
-    sdl->taxid = 0;
-
+    sdl->taxid = 0;    
     //get psiblast stuff
     
     if(m_SeqStatus){
@@ -376,35 +389,25 @@ void CShowBlastDefline::x_FillDeflineAndId(const CBioseq_Handle& handle,
             }   
         }
     }        
-    //get id (sdl->id, sdl-gi)    
+    //get id (sdl->id, sdl-gi)        
     sdl->id = CAlignFormatUtil::GetDisplayIds(handle,aln_id,use_this_seqid,&sdl->gi,&sdl->taxid,&sdl->textSeqID);
     sdl->alnIDFasta = aln_id.AsFastaString();
 
     //get linkout****
-    if((m_Option & eLinkout)){
-        bool linkout_not_found = true;
+    if((m_Option & eLinkout)){        
         bool getIdentProteins = !m_IsDbNa && bdl.size() > 1;
         for(list< CRef< CBlast_def_line > >::const_iterator iter = bdl.begin();
             iter != bdl.end(); iter++){
             CBioseq::TId& cur_id = (CBioseq::TId &)(*iter)->GetSeqid();
             TGi cur_gi =  FindGi(cur_id);            
-            if(use_this_seqid.empty()){
-                if(sdl->gi == cur_gi){                                     
-                    x_InitLinkOutInfo(sdl,cur_id,blast_rank,getIdentProteins);                    
-                    break;
-                }
-            } else if (CAlignFormatUtil::IsGiList(use_this_seqid)) {
-                list<TGi> use_this_gi = CAlignFormatUtil::StringGiToNumGiList(use_this_seqid);
-                ITERATE(list<TGi>, iter_gi, use_this_gi){
-                    if(cur_gi == *iter_gi){                                             
-                        x_InitLinkOutInfo(sdl,cur_id,blast_rank,getIdentProteins);                            
-                        linkout_not_found = false;
-                        break;
-                    }
-                }
-                if(!linkout_not_found){
-                    break;
-                } 
+            bool match = false;
+            if(!use_this_seqid.empty()){
+                wid = FindBestChoice(cur_id, CSeq_id::WorstRank);
+                match = CAlignFormatUtil::MatchSeqInSeqList(cur_gi, wid, use_this_seqid);
+            }
+            if((use_this_seqid.empty() && sdl->gi == cur_gi) || match) {                    
+                x_InitLinkOutInfo(sdl,cur_id,blast_rank,getIdentProteins); //only initialized if !(m_DeflineTemplates->advancedView && !is_mixed_database)
+                break;
             }            
         }
     }
@@ -428,7 +431,7 @@ void CShowBlastDefline::x_FillDeflineAndId(const CBioseq_Handle& handle,
 		CRange<TSeqPos> seqRange = ((int)m_ScoreList.size() >= blast_rank)? m_ScoreList[blast_rank - 1]->subjRange : CRange<TSeqPos>(0,0);
         bool flip = ((int)m_ScoreList.size() >= blast_rank) ? m_ScoreList[blast_rank - 1]->flip : false;        
         CAlignFormatUtil::SSeqURLInfo seqUrlInfo(user_url,m_BlastType,m_IsDbNa,m_Database,m_Rid,
-                                                 m_QueryNumber,sdl->gi, accession, sdl->linkout,
+                                                 m_QueryNumber,sdl->gi, accession, 0, //linkout = 0, not used any more
                                                  blast_rank,false,(m_Option & eNewTargetWindow) ? true : false,seqRange,flip); 
         seqUrlInfo.resourcesUrl = m_Reg.get() ? m_Reg->Get(m_BlastType, "RESOURCE_URL") : kEmptyStr;
         seqUrlInfo.useTemplates = useTemplates;        
@@ -1424,6 +1427,7 @@ CShowBlastDefline::x_GetDeflineInfo(CConstRef<CSeq_id> id, list<string> &use_thi
     sdl = new SDeflineInfo;
     sdl->id = id;
     sdl->defline = "Unknown";
+    
     try{
         const CBioseq_Handle& handle = m_ScopeRef->GetBioseqHandle(*id);
         x_FillDeflineAndId(handle, *id, use_this_seqid, sdl, blast_rank);
@@ -1435,7 +1439,7 @@ CShowBlastDefline::x_GetDeflineInfo(CConstRef<CSeq_id> id, list<string> &use_thi
         
         if((*id).Which() == CSeq_id::e_Gi){
             sdl->gi = (*id).GetGi();
-        } else {                        
+        } else {                                
             sdl->gi = ZERO_GI;
         }
         sdl->id = id;
@@ -1518,7 +1522,7 @@ string CShowBlastDefline::x_FormatSeqSetHeaders(int isGenomicSeq, bool formatHea
 
 string CShowBlastDefline::x_FormatDeflineTableLine(SDeflineInfo* sdl,SScoreInfo* iter,bool &first_new)
 {
-    string defLine = ((sdl->gi > ZERO_GI) && ((m_Option & eCheckboxChecked) || (m_Option & eCheckbox))) ? x_FormatPsi(sdl, first_new) : m_DeflineTemplates->defLineTmpl;   
+    string defLine = (((m_Option & eCheckboxChecked) || (m_Option & eCheckbox))) ? x_FormatPsi(sdl, first_new) : m_DeflineTemplates->defLineTmpl;
     string dflGi = (m_Option & eShowGi) && (sdl->gi > ZERO_GI) ? "gi|" + NStr::NumericToString(sdl->gi) + "|" : "";
     string seqid;
     if(!sdl->id.Empty()){
@@ -1558,22 +1562,22 @@ string CShowBlastDefline::x_FormatDeflineTableLine(SDeflineInfo* sdl,SScoreInfo*
     }
     /*****************This block of code is for future use with AJAX begin***************************/ 
     string deflId,deflFrmID,deflFastaSeq,deflAccs; 
-    if(sdl->gi == ZERO_GI) {
-        string accession;
+    if(sdl->gi == ZERO_GI) {        
         sdl->id->GetLabel(& deflId, CSeq_id::eContent);
-        deflFrmID =  CAlignFormatUtil::GetLabel(sdl->id);//Just accession without db part like GNOMON: or ti:
-        //deflFastaSeq = sdl->id->AsFastaString();
+        deflFrmID =  CAlignFormatUtil::GetLabel(sdl->id);//Just accession without db part like GNOMON: or ti:        
         deflFastaSeq = NStr::TruncateSpaces(sdl->alnIDFasta);
+        deflAccs = sdl->id->AsFastaString();
     }
     else {        
         deflFrmID = deflId = NStr::NumericToString(sdl->gi);        
         deflFastaSeq = "gi|" + NStr::NumericToString(sdl->gi);
-        deflFastaSeq = NStr::TruncateSpaces(sdl->alnIDFasta);        
-        deflAccs = sdl->id->AsFastaString();
+        deflFastaSeq = NStr::TruncateSpaces(sdl->alnIDFasta);         
+        sdl->id->GetLabel(&deflAccs, CSeq_id::eContent);
     }
     //Setup applog info structure
     if(m_AppLogInfo && (m_AppLogInfo->currInd < m_AppLogInfo->topMatchesNum)) {
         m_AppLogInfo->deflIdVec.push_back(deflId);
+        m_AppLogInfo->accVec.push_back(deflAccs);        
         m_AppLogInfo->taxidVec.push_back(NStr::IntToString(sdl->taxid));
         m_AppLogInfo->queryCoverageVec.push_back(NStr::IntToString(iter->percent_coverage));
         m_AppLogInfo->percentIdentityVec.push_back(NStr::IntToString(iter->percent_identity));
@@ -1650,7 +1654,12 @@ string CShowBlastDefline::x_FormatPsi(SDeflineInfo* sdl, bool &first_new)
 
     replaceBy = (m_Option & eCheckboxChecked) ? "checked=\"checked\"" : "";
     defline = CAlignFormatUtil::MapTemplate(defline,"gi_checked",replaceBy);    
-    defline = CAlignFormatUtil::MapTemplate(defline,"psiGi",NStr::NumericToString(sdl->gi));
+    if(sdl->gi > ZERO_GI) {
+        defline = CAlignFormatUtil::MapTemplate(defline,"psiGi",NStr::NumericToString(sdl->gi));
+    }
+    else {
+        defline = CAlignFormatUtil::MapTemplate(defline,"psiGi",sdl->textSeqID);
+    }
     
     return defline;
 }
