@@ -1,4 +1,4 @@
-/*  $Id: sparse_aln.cpp 520514 2016-11-29 16:11:43Z ivanov $
+/*  $Id: sparse_aln.cpp 520399 2016-11-28 16:26:10Z grichenk $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -699,7 +699,7 @@ string& CSparseAln::GetAlnSeqString(TNumrow row,
         ++it;
         is_first_seg = false;
     }
-    if ( translate ) {
+    if (translate  &&  aln_range.GetLength() >= trim_from + trim_to) {
         buffer.resize((aln_range.GetLength() - trim_from - trim_to) / 3);
     }
     return buffer;
