@@ -1,4 +1,4 @@
-/*  $Id: blastdb_formatter.cpp 389293 2013-02-14 18:42:54Z rafanovi $
+/*  $Id: blastdb_formatter.cpp 500404 2016-05-04 14:59:01Z camacho $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -31,11 +31,6 @@
  *  Implementation of the CBlastDbFormatter class
  */
 
-#ifndef SKIP_DOXYGEN_PROCESSING
-static char const rcsid[] = 
-    "$Id: blastdb_formatter.cpp 389293 2013-02-14 18:42:54Z rafanovi $";
-#endif /* SKIP_DOXYGEN_PROCESSING */
-
 #include <ncbi_pch.hpp>
 #include <objtools/blast/blastdb_format/invalid_data_exception.hpp>
 #include <objtools/blast/blastdb_format/blastdb_formatter.hpp>
