@@ -1,4 +1,4 @@
-/*  $Id: seqdbcommon.cpp 535605 2017-05-10 15:41:23Z ivanov $
+/*  $Id: seqdbcommon.cpp 559887 2018-03-15 17:48:06Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -40,6 +40,7 @@
 #include <objects/general/general__.hpp>
 #include <objtools/blast/seqdb_reader/impl/seqdbgeneral.hpp>
 #include <objtools/blast/seqdb_reader/impl/seqdbatlas.hpp>
+#include <objtools/blast/seqdb_reader/seqidlist_reader.hpp>
 #include <algorithm>
 
 BEGIN_NCBI_SCOPE
@@ -239,18 +240,9 @@ static bool s_SeqDB_DBExists(const string         & dbname,
 
     if (linkoutdb_search) {
         _ASSERT(dbtype == 'p');
-        path.append(".p");
-        vector<string> extn;
-        extn.reserve(4);
-        extn.push_back("ni");
-        extn.push_back("nd");
-        extn.push_back("si");
-        extn.push_back("sd");
-        ITERATE(vector<string>, e, extn) {
-            string candidate(path + *e);
-            if (access.DoesFileExist(candidate)) {
-                return true;
-            }
+        path.append(".sqlite3");
+        if (access.DoesFileExist(path)) {
+            return true;
         }
     } else {
         path.append(".-al");
@@ -603,12 +595,20 @@ void s_InsureOrder(TVector & v)
     }
 }
 
+void CSeqDBGiList::PreprocessIdsForISAMSiLookup()
+{
+	NON_CONST_ITERATE(vector<CSeqDBGiList::SSiOid>, itr, m_SisOids) {
+		string str_id = SeqDB_SimplifyAccession(itr->si);
+		itr->si = NStr::ToLower(str_id);
+	}
+}
 
 void CSeqDBGiList::InsureOrder(ESortOrder order)
 {
     // Code depends on OID order after translation, because various
     // methods of SeqDB use this class for filtering purposes.
-
+	static CFastMutex mtx;
+	CFastMutexGuard mtx_gurad(mtx);
     if ((order < m_CurrentOrder) || (order == eNone)) {
         NCBI_THROW(CSeqDBException,
                    eFileErr,
@@ -1243,9 +1243,9 @@ void SeqDB_ReadMemorySiList(const char * fbeginp,
         while (p< fendp && *p!=' ' && *p!='\t' && *p!='\n' && *p!='\r') ++p;
         if (p > head) {
             string acc(head, p);
-            string str_id = SeqDB_SimplifyAccession(acc);
+            string str_id = NStr::TruncateSpaces(acc, NStr::eTrunc_Both);
             if (str_id != "") {
-                sis.push_back(NStr::ToLower(str_id));
+                sis.push_back(str_id);
             } else {
                 cerr << "WARNING:  " << acc
                      << " is not a valid seqid string." << endl;
@@ -1342,7 +1342,7 @@ void SeqDB_ReadGiList(const string & fname, vector<CSeqDBGiList::SGiOid> & gis,
 
     Int8 file_size = mfile.GetSize();
     const char * fbeginp = (char*) mfile.GetPtr();
-    const char * fendp   = fbeginp + (int)file_size;
+    const char * fendp   = fbeginp + file_size;
 
     SeqDB_ReadMemoryGiList(fbeginp, fendp, gis, in_order);
 }
@@ -1354,7 +1354,7 @@ void SeqDB_ReadTiList(const string & fname, vector<CSeqDBGiList::STiOid> & tis,
 
     Int8 file_size = mfile.GetSize();
     const char * fbeginp = (char*) mfile.GetPtr();
-    const char * fendp   = fbeginp + (int)file_size;
+    const char * fendp   = fbeginp + file_size;
 
     SeqDB_ReadMemoryTiList(fbeginp, fendp, tis, in_order);
 }
@@ -1366,7 +1366,7 @@ void SeqDB_ReadMixList(const string & fname, vector<CSeqDBGiList::SGiOid> & gis,
 
     Int8 file_size = mfile.GetSize();
     const char *fbeginp = (char*) mfile.GetPtr();
-    const char *fendp   = fbeginp + (int) file_size;
+    const char *fendp   = fbeginp + file_size;
 
     SeqDB_ReadMemoryMixList(fbeginp, fendp, gis, tis, sis, in_order);
 }
@@ -1385,15 +1385,19 @@ void SeqDB_ReadGiList(const string & fname, vector<TGi> & gis, bool * in_order)
     }
 }
 
-void SeqDB_ReadSiList(const string & fname, vector<CSeqDBGiList::SSiOid> & sis, bool * in_order)
+void SeqDB_ReadSiList(const string & fname, vector<CSeqDBGiList::SSiOid> & sis, bool * in_order, SBlastSeqIdListInfo & db_info)
 {
-    CMemoryFile mfile(SeqDB_MakeOSPath(fname));
-
-    Int8 file_size = mfile.GetSize();
-    const char *fbeginp = (char*) mfile.GetPtr();
-    const char *fendp   = fbeginp + (int) file_size;
-
-    SeqDB_ReadMemorySiList(fbeginp, fendp, sis, in_order);
+	CMemoryFile mfile(SeqDB_MakeOSPath(fname));
+    if (CBlastSeqidlistFile::GetSeqidlist(mfile, sis, db_info)) {
+    	*in_order = true;
+    	return;
+    }
+    else {
+    	Int8 file_size = mfile.GetSize();
+    	const char *fbeginp = (char*) mfile.GetPtr();
+    	const char *fendp   = fbeginp + file_size;
+    	SeqDB_ReadMemorySiList(fbeginp, fendp, sis, in_order);
+    }
 }
 
 bool CSeqDBNegativeList::FindGi(TGi gi)
@@ -1490,14 +1494,18 @@ bool CSeqDBNegativeList::FindId(const CSeq_id & id, bool & match_type)
         }
     } else {
         match_type = (GetNumSis() > 0) ? true : false;
-        
+
+        if(match_type) {
+        	if(FindSi(GetBlastSeqIdString(id, true))) return true;
+        	if(FindSi(GetBlastSeqIdString(id, false))) return true;
+
+        	// For isam lookup
             Int8 num_id;
             string str_id;
             bool simpler;
             
             SeqDB_SimplifySeqid(*(const_cast<CSeq_id *>(&id)), 0, num_id, str_id, simpler);
             
-        if(match_type) {
             if (FindSi(str_id)) {
                 return true;
             }
@@ -1513,6 +1521,27 @@ bool CSeqDBNegativeList::FindId(const CSeq_id & id, bool & match_type)
     return false;
 }
 
+void CSeqDBNegativeList::PreprocessIdsForISAMSiLookup()
+{
+	NON_CONST_ITERATE(vector<string>, itr, m_Sis) {
+		string str_id = SeqDB_SimplifyAccession(*itr);
+		*itr = NStr::ToLower(str_id);
+	}
+}
+
+void CSeqDBNegativeList::InsureOrder()
+{
+   	static CFastMutex mtx;
+   	CFastMutexGuard mtx_gurad(mtx);
+    if (m_LastSortSize != (m_Gis.size() + m_Tis.size() +m_Sis.size())) {
+        std::sort(m_Gis.begin(), m_Gis.end());
+        std::sort(m_Tis.begin(), m_Tis.end());
+        std::sort(m_Sis.begin(), m_Sis.end());
+
+        m_LastSortSize = m_Gis.size() + m_Tis.size() + m_Sis.size();
+    }
+}
+
 bool CSeqDBGiList::FindId(const CSeq_id & id)
 {
     if (id.IsGi()) {
@@ -1526,6 +1555,10 @@ bool CSeqDBGiList::FindId(const CSeq_id & id)
 
         return FindTi(ti);
     } else {
+    	if(FindSi(GetBlastSeqIdString(id, true))) return true;
+    	if(FindSi(GetBlastSeqIdString(id, false))) return true;
+
+    	/// For isam lookup
         Int8 num_id;
         string str_id;
         bool simpler;
@@ -1554,7 +1587,7 @@ CSeqDBFileGiList::CSeqDBFileGiList(const string & fname, EIdType idtype)
             SeqDB_ReadTiList(fname, m_TisOids, & in_order);
             break;
         case eSiList:
-            SeqDB_ReadSiList(fname, m_SisOids, & in_order);
+            SeqDB_ReadSiList(fname, m_SisOids, & in_order, m_ListInfo);
             break;
         case eMixList:
         	SeqDB_ReadMixList(fname, m_GisOids, m_TisOids, m_SisOids, & in_order);
@@ -1562,7 +1595,7 @@ CSeqDBFileGiList::CSeqDBFileGiList(const string & fname, EIdType idtype)
     }
     m_CurrentOrder = in_order ? eGi : eNone;
 }
-
+/*
 CSeqDBFileGiList::CSeqDBFileGiList(vector<string> fnames, EIdType idtype)
 {
     bool in_order = false;
@@ -1585,7 +1618,7 @@ CSeqDBFileGiList::CSeqDBFileGiList(vector<string> fnames, EIdType idtype)
     }
     m_CurrentOrder = in_order ? eGi : eNone;
 }
-
+*/
 void SeqDB_CombineAndQuote(const vector<string> & dbs,
                            string               & dbname)
 {
@@ -2402,6 +2435,8 @@ s_SeqDB_ParseSeqIDs(const string              & line,
     return ! seqids.empty();
 }
 
+
+
 ESeqDBIdType SeqDB_SimplifyAccession(const string & acc,
                                      Int8         & num_id,
                                      string       & str_id,
@@ -2437,6 +2472,36 @@ ESeqDBIdType SeqDB_SimplifyAccession(const string & acc,
             str_id = seqids.front()->AsFastaString();
             str_id = NStr::ToLower(str_id);
         }
+        else if (!seqids.empty() && seqids.front()->IsLocal()) {
+        	// Chec for gnl dbs
+        	if( acc.find(":") != string::npos) {
+        		static const char* GNL_DBs[] = {"CDD", "SRA", "TSA", "GNOMON", NULL};
+        		string db_tag, gnl_id;
+        		NStr::SplitInTwo(acc, ":", db_tag, gnl_id);
+        		const char** p = GNL_DBs;
+        		for (;  p  &&  *p;  ++p) {
+        			if(NStr::EqualNocase(*p, db_tag.c_str())) {
+        				str_id  = "gnl|" + db_tag + "|"  + gnl_id;
+        				seqids.front().Reset();
+        				CRef<CSeq_id> new_id(new CSeq_id(str_id));
+        				seqids.front() = new_id;
+        				break;
+        			}
+        		}
+        		if(*p == NULL) {
+        			str_id =  acc;
+        		}
+        	}
+        	else {
+        		Int8 n_id = 0;
+        		if (NStr::StringToNumeric<Int8>(acc,&n_id, NStr::fConvErr_NoThrow)) {
+        			str_id = "lcl|" + acc;
+        		}
+        		else {
+        			str_id = acc;
+        		}
+        	}
+        }
         else {
             str_id = acc;
         }
@@ -2457,7 +2522,7 @@ const string SeqDB_SimplifyAccession(const string &acc)
     else return "";
 }
 
-void SeqDB_GetFileExtensions(bool db_is_protein, vector<string>& extn)
+void SeqDB_GetFileExtensions(bool db_is_protein, vector<string>& extn, EBlastDbVersion dbver)
 {
     // NOTE: If more extensions are added, please keep in sync with
     // updatedb.pl's DistributeBlastDbsToBackends
@@ -2472,11 +2537,17 @@ void SeqDB_GetFileExtensions(bool db_is_protein, vector<string>& extn)
     extn.push_back(kExtnMol + "sq");   // sequence file
     extn.push_back(kExtnMol + "ni");   // ISAM numeric index file
     extn.push_back(kExtnMol + "nd");   // ISAM numeric data file
-    extn.push_back(kExtnMol + "si");   // ISAM string index file
-    extn.push_back(kExtnMol + "sd");   // ISAM string data file
+    if (dbver == eBDB_Version4) {
+    	extn.push_back(kExtnMol + "si");   // ISAM string index file
+    	extn.push_back(kExtnMol + "sd");   // ISAM string data file
+    }
     extn.push_back(kExtnMol + "pi");   // ISAM PIG index file
     extn.push_back(kExtnMol + "pd");   // ISAM PIG data file
-
+    if (dbver == eBDB_Version5) {
+    	vector<string> lmdbs;
+    	SeqDB_GetLMDBFileExtensions(db_is_protein, lmdbs);
+    	extn.insert(extn.end(), lmdbs.begin(), lmdbs.end());
+    }
     // Contain masking information
     extn.push_back(kExtnMol + "aa");   // ISAM mask index file
     extn.push_back(kExtnMol + "ab");   // ISAM mask data file (big-endian)
@@ -2488,5 +2559,45 @@ void SeqDB_GetFileExtensions(bool db_is_protein, vector<string>& extn)
     extn.push_back(kExtnMol + "td");   // ISAM trace id data file
 }
 
+
+void SeqDB_GetLMDBFileExtensions(bool db_is_protein, vector<string>& extn)
+{
+
+	static const char * ext[]={"db", "os", "ot", "tf", "to", "db-lock", "tf-lock", NULL};
+	extn.clear();
+	const string kExtnMol(1, db_is_protein ? 'p' : 'n');
+	for(const char ** p=ext; *p != NULL; p++) {
+		extn.push_back(kExtnMol + (*p));
+	}
+}
+
+bool IsStringId(const CSeq_id & id)
+{
+	 switch(id.Which()) {
+	    case CSeq_id::e_Gi:
+	    	return false;
+	    break;
+	    case CSeq_id::e_General:
+	    {
+	    	const CDbtag & dbt = id.GetGeneral();
+            if (dbt.CanGetDb() && ((dbt.GetDb() == "PIG") || (dbt.GetDb() == "ti"))) {
+            	return false;
+            }
+	    }
+	    default:
+	    	return true;
+	    break;
+	 };
+}
+
+string GetBlastSeqIdString(const CSeq_id & seqid, bool version)
+{
+	if(seqid.IsPir() || seqid.IsPrf()) {
+		return seqid.AsFastaString();
+	}
+
+	return seqid.GetSeqIdString(version);
+}
+
 END_NCBI_SCOPE
 
