@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBFILTER_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBFILTER_HPP
 
-/*  $Id: seqdbfilter.hpp 398155 2013-05-03 11:37:45Z camacho $
+/*  $Id: seqdbfilter.hpp 515860 2016-10-06 12:29:07Z camacho $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -158,6 +158,17 @@ public:
         return m_MemBit;
     }
 
+    void DebugDump(CDebugDumpContext ddc, unsigned int depth) const
+    {
+        ddc.SetFrame("CSeqDB_AliasMask");
+        CObject::DebugDump(ddc, depth);
+        ddc.Log("m_MaskType", m_MaskType);
+        ddc.Log("m_Path", m_Path.GetPathS());
+        ddc.Log("m_Begin", m_Begin);
+        ddc.Log("m_End", m_End);
+        ddc.Log("m_MemBit", m_MemBit);
+    }
+
 private:
     /// Type of filtering to apply.
     EMaskType m_MaskType;
