@@ -1,7 +1,7 @@
 #ifndef OBJTOOLS_READERS_SEQDB__SEQDBGILISTSET_HPP
 #define OBJTOOLS_READERS_SEQDB__SEQDBGILISTSET_HPP
 
-/*  $Id: seqdbgilistset.hpp 255926 2011-03-01 13:20:37Z maning $
+/*  $Id: seqdbgilistset.hpp 553487 2017-12-18 14:23:38Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -40,6 +40,7 @@
 
 #include <objtools/blast/seqdb_reader/seqdbcommon.hpp>
 #include "seqdbvolset.hpp"
+#include "seqdblmdbset.hpp"
 
 BEGIN_NCBI_SCOPE
 
@@ -90,7 +91,8 @@ public:
                     const CSeqDBVolSet & vol_set,
                     TListRef             user_list,
                     TNegativeRef         neg_list,
-                    CSeqDBLockHold     & locked);
+                    CSeqDBLockHold     & locked,
+                    const CSeqDBLMDBSet & lmdb_set);
     
     /// Get a reference to a named GI list.
     ///
