@@ -1,4 +1,4 @@
-/*  $Id: seqdboidlist.cpp 536658 2017-05-22 15:48:20Z zaretska $
+/*  $Id: seqdboidlist.cpp 553487 2017-12-18 14:23:38Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -45,13 +45,14 @@ CSeqDBOIDList::CSeqDBOIDList(CSeqDBAtlas              & atlas,
                              CSeqDB_FilterTree        & filters,
                              CRef<CSeqDBGiList>       & gi_list,
                              CRef<CSeqDBNegativeList> & neg_list,
-                             CSeqDBLockHold           & locked)
+                             CSeqDBLockHold           & locked,
+                             const CSeqDBLMDBSet      & lmdb_set)
     : m_Atlas   (atlas),      
       m_Lease   (atlas),      
       m_NumOIDs (0)
 {
     _ASSERT(gi_list.NotEmpty() || neg_list.NotEmpty() || filters.HasFilter());
-    x_Setup( volset, filters, gi_list, neg_list, locked );
+    x_Setup( volset, filters, gi_list, neg_list, locked, lmdb_set);
 }
 
 CSeqDBOIDList::~CSeqDBOIDList()
@@ -65,7 +66,8 @@ void CSeqDBOIDList::x_Setup(const CSeqDBVolSet       & volset,
                             CSeqDB_FilterTree        & filters,
                             CRef<CSeqDBGiList>       & gi_list,
                             CRef<CSeqDBNegativeList> & neg_list,
-                            CSeqDBLockHold           & locked)
+                            CSeqDBLockHold           & locked,
+							const CSeqDBLMDBSet      & lmdb_set)
 {
     // First, get the memory space for the OID bitmap and clear it.
     
@@ -83,23 +85,30 @@ void CSeqDBOIDList::x_Setup(const CSeqDBVolSet       & volset,
                                 volset,
                                 gi_list,
                                 neg_list,
-                                locked);
-    
+                                locked,
+                                lmdb_set);
     // Then get the list of filenames and offsets to overlay onto it.
-    
+
     for(int i = 0; i < volset.GetNumVols(); i++) {
         const CSeqDBVolEntry * v1 = volset.GetVolEntry(i);
         
         CRef<CSeqDB_BitSet> vol_bits =
-            x_ComputeFilters(filters, *v1, gi_list_set, locked);
+            x_ComputeFilters(filters, *v1, gi_list_set, locked, lmdb_set.IsBlastDBVersion5());
         
         m_AllBits->UnionWith(*vol_bits, true);
     }
     
+    if (lmdb_set.IsBlastDBVersion5()  && filters.HasFilter()) {
+   		CSeqDB_BitSet si_bits(0, m_NumOIDs);
+   		si_bits.AssignBitRange(0, m_NumOIDs, true);
+       	x_ComputeFilters(volset, filters, lmdb_set, si_bits);
+		m_AllBits->IntersectWith(si_bits, true);
+    }
+
     if (gi_list.NotEmpty()) {
         x_ApplyUserGiList(*gi_list);
     } else if (neg_list.NotEmpty()) {
-        x_ApplyNegativeList(*neg_list);
+        x_ApplyNegativeList(*neg_list, lmdb_set.IsBlastDBVersion5());
     }
     
     while(m_NumOIDs && (! x_IsSet(m_NumOIDs - 1))) {
@@ -111,7 +120,9 @@ CRef<CSeqDB_BitSet>
 CSeqDBOIDList::x_ComputeFilters(const CSeqDB_FilterTree & filters,
                                 const CSeqDBVolEntry    & vol,
                                 CSeqDBGiListSet         & gis,
-                                CSeqDBLockHold          & locked)
+                                CSeqDBLockHold          & locked,
+                                bool					isBlastDBv5)
+
 {
     const string & vn = vol.Vol()->GetVolName();
     CRef<CSeqDB_FilterTree> ft = filters.Specialize(vn);
@@ -156,7 +167,7 @@ CSeqDBOIDList::x_ComputeFilters(const CSeqDB_FilterTree & filters,
         
         ITERATE(vector< CRef< CSeqDB_FilterTree > >, sub, ft->GetNodes()) {
             CRef<CSeqDB_BitSet> sub_bits =
-                x_ComputeFilters(**sub, vol, gis, locked);
+                x_ComputeFilters(**sub, vol, gis, locked, isBlastDBv5);
             
             volume_map->UnionWith(*sub_bits, true);
         }
@@ -204,13 +215,14 @@ CSeqDBOIDList::x_ComputeFilters(const CSeqDB_FilterTree & filters,
     ITERATE(TFilters, filt, ft->GetFilters()) {
         const CSeqDB_AliasMask & mask = **filt;
         
-        if (mask.GetType() == CSeqDB_AliasMask::eOidRange
-         || mask.GetType() == CSeqDB_AliasMask::eMemBit)
+        if ((mask.GetType() == CSeqDB_AliasMask::eOidRange)
+            || (mask.GetType() == CSeqDB_AliasMask::eMemBit)
+            || (isBlastDBv5 && (mask.GetType() == CSeqDB_AliasMask::eSiList))) {
             continue;
+        }
         
         CRef<CSeqDB_BitSet> f;
         CRef<CSeqDBGiList> idlist;
-        
         switch(mask.GetType()) {
         case CSeqDB_AliasMask::eOidList:
             f = x_GetOidMask(mask.GetPath(), vol_start, vol_end);
@@ -299,14 +311,32 @@ void CSeqDBOIDList::x_ApplyUserGiList(CSeqDBGiList   & gis)
         }
     }
     
+    const vector<blastdb::TOid> & oids_tax = gis.GetOidsForTaxIdsList();
+    for(unsigned int k = 0; k < oids_tax.size(); k++) {
+        if (oids_tax[k] < m_NumOIDs) {
+            gilist_oids->SetBit(oids_tax[k]);
+        }
+    }
     // Intersect the user GI list with the OID bit map.
     
     m_AllBits->IntersectWith(*gilist_oids, true);
 }
 
-void CSeqDBOIDList::x_ApplyNegativeList(CSeqDBNegativeList & nlist)
+void CSeqDBOIDList::x_ApplyNegativeList(CSeqDBNegativeList & nlist, bool is_v5)
                                         
 {
+    // We require a normalized list in order to turn bits off.
+	 m_AllBits->Normalize();
+	 if(is_v5 && ((nlist.GetNumSis() > 0 )|| (nlist.GetNumTaxIds() > 0))){
+	   	const vector<blastdb::TOid> & excluded_oids = nlist.GetExcludedOids();
+	   	for(unsigned int i=0; i < excluded_oids.size(); i++) {
+	   		m_AllBits->ClearBit(excluded_oids[i]);
+	   	}
+
+	   	if(nlist.GetNumGis() == 0 && nlist.GetNumTis() == 0) {
+	   		return;
+	   	}
+	}
     //m_Atlas.Lock(locked);
     
     // Intersect the user GI list with the OID bit map.
@@ -325,10 +355,6 @@ void CSeqDBOIDList::x_ApplyNegativeList(CSeqDBNegativeList & nlist)
         m_AllBits->IntersectWith(new_range, true);
     }
     
-    // We require a normalized list in order to turn bits off.
-    
-    m_AllBits->Normalize();
-    
     // If a 'get next included oid' method was added to the negative
     // list, the following loop could be made a bit faster.
     
@@ -337,6 +363,8 @@ void CSeqDBOIDList::x_ApplyNegativeList(CSeqDBNegativeList & nlist)
             m_AllBits->ClearBit(oid);
         }
     }
+
+
 }
 
 CRef<CSeqDB_BitSet>
@@ -454,5 +482,93 @@ CSeqDBOIDList::DebugDump(CDebugDumpContext ddc, unsigned int depth) const
     ddc.Log("m_AllBits", m_AllBits, depth);
 }
 
+void
+s_GetFilteredOidRange(const CSeqDBVolSet & volset, const vector<string> &  vol_basenames,
+		              vector<const CSeqDBVolEntry * >  & excluded_vols,
+		              CRef<CSeqDBGiList> & si_list)
+{
+	unsigned int num_vol = volset.GetNumVols();
+	vector<bool>  vol_included(num_vol, false);
+	excluded_vols.clear();
+	for(unsigned int i=0; i < num_vol; i++) {
+		const CSeqDBVol * vol = volset.GetVol(i);
+		if(std::find(vol_basenames.begin(), vol_basenames.end(), vol->GetVolName()) != vol_basenames.end()) {
+			vol->AttachVolumeGiList(si_list);
+			continue;
+		}
+		excluded_vols.push_back(volset.GetVolEntry(i));
+	}
+}
+
+bool
+s_IsOidInFilteredVol(blastdb::TOid oid, vector<const CSeqDBVolEntry * >  & excluded_vols)
+{
+	for(unsigned int i = 0; i < excluded_vols.size(); i++) {
+		 const CSeqDBVolEntry & entry = *(excluded_vols[i]);
+		 if ((entry.OIDStart() <= oid) && (entry.OIDEnd()   >  oid)) {
+			 return true;
+		 }
+	}
+	return false;
+}
+
+void
+CSeqDBOIDList::x_ComputeFilters(const CSeqDBVolSet       & volset,
+								const CSeqDB_FilterTree  & filters,
+                           		const CSeqDBLMDBSet      & lmdb_set,
+                           		CSeqDB_BitSet 			 & si_bit)
+{
+	vector<string> si_fnames;
+	vector< vector<string> > si_fnames_vols;
+
+	for(int i = 0; i < volset.GetNumVols(); i++) {
+		const CSeqDBVolEntry & vol = *(volset.GetVolEntry(i));
+	    const string & vn = vol.Vol()->GetVolName();
+        CRef<CSeqDB_FilterTree> ft = filters.Specialize(vn);
+       	ITERATE(CSeqDB_FilterTree::TFilters, itr, ft->GetFilters()){
+        	if((*itr)->GetType() == CSeqDB_AliasMask::eSiList){
+        		string si_name = (*itr)->GetPath().GetPathS();
+        		unsigned int j=0;
+        		for(; j < si_fnames.size(); j++) {
+        			if(si_fnames[j] == si_name) {
+        				si_fnames_vols[j].push_back(vn);
+        				break;
+        			}
+        		}
+        		if( si_fnames.size() == j) {
+        			vector<string> p(1,vn);
+        			si_fnames.push_back(si_name);
+        			si_fnames_vols.push_back(p);
+        		}
+        		si_bit.AssignBitRange(vol.OIDStart(), vol.OIDEnd(), false);
+        	}
+        }
+	}
+
+	for(unsigned int k=0; k < si_fnames.size(); k++) {
+		vector<const CSeqDBVolEntry * > excluded_vols;
+		CRef<CSeqDBGiList> si_list(new CSeqDBFileGiList(si_fnames[k], CSeqDBFileGiList::eSiList));
+		s_GetFilteredOidRange(volset, si_fnames_vols[k], excluded_vols, si_list);
+		vector<string> accs;
+		vector<blastdb::TOid> oids;
+		si_list->GetSiList(accs);
+		lmdb_set.AccessionsToOids(accs, oids);
+		for(unsigned int i=0; i < accs.size(); i++) {
+			if(oids[i] == kSeqDBEntryNotFound) {
+				continue;
+			}
+			if(excluded_vols.size() != 0) {
+				if (s_IsOidInFilteredVol(oids[i], excluded_vols)) {
+					continue;
+				}
+			}
+			si_list->SetSiTranslation(i, oids[i]);
+			si_bit.SetBit(oids[i]);
+		}
+	}
+	return;
+}
+
+
 END_NCBI_SCOPE
 
