@@ -1,4 +1,4 @@
-/*  $Id: seqdbvol.cpp 533967 2017-04-21 15:04:16Z zaretska $
+/*  $Id: seqdbvol.cpp 559232 2018-03-08 14:41:08Z fongah2 $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -116,7 +116,8 @@ CSeqDBVol::OpenSeqFile(CSeqDBLockHold & locked) const{
 
 void
 CSeqDBVol::x_OpenSeqFile(void) const {
-    //m_Atlas.Lock(locked);
+    static CFastMutex mtx;
+    CFastMutexGuard mtx_gurad(mtx);
     if (!m_SeqFileOpened && m_Idx->GetNumOIDs() != 0) {
         m_Seq.Reset(new CSeqDBSeqFile(m_Atlas, m_VolName, (m_IsAA?'p':'n')));
     }
@@ -125,7 +126,8 @@ CSeqDBVol::x_OpenSeqFile(void) const {
 
 void
 CSeqDBVol::x_OpenHdrFile(void) const {
-    //m_Atlas.Lock(locked);
+    static CFastMutex mtx;
+    CFastMutexGuard mtx_gurad(mtx);
     if (!m_HdrFileOpened && m_Idx->GetNumOIDs() != 0) {
         m_Hdr.Reset(new CSeqDBHdrFile(m_Atlas, m_VolName, (m_IsAA?'p':'n')));
     }
@@ -134,7 +136,8 @@ CSeqDBVol::x_OpenHdrFile(void) const {
 
 void
 CSeqDBVol::x_OpenPigFile(void) const{
-    //m_Atlas.Lock(locked);
+    static CFastMutex mtx;
+    CFastMutexGuard mtx_gurad(mtx);
     if (!m_PigFileOpened &&
          CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 'p') &&
          m_Idx->GetNumOIDs() != 0) {
@@ -150,7 +153,8 @@ CSeqDBVol::x_OpenPigFile(void) const{
 
 void
 CSeqDBVol::x_OpenGiFile(void) const{
-    //m_Atlas.Lock(locked);
+    static CFastMutex mtx;
+    CFastMutexGuard mtx_gurad(mtx);
     if (!m_GiFileOpened &&
          CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 'n') &&
          m_Idx->GetNumOIDs() != 0) {
@@ -166,7 +170,8 @@ CSeqDBVol::x_OpenGiFile(void) const{
 
 void
 CSeqDBVol::x_OpenStrFile(void) const{
-    //m_Atlas.Lock(locked);
+    static CFastMutex mtx;
+    CFastMutexGuard mtx_gurad(mtx);
     if (!m_StrFileOpened &&
          CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 's') &&
          m_Idx->GetNumOIDs() != 0) {
@@ -182,7 +187,8 @@ CSeqDBVol::x_OpenStrFile(void) const{
 
 void
 CSeqDBVol::x_OpenTiFile(void) const{
-    //m_Atlas.Lock(locked);
+    static CFastMutex mtx;
+    CFastMutexGuard mtx_gurad(mtx);
     if (!m_TiFileOpened &&
          CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 't') &&
          m_Idx->GetNumOIDs() != 0) {
@@ -198,7 +204,8 @@ CSeqDBVol::x_OpenTiFile(void) const{
 
 void
 CSeqDBVol::x_OpenHashFile(void) const{
-    //m_Atlas.Lock(locked);
+    static CFastMutex mtx;
+    CFastMutexGuard mtx_gurad(mtx);
     if (!m_HashFileOpened &&
          CSeqDBIsam::IndexExists(m_VolName, (m_IsAA?'p':'n'), 'h') &&
          m_Idx->GetNumOIDs() != 0) {
@@ -214,7 +221,8 @@ CSeqDBVol::x_OpenHashFile(void) const{
 
 void
 CSeqDBVol::x_OpenOidFile(void) const{
-    //m_Atlas.Lock(locked);
+    static CFastMutex mtx;
+    CFastMutexGuard mtx_gurad(mtx);
     if (!m_OidFileOpened &&
          CSeqDBGiIndex::IndexExists(m_VolName, (m_IsAA?'p':'n')) &&
          m_Idx->GetNumOIDs() != 0) {
@@ -236,6 +244,11 @@ char CSeqDBVol::x_GetSeqType() const
     return m_Idx->GetSeqType();
 }
 
+string CSeqDBVol::GetLMDBFileName() const
+{
+	return m_Idx->GetLMDBFileName();
+}
+
 int CSeqDBVol::GetSeqLengthProt(int oid, CSeqDBLockHold & locked) const
 {
     TIndx start_offset = 0;
@@ -1045,7 +1058,7 @@ CSeqDBVol::x_GetTaxonomy(int                    oid,
 
 {
     const bool provide_new_taxonomy_info = true;
-    const bool use_taxinfo_cache         = true;
+    const bool use_taxinfo_cache         = (CThread::GetSelf() == 0 ? true : false);
 
     const char * TAX_ORGREF_DB_NAME = "taxon";
 
@@ -1769,6 +1782,49 @@ CSeqDBVol::GetFilteredHeader(int                    oid,
     return x_GetFilteredHeader(oid, NULL);
 }
 
+bool s_IncludeDefline_Taxid(const CBlast_def_line & def, const set<int> & user_tax_ids)
+{
+	CBlast_def_line::TTaxIds tax_ids;
+	if (def.IsSetTaxid()) {
+	    tax_ids.insert(def.GetTaxid());
+	}
+	if(def.IsSetLinks()) {
+		CBlast_def_line::TLinks leaf_ids = def.GetLinks();
+		tax_ids.insert(leaf_ids.begin(), leaf_ids.end());
+	}
+
+	if(user_tax_ids.size() > tax_ids.size()) {
+		ITERATE(CBlast_def_line::TTaxIds, itr, tax_ids) {
+			if(user_tax_ids.find(*itr) != user_tax_ids.end()) {
+				return true;
+			}
+		}
+
+	}
+	else {
+		ITERATE(set<int>, itr, user_tax_ids) {
+			if(tax_ids.find(*itr) != tax_ids.end()) {
+				return true;
+			}
+		}
+	}
+	return false;
+}
+
+bool s_IncludeDefline_NegativeTaxid(const CBlast_def_line & def, const set<int> & user_tax_ids)
+{
+	CBlast_def_line::TTaxIds taxid_set = def.GetTaxIds();
+	if(taxid_set.size() > user_tax_ids.size()) {
+		return true;
+	}
+	ITERATE(CBlast_def_line::TTaxIds, itr, taxid_set) {
+		if(user_tax_ids.find(*itr) == user_tax_ids.end()) {
+			return true;
+		}
+	}
+	return false;
+}
+
 CRef<CBlast_def_line_set>
 CSeqDBVol::x_GetFilteredHeader(int                    oid,
                                bool                 * changed) const
@@ -1778,9 +1834,10 @@ CSeqDBVol::x_GetFilteredHeader(int                    oid,
 
     //m_Atlas.Lock(locked);
 
+    const bool useCache = (CThread::GetSelf() == 0 ? true : false);
     TDeflineCacheItem & cached = m_DeflineCache.Lookup(oid);
 
-    if (cached.first.NotEmpty()) {
+    if (useCache && cached.first.NotEmpty()) {
         if (changed) {
             *changed = cached.second;
         }
@@ -1840,6 +1897,13 @@ CSeqDBVol::x_GetFilteredHeader(int                    oid,
                 have_memb = have_user && have_volume;
             }
 
+            if(have_memb && (!m_UserGiList.Empty()) && (m_UserGiList->GetNumTaxIds() > 0)) {
+               	have_memb = s_IncludeDefline_Taxid(defline, m_UserGiList->GetTaxIdsList());
+            }
+            if(have_memb && (!m_NegativeList.Empty()) && (m_NegativeList->GetNumTaxIds() > 0)) {
+               	have_memb = s_IncludeDefline_NegativeTaxid(defline, m_NegativeList->GetTaxIdsList());
+            }
+
             if (! have_memb) {
                 TBDLLIter eraseme = iter++;
                 dl.erase(eraseme);
@@ -1850,12 +1914,15 @@ CSeqDBVol::x_GetFilteredHeader(int                    oid,
         }
     }
 
-    if (asn_changed) {
-        cached.first = BDLS;
-        cached.second = asn_changed;
-    } else {
-        cached.first = BDLS;
-        cached.second = asn_changed;
+    if (useCache)
+    {
+    	if (asn_changed) {
+       	 cached.first = BDLS;
+       	 cached.second = asn_changed;
+    	} else {
+       	 cached.first = BDLS;
+       	 cached.second = asn_changed;
+    	}
     }
 
     return BDLS;
@@ -2127,7 +2194,7 @@ void CSeqDBVol::IdsToOids(CSeqDBGiList   & ids,
         }
     }
 
-    if (ids.GetNumSis()) {
+    if (ids.GetNumSis() && (GetLMDBFileName() == kEmptyStr)) {
         if (!m_StrFileOpened) x_OpenStrFile();
         if (m_IsamStr.NotEmpty()) {
             m_IsamStr->IdsToOids(m_VolStart, m_VolEnd, ids);
