@@ -1,4 +1,4 @@
-/*  $Id: multisource_util.cpp 500404 2016-05-04 14:59:01Z camacho $
+/*  $Id: multisource_util.cpp 568561 2018-08-07 18:29:39Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -90,7 +90,7 @@ string AccessionToKey(const string & acc)
     if (CheckAccession(acc, gi, seqid, specific)) {
         if (seqid.Empty()) {
             if (gi != ZERO_GI) {
-                str = NStr::IntToString(GI_TO(int, gi));
+                str = NStr::NumericToString(gi);
             }
         } else {
             GetSeqIdKey(*seqid, str);
@@ -121,7 +121,7 @@ bool CheckAccession(const string  & acc,
     }
 
     if (digits) {
-        gi = GI_FROM(int, NStr::StringToInt(acc));
+        gi = NStr::StringToNumeric<TGi>(acc);
         return true;
     }
 
