@@ -1,4 +1,4 @@
-/*  $Id: writedb_impl.cpp 535592 2017-05-10 13:38:07Z rackerst $
+/*  $Id: writedb_impl.cpp 566903 2018-07-10 13:52:23Z rackerst $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -58,7 +58,8 @@ CWriteDB_Impl::CWriteDB_Impl(const string & dbname,
                              EIndexType     indices,
                              bool           parse_ids,
                              bool           long_ids,
-                             bool           use_gi_mask)
+                             bool           use_gi_mask,
+                             EBlastDbVersion    dbver)
     : m_Dbname           (dbname),
       m_Protein          (protein),
       m_Title            (title),
@@ -69,11 +70,13 @@ CWriteDB_Impl::CWriteDB_Impl(const string & dbname,
       m_MaskDataColumn   (-1),
       m_ParseIDs         (parse_ids),
       m_UseGiMask        (use_gi_mask),
+      m_DbVersion        (dbver),
       m_Pig              (0),
       m_Hash             (0),
       m_SeqLength        (0),
       m_HaveSequence     (false),
-      m_LongSeqId        (long_ids)
+      m_LongSeqId        (long_ids),
+      m_LmdbOid          (0)
 {
     CTime now(CTime::eCurrent);
 
@@ -113,6 +116,8 @@ void CWriteDB_Impl::x_ResetSequenceData()
     m_Ambig.erase();
     m_BinHdr.erase();
 
+    m_TaxIds.clear();
+
     NON_CONST_ITERATE(vector<int>, iter, m_HaveBlob) {
         *iter = 0;
     }
@@ -358,6 +363,16 @@ void CWriteDB_Impl::Close()
         if (m_VolumeList.size() > 1 || m_UseGiMask) {
             x_MakeAlias();
         }
+        if ((m_DbVersion == eBDB_Version5)  &&  m_Lmdbdb) {
+        	vector<string> vol_names(m_VolumeList.size());
+        	vector<blastdb::TOid> vol_num_oids(m_VolumeList.size());
+        	for(unsigned i=0; i < m_VolumeList.size(); i++) {
+        		CRef<CWriteDB_Volume> & v = m_VolumeList[i];
+        		vol_names[i] = CDirEntry(v->GetVolumeName()).GetName();
+        		vol_num_oids[i] = v->GetOID();
+        	}
+            m_Lmdbdb->InsertVolumesInfo(vol_names, vol_num_oids);
+        }
 
         m_Volume.Reset();
     }
@@ -678,6 +693,7 @@ CWriteDB_Impl::x_ExtractDeflines(CConstRef<CBioseq>             & bioseq,
                                  const vector< vector<int> >    & membbits,
                                  const vector< vector<int> >    & linkouts,
                                  int                              pig,
+                                 set<Int4>                      & tax_ids,
                                  int                              OID,
                                  bool                             parse_ids,
                                  bool                             long_ids)
@@ -789,6 +805,13 @@ CWriteDB_Impl::x_ExtractDeflines(CConstRef<CBioseq>             & bioseq,
 
         x_SetDeflinesFromBinary(bin_hdr, deflines);
     }
+
+    if ((! deflines.Empty()) && deflines->CanGet()) {
+    	ITERATE(list< CRef<CBlast_def_line> >, defline, deflines->Get()) {
+    		CBlast_def_line::TTaxIds taxid_set = (*defline)->GetTaxIds();
+            tax_ids.insert(taxid_set.begin(), taxid_set.end());
+        }
+    }
 }
 
 void CWriteDB_Impl::x_CookHeader()
@@ -803,6 +826,7 @@ void CWriteDB_Impl::x_CookHeader()
                       m_Memberships,
                       m_Linkouts,
                       m_Pig,
+                      m_TaxIds,
                       OID,
                       m_ParseIDs,
                       m_LongSeqId);
@@ -914,8 +938,8 @@ void CWriteDB_Impl::x_CookSequence()
              break;
 
         default:
-            msg = "Need to write conversion for data type [";
-            msg += NStr::IntToString((int) sd.Which());
+            msg = "Unable to process sequence for entry [";
+            msg += (m_Bioseq->GetId().front())->GetSeqIdString(false);
             msg += "].";
         }
 
@@ -1017,6 +1041,14 @@ void CWriteDB_Impl::x_Publish()
         return;
     }
 
+
+    if(m_DbVersion == eBDB_Version5 && m_Lmdbdb.Empty()) {
+        const string lmdb_fname_w_path = BuildLMDBFileName(m_Dbname, m_Protein);
+        m_Lmdbdb.Reset(new CWriteDB_LMDB(lmdb_fname_w_path));
+        m_Taxdb.Reset(new CWriteDB_TaxID(
+        		          GetFileNameFromExistingLMDBFile(lmdb_fname_w_path, ELMDBFileType::eTaxId2Offsets)));
+    }
+
     x_CookData();
 
     bool done = false;
@@ -1030,6 +1062,11 @@ void CWriteDB_Impl::x_Publish()
                                        m_Hash,
                                        m_Blobs,
                                        m_MaskDataColumn);
+        if (done  &&  (m_DbVersion == eBDB_Version5)  &&  m_Lmdbdb) {
+            m_Lmdbdb->InsertEntries(m_Ids,m_LmdbOid);
+            m_Taxdb->InsertEntries(m_TaxIds, m_LmdbOid);
+            m_LmdbOid++;
+        }
     }
 
     if (! done) {
@@ -1040,6 +1077,7 @@ void CWriteDB_Impl::x_Publish()
         }
 
         {
+
             m_Volume.Reset(new CWriteDB_Volume(m_Dbname,
                                                m_Protein,
                                                m_Title,
@@ -1047,7 +1085,8 @@ void CWriteDB_Impl::x_Publish()
                                                index,
                                                m_MaxFileSize,
                                                m_MaxVolumeLetters,
-                                               m_Indices));
+                                               m_Indices,
+                                               m_DbVersion));
 
             m_VolumeList.push_back(m_Volume);
 
@@ -1078,6 +1117,12 @@ void CWriteDB_Impl::x_Publish()
                                        m_Blobs,
                                        m_MaskDataColumn);
 
+        if (done  &&  (m_DbVersion == eBDB_Version5)  &&  m_Lmdbdb) {
+            m_Lmdbdb->InsertEntries(m_Ids,m_LmdbOid);
+            m_Taxdb->InsertEntries(m_TaxIds, m_LmdbOid);
+            m_LmdbOid++;
+        }
+
         if (! done) {
             NCBI_THROW(CWriteDBException,
                        eArgErr,
@@ -1285,11 +1330,17 @@ RegisterMaskAlgorithm(EBlast_filter_program   program,
                       const string          & options,
                       const string          & name)
 {
-    int algorithm_id = m_MaskAlgoRegistry.Add(program, options);
+    int algorithm_id = m_MaskAlgoRegistry.Add(program, options, name);
 
     string key = NStr::IntToString(algorithm_id);
-    string value = NStr::IntToString((int)program) + ":" +
-         s_EscapeColon(options);
+    string value;
+    if (program == EBlast_filter_program::eBlast_filter_program_other) {
+        value = NStr::IntToString((int)program) + ":" +
+                s_EscapeColon(options) + ":" +
+                s_EscapeColon(name) + ":";
+    } else {
+        value = NStr::IntToString((int)program) + ":" + s_EscapeColon(options);
+    }
 
     if (m_UseGiMask) {
         m_MaskAlgoMap[algorithm_id] = m_GiMasks.size();
@@ -1420,9 +1471,10 @@ CWriteDB_Impl::ExtractBioseqDeflines(const CBioseq & bs, bool parse_ids,
     CConstRef<CBlast_def_line_set> deflines;
     string binary_header;
     vector< vector<int> > v1, v2;
+    set<Int4> t;
 
     CConstRef<CBioseq> bsref(& bs);
-    x_ExtractDeflines(bsref, deflines, binary_header, v2, v2, 0, -1, parse_ids,
+    x_ExtractDeflines(bsref, deflines, binary_header, v2, v2, 0, t, -1, parse_ids,
                       long_seqids);
 
     // Convert to return type
@@ -1510,6 +1562,9 @@ void CWriteDB_Impl::ListFiles(vector<string> & files)
     if (m_VolumeList.size() > 1) {
         files.push_back(x_MakeAliasName());
     }
+    if (m_DbVersion == eBDB_Version5) {
+        files.push_back(BuildLMDBFileName(m_Dbname, m_Protein));
+    }
 }
 
 /// Compute the hash of a (raw) sequence.
@@ -1548,6 +1603,75 @@ void CWriteDB_Impl::x_ComputeHash(const CBioseq & sequence)
 
 #define TAB_REPLACEMENT "   "
 
+static bool s_IsValidPdb(const string& s)
+{
+    size_t len = s.size();
+    // If bare ID, only valid lengths are 4 (no chain ID),
+    // 6 (chain ID is in [A-Z0-9]), or 7 (chain ID is double [A-Z]).
+    // Char before chain ID is '_'.
+    // If long ID, it's prefixed with 'pdb|' and chain ID is preceded by '|'.
+    bool long_seqid = false;
+    if (len == 4  ||  len == 6  ||  len == 7) {
+        if (s.find('|') != NPOS) {
+            return false;
+        }
+    } else if (len == 8  ||  len == 10  ||  len == 11) {
+        string lc("pdb|");
+        string uc("PDB|");
+        if (s.compare(0, lc.size(), lc) != 0
+            &&  s.compare(0, uc.size(), uc) != 0) {
+            return false;
+        }
+        long_seqid = true;
+    } else {
+        return false;
+    }
+    char sep = long_seqid ? '|' : '_';
+    auto it = s.begin();
+    if (long_seqid) {
+        it += 4;    // skip "pdb|"
+    }
+    // Is first character a digit, excluding '0'?
+    if (!isdigit(*it)  ||  *it == '0') {
+        return false;
+    }
+    ++it;
+    // Is each of the next 3 characters either upper-case alpha or digit?
+    for (int i = 0; i < 3; ++i) {
+        if (!isupper(*it)  &&  !isdigit(*it)) {
+            return false;
+        }
+        ++it;
+    }
+    // We're done if length was 4 characters.
+    // Otherwise...
+    if (len > 4) {
+        // Is next character the correct separator?
+        if (*it != sep) {
+            return false;
+        }
+        ++it;
+        if (len == 6) {
+            // Is single-character chain ID either upper-case alpha or digit?
+            if (!isupper(*it)  &&  !isdigit(*it)) {
+                return false;
+            }
+        } else /* len == 7 */ {
+            // Is first character of 2-character chain ID upper-case alpha?
+            if (!isupper(*it)) {
+                return false;
+            }
+            // If so, is second character the same?
+            // (Double upper-case alpha represents a single lower-case alpha.)
+            auto c = *it++;
+            if (c != *it) {
+                return false;
+            }
+        }
+    }
+    return true;
+}
+
 void CWriteDB_Impl::
 x_GetFastaReaderDeflines(const CBioseq                  & bioseq,
                          CConstRef<CBlast_def_line_set> & deflines,
@@ -1679,32 +1803,30 @@ x_GetFastaReaderDeflines(const CBioseq                  & bioseq,
             string remaining(fasta, pos_next, fasta.size() - pos_next);
             fasta.swap(remaining);
 
-            // Parse '|' seperated ids.
+            // Parse ids.  They may or may not be bar-separated.
             list< CRef<CSeq_id> > seqids;
-            if ( (ids.find('|') == NPOS && long_seqids)
-                 || !isalpha((unsigned char)(ids[0]))) {
-
-                 seqids.push_back(CRef<CSeq_id> (new CSeq_id(CSeq_id::e_Local, ids)));
+            if ((ids.find('|') == NPOS  &&  long_seqids)
+                    ||  (!isalpha(ids[0])  &&  !s_IsValidPdb(ids))) {
+                seqids.push_back(CRef<CSeq_id> (new CSeq_id(CSeq_id::e_Local, ids)));
             } else {
-                 CSeq_id::ParseFastaIds(seqids, ids);
+                CSeq_id::ParseFastaIds(seqids, ids);
 
-                 if (!long_seqids) {
+                if (!long_seqids) {
+                    // If accession's molecule type is different than
+                    // expected, change sequence id to local. CFastaReader
+                    // cannot distingush between bare pir protein ids genbank
+                    // nucleotide ids.
+                    for (auto& it: seqids) {
+                        CSeq_id::EAccessionInfo info = it->IdentifyAccession();
+                        if (!it->IsLocal() && !it->IsGi() &&
+                                (info & (CSeq_id::fAcc_prot | CSeq_id::fAcc_nuc)) &&
+                                bioseq.IsAa() == !!(info & CSeq_id::fAcc_nuc)) {
 
-                     // If accession's molecule type is different than
-                     // expected, change sequence id to local. CFastaReader
-                     // cannot distingush between bare pir protein ids genbank
-                     // nucleotide ids.
-                     for (auto& it: seqids) {
-                         CSeq_id::EAccessionInfo info = it->IdentifyAccession();
-                         if (!it->IsLocal() && !it->IsGi() &&
-                             (info & (CSeq_id::fAcc_prot | CSeq_id::fAcc_nuc)) &&
-                             bioseq.IsAa() == !!(info & CSeq_id::fAcc_nuc)) {
-
-                             string label = it->GetSeqIdString(true);
-                             it.Reset(new CSeq_id(CSeq_id::e_Local, label));
-                         }
-                     }
-                 }
+                            string label = it->GetSeqIdString(true);
+                            it.Reset(new CSeq_id(CSeq_id::e_Local, label));
+                        }
+                    }
+                }
             }
 
             // Build the actual defline.
