@@ -1,4 +1,4 @@
-/* $Id: cleanup_utils.cpp 535438 2017-05-08 19:09:23Z kachalos $
+/* $Id: cleanup_utils.cpp 574795 2018-11-19 16:47:08Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -342,46 +342,6 @@ bool CleanVisStringJunk( string &str, bool allow_ellipses )
 }
 
 
-bool RemoveTrailingPeriod(string& str)
-{
-    if (str[str.length() - 1] == '.') {
-        bool remove = true;
-        size_t period = str.length() - 1;
-        size_t amp = str.find_last_of('&');
-        if (amp != NPOS) {
-            remove = false;
-            for (size_t i = amp + 1; i < period; ++i) {
-                if (isspace((unsigned char) str[i])) {
-                    remove = true;
-                    break;
-                }
-            }
-        }
-        if (remove) {
-            str.resize(period);
-            return true;
-        }
-    }
-    return false;
-}
-
-
-bool RemoveTrailingJunk(string& str)
-{
-    SIZE_TYPE end_str =  str.find_last_not_of(" \t\n\r,~;");
-    if (end_str == NPOS) {
-        end_str = 0; // everything is junk.
-    } else {
-        ++end_str; // indexes the first character to remove.
-    }
-    if (end_str >= str.length()) {
-        return false; // nothing to remove.
-    }
-    str.erase(end_str);
-    return true;
-}
-
-
 bool  RemoveSpacesBetweenTildes(string& str)
 {
     static string whites(" \t\n\r");
@@ -565,19 +525,15 @@ bool Asn2gnbkCompressSpaces(string& val)
         curr = next;
     }
 
-    if (curr > 0 && curr != ' ') {
-        *out = curr;
-        out++;
-    }
     *out = '\0';
 
     /* TrimSpacesAroundString but allow leading/trailing tabs/newlines */
 
-    if (str != NULL && str[0] != '\0') {
+    if (str[0] != '\0') {
         dst = str;
         ptr = str;
         ch = *ptr;
-        while (ch != '\0' && ch == ' ') {
+        while (ch == ' ') {
             ptr++;
             ch = *ptr;
         }
@@ -706,18 +662,6 @@ bool TrimSpacesSemicolonsAndCommas(string& val)
 }
 
 
-bool OnlyPunctuation (string str)
-{
-    bool found_other = false;
-    for (unsigned int offset = 0; offset < str.length() && ! found_other; offset++) {
-        if (str[offset] > ' ' && str[offset] != '.' && str[offset] != ','
-            && str[offset] != '~' && str[offset] != ';') {
-            return false;
-        }
-    }
-    return true;
-}
-
 bool RemoveSpaces(string& str)
 {
     if (str.empty()) {
@@ -755,7 +699,7 @@ private:
     CScope *m_scope;
 };
 
-CRef<CSeq_loc> ReadLocFromText(string text, const CSeq_id *id, CScope *scope)
+CRef<CSeq_loc> ReadLocFromText(const string& text, const CSeq_id *id, CScope *scope)
 {
     CGetSeqLocFromStringHelper_ReadLocFromText helper(scope);
     return GetSeqLocFromString(text, id, &helper);
@@ -802,7 +746,7 @@ static ProteinAbbrevData abbreviation_list[] =
 // or three-letter abbreviation.  
 // Use X if the abbreviation is not found.
 
-char ValidAminoAcid (string abbrev)
+char ValidAminoAcid (const string& abbrev)
 {
     char ch = 'X';
     
@@ -826,136 +770,6 @@ char ValidAminoAcid (string abbrev)
 }
 
 
-bool AuthListsMatch(const CAuth_list::TNames& list1, const CAuth_list::TNames& list2, bool all_authors_match)
-{
-    if (list1.Which() != list2.Which()) {
-        return false;
-    } else if (list1.Which() == CAuth_list::TNames::e_Std) {
-        if (all_authors_match) {
-            if (list1.GetStd().size() != list2.GetStd().size()) {
-                return false;
-            } else {
-                for ( CAuth_list::TNames::TStd::const_iterator it1 = (list1.GetStd()).begin(), 
-                                                               it1_end = (list1.GetStd()).end(),
-                                                               it2 = (list2.GetStd()).begin(),
-                                                               it2_end = list2.GetStd().end();
-                      it1 != it1_end && it2 != it2_end;
-                      ++it1, ++it2) {
-                    // each name must match, in order
-                    string name1, name2;
-                    name1.erase();
-                    name2.erase();
-                    (*it1)->GetName().GetLabel(&name1, CPerson_id::eGenbank);
-                    (*it2)->GetName().GetLabel(&name2, CPerson_id::eGenbank);
-                    if (!NStr::Equal(name1, name2)) {
-                        return false;
-                    }
-                }
-                return true;
-            }
-        } else {
-            // first name must match
-            string name1, name2;
-            name1.erase();
-            name2.erase();
-            list1.GetStd().front()->GetName().GetLabel(&name1, CPerson_id::eGenbank);
-            list2.GetStd().front()->GetName().GetLabel(&name1, CPerson_id::eGenbank);
-            return NStr::Equal(name1, name2);
-        }
-    } else if (list1.Which() == CAuth_list::TNames::e_Ml) {
-        if (all_authors_match) {
-            if (list1.GetMl().size() != list2.GetMl().size()) {
-                return false;
-            } else {
-                for ( CAuth_list::TNames::TMl::const_iterator it1 = (list1.GetMl()).begin(), 
-                                                               it1_end = (list1.GetMl()).end(),
-                                                               it2 = (list2.GetMl()).begin(),
-                                                               it2_end = list2.GetMl().end();
-                      it1 != it1_end && it2 != it2_end;
-                      ++it1, ++it2) {
-                    // each name must match, in order
-                    if (!NStr::Equal((*it1), (*it2))) {
-                        return false;
-                    }
-                }
-                return true;
-            }
-        } else {
-            // first name must match
-            return NStr::Equal(list1.GetMl().front(), list2.GetMl().front());
-        }
-        
-    } else if (list1.Which() == CAuth_list::TNames::e_Str) {
-        if (all_authors_match) {
-            if (list1.GetStr().size() != list2.GetStr().size()) {
-                return false;
-            } else {
-                for ( CAuth_list::TNames::TStr::const_iterator it1 = (list1.GetStr()).begin(), 
-                                                               it1_end = (list1.GetStr()).end(),
-                                                               it2 = (list2.GetStr()).begin(),
-                                                               it2_end = list2.GetStr().end();
-                      it1 != it1_end && it2 != it2_end;
-                      ++it1, ++it2) {
-                    // each name must match, in order
-                    if (!NStr::Equal((*it1), (*it2))) {
-                        return false;
-                    }
-                }
-                return true;
-            }
-        } else {
-            // first name must match
-            return NStr::Equal(list1.GetStr().front(), list2.GetStr().front());
-        }
-    } else {
-        return false;
-    } 
-}
-
-
-bool CitSubsMatch(const CCit_sub& sub1, const CCit_sub& sub2)
-{
-    // dates must match
-    if (sub1.CanGetDate()) {
-        if (sub2.CanGetDate()) {
-            if (sub1.GetDate().Compare(sub2.GetDate()) != CDate::eCompare_same) {
-                return false;
-            }
-        } else {
-            return false;
-        }
-    } else if (sub2.CanGetDate()) {
-        return false;
-    }
-    
-    // descriptions must match
-    if (sub1.CanGetDescr() && sub2.CanGetDescr()) {
-        if (!NStr::Equal(sub1.GetDescr(), sub2.GetDescr())) {
-            return false;
-        }
-    } else if (sub1.CanGetDescr() || sub2.CanGetDescr()) {
-        // one has a description, the other does not
-        return false;
-    }
-    
-    // author lists must be set and must match
-    // if both affiliations set, must match
-    if (! sub1.IsSetAuthors() 
-        || ! sub2.IsSetAuthors()
-        || ! sub1.GetAuthors().IsSetNames()
-        || ! sub2.GetAuthors().IsSetNames()
-        || !AuthListsMatch (sub1.GetAuthors().GetNames(), sub2.GetAuthors().GetNames(), true)) {
-        return false;
-    } else if (sub1.GetAuthors().IsSetAffil() && sub2.GetAuthors().IsSetAffil()
-               && !sub1.GetAuthors().GetAffil().Equals(sub2.GetAuthors().GetAffil())) {
-        return false;
-    } else {
-        return true;
-    }
-    
-}
-
-
 bool s_DbtagCompare (const CRef<CDbtag>& dbt1, const CRef<CDbtag>& dbt2)
 {
     // is dbt1 < dbt2
@@ -979,211 +793,6 @@ bool s_OrgrefSynEqual( const string & syn1, const string & syn2 )
     return NStr::EqualNocase(syn1, syn2);
 }
 
-CRef<CSeq_loc> MakeFullLengthLocation(CBioseq_Handle bh, CScope* scope, CRef<CSeq_loc> new_loc, bool &first)
-{
-    bool is_master_seq = false;
-
-    // if this is the master sequence, add whole locations for each of the parts
-    CSeq_entry_Handle seh = bh.GetParentEntry();
-    if (seh) {
-        seh = seh.GetParentEntry();
-    }
-    if (seh && seh.IsSet()) {
-        CBioseq_set_Handle bsh = seh.GetSet();
-        if (bsh.CanGetClass() && bsh.GetClass() == CBioseq_set::eClass_segset) {
-            // this is the master sequence
-            is_master_seq = true;
-            // add whole loc for each part
-            FOR_EACH_SEQENTRY_ON_SEQSET (it, *(bsh.GetCompleteBioseq_set())) {
-                if ((*it)->IsSet()) {
-                    const CBioseq_set& parts_set = (*it)->GetSet();
-                    if (parts_set.CanGetClass() && parts_set.GetClass() == CBioseq_set::eClass_parts) {
-                        FOR_EACH_SEQENTRY_ON_SEQSET (it2, parts_set) {
-                            if ((*it2)->IsSeq()) {
-                                new_loc = MakeFullLengthLocation (scope->GetBioseqHandle((*it2)->GetSeq()), scope, new_loc, first);
-                            }
-                        }
-                    }
-                }
-            }
-        }
-    }
-    if (!is_master_seq) {
-        CRef <CSeq_loc> loc_part(new CSeq_loc);
-
-        // Get best ID for location
-        const CSeq_id* id = FindBestChoice(bh.GetBioseqCore()->GetId(), CSeq_id::BestRank);
-        CRef <CSeq_id> new_id(new CSeq_id);
-        new_id->Assign(*id);
-        loc_part->SetInt().SetId(*new_id);
-        loc_part->SetInt().SetFrom(0);
-        loc_part->SetInt().SetTo(bh.GetInst_Length() - 1);
-        if (first) {
-            new_loc = loc_part;
-            first = false;
-        } else {
-            CRef<CSeq_loc> tmp_loc;
-            tmp_loc = sequence::Seq_loc_Add(*new_loc, *loc_part, 
-                                            CSeq_loc::fMerge_Abutting, scope);
-            new_loc = tmp_loc;
-        }
-    }
-    return new_loc;
-}
-
-CRef<CSeq_loc> MakeFullLengthLocation(const CSeq_loc& loc, CScope* scope)
-{
-    // Create a location that covers the entire sequence.
-
-    // if on segmented set, create location for each segment
-    CRef<CSeq_loc> new_loc(new CSeq_loc);
-    CSeq_loc_CI loc_it (loc);
-    bool first = true;
-    CBioseq_Handle last_bh;
-    while (loc_it) {
-        CBioseq_Handle bh = scope->GetBioseqHandle(loc_it.GetSeq_id());
-        if (!first && bh == last_bh) {
-            // skip - only one location per sequence 
-        } else {
-            new_loc = MakeFullLengthLocation (bh, scope, new_loc, first);
-        }
-        last_bh = bh;
-        ++loc_it;
-    }
-    return new_loc;   
-}
-
-
-bool IsFeatureFullLength(const CSeq_feat& cf, CScope* scope)
-{
-    // Create a location that covers the entire sequence and do
-    // a comparison.  Can't just check for the location type 
-    // of the feature to be "whole" because an interval could
-    // start at 0 and end at the end of the Bioseq.
-    CRef<CSeq_loc> whole_loc = MakeFullLengthLocation (cf.GetLocation(), scope);
-
-    if (sequence::Compare(*whole_loc, cf.GetLocation(), scope, sequence::fCompareOverlapping) == sequence::eSame) {
-        return true;
-    } else {
-        return false;
-    }
-}
-
-
-CBioSource::EGenome GenomeByOrganelle(string& organelle, bool strip, NStr::ECase use_case)
-{
-    string match = "";
-    
-    CBioSource::EGenome genome = CBioSource::GetGenomeByOrganelle (organelle, use_case, true);
-    if (genome != CBioSource::eGenome_unknown) {
-        match = CBioSource::GetOrganelleByGenome (genome);
-        if (strip && !NStr::IsBlank(match)) {
-            organelle = organelle.substr(match.length());
-            NStr::TruncateSpacesInPlace(organelle);
-        }
-    }
-        
-    return genome;
-}
-
-
-bool IsmRNA(CBioseq_Handle bsh)
-{
-    bool is_mRNA = false;
-    for (CSeqdesc_CI desc(bsh, CSeqdesc::e_Molinfo); desc && !is_mRNA; ++desc) {
-        if (desc->GetMolinfo().CanGetBiomol()
-            && desc->GetMolinfo().GetBiomol() == CMolInfo::eBiomol_mRNA) {
-            is_mRNA = true;
-        }
-    }
-    return is_mRNA;
-}
-
-
-bool IsmRNA(CBioseq_set_Handle bsh)
-{
-    bool is_mRNA = false;
-    if (bsh.CanGetClass() && bsh.GetClass() == CBioseq_set::eClass_segset) {
-        CSeq_entry_Handle seh = bsh.GetParentEntry();
-
-        for (CSeqdesc_CI desc(seh, CSeqdesc::e_Molinfo, 1); desc && !is_mRNA; ++desc) {
-            if (desc->GetMolinfo().CanGetBiomol()
-                && desc->GetMolinfo().GetBiomol() == CMolInfo::eBiomol_mRNA) {
-                is_mRNA = true;
-            }
-        }
-    }
-    return is_mRNA;
-}
-
-
-const CBioSource* GetAssociatedBioSource(CBioseq_set_Handle bh)
-{
-    CSeq_entry_Handle seh = bh.GetParentEntry();
-    CSeqdesc_CI desc_ci (seh, CSeqdesc::e_Source, 1);
-
-    if (desc_ci) {
-        return &(desc_ci->GetSource());
-    } else {
-        seh = seh.GetParentEntry();
-    
-        if (seh && seh.IsSet()) {
-            return GetAssociatedBioSource(seh.GetSet());
-        }
-    }
-    return NULL;        
-}
-
-
-const CBioSource* GetAssociatedBioSource(CBioseq_Handle bh)
-{
-    CSeqdesc_CI desc_ci (bh, CSeqdesc::e_Source, 1);
-
-    if (desc_ci) {
-        return &(desc_ci->GetSource());
-    } else {
-        CSeq_entry_Handle seh = bh.GetParentEntry();
-        seh = seh.GetParentEntry();
-    
-        if (seh && seh.IsSet()) {
-            return GetAssociatedBioSource(seh.GetSet());
-        }
-    }
-    return NULL;
-}
-
-
-bool IsArtificialSyntheticConstruct (const CBioSource *bsrc)
-{
-    if (bsrc 
-        && bsrc->CanGetOrigin() && bsrc->GetOrigin() == CBioSource::eOrigin_artificial
-        && bsrc->CanGetOrg() && bsrc->GetOrg().CanGetTaxname() && NStr::EqualNocase (bsrc->GetOrg().GetTaxname(), "synthetic construct")) {
-        return true;
-    } else {
-        return false;
-    }
-}
-
-
-bool IsArtificialSyntheticConstruct (CBioseq_Handle bsh)
-{
-    return IsArtificialSyntheticConstruct (GetAssociatedBioSource(bsh));
-}
-
-
-bool IsArtificialSyntheticConstruct (CBioseq_set_Handle bsh)
-{
-    CSeq_entry_Handle seh = bsh.GetParentEntry();
-    CSeqdesc_CI desc_ci (seh, CSeqdesc::e_Source);
-    while (desc_ci) {
-        if (IsArtificialSyntheticConstruct (&(desc_ci->GetSource()))) {
-            return true;
-        }
-        ++desc_ci;
-    }
-    
-    return IsArtificialSyntheticConstruct (GetAssociatedBioSource(bsh));
-}
 
 CRef<CAuthor> ConvertMltoSTD( const string& token)
 {
@@ -1209,8 +818,8 @@ bool ConvertAuthorContainerMlToStd( CAuth_list& authors )
 {
     CAuth_list::C_Names* names = new CAuth_list::C_Names;
     CAuth_list::C_Names::TStd& names_std = names->SetStd();
-    NON_CONST_ITERATE( CAuth_list::C_Names::TMl, author, authors.SetNames().SetMl() ) {
-        names_std.push_back( ConvertMltoSTD( *author ) );
+    for (auto& author : authors.GetNames().GetMl()) {
+        names_std.push_back( ConvertMltoSTD( author ) );
     }
     authors.SetNames( *names );
     return true;
