@@ -1,7 +1,7 @@
 #ifndef OBJECTS_GENERAL___CLEANUP_UTILS__HPP
 #define OBJECTS_GENERAL___CLEANUP_UTILS__HPP
 
-/* $Id: cleanup_utils.hpp 479180 2015-09-17 12:07:11Z bollin $
+/* $Id: cleanup_utils.hpp 574795 2018-11-19 16:47:08Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -75,12 +75,6 @@ bool CleanVisString( string &str );
 
 bool CleanVisStringJunk( string &str, bool allow_ellipses = false );
 
-/// remove a trailing period, 
-bool RemoveTrailingPeriod(string& str);
-
-/// remove trailing white space, commas, tildes and semicolons
-bool RemoveTrailingJunk(string& str);
-
 /// remove white space between pairs of tildes.
 /// "~ ~  ~ a~" -> "~~~ a~"
 bool  RemoveSpacesBetweenTildes(string& str);
@@ -98,8 +92,6 @@ bool TrimSpacesSemicolonsAndCommas (string& val);
 /// Change double to single quotes
 bool CleanDoubleQuote(string& str);
 
-bool OnlyPunctuation (string str);
-
 /// remove all spaces from a string
 bool RemoveSpaces(string& str);
 
@@ -390,13 +382,10 @@ if ((o).IsSet##x()) { \
     }
 
 // for converting strings to locations
-CRef<CSeq_loc> ReadLocFromText(string text, const CSeq_id *id, CScope *scope);
+CRef<CSeq_loc> ReadLocFromText(const string& text, const CSeq_id *id, CScope *scope);
 
 // for finding the correct amino acid letter given an abbreviation
-char ValidAminoAcid (string abbrev);
-
-// for matching equivalent cit-sub publications
-bool CitSubsMatch(const CCit_sub& sub1, const CCit_sub& sub2);
+char ValidAminoAcid (const string& abbrev);
 
 // for sorting and uniquing dbtags
 bool s_DbtagCompare (const CRef<CDbtag>& dbt1, const CRef<CDbtag>& dbt2);
@@ -442,23 +431,6 @@ bool is_sorted(Iter first, Iter last)
     return true;
 }
 
-CRef<CSeq_loc> MakeFullLengthLocation(const CSeq_loc& loc, CScope* scope);
-
-bool IsFeatureFullLength(const CSeq_feat& cf, CScope* scope);
-
-CBioSource::EGenome GenomeByOrganelle(string& organelle, bool strip, NStr::ECase use_case = NStr::eNocase);
-
-bool IsmRNA(CBioseq_Handle bsh);
-bool IsmRNA(CBioseq_set_Handle bsh);
-const CBioSource* GetAssociatedBioSource(CBioseq_set_Handle bh);
-const CBioSource* GetAssociatedBioSource(CBioseq_Handle bh);
-bool IsArtificialSyntheticConstruct (const CBioSource *bsrc);
-bool IsArtificialSyntheticConstruct (CBioseq_Handle bsh);
-bool IsArtificialSyntheticConstruct (CBioseq_set_Handle bsh);
-
-
-
-
 
 END_SCOPE(objects)
 END_NCBI_SCOPE
