@@ -151,6 +151,8 @@ public:
         CSeq_submit& ss
     );
 
+	void BasicCleanupSubmitblock(CSubmit_block& sb);
+
     void BasicCleanupSeqAnnot (
         CSeq_annot& sa
     );
@@ -302,6 +304,8 @@ private:
     void RnaFeatBC (CRNA_ref& rr, CSeq_feat& sf);
     void CdregionFeatBC (CCdregion& cds, CSeq_feat& seqfeat);
 
+    static bool x_IsCommentRedundantWithEC(const CSeq_feat& seqfeat, CScope& scope);
+
     void DeltaExtBC( CDelta_ext & delta_ext, CSeq_inst &seq_inst );
 
     void UserObjectBC( CUser_object &user_object );
@@ -318,10 +322,10 @@ private:
 
     // Extended Cleanup functions
     void BioSourceEC ( CBioSource& biosrc );
-    void x_RemoveUnnecessaryGeneXrefs ( CSeq_feat& feat );
     void AddProteinTitles (CBioseq& seq);
     void ProtRefEC( CProt_ref& pr);
     void CdRegionEC( CSeq_feat& sf);
+    bool x_FixParentPartials(const CSeq_feat& sf, CSeq_feat& parent);
     void x_ExtendFeatureToCoverSequence(CSeq_feat_Handle fh, const CBioseq& seq);
     void x_ExtendProteinFeatureOnProteinSeq(CBioseq& seq);
     void x_ExtendSingleGeneOnMrna(CBioseq& seq);
@@ -338,10 +342,8 @@ private:
     void x_RemoveUnseenTitles(CBioseq& seq);
     void x_RemoveUnseenTitles(CBioseq_set& set);
     void KeepLatestDateDesc(CSeq_descr & seq_descr);
-    void x_RemoveOrphanedProteins(CBioseq_set& set);
     void x_SingleSeqSetToSeq(CBioseq_set& set);
     void x_MergeDupBioSources(CSeq_descr & seq_descr);
-    void x_CleanupSubTypeEC(CBioSource::TSubtype& subtypes);
 
     // void XxxxxxBC (Cxxxxx& xxx);
 
@@ -387,7 +389,6 @@ private:
 
     void x_AddNcbiCleanupObject( CSeq_entry &seq_entry );
 
-    void x_CleanupConsSplice(CGb_qual& gbq);
 
     // for rpt_unit and replace GenBank qualifiers 
     bool x_CleanupRptUnit(CGb_qual& gbq);
@@ -417,15 +418,6 @@ private:
     // Date-related
     void x_DateStdBC( CDate_std& date );
 
-    // author-related
-    void x_AuthorBC  ( CAuthor& au, bool fix_initials );
-    void x_PersonIdBC( CPerson_id& pid, bool fix_initials );
-    void x_NameStdBC ( CName_std& name, bool fix_initials );
-    void x_ExtractSuffixFromInitials(CName_std& name);
-    void x_FixEtAl(CName_std& name);
-    void x_FixSuffix(CName_std& name);
-    void x_FixInitials(CName_std& name);
-
     void x_AddReplaceQual(CSeq_feat& feat, const string& str);
 
     void x_SeqIntervalBC( CSeq_interval & seq_interval );
@@ -460,9 +452,6 @@ private:
 
     void x_CleanupAndRepairInference( string &inference );
 
-    void x_CleanStructuredComment( CUser_object &user_object );
-    void x_RemoveEmptyFields(CUser_object& user_object);
-
     void x_MendSatelliteQualifier( string &val );
 
     // e.g. if ends with ",..", turn into "..."
@@ -500,8 +489,6 @@ private:
     void x_PostSeqFeat( CSeq_feat& seq_feat );
     void x_PostOrgRef( COrg_ref& org );
     void x_PostBiosource( CBioSource& biosrc );
-    void x_ConvertOrgref_modToSubSource( CBioSource& biosrc );
-    void x_ConvertOrgref_modToOrgMod(COrg_ref& org);
 
     void x_TranslateITSNameAndFlag( string &in_out_name ) ;
 
@@ -511,10 +498,6 @@ private:
 
     void x_AuthListBCWithFixInitials( CAuth_list& al );
 
-    void x_AddNumToUserField( CUser_field &field );
-
-    void x_GeneOntologyTermsBC( vector< CRef< CUser_field > > &go_terms );
-
     // After we've traversed the hierarchy of objects, there may be some
     // processing that can only be done after the traversal is complete.
     // This function does that processing.
@@ -601,6 +584,7 @@ private:
     void x_BioseqSetNucProtEC( CBioseq_set & bioseq_set );
     void x_BioseqSetGenBankEC(CBioseq_set & bioseq_set);
     void x_RemoveNestedGenBankSet(CBioseq_set & bioseq_set);
+    void x_RemoveNestedNucProtSet(CBioseq_set & bioseq_set);
     void x_MoveNpDBlinks(CBioseq_set& bioseq_set);
     void x_MoveNpSrc(CBioseq_set& bioseq_set);
     void x_MoveNpSrc(CRef<CSeqdesc>& srcdesc, CSeq_descr& descr);
