@@ -1,4 +1,4 @@
-/*  $Id: gbloader.cpp 534193 2017-04-25 14:00:21Z vasilche $
+/*  $Id: gbloader.cpp 572532 2018-10-16 12:00:58Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -82,17 +82,17 @@ BEGIN_SCOPE(objects)
 //
 
 
-NCBI_PARAM_DEF_EX(string, GENBANK, LOADER_METHOD, kEmptyStr,
+NCBI_PARAM_DEF_EX(string, GENBANK, LOADER_METHOD, "",
                   eParam_NoThread, GENBANK_LOADER_METHOD);
 typedef NCBI_PARAM_TYPE(GENBANK, LOADER_METHOD) TGenbankLoaderMethod;
 
 NCBI_PARAM_DECL(string, GENBANK, READER_NAME);
-NCBI_PARAM_DEF_EX(string, GENBANK, READER_NAME, kEmptyStr,
+NCBI_PARAM_DEF_EX(string, GENBANK, READER_NAME, "",
                   eParam_NoThread, GENBANK_READER_NAME);
 typedef NCBI_PARAM_TYPE(GENBANK, READER_NAME) TGenbankReaderName;
 
 NCBI_PARAM_DECL(string, GENBANK, WRITER_NAME);
-NCBI_PARAM_DEF_EX(string, GENBANK, WRITER_NAME, kEmptyStr,
+NCBI_PARAM_DEF_EX(string, GENBANK, WRITER_NAME, "",
                   eParam_NoThread, GENBANK_WRITER_NAME);
 typedef NCBI_PARAM_TYPE(GENBANK, WRITER_NAME) TGenbankWriterName;
 
@@ -278,14 +278,26 @@ string CGBDataLoader::GetLoaderNameFromArgs(const string& /*reader_name*/)
 }
 
 
+CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
+    CObjectManager& om,
+    EIncludeHUP     include_hup,
+    CObjectManager::EIsDefault is_default,
+    CObjectManager::TPriority  priority)
+{
+    return RegisterInObjectManager(om, include_hup, NcbiEmptyString, is_default,
+                                   priority);
+}
+
+
 CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
     CObjectManager& om,
     EIncludeHUP     /*include_hup*/,
+    const string& web_cookie,
     CObjectManager::EIsDefault is_default,
     CObjectManager::TPriority  priority)
 {
     CGBLoaderParams params("PUBSEQOS2:PUBSEQOS");
-    params.SetHUPIncluded();
+    params.SetHUPIncluded(true, web_cookie);
     TGBMaker maker(params);
     CDataLoader::RegisterInObjectManager(om, maker, is_default, priority);
     return maker.GetRegisterInfo();
@@ -297,16 +309,27 @@ string CGBDataLoader::GetLoaderNameFromArgs(EIncludeHUP /*include_hup*/)
     return GBLOADER_HUP_NAME;
 }
 
+CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
+    CObjectManager& om,
+    const string&   reader_name,
+    EIncludeHUP     include_hup,
+    CObjectManager::EIsDefault is_default,
+    CObjectManager::TPriority  priority)
+{
+    return RegisterInObjectManager(om, reader_name, include_hup, NcbiEmptyString,
+                                   is_default, priority);
+}
 
 CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
     CObjectManager& om,
     const string&   reader_name,
     EIncludeHUP     /*include_hup*/,
+    const string& web_cookie,
     CObjectManager::EIsDefault is_default,
     CObjectManager::TPriority  priority)
 {
     CGBLoaderParams params(reader_name);
-    params.SetHUPIncluded();
+    params.SetHUPIncluded(true, web_cookie);
     TGBMaker maker(params);
     CDataLoader::RegisterInObjectManager(om, maker, is_default, priority);
     return maker.GetRegisterInfo();
@@ -353,7 +376,18 @@ CGBDataLoader::TRegisterLoaderInfo CGBDataLoader::RegisterInObjectManager(
 
 string CGBDataLoader::GetLoaderNameFromArgs(const CGBLoaderParams& params)
 {
-    return params.HasHUPIncluded()? GBLOADER_HUP_NAME: GBLOADER_NAME;
+    if ( !params.GetLoaderName().empty() ) {
+        return params.GetLoaderName();
+    }
+    if (params.HasHUPIncluded()) {
+        const string& web_cookie = params.GetWebCookie();
+        if (web_cookie.empty())
+            return GBLOADER_HUP_NAME;
+        else
+            return GBLOADER_HUP_NAME + string("-") + web_cookie;
+    } else {
+        return GBLOADER_NAME;
+    }
 }
 
 
@@ -644,6 +678,9 @@ void CGBDataLoader::x_CreateDriver(const CGBLoaderParams& params)
     if ( !params.GetReaderName().empty() ) {
         string reader_name = params.GetReaderName();
         NStr::ToLower(reader_name);
+        if ( NStr::StartsWith(reader_name, "pubseqos") )
+            m_WebCookie = params.GetWebCookie();
+    
         if ( x_CreateReaders(reader_name, gb_params, preopen) ) {
             if ( reader_name == "cache" ||
                  NStr::StartsWith(reader_name, "cache;") ) {
@@ -707,7 +744,7 @@ bool CGBDataLoader::x_CreateReaders(const string& str,
         CRef<CReader> reader(x_CreateReader(str_list[i], params));
         if( reader ) {
             if ( HasHUPIncluded() ) {
-                reader->SetIncludeHUP();
+                reader->SetIncludeHUP(true, m_WebCookie);
             }
             if ( preopen != CGBLoaderParams::ePreopenNever ) {
                 reader->OpenInitialConnection(preopen == CGBLoaderParams::ePreopenAlways);
@@ -1518,16 +1555,18 @@ CGBDataLoader::GetExternalRecords(const CBioseq_Info& bioseq)
 
 
 CDataLoader::TTSE_LockSet
-CGBDataLoader::GetExternalAnnotRecords(const CSeq_id_Handle& idh,
-                                       const SAnnotSelector* sel)
+CGBDataLoader::GetExternalAnnotRecordsNA(const CSeq_id_Handle& idh,
+                                         const SAnnotSelector* sel,
+                                         TProcessedNAs* processed_nas)
 {
-    return x_GetRecords(idh, fBlobHasExtAnnot|fBlobHasNamedAnnot, sel);
+    return x_GetRecords(idh, fBlobHasExtAnnot|fBlobHasNamedAnnot, sel, processed_nas);
 }
 
 
 CDataLoader::TTSE_LockSet
-CGBDataLoader::GetExternalAnnotRecords(const CBioseq_Info& bioseq,
-                                       const SAnnotSelector* sel)
+CGBDataLoader::GetExternalAnnotRecordsNA(const CBioseq_Info& bioseq,
+                                         const SAnnotSelector* sel,
+                                         TProcessedNAs* processed_nas)
 {
     TTSE_LockSet ret;
     TIds ids = bioseq.GetId();
@@ -1535,7 +1574,7 @@ CGBDataLoader::GetExternalAnnotRecords(const CBioseq_Info& bioseq,
     ITERATE ( TIds, it, ids ) {
         if ( GetBlobId(*it) ) {
             // correct id is found
-            TTSE_LockSet ret2 = GetExternalAnnotRecords(*it, sel);
+            TTSE_LockSet ret2 = GetExternalAnnotRecordsNA(*it, sel, processed_nas);
             ret.swap(ret2);
             break;
         }
@@ -1549,12 +1588,13 @@ CGBDataLoader::GetExternalAnnotRecords(const CBioseq_Info& bioseq,
 
 
 CDataLoader::TTSE_LockSet
-CGBDataLoader::GetOrphanAnnotRecords(const CSeq_id_Handle& idh,
-                                     const SAnnotSelector* sel)
+CGBDataLoader::GetOrphanAnnotRecordsNA(const CSeq_id_Handle& idh,
+                                       const SAnnotSelector* sel,
+                                       TProcessedNAs* processed_nas)
 {
     bool load_external = m_AlwaysLoadExternal;
-    bool load_namedacc = m_AlwaysLoadNamedAcc &&
-        sel && sel->IsIncludedAnyNamedAnnotAccession();
+    bool load_namedacc =
+        m_AlwaysLoadNamedAcc && IsRequestedAnyNA(sel);
     if ( load_external || load_namedacc ) {
         TBlobContentsMask mask = 0;
         if ( load_external ) {
@@ -1563,10 +1603,10 @@ CGBDataLoader::GetOrphanAnnotRecords(const CSeq_id_Handle& idh,
         if ( load_namedacc ) {
             mask |= fBlobHasNamedAnnot;
         }
-        return x_GetRecords(idh, mask, sel);
+        return x_GetRecords(idh, mask, sel, processed_nas);
     }
     else {
-        return CDataLoader::GetOrphanAnnotRecords(idh, sel);
+        return CDataLoader::GetOrphanAnnotRecordsNA(idh, sel, processed_nas);
     }
 }
 
@@ -1574,7 +1614,8 @@ CGBDataLoader::GetOrphanAnnotRecords(const CSeq_id_Handle& idh,
 CDataLoader::TTSE_LockSet
 CGBDataLoader::x_GetRecords(const CSeq_id_Handle& sih,
                             TBlobContentsMask mask,
-                            const SAnnotSelector* sel)
+                            const SAnnotSelector* sel,
+                            TProcessedNAs* processed_nas)
 {
     TTSE_LockSet locks;
 
