@@ -1,4 +1,4 @@
-/*  $Id: processors.cpp 522272 2016-12-16 17:07:55Z vasilche $
+/*  $Id: processors.cpp 572633 2018-10-17 16:54:14Z ivanov $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -44,6 +44,7 @@
 
 #include <objmgr/impl/split_parser.hpp>
 #include <objmgr/impl/tse_split_info.hpp>
+#include <objmgr/impl/bioseq_set_info.hpp>
 #include <objmgr/annot_selector.hpp>
 
 #include <objects/id1/id1__.hpp>
@@ -909,8 +910,13 @@ void CProcessor::AddWGSMaster(CLoadLockSetter& blob)
         if ( CSeq_id_Handle id = s_GetWGSMasterSeq_id(*it) ) {
             int mask = kGoodDescrMask;
             if ( kAddMasterDescrToTSE ) {
-                // exclude existing descr types
-                mask &= ~lock->x_GetBaseInfo().x_GetExistingDescrMask();
+                // exclude existing descr types except User
+                mask &= ~lock->x_GetBaseInfo().x_GetExistingDescrMask() | (1<<CSeqdesc::e_User);
+                if ( lock->IsSet() ) {
+                    if ( auto first_entry = lock->GetSet().GetFirstEntry() ) {
+                        mask &= ~first_entry->x_GetBaseInfo().x_GetExistingDescrMask() | (1<<CSeqdesc::e_User);
+                    }
+                }
             }
             CRef<CTSE_Chunk_Info> chunk(new CWGSMasterChunkInfo(id, mask));
             lock->GetSplitInfo().AddChunk(*chunk);
@@ -2557,7 +2563,7 @@ void CProcessor_ExtAnnot::Process(CReaderRequestResult& result,
 
     CWriter* writer = GetWriter(result);
     if ( writer ) {
-        m_Dispatcher->LoadBlobVersion(result, blob_id);
+        //m_Dispatcher->LoadBlobVersion(result, blob_id);
         CRef<CWriter::CBlobStream> stream =
             (OpenStream(writer, result, blob_id, chunk_id, this));
         if ( stream ) {
@@ -2611,12 +2617,14 @@ void CProcessor_AnnotInfo::LoadBlob(CReaderRequestResult& result,
     CRef<CTSE_Chunk_Info> chunk(new CTSE_Chunk_Info(kDelayedMain_ChunkId));
     const CBlob_Annot_Info::TAnnotInfo& annot_infos =
         info.GetAnnotInfo()->GetAnnotInfo();
+    set<string> names;
     ITERATE ( CBlob_Annot_Info::TAnnotInfo, it, annot_infos ) {
         const CID2S_Seq_annot_Info& annot_info = **it;
         // create special external annotations blob
         CAnnotName name(annot_info.GetName());
         if ( name.IsNamed() && !ExtractZoomLevel(name.GetName(), 0, 0) ) {
-            setter.GetTSE_LoadLock()->SetName(name);
+            //setter.GetTSE_LoadLock()->SetName(name);
+            names.insert(name.GetName());
         }
 
         vector<SAnnotTypeSelector> types;
@@ -2655,6 +2663,9 @@ void CProcessor_AnnotInfo::LoadBlob(CReaderRequestResult& result,
             chunk->x_AddAnnotType(name, *it, loc);
         }
     }
+    if ( names.size() == 1 ) {
+        setter.GetTSE_LoadLock()->SetName(*names.begin());
+    }
     setter.GetSplitInfo().AddChunk(*chunk);
     _ASSERT(setter.GetTSE_LoadLock()->x_NeedsDelayedMainChunk());
 
