@@ -1,4 +1,4 @@
-/*  $Id: reader.cpp 534193 2017-04-25 14:00:21Z vasilche $
+/*  $Id: reader.cpp 568378 2018-08-03 14:54:41Z dondosha $
  * ===========================================================================
  *                            PUBLIC DOMAIN NOTICE
  *               National Center for Biotechnology Information
@@ -1305,7 +1305,7 @@ void CReader::ResetCache(void)
 }
 
 
-void CReader::SetIncludeHUP(bool include_hup)
+void CReader::SetIncludeHUP(bool include_hup, const string& web_cookie)
 {
     NCBI_THROW(CObjMgrException, eRegisterError,
                "HUP is supported only by pubseqos or pubseqos2 readers");
