@@ -1,4 +1,4 @@
-/*  $Id: cds_fix.cpp 547098 2017-09-26 12:05:24Z ivanov $
+/*  $Id: cds_fix.cpp 574263 2018-11-08 17:55:12Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -45,12 +45,14 @@
 #include <objects/seqfeat/SeqFeatXref.hpp>
 #include <objects/seqfeat/seqfeat_macros.hpp>
 #include <objects/seqfeat/Genetic_code_table.hpp>
+#include <util/checksum.hpp>
 #include <objmgr/bioseq_handle.hpp>
 #include <objmgr/seqdesc_ci.hpp>
 #include <objmgr/seq_annot_ci.hpp>
 #include <util/sequtil/sequtil_convert.hpp>
 #include <objmgr/util/sequence.hpp>
 #include <objmgr/seq_vector.hpp>
+#include <objects/general/Dbtag.hpp>
 
 BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects)
@@ -234,19 +236,19 @@ TSeqPos ExtendLocationForTranslExcept(objects::CSeq_loc& loc, objects::CScope& s
             overhang->SetStrand(eNa_strand_minus);
         }
     } else {
-        len = bsh.GetBioseqLength() - stop;
+        len = bsh.GetBioseqLength() - stop - 1;
         if (len > 3) {
             len = 3;
         }
         if (len > 0) {
             overhang->SetInt().SetFrom(stop + 1);
-            overhang->SetInt().SetTo(stop + len);
+            overhang->SetInt().SetTo(bsh.GetBioseqLength() - 1);
         }
     }
     if (len > 0) {
         CSeqVector vec(*overhang, scope, CBioseq_Handle::eCoding_Iupac);
         string seq_string;
-        vec.GetSeqData(0, len - 1, seq_string);   
+        vec.GetSeqData(0, len, seq_string);   
         if (vec[0] == 'T') {
             except_len++;
             if (len > 1 && vec[1] == 'A') {
@@ -1216,6 +1218,179 @@ ApplyCDSFrame::ECdsFrame ApplyCDSFrame::s_GetFrameFromName(const string& name)
     return frame;
 }
 
+const unsigned int MAX_ID_LENGTH = 50;
+
+static inline string GetIdHash(const string &str)
+{
+    return NStr::NumericToString(NHash::CityHash64(str), 0, 16);
+}
+
+string GetIdHashOrValue(const string &base, int offset)
+{
+    string new_str = base;
+    if (offset > 0)
+        new_str += "_" + NStr::NumericToString(offset);
+    if (new_str.length() <= MAX_ID_LENGTH)
+        return new_str;
+    string new_hash = GetIdHash(base);
+    if (offset > 0)
+        new_hash += "_" + NStr::NumericToString(offset);
+    return new_hash;
+}
+
+CRef<objects::CSeq_id> GetNewLocalProtId(const string &id_base, CScope &scope, int &offset)
+{   
+    string id_base_hash = GetIdHash(id_base);
+    CRef<objects::CSeq_id> new_id(new objects::CSeq_id());
+    string new_str = id_base;
+    if (offset > 0)
+        new_str += "_" + NStr::NumericToString(offset);
+    new_id->SetLocal().SetStr(new_str);
+    CRef<objects::CSeq_id> new_id_hash(new objects::CSeq_id());
+    string new_hash = id_base_hash;
+    if (offset > 0)
+        new_hash += "_" + NStr::NumericToString(offset);
+    new_id_hash->SetLocal().SetStr(new_hash);
+    objects::CBioseq_Handle b_found = scope.GetBioseqHandle(*new_id);
+    objects::CBioseq_Handle b_found_hash = scope.GetBioseqHandle(*new_id_hash); // as we consider ID_NUM and HASH_NUM to be synonyms we need to check for the existence of both at the same time 
+    // to avoid a situation where ID_1 and HASH_1 are both created
+    while (b_found || b_found_hash) 
+    {
+        offset++;
+        new_id->SetLocal().SetStr(id_base + "_" + NStr::NumericToString(offset));
+        b_found = scope.GetBioseqHandle(*new_id);
+        new_id_hash->SetLocal().SetStr(id_base_hash + "_" + NStr::NumericToString(offset));
+        b_found_hash = scope.GetBioseqHandle(*new_id_hash);
+    }
+    if (new_id->GetLocal().GetStr().size() <= MAX_ID_LENGTH)
+        return new_id;
+    return new_id_hash;
+}
+
+static CRef<objects::CSeq_id> GetNewGeneralProtId(const string &id_base, const string &db, CScope &scope, int &offset)
+{    
+    string id_base_hash = GetIdHash(id_base);
+    CRef<objects::CSeq_id> new_id(new objects::CSeq_id());
+    new_id->SetGeneral().SetDb(db);
+    string new_str = id_base;
+    if (offset > 0)
+        new_str += "_" + NStr::NumericToString(offset);
+    new_id->SetGeneral().SetTag().SetStr(new_str);
+    CRef<objects::CSeq_id> new_id_hash(new objects::CSeq_id());
+    new_id_hash->SetGeneral().SetDb(db);
+    string new_hash = id_base_hash;
+    if (offset > 0)
+        new_hash += "_" + NStr::NumericToString(offset);
+    new_id_hash->SetGeneral().SetTag().SetStr(new_hash);
+    objects::CBioseq_Handle b_found = scope.GetBioseqHandle(*new_id);
+    objects::CBioseq_Handle b_found_hash = scope.GetBioseqHandle(*new_id_hash); // as we consider ID_NUM and HASH_NUM to be synonyms we need to check for the existence of both at the same time 
+    // to avoid a situation where ID_1 and HASH_1 are both created
+    while (b_found || b_found_hash) 
+    {
+        offset++;
+        new_id->SetGeneral().SetTag().SetStr(id_base + "_" + NStr::NumericToString(offset));
+        b_found = scope.GetBioseqHandle(*new_id);
+        new_id_hash->SetGeneral().SetTag().SetStr(id_base_hash + "_" + NStr::NumericToString(offset));
+        b_found_hash = scope.GetBioseqHandle(*new_id_hash);
+    }
+    if (new_id->GetGeneral().GetTag().GetStr().size() <= MAX_ID_LENGTH)
+        return  new_id;
+    return new_id_hash;
+}
+
+static CRef<objects::CSeq_id> GetGeneralOrLocal(objects::CSeq_id_Handle hid, CScope &scope, int &offset, bool fall_through)
+{
+    CRef<objects::CSeq_id> new_id;
+    if (hid.GetSeqId()->IsLocal())
+    {
+        string id_base;           
+        if (hid.GetSeqId()->GetLocal().IsId())
+        {
+            id_base =  NStr::NumericToString(hid.GetSeqId()->GetLocal().GetId());
+        }
+        else
+        {
+            id_base = hid.GetSeqId()->GetLocal().GetStr();
+        }
+        new_id = GetNewLocalProtId(id_base, scope, offset);
+    }
+    else if (hid.GetSeqId()->IsGeneral() && hid.GetSeqId()->GetGeneral().IsSetTag()
+             && (!hid.GetSeqId()->GetGeneral().IsSetDb() || hid.GetSeqId()->GetGeneral().GetDb() != "TMSMART"))
+    {
+        string id_base;
+        if (hid.GetSeqId()->GetGeneral().GetTag().IsId())
+        {
+            id_base =  NStr::NumericToString(hid.GetSeqId()->GetGeneral().GetTag().GetId());
+        }
+        else
+        {
+            id_base = hid.GetSeqId()->GetGeneral().GetTag().GetStr();
+        }
+        new_id = GetNewGeneralProtId(id_base, hid.GetSeqId()->GetGeneral().GetDb(), scope, offset);
+    }
+    else if (fall_through) // if we don't care for the incoming seq-id to be specifically local or general take any input and create a local seq-id output
+    {
+        string id_base;
+        hid.GetSeqId()->GetLabel(&id_base, objects::CSeq_id::eContent);
+        new_id = GetNewLocalProtId(id_base, scope, offset);
+    }
+    return new_id;
+}
+
+vector<CRef<objects::CSeq_id> > GetNewProtIdFromExistingProt(objects::CBioseq_Handle bsh, int &offset, string& id_label)
+{
+    vector<CRef<objects::CSeq_id> > ids;
+    for(auto it : bsh.GetId()) 
+    {
+        if (it.GetSeqIdOrNull())
+        {
+            objects::CSeq_id_Handle hid = it;
+            CRef<objects::CSeq_id> new_id = GetGeneralOrLocal(hid, bsh.GetScope(), offset, false);
+            if (new_id)
+                ids.push_back(new_id);
+        }
+    }
+
+    if (ids.empty() && !bsh.GetId().empty())
+    {
+        CRef<objects::CSeq_id> new_id = GetGeneralOrLocal(bsh.GetId().front(), bsh.GetScope(), offset, true);
+        ids.push_back(new_id);
+    }
+
+    if (ids.empty())
+        NCBI_THROW(CException, eUnknown, "Seq-id not found");
+    
+    ids.front()->GetLabel(&id_label, objects::CSeq_id::eBoth);
+    return ids;
+}
+
+CRef<objects::CSeq_id> GetNewProtId(objects::CBioseq_Handle bsh, int &offset, string& id_label, bool general_only)
+{
+    objects::CSeq_id_Handle hid = sequence::GetId(bsh, sequence::eGetId_Best);
+    objects::CSeq_id_Handle gen_id;
+
+    for (auto it : bsh.GetId()) 
+    {
+        if (it.GetSeqId()->IsGeneral() && it.GetSeqId()->GetGeneral().IsSetDb() &&
+            !it.GetSeqId()->GetGeneral().IsSkippable()) 
+        {
+            gen_id = it;
+        }        
+    }
+    if (gen_id || general_only) 
+    {
+        hid = gen_id;
+    } 
+
+      
+    if (!hid)
+        NCBI_THROW(CException, eUnknown, "Seq-id of the requested type not found");
+  
+    CRef<objects::CSeq_id> new_id = GetGeneralOrLocal(hid, bsh.GetScope(), offset, !general_only);   
+    new_id->GetLabel(&id_label, objects::CSeq_id::eBoth);
+    return new_id;
+}
+
 
 END_SCOPE(edit)
 END_SCOPE(objects)
