@@ -1,4 +1,4 @@
-/*  $Id: feattable_edit.cpp 546402 2017-09-18 15:03:27Z ivanov $
+/*  $Id: feattable_edit.cpp 574199 2018-11-07 18:41:51Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -47,6 +47,10 @@
 
 #include <objtools/edit/loc_edit.hpp>
 #include <objtools/edit/feattable_edit.hpp>
+#include <corelib/ncbi_message.hpp>
+#include <objtools/edit/edit_error.hpp>
+#include <objtools/logging/listener.hpp>
+#include <objtools/edit/cds_fix.hpp>
 
 #include <sstream>
 
@@ -106,7 +110,7 @@ CFeatTableEdit::CFeatTableEdit(
     const string& locusTagPrefix,
     unsigned int locusTagNumber,
     unsigned int startingFeatId,
-    ILineErrorListener* pMessageListener) :
+    IObjtoolsListener* pMessageListener) :
     //  -------------------------------------------------------------------------
     mAnnot(annot),
     mpMessageListener(pMessageListener),
@@ -123,10 +127,8 @@ CFeatTableEdit::CFeatTableEdit(
 
 
 //  -------------------------------------------------------------------------
-CFeatTableEdit::~CFeatTableEdit()
+CFeatTableEdit::~CFeatTableEdit() = default;
 //  -------------------------------------------------------------------------
-{
-};
 
 //  -------------------------------------------------------------------------
 void CFeatTableEdit::GenerateMissingGeneForCds()
@@ -327,6 +329,14 @@ void CFeatTableEdit::EliminateBadQualifiers()
                 if (qualKey == "protein_id"  ||  qualKey == "transcript_id") {
                     continue;
                 }
+                if (qualKey == "orig_protein_id"  ||  qualKey == "orig_transcript_id") {
+                    continue;
+                }
+            }
+            if (subtype != CSeqFeatData::eSubtype_gene  &&  qualKey == "gene") {
+                //rw-570: remove gene qualifiers fro non-gene features before cleanup sees them
+                badQuals.push_back(qualKey);
+                continue;
             }
             CSeqFeatData::EQualifier qualType = CSeqFeatData::GetQualifierType(qualKey);
             if (CSeqFeatData::IsLegalQualifier(subtype, qualType)) {
@@ -342,8 +352,294 @@ void CFeatTableEdit::EliminateBadQualifiers()
 }
 
 
-//  ----------------------------------------------------------------------------
+
+// ---------------------------------------------------------------------------
 void CFeatTableEdit::GenerateProteinAndTranscriptIds()
+// ---------------------------------------------------------------------------
+{
+    mProcessedMrnas.clear();
+
+    {{
+        SAnnotSelector sel;
+        sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_cdregion);
+        for (CFeat_CI it(mHandle, sel); it; ++it) {
+            CMappedFeat cds = *it;
+            xAddTranscriptAndProteinIdsToCdsAndParentMrna(cds);
+        }
+     }}
+
+    {{
+        SAnnotSelector sel;
+        sel.IncludeFeatSubtype(CSeqFeatData::eSubtype_mRNA);
+        for (CFeat_CI it(mHandle, sel); it; ++it) {
+            CMappedFeat mrna = *it;
+            xAddTranscriptAndProteinIdsToUnmatchedMrna(mrna);
+        }
+    }}
+
+}
+
+
+// ---------------------------------------------------------------------------
+static bool s_ShouldConvertToGeneral(const string& id) 
+// ---------------------------------------------------------------------------
+{
+    return (!NStr::StartsWith(id, "gb|") &&
+            !NStr::StartsWith(id, "gnl|") &&
+            !NStr::StartsWith(id, "cds.gnl|") && 
+            !NStr::StartsWith(id, "mrna.gnl|"));
+}
+
+
+// ---------------------------------------------------------------------------
+static string s_GetTranscriptIdFromMrna(const CMappedFeat& mrna) 
+// ---------------------------------------------------------------------------
+{
+    string transcript_id = mrna.GetNamedQual("transcript_id");
+    if (NStr::IsBlank(transcript_id)) {
+        return mrna.GetNamedQual("ID");
+    }
+
+    return transcript_id;
+}
+
+
+// ---------------------------------------------------------------------------
+void CFeatTableEdit::xAddTranscriptAndProteinIdsToCdsAndParentMrna(CMappedFeat& cds)
+// ---------------------------------------------------------------------------
+{
+    CMappedFeat mrna = feature::GetBestMrnaForCds(cds, &mTree);
+    string protein_id = cds.GetNamedQual("protein_id");
+    const bool no_protein_id_qual = NStr::IsBlank(protein_id);
+    if (no_protein_id_qual) { // no protein_id qual on CDS - check mRNA and ID qual
+        if (mrna) {
+            protein_id = mrna.GetNamedQual("protein_id");
+        }
+        if (NStr::IsBlank(protein_id)) {
+            protein_id = cds.GetNamedQual("ID");
+        }
+    }
+    bool is_genbank_protein = NStr::StartsWith(protein_id, "gb|");
+    
+    string transcript_id = cds.GetNamedQual("transcript_id");
+    const bool no_transcript_id_qual = NStr::IsBlank(transcript_id);
+    if (no_transcript_id_qual) { // no transcript_id qual on CDS - check mRNA
+        if (mrna) { 
+            transcript_id = s_GetTranscriptIdFromMrna(mrna);
+        }
+    }
+    bool is_genbank_transcript = NStr::StartsWith(transcript_id, "gb|");
+
+   
+    if ((is_genbank_protein ||
+         NStr::StartsWith(protein_id, "gnl|")) &&
+        (is_genbank_transcript ||
+         NStr::StartsWith(transcript_id, "gnl|"))) { // No further processing of ids is required - simply assign to features
+        if (no_protein_id_qual) { // protein_id from ID qualifier or mRNA => need to add protein_id qual to CDS
+            xFeatureSetQualifier(cds, "protein_id", protein_id);
+        }
+        
+        if (no_transcript_id_qual) { // transcript_id from mRNA => need to add transcript_id qual to CDS
+            xFeatureSetQualifier(cds, "transcript_id", transcript_id);
+        }
+        
+        if (mrna) {
+            xAddTranscriptAndProteinIdsToMrna(transcript_id,  
+                                              protein_id,
+                                              mrna);
+        }
+        return;
+    }
+
+    // else need to generate and/or process ids before we add them to features
+    bool has_protein_id = !NStr::IsBlank(protein_id);
+    bool has_transcript_id = !NStr::IsBlank(transcript_id);
+
+    if ( has_protein_id &&
+         has_transcript_id ) {
+        if (!is_genbank_protein &&
+            (transcript_id == protein_id)) {
+            protein_id = "cds." + protein_id;
+        }
+    }
+    else {
+        if ( has_protein_id &&
+            !is_genbank_protein) {
+            transcript_id = "mrna." + protein_id;
+            has_transcript_id = true;
+
+        }
+        else 
+        if (has_transcript_id &&
+            !is_genbank_transcript) {
+            protein_id = "cds." + transcript_id;
+            has_protein_id = true;
+        }
+
+        // Generate new transcript_id and protein_id if necessary
+        if (!has_transcript_id) {
+            transcript_id = xNextTranscriptId(cds);
+        }
+
+        if (!has_protein_id) {
+            protein_id = xNextProteinId(cds);
+        }
+    } 
+
+    xConvertToGeneralIds(cds, transcript_id, protein_id);    
+
+    if (mrna) {
+        xAddTranscriptAndProteinIdsToMrna(transcript_id,  
+                                          protein_id,
+                                          mrna);
+    }
+
+
+    xFeatureSetQualifier(cds, "transcript_id", transcript_id);
+    xFeatureSetQualifier(cds, "protein_id", protein_id);
+}
+
+
+// ---------------------------------------------------------------------------
+void CFeatTableEdit::xAddTranscriptAndProteinIdsToMrna(const string& cds_transcript_id, 
+                                                       const string& cds_protein_id, 
+                                                       CMappedFeat& mrna)
+// ---------------------------------------------------------------------------
+{
+    if (mProcessedMrnas.find(mrna) != mProcessedMrnas.end()) {
+        return;
+    }
+   
+    bool use_local_ids = false; // 
+
+    string transcript_id = mrna.GetNamedQual("transcript_id");
+    if (NStr::IsBlank(transcript_id)) {
+        transcript_id = cds_transcript_id;
+    }
+    else {
+        use_local_ids = true;
+    }
+
+    string protein_id = mrna.GetNamedQual("protein_id");
+    if (NStr::IsBlank(protein_id)) {
+        protein_id = cds_protein_id;
+    }
+    else {
+        if ((protein_id == transcript_id) &&
+             !NStr::StartsWith(protein_id, "gb|")) {
+            protein_id = "cds." + protein_id;
+        }
+        use_local_ids = true;
+    }
+
+    if (use_local_ids) { // protein id and/or transcript id specified on mRNA.
+                         // Process these ids and check that the processed quals match the qualifiers on the child CDS feature
+        xConvertToGeneralIds(mrna, transcript_id, protein_id);
+
+        if (transcript_id  != cds_transcript_id) {
+            xPutErrorDifferingTranscriptIds(mrna);
+        }
+
+        if (protein_id != cds_protein_id) {
+            xPutErrorDifferingProteinIds(mrna);
+        }
+    }
+
+    xFeatureSetQualifier(mrna, "transcript_id", transcript_id);
+    xFeatureSetQualifier(mrna, "protein_id", protein_id);
+
+    mProcessedMrnas.insert(mrna);
+}
+
+
+// ---------------------------------------------------------------------------
+void CFeatTableEdit::xAddTranscriptAndProteinIdsToUnmatchedMrna(CMappedFeat& mrna)
+// ---------------------------------------------------------------------------
+{
+    if (mProcessedMrnas.find(mrna) != mProcessedMrnas.end()) {
+        return;
+    }
+
+    string transcript_id = mrna.GetNamedQual("transcript_id");
+    bool no_transcript_id_qual = NStr::IsBlank(transcript_id);
+    if (no_transcript_id_qual) {
+        transcript_id = mrna.GetNamedQual("ID");
+    }
+    const bool is_genbank_transcript = NStr::StartsWith(transcript_id, "gb|");
+
+    string protein_id = mrna.GetNamedQual("protein_id");
+    const bool is_genbank_protein = NStr::StartsWith(protein_id, "gb|");
+
+    if ((is_genbank_protein ||
+         NStr::StartsWith(protein_id, "gnl|")) &&
+        (is_genbank_transcript ||
+         NStr::StartsWith(transcript_id, "gnl|"))) {
+        if (no_transcript_id_qual) {
+            xFeatureSetQualifier(mrna, "transcript_id", transcript_id);
+        }
+        return;
+    } 
+    
+    if (!NStr::IsBlank(protein_id) && 
+        !NStr::IsBlank(transcript_id)) {
+        if ((transcript_id == protein_id) &&
+            !is_genbank_transcript) {
+            protein_id = "cds." + protein_id;
+        }
+    } 
+    else 
+    if (!is_genbank_protein && !NStr::IsBlank(protein_id)) {
+        transcript_id = "mrna." + protein_id;
+    }
+    else 
+    if (!is_genbank_transcript && !NStr::IsBlank(transcript_id)) {
+        protein_id = "cds." + transcript_id;
+    }
+
+    if (NStr::IsBlank(protein_id)) {
+        protein_id = xNextProteinId(mrna);
+    }
+    if (NStr::IsBlank(transcript_id)) {
+        transcript_id = xNextTranscriptId(mrna);
+    }
+
+    xConvertToGeneralIds(mrna, transcript_id, protein_id);
+    
+    xFeatureSetQualifier(mrna, "transcript_id", transcript_id);
+    xFeatureSetQualifier(mrna, "protein_id", protein_id);
+
+    mProcessedMrnas.insert(mrna);
+}
+
+
+void CFeatTableEdit::xConvertToGeneralIds(const CMappedFeat& mf, 
+                                          string& transcript_id,
+                                          string& protein_id)
+{
+    const bool update_protein_id = s_ShouldConvertToGeneral(protein_id);
+    const bool update_transcript_id = s_ShouldConvertToGeneral(transcript_id);
+
+    string locus_tag_prefix;
+    if (update_protein_id || update_transcript_id) {
+        locus_tag_prefix = xGetCurrentLocusTagPrefix(mf);
+        if (NStr::IsBlank(locus_tag_prefix)) {
+            xPutErrorMissingLocustag(mf);
+            return;
+        }
+    }
+
+    if (update_protein_id) {
+        protein_id = "gnl|" + locus_tag_prefix + "|" + protein_id;
+    }
+
+    if (update_transcript_id) {
+        transcript_id = "gnl|" + locus_tag_prefix + "|" + transcript_id;
+    }
+}
+
+
+//  ----------------------------------------------------------------------------
+void CFeatTableEdit::InstantiateProducts()
 //  ----------------------------------------------------------------------------
 {
     SAnnotSelector sel1;
@@ -351,31 +647,21 @@ void CFeatTableEdit::GenerateProteinAndTranscriptIds()
     sel1.IncludeFeatSubtype(CSeqFeatData::eSubtype_cdregion);
     for (CFeat_CI it(mHandle, sel1); it; ++it) {
         CMappedFeat mf = *it;
-        switch(mf.GetFeatSubtype()) {
-        default:
-            break;
-        case CSeqFeatData::eSubtype_mRNA:
-            xFeatureAddTranscriptIdMrna(mf);
-            break;
-        case CSeqFeatData::eSubtype_cdregion:
-            xFeatureAddProteinIdCds(mf);
-            break;
+
+        auto tid = mf.GetNamedQual("transcript_id");
+        if (!tid.empty()) {
+            xFeatureRemoveQualifier(mf, "transcript_id");
+            xFeatureAddQualifier(mf, "orig_transcript_id", tid);
         }
-    }
-    SAnnotSelector sel2;
-    for (CFeat_CI it(mHandle, sel2); it; ++it) {
-        CMappedFeat mf = *it;
-        switch(mf.GetFeatSubtype()) {
-        default:
-            xFeatureAddProteinIdDefault(mf);
-            xFeatureAddTranscriptIdDefault(mf);
-            break;
-        case CSeqFeatData::eSubtype_mRNA:
-            xFeatureAddProteinIdMrna(mf);
-            break;
-        case CSeqFeatData::eSubtype_cdregion:
-            xFeatureAddTranscriptIdCds(mf);
-            break;
+
+        if (mf.GetFeatSubtype() == CSeqFeatData::eSubtype_cdregion) {
+            auto pid = mf.GetNamedQual("protein_id");
+            if (!pid.empty()) {
+                if (!mf.IsSetProduct()) {
+                    xFeatureSetProduct(mf, pid);
+                }
+                xFeatureRemoveQualifier(mf, "protein_id");
+            }
         }
     }
 }
@@ -487,23 +773,21 @@ void CFeatTableEdit::xFeatureAddProteinIdMrna(
         // already what we want
         return;
     }
-    if (pid.empty()) {
-    // we need to upinherit the protein_id from the CDS:
-        CMappedFeat child = feature::GetBestCdsForMrna(mf, &mTree);
-        if (!child) {
-            // only permitted case of an mRNA without a protein_id
-            return;
-        }
-        auto pid = child.GetNamedQual("protein_id");
-        xFeatureAddQualifier(mf, "protein_id", pid);
+    //reformat any tags we already have:
+    if (!pid.empty()) {
+        pid = string("gnl|") + xGetCurrentLocusTagPrefix(mf) + "|" + pid;
+        xFeatureSetQualifier(mf, "protein_id", pid);
         return;
     }
 
-    // otherwise, we need to police the existing transcript_id:
-    pid = xGenerateTranscriptOrProteinId(mf, pid);
-    if (!pid.empty()) {
-        xFeatureSetQualifier(mf, "protein_id", pid);
+    CMappedFeat child = feature::GetBestCdsForMrna(mf, &mTree);
+    if (!child) {
+        // only permitted case of an mRNA without a protein_id
+        return;
     }
+    pid = child.GetNamedQual("protein_id");
+    xFeatureAddQualifier(mf, "protein_id", pid);
+    return;
 }
 
 //  ----------------------------------------------------------------------------
@@ -517,6 +801,7 @@ void CFeatTableEdit::xFeatureAddProteinIdCds(
     // if one exists already then police it
     // if it doen't have one then generate one following a strict set of rules
 
+
     auto orig_pid = mf.GetNamedQual("orig_protein_id");
     if (!orig_pid.empty()) {
         xFeatureRemoveQualifier(mf, "orig_protein_id");
@@ -527,14 +812,33 @@ void CFeatTableEdit::xFeatureAddProteinIdCds(
         // already what we want
         return;
     }
-
-    if (pid.empty()) {
-        pid = mf.GetNamedQual("ID"); 
-        if (pid.empty()) {
-            pid = xGetIdStr(mf);
-        }
+    //reformat any tags we already have:
+    if (!pid.empty()) {
+        pid = string("gnl|") + xGetCurrentLocusTagPrefix(mf) + "|" + pid;
+        xFeatureSetQualifier(mf, "protein_id", pid);
+        return;
     }
-    pid = xGenerateTranscriptOrProteinId(mf, pid);
+
+    auto id = mf.GetNamedQual("ID");
+    if (!id.empty()) {
+        pid = string("gnl|") + xGetCurrentLocusTagPrefix(mf) + "|" + id;
+        xFeatureSetQualifier(mf, "protein_id", pid);
+        return;
+    }
+
+    auto tid = mf.GetNamedQual("transcript_id");
+    if (!tid.empty()) {
+        CMappedFeat parent = feature::GetBestMrnaForCds(mf, &mTree);
+        auto mRnaTid = parent.GetNamedQual("transcript_id");
+        if (tid == mRnaTid) {
+            tid = string("cds.") + tid;
+        }
+        pid = string("gnl|") + xGetCurrentLocusTagPrefix(mf) + "|" + tid;
+        xFeatureSetQualifier(mf, "protein_id", pid);
+        return;
+    }
+
+    pid = xNextProteinId(mf);
     if (!pid.empty()) {
         xFeatureSetQualifier(mf, "protein_id", pid);
     }
@@ -559,7 +863,15 @@ void CFeatTableEdit::xFeatureAddProteinIdDefault(
         // already what we want
         return;
     }
-    pid = xGenerateTranscriptOrProteinId(mf, pid);
+    //reformat any tags we already have:
+    if (!pid.empty()) {
+        pid = string("gnl|") + xGetCurrentLocusTagPrefix(mf) + "|" + pid;
+        xFeatureSetQualifier(mf, "protein_id", pid);
+        return;
+    }
+
+    //pid = xGenerateTranscriptOrProteinId(mf, pid);
+    pid = xNextProteinId(mf);
     if (!pid.empty()) {
         xFeatureSetQualifier(mf, "protein_id", pid);
     }
@@ -587,14 +899,26 @@ void CFeatTableEdit::xFeatureAddTranscriptIdMrna(
         // already what we want
         return;
     }
-
-    if (tid.empty()) {
-        tid = mf.GetNamedQual("ID"); 
-        if (tid.empty()) {
-            tid = xGetIdStr(mf);
-        }
+    //reformat any tags we already have:
+    if (!tid.empty()) {
+        tid = string("gnl|") + xGetCurrentLocusTagPrefix(mf) + "|" + tid;
+        xFeatureSetQualifier(mf, "transcript_id", tid);
+        return;
     }
-    tid = xGenerateTranscriptOrProteinId(mf, tid);
+    else {
+    }
+
+
+
+    auto id = mf.GetNamedQual("ID");
+    if (!id.empty()) {
+        tid = string("gnl|") + xGetCurrentLocusTagPrefix(mf) + "|" + id;
+        xFeatureSetQualifier(mf, "transcript_id", tid);
+        return;
+    }
+
+    //tid = xGenerateTranscriptOrProteinId(mf, tid);
+    tid = xNextTranscriptId(mf);
     if (!tid.empty()) {
         xFeatureSetQualifier(mf, "transcript_id", tid);
     }
@@ -618,27 +942,25 @@ void CFeatTableEdit::xFeatureAddTranscriptIdCds(
     }
 
     auto tid = mf.GetNamedQual("transcript_id");
-    if (tid.empty()) {
-    // we need to down inherit the transcript_id from the mRNA:
-        CMappedFeat parent = feature::GetBestMrnaForCds(mf, &mTree);
-        if (!parent) {
-            // only permitted case of a CDS without a transcript_id
-            return;
-        }
-        auto tid = parent.GetNamedQual("transcript_id");
-        xFeatureAddQualifier(mf, "transcript_id", tid);
-        return;
-    }
-
     // otherwise, we need to police the existing transcript_id:
     if (NStr::StartsWith(tid, "gb|")  ||  NStr::StartsWith(tid, "gnl|")) {
         // already what we want
         return;
     }
-    tid = xGenerateTranscriptOrProteinId(mf, tid);
+    //reformat any tags we already have:
     if (!tid.empty()) {
+        tid = string("gnl|") + xGetCurrentLocusTagPrefix(mf) + "|" + tid;
         xFeatureSetQualifier(mf, "transcript_id", tid);
+        return;
     }
+
+    CMappedFeat parent = feature::GetBestMrnaForCds(mf, &mTree);
+    if (!parent) {
+        // only permitted case of a CDS without a transcript_id
+        return;
+    }
+    tid = parent.GetNamedQual("transcript_id");
+    xFeatureAddQualifier(mf, "transcript_id", tid);
 }
 
 
@@ -661,7 +983,13 @@ void CFeatTableEdit::xFeatureAddTranscriptIdDefault(
         // already what we want
         return;
     }
-    tid = xGenerateTranscriptOrProteinId(mf, tid);
+    if (!tid.empty()) {
+        tid = string("gnl|") + xGetCurrentLocusTagPrefix(mf) + "|" + tid;
+        xFeatureSetQualifier(mf, "transcript_id", tid);
+        return;
+    }
+
+    tid = this->xNextTranscriptId(mf);
     if (!tid.empty()) {
         xFeatureSetQualifier(mf, "transcript_id", tid);
     }
@@ -674,23 +1002,66 @@ string CFeatTableEdit::xGenerateTranscriptOrProteinId(
     const string& rawId)
 //  ----------------------------------------------------------------------------
 {
+    //weed out badly formatted original tags:
      if (string::npos != rawId.find("|")) {
         xPutError(
-            ILineError::eProblem_InvalidQualifier,
             "Feature " + xGetIdStr(mf) + 
                 " does not have a usable transcript_ or protein_id.");
         return "";
     }
 
+    //make sure we got the locus tag prefix necessary for tag generation:
     auto locusTagPrefix = xGetCurrentLocusTagPrefix(mf);
     if (locusTagPrefix.empty()) {
         xPutError(
-            ILineError::eProblem_InvalidQualifier,
-            "Cannot generate transcript_/protein_id for feature " + xGetIdStr(mf) + 
-                " without a locus tag.");
+            "Cannot generate transcript_/protein_id for feature " + xGetIdStr(mf) +
+            " without a locus tag.");
         return "";
     }
-    return string("gnl|") + locusTagPrefix + "|" + rawId;
+
+    //reformat any tags we already have:
+    if (!rawId.empty()) {
+        return string("gnl|") + locusTagPrefix + "|" + rawId;
+    }
+    
+    //attempt to make transcript_id from protein_id, or protein_id from transcript_id:
+    switch (mf.GetFeatSubtype()) {
+        case CSeqFeatData::eSubtype_mRNA: {
+                auto id = mf.GetNamedQual("protein_id");
+                if (id.empty()) {
+                    id = mf.GetNamedQual("ID");
+                }
+                if (id.empty()) {
+                    if (mf.GetId().IsLocal()) {
+                        id = mf.GetId().GetLocal().GetStr();
+                    }
+                }
+                if (!id.empty()) {
+                    return string("gnl|") + locusTagPrefix + "|mrna." + id;
+                }
+            }
+            break;
+
+        case CSeqFeatData::eSubtype_cdregion: {
+                auto id = mf.GetNamedQual("transcript_id");
+                if (id.empty()) {
+                    id = mf.GetNamedQual("ID");
+                }
+                if (!id.empty()) {
+                    return string("gnl|") + locusTagPrefix + "|cds." + id;
+                }
+            }
+            break;
+
+        default:
+            break;
+    }
+
+
+    xPutError(
+        "Cannot generate transcript_/protein_id for feature " + xGetIdStr(mf) +
+        " - insufficient context.");
+    return "";
 }
 
 
@@ -936,14 +1307,8 @@ void CFeatTableEdit::xFeatureAddQualifier(
 //  ----------------------------------------------------------------------------
 {
     const CSeq_feat& origFeat = mf.GetOriginalFeature();
-    CRef<CSeq_feat> pEditedFeat(new CSeq_feat);
-    pEditedFeat->Assign(origFeat);
-    CRef<CGb_qual> pQual(new CGb_qual);
-    pQual->SetQual(qualKey);
-    pQual->SetVal(qualVal);
-    pEditedFeat->SetQual().push_back(pQual);
     CSeq_feat_EditHandle feh(mpScope->GetObjectHandle(origFeat));
-    feh.Replace(*pEditedFeat);
+    feh.AddQualifier(qualKey, qualVal);
 }
 
 
@@ -954,11 +1319,8 @@ void CFeatTableEdit::xFeatureRemoveQualifier(
 //  ----------------------------------------------------------------------------
 {
     const CSeq_feat& origFeat = mf.GetOriginalFeature();
-    CRef<CSeq_feat> pEditedFeat(new CSeq_feat);
-    pEditedFeat->Assign(origFeat);
-    pEditedFeat->RemoveQualifier(qualKey);
     CSeq_feat_EditHandle feh(mpScope->GetObjectHandle(origFeat));
-    feh.Replace(*pEditedFeat);
+    feh.RemoveQualifier(qualKey);
 }
 
 //  ----------------------------------------------------------------------------
@@ -968,17 +1330,27 @@ void CFeatTableEdit::xFeatureSetQualifier(
     const string& qualVal)
 //  ----------------------------------------------------------------------------
 {
+    auto existing = mf.GetNamedQual(qualKey);
+    if (!existing.empty()) {
+        xFeatureRemoveQualifier(mf, qualKey);
+    }
+    xFeatureAddQualifier(mf, qualKey, qualVal);
+}
+
+//  ----------------------------------------------------------------------------
+void CFeatTableEdit::xFeatureSetProduct(
+    CMappedFeat mf,
+    const string& proteinIdStr)
+//  ----------------------------------------------------------------------------
+{
+    CRef<CSeq_id> pProteinId( 
+        new CSeq_id(proteinIdStr, 
+            CSeq_id::fParse_ValidLocal|CSeq_id::fParse_PartialOK));
+
     const CSeq_feat& origFeat = mf.GetOriginalFeature();
     CRef<CSeq_feat> pEditedFeat(new CSeq_feat);
     pEditedFeat->Assign(origFeat);
-    auto existing = pEditedFeat->GetNamedQual(qualKey);
-    if (!existing.empty()) {
-        pEditedFeat->RemoveQualifier(qualKey);
-    }
-    CRef<CGb_qual> pQual(new CGb_qual);
-    pQual->SetQual(qualKey);
-    pQual->SetVal(qualVal);
-    pEditedFeat->SetQual().push_back(pQual);
+    pEditedFeat->SetProduct().SetWhole(*pProteinId);
     CSeq_feat_EditHandle feh(mpScope->GetObjectHandle(origFeat));
     feh.Replace(*pEditedFeat);
 }
@@ -1018,31 +1390,36 @@ string CFeatTableEdit::xNextProteinId(
 {
     const string dbPrefix("gnl|");
 
-    // format: mLocusTagPrefix|<locus tag of gene>[_numeric disambiguation]
+    // format: mLocusTagPrefix|<locus tag of gene or hash>[_numeric disambiguation]
     CMappedFeat parentGene = feature::GetBestGeneForFeat(mf, &mTree);
-    if (!parentGene) {
-		return "";
-	}
-    if (!parentGene.GetData().GetGene().IsSetLocus_tag()) {
+    if (!parentGene ||
+        !parentGene.GetData().GetGene().IsSetLocus_tag()) {
+        xPutErrorMissingLocustag(mf);
         return "";
     }
+
+    int offset = 0;
     string locusTag = parentGene.GetData().GetGene().GetLocus_tag();
-	string disAmbig = "";
 	map<string, int>::iterator it = mMapProtIdCounts.find(locusTag);
 	if (it == mMapProtIdCounts.end()) {
 		mMapProtIdCounts[locusTag] = 0;
 	}
 	else {
 		++mMapProtIdCounts[locusTag];
-		disAmbig = string("_") + NStr::IntToString(mMapProtIdCounts[locusTag]);
+                offset = mMapProtIdCounts[locusTag];
 	}
     string db = mLocusTagPrefix;
+    if (locusTag.empty() &&
+        db.empty()) {
+        xPutErrorMissingLocustag(mf);
+    }
+
     if (db.empty()) {
         string prefix, suffix;
         NStr::SplitInTwo(locusTag, "_", prefix, suffix);
         db = prefix;
     }
-    string proteinId = dbPrefix + db + "|" + locusTag + disAmbig;
+    string proteinId = dbPrefix + db + "|" + GetIdHashOrValue(locusTag, offset);
     return proteinId;
 }
 
@@ -1053,27 +1430,37 @@ string CFeatTableEdit::xNextTranscriptId(
 {
     const string dbPrefix("gnl|");
 
-	// format: mLocusTagPrefix|mrna.<locus tag of gene>[_numeric disambiguation]
+
+	// format: mLocusTagPrefix|mrna.<locus tag of gene or hash>[_numeric disambiguation]
 	CMappedFeat parentGene = feature::GetBestGeneForFeat(cds, &mTree);
-    if (!parentGene) {
+    if (!parentGene ||
+        !parentGene.GetData().GetGene().IsSetLocus_tag()) {
+        xPutErrorMissingLocustag(cds);
 		return "";
 	}
-    if (!parentGene.GetData().GetGene().IsSetLocus_tag()) {
-        return "";
-    }
+
+
     string locusTag = parentGene.GetData().GetGene().GetLocus_tag();
-    string disAmbig = "";
+    int offset = 0;
 	map<string, int>::iterator it = mMapProtIdCounts.find(locusTag);
 	if (it != mMapProtIdCounts.end()  &&  mMapProtIdCounts[locusTag] != 0) {
-		disAmbig = string("_") + NStr::IntToString(mMapProtIdCounts[locusTag]);
+		offset = mMapProtIdCounts[locusTag];
 	}
     string db = mLocusTagPrefix;
+
+    if (locusTag.empty() &&
+        db.empty()) {
+        xPutErrorMissingLocustag(cds);
+    }
+
+
     if (db.empty()) {
         string prefix, suffix;
         NStr::SplitInTwo(locusTag, "_", prefix, suffix);
         db = prefix;
     }
-    string transcriptId = dbPrefix + db + "|mrna." + locusTag + disAmbig;
+
+    string transcriptId = dbPrefix + db + "|mrna." + GetIdHashOrValue(locusTag, offset);
 	return transcriptId;
 }
 
@@ -1105,23 +1492,22 @@ CRef<CSeq_feat> CFeatTableEdit::xMakeGeneForFeature(
     return pGene;
 }
 
+
 //  ----------------------------------------------------------------------------
 void
 CFeatTableEdit::xPutError(
-    ILineError::EProblem problem,
     const string& message)
 //  ----------------------------------------------------------------------------
-{
-    AutoPtr<CObjReaderLineException> pErr(
-        CObjReaderLineException::Create(
-        eDiag_Error,
-        0,
-        message,
-        problem));
-    pErr->SetLineNumber(0);
-    mpMessageListener->PutError(*pErr);
+{   
+    if (!mpMessageListener) {
+        return;
+    }
+
+    mpMessageListener->PutMessage(
+        CObjEditMessage(message, eDiag_Error));
 }
 
+
 //  ----------------------------------------------------------------------------
 void
 CFeatTableEdit::xPutErrorMissingLocustag(
@@ -1139,14 +1525,9 @@ CFeatTableEdit::xPutErrorMissingLocustag(
     subName = NStr::IntToString(lower) + ".." + NStr::IntToString(upper) + " " +
         subName;
 
-    AutoPtr<CObjReaderLineException> pErr(
-        CObjReaderLineException::Create(
-        eDiag_Error,
-        0,
-        subName + " feature is missing locus tag.",
-        ILineError::eProblem_Missing));
-    pErr->SetLineNumber(0);
-    mpMessageListener->PutError(*pErr);
+    string message = subName + " feature is missing locus tag.";
+
+    xPutError(message);
 }
 
 //  ----------------------------------------------------------------------------
@@ -1166,14 +1547,9 @@ CFeatTableEdit::xPutErrorMissingTranscriptId(
     subName = NStr::IntToString(lower) + ".." + NStr::IntToString(upper) + " " +
         subName;
 
-    AutoPtr<CObjReaderLineException> pErr(
-        CObjReaderLineException::Create(
-        eDiag_Error,
-        0,
-        subName + " feature is missing transcript ID.",
-        ILineError::eProblem_Missing));
-    pErr->SetLineNumber(0);
-    mpMessageListener->PutError(*pErr);
+    string message = subName + " feature is missing transcript ID.";
+
+    xPutError(message);
 }
 
 //  ----------------------------------------------------------------------------
@@ -1192,17 +1568,48 @@ CMappedFeat mf)
     unsigned int upper = mf.GetLocation().GetStop(eExtreme_Positional);
     subName = NStr::IntToString(lower) + ".." + NStr::IntToString(upper) + " " +
         subName;
-
-    AutoPtr<CObjReaderLineException> pErr(
-        CObjReaderLineException::Create(
-        eDiag_Error,
-        0,
-        subName + " feature is missing protein ID.",
-        ILineError::eProblem_Missing));
-    pErr->SetLineNumber(0);
-    mpMessageListener->PutError(*pErr);
+        
+    string message = subName + " feature is missing protein ID.";
+    xPutError(message);
 }
 
+
+//  ----------------------------------------------------------------------------
+void
+CFeatTableEdit::xPutErrorDifferingProteinIds(
+const CMappedFeat& mrna)
+//  ----------------------------------------------------------------------------
+{
+    if (!mpMessageListener) {
+        return;
+    }
+
+    if (mrna.GetFeatSubtype() != CSeqFeatData::eSubtype_mRNA) {
+        return;
+    }
+    xPutError("Protein ID on mRNA feature differs from protein ID on child CDS.");
+
+//    xPutError(ILineError::eProblem_InconsistentQualifiers,
+//        "Protein ID on mRNA feature differs from protein ID on child CDS.");
+}
+
+void
+CFeatTableEdit::xPutErrorDifferingTranscriptIds(
+const CMappedFeat& mrna)
+//  ----------------------------------------------------------------------------
+{
+    if (!mpMessageListener) {
+        return;
+    }
+
+    if (mrna.GetFeatSubtype() != CSeqFeatData::eSubtype_mRNA) {
+        return;
+    }
+   
+    xPutError("Transcript ID on mRNA feature differs from transcript ID on child CDS.");
+}
+
+
 //  ----------------------------------------------------------------------------
 string
 CFeatTableEdit::xGetIdStr(
