@@ -1,4 +1,4 @@
-/*  $Id: comment_item.cpp 528985 2017-02-28 01:19:12Z kans $
+/*  $Id: comment_item.cpp 559026 2018-03-06 01:23:45Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -100,7 +100,9 @@ CCommentItem::CCommentItem
     m_NeedPeriod(true)
 {
     m_Comment.push_back( comment );
-    ExpandTildes(m_Comment.back(), eTilde_comment);
+    if (! ctx.Config().IsFormatGBSeq() && ! ctx.Config().IsFormatINSDSeq()) {
+        ExpandTildes(m_Comment.back(), eTilde_comment);
+    }
     swap(m_First, sm_FirstComment);
     if ( obj != 0 ) {
         x_SetObject(*obj);
@@ -1464,6 +1466,7 @@ string CCommentItem::GetStringForBaseMod(CBioseqContext& ctx)
             FOR_EACH_STRING_IN_VECTOR (itr, sBasemodURLs) {
                 string url = *itr;
                 if ( ! url.empty() ) {
+                    NStr::ReplaceInPlace( url, "\"", "" );
                     str << "<a href=\"" << url << "\">" << "base modification file" << "</a>";
                 }
             }
@@ -1482,6 +1485,7 @@ string CCommentItem::GetStringForBaseMod(CBioseqContext& ctx)
             FOR_EACH_STRING_IN_VECTOR (itr, sBasemodURLs) {
                 string url = *itr;
                 if ( ! url.empty() ) {
+                    NStr::ReplaceInPlace( url, "\"", "" );
                     j++;
                     str << pfx << "<a href=\"" << url << "\">" << j << "</a>";
                     if ( numBases == 2 ) {
@@ -1533,7 +1537,7 @@ void CCommentItem::x_GatherInfo(CBioseqContext& ctx)
     }
     const CSeqdesc* desc = dynamic_cast<const CSeqdesc*>(obj);
     if (desc != NULL) {
-        x_GatherDescInfo(*desc);
+        x_GatherDescInfo(*desc, ctx);
     } else {
         const CSeq_feat* feat = dynamic_cast<const CSeq_feat*>(obj);
         if (feat != NULL) {
@@ -1549,7 +1553,8 @@ void CCommentItem::x_GatherInfo(CBioseqContext& ctx)
 
 // returns the data_str, but wrapped in appropriate <a href...>...</a> if applicable
 static
-string s_HtmlizeStructuredCommentData( const bool is_html, const string &label_str, const string &data_str, const char* provider, const char* status )
+string s_HtmlizeStructuredCommentData( const bool is_html, const string &label_str, const string &data_str,
+                                       const char* provider, const char* status, bool has_name )
 {
     if( ! is_html ) {
         return data_str;
@@ -1566,7 +1571,16 @@ string s_HtmlizeStructuredCommentData( const bool is_html, const string &label_s
                << data_str
                << "\">" << data_str << "</a>";
         return CNcbiOstrstreamToString(result);
-    } else if ( NStr::Equal (label_str, "Annotation Version") && NStr::Equal (provider, "NCBI") && NStr::Equal (status, "Full annotation") ) {
+    } else if ( NStr::Equal (label_str, "Annotation Name") && NStr::Equal (provider, "NCBI") ) {
+        string fst;
+        string snd;
+        NStr::Replace( data_str, " Annotation Release ", "/", fst );
+        NStr::Replace( fst, " ", "_", snd );
+        result << "<a href=\"https://www.ncbi.nlm.nih.gov/genome/annotation_euk/"
+               << snd
+               << "\">" << data_str << "</a>";
+        return CNcbiOstrstreamToString(result);
+    } else if ( NStr::Equal (label_str, "Annotation Version") && NStr::Equal (provider, "NCBI") && NStr::Equal (status, "Full annotation") && (! has_name) ) {
         string fst;
         string snd;
         NStr::Replace( data_str, " Annotation Release ", "/", fst );
@@ -1599,6 +1613,7 @@ void s_GetStrForStructuredComment(
     const char* suffix = "##Metadata-END##";
     const char* provider = "";
     const char* status = "";
+    bool has_name = false;
 
     bool fieldOverThreshold = false;
 
@@ -1619,6 +1634,8 @@ void s_GetStrForStructuredComment(
                     provider = (*it_for_len)->GetData().GetStr().c_str();
                 } else if ( label == "Annotation Status" ) {
                     status = (*it_for_len)->GetData().GetStr().c_str();
+                } else if ( label == "Annotation Name" ) {
+                    has_name = true;
                 }
                 const string::size_type label_len = label.length();
                 if( (label_len > longest_label_len) && (label_len <= kFieldLenThreshold) ) {
@@ -1668,7 +1685,7 @@ void s_GetStrForStructuredComment(
             next_line.resize( max( next_line.size(), longest_label_len), ' ' );
         }
         next_line.append( " :: " );
-        next_line.append( s_HtmlizeStructuredCommentData( is_html, (*it)->GetLabel().GetStr(), (*it)->GetData().GetStr(), provider, status ) );
+        next_line.append( s_HtmlizeStructuredCommentData( is_html, (*it)->GetLabel().GetStr(), (*it)->GetData().GetStr(), provider, status, has_name ) );
         next_line.append( "\n" );
 
         ExpandTildes(next_line, eTilde_comment);
@@ -1678,7 +1695,7 @@ void s_GetStrForStructuredComment(
     out_lines.back().append( "\n" );
 }
 
-void CCommentItem::x_GatherDescInfo(const CSeqdesc& desc)
+void CCommentItem::x_GatherDescInfo(const CSeqdesc& desc, CBioseqContext& ctx)
 {
     // true for most desc infos
     EPeriod can_add_period = ePeriod_Add;
@@ -1756,7 +1773,7 @@ void CCommentItem::x_GatherDescInfo(const CSeqdesc& desc)
     if (str.empty()  ||  str == ".") {
         return;
     }
-    x_SetCommentWithURLlinks(prefix, str, suffix, can_add_period);
+    x_SetCommentWithURLlinks(prefix, str, suffix, ctx, can_add_period);
     
 }
 
@@ -1769,7 +1786,7 @@ void CCommentItem::x_GatherFeatInfo(const CSeq_feat& feat, CBioseqContext& ctx)
         return;
     }
 
-    x_SetCommentWithURLlinks(kEmptyStr, feat.GetComment(), kEmptyStr, ePeriod_Add);
+    x_SetCommentWithURLlinks(kEmptyStr, feat.GetComment(), kEmptyStr, ctx, ePeriod_Add);
 }
 
 void CCommentItem::x_GatherUserObjInfo(const CUser_object& userObject )
@@ -1795,7 +1812,17 @@ void CCommentItem::x_SetComment(const string& comment)
 {
     m_Comment.clear();
     m_Comment.push_back( comment );
-    ExpandTildes(m_Comment.back(), eTilde_comment);;
+    ExpandTildes(m_Comment.back(), eTilde_comment);
+}
+
+
+void CCommentItem::x_SetComment(const string& comment, CBioseqContext& ctx)
+{
+    m_Comment.clear();
+    m_Comment.push_back( comment );
+    if (! ctx.Config().IsFormatGBSeq() && ! ctx.Config().IsFormatINSDSeq()) {
+        ExpandTildes(m_Comment.back(), eTilde_comment);
+    }
 }
 
 
@@ -1803,6 +1830,7 @@ void CCommentItem::x_SetCommentWithURLlinks
 (const string& prefix,
  const string& str,
  const string& suffix,
+ CBioseqContext& ctx,
  EPeriod can_add_period)
 {
     // !!! test for html - find links within the comment string
@@ -1810,7 +1838,9 @@ void CCommentItem::x_SetCommentWithURLlinks
     comment += str;
     comment += suffix;
 
-    ExpandTildes(comment, eTilde_comment);
+    if (! ctx.Config().IsFormatGBSeq() && ! ctx.Config().IsFormatINSDSeq()) {
+        ExpandTildes(comment, eTilde_comment);
+    }
     if (NStr::IsBlank(comment)) {
         return;
     }
@@ -1989,7 +2019,7 @@ void CGenomeAnnotComment::x_GatherInfo(CBioseqContext& ctx)
     }
 
     string s = (string)(CNcbiOstrstreamToString(text));
-    x_SetComment(s);
+    x_SetComment(s, ctx);
 }
 
 
