@@ -1,4 +1,4 @@
-/*  $Id: context.cpp 530533 2017-03-15 17:22:48Z gotvyans $
+/*  $Id: context.cpp 573606 2018-10-30 11:57:09Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -120,6 +120,7 @@ CBioseqContext::CBioseqContext
     m_HasMultiIntervalGenes(true), // true is the safe choice if we're not sure
     m_IsGenomeAssembly(false),
     m_IsCrossKingdom(false),
+    m_UsePDBCompoundForComment(false),
     m_fUnverified(fUnverified_None),
     m_ShowAnnotCommentAsCOMMENT(false),
     m_ShowAnnotCommentAsCOMMENT_checked(false),
@@ -181,6 +182,7 @@ CBioseqContext::CBioseqContext
     m_HasMultiIntervalGenes(true), // true is the safe choice if we're not sure
     m_IsGenomeAssembly(false),
     m_IsCrossKingdom(false),
+    m_UsePDBCompoundForComment(false),
     m_fUnverified(fUnverified_None),
     m_ShowAnnotCommentAsCOMMENT(false),
     m_ShowAnnotCommentAsCOMMENT_checked(false),
@@ -251,7 +253,7 @@ void CBioseqContext::x_Init(const CBioseq_Handle& seq, const CSeq_loc* user_loc)
     m_ShowAnnotCommentAsCOMMENT = false;
     m_ShowAnnotCommentAsCOMMENT_checked = false;
     
-    m_HasOperon = x_HasOperon();
+    // m_HasOperon = x_HasOperon();
 
     if (IsRefSeq()) {
         m_FFCtx.SetConfig().SetRefSeqConventions();
@@ -260,9 +262,9 @@ void CBioseqContext::x_Init(const CBioseq_Handle& seq, const CSeq_loc* user_loc)
     SAnnotSelector sel = SetAnnotSelector();
     sel.SetResolveAll();
 
-    x_SetHasMultiIntervalGenes();
+    // x_SetHasMultiIntervalGenes();
 
-    x_SetTaxname();
+    // x_SetTaxname();
 
     x_SetOpticalMapPoints();
 }
@@ -325,7 +327,7 @@ void CBioseqContext::x_SetMapper(const CSeq_loc& loc)
     }
 }
 
-void CBioseqContext::x_SetHasMultiIntervalGenes(void)
+void CBioseqContext::x_SetHasMultiIntervalGenes(void) const
 {
     m_HasMultiIntervalGenes = false;
 
@@ -350,7 +352,21 @@ void CBioseqContext::x_SetHasMultiIntervalGenes(void)
     }
 }
 
-void CBioseqContext::x_SetTaxname(void)
+bool CBioseqContext::HasMultiIntervalGenes(void) const
+{
+    if (UsingSeqEntryIndex()) {
+        CRef<CSeqEntryIndex> idx = GetSeqEntryIndex();
+        if (! idx) return false;
+        CRef<CBioseqIndex> bsx = idx->GetBioseqIndex (m_Handle);
+        if (! bsx) return false;
+        return bsx->HasMultiIntervalGenes();
+    }
+
+    x_SetHasMultiIntervalGenes();
+    return m_HasMultiIntervalGenes;
+}
+
+void CBioseqContext::x_SetTaxname(void) const
 {
     // look for taxname in Seqdescs
     int num_super_kingdom = 0;
@@ -425,6 +441,56 @@ void CBioseqContext::x_SetTaxname(void)
     }
 }
 
+const string& CBioseqContext::GetTaxname(void) const
+{
+    // check for indexed version first
+    if (UsingSeqEntryIndex()) {
+        CRef<CSeqEntryIndex> idx = GetSeqEntryIndex();
+        if (idx) {
+            CRef<CBioseqIndex> bsx = idx->GetBioseqIndex (m_Handle);
+            if (bsx) {
+                m_Taxname = bsx->GetTaxname();
+            }
+        }
+        return m_Taxname;
+    }
+
+    x_SetTaxname();
+    return m_Taxname;
+}
+
+
+bool CBioseqContext::IsCrossKingdom(void) const
+{
+    // check for indexed version first
+    if (UsingSeqEntryIndex()) {
+        CRef<CSeqEntryIndex> idx = GetSeqEntryIndex();
+        if (idx) {
+            CRef<CBioseqIndex> bsx = idx->GetBioseqIndex (m_Handle);
+            if (bsx) {
+                m_IsCrossKingdom = bsx->IsCrossKingdom();
+            }
+        }
+        return m_IsCrossKingdom;
+    }
+
+    x_SetTaxname();
+    return m_IsCrossKingdom;
+}
+
+
+bool CBioseqContext::UsePDBCompoundForComment(void) const
+{
+    return m_UsePDBCompoundForComment;
+}
+
+
+void CBioseqContext::SetPDBCompoundForComment(bool value)
+{
+    m_UsePDBCompoundForComment = value;
+}
+
+
 void CBioseqContext::x_SetFiletrackURL(const CUser_object& uo)
 {
     if ( ! FIELD_IS_SET_AND_IS(uo, Type, Str) || 
@@ -546,8 +612,9 @@ void CBioseqContext::x_SetDataFromUserObjects(void)
 
     for (CSeqdesc_CI it(m_Handle, CSeqdesc::e_User);  it;  ++it) {
         const CUser_object& uo = it->GetUser();
+        CUser_object::EObjectType utype = uo.GetObjectType();
         if (uo.IsSetType()  &&  uo.GetType().IsStr()) {
-            if (NStr::EqualNocase(uo.GetType().GetStr(), "StructuredComment")) {
+            if (utype == CUser_object::eObjectType_StructuredComment) {
                 if( uo.IsSetData() ) {
                     ITERATE( CUser_object::TData, field_iter, uo.GetData() ) {
                         const CUser_field &field = **field_iter;
@@ -571,37 +638,28 @@ void CBioseqContext::x_SetDataFromUserObjects(void)
                         }
                     }
                 }
-            } else if(NStr::EqualNocase(uo.GetType().GetStr(), "Unverified")) {
-                if( uo.IsSetData() ) {
-                    ITERATE( CUser_object::TData, field_iter, uo.GetData() ) {
-                        const CUser_field &field = **field_iter;
-                        if( ! field.IsSetLabel() || ! field.GetLabel().IsStr() ||
-                            field.GetLabel().GetStr() != "Type" ) 
-                        {
-                                continue;
-                        }
-                        if( ! field.IsSetData() || ! field.GetData().IsStr() )
-                        {
-                            continue;
-                        }
-                        if( NStr::EqualNocase( field.GetData().GetStr(), "Organism") ) {
-                            m_fUnverified |= fUnverified_Organism;
-                        } else if( NStr::EqualNocase( field.GetData().GetStr(), "Features") ) {
-                            m_fUnverified |= fUnverified_SequenceOrAnnotation;
-                        } else if( NStr::EqualNocase( field.GetData().GetStr(), "Misassembled") ) {
-                            m_fUnverified |= fUnverified_Misassembled;
-                        }
-                    }
+            } else if (utype == CUser_object::eObjectType_Unverified) {
+                if (uo.IsUnverifiedOrganism()) {
+                    m_fUnverified |= fUnverified_Organism;
+                }
+                if (uo.IsUnverifiedFeature()) {
+                    m_fUnverified |= fUnverified_SequenceOrAnnotation;
+                }
+                if (uo.IsUnverifiedMisassembled()) {
+                    m_fUnverified |= fUnverified_Misassembled;
+                }
+                if (uo.IsUnverifiedContaminant()) {
+                    m_fUnverified |= fUnverified_Contaminant;
                 }
                 // default in the past was to use feature
                 if( m_fUnverified == fUnverified_None ) {
                     m_fUnverified = fUnverified_SequenceOrAnnotation;
                 }
-            } else if( NStr::EqualNocase(uo.GetType().GetStr(), "FileTrack") ) {
+            } else if ( utype == CUser_object::eObjectType_FileTrack ) {
                 x_SetFiletrackURL(uo);
-            } else if( NStr::EqualNocase(uo.GetType().GetStr(), "AuthorizedAccess") ) {
+            } else if ( NStr::EqualNocase(uo.GetType().GetStr(), "AuthorizedAccess") ) {
                 x_SetAuthorizedAccess(uo);
-            } else if( NStr::EqualNocase(uo.GetType().GetStr(), "ENCODE") ) {
+            } else if ( NStr::EqualNocase(uo.GetType().GetStr(), "ENCODE") ) {
                 x_SetEncode(uo);
             }
         }
@@ -627,7 +685,10 @@ void CBioseqContext::x_CheckForShowComments() const
         return;
     }
 
-    CAnnot_CI annot_ci(m_Handle, SAnnotSelector(CSeq_annot::TData::e_Ftable));
+    // JIRA SQD-4444 : copy annot selector from the one saved in this context structure
+    SAnnotSelector sel = m_FFCtx.SetAnnotSelector();
+    sel.SetAnnotType(CSeq_annot::TData::e_Ftable);
+    CAnnot_CI annot_ci(m_Handle, sel);
     for( ; annot_ci; ++annot_ci ) {
         if( ! annot_ci->Seq_annot_IsSetDesc() ) {
             continue;
@@ -679,6 +740,21 @@ bool CBioseqContext::x_HasOperon(void) const
                     CSeqFeatData::eSubtype_operon);
 }
 
+bool CBioseqContext::HasOperon(void) const
+{
+    // check for indexed version first
+    if (UsingSeqEntryIndex()) {
+        CRef<CSeqEntryIndex> idx = GetSeqEntryIndex();
+        if (! idx) return false;
+        CRef<CBioseqIndex> bsx = idx->GetBioseqIndex (m_Handle);
+        if (! bsx) return false;
+        return bsx->HasOperon();
+    }
+
+    m_HasOperon = x_HasOperon();
+    return m_HasOperon;
+}
+
 
 void CBioseqContext::x_SetId(void)
 {
@@ -815,14 +891,12 @@ void CBioseqContext::x_SetId(void)
         } 
 
         // TSA
-        m_IsTSA = m_IsTSA  ||  (acc_div == CSeq_id::eAcc_tsa);
+        m_IsTSA = m_IsTSA  ||  ( GetTech() == CMolInfo::eTech_tsa );
         
         if ( m_IsTSA  &&  !acc.empty() ) {
-            size_t len = acc.length();
-            m_IsTSAMaster = 
-                ( ((len == 12  ||  len == 15)  &&  NStr::EndsWith(acc, "000000"))  ||
-                  (len == 13  &&  NStr::EndsWith(acc, "0000000")) ) &&
-                ( m_Repr == CSeq_inst::eRepr_virtual );
+            if ( m_Repr == CSeq_inst::eRepr_virtual) {
+                m_IsTSAMaster = true;
+            }
             if ( m_IsTSAMaster ) {
                 m_TSAMasterAccn = acc;
                 m_TSAMasterName = tsip->CanGetName() ? tsip->GetName() : kEmptyStr;
@@ -845,6 +919,15 @@ void CBioseqContext::x_SetId(void)
 
         // GBB source
         m_ShowGBBSource = m_ShowGBBSource  ||  (acc_info == CSeq_id::eAcc_gsdb_dirsub);
+
+        if (m_IsGenbank || m_IsEMBL || m_IsDDBJ) {
+            if (acc.length() == 6) {
+                char ch = acc[0];
+                if (ch == 'J' || ch == 'K' || ch == 'L' || ch == 'M') {
+                    m_ShowGBBSource = true;
+                }
+            }
+        }
     }
 
     // Genbank/Embl/Ddbj (GED)
