@@ -1,4 +1,4 @@
-/*  $Id: dbsource_item.cpp 545754 2017-09-11 11:57:08Z ivanov $
+/*  $Id: dbsource_item.cpp 568762 2018-08-09 21:34:53Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -259,7 +259,7 @@ void CDBSourceItem::x_AddPIRBlock(CBioseqContext& ctx)
 
     x_SetObject(*dsc);
 
-    bool containsHostLine = false; // another hack to try to match C's whitespace
+    bool containsHostLine = false; // try to match C's whitespace
 
     const CPIR_block& pir = dsc->GetPir();
     if (pir.CanGetHost()) {
@@ -320,7 +320,7 @@ void CDBSourceItem::x_AddPIRBlock(CBioseqContext& ctx)
         if( &*it == &m_DBSource.front() ) {
             // first one has newline AFTER the semicolon
             *it += ";\n";
-            // another hack to match C toolkit
+            // match C toolkit
             /* if( (it + 1) != m_DBSource.end() && ! NStr::StartsWith(*(it + 1), "host")  ) {
                 *it += ";\n";
             } */
@@ -334,7 +334,7 @@ void CDBSourceItem::x_AddPIRBlock(CBioseqContext& ctx)
         // *it += (&*it == &m_DBSource.back() ? "." : "\n;");
     }
 
-    // hack to match C's whitespace
+    // match C's whitespace
     if( ! containsHostLine ) {
         m_DBSource.front() += "\n";
     }
@@ -446,7 +446,7 @@ void CDBSourceItem::x_AddSPBlock(CBioseqContext& ctx)
                 if (ctx.Config().DoHTML()) {
                     xrefs.push_back
                         ("MIM <a href=\""
-                         "https://www.ncbi.nlm.nih.gov/entrez/dispomim.cgi?id=" + id
+                         "https://omim.org/entry/" + id
                          + "\">" + id + "</a>");
                 } else {
                     xrefs.push_back("MIM:" + id);
@@ -573,10 +573,14 @@ string CDBSourceItem::x_FormatDBSourceID(const CSeq_id_Handle& idh)
                 s += "molecule " + pdb.GetMol().Get();
                 sep = ", ";
             }
-            if (pdb.GetChain() > 0) {
+            if (pdb.IsSetChain() && pdb.GetChain() > 0) {
                 s += sep + "chain " + NStr::IntToString(pdb.GetChain());
                 sep = ", ";
             }
+            if (pdb.IsSetChain_id()) {
+                s += sep + "chain " + pdb.GetChain_id();
+                sep = ", ";
+            }
             if (pdb.CanGetRel()) {
                 s += sep + "release ";
                 s_FormatDate(pdb.GetRel(), s);
@@ -591,11 +595,12 @@ string CDBSourceItem::x_FormatDBSourceID(const CSeq_id_Handle& idh)
                 return kEmptyStr;
             }
             string s, sep, comma, ht;
+            bool is_uniprot = false;
             switch (choice) {
             case CSeq_id::e_Embl:       s = "embl ";        comma = ",";  break;
             case CSeq_id::e_Other:      s = "REFSEQ: ";                   break;
-            case CSeq_id::e_Swissprot:  s = "UniProtKB: ";  comma = ",";  break;
-            case CSeq_id::e_Pir:        s = "UniProtKB: ";                break;
+            case CSeq_id::e_Swissprot:  s = "UniProtKB: "; is_uniprot = true; comma = ",";  break;
+            case CSeq_id::e_Pir:        s = "UniProtKB: "; is_uniprot = true;               break;
             case CSeq_id::e_Prf:        s = "prf: ";                      break;
             default:                    break;
             }
@@ -612,7 +617,16 @@ string CDBSourceItem::x_FormatDBSourceID(const CSeq_id_Handle& idh)
                     acc += '.' + NStr::IntToString(tsid->GetVersion());
                 }
 #ifdef NEW_HTML_FMT
+#if 0
                 GetContext()->Config().GetHTMLFormatter().FormatNucId(ht, *idh.GetSeqId(), GetContext()->GetScope().GetGi(idh), acc);
+#else
+                if (is_uniprot) {
+                    GetContext()->Config().GetHTMLFormatter().FormatUniProtId(ht, acc);
+                } else {
+                    GetContext()->Config().GetHTMLFormatter().FormatNucId(ht, *idh.GetSeqId(),
+                        GetContext()->GetScope().GetGi(idh), acc);
+                }
+#endif
                 s += comma + sep + "accession " + ht;
 #else
                 if (is_html) {
