@@ -1,4 +1,4 @@
-/*  $Id: flat_file_config.cpp 544802 2017-08-28 14:35:41Z ivanov $
+/*  $Id: flat_file_config.cpp 574582 2018-11-15 15:10:09Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -296,32 +296,6 @@ CFlatFileConfig::CGenbankBlockCallback::notify(
     return unified_notify(block_text, ctx,  slash_item, fGenbankBlocks_Slash);
 }
 
-// -- constructor
-CFlatFileConfig::CFlatFileConfig(
-    TFormat format,
-    TMode mode,
-    TStyle style,
-    TFlags flags,
-    TView view,
-    TGffOptions gff_options,
-    TGenbankBlocks genbank_blocks,
-    CGenbankBlockCallback* pGenbankBlockCallback,
-    const ICanceled * pCanceledCallback,
-    bool basicCleanup, TCustom custom ) :
-    m_Format(format), m_Mode(mode), m_Style(style), 
-    m_Flags(flags), m_View(view),
-    m_RefSeqConventions(false), m_GffOptions(gff_options),
-    m_fGenbankBlocks(genbank_blocks),
-    m_GenbankBlockCallback(pGenbankBlockCallback),
-    m_pCanceledCallback(pCanceledCallback),
-    m_BasicCleanup(basicCleanup), m_Custom(custom)
-{
-    // FTable always requires master style
-    if (m_Format == eFormat_FTable) {
-        m_Style = eStyle_Master;
-    }
-    m_html_formatter.Reset(new CHTMLEmptyFormatter);
-}
 
 
 CFlatFileConfig::CFlatFileConfig(
@@ -365,8 +339,9 @@ const bool CFlatFileConfig::sm_ModeFlags[4][32] = {
         false, false
     },
     // Entrez
+    // ID-4625 : do not drop illegal qualifiers
     {
-        false, true, true, true, true, false, true, true, true, true,
+        false, true, true, true, true, false, true, true, false, true,
         true, true, true, true, true, true, true, false, true, true,
         true, true, true, true, false, true, true, true, false, true, 
         false, false
@@ -528,7 +503,7 @@ void CFlatFileConfig::AddArgumentDescriptions(CArgDescriptions& args)
                                  CArgDescriptions::eString, "genbank");
          arg_desc->SetConstraint("format",
                                  &(*new CArgAllow_Strings,
-                                   "genbank", "embl", "ddbj", "gbseq", "ftable", "gff", "gff3", "lite"));
+                                   "genbank", "embl", "ddbj", "gbseq", "insdseq", "ftable", "gff", "gff3", "lite"));
 
          // mode (default: dump)
          arg_desc->AddDefaultKey("mode", "Mode",
@@ -586,7 +561,9 @@ void CFlatFileConfig::AddArgumentDescriptions(CArgDescriptions& args)
                                  "Custom flat file output bits.  The value is the bitwise OR (logical addition) of:\n"
                                  "         1 - hide protein_id and transcript_id\n"
                                  "         2 - hide GI number\n"
-                                 "         4 - allow long locus lines",
+                                 "         4 - allow long locus lines"
+                                 "      1024 - disable annot references"
+                                 "      2048 - use SeqEntryIndexer",
 
                                  CArgDescriptions::eInteger, "0");
 
@@ -610,6 +587,9 @@ void CFlatFileConfig::AddArgumentDescriptions(CArgDescriptions& args)
          arg_desc->AddFlag("no-external",
                            "Disable all external annotation sources");
 
+         arg_desc->AddFlag("enable-external",
+                           "Enable loading of external annotation sources for local file");
+
          arg_desc->AddFlag("resolve-all",
                            "Resolves all, e.g. for contigs.");
 
@@ -663,6 +643,9 @@ void CFlatFileConfig::AddArgumentDescriptions(CArgDescriptions& args)
          // repetition
          arg_desc->AddDefaultKey("count", "Count", "Number of runs",
                                  CArgDescriptions::eInteger, "1");
+
+         // test faster looping
+         arg_desc->AddFlag("faster", "Test faster internal looping");
      }}
 }
 
@@ -679,6 +662,8 @@ CFlatFileConfig::EFormat x_GetFormat(const CArgs& args)
         return CFlatFileConfig::eFormat_DDBJ;
     } else if ( format == "gbseq" ) {
         return CFlatFileConfig::eFormat_GBSeq;
+    } else if ( format == "insdseq" ) {
+        return CFlatFileConfig::eFormat_INSDSeq;
     } else if ( format == "ftable" ) {
         return CFlatFileConfig::eFormat_FTable;
     } else if ( format == "lite" ) {
@@ -864,7 +849,7 @@ CFlatFileConfig::TGenbankBlocks x_GetGenbankBlocks(const CArgs& args)
     // turn the blocks into one mask
     CFlatFileConfig::TGenbankBlocks fBlocksGiven = 0;
     vector<string> vecOfBlockNames;
-    NStr::Split(blocks_arg, ",", vecOfBlockNames, NStr::fSplit_NoMergeDelims);
+    NStr::Split(blocks_arg, ",", vecOfBlockNames);
     ITERATE(vector<string>, name_iter, vecOfBlockNames) {
         // Note that StringToGenbankBlock throws an
         // exception if it gets an illegal value.
@@ -904,6 +889,11 @@ void CHTMLEmptyFormatter::FormatProteinId(string& str, const CSeq_id& seq_id, co
     str = prot_id;
 }
 
+void CHTMLEmptyFormatter::FormatTranscriptId(string& str, const CSeq_id& seq_id, const string& nuc_id) const
+{
+    str = nuc_id;
+}
+
 void CHTMLEmptyFormatter::FormatNucId(string& str, const CSeq_id& seq_id, TIntId gi, const string& acc_id) const
 {
     str = acc_id;
@@ -938,6 +928,16 @@ void CHTMLEmptyFormatter::FormatGeneralId(CNcbiOstream& os, const string& id) co
 {
     os << id;
 }
+
+void CHTMLEmptyFormatter::FormatGapLink(CNcbiOstream& os, TSeqPos gap_size, const string& id, bool is_prot) const
+{
+    os << "          [gap " << gap_size << " " << (is_prot ? "aa" : "bp" ) << "]";
+}
+
+void CHTMLEmptyFormatter::FormatUniProtId(string& str, const string& prot_id) const
+{
+    str = prot_id;
+}
 #endif
 
 END_SCOPE(objects)
