@@ -1,4 +1,4 @@
-/*  $Id: flat_file_generator.cpp 532240 2017-04-03 15:25:21Z foleyjp $
+/*  $Id: flat_file_generator.cpp 573607 2018-10-30 11:57:34Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -71,6 +71,7 @@ USING_SCOPE(sequence);
 CFlatFileGenerator::CFlatFileGenerator(const CFlatFileConfig& cfg) :
     m_Ctx(new CFlatFileContext(cfg))
 {
+    m_Failed = false;
      if ( !m_Ctx ) {
          NCBI_THROW(CFlatException, eInternal, "Unable to initialize context");
      }
@@ -86,6 +87,7 @@ CFlatFileGenerator::CFlatFileGenerator
  CFlatFileConfig::TCustom custom) :
     m_Ctx(new CFlatFileContext(CFlatFileConfig(format, mode, style, flags, view)))
 {
+    m_Failed = false;
     if ( !m_Ctx ) {
        NCBI_THROW(CFlatException, eInternal, "Unable to initialize context");
     }
@@ -112,6 +114,16 @@ void CFlatFileGenerator::SetFeatTree(feature::CFeatTree* tree)
     m_Ctx->SetFeatTree(tree);
 }
 
+void CFlatFileGenerator::SetSeqEntryIndex(CRef<CSeqEntryIndex> idx)
+{
+    m_Ctx->SetSeqEntryIndex(idx);
+}
+
+void CFlatFileGenerator::ResetSeqEntryIndex(void)
+{
+    m_Ctx->ResetSeqEntryIndex();
+}
+
 
 
     /*
@@ -253,9 +265,55 @@ void CFlatFileGenerator::Generate
     SAnnotSelector sel = m_Ctx->SetAnnotSelector();
     m_Ctx->SetEntry(entry);
 
+    if ( m_Ctx->GetConfig().UseSeqEntryIndexer() ) {
+        // CSeq_entry& top = const_cast<CSeq_entry&> (*topent);
+        CSeq_entry_Handle topseh = entry.GetTopLevelEntry();
+        if (m_Ctx->UsingSeqEntryIndex()) {
+            const CRef<CSeqEntryIndex> idx = m_Ctx->GetSeqEntryIndex();
+            if (idx) {
+                const CRef<CSeqMasterIndex>& midx = idx->GetMasterIndex();
+                if (midx) {
+                    if (midx->GetTopSEH() != topseh) {
+                        m_Ctx->ResetSeqEntryIndex();
+                    }
+                }
+            }
+        }
+        if (! m_Ctx->UsingSeqEntryIndex()) {
+            try {
+                CSeqEntryIndex::EPolicy policy = CSeqEntryIndex::eAdaptive;
+                CSeqEntryIndex::TFlags flags = CSeqEntryIndex::fDefault;
+                if ( m_Ctx->GetConfig().OnlyNearFeatures() ) {
+                    policy = CSeqEntryIndex::eInternal;
+                }
+                if ( m_Ctx->GetConfig().HideSNPFeatures() ) {
+                    flags |= CSeqEntryIndex::fHideSNPFeats;
+                }
+                if ( m_Ctx->GetConfig().HideCDDFeatures() ) {
+                    flags |= CSeqEntryIndex::fHideCDDFeats;
+                }
+                if ( m_Ctx->GetConfig().ShowSNPFeatures() ) {
+                    flags |= CSeqEntryIndex::fShowSNPFeats;
+                }
+                if ( m_Ctx->GetConfig().ShowCDDFeatures() ) {
+                    flags |= CSeqEntryIndex::fShowCDDFeats;
+                }
+                CRef<CSeqEntryIndex> idx(new CSeqEntryIndex( topseh, policy, flags ));
+                m_Ctx->SetSeqEntryIndex(idx);
+                if (idx->IsIndexFailure()) {
+                    m_Failed = true;
+                    return;
+                }
+            } catch(CException &) {
+                m_Failed = true;
+                return;
+            }
+        }
+    }
+
 
     bool onlyNearFeats = false;
-    bool nearFeatsSuppress = false;
+    // bool nearFeatsSuppress = false;
 
     bool isNc = false;
     bool isNgNtNwNz = false;
@@ -405,11 +463,17 @@ void CFlatFileGenerator::Generate
     /// restore the selector to its former glory
     m_Ctx->Reset();
     m_Ctx->SetAnnotSelector() = sel;
+
+    /*
+    if ( m_Ctx->GetConfig().UseSeqEntryIndexer() ) {
+        m_Ctx->ResetSeqEntryIndex();
+    }
+    */
 }
 
 
 void CFlatFileGenerator::Generate
-(CSeq_submit& submit,
+(const CSeq_submit& submit,
  CScope& scope,
  CFlatItemOStream& item_os)
 {
@@ -421,7 +485,7 @@ void CFlatFileGenerator::Generate
     // NB: though the spec specifies a submission may contain multiple entries
     // this is not the case. A submission should only have a single Top-level
     // Seq-entry
-    CRef<CSeq_entry> e(submit.SetData().SetEntrys().front());
+    CConstRef<CSeq_entry> e(submit.GetData().GetEntrys().front());
     if (e.NotEmpty()) {
         // get Seq_entry_Handle from scope
         CSeq_entry_Handle entry;
@@ -508,8 +572,165 @@ void CFlatFileGenerator::Generate
 }
 
 
+// This version iterates Bioseqs within the Bioseq_set
 void CFlatFileGenerator::Generate
-(CSeq_submit& submit,
+(const CSeq_entry_Handle& entry,
+ CFlatItemOStream& item_os,
+ bool useSeqEntryIndexing)
+{
+    if (useSeqEntryIndexing) {
+        m_Ctx->SetConfig().SetUseSeqEntryIndexer(true);
+    } else {
+        SetFeatTree(new feature::CFeatTree(entry));
+    }
+
+    _ASSERT(entry  &&  entry.Which() != CSeq_entry::e_not_set);
+
+    if ( m_Ctx->GetConfig().BasicCleanup() )
+    {
+
+        entry.GetTopLevelEntry().GetCompleteObject();
+        CSeq_entry_EditHandle tseh = entry.GetTopLevelEntry().GetEditHandle();
+        CBioseq_set_EditHandle bseth;
+        CBioseq_EditHandle bseqh;
+        CRef<CSeq_entry> tmp_se(new CSeq_entry);
+
+        if ( tseh.IsSet() ) {
+            bseth = tseh.SetSet();
+            CConstRef<CBioseq_set> bset = bseth.GetCompleteObject();
+            bseth.Remove(bseth.eKeepSeq_entry);
+            tmp_se->SetSet(const_cast<CBioseq_set&>(*bset));
+        }
+        else {
+            bseqh = tseh.SetSeq();
+            CConstRef<CBioseq> bseq = bseqh.GetCompleteObject();
+            bseqh.Remove(bseqh.eKeepSeq_entry);
+            tmp_se->SetSeq(const_cast<CBioseq&>(*bseq));
+        }
+
+        CCleanup cleanup;
+        cleanup.BasicCleanup( *tmp_se );
+
+        if ( tmp_se->IsSet() ) {
+            tseh.SelectSet(bseth);
+        }
+        else {
+            tseh.SelectSeq(bseqh);
+        }
+    }
+
+    m_Ctx->SetSGS(false);
+    CConstRef<CSeq_entry> topent = entry.GetTopLevelEntry().GetCompleteSeq_entry();
+    if (topent && topent->IsSet()) {
+        /*
+        const CBioseq_set& topset = topent->GetSet();
+        VISIT_ALL_SEQSETS_WITHIN_SEQSET (itr, topset) {
+            const CBioseq_set& bss = *itr;
+            if (bss.GetClass() == CBioseq_set::eClass_small_genome_set) {
+                    m_Ctx->SetSGS(true);
+            }
+        }
+        */
+        VisitAllSeqSets(*topent, [this](const CBioseq_set& bss){
+            if (bss.GetClass() == CBioseq_set::eClass_small_genome_set) {
+                m_Ctx->SetSGS(true);
+            }});
+    }
+
+    CRef<CFlatItemOStream> pItemOS( & item_os );
+    // If there is a ICancel callback, wrap the item_os so
+    // that every call checks it.
+    const ICanceled * pCanceled = 
+        m_Ctx->GetConfig().GetCanceledCallback();
+    if( pCanceled ) {
+        pItemOS.Reset( 
+            new CCancelableFlatItemOStreamWrapper(
+            item_os, pCanceled) );
+    }
+
+    /// archive a copy of the annot selector before we generate!
+    SAnnotSelector sel = m_Ctx->SetAnnotSelector();
+    m_Ctx->SetEntry(entry);
+
+    if ( m_Ctx->GetConfig().UseSeqEntryIndexer() ) {
+        // CSeq_entry& top = const_cast<CSeq_entry&> (*topent);
+        CSeq_entry_Handle topseh = entry.GetTopLevelEntry();
+        if (m_Ctx->UsingSeqEntryIndex()) {
+            const CRef<CSeqEntryIndex> idx = m_Ctx->GetSeqEntryIndex();
+            if (idx) {
+                const CRef<CSeqMasterIndex>& midx = idx->GetMasterIndex();
+                if (midx) {
+                    if (midx->GetTopSEH() != topseh) {
+                        m_Ctx->ResetSeqEntryIndex();
+                    }
+                }
+            }
+        }
+        if (! m_Ctx->UsingSeqEntryIndex()) {
+            try {
+                CSeqEntryIndex::EPolicy policy = CSeqEntryIndex::eAdaptive;
+                CSeqEntryIndex::TFlags flags = CSeqEntryIndex::fDefault;
+                if ( m_Ctx->GetConfig().OnlyNearFeatures() ) {
+                    policy = CSeqEntryIndex::eInternal;
+                }
+                if ( m_Ctx->GetConfig().HideSNPFeatures() ) {
+                    flags |= CSeqEntryIndex::fHideSNPFeats;
+                }
+                if ( m_Ctx->GetConfig().HideCDDFeatures() ) {
+                    flags |= CSeqEntryIndex::fHideCDDFeats;
+                }
+                if ( m_Ctx->GetConfig().ShowSNPFeatures() ) {
+                    flags |= CSeqEntryIndex::fShowSNPFeats;
+                }
+                if ( m_Ctx->GetConfig().ShowCDDFeatures() ) {
+                    flags |= CSeqEntryIndex::fShowCDDFeats;
+                }
+                CRef<CSeqEntryIndex> idx(new CSeqEntryIndex( topseh, policy, flags ));
+                m_Ctx->SetSeqEntryIndex(idx);
+                if (idx->IsIndexFailure()) {
+                    m_Failed = true;
+                    return;
+                }
+            } catch(CException &) {
+                m_Failed = true;
+                return;
+            }
+        }
+    }
+
+    CFlatFileConfig::TFormat format = m_Ctx->GetConfig().GetFormat();
+    CRef<CFlatItemFormatter> formatter(CFlatItemFormatter::New(format));
+    if ( !formatter ) {
+        NCBI_THROW(CFlatException, eInternal, "Unable to initialize formatter");
+    }
+    formatter->SetContext(*m_Ctx);
+    pItemOS->SetFormatter(formatter);
+
+    CRef<CFlatGatherer> gatherer(CFlatGatherer::New(format));
+    if ( !gatherer ) {
+        NCBI_THROW(CFlatException, eInternal, "Unable to initialize gatherer");
+    }
+
+    // this version of Gather calls method with internal Bioseq iterator
+    gatherer->Gather(*m_Ctx, *pItemOS, entry, useSeqEntryIndexing);
+
+    /// reset the context, but preserve our selector
+    /// we do this a bit oddly since resetting the context erases the selector;
+    /// since the caller is reusing this object (most likely), we automatically
+    /// restore the selector to its former glory
+    m_Ctx->Reset();
+    m_Ctx->SetAnnotSelector() = sel;
+
+    /*
+    if ( m_Ctx->GetConfig().UseSeqEntryIndexer() ) {
+        m_Ctx->ResetSeqEntryIndex();
+    }
+    */
+}
+
+
+void CFlatFileGenerator::Generate
+(const CSeq_submit& submit,
  CScope& scope,
  CNcbiOstream& os)
 {
@@ -583,6 +804,18 @@ void CFlatFileGenerator::Generate
 }
 
 
+void CFlatFileGenerator::Generate
+(const CSeq_entry_Handle& entry,
+ CNcbiOstream& os,
+ bool useSeqEntryIndexing)
+{
+    CRef<CFlatItemOStream> 
+        item_os(new CFormatItemOStream(new COStreamTextOStream(os)));
+
+    Generate(entry, *item_os, useSeqEntryIndexing);
+}
+
+
 //void CFlatFileGenerator::Reset(void)
 //{
 //    m_Ctx->Reset();
@@ -629,6 +862,94 @@ string CFlatFileGenerator::GetSeqFeatText
     return text;
 }
 
+void CFlatFileGenerator::x_GetLocation
+(const CSeq_entry_Handle& entry,
+ TSeqPos from,
+ TSeqPos to,
+ ENa_strand strand,
+ CSeq_loc& loc)
+{
+    _ASSERT(entry);
+
+    CBioseq_Handle h = x_DeduceTarget(entry);
+    if ( !h ) {
+        NCBI_THROW(CFlatException, eInvalidParam,
+            "Cannot deduce target bioseq.");
+    }
+    TSeqPos length = h.GetInst_Length();
+
+    if ( from == CRange<TSeqPos>::GetWholeFrom()  &&  to == length ) {
+        // whole
+        loc.SetWhole().Assign(*h.GetSeqId());
+    } else {
+        // interval
+        loc.SetInt().SetId().Assign(*h.GetSeqId());
+        loc.SetInt().SetFrom(from);
+        loc.SetInt().SetTo(to);
+        if ( strand > 0 ) {
+            loc.SetInt().SetStrand(strand);
+        }
+    }
+}
+
+// if the 'from' or 'to' flags are specified try to guess the bioseq.
+CBioseq_Handle CFlatFileGenerator::x_DeduceTarget(const CSeq_entry_Handle& entry)
+{
+    if ( entry.IsSeq() ) {
+        return entry.GetSeq();
+    }
+
+    _ASSERT(entry.IsSet());
+    CBioseq_set_Handle bsst = entry.GetSet();
+    if ( !bsst.IsSetClass() ) {
+        NCBI_THROW(CFlatException, eInvalidParam,
+            "Cannot deduce target bioseq.");
+    }
+    _ASSERT(bsst.IsSetClass());
+    switch ( bsst.GetClass() ) {
+    case CBioseq_set::eClass_nuc_prot:
+        // return the nucleotide
+        for ( CSeq_entry_CI it(entry); it; ++it ) {
+            if ( it->IsSeq() ) {
+                CBioseq_Handle h = it->GetSeq();
+                if ( h  &&  CSeq_inst::IsNa(h.GetInst_Mol()) ) {
+                    return h;
+                }
+            }
+        }
+        break;
+    case CBioseq_set::eClass_gen_prod_set:
+        // return the genomic
+        for ( CSeq_entry_CI it(bsst); it; ++it ) {
+            if ( it->IsSeq()  &&
+                 it->GetSeq().GetInst_Mol() == CSeq_inst::eMol_dna ) {
+                 return it->GetSeq();
+            }
+        }
+        break;
+    case CBioseq_set::eClass_segset:
+        // return the segmented bioseq
+        for ( CSeq_entry_CI it(bsst); it; ++it ) {
+            if ( it->IsSeq() ) {
+                return it->GetSeq();
+            }
+        }
+        break;
+    case CBioseq_set::eClass_genbank:
+        {
+            CBioseq_CI bi(bsst, CSeq_inst::eMol_na);
+            if (bi) {
+                return *bi;
+            }
+        }
+        break;
+    default:
+        break;
+    }
+    NCBI_THROW(CFlatException, eInvalidParam,
+            "Cannot deduce target bioseq.");
+}
+
 void 
 CFlatFileGenerator::CCancelableFlatItemOStreamWrapper::SetFormatter(
     IFormatter* formatter)
