@@ -1,4 +1,4 @@
-/*  $Id: qualifiers.cpp 544492 2017-08-23 17:44:32Z ivanov $
+/*  $Id: qualifiers.cpp 564513 2018-05-29 17:40:10Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -862,6 +862,49 @@ void CFlatMolTypeQVal::Format(TFlatQuals& q, const CTempString& name,
     }
 }
 
+void CFlatSubmitterSeqidQVal::Format(TFlatQuals& q, const CTempString& name,
+                            CBioseqContext& ctx, IFlatQVal::TFlags flags) const
+{
+    if ( ctx.Config().IsModeGBench()  ||  ctx.Config().IsModeDump() ) {
+        switch ( m_Tech ) {
+        case CMolInfo::eTech_wgs:
+        case CMolInfo::eTech_tsa:
+        case CMolInfo::eTech_targeted:
+            ITERATE (CBioseq::TId, itr, ctx.GetBioseqIds()) {
+                const CSeq_id& id = **itr;
+                if ( id.Which() != CSeq_id::e_General ) continue;
+                const CDbtag& dbtag = id.GetGeneral();
+                if ( ! dbtag.IsSetDb() ) continue;
+                string dbname = dbtag.GetDb();
+                if ( dbname.length() != 10 ) continue;
+                if ( ! NStr::StartsWith(dbname, "WGS:" ) && ! NStr::StartsWith(dbname, "TSA:" ) && ! NStr::StartsWith(dbname, "TLS:" ) ) continue;
+                bool bail = false;
+                for ( int i = 4; i < 8; i++ ) {
+                    char ch = dbname[i];
+                    if ( ! isupper(ch) && ! islower(ch) ) {
+                        bail = true;
+                    }
+                }
+                if ( bail ) continue;
+                for ( int i = 8; i < 10; i++ ) {
+                    char ch = dbname[i];
+                    if ( ! isdigit(ch) ) {
+                        bail = true;
+                    }
+                }
+                if ( bail ) continue;
+                if ( dbtag.IsSetTag() && dbtag.GetTag().IsStr() ) {
+                    string tag = dbtag.GetTag().GetStr();
+                    x_AddFQ(q, name, tag);
+                }
+            }
+            break;
+        default:
+            break;
+        }
+    }
+}
+
 void CFlatOrgModQVal::Format(TFlatQuals& q, const CTempString& name,
                            CBioseqContext& ctx, IFlatQVal::TFlags flags) const
 {
@@ -979,15 +1022,26 @@ void CFlatPubSetQVal::Format(TFlatQuals& q, const CTempString& name,
             if( (*ref_iter)->Matches( **pub_iter ) ) {
                 // We have a match, so create the qual
                 string value;
-                if( (*ref_iter)->GetPMID() > 0 && bHtml ) {
+                string pub_id_str;
+                int serial = (*ref_iter)->GetSerial();
+                int pmid = (*ref_iter)->GetPMID();
+                if (serial) {
+                    pub_id_str = NStr::IntToString(serial);
+                } else if (pmid) {
+                    pub_id_str = NStr::IntToString(pmid);
+                }
+                /*
+                string pub_id_str =
+                    ((*ref_iter)->GetPMID() ? NStr::IntToString((*ref_iter)->GetPMID()) :
+                     NStr::IntToString((*ref_iter)->GetSerial()));
+                */
+
+                if(bHtml && pmid) {
                     // create a link
-                    const int pmid = (*ref_iter)->GetPMID();
                     value  = "[<a href=\"";
-                    value += strLinkBasePubmed + NStr::IntToString(pmid) + "\">" + 
-                        NStr::IntToString((*ref_iter)->GetSerial()) + 
-                        "</a>]";
+                    value += strLinkBasePubmed + NStr::IntToString(pmid) + "\">" + pub_id_str + "</a>]";
                 } else {
-                    value = '[' + NStr::IntToString((*ref_iter)->GetSerial()) + ']';
+                    value = '[' + pub_id_str + ']';
                 }
                 x_AddFQ(q, name, value, CFormatQual::eUnquoted);
                 
@@ -1005,15 +1059,15 @@ void CFlatPubSetQVal::Format(TFlatQuals& q, const CTempString& name,
         for (; pub_iter != unusedPubs.end(); ++pub_iter) {
             if ((*pub_iter)->IsPmid()) {
                 const int pmid = (*pub_iter)->GetPmid().Get();
-
+                string pmid_str = NStr::NumericToString(pmid);
                 pubmed = "[PUBMED ";
                 if (bHtml) {
                     pubmed += "<a href=\"";
                     pubmed += strLinkBasePubmed;
-                    pubmed += NStr::NumericToString(pmid);
+                    pubmed += pmid_str;
                     pubmed += "\">";
                 }
-                pubmed += pmid;
+                pubmed += pmid_str;
                 if (bHtml) {
                     pubmed += "</a>";
                 }
@@ -1064,6 +1118,9 @@ void CFlatSeqIdQVal::Format(TFlatQuals& q, const CTempString& name,
     if (name == "protein_id") {
        ctx.Config().GetHTMLFormatter().FormatProteinId(id_str, *m_Value, string(id_str));
     }
+    if (name == "transcript_id") {
+       ctx.Config().GetHTMLFormatter().FormatTranscriptId(id_str, *m_Value, string(id_str));
+    }
 #else
     if (bHtml && name == "protein_id") {
         string raw_id_str = id_str;
@@ -1708,7 +1765,11 @@ void CFlatAnticodonQVal::Format
         return;
     }
 
-    string locationString = CFlatSeqLoc(*m_Anticodon, ctx).GetString();
+    const CSeq_loc& slp = *m_Anticodon;
+    CRef<CSeq_loc> loc2(new CSeq_loc);
+    loc2->Assign(slp);
+    loc2->SetId(*ctx.GetPrimaryId());
+    string locationString = CFlatSeqLoc(*loc2, ctx).GetString();
 
     string text;
     text = "(pos:";
