@@ -1,4 +1,4 @@
-/*  $Id: reference_item.cpp 529217 2017-03-01 22:06:32Z kans $
+/*  $Id: reference_item.cpp 557236 2018-02-12 22:36:27Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -707,7 +707,7 @@ void CReferenceItem::x_GatherInfo(CBioseqContext& ctx)
     }
 
     // gather Genbank specific fields (formats: Genbank, GBSeq, DDBJ)
-    if ( ctx.IsGenbankFormat() ) {
+    if ( ctx.IsGenbankFormat() || ctx.Config().IsFormatGBSeq() || ctx.Config().IsFormatINSDSeq() ) {
         x_GatherRemark(ctx);
     }
 
@@ -923,8 +923,8 @@ static bool PiiOk(const string& str)
 
     char ch = str[i];
 
-    if (ch != 'e' && ch != 'E') return false;
-    if (ch == 'e' || ch == 'E') {
+    // if (! isupper(ch) && ! islower(ch)) return false;
+    if (isupper(ch) || islower(ch)) {
         i++;
         ch = str[i];
     }
@@ -935,12 +935,12 @@ static bool PiiOk(const string& str)
     }
     if (i >= max) return true;
 
-    if (ch == 'e' || ch == 'E') return false;
+    if (isupper(ch) || islower(ch)) return false;
     if (ch == '-') {
         i++;
         ch = str[i];
     }
-    if (ch == 'e' || ch == 'E') {
+    if (isupper(ch) || islower(ch)) {
         i++;
         ch = str[i];
     }
@@ -1020,6 +1020,24 @@ void CReferenceItem::x_Init(const CCit_art& art, CBioseqContext& ctx)
                     m_MUID = (*it)->GetMedline();
                 }
                 break;
+            case CArticleId::e_Doi:
+                {
+                    const string& doi = (*it)->GetDoi();
+                    if (! doi.empty()) {
+                        m_DOI = doi;
+                    }
+                }
+                break;
+            case CArticleId::e_Pii:
+                {
+                    if (not_in_press && is_epublish) {
+                        const string& pii = (*it)->GetPii();
+                        if (PiiOk (pii)) {
+                            m_OldPII = pii;
+                        }
+                    }
+                }
+                break;
             case CArticleId_Base::e_Other:
                 {
                     const CDbtag& dbt = (*it)->GetOther();
@@ -1511,13 +1529,14 @@ void CReferenceItem::x_GatherRemark(CBioseqContext& ctx)
         }
     }
 
-    // for GBSeq format collect remarks only from comments.
-    if ( ctx.Config().IsFormatGBSeq() ) {
+    /*
+    if ( ctx.Config().IsFormatGBSeq() || ctx.Config().IsFormatINSDSeq() ) {
         if ( !l.empty() ) {
             m_Remark = l.front();
         }
         return;
     }
+    */
 
     // GIBBSQ
     CSeq_id::TGibbsq gibbsq = 0;
@@ -1613,42 +1632,44 @@ void CReferenceItem::x_GatherRemark(CBioseqContext& ctx)
                     }
                 }
 
-                if( (*it)->GetArticle().CanGetIds() ) {
-                    const CCit_art_Base::TIds & ids = (*it)->GetArticle().GetIds();
-                    if( ids.CanGet() ) {
+                if ( ! ctx.Config().IsFormatGBSeq() && ! ctx.Config().IsFormatINSDSeq() ) {
+                    if( (*it)->GetArticle().CanGetIds() ) {
+                        const CCit_art_Base::TIds & ids = (*it)->GetArticle().GetIds();
+                        if( ids.CanGet() ) {
 
-                        // no DOIs pritned if there's a pmid or muid
-                        bool hasPmidOrMuid = false;
-                        ITERATE( CArticleIdSet_Base::Tdata, it, ids.Get() ) {
-                            if( (*it)->IsPubmed() && (*it)->GetPubmed().Get() != 0 ) {
-                                hasPmidOrMuid = true;
-                                break;
-                            } else if(  (*it)->IsMedline() && (*it)->GetMedline().Get() != 0 ) {
-                                hasPmidOrMuid = true;
-                                break;
-                            }
-                        }
-
-                        if( ! hasPmidOrMuid ) {
+                            // no DOIs pritned if there's a pmid or muid
+                            bool hasPmidOrMuid = false;
                             ITERATE( CArticleIdSet_Base::Tdata, it, ids.Get() ) {
-                                if( (*it)->Which() == CArticleId_Base::e_Doi) {
-                                    const string & doi = (*it)->GetDoi().Get();
-                                    if( NStr::StartsWith( doi, "10." ) ) {
-                                        if( ctx.Config().DoHTML() && ! CommentHasSuspiciousHtml(doi) ) {
-                                            CNcbiOstrstream result;
-                                            result << "DOI: <a href=\""
-                                                   << kDoiLink << doi << "\">"
-                                                   << doi << "</a>";
-                                            l.push_back( CNcbiOstrstreamToString(result) );
-                                        } else {
-                                            l.push_back( "DOI: " + doi );
+                                if( (*it)->IsPubmed() && (*it)->GetPubmed().Get() != 0 ) {
+                                    hasPmidOrMuid = true;
+                                    break;
+                                } else if(  (*it)->IsMedline() && (*it)->GetMedline().Get() != 0 ) {
+                                    hasPmidOrMuid = true;
+                                    break;
+                                }
+                            }
+
+                            if( ! hasPmidOrMuid ) {
+                                ITERATE( CArticleIdSet_Base::Tdata, it, ids.Get() ) {
+                                    if( (*it)->Which() == CArticleId_Base::e_Doi) {
+                                        const string & doi = (*it)->GetDoi().Get();
+                                        if( NStr::StartsWith( doi, "10." ) ) {
+                                            if( ctx.Config().DoHTML() && ! CommentHasSuspiciousHtml(doi) ) {
+                                                CNcbiOstrstream result;
+                                                result << "DOI: <a href=\""
+                                                       << kDoiLink << doi << "\">"
+                                                       << doi << "</a>";
+                                                l.push_back( CNcbiOstrstreamToString(result) );
+                                            } else {
+                                                l.push_back( "DOI: " + doi );
+                                            }
+                                            break;
                                         }
-                                        break;
                                     }
                                 }
                             }
-                        }
 
+                        }
                     }
                 }
             } else if ( (*it)->IsSub() ) {
@@ -1661,7 +1682,11 @@ void CReferenceItem::x_GatherRemark(CBioseqContext& ctx)
     }
 
     if (!l.empty()) {
-        m_Remark = NStr::Join(l, "\n");
+        if ( ctx.Config().IsFormatGBSeq() || ctx.Config().IsFormatINSDSeq() ) {
+            m_Remark = NStr::Join(l, "; ");
+        } else {
+            m_Remark = NStr::Join(l, "\n");
+        }
     }
 }
 
