@@ -1,4 +1,4 @@
-/*  $Id: source_item.cpp 530087 2017-03-10 18:26:41Z kans $
+/*  $Id: source_item.cpp 563713 2018-05-14 16:53:23Z kans $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -139,6 +139,51 @@ static CConstRef<CSeq_feat> x_GetSourceFeatFromCDS  (
     return CConstRef<CSeq_feat> ();
 }
 
+static const char * legal_organelles[] = {
+    "chloroplast",
+    "chromoplast",
+    "kinetoplast",
+    "mitochondrion",
+    "plastid",
+    "cyanelle",
+    "nucleomorph",
+    "apicoplast",
+    "leucoplast",
+    "proplastid",
+    "hydrogenosome",
+    "chromatophore"
+};
+
+void CSourceItem::x_GatherInfoIdx(CBioseqContext& ctx)
+{
+    CRef<CSeqEntryIndex> idx = ctx.GetSeqEntryIndex();
+    if (! idx) return;
+    const CBioseq_Handle& bsh = ctx.GetHandle();
+    CRef<CBioseqIndex> bsx = idx->GetBioseqIndex (bsh);
+    if (! bsx) return;
+
+    m_Taxname = &bsx->GetTaxname();
+    m_Common = &bsx->GetCommon();
+    m_Taxid = bsx->GetTaxid();
+    m_UsingAnamorph = bsx->IsUsingAnamorph();
+
+    const string& lineage = bsx->GetLineage();
+    if (lineage.empty()) {
+        m_Lineage = scm_Unclassified;
+    } else {
+        m_Lineage = bsx->GetLineage();
+    }
+
+    const string* orgnlle = &bsx->GetOrganelle();
+    for (int i = 0; i < sizeof (legal_organelles) / sizeof (const char*); i++) {
+        CTempString str = legal_organelles [i];
+        if (NStr::CompareNocase (*orgnlle, str) == 0) {
+            m_Organelle = orgnlle;
+            return;
+        }
+    }
+}
+
 void CSourceItem::x_GatherInfo(CBioseqContext& ctx)
 {
     CConstRef<CSeq_feat>   cds_feat;
@@ -146,6 +191,13 @@ void CSourceItem::x_GatherInfo(CBioseqContext& ctx)
     CConstRef<CBioSource>  src_ref;
     CConstRef<CSeq_feat>   src_feat;
 
+    /*
+    if (ctx.UsingSeqEntryIndex()) {
+        x_GatherInfoIdx(ctx);
+        return;
+    }
+    */
+
     if (ctx.IsProt()) {
         const CBioseq_Handle& bsh = ctx.GetHandle();
         src_feat = x_GetSourceFeatFromCDS (bsh);
@@ -188,9 +240,15 @@ void CSourceItem::x_GatherInfo(CBioseqContext& ctx, const CBioSource& bsrc, cons
     CConstRef<CSeq_feat>   cds_feat;
     CConstRef<CSeq_loc>    cds_loc;
     CConstRef<CBioSource>  src_ref;
-
     CConstRef<CSeq_feat>   src_feat;
 
+    /*
+    if (ctx.UsingSeqEntryIndex()) {
+        x_GatherInfoIdx(ctx);
+        return;
+    }
+    */
+
     if (ctx.IsProt()) {
         const CBioseq_Handle& bsh = ctx.GetHandle();
         src_feat = x_GetSourceFeatFromCDS (bsh);
