@@ -1,4 +1,4 @@
-/*  $Id: agp_validate_reader.cpp 465301 2015-04-20 16:41:10Z sapojnik $
+/*  $Id: agp_validate_reader.cpp 529752 2017-03-07 17:57:23Z sapojnik $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -369,13 +369,17 @@ void CAgpValidateReader::OnGapOrComponent()
         if(m_this_row->component_type == 'W') m_AgpErr->Msg(CAgpErr::W_CompIsNotWgsTypeIs);
       }
       else if( div != CSeq_id::eAcc_other // no further classification
+          &&  div != CSeq_id::eAcc_dirsub // direct submission of anything, e.g. FO081906.4 (see GCOL-6194)
       ) {
-        if( string("ADF").find(m_this_row->component_type)!=NPOS ) m_AgpErr->Msg(CAgpErr::W_CompIsNotHtgTypeIs, m_this_row->GetComponentId());
+        if( string("ADF").find(m_this_row->component_type)!=NPOS )
+            m_AgpErr->Msg(CAgpErr::W_CompIsNotHtgTypeIs, m_this_row->GetComponentId());
       }
     }
     else {
-      if( string("ADF").find(m_this_row->component_type)!=NPOS ) m_AgpErr->Msg(CAgpErr::W_CompIsNotHtgTypeIs, m_this_row->GetComponentId());
-      else if(m_this_row->component_type!='W') m_AgpErr->Msg( CAgpErr::W_CompIsLocalTypeNotW, m_this_row->GetComponentId() );
+      if( string("ADF").find(m_this_row->component_type)!=NPOS )
+          m_AgpErr->Msg(CAgpErr::W_CompIsNotHtgTypeIs, m_this_row->GetComponentId());
+      else if(m_this_row->component_type!='W')
+          m_AgpErr->Msg( CAgpErr::W_CompIsLocalTypeNotW, m_this_row->GetComponentId() );
     }
 
     if( m_comp2len->size() ) {
@@ -460,6 +464,7 @@ void CAgpValidateReader::OnGapOrComponent()
   // m_this_row = current gap or component (check with m_this_row->IsGap())
 }
 
+// this function can use m_prev_row - but not m_this_row
 void CAgpValidateReader::OnScaffoldEnd()
 {
   NCBI_ASSERT(m_componentsInLastScaffold>0 || m_gapsInLastScaffold>0,
@@ -470,10 +475,10 @@ void CAgpValidateReader::OnScaffoldEnd()
     m_SingleCompScaffolds++;
     if(m_gapsInLastScaffold) m_SingleCompScaffolds_withGaps++;
 
-    if(m_unplaced && m_prev_orientation) {
+    if((m_unplaced || NStr::StartsWith(m_prev_row->GetObject(), "un", NStr::eNocase) ) && m_prev_orientation) {
       if(m_prev_orientation!='+') m_AgpErr->Msg( CAgpErrEx::W_UnSingleOriNotPlus   , CAgpErr::fAtPrevLine );
 
-      TMapStrInt::iterator it = m_comp2len->find( m_this_row->GetComponentId() );
+      TMapStrInt::iterator it = m_comp2len->find( m_prev_row->GetComponentId() );
       if( it!=m_comp2len->end() ) {
         int len = it->second;
         if(m_prev_component_beg!=1 || m_prev_component_end<len ) {
@@ -848,6 +853,10 @@ void CAgpValidateReader::x_PrintTotals(CNcbiOstream& out, bool use_xml) // witho
   e_count+=m_AgpErr->CountTotals(CAgpErrEx::G_Last);
   int w_count=m_AgpErr->CountTotals(CAgpErrEx::W_Last);
 
+  if(m_GapCount==0) {
+    if(m_AgpErr->m_strict) e_count++; else w_count++;
+  }
+
   if(not_in_agp_msg.size()) {
     if(m_AgpErr->m_strict) e_count++; else w_count++;
   }
@@ -905,6 +914,9 @@ void CAgpValidateReader::x_PrintTotals(CNcbiOstream& out, bool use_xml) // witho
   if(not_in_agp_msg.size() && !use_xml) {
     out << "      -   -   " << not_in_agp_msg << "\n";
   }
+  if(m_GapCount==0 && !use_xml) {
+    out << "      -   -   no gap lines\n";
+  }
   xprint.line();
 
   //// Prepare component/gap types and counts for later printing
