@@ -1,4 +1,4 @@
-/*  $Id: bed_reader.cpp 538707 2017-06-13 16:57:06Z ivanov $
+/*  $Id: bed_reader.cpp 572342 2018-10-11 17:14:40Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -90,6 +90,7 @@
 #include <objtools/error_codes.hpp>
 
 #include <algorithm>
+#include <deque>
 
 
 #define NCBI_USE_ERRCODE_X   Objtools_Rd_RepMask
@@ -97,6 +98,110 @@
 BEGIN_NCBI_SCOPE
 BEGIN_objects_SCOPE // namespace ncbi::objects::
 
+//  ============================================================================
+class CLinePreBuffer
+//  ============================================================================
+{
+public:
+    using LinePreIt = deque<string>::const_iterator;
+
+    CLinePreBuffer(
+        ILineReader& lineReader):
+        mLineReader(lineReader),
+        mLineNumber(0)
+    {};
+
+    virtual ~CLinePreBuffer() {};
+
+    bool FillBuffer(
+        size_t numLines)
+    {
+        CTempString line;
+        while (numLines  &&  !mLineReader.AtEOF()) {
+            line = *++mLineReader;
+            NStr::TruncateSpacesInPlace(line);
+            mBuffer.push_back(line);
+            if (!IsCommentLine(line)) {
+                --numLines;
+            }
+        }
+        return true;
+    }
+            
+    virtual bool IsCommentLine(
+        const CTempString& line)
+    {
+        if (NStr::StartsWith(line, "#")) {
+            return true;
+        }
+        if (NStr::IsBlank(line)) {
+            return true;
+        }
+        return false;
+    };
+
+    bool GetLine(
+        string& line)
+    {
+        while (!mBuffer.empty()  ||  !mLineReader.AtEOF()) {
+            string temp;
+            if (!mBuffer.empty()) {
+                temp = mBuffer.front();
+                mBuffer.pop_front();
+            }
+            else {
+                temp = *++mLineReader;
+                NStr::TruncateSpacesInPlace(temp);
+            }
+            if (!IsCommentLine(temp)) {
+                line = temp;
+                ++mLineNumber;
+                return true;
+            }
+        }
+        return false;   
+    };
+
+    bool UngetLine(
+        const string& line)
+    {
+        mBuffer.push_front(line);
+        --mLineNumber;
+        return true;
+    }
+
+    int LineNumber() const
+    {
+        return mLineNumber;
+    };
+
+    LinePreIt begin()
+    {
+        return mBuffer.begin();
+    };
+
+    LinePreIt end()
+    {
+        return mBuffer.end();
+    };
+
+    void
+    AssignReader(
+        ILineReader& lineReader) {
+        if (&mLineReader != &lineReader) {
+            mLineReader = lineReader;
+            mBuffer.clear();
+            mLineNumber = 0;
+        }
+    };
+
+protected:
+    ILineReader& mLineReader;
+    deque<string> mBuffer;
+    int mLineNumber;
+};
+
+
 //  ----------------------------------------------------------------------------
 CBedReader::CBedReader(
     int flags,
@@ -105,11 +210,16 @@ CBedReader::CBedReader(
 //  ----------------------------------------------------------------------------
     CReaderBase(flags, annotName, annotTitle),
     m_currentId(""),
-    m_columncount(0),
+    mColumnSeparator(""),
+    mColumnSplitFlags(0),
+    mRealColumnCount(0),
+    mValidColumnCount(0),
+    mAssumeErrorsAreRecordLevel(true),
     m_CurrentFeatureCount(0),
     m_usescore(false),
     m_CurBatchSize(0),
-    m_MaxBatchSize(10000)
+    m_MaxBatchSize(10000),
+    mLinePreBuffer(nullptr)
 {
 }
 
@@ -134,11 +244,11 @@ CBedReader::ReadSeqAnnot(
 //  ----------------------------------------------------------------------------                
 CRef< CSeq_annot >
 CBedReader::ReadSeqAnnot(
-    ILineReader& lr,
+    ILineReader& lineReader,
     ILineErrorListener* pEC ) 
 //  ----------------------------------------------------------------------------                
 {
-    xProgressInit(lr);
+    xProgressInit(lineReader);
 
     CRef<CSeq_annot> annot;
     CRef<CAnnot_descr> desc;
@@ -152,36 +262,54 @@ CBedReader::ReadSeqAnnot(
     xParseTrackLine("track", pEC);
 
     string line;
-    while (xGetLine(lr, line)) {
+    if (!mLinePreBuffer) {
+        mLinePreBuffer.reset(new CLinePreBuffer(lineReader));
+    }
+    else {
+        mLinePreBuffer->AssignReader(lineReader);
+    }
 
-        // interact with calling party
-        if (IsCanceled()) {
-            AutoPtr<CObjReaderLineException> pErr(
-                CObjReaderLineException::Create(
-                eDiag_Info,
-                0,
-                "Reader stopped by user.",
-                ILineError::eProblem_ProgressInfo));
-            ProcessError(*pErr, pEC);
-            return CRef<CSeq_annot>();
-        }
-        xReportProgress(pEC);
+    //CLinePreBuffer preBuffer(lineReader);
 
-        if (xIsTrackLine(line)) {
-            if (!m_CurrentFeatureCount) {
-                xParseTrackLine(line, pEC);
+    try {
+        xDetermineLikelyColumnCount(*mLinePreBuffer, pEC);
+        while (mLinePreBuffer->GetLine(line)) {
+            m_uLineNumber = mLinePreBuffer->LineNumber();
+            // interact with calling party
+            if (IsCanceled()) {
+                AutoPtr<CObjReaderLineException> pErr(
+                    CObjReaderLineException::Create(
+                    eDiag_Info,
+                    0,
+                    "Reader stopped by user.",
+                    ILineError::eProblem_ProgressInfo));
+                ProcessError(*pErr, pEC);
+                return CRef<CSeq_annot>();
+            }
+            xReportProgress(pEC);
+
+            if (xIsTrackLine(line)) {
+                if (!m_CurrentFeatureCount) {
+                    xParseTrackLine(line, pEC);
+                    continue;
+                }
+                mLinePreBuffer->UngetLine(line);
+                //xDetermineLikelyColumnCount(*mLinePreBuffer, pEC);
+                break;
+            }
+            if (xParseBrowserLine(line, annot, pEC)) {
+                continue;
+            }
+            if (xParseFeature(line, annot, pEC)) {
                 continue;
             }
-            xUngetLine(lr);
-            break;
-        }
-        if (xParseBrowserLine(line, annot, pEC)) {
-            continue;
-        }
-        if (xParseFeature(line, annot, pEC)) {
-            continue;
         }
     }
+    catch(CObjReaderLineException& err) {
+        ProcessError(err, pEC);
+        return CRef<CSeq_annot>();
+    }
+
     //  Only return a valid object if there was at least one feature
     if (0 == m_CurrentFeatureCount) {
         return CRef<CSeq_annot>();
@@ -190,6 +318,204 @@ CBedReader::ReadSeqAnnot(
     return annot;
 }
 
+//  ----------------------------------------------------------------------------
+bool CBedReader::xAddDefaultColumns(
+    vector<string>& columns)
+//  ----------------------------------------------------------------------------
+{
+    if (mValidColumnCount > 4  &&  columns[4].empty()  &&  m_usescore) {
+        columns[4] = "0";
+    }
+    if (mValidColumnCount > 5  &&  columns[5].empty()) {
+        columns[5] = ".";
+    }
+    if (mValidColumnCount > 6  &&  columns[6].empty()) {
+        columns[6] = columns[1];
+    }
+    if (mValidColumnCount > 7  &&  columns[7].empty()) {
+    columns[7] = columns[2];
+    }
+
+    return true;
+}
+
+//  ----------------------------------------------------------------------------
+bool CBedReader::xSplitColumns(
+    const string& line,
+    vector<string>& columns)
+//  ----------------------------------------------------------------------------
+{
+    if (mColumnSeparator.empty()) {
+        columns.clear();
+        mColumnSeparator = "\t";
+        NStr::Split(line, mColumnSeparator, columns, mColumnSplitFlags);
+        if (columns.size() > 2) {
+            return true;
+        }
+        mColumnSeparator = " \t";
+        mColumnSplitFlags = NStr::fSplit_MergeDelimiters;
+    }
+    columns.clear();
+    NStr::Split(line, mColumnSeparator, columns, mColumnSplitFlags);
+    return (columns.size() > 2);
+}
+
+//  ----------------------------------------------------------------------------
+bool CBedReader::xDetermineLikelyColumnCount(
+    CLinePreBuffer& preBuffer,
+    ILineErrorListener* pEc)
+//  ----------------------------------------------------------------------------
+{
+    using LineIt = CLinePreBuffer::LinePreIt;
+
+    AutoPtr<CObjReaderLineException> pErrColumnCount(
+        CObjReaderLineException::Create(
+            eDiag_Fatal,
+            0,
+            "Bad data line: Inconsistent column count." ) );
+    AutoPtr<CObjReaderLineException> pErrChromBounds(
+        CObjReaderLineException::Create(
+            eDiag_Fatal,
+            0,
+            "Bad data line: Invalid chrom boundaries." ) );
+
+    const size_t MIN_SAMPLE_SIZE = 50;
+    preBuffer.FillBuffer(MIN_SAMPLE_SIZE);
+
+    mRealColumnCount = mValidColumnCount = 0;
+    vector<string>::size_type realColumnCount = 0;
+    vector<string>::size_type validColumnCount = 0;
+    for (LineIt lineIt = preBuffer.begin(); lineIt != preBuffer.end(); ++lineIt) {
+        const auto& line = *lineIt;
+        if (preBuffer.IsCommentLine(line)) {
+            continue;
+        }
+        if (this->xIsTrackLine(line)) {
+            continue;
+        }
+        if (this->xIsBrowserLine(line)) {
+            continue;
+        }
+
+
+        vector<string> columns;
+        if (!xSplitColumns(line, columns)) {
+            pErrColumnCount->Throw();
+        }
+        xCleanColumnValues(columns);
+
+
+        if (realColumnCount == 0 ) {
+            realColumnCount = columns.size();
+        }
+        if (realColumnCount != columns.size()) {
+            pErrColumnCount->Throw();
+        }
+        
+        if (validColumnCount == 0) {
+            validColumnCount = realColumnCount;
+            if (validColumnCount > 12) {
+                validColumnCount = 12;
+            }
+        }
+        unsigned long chromStart = 0, chromEnd = 0;
+        try {
+            chromStart = NStr::StringToULong(columns[1]);
+            chromEnd = NStr::StringToULong(columns[2]);
+        }
+        catch (CException&) {
+            pErrChromBounds->Throw();
+        }
+        if (validColumnCount >= 7) {
+            try {
+                auto thickStart = NStr::StringToULong(columns[6]);
+                if (thickStart < chromStart  ||  chromEnd < thickStart) {
+                    validColumnCount = 6;
+                }
+            }
+            catch(CException&) {
+                validColumnCount = 6;
+            }
+        }
+        if (validColumnCount >= 8) {
+            try {
+                auto thickEnd = NStr::StringToULong(columns[7]);
+                if (thickEnd < chromStart  ||  chromEnd < thickEnd) {
+                    validColumnCount = 6;
+                }
+            }
+            catch(CException&) {
+                validColumnCount = 6;
+            }
+        }
+
+        int blockCount;
+        if (validColumnCount >= 10) {
+            try {
+                blockCount = NStr::StringToInt(
+                    columns[9], NStr::fDS_ProhibitFractions);
+                if (blockCount < 1) {
+                    validColumnCount = 9;
+                }
+            }
+            catch(CException&) {
+                validColumnCount = 9;
+            }
+        }
+        if (validColumnCount >= 11) {
+            vector<string> blockSizes;
+            auto col10 = columns[10];
+            if (NStr::EndsWith(col10, ",")) {
+                col10 = col10.substr(0, col10.size()-1);
+            }
+            NStr::Split(col10, ",", blockSizes, NStr::fSplit_MergeDelimiters);
+            if (blockSizes.size() != blockCount) {
+                validColumnCount = 9;
+            }
+            else {
+                try {
+                    for (auto blockSize: blockSizes) {
+                        NStr::StringToULong(blockSize);
+                    }
+                }
+                catch(CException&) {
+                    validColumnCount = 9;
+                }
+            }
+        }
+        if (validColumnCount >= 12) {
+            vector<string> blockStarts;
+            auto col11 = columns[11];
+            if (NStr::EndsWith(col11, ",")) {
+                col11 = col11.substr(0, col11.size()-1);
+            }
+            NStr::Split(col11, ",", blockStarts, NStr::fSplit_MergeDelimiters);
+            if (blockStarts.size() != blockCount) {
+                validColumnCount = 9;
+            }
+            else {
+                try {
+                    for (auto blockStart: blockStarts) {
+                        NStr::StringToULong(blockStart);
+                    }
+                }
+                catch(CException&) {
+                    validColumnCount = 9;
+                }
+            }
+        }   
+    } 
+    mRealColumnCount = realColumnCount;
+    mValidColumnCount = validColumnCount;
+    mAssumeErrorsAreRecordLevel = (
+        validColumnCount == realColumnCount  &&
+        validColumnCount != 7  &&
+        validColumnCount != 10  &&
+        validColumnCount != 11);
+        
+    return true;
+}
+
 //  ----------------------------------------------------------------------------
 void CBedReader::xPostProcessAnnot(
     CRef<CSeq_annot>& annot,
@@ -235,9 +561,6 @@ CBedReader::xParseTrackLine(
             return false;
         }
     }
-    if ( !m_currentId.empty() ) {
-        m_columncount = 0;
-    }
     m_currentId.clear();
     if (!CReaderBase::xParseTrackLine(strLine, pEC)) {
         AutoPtr<CObjReaderLineException> pErr(
@@ -276,9 +599,10 @@ CBedReader::xParseFeature(
 
     //  parse
     vector<string> fields;
-    NStr::Split(record_copy, " \t", fields, NStr::fSplit_MergeDelimiters);
+    xSplitColumns(line, fields);
     try {
         xCleanColumnValues(fields);
+        xAddDefaultColumns(fields);
     }
     catch(CObjReaderLineException& err) {
         ProcessError(err, pEC);
@@ -286,9 +610,9 @@ CBedReader::xParseFeature(
     }
     if (xParseFeature(fields, pAnnot, pEC)) {
         ++m_CurrentFeatureCount;
-        return false;
+        return true;
     }
-    return true;
+    return false;
 }
 
 //  ----------------------------------------------------------------------------
@@ -301,19 +625,13 @@ bool CBedReader::xParseFeature(
     static int count = 0;
     count++;
 
-    if (fields.size() != m_columncount) {
-        if ( 0 == m_columncount ) {
-            m_columncount = fields.size();
-        }
-        else {
-            AutoPtr<CObjReaderLineException> pErr(
-                CObjReaderLineException::Create(
-                eDiag_Error,
-                0,
-                "Bad data line: Inconsistent column count." ) );
-            ProcessError(*pErr, pEC );
-            return false;
-        }
+    if (fields.size()!= mRealColumnCount) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Bad data line: Inconsistent column count." ) );
+        pErr->Throw();
     }
 
     if (m_iFlags & CBedReader::fThreeFeatFormat) {
@@ -357,18 +675,33 @@ bool CBedReader::xParseFeatureGeneModelFormat(
     ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
-     unsigned int baseId = 3*m_CurrentFeatureCount;
+    unsigned int baseId = 3*m_CurrentFeatureCount;
 
-   if (!xAppendFeatureGene(fields, annot, baseId, pEC)) {
+    CRef<CSeq_feat> pGene = xAppendFeatureGene(fields, annot, baseId, pEC);
+    if (!pGene) {
         return false;
     }
-    if (xContainsCdsFeature(fields)  &&  
-            !xAppendFeatureCds(fields, annot, baseId, pEC)) {
-        return false;
+
+    CRef<CSeq_feat> pRna;
+    if (xContainsRnaFeature(fields)) {//blocks
+        pRna = xAppendFeatureRna(fields, annot, baseId, pEC);
+        if (!pRna) {
+            return false;
+        }
     }
-    if (xContainsRnaFeature(fields)  &&
-            !xAppendFeatureRna(fields, annot, baseId, pEC)) {
-        return false;
+
+    CRef<CSeq_feat> pCds;
+    if (xContainsCdsFeature(fields)) {//thick
+        pCds = xAppendFeatureCds(fields, annot, baseId, pEC);
+        if (!pCds) {
+            return false;
+        }
+    }
+
+    if (pRna  &&  pCds) {
+        CRef<CSeq_loc> pRnaLoc(new CSeq_loc);
+        CRef<CSeq_loc> pClippedLoc = pRna->GetLocation().Intersect(pCds->GetLocation(), 0, 0);
+        pCds->SetLocation(*pClippedLoc);
     }
     return true;
 }
@@ -388,7 +721,7 @@ bool CBedReader::xAppendFeatureChrom(
         //xSetFeatureTitle(feature, fields);
         xSetFeatureLocationChrom(feature, fields);
         xSetFeatureIdsChrom(feature, fields, baseId);
-        xSetFeatureBedData(feature, fields);
+        xSetFeatureBedData(feature, fields, pEC);
     }
     catch(CObjReaderLineException& err) {
         //m_currentId.clear();
@@ -401,7 +734,7 @@ bool CBedReader::xAppendFeatureChrom(
 }
 
 //  ----------------------------------------------------------------------------
-bool CBedReader::xAppendFeatureGene(
+CRef<CSeq_feat> CBedReader::xAppendFeatureGene(
     const vector<string>& fields,
     CRef<CSeq_annot>& annot,
     unsigned int baseId,
@@ -415,16 +748,16 @@ bool CBedReader::xAppendFeatureGene(
         ////xSetFeatureTitle(feature, fields);
         xSetFeatureLocationGene(feature, fields);
         xSetFeatureIdsGene(feature, fields, baseId);
-        xSetFeatureBedData(feature, fields);
+        xSetFeatureBedData(feature, fields, pEC);
     }
     catch(CObjReaderLineException& err) {
         //m_currentId.clear();
         ProcessError(err, pEC);
-        return false;
+        return CRef<CSeq_feat>();
     }
     ftable.push_back(feature);
     m_currentId = fields[0];
-    return true;
+    return feature;
 }
 
 //  ----------------------------------------------------------------------------
@@ -442,7 +775,7 @@ bool CBedReader::xAppendFeatureThick(
         //xSetFeatureTitle(feature, fields);
         xSetFeatureLocationThick(feature, fields);
         xSetFeatureIdsThick(feature, fields, baseId);
-        xSetFeatureBedData(feature, fields);
+        xSetFeatureBedData(feature, fields, pEC);
     }
     catch(CObjReaderLineException& err) {
         //m_currentId.clear();
@@ -454,7 +787,7 @@ bool CBedReader::xAppendFeatureThick(
 }
 
 //  ----------------------------------------------------------------------------
-bool CBedReader::xAppendFeatureCds(
+CRef<CSeq_feat> CBedReader::xAppendFeatureCds(
     const vector<string>& fields,
     CRef<CSeq_annot>& annot,
     unsigned int baseId,
@@ -468,15 +801,15 @@ bool CBedReader::xAppendFeatureCds(
         ////xSetFeatureTitle(feature, fields);
         xSetFeatureLocationCds(feature, fields);
         xSetFeatureIdsCds(feature, fields, baseId);
-        xSetFeatureBedData(feature, fields);
+        xSetFeatureBedData(feature, fields, pEC);
     }
     catch(CObjReaderLineException& err) {
         //m_currentId.clear();
         ProcessError(err, pEC);
-        return false;
+        return CRef<CSeq_feat>();
     }
     ftable.push_back(feature);
-    return true;
+    return feature;
 }
 
 //  ----------------------------------------------------------------------------
@@ -494,7 +827,7 @@ bool CBedReader::xAppendFeatureBlock(
         ////xSetFeatureTitle(feature, fields);
         xSetFeatureLocationBlock(feature, fields);
         xSetFeatureIdsBlock(feature, fields, baseId);
-        xSetFeatureBedData(feature, fields);
+        xSetFeatureBedData(feature, fields, pEC);
     }
     catch(CObjReaderLineException& err) {
         //m_currentId.clear();
@@ -506,7 +839,7 @@ bool CBedReader::xAppendFeatureBlock(
 }
 
 //  ----------------------------------------------------------------------------
-bool CBedReader::xAppendFeatureRna(
+CRef<CSeq_feat> CBedReader::xAppendFeatureRna(
     const vector<string>& fields,
     CRef<CSeq_annot>& annot,
     unsigned int baseId,
@@ -520,15 +853,15 @@ bool CBedReader::xAppendFeatureRna(
         //xSetFeatureTitle(feature, fields);
         xSetFeatureLocationRna(feature, fields);
         xSetFeatureIdsRna(feature, fields, baseId);
-        xSetFeatureBedData(feature, fields);
+        xSetFeatureBedData(feature, fields, pEC);
     }
     catch(CObjReaderLineException& err) {
         //m_currentId.clear();
         ProcessError(err, pEC);
-        return false;
+        return CRef<CSeq_feat>();
     }
     ftable.push_back(feature);
-    return true;
+    return feature;
 }
 
 
@@ -566,7 +899,7 @@ void CBedReader::x_SetFeatureDisplayData(
 {
     CRef<CUser_object> display_data( new CUser_object );
     display_data->SetType().SetStr( "Display Data" );
-    if ( m_columncount >= 4 ) {
+    if (mValidColumnCount >= 4) {
         display_data->AddField( "name", fields[3] );
     }
     else {
@@ -574,7 +907,7 @@ void CBedReader::x_SetFeatureDisplayData(
         feature->SetData().SetUser( *display_data );
         return;
     }
-    if ( m_columncount >= 5 ) {
+    if (mValidColumnCount >= 5) {
         if ( !m_usescore ) {
             display_data->AddField( 
                 "score",
@@ -588,30 +921,30 @@ void CBedReader::x_SetFeatureDisplayData(
 				NStr::fConvErr_NoThrow|NStr::fAllowTrailingSymbols) );
         }
     }
-    if ( m_columncount >= 7 ) {
+    if (mValidColumnCount >= 7) {
         display_data->AddField( 
             "thickStart",
-            NStr::StringToInt(fields[6], NStr::fConvErr_NoThrow) );
+            NStr::StringToInt(fields[6], NStr::fDS_ProhibitFractions) );
     }
-    if ( m_columncount >= 8 ) {
+    if (mValidColumnCount >= 8) {
         display_data->AddField( 
             "thickEnd",
-            NStr::StringToInt(fields[7], NStr::fConvErr_NoThrow) - 1 );
+            NStr::StringToInt(fields[7], NStr::fDS_ProhibitFractions) - 1 );
     }
-    if ( m_columncount >= 9 ) {
+    if (mValidColumnCount >= 9) {
         display_data->AddField( 
             "itemRGB",
             fields[8]);
     }
-    if ( m_columncount >= 10 ) {
+    if (mValidColumnCount >= 10) {
         display_data->AddField( 
             "blockCount",
-            NStr::StringToInt(fields[9], NStr::fConvErr_NoThrow) );
+            NStr::StringToInt(fields[9], NStr::fDS_ProhibitFractions) );
     }
-    if ( m_columncount >= 11 ) {
+    if (mValidColumnCount >= 11) {
         display_data->AddField( "blockSizes", fields[10] );
     }
-    if ( m_columncount >= 12 ) {
+    if (mValidColumnCount >= 12) {
         display_data->AddField( "blockStarts", fields[11] );
     }
     feature->SetData().SetUser( *display_data );
@@ -876,8 +1209,9 @@ void CBedReader::xSetFeatureLocationBlock(
     for (size_t i=0; i < blockCount; ++i) {
         CRef<CSeq_interval> pInterval(new CSeq_interval);
         pInterval->SetId(*pId);
-        pInterval->SetFrom(blockStarts[i]);
-        pInterval->SetTo(blockStarts[i] + blockSizes[i] - 1);
+        pInterval->SetFrom(static_cast<CSeq_interval::TFrom>(blockStarts[i]));
+        pInterval->SetTo(static_cast<CSeq_interval::TTo>(
+            blockStarts[i] + blockSizes[i] - 1));
         pInterval->SetStrand(strand);
         if (negative)
             blocks.insert(blocks.begin(), pInterval);
@@ -974,8 +1308,9 @@ void CBedReader::xSetFeatureLocationRna(
     for (size_t i=0; i < blockCount; ++i) {
         CRef<CSeq_interval> pInterval(new CSeq_interval);
         pInterval->SetId(*pId);
-        pInterval->SetFrom(blockStarts[i]);
-        pInterval->SetTo(blockStarts[i] + blockSizes[i] -1);
+        pInterval->SetFrom(static_cast<CSeq_interval::TFrom>(blockStarts[i]));
+        pInterval->SetTo(static_cast<CSeq_interval::TTo>(
+            blockStarts[i] + blockSizes[i] -1));
         pInterval->SetStrand(strand);
         if (negative)
             blocks.insert(blocks.begin(), pInterval);
@@ -1076,6 +1411,13 @@ void CBedReader::xSetFeatureIdsCds(
         pXrefBlock->SetId(*pIdBlock);  
         feature->SetXref().push_back(pXrefBlock);   
     }
+    else {
+        CRef<CFeat_id> pIdChrom(new CFeat_id);
+        pIdChrom->SetLocal().SetId(baseId);
+        CRef<CSeqFeatXref> pXrefChrom(new CSeqFeatXref);
+        pXrefChrom->SetId(*pIdChrom);  
+        feature->SetXref().push_back(pXrefChrom);   
+    }
 }
 
 //  ----------------------------------------------------------------------------
@@ -1192,7 +1534,8 @@ void CBedReader::xSetFeatureScore(
 //  ----------------------------------------------------------------------------
 void CBedReader::xSetFeatureColor(
     CRef<CUser_object> pDisplayData,
-    const vector<string>& fields )
+    const vector<string>& fields,
+    ILineErrorListener* pEC )
 //  ----------------------------------------------------------------------------
 {
     //1: if track line itemRgb is set, try that first:
@@ -1200,7 +1543,7 @@ void CBedReader::xSetFeatureColor(
     if (trackItemRgb == "On"  &&  fields.size() >= 9) {
         string featItemRgb = fields[8];
         if (featItemRgb != ".") {
-            xSetFeatureColorFromItemRgb(pDisplayData, featItemRgb);
+            xSetFeatureColorFromItemRgb(pDisplayData, featItemRgb, pEC);
             return;
         }
     }
@@ -1220,14 +1563,14 @@ void CBedReader::xSetFeatureColor(
     if (!trackColorByStrand.empty()  && fields.size() >= 6) {
         ENa_strand strand = 
             (fields[5] == "-") ? eNa_strand_minus : eNa_strand_plus;
-        xSetFeatureColorByStrand(pDisplayData, trackColorByStrand, strand);
+        xSetFeatureColorByStrand(pDisplayData, trackColorByStrand, strand, pEC);
         return;
     }
     //4: if none of the track color attributes are set, attempt feature itemRgb:
     if (fields.size() >= 9) {
         string featItemRgb = fields[8];
         if (featItemRgb != ".") {
-            xSetFeatureColorFromItemRgb(pDisplayData, featItemRgb);
+            xSetFeatureColorFromItemRgb(pDisplayData, featItemRgb, pEC);
             return;
         }
     }
@@ -1249,14 +1592,15 @@ void CBedReader::xSetFeatureColorDefault(
 void CBedReader::xSetFeatureColorByStrand(
     CRef<CUser_object> pDisplayData,
     const string& trackColorByStrand,
-    ENa_strand strand)
+    ENa_strand strand,
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     try {
         string colorPlus, colorMinus;
         NStr::SplitInTwo(trackColorByStrand, " ", colorPlus, colorMinus);
         string useColor = (strand == eNa_strand_minus) ? colorMinus : colorPlus;
-        xSetFeatureColorFromItemRgb(pDisplayData, useColor);
+        xSetFeatureColorFromItemRgb(pDisplayData, useColor, pEC);
     }
     catch (std::exception&) {
         AutoPtr<CObjReaderLineException> pErr(
@@ -1276,7 +1620,7 @@ void CBedReader::xSetFeatureColorFromScore(
 {
     int score = 0;
     try {
-        score = NStr::StringToInt(featScore);
+        score = static_cast<int>(NStr::StringToDouble(featScore));
     }
     catch (const std::exception&) {
         AutoPtr<CObjReaderLineException> pErr(
@@ -1294,7 +1638,7 @@ void CBedReader::xSetFeatureColorFromScore(
             "Invalid data line: Bad score value to be used for color.") );
         pErr->Throw();
     }
-    string greyValue  = NStr::IntToString(255 - (score/4));
+    string greyValue  = NStr::DoubleToString(255 - (score/4));
     vector<string> srgb{ greyValue, greyValue, greyValue};
     string rgbValue = NStr::Join(srgb, " ");
     pDisplayData->AddField("color", rgbValue);
@@ -1303,57 +1647,95 @@ void CBedReader::xSetFeatureColorFromScore(
 //  ----------------------------------------------------------------------------
 void CBedReader::xSetFeatureColorFromItemRgb(
     CRef<CUser_object> pDisplayData,
-    const string& itemRgb )
+    const string& itemRgb,
+    ILineErrorListener* pEC )
 //  ----------------------------------------------------------------------------
 {
-    vector<string> srgb;
-    if (itemRgb == "0") {
-        srgb.push_back("0");
-        srgb.push_back("0");
-        srgb.push_back("0");
-    }
-    else {
-        NStr::Split(itemRgb, ",", srgb);
-    }
-    if (srgb.size() != 3) {
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
-            eDiag_Error,
-            0,
-            "Invalid data line: Bad color value.") );
-        pErr->Throw();
-    }
-    try {
-        for (int i=0; i < 3; i++)
-        {
-           int x = NStr::StringToInt(srgb[i]);
-           if (x<0 || x>255) {
-                AutoPtr<CObjReaderLineException> pErr(
-                    CObjReaderLineException::Create(
-                    eDiag_Error,
-                    0,
-                    "Invalid data line: Bad color value.") );
-                pErr->Throw();
-           }
+    AutoPtr<CObjReaderLineException> pErr(
+        CObjReaderLineException::Create(
+            eDiag_Warning,
+            this->m_uLineNumber,
+            "Bad color value - converted to BLACK.") );
+    const string rgbDefault = "0 0 0";
 
+    //optimization for common case:
+    if (itemRgb == "0") {
+        pDisplayData->AddField("color", rgbDefault);
+        return;
+    }
+
+    vector<string> srgb;
+    NStr::Split(itemRgb, ",", srgb);
+
+    if (srgb.size() == 3) {
+        auto valuesOk = true;
+        for (auto i=0; i<3; ++i) {
+            int test;
+            try {
+                test = NStr::StringToInt(srgb[i], NStr::fDS_ProhibitFractions);
+            }
+            catch(CException&) {
+                valuesOk = false;
+                break;
+            }
+            if ((test < 0)  ||  (256 <= test)) {
+                valuesOk = false;
+                break;
+            }
         }
+        if (!valuesOk) {
+            ProcessWarning(*pErr, pEC);
+            pDisplayData->AddField("color", rgbDefault);
+            return;
+        }
+        auto outValue = srgb[0] + " " + srgb[1] + " " + srgb[2];
+        pDisplayData->AddField("color", outValue);
+        return;
+    } 
+
+    if (srgb.size() == 1) {
+        auto assumeHex = false;
+        string itemRgbCopy(itemRgb);
+        if (NStr::StartsWith(itemRgbCopy, "0x")) {
+            assumeHex = true;
+            itemRgbCopy = itemRgb.substr(2);
+        }
+        else if (NStr::StartsWith(itemRgbCopy, "#")) {
+            assumeHex = true;
+            itemRgbCopy = itemRgbCopy.substr(1);
+        }
+        unsigned long colorValue;
+        int radix = (assumeHex ? 16 : 10);
+        try {
+            colorValue = NStr::StringToULong(
+                itemRgbCopy, NStr::fDS_ProhibitFractions, radix);
+        }
+        catch (CStringException&) {
+            ProcessWarning(*pErr, pEC);
+            pDisplayData->AddField("color", rgbDefault);
+            return;
+        }
+        int blue = colorValue & 0xFF;
+        colorValue >>= 8;
+        int green = colorValue & 0xFF;
+        colorValue >>= 8;
+        int red = colorValue & 0xFF;
+        auto outValue = NStr::IntToString(red) + " " + NStr::IntToString(green) + 
+            " " + NStr::IntToString(blue);
+        pDisplayData->AddField("color", outValue);
+        return;
     }
-    catch(const std::exception&) {
-        AutoPtr<CObjReaderLineException> pErr(
-            CObjReaderLineException::Create(
-            eDiag_Error,
-            0,
-            "Invalid data line: Bad color value.") );
-        pErr->Throw();
-    }
-    string rgbValue = NStr::Join(srgb, " ");
-    pDisplayData->AddField("color", rgbValue);
+
+    ProcessWarning(*pErr, pEC);
+    pDisplayData->AddField("color", rgbDefault);
+    return;
 }
 
 //  ----------------------------------------------------------------------------
 void CBedReader::xSetFeatureBedData(
     CRef<CSeq_feat>& feature,
-    const vector<string>& fields )
+    const vector<string>& fields,
+    ILineErrorListener* pEc )
 //  ----------------------------------------------------------------------------
 {
     CSeqFeatData& data = feature->SetData();
@@ -1371,7 +1753,7 @@ void CBedReader::xSetFeatureBedData(
     exts.push_front(pDisplayData);
 
     xSetFeatureScore(pDisplayData, fields);
-    xSetFeatureColor(pDisplayData, fields);
+    xSetFeatureColor(pDisplayData, fields, pEc);
 }
 
 //  ----------------------------------------------------------------------------
@@ -1514,19 +1896,14 @@ CBedReader::xReadBedRecordRaw(
         return false;
     }
 
-    if (columns.size() != m_columncount) {
-        if ( 0 == m_columncount ) {
-            m_columncount = columns.size();
-        }
-        else {
-            AutoPtr<CObjReaderLineException> pErr(
-                CObjReaderLineException::Create(
-                eDiag_Error,
-                0,
-                "Bad data line: Inconsistent column count." ) );
-            ProcessError(*pErr, pMessageListener);
-            return false;
-        }
+    if (columns.size() != mRealColumnCount) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+            eDiag_Error,
+            0,
+            "Bad data line: Inconsistent column count." ) );
+        ProcessError(*pErr, pMessageListener);
+        return false;
     }
 
     //assign columns to record:
@@ -1561,7 +1938,7 @@ CBedReader::xReadBedRecordRaw(
     }
 
     int score(-1);
-    if (m_columncount >= 5  &&  columns[4] != ".") {
+    if (mValidColumnCount >= 5  &&  columns[4] != ".") {
         try {
             score = NStr::StringToInt(columns[4], 
 			NStr::fConvErr_NoThrow|NStr::fAllowTrailingSymbols);
@@ -1577,7 +1954,7 @@ CBedReader::xReadBedRecordRaw(
         }
     }
     ENa_strand strand = eNa_strand_plus;
-    if (m_columncount >= 6) {
+    if (mValidColumnCount >= 6) {
         if (columns[5] == "-") {
             strand = eNa_strand_minus;
         }
@@ -1595,7 +1972,7 @@ CBedReader::xContainsThickFeature(
     const vector<string>& fields) const
 //  ----------------------------------------------------------------------------
 {
-    if (fields.size() < 8) {
+    if (fields.size() < 8  ||  mValidColumnCount < 8) {
         return false;
     }
 
@@ -1622,11 +1999,11 @@ CBedReader::xContainsThickFeature(
 
 //  ----------------------------------------------------------------------------
 bool
-CBedReader::xContainsCdsFeature(
+CBedReader::xContainsRnaFeature(
     const vector<string>& fields) const
 //  ----------------------------------------------------------------------------
 {
-    if (fields.size() < 8) {
+    if (fields.size() < 12  ||  mValidColumnCount < 12) {
         return false;
     }
 
@@ -1657,17 +2034,17 @@ CBedReader::xContainsBlockFeature(
     const vector<string>& fields) const
 //  ----------------------------------------------------------------------------
 {
-    return (fields.size() >= 12);
+    return (fields.size() >= 12  &&  mValidColumnCount >= 12);
 }
 
 
 //  ----------------------------------------------------------------------------
 bool
-CBedReader::xContainsRnaFeature(
+CBedReader::xContainsCdsFeature(
     const vector<string>& fields) const
 //  ----------------------------------------------------------------------------
 {
-    return (fields.size() >= 12);
+    return (fields.size() >= 8  &&  mValidColumnCount >= 8);
 }
 
 
@@ -1751,16 +2128,16 @@ CBedReader::xAssignBedColumnCount(
     CSeq_annot& annot)
 //  ----------------------------------------------------------------------------
 {
-    if(m_columncount < 3) {
+    if(mValidColumnCount < 3) {
         return;
     }
-    CRef<CUser_object> columnCountUser( new CUser_object() );
-    columnCountUser->SetType().SetStr( "NCBI_BED_COLUMN_COUNT" );
-    columnCountUser->AddField("NCBI_BED_COLUMN_COUNT", int ( m_columncount ) );
+    CRef<CUser_object> columnCountUser(new CUser_object());
+    columnCountUser->SetType().SetStr("NCBI_BED_COLUMN_COUNT");
+    columnCountUser->AddField("NCBI_BED_COLUMN_COUNT", int (mValidColumnCount));
     
-    CRef<CAnnotdesc> userDesc( new CAnnotdesc() );
-    userDesc->SetUser().Assign( *columnCountUser );
-    annot.SetDesc().Set().push_back( userDesc );
+    CRef<CAnnotdesc> userDesc(new CAnnotdesc());
+    userDesc->SetUser().Assign(*columnCountUser);
+    annot.SetDesc().Set().push_back(userDesc);
 }                   
 
 END_objects_SCOPE
