@@ -1,4 +1,4 @@
-/*  $Id: fasta_reader_utils.cpp 543881 2017-08-15 13:56:46Z ivanov $
+/*  $Id: fasta_reader_utils.cpp 566929 2018-07-10 16:29:24Z foleyjp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -36,6 +36,7 @@
 #include <corelib/ncbidiag.hpp>
 #include <objtools/error_codes.hpp>
 #include <objtools/readers/fasta.hpp>
+#include <objects/general/Dbtag.hpp>
 #include <objects/general/Object_id.hpp>
 #include <objtools/readers/fasta_reader_utils.hpp>
 
@@ -44,157 +45,250 @@
 BEGIN_NCBI_SCOPE
 BEGIN_SCOPE(objects)
 
-// For reasons of efficiency, this method does not use CRef<CSeq_interval> to access range 
-// information - RW-26
-void CFastaDeflineReader::ParseDefline(const string& defline,
-    const SDeflineParseInfo& info,
-    const TIgnoredProblems& ignoredErrors,
-    list<CRef<CSeq_id>>& ids,
-    bool& hasRange,
-    TSeqPos& rangeStart,
-    TSeqPos& rangeEnd,
-    TSeqTitles& seqTitles, 
-    ILineErrorListener* pMessageListener) 
-{
-    size_t range_len = 0;
-    const TFastaFlags& fFastaFlags = info.fFastaFlags;
-    const TSeqPos& lineNumber = info.lineNumber;
-    hasRange = false;
+size_t CFastaDeflineReader::s_MaxLocalIDLength = CSeq_id::kMaxLocalIDLength;
+size_t CFastaDeflineReader::s_MaxGeneralTagLength = CSeq_id::kMaxGeneralTagLength;
+size_t CFastaDeflineReader::s_MaxAccessionLength = CSeq_id::kMaxAccessionLength;
 
-    const size_t len = defline.length(); 
-    if (len <= 1 || 
-        NStr::IsBlank(defline.substr(1))) {
+static bool s_IsValidLocalID(const CSeq_id& id, const CSeqIdCheck::TInfo& info) 
+{
+    if (id.IsLocal()) {
+        if (id.GetLocal().IsId()) {
+            return true;
+        }
+        if (id.GetLocal().IsStr()) {
+            auto id_string = id.GetLocal().GetStr();
+            auto string_to_check = (info.fFastaFlags & CFastaReader::fQuickIDCheck) ?
+                                    id_string.substr(0,1) :
+                                    id_string;
+            return !(CSeq_id::CheckLocalID(string_to_check)&CSeq_id::fInvalidChar);
+        }
+    }
+
+    return false;
+}
+
+
+static void s_PostError(ILineErrorListener* pMessageListener,
+    const TSeqPos lineNumber,
+    const string& errMessage,
+    const CObjReaderLineException::EProblem problem,
+    const CObjReaderParseException::EErrCode errCode) 
+{
+
+    unique_ptr<CObjReaderLineException> pLineExpt(
+        CObjReaderLineException::Create(
+        eDiag_Error,
+        lineNumber,
+        errMessage, 
+        problem,
+        "", "", "", "",
+        errCode));
+
+    if (!pMessageListener || !pMessageListener->PutError(*pLineExpt)) {
+        throw CObjReaderParseException(DIAG_COMPILE_INFO, 0, errCode, errMessage, lineNumber, eDiag_Error);
+    }
+}
+
+static void s_PostWarning(ILineErrorListener* pMessageListener, 
+    const TSeqPos lineNumber,
+    const string& errMessage, 
+    const CObjReaderLineException::EProblem problem, 
+    const CObjReaderParseException::EErrCode errCode) 
+{
+    unique_ptr<CObjReaderLineException> pLineExpt(
+        CObjReaderLineException::Create(
+        eDiag_Warning,
+        lineNumber,
+        errMessage, 
+        problem,
+        "", "", "", "",
+        errCode));
+
+    if (!pMessageListener) {
+        LOG_POST_X(1, Warning << pLineExpt->Message());
         return;
     }
 
-    if (defline.front() != '>') {
-        NCBI_THROW2(CObjReaderParseException, eFormat, 
-            "Invalid defline. First character is not '>'", 0);
+    if (!pMessageListener->PutError(*pLineExpt)) {
+        throw CObjReaderParseException(DIAG_COMPILE_INFO, 0, errCode, errMessage, lineNumber, eDiag_Warning);
     }
+}
+
+// For reasons of efficiency, this method does not use CRef<CSeq_interval> to access range 
+// information - RW-26
+void CFastaDeflineReader::ParseDefline(const CTempString& defline,
+    const SDeflineParseInfo& info,
+    const TIgnoredProblems& ignored_errors,
+    TIds& ids,
+    bool& has_range,
+    TSeqPos& range_start,
+    TSeqPos& range_end,
+    TSeqTitles& titles, 
+    ILineErrorListener* pMessageListener) 
+{
+    SDeflineData data;
+    ParseDefline(defline, info, data, pMessageListener);
+    has_range   = data.has_range;
+    range_start = data.range_start;
+    range_end   = data.range_end;
+    titles  = move(data.titles);
+}
+
+void CFastaDeflineReader::ParseDefline(const CTempString& defline,
+        const SDeflineParseInfo& info,
+        SDeflineData& data,
+        ILineErrorListener* pMessageListener)
+{
+    static CSeqIdCheck fn_idcheck;
+    ParseDefline(defline, info, data, pMessageListener, fn_idcheck);
+}
 
 
-    // ignore spaces between '>' and the sequence ID
-    size_t start;
-    for(start = 1 ; start < len; ++start ) {
-        if( ! isspace(defline[start]) ) {
-            break;
-        }
-    }
-
-    string id_string;
-    size_t pos;
-    size_t title_start = NPOS;
-    if ((fFastaFlags & CFastaReader::fNoParseID)) {
-        title_start = start;
-    }
-    else 
+void CFastaDeflineReader::ParseDefline(const CTempString& defline,
+        const SDeflineParseInfo& info,
+        SDeflineData& data,
+        ILineErrorListener* pMessageListener,
+        FIdCheck fn_idcheck) 
     {
-    // This loop finds the end of the sequence ID
-        for ( pos = start;  pos < len;  ++pos) {
-            unsigned char c = defline[pos];
-                
-            if (c <= ' ' ) { // assumes ASCII
+        size_t range_len = 0;
+        const TFastaFlags& fFastaFlags = info.fFastaFlags;
+        const TSeqPos& lineNumber = info.lineNumber;
+        data.has_range = false;
+
+        const size_t len = defline.length();
+        if (len <= 1 || 
+            NStr::IsBlank(defline.substr(1))) {
+            return;
+        }
+
+        if (defline[0] != '>') {
+            NCBI_THROW2(CObjReaderParseException, eFormat, 
+                "Invalid defline. First character is not '>'", 0);
+        }
+
+        // ignore spaces between '>' and the sequence ID
+        size_t start;
+        for(start = 1 ; start < len; ++start ) {
+            if( ! isspace(defline[start]) ) {
                 break;
-            } else if( c == '[' ) {
-
-            // see if this is part of a FASTA mod, which
-            // implies a pattern like "[key=value]".  We only check
-            // that it looks *roughly* like a FASTA mod and only before the '='
-            //
-            // It might be worth it to put the body of this "if" into its own function,
-            // for clarity if nothing else.
-
-            const size_t left_bracket_pos = pos;
-            ++pos;
-
-                // arbitrary, but shouldn't be too much bigger than the largest possible mod key for efficiency
-                const static size_t kMaxCharsToLookAt = 30; 
-                // we give up much sooner than the length of the string, if the string is long.
-                // also note that we give up *before* the end so even if pos
-                // reaches bracket_give_up_pos, we can still say defline[pos] without worrying
-                // about array-out-of-bounds issues.
-                const size_t bracket_give_up_pos = min(len - 1, kMaxCharsToLookAt);
-                // keep track of the first space we find, because that becomes the end of the seqid
-                // if this turns out not to be a FASTA mod.
-                size_t first_space_pos = kMax_UI4;
-
-                // find the end of the key
-                for( ; pos < bracket_give_up_pos ; ++pos ) {
-                    const unsigned char c = defline[pos];
-                    if( c == '=' ) {
-                        break;
-                    } else if( c <= ' ' ) {
-                        first_space_pos = min(first_space_pos, pos);
-                        // keep going
-                    } else if( isalnum(c) || c == '-' || c == '_' ) {
-                        // this is fine; keep going
-                    } else {
-                        // bad character, so this is NOT a FASTA mod
-                        break;
-                    }
-                }
-
-                if( defline[pos] == '=' ) {
-                    // this seems to be a FASTA mod, so consider the left square bracket
-                    // to be the end of the seqid
-                    pos = left_bracket_pos;
-                    break;
-                } else {
-                    // if we stopped on anything but an equal sign, this is NOT a 
-                    // FASTA mod.
-                    if( first_space_pos < len ) {
-                        // If we've found a space at any point, we consider that the end of the seq-id
-                        pos = first_space_pos;
-                        break;
-                    }
-
-                    // it's not a FASTA mod and we didn't find any spaces, so just
-                    // keep going as normal, continuing from where we let off at "pos"
-                }
             }
         }
 
-        range_len = ParseRange(defline.substr(start, pos - start),
-            rangeStart, rangeEnd, pMessageListener);
-
-        id_string = defline.substr(start, pos - start - range_len);
-        if (NStr::IsBlank(id_string)) {
-            NCBI_THROW2(CObjReaderParseException, eFormat, 
-                "Unable to locate sequence id in definition line", 0);
+        size_t pos;
+        size_t title_start = NPOS;
+        if ((fFastaFlags & CFastaReader::fNoParseID)) {
+            title_start = start;
         }
+        else 
+        {
+        // This loop finds the end of the sequence ID
+            for ( pos = start;  pos < len;  ++pos) {
+                unsigned char c = defline[pos];
+                    
+                if (c <= ' ' ) { // assumes ASCII
+                    break;
+                } else if( c == '[' ) {
+
+                    // see if this is part of a FASTA mod, which
+                    // implies a pattern like "[key=value]".  We only check
+                    // that it looks *roughly* like a FASTA mod and only before the '='
+                    //
+                    // It might be worth it to put the body of this "if" into its own function,
+                    // for clarity if nothing else.
+
+                    const size_t left_bracket_pos = pos;
+                    ++pos;
+
+                    // arbitrary, but shouldn't be too much bigger than the largest possible mod key for efficiency
+                    const static size_t kMaxCharsToLookAt = 30; 
+                    // we give up much sooner than the length of the string, if the string is long.
+                    // also note that we give up *before* the end so even if pos
+                    // reaches bracket_give_up_pos, we can still say defline[pos] without worrying
+                    // about array-out-of-bounds issues.
+                    const size_t bracket_give_up_pos = min(len - 1, kMaxCharsToLookAt);
+                    // keep track of the first space we find, because that becomes the end of the seqid
+                    // if this turns out not to be a FASTA mod.
+                    size_t first_space_pos = kMax_UI4;
+
+                    // find the end of the key
+                    for( ; pos < bracket_give_up_pos ; ++pos ) {
+                        const unsigned char c = defline[pos];
+                        if( c == '=' ) {
+                            break;
+                        } else if( c <= ' ' ) {
+                            first_space_pos = min(first_space_pos, pos);
+                            // keep going
+                        } else if( isalnum(c) || c == '-' || c == '_' ) {
+                            // this is fine; keep going
+                        } else {
+                            // bad character, so this is NOT a FASTA mod
+                            break;
+                        }
+                    }
+
+                    if(defline[pos] == '=' ) {
+                        // this seems to be a FASTA mod, so consider the left square bracket
+                        // to be the end of the seqid
+                        pos = left_bracket_pos;
+                        break;
+                    } else {
+                        // if we stopped on anything but an equal sign, this is NOT a 
+                        // FASTA mod.
+                        if( first_space_pos < len ) {
+                            // If we've found a space at any point, we consider that the end of the seq-id
+                            pos = first_space_pos;
+                            break;
+                        }
+                        // it's not a FASTA mod and we didn't find any spaces, so just
+                        // keep going as normal, continuing from where we let off at "pos"
+                    }
+                }
+            }
+
+
+            if ( ! (fFastaFlags & CFastaReader::fDisableParseRange) ) {
+                range_len = ParseRange(defline.substr(start, pos - start),
+                                       data.range_start, data.range_end, pMessageListener);
+            }
+
+            auto id_string = defline.substr(start, pos - start - range_len);
+            if (NStr::IsBlank(id_string)) {
+                NCBI_THROW2(CObjReaderParseException, eFormat, 
+                    "Unable to locate sequence id in definition line", 0);
+            }
 
         title_start = pos;
         x_ProcessIDs(id_string,
             info,
-            ids, 
-            pMessageListener);
-
-        hasRange = (range_len>0);
+            data.ids, 
+            pMessageListener,
+            fn_idcheck);
+        data.has_range = (range_len>0);
     }
 
     
     // trim leading whitespace from title (is this appropriate?)
     while (title_start < len
-        &&  isspace((unsigned char) defline[title_start])) {
+        &&  isspace((unsigned char)defline[title_start])) {
         ++title_start;
     }
 
     if (title_start < len) { 
         for (pos = title_start + 1;  pos < len;  ++pos) {
-            if ((unsigned char) defline[pos] < ' ') {
+            if ((unsigned char)defline[pos] < ' ') {
             break;
             }
         }
         // Parse the title elsewhere - after the molecule has been deduced
-        seqTitles.push_back(
+        data.titles.push_back(
             SLineTextAndLoc(
-            defline.substr(title_start, pos - title_start), lineNumber));
+                defline.substr(title_start, pos - title_start), lineNumber));
     }
 }
 
 
 TSeqPos CFastaDeflineReader::ParseRange(
-    const string& s, 
+    const CTempString& s, 
     TSeqPos& start, 
     TSeqPos& end, 
     ILineErrorListener * pMessageListener)
@@ -236,31 +330,22 @@ TSeqPos CFastaDeflineReader::ParseRange(
     }
     --start;
     --end;
-    return s.length() - pos;
+    return TSeqPos(s.length() - pos);
 }
 
 
 void CFastaDeflineReader::x_ProcessIDs(
-    const string& id_string,
+    const CTempString& id_string,
     const SDeflineParseInfo& info,
-    list<CRef<CSeq_id>>& ids,
-    ILineErrorListener* pMessageListener)
+    TIds& ids,
+    ILineErrorListener* pMessageListener,
+    FIdCheck f_id_check)
 {
-
-
-    x_CheckForExcessiveSeqDataInID(
-            id_string,
-            info,
-            pMessageListener);
-
-
     if (info.fBaseFlags & CReaderBase::fAllIdsAsLocal) 
     {
-        if (!x_IsValidLocalID(id_string, info.fFastaFlags)) {
-            NCBI_THROW2(CObjReaderParseException, eFormat, 
-                "'" + id_string + "' is not a valid local ID", 0);
-        }
-        ids.push_back(Ref(new CSeq_id(CSeq_id::e_Local, id_string)));
+        CRef<CSeq_id> pSeqId(new CSeq_id(CSeq_id::e_Local, id_string));
+        ids.push_back(pSeqId);
+        f_id_check(ids, info, pMessageListener);
         return;
     }
 
@@ -272,24 +357,31 @@ void CFastaDeflineReader::x_ProcessIDs(
         flags |= CSeq_id::fParse_RawText;
     }
 
-    string local_id_string = id_string;
+    string local_copy;
+    auto to_parse = id_string;
     if (id_string.find(',') != NPOS &&
         id_string.find('|') == NPOS) {
-            
         const string err_message = 
-            "CFastaReader: Near line " + NStr::NumericToString(info.lineNumber)
+            "Near line " + NStr::NumericToString(info.lineNumber)
             + ", the sequence id string contains 'comma' symbol, which has been replaced with 'underscore' "
             + "symbol. Please correct the sequence id string.";
 
-        x_PostWarning(pMessageListener,
+        s_PostWarning(pMessageListener,
             info.lineNumber,
             err_message,
+            ILineError::eProblem_GeneralParsingError,
             CObjReaderParseException::eFormat);
             
-        local_id_string = NStr::Replace(id_string, ",", "_");
+        local_copy = id_string;
+        for (auto& rit : local_copy)
+            if (rit == ',')
+                rit = '_';
+
+        to_parse = local_copy;
     }
 
-    CSeq_id::ParseIDs(ids, local_id_string, flags);
+    CSeq_id::ParseIDs(ids, to_parse, flags);
+
     ids.remove_if([](CRef<CSeq_id> id_ref){ return NStr::IsBlank(id_ref->GetSeqIdString()); });
     if (ids.empty()) {
         NCBI_THROW2(CObjReaderParseException, eNoIDs,
@@ -301,35 +393,15 @@ void CFastaDeflineReader::x_ProcessIDs(
         x_ConvertNumericToLocal(ids);
     }
 
-
-    for (const auto& id : ids) {
-        if (id->IsLocal() &&
-            !x_IsValidLocalID(*id, info.fFastaFlags)) {
-            NCBI_THROW2(CObjReaderParseException, eInvalidID, 
-             "'" + id->GetLocal().GetStr() + "' is not a valid local ID", 0);
-        }
-    }
-
-    if (x_ExceedsMaxLength(id_string, info.maxIdLength)) {
-        const string err_message =
-            "CFastaReader: Near line " + NStr::NumericToString(info.lineNumber)
-            + ", the sequence ID is too long.  Its length is " + NStr::NumericToString(id_string.length())
-            + " but the max length allowed is "+  NStr::NumericToString(info.maxIdLength)
-            + ".  Please find and correct all sequence IDs that are too long.";
-        
-        x_PostError(pMessageListener, 
-            info.lineNumber,
-            err_message,
-            CObjReaderParseException::eIDTooLong);
-    }
+    f_id_check(ids, info, pMessageListener);
 }
 
 
 bool CFastaDeflineReader::ParseIDs(
-    const string& s, 
+    const CTempString& s,
     const SDeflineParseInfo& info,
     const TIgnoredProblems& ignoredErrors,
-    list<CRef<CSeq_id>>& ids, 
+    TIds& ids, 
     ILineErrorListener* pMessageListener) 
 {
     if (s.empty()) {
@@ -359,18 +431,22 @@ bool CFastaDeflineReader::ParseIDs(
     try {
         if (s.find(',') != NPOS && s.find('|') == NPOS)
         {
-            string temp = NStr::Replace(s, ",", "_");
-            num_ids = CSeq_id::ParseIDs(ids, temp, flags);
+            string local_copy = s;
+            for (auto& ch : local_copy)
+                if (ch == ',')
+                    ch = '_';
+            num_ids = CSeq_id::ParseIDs(ids, local_copy, flags);
 
             const string errMessage = 
-                "CFastaReader: Near line " + NStr::NumericToString(info.lineNumber) 
+                "Near line " + NStr::NumericToString(info.lineNumber) 
                 + ", the sequence contains 'comma' symbol and replaced with 'underscore' "
                 + "symbol. Please find and correct the sequence id.";
 
             if (!ignoreError) {
-                x_PostWarning(pMessageListener, 
+                s_PostWarning(pMessageListener, 
                             info.lineNumber,
                             errMessage,
+                            ILineError::eProblem_GeneralParsingError,
                             CObjReaderParseException::eFormat);
 
             }
@@ -392,23 +468,24 @@ bool CFastaDeflineReader::ParseIDs(
     if (num_ids == 1) {
         if (ids.front()->IsLocal()) 
         {
-            if (!x_IsValidLocalID(*ids.front(), info.fFastaFlags)) {
+            if (!s_IsValidLocalID(*ids.front(), info)) {
                 ids.clear();
                 return false;
             }
         }
         if ( x_ExceedsMaxLength(s, info.maxIdLength) ) {
             const string errMessage =
-                "CFastaReader: Near line " + NStr::NumericToString(info.lineNumber)
+                "Near line " + NStr::NumericToString(info.lineNumber)
                 + ", the sequence ID is too long.  Its length is " + NStr::NumericToString(s.length())
                 + " but the max length allowed is "+  NStr::NumericToString(info.maxIdLength)
                 + ".  Please find and correct all sequence IDs that are too long.";
 
 
             if (!ignoreError) {
-                x_PostError(pMessageListener, 
+                s_PostError(pMessageListener, 
                     info.lineNumber,
                     errMessage,
+                    ILineError::eProblem_GeneralParsingError, 
                     CObjReaderParseException::eIDTooLong);
             }
 
@@ -422,102 +499,24 @@ bool CFastaDeflineReader::ParseIDs(
 }
 
 
-bool CFastaDeflineReader::x_IsValidLocalID(const CSeq_id& id, TFastaFlags fasta_flags) 
-{
-    string id_label;
-    id.GetLabel(&id_label, 0, CSeq_id::eContent);
-    return x_IsValidLocalID(id_label, fasta_flags);
-}
-
-
-bool CFastaDeflineReader::x_IsValidLocalID(const string& id_string,
-    const TFastaFlags fasta_flags)
-{
-    if ( fasta_flags & CFastaReader::fQuickIDCheck) { // check only the first character
-        return CSeq_id::IsValidLocalID(id_string.substr(0,1));
-    }
-    return CSeq_id::IsValidLocalID(id_string);
-}
-
-
 void CFastaDeflineReader::x_ConvertNumericToLocal(
     list<CRef<CSeq_id>>& ids)
 {
     for (auto id : ids) {
         if (id->IsGi()) {
             const TGi gi = id->GetGi();
-            id->SetLocal().SetStr(NStr::NumericToString(gi));
+            id->SetLocal().SetStr() = NStr::NumericToString(gi);
         }
     }
 }
 
 
-void CFastaDeflineReader::x_PostWarning(ILineErrorListener* pMessageListener,
-    const TSeqPos lineNumber,
-    const string& errMessage,
-    const CObjReaderParseException::EErrCode errCode) 
-{
-    x_PostWarning(pMessageListener,
-        lineNumber,
-        errMessage,
-        ILineError::eProblem_GeneralParsingError,
-        errCode);
-}
-
-
-void CFastaDeflineReader::x_PostWarning(ILineErrorListener* pMessageListener, 
-    const TSeqPos lineNumber,
-    const string& errMessage, 
-    const CObjReaderLineException::EProblem problem, 
-    const CObjReaderParseException::EErrCode errCode) 
-{
-    unique_ptr<CObjReaderLineException> pLineExpt(
-        CObjReaderLineException::Create(
-        eDiag_Warning,
-        lineNumber,
-        errMessage, 
-        problem,
-        "", "", "", "",
-        errCode));
-
-    if (!pMessageListener) {
-        LOG_POST_X(1, Warning << pLineExpt->Message());
-        return;
-    }
-
-    if (!pMessageListener->PutError(*pLineExpt)) {
-        throw CObjReaderParseException(DIAG_COMPILE_INFO, 0, errCode, errMessage, lineNumber, eDiag_Warning);
-    }
-}
-
-
-void CFastaDeflineReader::x_PostError(ILineErrorListener* pMessageListener,
-    const TSeqPos lineNumber,
-    const string& errMessage,
-    const CObjReaderParseException::EErrCode errCode) 
-{
-
-    unique_ptr<CObjReaderLineException> pLineExpt(
-        CObjReaderLineException::Create(
-        eDiag_Error,
-        lineNumber,
-        errMessage, 
-        ILineError::eProblem_GeneralParsingError,
-        "", "", "", "",
-        errCode));
-
-    if (!pMessageListener || !pMessageListener->PutError(*pLineExpt)) {
-        throw CObjReaderParseException(DIAG_COMPILE_INFO, 0, errCode, errMessage, lineNumber, eDiag_Error);
-    }
-}
-
-
 bool 
-CFastaDeflineReader::x_ExceedsMaxLength(const string& title,
+CFastaDeflineReader::x_ExceedsMaxLength(const CTempString& title,
     const TSeqPos max_length)
 {
     auto last = title.rfind('|');
-    string substring = (last == NPOS) ? title : title.substr(last+1);
+    auto substring = (last == NPOS) ? title : title.substr(last+1);
 
     return (substring.length() > max_length);
 }
@@ -540,39 +539,142 @@ static bool s_ASCII_IsUnAmbigNuc(unsigned char c)
     }
 }
 
-void 
-CFastaDeflineReader::x_CheckForExcessiveSeqDataInID(
-    const string& id_string,
-    const SDeflineParseInfo& info,
-    ILineErrorListener* pMessageListener) 
+
+void CSeqIdCheck::operator()(const TIds& ids, 
+                             const TInfo& info,
+                             ILineErrorListener* listener) 
 {
+    if (ids.empty()) {
+        return;
+    }
+
+    x_CheckForExcessiveSeqData(*(ids.back()), info, listener);
+
+    for (const auto& pId : ids) {
+        if (pId->IsLocal() && 
+            !x_IsValidLocalID(*pId, info)) {
+            NCBI_THROW2(CObjReaderParseException, eInvalidID, 
+                "'" + pId->GetSeqIdString() + "' is not a valid local ID", 0);
+        }
+        x_CheckIDLength(*pId, info, listener);
+    }
+}
+
+
+bool CSeqIdCheck::x_IsValidLocalID(const CSeq_id& id, 
+                                 const TInfo& info)
+{
+    return s_IsValidLocalID(id, info);
+}
+
+
+void CSeqIdCheck::x_PostIDLengthError(const size_t id_length,
+                                      const string& type_string,
+                                      const size_t max_length,
+                                      const TInfo& info,
+                                      ILineErrorListener* listener)
+{
+
+    const string& err_message =
+        "Near line " + NStr::NumericToString(info.lineNumber) +
+        + ", the " + type_string + " is too long.  Its length is " + NStr::NumericToString(id_length)
+        + " but the maximum allowed " + type_string + " length is "+  NStr::NumericToString(max_length)
+        + ".  Please find and correct all " + type_string + "s that are too long.";
+
+    s_PostError(listener, 
+        info.lineNumber,
+        err_message,
+        ILineError::eProblem_GeneralParsingError,
+        CObjReaderParseException::eIDTooLong);
+
+}
+
+
+void CSeqIdCheck::x_CheckIDLength(const CSeq_id& id, 
+                                    const TInfo& info,
+                                    ILineErrorListener* listener) 
+{
+    if (id.IsLocal()) {
+        if (id.GetLocal().IsStr() &&
+            id.GetLocal().GetStr().length() > CFastaDeflineReader::s_MaxLocalIDLength) {
+            x_PostIDLengthError(id.GetLocal().GetStr().length(),
+                                "local id", CFastaDeflineReader::s_MaxLocalIDLength,
+                                info, listener);
+        }
+        return;
+    }
+
+
+    if (id.IsGeneral()) {
+        if (id.GetGeneral().IsSetTag() &&
+            id.GetGeneral().GetTag().IsStr()) {
+            const auto length = id.GetGeneral().GetTag().GetStr().length();
+            if (length > CFastaDeflineReader::s_MaxGeneralTagLength) {
+                x_PostIDLengthError(length,
+                                    "general id string", CFastaDeflineReader::s_MaxGeneralTagLength,
+                                    info, listener);
+            }
+        }
+        return;
+    }
+
+
+   auto pTextId = id.GetTextseq_Id();
+   if (pTextId &&
+       pTextId->IsSetAccession()) {
+        const auto length = pTextId->GetAccession().length();
+        if (length > CFastaDeflineReader::s_MaxAccessionLength) {
+            x_PostIDLengthError(length,
+                                "accession", CFastaDeflineReader::s_MaxAccessionLength, 
+                                info, listener);
+        }
+   }
+}
+
+
+void CSeqIdCheck::x_CheckForExcessiveSeqData(const CSeq_id& id,
+                                               const TInfo& info,
+                                               ILineErrorListener* listener)
+{
+    const auto& id_string = id.GetSeqIdString();
+
     const TSeqPos kWarnNumNucCharsAtEnd = 20;
     const TSeqPos kWarnNumAminoAcidCharsAtEnd = 50;
-
+    const TSeqPos kErrNumNucCharsAtEnd = 25;
 
     const bool assume_prot = (info.fFastaFlags & CFastaReader::fAssumeProt);
 
     if (!assume_prot && id_string.length() > kWarnNumNucCharsAtEnd) {
         TSeqPos numNucChars = 0;
-        for (auto rit=id_string.crbegin(); rit!=id_string.crend(); ++rit) {
-            if (!s_ASCII_IsUnAmbigNuc(*rit) && (*rit != 'N')) {
+        for (size_t i = id_string.size(); i>0; i--) {
+            const auto ch = id_string[i - 1];
+            if (!s_ASCII_IsUnAmbigNuc(ch) && (ch != 'N')) {
                 break;
             }
             ++numNucChars;
         }
+
         if (numNucChars > kWarnNumNucCharsAtEnd) {
             const string err_message = 
             "Fasta Reader: sequence id ends with " +
             NStr::NumericToString(numNucChars) +
             " valid nucleotide characters. " +
             " Was the sequence accidentally placed in the definition line?";    
-        
-            x_PostWarning(pMessageListener,
-                info.lineNumber,
-                err_message,
-                ILineError::eProblem_UnexpectedNucResidues,
-                CObjReaderParseException::eFormat);
+       
+            if (numNucChars > kErrNumNucCharsAtEnd) {
+                s_PostError(listener,
+                            info.lineNumber,
+                            err_message,
+                            ILineError::eProblem_UnexpectedNucResidues,
+                            CObjReaderParseException::eFormat);
             
+            } else{ 
+                s_PostWarning(listener,
+                              info.lineNumber,
+                              err_message,
+                              ILineError::eProblem_UnexpectedNucResidues,
+                              CObjReaderParseException::eFormat);
+            }
             return;        
         }
     }
@@ -581,8 +683,8 @@ CFastaDeflineReader::x_CheckForExcessiveSeqDataInID(
     // Check for Aa sequence
     if (!assume_nuc && id_string.length() > kWarnNumAminoAcidCharsAtEnd) {
         TSeqPos numAaChars = 0;
-        for (auto rit=id_string.crbegin(); rit!=id_string.crend(); ++rit) {
-            const auto ch = *rit;
+        for (size_t i = id_string.size(); i>0; i--) {
+            const auto ch = id_string[i - 1];
             if ( !(ch >= 'A' && ch <= 'Z')  &&
                  !(ch >= 'a' && ch <= 'z') ) {
                 break;
@@ -596,11 +698,11 @@ CFastaDeflineReader::x_CheckForExcessiveSeqDataInID(
             " valid amino-acid characters. " +
             " Was the sequence accidentally placed in the definition line?";    
         
-            x_PostWarning(pMessageListener,
-                info.lineNumber,
-                err_message,
-                ILineError::eProblem_UnexpectedAminoAcids,
-                CObjReaderParseException::eFormat);
+            s_PostWarning(listener,
+                          info.lineNumber,
+                          err_message,
+                          ILineError::eProblem_UnexpectedAminoAcids,
+                          CObjReaderParseException::eFormat);
         }
     }
 }
@@ -608,7 +710,7 @@ CFastaDeflineReader::x_CheckForExcessiveSeqDataInID(
 
 
 bool 
-CFastaDeflineReader::x_ExcessiveSeqDataInTitle(const string& title, 
+CFastaDeflineReader::x_ExcessiveSeqDataInTitle(const CTempString& title, 
     TFastaFlags fasta_flags) 
 {
     if (fasta_flags & CFastaReader::fAssumeProt) {
@@ -622,8 +724,9 @@ CFastaDeflineReader::x_ExcessiveSeqDataInTitle(const string& title,
     // Check for nuc sequence
     if (title.length() > kWarnNumNucCharsAtEnd) {
         TSeqPos numNucChars = 0;
-        for (auto rit=title.crbegin(); rit!=title.crend(); ++rit) {
-            if (!s_ASCII_IsUnAmbigNuc(*rit) && (*rit != 'N')) {
+        for (size_t i = title.size(); i>0; i--) {
+            const auto ch = title[i-1];
+            if (!s_ASCII_IsUnAmbigNuc(ch) && (ch != 'N')) {
                 break;
             }
             ++numNucChars;
@@ -636,8 +739,8 @@ CFastaDeflineReader::x_ExcessiveSeqDataInTitle(const string& title,
     // Check for Aa sequence
     if (title.length() > kWarnNumAminoAcidCharsAtEnd) {
         TSeqPos numAaChars = 0;
-        for (auto rit=title.crbegin(); rit!=title.crend(); ++rit) {
-            const auto ch = *rit;
+        for (size_t i=title.size(); i>0; i--) {
+            const auto ch = title[i-1];
             if ( !(ch >= 'A' && ch <= 'Z')  &&
                  !(ch >= 'a' && ch <= 'z') ) {
                 break;
@@ -683,7 +786,10 @@ CRef<CSeq_id> CSeqIdGenerator::GenerateID(const bool advance)
 CRef<CSeq_id> CSeqIdGenerator::GenerateID(const string& defline, const bool advance)
 {
     CRef<CSeq_id> seq_id(new CSeq_id);
-    int n = advance ? m_Counter.Add(1) - 1 : m_Counter.Get();
+    auto n = m_Counter.load();
+    if (advance)
+        m_Counter++;
+
     if (m_Prefix.empty()  &&  m_Suffix.empty()) {
         seq_id->SetLocal().SetId(n);
     } else {
