@@ -1,4 +1,4 @@
-/*  $Id: gff2_data.cpp 546403 2017-09-18 15:03:56Z ivanov $
+/*  $Id: gff2_data.cpp 561166 2018-04-03 13:59:57Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -396,13 +396,13 @@ CRef<CSeq_loc> CGff2Record::GetSeqLoc(
 {
     CRef<CSeq_loc> pLocation(new CSeq_loc);
     pLocation->SetInt().SetId(*GetSeqId(flags, seqidresolve));
-    pLocation->SetInt().SetFrom(SeqStart());
-    pLocation->SetInt().SetTo(SeqStop());
-        if (IsSetStrand()) {
+    pLocation->SetInt().SetFrom(static_cast<TSeqPos>(SeqStart()));
+    pLocation->SetInt().SetTo(static_cast<TSeqPos>(SeqStop()));
+    if (IsSetStrand()) {
         pLocation->SetInt().SetStrand(Strand());
-        }
-    return pLocation;
     }
+    return pLocation;
+}
 
 //  ----------------------------------------------------------------------------
 string CGff2Record::xNormalizedAttributeKey(
@@ -609,11 +609,16 @@ bool CGff2Record::UpdateFeature(
     if (subtype == CSeqFeatData::eSubtype_gene) {
         return true;
     }
+    auto recType = Type();
+    NStr::ToLower(recType);
     const CSeq_loc& target = pFeature->GetLocation();
     CRef<CSeq_loc> pAddLoc = GetSeqLoc(flags, seqidresolve);
 
     if (target.IsInt()  &&  target.GetInt().GetFrom() <= SeqStart()  &&
             target.GetInt().GetTo() >= SeqStop() ) {
+        if (recType == "start_codon"  ||  recType == "stop_codon") {
+            return true;
+        }
         // indicates current feature location is a placeholder interval to be
         //  totally overwritten by the constituent sub-intervals
         pFeature->SetLocation().SetMix().AddSeqLoc(*pAddLoc);
@@ -628,9 +633,16 @@ bool CGff2Record::UpdateFeature(
             pFeature->SetLocation().SetMix().AddSeqLoc(*pOld);
         }
     }
-    if (!this->xUpdateFeatureData(flags, pFeature)) {
+    if (!xUpdateFeatureData(flags, pFeature)) {
         return false;
     }
+    if (subtype == CSeqFeatData::eSubtype_cdregion  &&  recType == "cds") {
+        string cdsId;
+        GetAttribute("ID", cdsId);
+        if (!cdsId.empty()) {
+            pFeature->AddOrReplaceQualifier("ID", cdsId);
+        }
+    }
     return true;
 }
 
@@ -893,17 +905,22 @@ bool CGff2Record::x_MigrateAttributes(
 
     it = attrs_left.find("inference");
     if (it != attrs_left.end()) {
+        auto inferenceVal = it->second;
         const string strInferenceDefault(
             "non-experimental evidence, no additional details recorded" );
-       string value = xNormalizedAttributeValue(it->second);
+       auto value = xNormalizedAttributeValue(inferenceVal);
         if (value == strInferenceDefault) {
             pFeature->SetExp_ev(CSeq_feat::eExp_ev_not_experimental);
         }
         else {
-            CRef<CGb_qual> pQual(new CGb_qual);
-            pQual->SetQual("inference");
-            pQual->SetVal(value);
-            pFeature->SetQual().push_back(pQual);
+            vector<string> inferenceVals;
+            NStr::Split(inferenceVal, ",", inferenceVals);
+            for (auto val: inferenceVals) {
+                CRef<CGb_qual> pQual(new CGb_qual);
+                pQual->SetQual("inference");
+                pQual->SetVal(xNormalizedAttributeValue(val));
+                pFeature->SetQual().push_back(pQual);
+            }
         }
         attrs_left.erase(it);
     }
@@ -1204,6 +1221,8 @@ bool CGff2Record::xInitFeatureData(
     CRef<CSeq_feat> pFeature ) const
 //  ----------------------------------------------------------------------------
 {
+    bool invalidFeaturesToRegion = !(flags & CGff2Reader::fGenbankMode);
+
     string gbkey;
     if (GetAttribute("gbkey", gbkey)) {
         if (gbkey == "Src") {
@@ -1218,14 +1237,25 @@ bool CGff2Record::xInitFeatureData(
             if (qual_type == "other") {
                 qual_type = "ncRNA";
             }
-            if (CSoMap::SoTypeToFeature(qual_type, *pFeature)) {
+            if (CSoMap::SoTypeToFeature(
+                    qual_type, *pFeature, invalidFeaturesToRegion)) {
                 return true;
             }
         }
     }
-
-    if (!CSoMap::SoTypeToFeature(Type(), *pFeature)) {
-        return false;
+    auto recognizedType = Type();
+    if (recognizedType == "start_codon"  || recognizedType == "stop_codon") {
+        recognizedType = "cds";
+    }
+    if (!CSoMap::SoTypeToFeature(
+            recognizedType, *pFeature, invalidFeaturesToRegion)) {
+        AutoPtr<CObjReaderLineException> pErr(
+            CObjReaderLineException::Create(
+                eDiag_Error,
+                0,
+                string("Bad data line: Invalid feature type \"") + recognizedType + "\"",
+                ILineError::eProblem_UnrecognizedFeatureName) );
+        pErr->Throw();
     }
 
     CSeqFeatData::ESubtype subtype = pFeature->GetData().GetSubtype();
