@@ -1,4 +1,4 @@
-/*  $Id: gff2_reader.cpp 542725 2017-08-02 11:32:01Z ivanov $
+/*  $Id: gff2_reader.cpp 560241 2018-03-20 17:57:35Z foleyjp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -185,43 +185,50 @@ CGff2Reader::ReadSeqAnnot(
                 &&  !NStr::StartsWith(line, "##sequence-region") ) {
             continue;
         }
-        if (xIsTrackLine(line)) {
-            if (!mCurrentFeatureCount) {
-                xParseTrackLine(line, pEC);
+
+        try {
+            if (xIsTrackLine(line)) {
+                if (!mCurrentFeatureCount) {
+                    xParseTrackLine(line, pEC);
+                    continue;
+                }
+                m_PendingLine = line;
+                break;
+            }
+            if (xIsTrackTerminator(line)) {
+                if (!mCurrentFeatureCount) {
+                    xParseTrackLine("track", pEC);
+                    continue;
+                }
+                break;
+            }
+
+            if (xNeedsNewSeqAnnot(line)) {
+                break;
+            }
+           
+            if (xParseBrowserLine(line, pAnnot, pEC)) {
                 continue;
             }
-            m_PendingLine = line;
-            break;
-        }
-        if (xIsTrackTerminator(line)) {
-            if (!mCurrentFeatureCount) {
-                xParseTrackLine("track", pEC);
-                continue;
+
+            if (!xIsCurrentDataType(line)) {
+                xUngetLine(lr);
+                break;
             }
-            //if (line == "###") {
-            //    m_PendingLine = "track";
-            //}
-            break;
-        }
 
-        if (xParseBrowserLine(line, pAnnot, pEC)) {
-            continue;
-        }
+            if ( CGff2Reader::IsAlignmentData(line)) {
+                if ((m_iFlags & fGenbankMode)  ||  
+                        x_ParseAlignmentGff(line, id_list, alignments)) {
+                    continue;
+                }
+            }
 
-        if (!xIsCurrentDataType(line)) {
-            xUngetLine(lr);
-            break;
-        }
-
-        if ( CGff2Reader::IsAlignmentData(line)) {
-            if ((m_iFlags && fGenbankMode)  ||  
-                    x_ParseAlignmentGff(line, id_list, alignments)) {
+            if (xParseFeature(line, pAnnot, pEC)) {
                 continue;
             }
         }
-
-        if (xParseFeature(line, pAnnot, pEC)) {
-            continue;
+        catch (CObjReaderLineException& err) {
+            ProcessError(err, pEC);
         }
     }
 
@@ -256,18 +263,78 @@ void
 CGff2Reader::ReadSeqAnnots(
     TAnnots& annots,
     ILineReader& lr,
-    ILineErrorListener* pEC )
+    ILineErrorListener* pEC)
 //  ----------------------------------------------------------------------------
 {
     xProgressInit(lr);
-    CRef<CSeq_annot> pAnnot = ReadSeqAnnot(lr, pEC);
-    while (pAnnot) {
-        annots.push_back(pAnnot);
-        pAnnot = ReadSeqAnnot(lr, pEC);
+    while (!lr.AtEOF()) {
+        CRef<CSeq_annot> pNext = this->ReadSeqAnnot(lr, pEC);
+        if (pNext) {
+            annots.push_back(pNext);
+        }
     }
     return;
-}
 
+    string line;
+    while (xGetLine(lr, line)) {
+        if (IsCanceled()) {
+            AutoPtr<CObjReaderLineException> pErr(
+                CObjReaderLineException::Create(
+                eDiag_Info,
+                0,
+                "Reader stopped by user.",
+                ILineError::eProblem_ProgressInfo));
+            ProcessError(*pErr, pEC);
+            annots.clear();
+            return;
+        }
+        xReportProgress(pEC);
+        if ( xParseStructuredComment(line) ) {
+            continue;
+        }
+
+        try {
+            if (xIsTrackTerminator(line)) {
+                continue;
+            }
+            if (x_ParseBrowserLineGff(line, m_CurrentBrowserInfo)) {
+                continue;
+            }
+            if (xIsTrackLine(line)) {
+                //completely ignore in Genbank mode
+                if (m_iFlags & CGff2Reader::fGenbankMode) {
+                    continue;
+                }
+                if (!annots.empty()) {
+                    xPostProcessAnnot(annots.back(), pEC);
+                }
+                xParseTrackLine(line, pEC);
+                CRef< CSeq_annot > pAnnot( new CSeq_annot );
+                annots.push_back(pAnnot);
+                continue;
+            }
+            if (xNeedsNewSeqAnnot(line)) {
+                if (!annots.empty()) {
+                    xPostProcessAnnot(annots.back(), pEC);
+                }
+                mCurrentFeatureCount = 0;
+                mParsingAlignment = false;
+                CRef< CSeq_annot > pAnnot( new CSeq_annot );
+                annots.push_back(pAnnot);
+                continue;
+            }                
+            if (x_ParseFeatureGff(line, annots, pEC)) {
+                continue;
+            }
+        }
+        catch(CObjReaderLineException& err) {
+            err.SetLineNumber(m_uLineNumber);
+        }
+    }
+    if (!annots.empty()) {
+        xPostProcessAnnot(annots.back(), pEC);
+    }
+}
 
 //  ----------------------------------------------------------------------------                
 CRef< CSeq_entry >
@@ -320,7 +387,7 @@ void CGff2Reader::xPostProcessAnnot(
 //  ----------------------------------------------------------------------------
 {
     xAssignAnnotId(pAnnot);
-    if (!(m_iFlags && fGenbankMode)) {
+    if (!(m_iFlags & fGenbankMode)) {
         //xAssignTrackData(pAnnot);
         xAddConversionInfo(pAnnot, pEC);
         xGenerateParentChildXrefs(pAnnot);
@@ -338,15 +405,14 @@ void CGff2Reader::xAssignAnnotId(
     }
 
     string annotId(givenId);
-    if (annotId.empty()  &&  pAnnot->GetData().IsFtable()) {
-        const CSeq_annot::TData::TFtable ftable = pAnnot->GetData().GetFtable();
-        if (ftable.empty()) {
-            return;
-        }
-        const CSeq_feat& front = *ftable.front();
-        annotId = front.GetLocation().GetId()->GetSeqIdString(true);
+    if (annotId.empty() && 
+        (!(m_iFlags & fGenbankMode)) &&
+        m_pTrackDefaults) {
+        annotId = m_pTrackDefaults->Name();
     }
-
+    if (annotId.empty()) {
+        return;
+    }    
     CRef< CAnnot_id > pAnnotId(new CAnnot_id);
     pAnnotId->SetLocal().SetStr(annotId);
     pAnnot->SetId().push_back(pAnnotId);   
@@ -2021,5 +2087,27 @@ bool CGff2Reader::xIsIgnoredFeatureId(
     return false;
 }
 
+//  ---------------------------------------------------------------------------
+bool
+CGff2Reader::xNeedsNewSeqAnnot(
+    const string& line)
+//  ---------------------------------------------------------------------------
+{
+    if (m_iFlags & CGff2Reader::fGenbankMode) {
+        vector<string> columns;
+        NStr::Split(line, "\t ", columns, NStr::eMergeDelims);
+        string seqId = columns[0];
+        if (m_CurrentSeqId == seqId) {
+            return false;
+        }
+        m_CurrentSeqId = seqId;
+        m_PendingLine = line;
+        return true;
+    }
+    //in normal mode, segmentation is driven by track line markers rather than 
+    // Seq-ids, and that test has already been done
+    return false;
+}
+
 END_objects_SCOPE
 END_NCBI_SCOPE
