@@ -1,4 +1,4 @@
-/*  $Id: gtf_reader.cpp 545908 2017-09-12 14:24:34Z ivanov $
+/*  $Id: gtf_reader.cpp 552058 2017-11-28 19:16:52Z ludwigf $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -212,100 +212,6 @@ CGtfReader::~CGtfReader()
 {
 }
 
-//  ---------------------------------------------------------------------------                       
-void
-CGtfReader::ReadSeqAnnots(
-    TAnnots& annots,
-    ILineReader& lr,
-    ILineErrorListener* pEC)
-//  ----------------------------------------------------------------------------
-{
-    xProgressInit(lr);
-
-    string line;
-    while (xGetLine(lr, line)) {
-        if (IsCanceled()) {
-            AutoPtr<CObjReaderLineException> pErr(
-                CObjReaderLineException::Create(
-                eDiag_Info,
-                0,
-                "Reader stopped by user.",
-                ILineError::eProblem_ProgressInfo));
-            ProcessError(*pErr, pEC);
-            annots.clear();
-            return;
-        }
-        xReportProgress(pEC);
-        if ( xParseStructuredComment(line) ) {
-            continue;
-        }
-
-        try {
-            if (xIsTrackTerminator(line)) {
-                continue;
-            }
-            if (x_ParseBrowserLineGff(line, m_CurrentBrowserInfo)) {
-                continue;
-            }
-            if (xIsTrackLine(line)) {
-                if (!annots.empty()) {
-                    xPostProcessAnnot(annots.back(), pEC);
-                }
-                xParseTrackLine(line, pEC);
-                CRef< CSeq_annot > pAnnot( new CSeq_annot );
-                annots.push_back(pAnnot);
-                continue;
-            }
-            if (xNeedsNewSeqAnnot(line)) {
-                if (!annots.empty()) {
-                    xPostProcessAnnot(annots.back(), pEC);
-                }
-                CRef< CSeq_annot > pAnnot( new CSeq_annot );
-                annots.push_back(pAnnot);
-                continue;
-            }                
-            if (x_ParseFeatureGff(line, annots, pEC)) {
-                continue;
-            }
-        }
-        catch(CObjReaderLineException& err) {
-            err.SetLineNumber(m_uLineNumber);
-        }
-    }
-    if (!annots.empty()) {
-        xPostProcessAnnot(annots.back(), pEC);
-    }
-}
-
-//  ---------------------------------------------------------------------------
-bool
-CGtfReader::xNeedsNewSeqAnnot(
-    const string& line)
-//  ---------------------------------------------------------------------------
-{
-    vector<string> columns;
-    NStr::Split(line, "\t ", columns, NStr::eMergeDelims);
-    string seqId = columns[0];
-    if (m_CurrentSeqId == seqId) {
-        return false;
-    }
-    m_CurrentSeqId = seqId;
-    m_PendingLine = line;
-    return true;
-}
-
-//  --------------------------------------------------------------------------- 
-void
-CGtfReader::ReadSeqAnnots(
-    TAnnots& annots,
-    CNcbiIstream& istr,
-    ILineErrorListener* pMessageListener )
-//  ---------------------------------------------------------------------------
-{
-    CStreamLineReader lr( istr );
-    ReadSeqAnnots( annots, lr, pMessageListener );
-}
-
 //  ----------------------------------------------------------------------------
 bool CGtfReader::x_UpdateAnnotFeature(
     const CGff2Record& gff,
@@ -376,7 +282,6 @@ bool CGtfReader::x_UpdateAnnotCds(
         if (!x_CreateParentGene(gff, pAnnot)  ||  !x_FindParentGene(gff, pGene)) {
             return false;
         }
-        needXref = true;
     }
     else {
         if (!x_MergeParentGene(gff, pGene)) {
@@ -396,7 +301,6 @@ bool CGtfReader::x_UpdateAnnotCds(
         if (!x_CreateParentCds(gff, pAnnot)  ||  !x_FindParentCds(gff, pCds)) {
             return false;
         }
-        needXref = true;
     }
     else {
         //
@@ -406,7 +310,7 @@ bool CGtfReader::x_UpdateAnnotCds(
             return false;
         }
     }
-
+        
     if ( x_CdsIsPartial( gff ) ) {
         CRef<CSeq_feat> pParent;
         if ( x_FindParentMrna( gff, pParent ) ) {
@@ -568,11 +472,9 @@ bool CGtfReader::x_CreateMrnaXrefs(
     pXrefToChild->SetId( pFeature->SetId() );
     pParent->SetXref().push_back( pXrefToChild );
 
-    if (m_iFlags & CGtfReader::fGenerateChildXrefs) {
-        CRef< CSeqFeatXref > pXrefToParent( new CSeqFeatXref );
-        pXrefToParent->SetId( pParent->SetId() );    
-        pFeature->SetXref().push_back( pXrefToParent );
-    }
+    CRef< CSeqFeatXref > pXrefToParent( new CSeqFeatXref );
+    pXrefToParent->SetId( pParent->SetId() );    
+    pFeature->SetXref().push_back( pXrefToParent );
 
     return true;
 }
@@ -798,6 +700,8 @@ bool CGtfReader::x_CreateParentCds(
         return false;
     }
 
+    m_CdsMap[ s_FeatureKey( gff ) ] = pFeature;
+
     if ( ! x_FeatureSetDataCDS( gff, pFeature ) ) {
         return false;
     }
@@ -817,8 +721,6 @@ bool CGtfReader::x_CreateParentCds(
         return false;
     }
 
-    m_CdsMap[ s_FeatureKey( gff ) ] = pFeature;
-
     return xAddFeatureToAnnot( pFeature, pAnnot );
 }
 
