@@ -1,4 +1,4 @@
-/*  $Id: idmapper_config.cpp 501592 2016-05-17 15:27:30Z grichenk $
+/*  $Id: idmapper_config.cpp 536284 2017-05-17 12:39:01Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -205,7 +205,7 @@ CIdMapperConfig::AddMapEntry(
 //  ============================================================================
 {
     vector<string> columns;
-    NStr::Split(strLine, " \t", columns, NStr::fSplit_MergeDelims);
+    NStr::Split(strLine, " \t", columns, NStr::fSplit_MergeDelimiters | NStr::fSplit_Truncate);
     
     //sanity check: two or three columns. If three columns, the last better be
     //integer
