@@ -1,4 +1,4 @@
-/*  $Id: rm_reader.cpp 472138 2015-07-07 16:07:55Z grichenk $
+/*  $Id: rm_reader.cpp 515707 2016-10-05 12:32:29Z ivanov $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -890,7 +890,7 @@ bool CRepeatMaskerReader::ParseRecord(const string& record, SRepeatRegion& mask_
 
     string line = NStr::TruncateSpaces( record );
     list< string > values;
-    if ( NStr::Split( line, " \t", values ).size() < MIN_VALUE_COUNT ) {
+    if ( NStr::Split( line, " \t", values, NStr::fSplit_Tokenize ).size() < MIN_VALUE_COUNT ) {
         return false;
     }
 
