@@ -1,4 +1,4 @@
-/*  $Id: choicestr.cpp 532212 2017-04-03 13:24:13Z gouriano $
+/*  $Id: choicestr.cpp 547919 2017-10-05 17:48:53Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -40,6 +40,7 @@
 #include "stdstr.hpp"
 #include "code.hpp"
 #include "srcutil.hpp"
+#include "stlstr.hpp"
 #include <serial/serialdef.hpp>
 #include "statictype.hpp"
 #include "unitype.hpp"
@@ -138,7 +139,7 @@ bool CChoiceTypeStrings::x_IsNullType(TVariants::const_iterator i) const
     return (dynamic_cast<CNullTypeStrings*>(i->type.get()) != 0);
 }
 
-bool CChoiceTypeStrings::x_IsNullWithAttlist(TVariants::const_iterator i) const
+bool CChoiceTypeStrings::x_IsNullWithAttlist(TVariants::const_iterator i, string& memname) const
 {
     if (i->dataType) {
         const CDataType* resolved = i->dataType->Resolve();
@@ -147,6 +148,7 @@ bool CChoiceTypeStrings::x_IsNullWithAttlist(TVariants::const_iterator i) const
             if (typeStr) {
                 ITERATE ( TMembers, ir, typeStr->m_Members ) {
                     if (ir->simple) {
+                        memname = ir->cName;
                         return CClassTypeStrings::x_IsNullType(ir);
                     }
                 }
@@ -920,8 +922,9 @@ void CChoiceTypeStrings::GenerateClassCode(CClassCode& code,
             string rType = methodPrefix + tType;
 #endif
             CTypeStrings::EKind kind = i->type->GetKind();
+            string extname;
             bool isNull = x_IsNullType(i);
-            bool isNullWithAtt = x_IsNullWithAttlist(i);
+            bool isNullWithAtt = x_IsNullWithAttlist(i, extname);
 
             if (!CClassCode::GetDoxygenComments()) {
                 if (!isNull) {
@@ -1184,7 +1187,7 @@ void CChoiceTypeStrings::GenerateClassCode(CClassCode& code,
                     if (isNullWithAtt) {
                         code.Methods(inl) <<
                             "    "<<rType<<"& value = "<<memberRef<<";\n" <<
-                            "    value.Set"<<i->cName<<"();\n" <<
+                            "    value.Set"<<extname<<"();\n" <<
                             "    return value;\n";
                     } else {
                         code.Methods(inl) <<
@@ -1555,9 +1558,11 @@ void CChoiceTypeStrings::GenerateClassCode(CClassCode& code,
             if (i->dataType && i->dataType->GetDataMember() && i->dataType->GetDataMember()->Optional()) {
                 methods << "->SetOptional()";
             }
+#if 0
             if (i->noPrefix) {
                 methods << "->SetNoPrefix()";
             }
+#endif
             if (i->attlist) {
                 methods << "->SetAttlist()";
             }
@@ -1601,6 +1606,34 @@ void CChoiceTypeStrings::GenerateClassCode(CClassCode& code,
                     methods << ")";
                 }
             }
+            if (!DataTool().IsSetCodeGenerationStyle(CDataTool::eNoRestrictions) && i->dataType && i->dataType->GetDataMember()) {
+                const list<CMemberFacet>& con = i->dataType->GetDataMember()->GetRestrictions();
+                if (!con.empty()) {
+                    for (const CMemberFacet& c : con) {
+                        ESerialFacet ct = c.GetType();
+                        if (ct == ESerialFacet::eInclusiveMinimum || ct == ESerialFacet::eExclusiveMinimum ||
+                            ct == ESerialFacet::eInclusiveMaximum || ct == ESerialFacet::eExclusiveMaximum ||
+                            ct == ESerialFacet::eMultipleOf)
+                        {
+                            const CTemplate1TypeStrings* tt = dynamic_cast<const CTemplate1TypeStrings*>(i->type.get());
+                            if (tt) {
+                                methods << "->RestrictV<" << tt->GetArg1Type()->GetCType(GetNamespace()) << ">(";
+                            } else {
+                                methods << "->RestrictV<decltype(" << "m_"<<i->cName << ")>(";
+                            }
+                        } else {
+                            methods << "->Restrict(";
+                        }
+                        methods << GetFacetString(ct) << ",";
+                        if (ct == ESerialFacet::ePattern) {
+                            methods << "\"" << c.GetValue() << "\"";
+                        } else {
+                            methods << c.GetValue();
+                        }
+                        methods << ")";
+                    }
+                }
+            }
             methods << ";\n";
         }
     }
