@@ -1,4 +1,4 @@
-/*  $Id: dtdaux.cpp 534447 2017-04-27 14:18:47Z gouriano $
+/*  $Id: dtdaux.cpp 547690 2017-10-03 13:59:36Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -289,6 +289,8 @@ DTDElement::DTDElement(void)
     m_Named= false;
     m_Qualified = false;
     m_Nillable = false;
+    m_GlobalType = false;
+    m_GlobalGroup = false;
 }
 
 DTDElement::DTDElement(const DTDElement& other)
@@ -313,8 +315,11 @@ DTDElement& DTDElement::operator=(const DTDElement& other)
     m_Named    = other.m_Named;
     m_Qualified= other.m_Qualified;
     m_Nillable = other.m_Nillable;
+    m_GlobalType = other.m_GlobalType;
+    m_GlobalGroup = other.m_GlobalGroup;
     m_Comments = other.m_Comments;
     m_AttribComments = other.m_AttribComments;
+    m_Restrictions = other.m_Restrictions;
     return *this;
 }
 
@@ -334,6 +339,9 @@ int DTDElement::GetSourceLine(void) const
 void DTDElement::SetName(const string& name)
 {
     m_Name = name;
+    if (!IsEmbedded()) {
+        SetNamed();
+    }
 }
 const string& DTDElement::GetName(void) const
 {
