@@ -1,4 +1,4 @@
-/*  $Id: generate.cpp 485907 2015-11-30 14:26:20Z gouriano $
+/*  $Id: generate.cpp 554977 2018-01-11 14:18:53Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -761,7 +761,7 @@ void CCodeGenerator::GenerateModuleHPP(const string& path, list<string>& generat
 {
     set<string> modules;
     string module_name, current_module, filename, hppDefine;
-    auto_ptr<CDelayedOfstream> out;
+    unique_ptr<CDelayedOfstream> out;
     CNamespace ns;
 
     bool isfound = false;
@@ -844,7 +844,7 @@ void CCodeGenerator::GenerateModuleCPP(const string& path, list<string>& generat
 {
     set<string> modules;
     string module_name, current_module, filename, hppDefine;
-    auto_ptr<CDelayedOfstream> out;
+    unique_ptr<CDelayedOfstream> out;
     CNcbiOstrstream out_inc;
     CNcbiOstrstream out_code;
     CNamespace ns;
