@@ -1,4 +1,4 @@
-/*  $Id: jsdparser.cpp 534655 2017-05-01 13:02:07Z gouriano $
+/*  $Id: jsdparser.cpp 569883 2018-08-30 16:59:22Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -109,6 +109,8 @@ void JSDParser::ParseMemberDefinition(DTDElement* owner)
     item.SetNamed();
     if (owner) {
         item.SetEmbedded();
+    } else {
+        item.SetGlobalType(true);
     }
     TToken tok = GetNextToken();
     if (tok == K_BEGIN_OBJECT) {
@@ -125,9 +127,14 @@ void JSDParser::ParseArrayContent(DTDElement& node)
     string node_id = NStr::Join(m_URI,"/");
     DTDElement& item = m_MapElement[node_id];
     item.SetOccurrence(DTDElement::eZeroOrMore);
-    ParseNode(item);
     item.SetEmbedded();
+    ParseNode(item);
 	AddElementContent(node,node_id);
+    if (!item.IsNamed()) {
+        for( const CMemberFacet& f : item.GetRestrictions()) {
+            node.AddFacet(f);
+        }
+    }
 }
 
 void JSDParser::ParseNode(DTDElement& node)
@@ -136,6 +143,9 @@ void JSDParser::ParseNode(DTDElement& node)
     TToken tok;
     bool is_object = false;
     bool has_additional_prop = false;//true;
+    string min_value, max_value;
+    bool excl_min = false, excl_max = false;
+
     node.SetSourceLine( Lexer().CurrentLine());
     for (tok = GetNextToken(); tok != K_END_OBJECT; tok = GetNextToken()) {
         if (tok == K_KEY) {
@@ -146,6 +156,7 @@ void JSDParser::ParseNode(DTDElement& node)
                 } else if (key == "title") {
                     if (!node.IsNamed()) {
                         node.SetName(Value());
+                        node.SetNamed();
                     }
                 } else if (key == "type") {
                     if (Value() == "string") {
@@ -171,12 +182,64 @@ void JSDParser::ParseNode(DTDElement& node)
                 } else if (key == "$ref") {
                     node.SetTypeName(Value());
                     node.SetType(DTDElement::eAlias);
+                } else if (key == "pattern") {
+                    node.AddFacet( CMemberFacet( ESerialFacet::ePattern, Value()));
                 } else {
                     ERR_POST_X(8, Warning << GetLocation() << "Unsupported property: " << key);
                 }
             } else if (tok == T_NUMBER) {
                 if (key == "minItems") {
                     AdjustMinOccurence(node, NStr::StringToInt(Value()));
+                } else if (key == "maxItems") {
+                    node.AddFacet( CMemberFacet( ESerialFacet::eMaxItems, Value()));
+                } else if (key == "minLength") {
+                    node.AddFacet( CMemberFacet( ESerialFacet::eMinLength, Value()));
+                } else if (key == "maxLength") {
+                    node.AddFacet( CMemberFacet( ESerialFacet::eMaxLength, Value()));
+                } else if (key == "minimum") {
+                    min_value = Value();
+                } else if (key == "maximum") {
+                    max_value = Value();
+                } else if (key == "exclusiveMinimum") {
+                    min_value = Value();
+                    excl_max = true;
+                } else if (key == "exclusiveMaximum") {
+                    max_value = Value();
+                    excl_max = true;
+                } else if (key == "multipleOf") {
+                    node.AddFacet( CMemberFacet( ESerialFacet::eMultipleOf, Value()));
+                } else if (key == "default") {
+                    node.SetDefault(Value());
+                    AdjustMinOccurence(node, 0);
+                } else {
+                    ERR_POST_X(8, Warning << GetLocation() << "Unsupported property: " << key);
+                }
+            } else if (tok == T_DOUBLE) {
+                if (key == "minimum") {
+                    node.AddFacet( CMemberFacet( ESerialFacet::eInclusiveMinimum, Value()));
+                } else if (key == "maximum") {
+                    node.AddFacet( CMemberFacet( ESerialFacet::eInclusiveMaximum, Value()));
+                } else if (key == "exclusiveMinimum") {
+                    min_value = Value();
+                    excl_max = true;
+                } else if (key == "exclusiveMaximum") {
+                    max_value = Value();
+                    excl_max = true;
+                } else if (key == "default") {
+                    node.SetDefault(Value());
+                    AdjustMinOccurence(node, 0);
+                } else {
+                    ERR_POST_X(8, Warning << GetLocation() << "Unsupported property: " << key);
+                }
+            } else if (tok == K_TRUE || tok == K_FALSE) {
+                if (key == "exclusiveMinimum") {
+                    excl_min = true;
+                } else if (key == "exclusiveMaximum") {
+                    excl_max = true;
+                } else if (key == "uniqueItems") {
+                    node.AddFacet( CMemberFacet( ESerialFacet::eUniqueItems, Value()));
+                } else if (key == "additionalProperties") {
+//                    has_additional_prop = tok == K_TRUE;
                 } else {
                     ERR_POST_X(8, Warning << GetLocation() << "Unsupported property: " << key);
                 }
@@ -214,23 +277,23 @@ void JSDParser::ParseNode(DTDElement& node)
                     ParseError("type arrays not supported", "string");
                 } else if (key == "items") {
                     ParseError("tuple validation not supported", "{");
+                } else if (key == "examples") {
+                    SkipUnknown(K_END_ARRAY);
                 } else {
                     ERR_POST_X(8, Warning << GetLocation() << "Unsupported property: " << key);
                     SkipUnknown(K_END_ARRAY);
                 }
-#if 0
-            } else if (tok == K_TRUE || tok == K_FALSE) {
-                if (key == "additionalProperties") {
-                    has_additional_prop = tok == K_TRUE;
-                } else {
-                    ERR_POST_X(8, Warning << GetLocation() << "Unsupported property: " << key);
-                }
-#endif
             } else {
                 ERR_POST_X(8, Warning << GetLocation() << "Unsupported property: " << key);
             }
         }
     }
+    if (!min_value.empty()) {
+        node.AddFacet( CMemberFacet( excl_min ?  ESerialFacet::eExclusiveMinimum : ESerialFacet::eInclusiveMinimum, min_value));
+    }
+    if (!max_value.empty()) {
+        node.AddFacet( CMemberFacet( excl_max ?  ESerialFacet::eExclusiveMaximum : ESerialFacet::eInclusiveMaximum, max_value));
+    }
     if (is_object && has_additional_prop) {
         string item_id = NStr::Join(m_URI,"/") + "*";
         DTDElement& item = m_MapElement[item_id];
@@ -254,7 +317,7 @@ void JSDParser::AdjustMinOccurence(DTDElement& node, int occ)
         }
     } else {
         if (occ != 1) {
-            ERR_POST_X(8, Warning << GetLocation() << "Unsupported property minItems= " << occ);
+            node.AddFacet( CMemberFacet( ESerialFacet::eMinItems, Value()));
         }
         if (occNow == DTDElement::eZeroOrOne) {
             occNew = DTDElement::eOne;
@@ -296,8 +359,8 @@ void JSDParser::ParseDependencies(DTDElement& node)
                 string seq_id = NStr::Join(m_URI,"/") + "/" + NStr::NumericToString(i++);
                 DTDElement& seq = m_MapElement[seq_id];
                 seq.SetType(DTDElement::eSequence);
-                seq.SetName(seq_id);
                 seq.SetEmbedded();
+                seq.SetName(seq_id);
                 seq.SetOccurrence(occ);
                 seq.SetOccurrence(node_id, occ);
                 AddElementContent(seq, node_id);
@@ -313,8 +376,8 @@ void JSDParser::ParseDependencies(DTDElement& node)
                             node_id = NStr::Join(m_URI,"/");
                             m_URI.pop_back();
                             node.RemoveContent(node_id);
-                            DTDElement::EOccurrence occ = node.GetOccurrence(node_id);
-                            seq.SetOccurrence(node_id, occ);
+                            DTDElement::EOccurrence x_occ = node.GetOccurrence(node_id);
+                            seq.SetOccurrence(node_id, x_occ);
                             AddElementContent(seq, node_id);
                         }
                     }
@@ -421,10 +484,15 @@ void JSDParser::ParseOneOf(DTDElement& node)
             if (c.GetType() == DTDElement::eAlias) {
                 item_id = c.GetTypeName();
             } else {
-                if (!c.IsNamed()) {
-                    c.SetName(item_id);
+                if(!c.IsNamed() && c.GetContent().size() == 1) {
+                    item_id = c.GetContent().front();
+                } else {
+                    if (!c.IsNamed()) {
+                        c.SetName(item_id);
+                        c.SetNamed(false);
+                    }
+                    m_MapElement[item_id] = c;
                 }
-                m_MapElement[item_id] = c;
             }
     	    AddElementContent(node,item_id);
         }
@@ -460,10 +528,12 @@ void JSDParser::ParseAllOf(DTDElement& node)
         string item_id;
         if (c.GetType() != DTDElement::eAlias && e.IsNamed()) {
     	    item_id = e.GetName();
+            node.SetOccurrence(item_id, e.GetOccurrence(item_id));
             AddElementContent(node,item_id);
         } else {
             for(const string& ref : e.GetContent()) {
                 item_id = ref;
+                node.SetOccurrence(item_id, e.GetOccurrence(item_id));
     	        AddElementContent(node,item_id);
             }
         }
