@@ -1,4 +1,4 @@
-/*  $Id: statictype.cpp 532212 2017-04-03 13:24:13Z gouriano $
+/*  $Id: statictype.cpp 573070 2018-10-22 18:55:59Z fukanchi $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -57,6 +57,82 @@ void CStaticDataType::PrintASN(CNcbiOstream& out, int /*indent*/) const
     out << GetASNKeyword();
 }
 
+void CStaticDataType::PrintJSONSchema(CNcbiOstream& out, int indent, list<string>& required, bool) const
+{
+    string asnk(GetASNKeyword());
+    const CDataMember* mem = GetDataMember();
+    string type("string");
+    if (asnk == "NULL") {
+        type = "null";
+    } else if (asnk == "BOOLEAN") {
+        type = "boolean";
+    } else if (asnk == "INTEGER" || asnk == "BigInt") {
+        type = "integer";
+    } else if (asnk == "REAL") {
+        type = "number";
+    }
+    PrintASNNewLine(out, indent);
+    if (IsNillable()) {
+        out << "\"oneOf\": [{";
+    }
+    out << "\"type\": \"" << type << "\"";
+    if (mem) {
+        const list<CMemberFacet>& con = mem->GetRestrictions();
+        string pattern;
+        for (const CMemberFacet& f : con) {
+            string fname;
+            switch (f.GetType()) {
+            case ESerialFacet::eMinLength:
+                out << ",";  PrintASNNewLine(out, indent) << "\"minLength\": " << f.GetValue();
+                break;
+            case ESerialFacet::eMaxLength:
+                out << ",";  PrintASNNewLine(out, indent) << "\"maxLength\": " << f.GetValue();
+                break;
+            case ESerialFacet::eExclusiveMinimum:
+                out << ",";  PrintASNNewLine(out, indent) << "\"exclusiveMinimum\": " << f.GetValue();
+                break;
+            case ESerialFacet::eInclusiveMinimum:
+                out << ",";  PrintASNNewLine(out, indent) << "\"minimum\": " << f.GetValue();
+                break;
+            case ESerialFacet::eExclusiveMaximum:
+                out << ",";  PrintASNNewLine(out, indent) << "\"exclusiveMaximum\": " << f.GetValue();
+                break;
+            case ESerialFacet::eInclusiveMaximum:
+                out << ",";  PrintASNNewLine(out, indent) << "\"maximum\": " << f.GetValue();
+                break;
+            case ESerialFacet::eMultipleOf:
+                out << ",";  PrintASNNewLine(out, indent) << "\"multipleOf\": " << f.GetValue();
+                break;
+            case ESerialFacet::ePattern:
+                if (!pattern.empty()) {
+                    pattern.append("|");
+                }
+                pattern.append(f.GetValue());
+                break;
+            default: break;
+            }
+        }
+        if (!pattern.empty()) {
+            out << ",";  PrintASNNewLine(out, indent) << "\"pattern\": \"" << pattern << "\"";
+        }
+    }
+    if (IsNillable()) {
+        out << "}, {\"type\": \"null\"}]";
+    }
+    if (mem) {
+        if (mem->GetDefault()) {
+            out << ", \"default\": \"" << mem->GetDefault()->GetXmlString() << "\"";
+        }
+        if (!mem->Optional()) {
+            if (mem->SimpleType()) {
+                required.push_back(string("#") + mem->GetName());
+            } else {
+                required.push_back(mem->GetName());
+            }
+        }
+    }
+}
+
 // XML schema generator submitted by
 // Marc Dumontier, Blueprint initiative, dumontier@mshri.on.ca
 // modified by Andrei Gourianov, gouriano@ncbi
@@ -67,7 +143,12 @@ void CStaticDataType::PrintXMLSchema(CNcbiOstream& out,
     string xsdk("element"), use, form;
     const CDataMember* mem = GetDataMember();
     bool optional = mem ? mem->Optional() : false;
+    bool isGlobalType = GetGlobalType() == CDataType::eType;
+    bool hasFacets = mem ? !mem->GetRestrictions().empty() : !GetRestrictions().empty();
 
+    if (isGlobalType) {
+        xsdk = "complexType";
+    }
     if (GetParentType() && GetParentType()->GetDataMember()) {
         if (GetParentType()->GetDataMember()->Attlist()) {
             xsdk = "attribute";
@@ -86,7 +167,7 @@ void CStaticDataType::PrintXMLSchema(CNcbiOstream& out,
     }
     PrintASNNewLine(out, indent) << "<xs:" << xsdk << " name=\"" << tag << "\"";
     string type = GetSchemaTypeString();
-    if (!type.empty()) {
+    if (!isGlobalType && !type.empty() && !hasFacets) {
         out << " type=\"" << type << "\"";
     }
     if (!use.empty()) {
@@ -120,6 +201,42 @@ void CStaticDataType::PrintXMLSchema(CNcbiOstream& out,
     if (IsNillable()) {
         out << " nillable=\"true\"";
     }
+    if (isGlobalType && !type.empty()) {
+        out << ">";
+        PrintASNNewLine(out, ++indent) << "<xs:simpleContent>";
+        PrintASNNewLine(out, ++indent) << "<xs:extension base=\"" << type << "\"/>";
+        PrintASNNewLine(out, --indent) << "</xs:simpleContent>";
+        PrintASNNewLine(out, --indent) << "</xs:" << xsdk << ">";
+        return;
+    }
+    if (hasFacets && !type.empty()) {
+        out << ">";
+        PrintASNNewLine(out, ++indent) << "<xs:simpleType>";
+        PrintASNNewLine(out, ++indent) << "<xs:restriction base=\"" << type << "\">";
+        ++indent;
+        const list<CMemberFacet>& con = mem ? mem->GetRestrictions() : GetRestrictions();
+        for (const CMemberFacet& f : con) {
+            string fname;
+            switch (f.GetType()) {
+            case ESerialFacet::eMinLength:         fname = "minLength";    break;
+            case ESerialFacet::eMaxLength:         fname = "maxLength";    break;
+            case ESerialFacet::eLength:            fname = "length";       break;
+            case ESerialFacet::ePattern:           fname = "pattern";      break;
+            case ESerialFacet::eInclusiveMinimum:  fname = "minInclusive"; break;
+            case ESerialFacet::eExclusiveMinimum:  fname = "minExclusive"; break;
+            case ESerialFacet::eInclusiveMaximum:  fname = "maxInclusive"; break;
+            case ESerialFacet::eExclusiveMaximum:  fname = "maxExclusive"; break;
+            default: break;
+            }
+            if (!fname.empty()) {
+                PrintASNNewLine(out, indent) << "<xs:" << fname << " value=\"" << f.GetValue() << "\"/>";
+            }
+        }
+        PrintASNNewLine(out, --indent) << "</xs:restriction>";
+        PrintASNNewLine(out, --indent) << "</xs:simpleType>";
+        PrintASNNewLine(out, --indent) << "</xs:" << xsdk << ">";
+        return;
+    }
     if (PrintXMLSchemaContents(out,indent+1,mem)) {
         PrintASNNewLine(out, indent) << "</xs:" << xsdk << ">";
     } else {
@@ -130,7 +247,8 @@ void CStaticDataType::PrintXMLSchema(CNcbiOstream& out,
 bool CStaticDataType::PrintXMLSchemaContents(CNcbiOstream& out, int indent, const CDataMember* mem) const
 {
 #if _DATATOOL_USE_SCHEMA_STYLE_COMMENTS
-    return (mem && mem->GetComments().PrintSchemaComments(out,indent));
+    const CComments& cm(mem ? mem->GetComments() : Comments());
+    return cm.PrintSchemaComments(out,indent);
 #else
     return false;
 #endif
@@ -209,9 +327,15 @@ bool CNullDataType::PrintXMLSchemaContents(CNcbiOstream& out, int indent, const
 {
 #if _DATATOOL_USE_SCHEMA_STYLE_COMMENTS
     if (!CStaticDataType::PrintXMLSchemaContents(out, indent, mem)) {
+        if (GetGlobalType() != CDataType::eElement) {
+            return false;
+        }
         out << ">";
     }
 #else
+    if (IsGlobalType()) {
+        return false;
+    }
     out << ">";
 #endif
     PrintASNNewLine(out, indent) << "<xs:complexType/>";
@@ -283,7 +407,9 @@ bool CBoolDataType::PrintXMLSchemaContents(CNcbiOstream& out, int indent, const
 #else
     bool tagclosed = false;
 #endif
-
+    if (!IsASNDataSpec()) {
+        return tagclosed;
+    }
     if (GetParentType() && 
         GetParentType()->GetDataMember() &&
         GetParentType()->GetDataMember()->Attlist()) {
@@ -618,6 +744,18 @@ const char* CBitStringDataType::GetDEFKeyword(void) const
     return "_BIT_STRING_";
 }
 
+CBitStringDataType::CBitStringDataType(CDataType* bitenum)
+    : m_BitStringEnum(bitenum)
+{
+}
+
+void CBitStringDataType::FixTypeTree(void) const
+{
+    if (m_BitStringEnum) {
+        m_BitStringEnum->SetParent(GetModule(), GetMemberName());
+    }
+}
+
 bool CBitStringDataType::CheckValue(const CDataValue& value) const
 {
     CheckValueType(value, CBitStringDataValue, "BIT STRING");
@@ -638,7 +776,8 @@ bool CBitStringDataType::NeedAutoPointer(TTypeInfo /*typeInfo*/) const
 
 AutoPtr<CTypeStrings> CBitStringDataType::GetFullCType(void) const
 {
-    return AutoPtr<CTypeStrings>(new CBitStringTypeStrings( GetDefaultCType(), Comments() ));
+    return AutoPtr<CTypeStrings>(new CBitStringTypeStrings( GetDefaultCType(), Comments(),
+        m_BitStringEnum ? m_BitStringEnum->GetFullCType().release() : nullptr ));
 }
 
 const char* CBitStringDataType::GetDefaultCType(void) const
@@ -876,6 +1015,10 @@ void CAnyContentDataType::PrintASN(CNcbiOstream& out, int /* indent */) const
     out << GetASNKeyword();
 }
 
+void CAnyContentDataType::PrintJSONSchema(CNcbiOstream&, int, list<string>&, bool) const
+{
+}
+
 void CAnyContentDataType::PrintXMLSchema(CNcbiOstream& out, int indent, bool contents_only) const
 {
     const CDataMember* mem = GetDataMember();
