@@ -1,4 +1,4 @@
-/*  $Id: objistrxml.cpp 534803 2017-05-02 16:42:54Z gouriano $
+/*  $Id: objistrxml.cpp 570663 2018-09-12 18:28:27Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -331,7 +331,9 @@ void CObjectIStreamXml::EndTag(void)
 bool CObjectIStreamXml::EndOpeningTagSelfClosed(void)
 {
     if (!StackIsEmpty() && TopFrame().GetNotag()) {
-        return SelfClosedTag();
+        if (SelfClosedTag()) {
+            return true;
+        }
     }
     if( InsideOpeningTag() ) {
         char c = SkipWS();
@@ -1146,6 +1148,8 @@ void CObjectIStreamXml::SkipAnyContentObject(void)
     string tagName;
     if (!m_RejectedTag.empty()) {
         tagName = RejectedName();
+    } else if (OutsideTag()) {
+        tagName = ReadName(BeginOpeningTag());
     }
     if (SkipAnyContent() && !tagName.empty()) {
         CloseTag(tagName);
@@ -1343,7 +1347,7 @@ void CObjectIStreamXml::ReadWord(string& str, EStringType type)
 TEnumValueType CObjectIStreamXml::ReadEnum(const CEnumeratedTypeValues& values)
 {
     TEnumValueType value;
-    bool valueonly = GetRecentTypeInfo() && GetRecentTypeInfo()->GetDataSpec() == EDataSpec::eJSON;
+    bool valueonly = m_StdXml;
     if (valueonly) {
         if (values.IsInteger()) {
             value = ReadInt4();
@@ -1428,7 +1432,7 @@ CObjectIStream::EPointerType CObjectIStreamXml::ReadPointerType(void)
             return eNullPointer;
         }
     }
-    if ( !HasAttlist() && ((InsideOpeningTag() && EndOpeningTagSelfClosed()) || SelfClosedTag()) ) {
+    if ( (!m_SkipNextTag || ExpectSpecialCase()) && !HasAttlist() && ((InsideOpeningTag() && EndOpeningTagSelfClosed()) || SelfClosedTag()) ) {
         // self closed tag
         return eNullPointer;
     }
@@ -1746,7 +1750,9 @@ bool CObjectIStreamXml::HasMoreElements(TTypeInfo elementType)
         TTypeInfo type = GetRealTypeInfo(elementType);
         // this is to handle STL containers of primitive types
         if (GetRealTypeFamily(type) == eTypeFamilyPrimitive) {
-            if (!m_RejectedTag.empty()) {
+            if (m_SkipNextTag) {
+                return true;
+            } else if (!m_RejectedTag.empty()) {
                 m_LastPrimitive = m_RejectedTag;
                 return true;
             } else {
@@ -1765,6 +1771,9 @@ bool CObjectIStreamXml::HasMoreElements(TTypeInfo elementType)
             dynamic_cast<const CClassTypeInfoBase*>(type);
         const CAliasTypeInfo* aliasType = classType ? NULL :
             dynamic_cast<const CAliasTypeInfo*>(type);
+        if (aliasType && aliasType->IsFullAlias()) {
+            classType = dynamic_cast<const CClassTypeInfoBase*>(GetRealTypeInfo(aliasType));
+        }
         if (classType || aliasType) {
             if (m_RejectedTag.empty()) {
                 if (!NextIsTag()) {
@@ -1780,8 +1789,7 @@ bool CObjectIStreamXml::HasMoreElements(TTypeInfo elementType)
                 return classType->GetItems().FindDeep(tagName) != kInvalidMember ||
                     HasAnyContent(classType) != kInvalidMember;
             }
-            TTypeInfo nextType = classType ? (TTypeInfo)classType : (TTypeInfo)aliasType;
-            return tagName == nextType->GetName();
+            return (classType && tagName == classType->GetName()) || (aliasType && tagName == aliasType->GetName());
         }
     }
     return true;
@@ -1864,6 +1872,10 @@ void CObjectIStreamXml::BeginArrayElement(TTypeInfo elementType)
             TopFrame().SetNotag();
             return;
         }
+        if (m_SkipNextTag && type.GetTypeFamily() == eTypeFamilyPrimitive) {
+            TopFrame().SetNotag();
+            return;
+        }
     }
     OpenStackTag(0);
 }
@@ -1960,25 +1972,28 @@ void CObjectIStreamXml::SkipContainerContents(const CContainerTypeInfo* cType)
 
 void CObjectIStreamXml::BeginNamedType(TTypeInfo namedTypeInfo)
 {
-    bool isclass = false;
-    if (m_SkipNextTag) {
+    CheckStdXml(namedTypeInfo);
+    if (m_SkipNextTag || namedTypeInfo->GetName().empty()) {
         TopFrame().SetNotag();
         m_SkipNextTag = false;
     } else {
-        const CClassTypeInfo* classType =
-            dynamic_cast<const CClassTypeInfo*>(namedTypeInfo);
-        if (classType) {
-            CheckStdXml(classType);
-            isclass = true;
+        TTypeInfo realtype = GetRealTypeInfo(namedTypeInfo);
+        if (realtype->GetTypeFamily() == eTypeFamilyPrimitive &&
+            GetStackDepth() > 2 && m_StdXml) {
+            TopFrame().SetNotag();
+            m_SkipNextTag = false;
+            return;
         }
         OpenTag(namedTypeInfo);
     }
-    if (!isclass) {
-        const CAliasTypeInfo* aliasType = 
-            dynamic_cast<const CAliasTypeInfo*>(namedTypeInfo);
-        if (aliasType) {
-            m_SkipNextTag = aliasType->IsFullAlias();
-        }
+    const CAliasTypeInfo* aliasType = 
+        dynamic_cast<const CAliasTypeInfo*>(namedTypeInfo);
+    if (aliasType) {
+        m_SkipNextTag = aliasType->IsFullAlias();
+    }
+    else if (m_StdXml) {
+        const CClassTypeInfo* classType = dynamic_cast<const CClassTypeInfo*>(namedTypeInfo);
+        m_SkipNextTag = (classType && classType->Implicit());
     }
 }
 
@@ -2008,20 +2023,28 @@ void CObjectIStreamXml::ReadNamedType(TTypeInfo namedTypeInfo,
 }
 #endif
 
-void CObjectIStreamXml::CheckStdXml(const CClassTypeInfoBase* classType)
+void CObjectIStreamXml::CheckStdXml(TTypeInfo typeinfo)
 {
-    TMemberIndex first = classType->GetItems().FirstIndex();
-    m_StdXml = classType->GetItems().GetItemInfo(first)->GetId().HaveNoPrefix();
+    if (typeinfo->GetCodeVersion() > 21600) {
+        m_StdXml = typeinfo->GetDataSpec() != EDataSpec::eASN;
+    } else {
+        const CClassTypeInfo* classType =
+            dynamic_cast<const CClassTypeInfo*>(typeinfo);
+        if (classType) {
+            TMemberIndex first = classType->GetItems().FirstIndex();
+            m_StdXml = classType->GetItems().GetItemInfo(first)->GetId().HaveNoPrefix();
+        }
+    }
 }
 
 void CObjectIStreamXml::BeginClass(const CClassTypeInfo* classInfo)
 {
+    CheckStdXml(classInfo);
     if (m_SkipNextTag) {
         TopFrame().SetNotag();
         m_SkipNextTag = false;
         return;
     }
-    CheckStdXml(classInfo);
     if (x_IsStdXml()) {
         if (!m_Attlist) {
 // if class spec defines no attributes, but there are some - skip them
@@ -2222,9 +2245,16 @@ CObjectIStreamXml::BeginClassMember(const CClassTypeInfo* classType,
                     }
                 }
             } else {
-                if (CanSkipUnknownMembers() && NextIsTag()) {
-                    SetFailFlags(fUnknownValue);
-                    SkipAnyContent();
+                if (CanSkipUnknownMembers()) {
+                    while (NextIsTag()) {
+                        tagName = ReadName(BeginOpeningTag());
+                        UndoClassMember();
+                        if (IsKnownElement(tagName)) {
+                            break;
+                        }
+                        SetFailFlags(fUnknownValue);
+                        SkipAnyContentObject();
+                    }
                 }
                 return kInvalidMember;
             }
@@ -2240,7 +2270,7 @@ CObjectIStreamXml::BeginClassMember(const CClassTypeInfo* classType,
     TMemberIndex ind = classType->GetMembers().Find(tagName);
     if (ind == kInvalidMember) {
         ind = classType->GetMembers().FindDeep(tagName);
-        if (ind != kInvalidMember) {
+        if (ind != kInvalidMember && ind >= pos) {
             TopFrame().SetNotag();
             UndoClassMember();
             return ind;
@@ -2258,8 +2288,8 @@ CObjectIStreamXml::BeginClassMember(const CClassTypeInfo* classType,
                         elem_type->GetName() == mem_type->GetName());
                 }
             } else {
-                needUndo = (type != eTypeFamilyPrimitive) ||
-                    mem_info->GetId().HasAnyContent();
+                needUndo = mem_info->GetId().HasNotag() || mem_info->GetId().HasAnyContent() || type == eTypeFamilyContainer;
+                m_SkipNextTag = type != eTypeFamilyPrimitive && type != eTypeFamilyContainer;
             }
             if (needUndo) {
                 TopFrame().SetNotag();
@@ -2305,6 +2335,7 @@ CObjectIStreamXml::BeginClassMember(const CClassTypeInfo* classType,
 
 void CObjectIStreamXml::EndClassMember(void)
 {
+    m_SkipNextTag = false;
     if (TopFrame().GetNotag()) {
         TopFrame().SetNotag(false);
     } else {
@@ -2325,12 +2356,12 @@ void CObjectIStreamXml::UndoClassMember(void)
 
 void CObjectIStreamXml::BeginChoice(const CChoiceTypeInfo* choiceType)
 {
+    CheckStdXml(choiceType);
     if (m_SkipNextTag) {
         TopFrame().SetNotag();
         m_SkipNextTag = false;
         return;
     }
-    CheckStdXml(choiceType);
     OpenTagIfNamed(choiceType);
 }
 void CObjectIStreamXml::EndChoice(void)
@@ -2414,7 +2445,8 @@ TMemberIndex CObjectIStreamXml::BeginChoiceVariant(const CChoiceTypeInfo* choice
                         elem_type->GetName() == var_type->GetName());
                 }
             } else {
-                needUndo = (type != eTypeFamilyPrimitive);
+                needUndo = var_info->GetId().HasNotag() || var_info->GetId().HasAnyContent() || type == eTypeFamilyContainer;
+                m_SkipNextTag = type != eTypeFamilyPrimitive && type != eTypeFamilyContainer;
             }
             if (needUndo) {
                 TopFrame().SetNotag();
@@ -2442,6 +2474,7 @@ TMemberIndex CObjectIStreamXml::BeginChoiceVariant(const CChoiceTypeInfo* choice
 
 void CObjectIStreamXml::EndChoiceVariant(void)
 {
+    m_SkipNextTag = false;
     if (TopFrame().GetNotag()) {
         TopFrame().SetNotag(false);
     } else {
