@@ -1,4 +1,4 @@
-/*  $Id: objstack.cpp 534119 2017-04-24 17:07:34Z gouriano $
+/*  $Id: objstack.cpp 570663 2018-09-12 18:28:27Z ivanov $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -34,6 +34,7 @@
 #include <serial/impl/objstack.hpp>
 #include <serial/impl/ptrinfo.hpp>
 #include <serial/impl/continfo.hpp>
+#include <serial/impl/classinfob.hpp>
 
 BEGIN_NCBI_SCOPE
 
@@ -59,6 +60,14 @@ void CObjectStack::ResetState(void)
     m_PathValid = false;
 }
 
+void CObjectStack::ResetPathHooks(void)
+{
+    for (auto& h : m_PathHooks) {
+        h->Erase(this);
+    }
+    m_PathHooks.clear();
+}
+
 void CObjectStack::UnendedFrame(void)
 {
 }
@@ -296,6 +305,29 @@ void CObjectStack::PopErrorFrame(void)
     PopFrame();
 }
 
+bool CObjectStack::IsKnownElement(const CTempString& name) const
+{
+    size_t s, depth = GetStackDepth();
+    for (s=1; s < depth; ++s) {
+        const TFrame& frame = FetchFrameFromTop(s);
+        if (frame.GetFrameType() == CObjectStackFrame::eFrameClass ||
+            frame.GetFrameType() == CObjectStackFrame::eFrameChoice) {
+            const CClassTypeInfoBase* type = dynamic_cast<const CClassTypeInfoBase*>(frame.GetTypeInfo());
+            TMemberIndex i = type->GetItems().FindDeep(name);
+            if (i != kInvalidMember) {
+                return true;
+            }
+        }
+        else if (frame.HasTypeInfo() && !frame.GetTypeInfo()->GetName().empty()) {
+            break;
+        }
+        else if (!frame.GetNotag()) {
+            break;
+        }
+    }
+    return false;
+}
+
 TTypeInfo CObjectStack::GetRealTypeInfo(TTypeInfo typeInfo)
 {
     if (typeInfo->GetTypeFamily() == eTypeFamilyPointer) {
@@ -387,7 +419,7 @@ string CObjectStackFrame::GetFrameName(void) const
         {
             if ( m_MemberId ) {
                 const CMemberId& id = *m_MemberId;
-                info = '.';
+//                info = '.';
                 if ( !id.GetName().empty() ) {
                     info += id.GetName();
                 }
@@ -406,7 +438,9 @@ string CObjectStackFrame::GetFrameName(void) const
         info = "[]";
         break;
     case eFrameNamed:
-        info = GetTypeInfo()->GetName();
+        if (!GetNotag()) {
+            info = GetTypeInfo()->GetName();
+        }
         break;
     default:
         {
