@@ -1,4 +1,4 @@
-/*  $Id: stdtypes.cpp 507801 2016-07-21 17:37:25Z gouriano $
+/*  $Id: stdtypes.cpp 539979 2017-06-29 15:25:18Z grichenk $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -1781,80 +1781,4 @@ void CPrimitiveTypeInfoBitString::SetValueBitString(TObjectPtr objectPtr,
 }
 
 
-#ifdef NCBI_STRICT_GI
-class CGiFunctions : public CPrimitiveTypeFunctions<TGi>
-{
-public:
-    static TObjectPtr Create(TTypeInfo /*typeInfo*/,
-                             CObjectMemoryPool* /*memoryPool*/)
-        {
-            return new TObjectType();
-        }
-    static bool IsDefault(TConstObjectPtr objectPtr)
-        {
-            return Get(objectPtr) == TObjectType();
-        }
-    static void SetDefault(TObjectPtr objectPtr)
-        {
-            Get(objectPtr) = TObjectType();
-        }
-};
-
-CPrimitiveTypeInfoGi::CPrimitiveTypeInfoGi(void)
-    : CParent(sizeof(TGi), ePrimitiveValueGi)
-{
-    typedef CPrimitiveTypeFunctions<ncbi::TGi> TFunctions;
-    SetMemFunctions(&CGiFunctions::Create,
-                    &CGiFunctions::IsDefault,
-                    &CGiFunctions::SetDefault,
-                    &TFunctions::Equals,
-                    &TFunctions::Assign);
-    SetIOFunctions(&TFunctions::Read,
-                   &TFunctions::Write,
-                   &TFunctions::Copy,
-                   &TFunctions::Skip);
-//    CPrimitiveTypeFunctions<CGi>::SetMemFunctions(this);
-//    CPrimitiveTypeFunctions<CGi>::SetIOFunctions(this);
-}
-
-TTypeInfo CStdTypeInfo<TGi>::GetTypeInfo(void)
-{
-    static TTypeInfo info = CreateTypeInfo();
-    return info;
-}
-
-CTypeInfo* CStdTypeInfo<TGi>::CreateTypeInfo(void)
-{
-    return new CPrimitiveTypeInfoGi();
-}
-
-void CPrimitiveTypeInfoGi::GetValueGi(TConstObjectPtr objectPtr,
-                                      TGi& value) const
-{
-    typedef CPrimitiveTypeFunctions<ncbi::TGi> TFunctions;
-    value = TFunctions::Get(objectPtr);
-}
-
-void CPrimitiveTypeInfoGi::SetValueGi(TObjectPtr objectPtr,
-                                      const TGi& value)
-    const
-{
-    typedef CPrimitiveTypeFunctions<ncbi::TGi> TFunctions;
-    TFunctions::Get(objectPtr) = value;
-}
-
-
-void CPrimitiveTypeInfo::GetValueGi(TConstObjectPtr /*objectPtr*/,
-                                    TGi& /*value*/) const
-{
-    ThrowIncompatibleValue();
-}
-void CPrimitiveTypeInfo::SetValueGi(TObjectPtr /*objectPtr*/,
-                                    const TGi& /*value*/) const
-{
-    ThrowIncompatibleValue();
-}
-#endif
-
-
 END_NCBI_SCOPE
