@@ -1,4 +1,4 @@
-/*  $Id: variant.cpp 358154 2012-03-29 15:05:12Z gouriano $
+/*  $Id: variant.cpp 547822 2017-10-04 15:45:27Z gouriano $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -470,6 +470,9 @@ void CVariantInfoFunctions::ReadInlineVariant(CObjectIStream& in,
     choiceType->SetIndex(choicePtr, index, in.GetMemoryPool());
     in.ReadObject(variantInfo->GetItemPtr(choicePtr),
                   variantInfo->GetTypeInfo());
+    if (in.GetVerifyData() == eSerialVerifyData_Yes) {
+        variantInfo->Validate(choicePtr, in);
+    }
 }
 
 void CVariantInfoFunctions::ReadPointerVariant(CObjectIStream& in,
@@ -568,6 +571,9 @@ void CVariantInfoFunctions::WriteInlineVariant(CObjectOStream& out,
     _ASSERT(variantInfo->IsInline());
     _ASSERT(variantInfo->GetChoiceType()->GetIndex(choicePtr) ==
             variantInfo->GetIndex());
+    if (out.GetVerifyData() == eSerialVerifyData_Yes) {
+        variantInfo->Validate(choicePtr, out);
+    }
     out.WriteObject(variantInfo->GetItemPtr(choicePtr),
                     variantInfo->GetTypeInfo());
 }
